REM * Filename: dnddoor.bas Version: v5.0a r1.0a for QB64.
REM * This subprogram contains the main input and communication routines.

REM **** BEGIN INCLUDE FILE ****

REM * Filename: dnddoor.inc  Version: v5.0a r1.0a
REM * Variables here are included common to all subprograms.

' define all default variables as integers
DEFINT A-Z

' allocate common variables and arrays at runtime
REM $DYNAMIC

' declare titlebar text
_TITLE "DNDDOOR"

'  define structure for containers
TYPE ContainerType
    Closed AS INTEGER ' closed flag 1=closed, 0=n/a, -1=open
    ContainerName AS STRING * 30 ' container name from treasure.dat file
    Locked AS INTEGER ' locked flag 1=locked, 0=n/a, -1=unlocked
    Keyed AS INTEGER ' container key number 1-32767
    ShortName AS STRING * 30 ' mnemonic of container name from treasure.dat file
    Charges AS STRING * 10 ' treasure charges of container contents
    Inventory AS STRING * 10 ' index to treasure.dat of container contents
    Invisible AS STRING * 10 ' invisible flag -1/0 of container contents
    Unused AS STRING * 150 ' treasure name of container contents (unused)
    Permanent AS INTEGER ' flag for permanent container
END TYPE

'  action structure
TYPE ActionType
    Attribute1 AS INTEGER ' room attribute one, 0=lit/-1=unlit
    Attribute2 AS INTEGER ' room attribute two, 0=land/1=air/2=underwater
    EncounterRate AS INTEGER ' rate of action prompts to encounter monster
    Fumble AS INTEGER ' fumble weapons/shield result
    HealthRate AS INTEGER ' rate of action prompts to increase user health
    HitPoints AS INTEGER ' negative/positive hits for vit/fat result
    Inventory AS INTEGER ' weapons=1/shields=2/armor=3/magic items=4 breakage
    Level AS INTEGER ' negative/positive value=lower/highest user level to enter
    MonsterTalk AS INTEGER ' talk to monster number trigger
    MonsterTrigger AS INTEGER ' number of room monster trigger
    Restrictions AS INTEGER ' level direction is restricted to, 12 bit flags
    RustRate AS INTEGER ' rate of action prompts weapons/shields rust
    SpellTrigger AS INTEGER ' number of room spell trigger
    StealRate AS INTEGER ' rate of action prompts monsters steal
    Teleport AS SINGLE ' teleport trap room number result
END TYPE

'  define room structure. this main data file contains room descriptions,
'  directions, one container, action number, object and treasure indexes.
TYPE RoomType
    Action AS INTEGER ' index to action file
    Direct AS STRING * 48 ' direction link values to next room number
    Flags AS STRING * 40 ' treasure is: hidden=1/trap=2/magically hidden=3
    LongDesc AS STRING * 400 ' five line long description of room
    MonsterClass AS INTEGER ' number of room monster class in monclass.dat file
    Object AS STRING * 40 ' number of object from objects.dat file
    HiddenObj AS STRING * 40 ' object flags, -1=hidden
    ShortDesc AS STRING * 80 ' one line short description of room for brief mode
    Treasure AS STRING * 40 ' number of treasure from treasure.dat file
    TreCharges AS STRING * 40 ' treasure charges
    Container AS ContainerType ' single container per room
END TYPE

'  define user structure. this main data file contains user statistics,
'  values, flags, object and treasure indexes.
TYPE UserType
    CodeName AS STRING * 30 ' encrypted user codename
    PassWord AS STRING * 20 ' encrypted user password
    ClassName AS STRING * 20 ' encrypted user classname
    ClassType AS INTEGER ' classname number 1-10
    ' user's maximum of 3 containers
    Container1 AS ContainerType: Container2 AS ContainerType: Container3 AS ContainerType
    Inv AS STRING * 40 ' user's inventory index to treasure.dat file
    Charges AS STRING * 40 ' treasure's charges/plus
    Object AS STRING * 40 ' user's object index to objects.dat file
    TrainRoom AS STRING * 40 ' training room array
    Stats AS STRING * 14 ' player statistics str/int/wis/dex/con/pie/cha
    Weapons AS STRING * 8 ' weapons proficiency percent blu/pol/sha/thr
    Room AS SINGLE ' room number
    Level AS INTEGER ' player's level
    Experience AS DOUBLE ' player's experience
    Gold AS DOUBLE ' collected gold
    NumCalls AS INTEGER ' number of calls during one day
    Fatigue AS INTEGER ' fatigue points during fighting
    FatigueMax AS INTEGER ' maximum possible fatigue points
    Vitality AS INTEGER ' vitality points during fighting
    VitalityMax AS INTEGER ' maximum possible vitality points
    Magic AS INTEGER ' magic points during fighting/using spells
    MagicMax AS INTEGER ' maximum possible magic points
    Poison AS INTEGER ' user is poisoned true/false, vitality decreasing
    Borrow AS DOUBLE ' gold borrowed from bank
    Invisible AS INTEGER ' user is invisible true/false
    Race AS INTEGER ' player's race 1-8, hum/elf/gno/dwa/hal/hal-elf/hal-orc/ogr
    Proficiency AS INTEGER ' weapon user is proficient in 1-4
    Bank AS DOUBLE ' amount gold stored in bank
    Sort AS INTEGER ' sort inventory (-1/0/1)
    MonstersKilled AS SINGLE ' total of all monsters killed during play
    Brief AS INTEGER ' brief mode true/false
    DateOn AS STRING * 10 ' last date user logged on, form: MM-DD-YYYY
    Beauty AS INTEGER ' lady class beauty points
    Glamour AS INTEGER ' lady class glamour points
    Psionic AS INTEGER ' psionic points during fighting/casting psi spells
    PsionicMax AS INTEGER ' maximum possible psionic points
    Align1 AS INTEGER ' alignment type 1, good/neutral/evil
    Align2 AS INTEGER ' alignment type 2, lawful/neutral/chaotic
    Flags AS INTEGER ' 1=rerolled/2-128=stat wish/256=wish/512=realigned/chars
    MaxCalls AS INTEGER ' maximum calls user is allowed per day
    FromHour AS INTEGER ' time restriction logon from hour,
    FromMin AS INTEGER ' time restriction logon from minute,
    ToHour AS INTEGER ' time restriction logon to hour,
    ToMin AS INTEGER ' time restriction logon to minute.
    Echo AS INTEGER ' echo mode 0=on,-1=off
    LastMessage AS INTEGER ' last message read
    Linefeeds AS INTEGER ' linefeed mode 0=on,-1=off
    Linelength AS INTEGER ' linelength for display
    Pagelength AS INTEGER ' pagelength for display
    Wordwrap AS INTEGER ' word wrap 0=on,-1=off
END TYPE

'  define spell structure. this data file contains spell statistics,
'  values, and flags. this struture is used by treasure and spells.
TYPE SpellType
    SpellName AS STRING * 30 ' full name of spell
    Chant AS STRING * 80 ' chant to be used for spell
    Level AS INTEGER ' level of spell/level user must be at least to cast spell
    SpellType AS INTEGER ' spell type 1-28
    Teleport AS SINGLE ' spell teleports when cast
    Psionic AS INTEGER ' spell is psionic, true/false
    PsionicMode AS INTEGER ' spell is psionic, 1=attack/2=defense
    ClassType AS INTEGER ' user class allowed to cast spell, flags 1-8
    Desc AS STRING * 80 ' description displayed when spell is cast
    Ingred AS STRING * 10 ' index to treasure.dat of required ingredients
    SpellFlag AS INTEGER ' ingredients type required flag, 1=use/2=read/4=cast
END TYPE

'  define monster class structure. this data file contains lists of monsters.
TYPE MonclassType
    Monsters AS STRING * 20 ' indexed list of ten monsters from monster.dat
END TYPE

'  define object structure. this data file contains object statistics,
'  values, and flags. this struture is used by users and rooms.
TYPE ObjectType
    ObjectName AS STRING * 30 ' full name of object
    ShortName AS STRING * 30 ' mnemonic of object name
    RoomLink AS SINGLE ' link to room number for entering object, or for jail
    Invisible AS INTEGER ' is invisible (-1/0)
    JailTrap AS INTEGER ' jails user if attacked, roomlink is jail room number
    DoorLock AS INTEGER ' locked (0=n/a,1=unlocked/2=locked)
    Destination AS INTEGER ' n/a (used to be room object was created)
    Permanent AS INTEGER ' cannot be picked up. default.
    Hidden AS INTEGER ' hidden (cannot not be seen or searched for)
    Closed AS INTEGER ' closed (-1=closed/0=open)
    Keyed AS INTEGER ' key number 1-32767
    Relocks AS INTEGER ' relocks after entering
    LongDesc AS STRING * 80 ' description displayed when object looked at
    FromHour AS INTEGER ' lights room from hour,
    FromMin AS INTEGER ' lights room from minute,
    ToHour AS INTEGER ' lights room to hour,
    ToMin AS INTEGER ' lights room to minute.
    LightRoom AS INTEGER ' lights room (-1=true/0=false)
    LightTime AS INTEGER ' light time (-1=from-to/0=anytime)
    ShortDesc AS STRING * 80 ' description displayed when object entered
    Trap AS INTEGER ' trapped portal (1=poison/2=teleport/3=hits for fat/vit)
    Teleport AS SINGLE ' (room number, trap=2), (hits -fat/hits +vit, trap=3)
END TYPE

'  define monster structure. this data file contains monster statistics,
'  values, and flags.
TYPE MonsterType
    MonsterName AS STRING * 30 ' full name of monster
    PluralName AS STRING * 30 ' mnemonic plural of monster name
    Treasure AS STRING * 10 ' five indexes to treasure.dat
    Experience AS DOUBLE ' experience gained by killing monster
    Gold AS DOUBLE ' money gained by killing monster
    NumberAppearing AS INTEGER ' maximum monsters encontered at one time
    Level AS INTEGER ' monster level/level of user required to kill monster
    Hits AS INTEGER ' hit points monster has for user to kill
    Poison AS INTEGER ' monster can poison=-1/monster is poisoned>0
    LevelDrain AS INTEGER ' can drain user levels (-1/0)
    Spell AS INTEGER ' can cast spells (equal to spell record number)
    Block AS INTEGER ' can block user from exiting room
    Prevent AS INTEGER ' can prevent user from taking treasure
    Follow AS INTEGER ' can follow user, otherwise contains jail room number
    Magic AS INTEGER ' casts magic spell (equals index to spell record number)
    Jail AS INTEGER ' can jail attacker (-1/0), room number is in follow
    Teleport AS SINGLE ' can teleport (equals room number)
    FollowPercent AS INTEGER ' percentage to follow
    BlockPercent AS INTEGER ' percentage to block
    PreventPercent AS INTEGER ' percentage to prevent
    SpellPercent AS INTEGER ' percentage to cast spell
    PoisonPercent AS INTEGER ' percentage to poison
    DrainPercent AS INTEGER ' percentage to drain levels
    Rate AS INTEGER ' encounter rate (action prompts)
    RatePercent AS INTEGER ' percentage encountered at rate
    Permanent AS INTEGER ' monster is permanent to room
    Talk AS STRING * 10 ' five indexes to montalk.dat
    Psionic AS INTEGER ' monster casts psionic spells
    PsionicSpell AS INTEGER ' psionic spell casted
END TYPE

'  define treasure structure. this data file contains treasure statistics,
'  values, and flags. this struture is used by users and rooms.
TYPE TreasureType
    TreasureName AS STRING * 30 ' full treasure name
    ShortName AS STRING * 30 ' mnemonic treasure name
    Weight AS INTEGER ' weight in gold of treasure
    Gold AS INTEGER ' value in gold of treasure
    Spell AS INTEGER ' treasure is magical (index of spell record)
    Plus AS INTEGER ' treasure plus (0=normal treasure/greater>0=weapon plus)
    Coin AS INTEGER ' coins (-1/0)
    Keyed AS INTEGER ' key number (1-32767)
    Scroll AS INTEGER ' scroll (-1/0)
    Charges AS INTEGER ' charges for weapon/armor/shield/spell
    Potion AS INTEGER ' potion (-1/0)
    Armor AS INTEGER ' treasure is armor
    Shield AS INTEGER ' treasure is shield
    Weapon AS INTEGER ' treasure is weapon
    Container AS INTEGER ' container (-1/0)
    Type AS INTEGER ' treasure type (-1=shield/0=normal/1=armor/0,<plus>=weapon)
    Locked AS INTEGER ' container (1=locked,0=n/a,-1=unlocked)
    Closed AS INTEGER ' container (1=closed,0=n/a,-1=open)
    Loadable AS INTEGER ' can be loaded with ammunition (-1/0)
    Ammunition AS INTEGER ' is ammunition (-1/0)
    AmmoLoads AS INTEGER ' if ammunition, then index to treasure.dat it loads
    Invisible AS INTEGER ' invisible (-1/0)
    Permanent AS INTEGER ' treasure remains in room (not default)
    Proficiency AS INTEGER ' weapon proficiency related (1-4)
    RingType AS INTEGER ' is a ring (1=antipoison/2=antileveldrain/3=antispell)
    RingSpell AS INTEGER ' ring antispell (-1=generic/greater>0=spell number)
    Edible AS INTEGER ' edible (-1/0)
    Rustable AS INTEGER ' rusts (-1/0)
    RustPercent AS INTEGER ' percentage rusts during room rust rate
    Stealable AS INTEGER ' stealable (-1/0)
    StealPercent AS INTEGER ' percentage monster steals during room steal rate
    Launchable AS INTEGER ' can be launched with launch ammunition (-1/0)
    LaunchLoads AS INTEGER ' is launch ammunition (-1/0)
    LaunchAmmo AS INTEGER ' if launch ammo, then index to treasure.dat it loads
    Movable AS INTEGER ' launchable device can be moved (-1/0)
    Vehicle AS INTEGER ' vehicle (-1/0)
    VehicleHits AS INTEGER ' vehicle hit points for combat
    VehicleType AS INTEGER ' vehicle type (0=land,1=air,2=underwater,3=any)
    LightType AS INTEGER ' is a light (-1/0)
    LightCharges AS INTEGER ' light charges
    FuelType AS INTEGER ' is light fuel (-1/0)
    FuelCharges AS INTEGER ' fuel charges
END TYPE

'  define monster talk structure. this data file contains lists of strings
'  monsters can respond with when talked to.
TYPE MonsterTalkType
    TalkMessage AS STRING * 80 ' an 80 character string response
END TYPE

'  help file record structure. used for help command during play,
'  uses files help.dat, edithelp.dat, dmhelp.dat, and logon.dat.
TYPE HelpType
    Text AS STRING * 79 ' text to display (updated 11/16/2001).
    CName AS STRING * 15 ' command name of command to display
END TYPE

'  structure for messages text file. indexed by the table record structure.
TYPE MessageType
    Message AS STRING * 80 ' length of a message line
END TYPE

'  table for messages. stores subject, from/to, date entered, private message
'  and if the message is deleted. fields for start and length define the
'  position of the message in the messages text file.
TYPE TableType
    ClassType AS INTEGER ' class type of the player who entered the message
    Clock AS STRING * 21 ' stores the time the message was entered from date/time
    Date AS STRING * 10 ' system date message was stored
    Flags AS INTEGER ' user record flags for special characters entered message
    From AS STRING * 30 ' field storing user who entered the message
    Killed AS INTEGER ' true if the message is deleted
    Length AS INTEGER ' length of lines in messages text file
    Private AS INTEGER ' true if the message is private
    Received AS INTEGER ' flag indicating message was read
    Reply AS INTEGER ' flag for message reply
    Start AS SINGLE ' beginning of the line of the message in the text file
    Subject AS STRING * 20 ' stores the subject of the message
    Thread AS INTEGER ' number of message replied to
    Time AS STRING * 10 ' stores system time message was entered
    Timer AS SINGLE ' stores system time in seconds message was entered
    TimesRead AS INTEGER ' stores number of times message was read
    ToUser AS STRING * 30 ' stores name of the user the message is to, or empty for all
END TYPE

'  define variables, fixed length/variable strings, and
'  dynamic arrays common to all subprograms.
DIM SHARED ActionRecord AS ActionType ' defines action structure
DIM SHARED Action.Prompt AS STRING ' stores action prompt Next: or Command>
DIM SHARED Alignment.Name1(1) AS STRING * 7 ' stores three type 1 alignment names
DIM SHARED Alignment.Name2(1) AS STRING * 7 ' stores three type 2 alignment names
DIM SHARED Allow.Break AS INTEGER ' allows control-k checking during file output
DIM SHARED Array(1) AS STRING ' string array for mail
DIM SHARED Array.List(1) AS STRING * 16 ' string array for command help
DIM SHARED Array.Number AS INTEGER ' contains number of array element searched for
DIM SHARED Baud.Override AS INTEGER ' command line baud override
DIM SHARED BBS.Name AS STRING ' name of calling bbs name from rbbs .def file
DIM SHARED Break AS INTEGER ' returns -1 is control-k pressed
DIM SHARED Buffer AS STRING ' stores modem i/o
DIM SHARED Carriage.Return AS INTEGER ' supresses linefeed during output
DIM SHARED Charges.Number AS INTEGER ' contains charges of item searched for
DIM SHARED Chained.Rooms AS INTEGER ' used in launch routine
DIM SHARED Chat AS INTEGER ' chat mode flag
DIM SHARED Choice AS INTEGER ' monster number selected
DIM SHARED Class.Name(1) AS STRING * 15 ' stores ten class names
DIM SHARED Color.Code AS INTEGER ' current ansi color
DIM SHARED Color.Graphics AS INTEGER ' ansi color activated remote
DIM SHARED Command.Name AS STRING ' storage of user command
DIM SHARED Continue AS INTEGER ' flag for continuous output
DIM SHARED ContainerRec AS ContainerType ' container structure
DIM SHARED Direction AS INTEGER ' direction 1-11
DIM SHARED Direction(1) AS STRING * 9 ' stores eleven direction names
DIM SHARED Door.Name AS STRING ' name of calling bbs user from rbbs .def file
DIM SHARED Door.Time AS SINGLE ' time left from rbbs .def file
DIM SHARED DOS.Major AS INTEGER ' dos major version
DIM SHARED DOS.Minor AS INTEGER ' dos minor version
DIM SHARED Dungeon.Master AS INTEGER ' user is dungeon master (-1/0)
DIM SHARED Dungeon.Master.Assistant AS INTEGER ' user is asst. DM (-1/0)
DIM SHARED Entry.Command AS INTEGER ' last command number used for room entry
DIM SHARED FileName AS STRING ' file name storage
DIM SHARED FunctionBuffer AS STRING ' buffer of interactive pressed function key
DIM SHARED Governor AS INTEGER ' user is governor (-1/0)
DIM SHARED Graphics.Off AS INTEGER ' set ansi graphics off (to white) (-1/0)
DIM SHARED Guild.Master AS INTEGER ' user is guild master (-1/0)
DIM SHARED HelpRecord AS HelpType ' help record structure
DIM SHARED Hidden.Player AS INTEGER ' player is hidden (-1/0)
DIM SHARED Hidden AS INTEGER ' mask input for security i/o with Mask$
DIM SHARED High.Class.Name(1) AS STRING * 15 ' ten high class names
DIM SHARED Inpt AS STRING ' user i/o input
DIM SHARED Index.Number AS INTEGER ' contains index number of an item searched for
DIM SHARED Intoxicated AS INTEGER ' user is intoxicated (equal to action prompts)
DIM SHARED Invisible AS INTEGER ' user is invisible (-1/0)
DIM SHARED Jail.Room.Number AS SINGLE ' room to send user when jailed
DIM SHARED Last.Command AS STRING ' last command entered
DIM SHARED Last.Command.Number AS INTEGER ' number of last command entered
DIM SHARED Last.Monster AS STRING ' name of last monster attacked
DIM SHARED Last.Room AS SINGLE ' stores temporary room number
DIM SHARED Learned.Spells AS STRING ' string of 1's and 0's of spells learned
DIM SHARED Line.Length AS INTEGER ' maximum length of input allowed
DIM SHARED Line.Limit AS INTEGER ' used with word wrap
DIM SHARED Local.Mode AS INTEGER ' user is logged in at console
DIM SHARED Locks.Smashed AS INTEGER ' used in launch routine
DIM SHARED Logged.In AS INTEGER ' flag indicating user is logged in
DIM SHARED Magic.Spell AS INTEGER ' magic spell number of spells.dat being cast
DIM SHARED Max.Row AS INTEGER ' maximum rows used for screen scrolling/output
DIM SHARED Message1 AS STRING ' stores message when user killed for death routine
DIM SHARED Message.Length AS INTEGER ' stores length of message in mail
DIM SHARED Message.Reply AS INTEGER ' flag for message reply
DIM SHARED Message.Thread AS INTEGER ' message number replied to
DIM SHARED Message.To AS STRING ' stores message to in mail
DIM SHARED MessageRecord AS MessageType ' message record structure for mail
DIM SHARED Modem.Access AS INTEGER ' modem in use flag
DIM SHARED Modem.Error AS INTEGER ' modem error flag
DIM SHARED Modem.Baud AS INTEGER ' connect speed, 00 truncated
DIM SHARED Monclass AS INTEGER ' current room monster class
DIM SHARED Monclass.Max AS INTEGER ' maximum monster classes in monclass array
DIM SHARED MonclassRecord AS MonclassType ' monster class record structure
DIM SHARED Monster.Class(1, 1) AS INTEGER ' array of monster classes
DIM SHARED Monster.Cycle AS INTEGER ' incrementing counter for monster attacks
DIM SHARED Monster.Number AS INTEGER ' stores currently accessed monster number
DIM SHARED Monster.Percent(1, 1) AS INTEGER ' array of monster percents
DIM SHARED Monster.Rate(1, 1) AS INTEGER ' array of monster encounter rates
DIM SHARED Monster.Rate1 AS INTEGER ' counts file monster rates
DIM SHARED Monster.Rate2 AS INTEGER ' counts room monster rates
DIM SHARED MonsterArray(1) AS MonsterType ' array of 20 monster record structures
DIM SHARED MonsterIndex(1) AS INTEGER ' array of 20 indexes to monster.dat file
DIM SHARED MonsterRecord AS MonsterType ' defines monster structure
DIM SHARED Monsters.Killed AS INTEGER ' total monsters killed in session
DIM SHARED MonsterTalkRecord AS MonsterTalkType ' monster class structure
DIM SHARED Multiplier AS INTEGER ' multiplier used in combat
DIM SHARED New.Room AS INTEGER ' added new room flag
DIM SHARED Next.Room AS SINGLE ' next room storage
DIM SHARED No AS INTEGER ' entered N during input (-1/0)
DIM SHARED No.Echo AS INTEGER ' do not echo user input
DIM SHARED No.Input AS INTEGER ' null was entered (-1/0)
DIM SHARED No.Input.Out AS STRING ' string to return if null entered during input
DIM SHARED Node AS STRING ' node in string form
DIM SHARED Node1 AS INTEGER ' node number (0-9,A-Z)
DIM SHARED Node2 AS INTEGER ' node number (0-99)
DIM SHARED Normal.User AS INTEGER ' user has special attributes (-1/0)
DIM SHARED Number.Appearing AS INTEGER ' maximum monsters of one type encountered
DIM SHARED Number.Monsters AS INTEGER ' current number of active monsters in room
DIM SHARED ObjectRecord AS ObjectType ' object structure
DIM SHARED Outpt AS STRING ' user i/o output
DIM SHARED Outpts AS STRING ' temporary user i/o output variable
DIM SHARED Parse.Count AS INTEGER ' parsed count to decrement
DIM SHARED Parse.Number AS INTEGER ' stores command numbers (pound sign) quantity
DIM SHARED Parsed.Command1 AS STRING ' first or second command parsed from input
DIM SHARED Parsed.Command2 AS STRING ' first or second command parsed from input
DIM SHARED Parser AS INTEGER ' position of command second word parsed
DIM SHARED Pass.Door AS INTEGER ' passdoor spell cast
DIM SHARED Player.CodeName AS STRING ' storage for user codename
DIM SHARED Player.PassWord AS STRING ' storage for user password
DIM SHARED Port AS INTEGER ' modem port being used (0-7)
DIM SHARED Port.Override AS INTEGER ' command line override
DIM SHARED Prefix1 AS STRING ' the/or/an
DIM SHARED Previous.Command AS INTEGER ' -1=use,0=n/a,equals last command number
DIM SHARED Previous.Command1 AS STRING ' storage of previous command
DIM SHARED Previous.Command2 AS STRING ' storage of previous parsed command
DIM SHARED Previous.Command3 AS STRING ' storage of previous second parsed command
DIM SHARED Private.Message AS INTEGER ' flag for mail
DIM SHARED Psi.Attack.Mode AS INTEGER ' last offense psi spell cast
DIM SHARED Psi.Defense.Mode AS INTEGER ' last defense psi spell cast
DIM SHARED Quit AS INTEGER ' true if Q entered
DIM SHARED Race(1) AS STRING * 8 ' stores eight race names
DIM SHARED Resurrection.Room AS INTEGER ' specifies room number to be resurrected to
DIM SHARED Room AS SINGLE ' current room number
DIM SHARED Room.Health.Rate AS INTEGER ' number of action prompts for health update
DIM SHARED Room.Monster.Rate AS INTEGER ' number of action promptsfor monster update
DIM SHARED Room.Rate AS INTEGER ' counter of action prompts for room rate
DIM SHARED Room.Rust.Rate AS INTEGER ' counter of room rust rate
DIM SHARED Room.Steal.Rate AS INTEGER ' counter of room steal rate
DIM SHARED RoomRecord AS RoomType ' room structure
DIM SHARED Rust.Rate AS INTEGER ' rust rate in action prompts
DIM SHARED Share.Count AS INTEGER ' count violations
DIM SHARED Sorting AS INTEGER ' inventory sorting (-1/0/1)
DIM SHARED SpellRecord AS SpellType ' spell structure
DIM SHARED Spell.Number AS INTEGER ' spell number being used
DIM SHARED Spell.Cast.Type AS INTEGER ' type of spell being cast
DIM SHARED Stat(1) AS STRING * 12 ' storage of eight statistics names
DIM SHARED Steal.Rate AS INTEGER ' steal rate in action prompts
DIM SHARED Stored.Parsed.Command1 AS STRING ' first parsed command from user input
DIM SHARED Stored.Parsed.Command2 AS STRING ' second parsed command from user input
DIM SHARED Subject AS STRING ' store mail subject
DIM SHARED Sysop AS INTEGER ' contains -1 if player is sysop
DIM SHARED TableRecord AS TableType ' message table for mail
DIM SHARED Teleported AS INTEGER ' user entered room by teleporting (-1/0)
DIM SHARED TempRecord AS MessageType ' temporary structure
DIM SHARED Temp, Temp!, Temp$ ' global temporary variables
DIM SHARED Temp.Array1(1) AS INTEGER ' global temporary variables
DIM SHARED Temp.ArrayX(1) AS INTEGER ' global temporary variables
DIM SHARED Temp.ArrayZ(1) AS DOUBLE ' global temporary variables
DIM SHARED Temp0, Temp1, Temp2, Temp3, Temp4 ' global temporary variables
DIM SHARED Temp5, Temp6, Temp7, Temp8, Temp9 ' global temporary variables
DIM SHARED Time.Calc AS SINGLE ' time storage variable
DIM SHARED Time.Left AS SINGLE ' amount of time user is allowed on in seconds
DIM SHARED Time.On AS STRING ' time user logged on in Date$ format
DIM SHARED Timeon AS SINGLE ' time user logged on in Timer format
DIM SHARED Timelogged.On AS STRING * 8 ' time user logged on
DIM SHARED Town.Mayor AS INTEGER ' user is town mayor (-1/0)
DIM SHARED Training.Stats(1, 1) AS INTEGER ' storage for training statistics
DIM SHARED TreasureRecord AS TreasureType ' treasure structure
DIM SHARED Two.Minutes.Left AS INTEGER ' flag indicating two minutes warning shown
DIM SHARED TypeNumber AS INTEGER ' flag for type of object in room or inventory
DIM SHARED Upper.Case AS INTEGER ' flag to force first character uppercase in io.i
DIM SHARED User.Echo AS INTEGER ' stores online user echo preference
DIM SHARED User.LineFeeds AS INTEGER ' stores online user linefeed preference
DIM SHARED User.LineLength AS INTEGER ' stores online user linelength preference
DIM SHARED User.PageLength AS INTEGER ' stores online user pagelength preference
DIM SHARED User.Wordwrap AS INTEGER ' stores online user wordwrap preference
DIM SHARED User.Command AS STRING ' stores original user input
DIM SHARED User.Index AS INTEGER ' index of user number to users.dat file
DIM SHARED User.Number AS INTEGER ' alternate user index variable
DIM SHARED UserRecord AS UserType ' user structure
DIM SHARED UserRecord2 AS UserType ' alternate user structure
DIM SHARED Vehicle1 AS INTEGER ' vehicle index to room treasure array
DIM SHARED Vehicle2 AS INTEGER ' charges in vehicle
DIM SHARED Vehicle3 AS INTEGER ' index to treasure.dat file
DIM SHARED Vehicle4 AS INTEGER ' vehicle type (0-3)
DIM SHARED WeaponTypeName(1) AS STRING * 9 ' storage of four weapon names
DIM SHARED Weapon1 AS INTEGER ' plus of armor worn
DIM SHARED Weapon2 AS INTEGER ' plus of weapon held
DIM SHARED Weapon3 AS INTEGER ' plus in shield held
DIM SHARED Weapon4 AS INTEGER ' index to user inventory array of armor worn
DIM SHARED Weapon5 AS INTEGER ' index to user inventory array of shield held
DIM SHARED Weapon6 AS INTEGER ' index to user inventory array of weapon held
DIM SHARED Weapon7 AS INTEGER ' index to user inventory array of ring worn
DIM SHARED Weapon8 AS INTEGER ' ring type (1=antipoison/2=antileveldrain/3=antispell)
DIM SHARED Weapon9 AS INTEGER ' ring antispell (-1=generic/greater>0=spell number)
DIM SHARED Weapon10 AS INTEGER ' proficiency number of weapon held
DIM SHARED Weapon11 AS INTEGER ' item plus of ring being worn
DIM SHARED Weight AS INTEGER ' total amount in gold of user's inventory
DIM SHARED Win.Major AS INTEGER ' windows major version
DIM SHARED Win.Minor AS INTEGER ' windows minor version
DIM SHARED Word.Wrap AS INTEGER ' used for word wrap during input
DIM SHARED Yes AS INTEGER ' user entered Y during input (-1/0)

'  constants used in place of variables inline programs
CONST Author$ = "Erik Jon Oredson" ' name of author
CONST Dead$ = "-dead-" ' used internal to user file
CONST Deleted$ = "<deleted>" ' used internal to data files
CONST Email$ = "eoredson@gmail.com" ' author's email address
CONST Enter$ = "<enter>" ' generic response substring
CONST False = 0 ' false
CONST Mask$ = "#" ' define mask character
CONST MaxCommands = 85 ' max user commands
CONST MaxSysopCommands = 9 ' max sysop commands
CONST MaxInt = 32767 ' maximum integer
CONST MaxStat = 250 ' maximum player statistic
CONST MaxViolations = 1024 ' maximum share violations
CONST MinInt = -32768 ' minimum integer
CONST More$ = "More(y/n/c/q)?" ' generic response substring
CONST None$ = "<null>" ' used internally
CONST Nul = "" ' empty string
CONST Program$ = "DNDDOOR" ' program name
CONST Range$ = "<range error>" ' generic response substring
CONST Release$ = "1.0a" ' program minor version
CONST True = NOT False ' true
CONST UnTrue = False + 1 ' untrue
CONST Version$ = "5.0a" ' program major version

'  constant variables corresponding to last command entered
CONST Picklock.Command = 26 ' compare to picklock
CONST Hide.Command = 49 ' compare to hide
CONST Identify.Command = 52 ' compare to identify

'  constant variables for driving vehicles
CONST RideVehicle = 48 ' ride vehicle command number
CONST DriveVehicle = 73 ' drive vehicle command number

'  constant variables of weapon proficiency types
CONST Blunt = 1 ' blunt weapon number
CONST Pole = 2 ' pole weapon number
CONST Sharp = 3 ' sharp weapon number
CONST Thrusting = 4 ' thrusting weapon number

'  bit constants of room record flags array attributes of room inventory
CONST Hidden.Object = 1 ' object was hidden with hide command
CONST Magic.Trap = 2 ' object was made into a trap with spell
CONST Magically.Hidden = 4 ' object was magically hidden with spell

'  constant variable used to indicate spell method being used
CONST Use.Spell.Type = 1 ' spell cast is by the use command
CONST Scroll.Spell.Type = 2 ' spell cast by using a scroll
CONST Cast.Spell.Type = 4 ' spell cast with the cast command

'  bit constants of player class, character, and command flags,
'  bits 2 through 128 correspond to the seven statistics players may wish for.
CONST Rerolled = 1 ' bit flag of user record flags field for rerolled player
CONST Wished = 256 ' bit flag of user record flags field of player wished
CONST Alignmented = 512 ' bit flag of user record flags field of player align
CONST Locked.User = 1024 ' bit for locked mailbox
CONST Special.Char1 = 2048 ' bit flag of user record flags field town mayor
CONST Special.Char2 = 4096 ' bit flag of user record flags field governor
CONST Special.Char3 = 8192 ' bit flag of user record flags field guild master
CONST Special.Char4 = 16384 ' bit flag of user record flags field sysop

' constants of class type numbers
CONST Fighter = 1 ' fighter class
CONST MagicUser = 2 ' MU class
CONST Thief = 3 ' thief class
CONST Cleric = 4 ' cleric class
CONST Paladin = 5 ' paladin class
CONST Ranger = 6 ' ranger class
CONST Druid = 7 ' druid class
CONST Lady = 8 ' lady class
CONST AsstDM = 9 ' asst. DM class
CONST DM = 10 ' DM class

' constants of attack command numbers
CONST Charm = 31 ' attack command number constant
CONST Beguile = 32 ' attack command number constant
CONST Shield = 34 ' attack command number constant
CONST Guard = 35 ' attack command number constant
CONST Lunge = 39 ' attack command number constant
CONST Dodge = 40 ' attack command number constant
CONST Killed = 55 ' attack command number constant
CONST Backstab = 59 ' attack command number constant
CONST Circled = 60 ' attack command number constant
CONST Feint = 61 ' attack command number constant
CONST Parry = 62 ' attack command number constant
CONST Thrust = 63 ' attack command number constant
CONST Charge = 64 ' attack command number constant
CONST Turn = 65 ' attack command number constant
CONST Counter = 66 ' attack command number constant
CONST Bemuse = 67 ' attack command number constant
CONST Befuddle = 68 ' attack command number constant
CONST Pummel = 69 ' attack command number constant
CONST Resist = 70 ' attack command number constant
CONST PsiMode = 71 ' attack command number constant
CONST Beat = 82 ' attack command number constant
CONST Punch = 83 ' attack command number constant
CONST Bewitch = 84 ' attack command number constant
CONST Bewilder = 85 ' attack command number constant

' constants of spell type numbers
CONST Enchant = 1 ' spell number constant
CONST Offense = 2 ' spell number constant
CONST Bless = 3 ' spell number constant
CONST Wish = 4 ' spell number constant
CONST Poison = 5 ' spell number constant
CONST Vigor = 6 ' spell number constant
CONST Heal = 7 ' spell number constant
CONST CurePoison = 8 ' spell number constant
CONST LevelDrain = 9 ' spell number constant
CONST Teleport = 10 ' spell number constant
CONST Befuddled = 11 ' spell number constant
CONST TurnUndead = 12 ' spell number constant
CONST PassDoor = 13 ' spell number constant
CONST Conjure = 14 ' spell number constant
CONST Psionic = 15 ' spell number constant
CONST DetectLock = 16 ' spell number constant
CONST DetectEvil = 17 ' spell number constant
CONST DetectTrap = 18 ' spell number constant
CONST Intoxicate = 19 ' spell number constant
CONST SetTrap = 20 ' spell number constant
CONST Hiding = 21 ' spell number constant
CONST Search = 22 ' spell number constant
CONST Invisibility = 23 ' spell number constant
CONST Identify = 24 ' spell number constant
CONST Enlighten = 25 ' spell number constant
CONST Illuminate = 26 ' spell number constant
CONST Psyche = 27 ' spell number constant
CONST Telepathy = 28 ' spell number constant (not used)

' constants for psionic spells
CONST PsiAttack = 1 ' psionic attack mode
CONST PsiDefense = 2 ' psionic attack mode

' constants for room attribute 1
CONST LitRoom = 0 ' flag constant for dark room
CONST UnlitRoom = -1 ' bit flag constant for lit room

' constants for room attribute 2 and vehicle type
CONST Land = 0 ' flag value for ground type room
CONST Air = 1 ' flag value for above ground room
CONST Water = 2 ' flag value constant for underwater room

' constants of room entry command
CONST Fly = 74 ' entry command number
CONST Swim = 79 ' entry command number
 
' constants for vehicle type
CONST AllTerrain = 4 ' vehicle terrain type

' constants of file numbers
CONST UserFile = 1 ' file number
CONST NonPlayerFile = 2 ' file number
CONST RoomFile = 3 ' file number
CONST ObjectFile = 4 ' file number
CONST MonsterFile = 5 ' file number
CONST TreasureFile = 6 ' file number
CONST MonTalkFile = 7 ' file number
CONST MonClassFile = 8 ' file number
CONST SpellFile = 9 ' file number
CONST TableFile = 10 ' file number
CONST MessageFile = 11 ' file number
CONST ActionFile = 12 ' file number
CONST TempFile = 13 ' file number
CONST TempFile2 = 14 ' file number
CONST HelpFile = 15 ' file number
CONST UserFile2 = 16 ' file number
CONST ModemFile = 17 ' modem file number

SysopCommands1:
DATA "ABORT","HINT","CALL","EDIT","HELP","INVISIBLE","LINK","<>","STATUS"

SysopCommands2:
DATA "<>","CALL","DISCARD","EDIT","GET","HELP","KILL","REDUCE","TELEPORT"

CommandSet1:
DATA "KILL","ATTACK","HIT","STRIKE","READ","LOOK","RUN","PANIC","SEARCH","SAVE"
DATA "INFORMATION","INVENTORY","SUICIDE","TRAIN","HEALTH","APPEAL","BYE","STOP","QUIT","END"
DATA "EXPERIENCE","HELP","HIDE","CAST","LIST","BRIEF","BANK","TIME","DATE","CLOCK"
DATA "CHARM","BEGUILE","ANSI","SHIELD","GUARD","<>","<>","<>","LUNGE","DODGE"
DATA "TOP","BLESS","CURSE","PASSWORD","MAIL","USERS","CHAT","EXIT","LEAVE","CLIMB"
DATA "STORE","LOGIN","RELOGIN","STATUS","LINELENGTH","CATALOG","LINEFEEDS","PAGELENGTH","ECHO","WORDWRAP"
DATA "ALIGNMENT","<>","<>","NORTH","EAST","SOUTH","WEST","NORTHEAST","SOUTHEAST","SOUTHWEST"
DATA "NORTHWEST","UP","DOWN","IN","OUT","HINT","SORT","COMMANDS","<>","<>"
DATA "<>","<>","REROLL","<>","WISH"

CommandSet2:
DATA "GO","ENTER","READ","EXAMINE","LOOK","USE","GET","TAKE","DROP","PUT"
DATA "SELL","PAWN","BREAK","SMASH","CLOSE","OPEN","DRAW","WIELD","HOLD","WEAR"
DATA "RETURN","CAST","BUY","LOCK","UNLOCK","PICKLOCK","FIX","REPAIR","RECHARGE","DRINK"
DATA "<>","THROW","STEAL","<>","BLESS","CURSE","FUEL","<>","<>","TALK"
DATA "PARLEY","BRIBE","OFFER","LOAD","FIRE","MOUNT","DISMOUNT","RIDE","HIDE","SEARCH"
DATA "HELP","IDENTIFY","<>","<>","KILL","ATTACK","HIT","STRIKE","BACKSTAB","CIRCLE"
DATA "FEINT","PARRY","THRUST","CHARGE","TURN","COUNTER","BEMUSE","BEFUDDLE","PUMMEL","RESIST"
DATA "PSIONIC","EAT","DRIVE","FLY","MOVE","LAUNCH","LEARN","LIGHT","SWIM","EXIT"
DATA "LEAVE","BEAT","PUNCH","BEWITCH","BEWILDER"

ConfigDATA:
DATA 12,4,6,3,9,6,10,5,10,3,5,4,9,6,5,4,11,5,6,3,11,4,6,3,10,3,7,3,9,6,8,3,125,125,125,125,250,250,250,250
DATA "Lord","Wizard","Assassin","Bard","Monk","Knight","Priest","Empress","Assistant DM","Dungeon Master"
DATA "Human","Elf","Gnome","Dwarf","Halfling","Half-elf","Half-orc","Ogre"
DATA "Fighter","Magic User","Thief","Cleric","Paladin","Ranger","Druid","Lady","Assistant DM","Dungeon Master"
DATA "Strength","Intelligence","Wisdom","Dexterity","Constitution","Piety","Charisma"
DATA "north","east","south","west","northeast","southeast","southwest","northwest","up","down","in","out"
DATA "blunt","pole","sharp","thrusting","good","neutral","evil","lawful","neutral","chaotic"

REM **** END INCLUDE FILE ****

'  main program. all processing starts here.
ON ERROR GOTO Error.Routine ' error trapping
RANDOMIZE TIMER ' reseed random generator
CALL Read.Config ' loads all necessary variables, and configuration
CALL Login ' logs in local or remote player
DO ' main input loop
    CALL Encounter.Monster ' search for random monster encounter
    CALL Health.Update ' increment/decrement player statistics
    CALL Monster.Attack ' search for active monsters that attack
    CALL New.Stats ' recalculate player statistics
    CALL Restricted.Login ' compute restrictive time
    CALL Rust.Weapon ' get weapon rusting rate
    CALL Steal.Treasure ' get treasure stealing rate
    CALL Sorter ' sorts player inventory
    IF Room.Rate = False THEN ' compare room rate reset
        CALL Status.Line(False) ' redisplay status line with player statistics
    END IF
    CALL Main.Input ' get and process player input
LOOP ' end main input loop
END ' prevent drop through of loop into routines

' global error routine
Error.Routine:
IF ERR = 70 THEN
    Share.Count = Share.Count + 1
    IF Share.Count < MaxViolations THEN
        RESUME
    END IF
    PRINT "Share violation!"
    PRINT "IDE Line: "; _ERRORLINE
    RESUME NEXT
END IF
IF Modem.Access THEN
    Modem.Error = ERR
    RESUME NEXT
END IF
COLOR 7, 0 ' reset color
PRINT "Error:" + STR$(ERR) ' print message
PRINT "IDE Line: "; _ERRORLINE
RESUME NEXT

REM * Configure routine:
REM * routine to read command line, dimension and load dynamic arrays,
REM * open files, read door file, and read monster class arrays.

SUB Read.Config
CALL Get.Command ' read command line
CALL Read.Arrays ' load runtime arrays
CALL Open.Files ' open work files
CALL Read.Door ' read door file
CALL Read.Monclass ' load monster class arrays
END SUB ' end routine to read configure

REM * routine to process action prompt input. all commands processed
REM * in this routine call most major routines.

SUB Main.Input
Graphics.Off = False ' ansi/color on
Outpt = Action.Prompt ' set prompt (normally Do? is brief, Action? is verbose)
Upper.Case = True ' set uppercase flag
Word.Wrap = True ' set word wrap
CALL IO.I ' get player command input
Upper.Case = False ' reset uppercase flag
Word.Wrap = False ' reset word wrap
IF LEN(FunctionBuffer) THEN ' process funtion key during remote player on
    FunctionKeyNumber = ASC(FunctionBuffer) ' convert function key to variable
    FunctionBuffer = Nul ' reset function key buffer to null
    CALL Function.Key(FunctionKeyNumber) ' process function key
    EXIT SUB ' exit routine
END IF ' end process function key
Inpt = LTRIM$(Inpt) ' strip leading blanks from command
IF Inpt = "?" THEN ' command is a question mark
    Graphics.Off = True ' ansi/color to white
    Outpt = "Type Hint for for short help, Help for detailed help," ' message
    CALL IO.O ' send help output
    Outpt = "Catalog for command list, or Commands for detailed command list."
    CALL IO.O ' send help output
    Graphics.Off = False ' set ansi/color on
    EXIT SUB ' return to main
END IF ' end question mark command
IF Inpt = "??" THEN ' check for brief help
    MaxArrayList = MaxCommands * 2
    REDIM Array.List(1 TO MaxArrayList) AS STRING * 16 ' dimension string array
    FOR Last.Command.Number = 1 TO MaxArrayList
        Array.List(Last.Command.Number) = Nul
    NEXT
    RESTORE CommandSet1 ' reset data pointer
    DataElement = False ' reset data read counter
    FOR DataNumber = 1 TO MaxArrayList ' loop to read all user commands
        READ Outpt ' read command
        Outpt = LCASE$(Outpt) ' lowercase command
        MID$(Outpt, 1, 1) = UCASE$(MID$(Outpt, 1, 1)) ' uppercase start
        IF RTRIM$(Outpt) <> "<>" THEN ' check command name
            DataElement = DataElement + 1 ' increment data read counter
            Array.List(DataElement) = Outpt ' store command
        END IF ' end check data read counter
    NEXT ' end command read loop
    FOR Sort1 = 1 TO MaxArrayList ' bubble sort loop
        FOR Sort2 = Sort1 + 1 TO MaxArrayList ' bubble sort inner loop
            IF Array.List(Sort1) > Array.List(Sort2) THEN ' compare values
                SWAP Array.List(Sort1), Array.List(Sort2) ' swap values
            END IF ' end compare values
        NEXT ' end inner sort
    NEXT ' end sort loop
    Item.Counter = False ' reset line count
    Line.Counter = 2 ' reset page count
    Outpt = Nul ' reset output line
    Previous.Item$ = Nul ' reset display item
    FOR DataNumber = 1 TO MaxArrayList ' loop to read all user commands
        DataItem$ = RTRIM$(Array.List(DataNumber)) ' store command
        IF Previous.Item$ <> DataItem$ THEN ' compare duplicates
            Previous.Item$ = DataItem$ ' store previous item
            Outpt = Outpt + DataItem$ + SPACE$(15 - LEN(DataItem$)) ' make output line
            Item.Counter = Item.Counter + 1
            IF Item.Counter = 5 THEN ' count line length
                Item.Counter = False ' reset line count
                Outpt = RTRIM$(Outpt) ' strip blanks
                Graphics.Off = True ' ansi color off
                CALL IO.O ' send output
                Line.Counter = Line.Counter + 1 ' increment page count
                IF Line.Counter >= User.PageLength THEN ' compare to page length
                    Line.Counter = 2 ' reset lines counted
                    CALL More.Prompt ' routine for more
                    IF No THEN ' check no reply
                        EXIT SUB ' exit routine
                    END IF ' end check reply
                END IF ' end check page count
            END IF ' end counter line length
        END IF ' end compare duplicates
    NEXT ' end command read loop
    ERASE Array.List ' clear temporary array
    EXIT SUB ' exit routine
END IF ' end check help
Sysop.Command = False ' set sysop entered command flag to false
IF LEFT$(Inpt, 1) = "!" THEN ' sysop entered a command (sysop prefix of !)
    Sysop.Command = True ' set sysop entered command flag to true
    IF Normal.User THEN ' verify player has no special character
        Sysop.Command = False ' reset sysop command flag
        Outpt = "That command is reserved for DMs only!" ' format message
        CALL IO.O ' send output
        EXIT SUB ' return to main
    END IF ' end verify normal player
    IF LEN(Inpt) <= 1 THEN ' length of command is too brief
        Sysop.Command = False ' reset sysop command flag
        Outpt = "That command is reserved for DMs only!" ' message
        CALL IO.O ' send output
        EXIT SUB ' return to main
    END IF ' end compare command length
    IF Inpt = "!?" THEN ' check for short dm help
        Outpt = "Brief list of DM commands:" ' make display message
        CALL IO.O ' send output
        Command.Count = False ' reset counter
        MaxArrayList = MaxSysopCommands * 2
        REDIM Array.List(1 TO MaxArrayList) AS STRING * 16 ' dimension string array
        FOR Last.Command.Number = 1 TO MaxArrayList
            Array.List(Last.Command.Number) = Nul
        NEXT
        Graphics.Off = True ' ansi color off
        RESTORE SysopCommands1 ' reset data pointer
        FOR Last.Command.Number = 1 TO MaxArrayList ' loop to read all sysop commands
            READ Outpt ' read command
            IF Outpt <> "<>" THEN ' check valid command
                SELECT CASE Last.Command.Number ' compare unique command
                    CASE 1, 2, 3, 4, 5, 6, 7, 9, 12, 14, 16, 17, 18 ' commands
                        Command.Count = Command.Count + 1 ' increment counter
                        Outpt = LCASE$(Outpt) ' lowercase command
                        MID$(Outpt, 1, 1) = UCASE$(MID$(Outpt, 1, 1)) ' uppercase start
                        ' store prepended DM prefixed command
                        Array.List(Command.Count) = "!" + Outpt
                END SELECT
            END IF ' end check command
        NEXT ' end command read loop
        FOR Sort1 = 1 TO MaxArrayList ' bubble sort loop
            FOR Sort2 = Sort1 + 1 TO MaxArrayList ' bubble sort inner loop
                IF Array.List(Sort1) > Array.List(Sort2) THEN ' compare values
                    SWAP Array.List(Sort1), Array.List(Sort2) ' swap values
                END IF ' end compare values
            NEXT ' end inner sort
        NEXT ' end sort loop
        FOR Last.Command.Number = 1 TO MaxArrayList ' loop to display all sysop commands
            Outpt = RTRIM$(Array.List(Last.Command.Number)) ' read DM command
            IF LEN(Outpt) THEN
                CALL IO.O ' send output
            END IF
        NEXT ' end command read loop
        ERASE Array.List ' clear temporary array
        EXIT SUB ' exit command routine
    END IF ' end check short dm help
END IF ' end compute sysop entered a command
Stored.Parsed.Command1 = Inpt ' store the original command entered
Inpt = UCASE$(Inpt) ' convert original command to upper case
Stored.Parsed.Command2 = Inpt ' store the upper case command
IF LEFT$(Inpt, 1) = "\" THEN ' verify command is the repeat command character
    User.Command = Previous.Command1 ' restore previous command
    Stored.Parsed.Command1 = Previous.Command2 ' restore previous parsed
    Stored.Parsed.Command2 = Previous.Command3 ' restore previous parsed
ELSE ' not the repeat command
    User.Command = Inpt ' store original command
    Previous.Command1 = User.Command ' store original command
    Previous.Command2 = Stored.Parsed.Command1 ' store original command
    Previous.Command3 = Stored.Parsed.Command2 ' store upper case command
END IF ' end compare repeat command
IF User.Command = Nul THEN ' command entered is null
    EXIT SUB ' return to main
END IF ' end command is null
Magic.Spell = False ' reset magic spell is being cast flag
Command.Name = User.Command ' store command entered
IF INSTR(Command.Name, " ") THEN ' check temporary command variable for space
    Command.Name = LEFT$(Command.Name, INSTR(Command.Name, " ") - 1) ' parse left of
END IF ' space and store in temporary variable
Parser = INSTR(User.Command, " ") ' set parsed flag if command has space
IF Parser = False THEN ' check command has no space/parameters
    IF Sysop.Command THEN ' command entered is sysop command
        RESTORE SysopCommands1 ' reset data pointer
        Command.Number = False ' set variable containing command number to false
        Command.Name = MID$(Command.Name, 2) ' strip leading ! from command
        FOR Last.Command.Number = 1 TO MaxSysopCommands ' loop to read all nine sysop commands
            READ Outpt ' read command
            IF LEFT$(Outpt, LEN(Command.Name)) = Command.Name THEN ' compare to
                Command.Number = Last.Command.Number ' and store command number
                OutX$ = Outpt ' store command from data
                EXIT FOR ' exit the loop, command found
            END IF ' end compare
        NEXT ' end data read loop
        IF Command.Number = False THEN ' no command found
            Outpt = "Unknown command. Type !? for help." ' message
            CALL IO.O ' send output
            EXIT SUB ' return to main
        END IF ' end check command variable
        Last.Command = OutX$ ' store data command
        Last.Command.Number = Command.Number ' store command number
        SELECT CASE Command.Number ' select routine to call
            CASE 1 ' abort function
                CALL Abort
            CASE 2 ' hint display
                CALL Out.File("dmhint.doc")
            CASE 3 ' summon random monster
                CALL Call.Monster
            CASE 4 ' editor routine
                CALL Editor ' editor routine
            CASE 5 ' read dm help
                Stored.Parsed.Command1 = "HELP" ' help text to search for
                CALL Read.Help(True) ' true equals read dmhelp
            CASE 6 ' toggle sysop invisible status
                CALL Toggle.Invisible
            CASE 7 ' enter room link edit routine
                CALL Link.Room
            CASE 9 ' display memory, free data space
                CALL Display.Memory
        END SELECT ' end select routine
    ELSE ' command entered is not sysop command
        IF User.Command = "OU" THEN ' compare player command to parse of OUT
            User.Command = "O" ' reset player command to abbreviation of out
        END IF ' end compare player command
        IF User.Command = "OUT" THEN ' compare player command to parse of OUT
            User.Command = "O" ' reset player command to abbreviation of out
        END IF ' end compare player command
        ' routine to verify command entered is direction
        CALL Get.Direction(Direction.Number)
        IF Direction.Number THEN ' returned variable true if direction
            CALL Go.Direction(Direction.Number) ' routine goes certain direction
            EXIT SUB ' return to main
        END IF ' end verify direction entered
        Command.Number = False ' set variable containing command number to false
        RESTORE CommandSet1 ' reset data pointer
        FOR Last.Command.Number = 1 TO MaxCommands ' loop to read all user commands
            READ Last.Command ' read command
            IF Command.Name = LEFT$(Last.Command, LEN(Command.Name)) THEN ' compare
                Command.Number = Last.Command.Number ' command from data, left part
                EXIT FOR ' of player command, set variable to command number and
            END IF ' exit data read loop
        NEXT ' next data item
        IF Command.Number = False THEN ' loop ended without any matching commands
            Graphics.Off = True ' ansi/color off
            Outpt = "Unknown command! Type ? for help or ?? for a brief list of all commands."
            CALL IO.O ' send output
            Graphics.Off = False ' ansi/color on
            EXIT SUB ' return to main
        END IF ' end command number
        Last.Command.Number = Command.Number ' set command variable to match
        SELECT CASE Command.Number ' select command number
            CASE 1 TO 4 ' attack commands
                Parsed.Command1 = Last.Monster ' restore last monster number attacked
                Last.Command.Number = 55 ' attack command number
                CALL Attack.Monster ' go attack the monster
            CASE 5, 6 ' display the current room
                CALL Display.Room
            CASE 7, 8 ' panic the character
                CALL Panic
            CASE 9 ' search the current room
                CALL Search.Room
            CASE 10, 51 ' update the player file
                CALL Share.Record(UserFile, User.Index) ' store player record
                Outpt = "Player file updated." ' message
                CALL IO.O ' send output
            CASE 11 ' display player information
                CALL Display.Stats
            CASE 12 ' display player inventory
                CALL Display.Inventory
            CASE 13 ' kill of the character
                CALL Suicide
            CASE 14 ' train character (must have 0 experience and enough gold)
                CALL Train ' train routine
            CASE 15 ' health command
                CALL Display.Health
            CASE 16 ' appeal for low stats/room w/o exits
                CALL Appeal
            CASE 17 TO 20 ' end session
                CALL Quit.Game
            CASE 21 ' experience
                CALL Display.Experience
            CASE 22 ' enter help
                Stored.Parsed.Command1 = "HELP" ' get help text
                CALL Read.Help(False) ' false equals help.dat file
            CASE 23 ' hide
                CALL Hide.User
            CASE 24 ' cast command
                CALL Cast.Spell(True) ' true equals enter chant
            CASE 25 ' list command
                CALL Weapon.List
            CASE 26 ' brief mode toggle/must be level 2 at least
                CALL Brief.Mode
            CASE 27 ' bank command
                CALL Bank
            CASE 28 TO 30 ' display time on/time left
                CALL Time.Online
            CASE 33 ' ansi on/off
                CALL Toggle.ANSI
            CASE 39 ' lunge attack command
                Parsed.Command1 = Last.Monster ' restore last monster attacked
                Last.Command.Number = 39 ' store lunge command number
                CALL Attack.Monster ' go attack the monster
            CASE 40 ' dodge attack command
                Parsed.Command1 = Last.Monster ' restore last monster attacked
                Last.Command.Number = 40 ' store dodge command number
                CALL Attack.Monster ' go attack the monster
            CASE 41 ' display top ten players/create ranklist.dat
                CALL Share.Record(UserFile, User.Index) ' store player record
                CALL Top.Ten ' generate top ten list
                CALL Read.Record(UserFile, User.Index) ' read player record
            CASE 42 ' bless w/o parameter
                CALL Bless.Self ' therefore bless self
            CASE 43 ' curse w/o parameters
                CALL Curse.Self ' therefore curse self
            CASE 44 ' change password
                CALL Change.PassWord ' call routine
                CALL Share.Record(UserFile, User.Index) ' store player record
            CASE 45 ' mail routine
                CALL Mail
            CASE 46 ' display list of all players
                CALL Share.Record(UserFile, User.Index) ' store player record
                CALL User.List ' generate player list
                CALL Read.Record(UserFile, User.Index) ' store player record
            CASE 47 ' page sysop
                CALL Page.Sysop
            CASE 48, 49 ' exit/leave room
                CALL Exit.Room
            CASE 50 ' climb/go up/up
                CALL Climb
            CASE 52, 53 ' relogin if player level >1
                CALL Relogin
            CASE 54 ' display preferences
                CALL Display.Prefs ' routine to display preferences
            CASE 55 ' change line length
                CALL Change.Linelength ' routine to change line length
            CASE 56 ' display catalog file
                CALL Out.File("catalog.doc") ' display text file
            CASE 57 ' toggle linefeed mode
                CALL Toggle.Linefeeds ' routine to toggle linefeed mode
            CASE 58 ' change pagelength
                CALL Change.Pagelength ' routine to change pagelength
            CASE 59 ' change echo mode
                CALL Toggle.Echo ' routine to toggle echo mode
            CASE 60 ' toggle word wrap
                CALL Toggle.Wordwrap ' routine to toggle word wrap
            CASE 31 TO 35 ' attack commands
                CALL Attack.Monster ' go attack monster
            CASE 61 ' realign once per character
                CALL Align
            CASE 64 TO 75 ' direction entered
                ' set direction of data set number to 1-12
                Direction.Number = Command.Number - 63
                ' get the direction and move player
                CALL Go.Direction(Direction.Number)
            CASE 76 ' display hint file
                CALL Out.File("hint.doc") ' display text file
            CASE 77 ' toggle inventory sorting preference
                CALL Sort.Inventory
            CASE 78 ' display summary file
                CALL Out.File("summary.doc") ' display text file
            CASE 83 ' reroll all character statistics once per character
                CALL Reroll.Character
            CASE 85 ' wish for an item
                CALL Wish.Item
        END SELECT
    END IF ' end compare player command
ELSE ' command has parameter(s)
    Parsed.Command1 = MID$(User.Command, Parser + 1) ' get first parsed part of
    Parsed.Command2 = Parsed.Command1 ' command and store remaining parsed
    ' truncate stored parsed command
    Stored.Parsed.Command1 = MID$(Stored.Parsed.Command1, INSTR(Stored.Parsed.Command1, " ") + 1)
    Stored.Parsed.Command2 = MID$(Stored.Parsed.Command2, INSTR(Stored.Parsed.Command2, " ") + 1)
    CALL Numeric ' store parse number
    IF Sysop.Command THEN ' command entered is sysop command
        RESTORE SysopCommands2 ' reset data pointer
        Command.Number = False ' set variable containing command number to false
        Command.Name = MID$(Command.Name, 2) ' strip leading ! from command
        FOR Last.Command.Number = 1 TO MaxSysopCommands ' loop to read all nine commands
            READ Outpt ' read command
            IF LEFT$(Outpt, LEN(Command.Name)) = Command.Name THEN ' compare to,
                Command.Number = Last.Command.Number ' set command number variable
                OutX$ = Outpt ' store command from data
                EXIT FOR ' exit loop, command found
            END IF ' end compare sysop command
        NEXT ' end loop through data
        IF Command.Number = False THEN ' if no matching command in variable
            Outpt = "Unknown command. Type !? for help." ' message
            CALL IO.O ' send output
            EXIT SUB ' return to main
        END IF ' end matching compare
        Last.Command = OutX$ ' store data command
        Last.Command.Number = Command.Number ' store command number
        SELECT CASE Command.Number ' select sysop command number
            CASE 2 ' get monster number and call
                CALL Summon.Monster
            CASE 3 ' get object/treasure name and discard
                CALL Discard.Object
            CASE 4 ' edit room and room monclass
                Change.Number = VAL(Parsed.Command1) ' room number parameter to edit
                IF Change.Number > False THEN ' check file bounds
                    IF Change.Number <= LOF(RoomFile) / LEN(RoomRecord) THEN ' file bounds
                        CALL Change.Room(Change.Number) ' edit room number
                    END IF ' end check room number range
                END IF ' end check room number range
            CASE 5 ' get object/treasure name
                CALL Drop.Object ' get from files, drop in room
            CASE 6 ' call DM help about command in parameter
                CALL DM.Help
            CASE 7 ' kill monster name in parameter
                CALL Kill.Monster
            CASE 8 ' reduce monsters by the value in number parameter
                CALL Reduce.Monsters
            CASE 9 ' teleport self/online player to room number in parameter
                CALL Teleport.User
        END SELECT ' end select sysop command
    ELSE ' not sysop command
        Command.Number = False ' set variable containing command number to false
        RESTORE CommandSet2 ' reset data pointer
        FOR Last.Command.Number = 1 TO MaxCommands ' loop to read all user commands
            READ Last.Command ' read command
            IF Command.Name = LEFT$(Last.Command, LEN(Command.Name)) THEN ' compare
                Command.Number = Last.Command.Number ' to and store command number
                EXIT FOR ' exit read data loop
            END IF ' end compare
        NEXT ' end data loop
        IF Command.Number = False THEN ' if command number not matched
            Graphics.Off = True ' ansi/color off
            Outpt = "Unknown command! Type ? for help or ?? for a brief list of all commands."
            CALL IO.O ' send output
            Graphics.Off = False ' ansi/color on
            EXIT SUB ' return to main
        END IF ' end command number match
        Last.Command.Number = Command.Number ' store command number
        SELECT CASE Command.Number ' select command number
            CASE 1, 2, 74, 79 ' enter/go object command
                CALL Enter.Object
            CASE 3 TO 5 ' display player information commands
                CALL Display.Information
            CASE 6 ' cast spell on target
                CALL Cast.Spell(False) ' false equals get chant
            CASE 7, 8 ' take command
                IF INSTR(Parsed.Command2, " ") THEN ' second parameter
                    CALL Take.From.Container ' must be from container
                ELSE ' no extra parameters
                    CALL Take.Object ' must be from room
                END IF ' end compare second parameter
            CASE 9, 10 ' drop command
                IF INSTR(Parsed.Command2, " ") THEN ' second parameter
                    CALL Drop.Into.Container ' must be into container
                ELSE ' no extra parameters
                    CALL Drop.Item ' must be into room
                END IF ' end compare second parameter
            CASE 11, 12 ' sell an item
                CALL Pawn.Shop
            CASE 13, 14 ' smash commands
                CALL Smash.Object
            CASE 15 ' close command
                CALL Close.Object
            CASE 16 ' open command
                CALL Open.Object
            CASE 17 TO 19 ' hold/wield command
                CALL Hold.Object
            CASE 20 ' wear command
                CALL Wear.Object
            CASE 21 ' return a weapon command
                CALL Return.Object
            CASE 22 ' cast spell on target
                CALL Cast.Spell(True) ' true equals no chant
            CASE 23 ' repair item
                CALL Weapons.Shoppe
            CASE 24 ' lock command
                CALL Lock.Object
            CASE 25, 26 ' unlock/picklock commands
                CALL Unlock.Object
            CASE 27 ' fix command
                CALL Fix.Object
            CASE 28 ' buy command
                CALL Weapons.Shop
            CASE 29 ' charge to repair magic item
                CALL Alchemist
            CASE 30 ' drink command
                CALL Drink.Potion
            CASE 32 ' throw command
                CALL Throw.Object
            CASE 33 ' steal command
                CALL Steal.Object
            CASE 35 ' bless a target
                CALL Bless.Object
            CASE 36 ' curse a target
                CALL Curse.Object
            CASE 37 ' fuel command
                CALL Fuel.Object
            CASE 40, 41 ' talk/parley commands
                CALL Talk.To.Monster
            CASE 42, 43 ' offer commands
                CALL Offer
            CASE 44 ' load command
                CALL Load.Object
            CASE 45 ' fire command
                CALL Fire.Object
            CASE 46 ' enter vehicle command
                CALL Enter.Vehicle
            CASE 48, 73 ' ride/go vehicle commands
                CALL Ride.Vehicle
            CASE 49 ' hide command
                IF INSTR(Parsed.Command2, " ") THEN ' second parameter
                    CALL Drop.Into.Container ' must be hide in container
                ELSE ' no extra parameters
                    CALL Drop.Item ' must be hide in room
                END IF ' end check hide parameters
            CASE 50 ' search command
                CALL Search.Object
            CASE 51 ' help w/ parameter of help topic
                CALL Read.Help(False) ' read help file
            CASE 52 ' identify/extended information
                CALL Identify.Object
            CASE 55 TO 58 ' attack routine
                Last.Command.Number = 55
                CALL Attack.Monster
            CASE 59 TO 70, 82 TO 85 ' attack routine
                CALL Attack.Monster
            CASE 71 ' psi mode, attack/defense
                CALL Psi.Mode
            CASE 72 ' eat command
                CALL Eat.Object
            CASE 75 ' move command
                CALL Move.Object
            CASE 76 ' launch command
                CALL Launch.Object
            CASE 77 ' learn spell, parameter equals spell name to learn
                CALL Learn.Spell ' prompts for spell chant
            CASE 78 ' light command
                CALL Light.Object
            CASE 47, 80, 81 ' exit vehicle commands
                CALL Exit.Vehicle
        END SELECT ' end select commmand number
    END IF ' end command type
END IF ' end parameter type
END SUB ' end command processing

REM * routine to logoff player and terminate program.
REM * any call to this routine results in program termination.

SUB Terminate.Program
Logged.In = False ' reset player logged in flag
CALL Status.Line(-2) ' erase status lines
CALL Put.User.Record ' record player record structure
CALL Clean.Room ' remove extra treasure from room
Graphics.Off = False ' reset color
Outpt = "Your online time was:" ' make message
CALL IO.O ' display message
Graphics.Off = True ' reset color
CALL Time.Online ' display time used and time left
CALL More.Prompt ' pause for more
CALL Top.Ten ' top player list
CALL Out.File("logoff.dat") ' display logoff text file
Outpt = "Press " + Enter$ + " to exit the adventure:" ' exit program prompt
No.Echo = True ' no echo of player input
Line.Length = 1 ' get one character
CALL IO.I ' get player input
No.Echo = False ' reset no echo player input
CALL IO.O ' empty C/R
Outpt = "Returning you to " + BBS.Name + ".." ' display return message
CALL IO.O ' send output
CALL Reset.ANSI ' restore local color and remote ansi color
SLEEP 5 ' pause for 5 seconds
SYSTEM ' terminate program
END SUB ' end terminate program routine

REM * routine to display current time, user's time on, and user's time left.
REM * input variables:
REM *   Timelogged.On - containing the user time on in system time format hh:mm:ss.
REM *   Time.Left - containing the user's time limit in seconds from login.
REM * processing variables:
REM *   OnTime1 - contains the time calculation.
REM *   Hours1 - contains hours since login.
REM *   Minutes1 - contains minutes since login.
REM *   Seconds1 - contains seconds since login.

SUB Time.Online
Graphics.Off = True ' reset color
Outpt = "It is now " + clock$ + "." ' make display message
CALL IO.O ' send output message
Var1! = VAL(MID$(TIME$, 1, 2)) * 3600! + VAL(MID$(TIME$, 4, 2)) * 60! + VAL(MID$(TIME$, 7, 2))
Var2! = VAL(MID$(Timelogged.On$, 1, 2)) * 3600! + VAL(MID$(Timelogged.On$, 4, 2)) * 60! + VAL(MID$(Timelogged.On$, 7, 2))
OnTime! = Var1! - Var2!: IF OnTime! < 0! THEN OnTime! = OnTime! + 86400!
OnTime2! = OnTime!
Outpt = "You have been on for" ' format time display message
Time.Display$ = Nul ' time display message
GOSUB Time.Display ' subroutine to display message
OnTime! = Time.Left! - OnTime2!
Outpt = "You have" ' format time display message
Time.Display$ = " remaining" ' time display message
GOSUB Time.Display ' subroutine to display message
EXIT SUB ' exit routine

' subroutine to display time message
Time.Display:
Hour1 = INT(OnTime! / 3600!): OnTime! = OnTime! - Hour1 * 3600!
Minute1 = INT(OnTime! / 60!): OnTime! = OnTime! - Minute1 * 60!: Second1 = OnTime!
IF Hour1 > 0 THEN ' compare hours of time variable
    IF Hour1 = 1 THEN
        Outpt = Outpt + " 1 hour,"
    ELSE
        Outpt = Outpt + STR$(Hour1) + " hours," ' append hours to string
    END IF
END IF ' end compare hours
IF Minute1 > 0 THEN ' compare minutes of time variable
    IF Minute1 = 1 THEN
        Outpt = Outpt + " 1 minute,"
    ELSE
        Outpt = Outpt + STR$(Minute1) + " minutes," ' append minutes to string
    END IF
END IF ' end compare minutes
IF Second1 > 0 THEN ' compare seconds of time variable
    IF Second1 = 1 THEN
        Outpt = Outpt + " 1 second,"
    ELSE
        Outpt = Outpt + STR$(Second1) + " seconds," ' append seconds to string
    END IF
END IF ' end compare seconds
IF Hour1 <= 0 AND Minute1 <= 0 AND Second1 <= 0 THEN
    Outpt = Outpt + " 0 seconds."
END IF
Outpt = LEFT$(Outpt, LEN(Outpt) - 1) ' trim trailing comma
Outpt = Outpt + Time.Display$ + "." ' combine message
CALL IO.O ' send output message
RETURN ' exit time display subroutine
END SUB ' end routine to display time on

REM * routine to allow player login with another codename.

SUB Relogin
IF Normal.User THEN ' check non DM
    IF UserRecord.Level <= 1 THEN ' minimum player level requirement to relogin
        Outpt = "You can't relogin until level two!" ' display restriction
        CALL IO.O ' send output
        EXIT SUB ' return from routine
    END IF ' end check minimum level
END IF ' end check normal player
Outpt = "Are you sure you want to relogin(y/n)? " ' verify player relogin
No.Input.Out = "N" ' default input to no
CALL IO.I ' get player input
IF Yes THEN ' verified to suicide
    Logged.In = False ' reset player logged in flag
    CALL Put.User.Record ' store player record structure
    CALL Clean.Room ' remove extra treasure from room
    CALL Login ' get player codename
END IF ' end check response
END SUB ' end relogin routine

REM * routine allows player to erase character.
REM * sometimes player characters are no longer useful
REM * when their statistics become too low.

SUB Suicide
Outpt = "Are you sure you want to commit suicide(y/n)? " ' verify player suicide
No.Input.Out = "N" ' default input to no
CALL IO.I ' get player input
IF Yes THEN ' verified to suicide
    Logged.In = False ' reset player logged in flag
    Graphics.Off = True ' ansi/color to white
    Outpt = "Your character falls into deep sleep.." ' display suicide message
    CALL IO.O ' send output
    Outpt = "   The Ghods take your player to another world.." ' display
    CALL IO.O ' send output
    Graphics.Off = False ' ansi/color to normal
    CALL Delete.User ' remove player record from file
    CALL Share.Record(UserFile, User.Index) ' put deleted record in player file
    CALL Terminate.Program ' end program
END IF ' end check response
END SUB ' end suicide routine

REM * routine for normal logoff.

SUB Quit.Game
Graphics.Off = True ' ansi/color to white
Outpt = "Are you sure you want to quit(y/n)? " ' verify player to quit
No.Input.Out = "N" ' default to no
CALL IO.I ' get input
IF Yes THEN ' verified to quit
    Logged.In = False ' reset player logged in flag
    CALL Terminate.Program ' end program
END IF ' end check response
END SUB ' end quit routine

REM * routine to quit and terminate quick.

SUB Abort
Outpt = "Are you sure you want to abort(y/n)? " ' verify abort
No.Input.Out = "N" ' default to no
CALL IO.I ' get input
IF Yes THEN ' verified yes
    Logged.In = False ' reset player logged in flag
    Outpt = "Program task abort!" ' display abort message
    CALL IO.O ' output
    CALL Put.User.Record ' store player record structure
    CALL Reset.ANSI ' restore ansi/color
    CALL Status.Line(-2) ' remove status lines
    SLEEP 5 ' pause for 5 seconds
    SYSTEM ' end program
END IF ' end check response
END SUB ' end abort routine

REM * routine reads command line node number 0-9, A-Z, 10-99, and switches.

SUB Get.Command
LOCATE , , 1 ' turn on cursor
Max.Row = 24 ' set maximum rows on screen before scroll
Local.Mode = False ' default not local mode
Node1 = False ' reset node number
Node2 = False ' reset node number
User.Echo = False ' reset preference
User.LineFeeds = False ' reset preference
User.LineLength = 80 ' reset preference
User.PageLength = 24 ' reset preference
User.Wordwrap = False ' reset preference
DO
    Command.Line$ = UCASE$(COMMAND$) ' store command line
    DO ' loop to remove spaces form command
        Imbedded = INSTR(Command.Line$, " ") ' check for space
        IF Imbedded THEN ' verify space
            Command.Line$ = LEFT$(Command.Line$, Imbedded - 1) + MID$(Command.Line$, Imbedded + 1) ' remove space
        ELSE ' verify space
            EXIT DO ' exit no space
        END IF ' end check space
    LOOP ' end loop to remove spaces
    IF Command.Line$ = "" OR Command.Line$ = "/?" THEN ' check command
        EXIT DO ' jump to command display
    END IF ' end check command
    Imbedded = INSTR(Command.Line$, "/P:") ' check port over-ride switch
    IF Imbedded THEN ' check switch
        Store.Command$ = LEFT$(Command.Line$, Imbedded - 1) ' store command line
        Command.Line$ = MID$(Command.Line$, Imbedded + 3) ' get remaining command line
        GOSUB Get.Numeric ' convert command to value
        Command.Line$ = Store.Command$ + Command.Line$ ' restore command line
        Port.Value = INT(VAL(Value$)) ' store value
        IF Port.Value >= 1 AND Port.Value <= 16 THEN ' compare value range
            Port.Override = Port.Value ' store value
        END IF ' end compare
        IF Port.Override = False THEN ' check port value
            EXIT DO ' jump to usage
        END IF ' end check port
    END IF ' end check switch
    Imbedded = INSTR(Command.Line$, "/B:") ' check baud over-ride switch
    IF Imbedded THEN ' check switch
        Store.Command$ = LEFT$(Command.Line$, Imbedded - 1) ' store command line
        Command.Line$ = MID$(Command.Line$, Imbedded + 3) ' get remaining command line
        GOSUB Get.Numeric ' convert command to value
        Command.Line$ = Store.Command$ + Command.Line$ ' restore command line
        Baud.Value! = INT(VAL(Value$)) ' store value
        IF Baud.Value! > 0! THEN ' compare value range
            Baud.Override = CINT(Baud.Value! / 100!) ' store value
        END IF ' end compare
        IF Baud.Override = False THEN ' check baud value
            EXIT DO ' jump to usage
        END IF ' end check baud
    END IF ' end check switch
    Imbedded = INSTR(Command.Line$, "/L") ' check command
    IF Imbedded THEN ' verify command is local mode
        ' take switch from command
        Command.Line$ = LEFT$(Command.Line$, Imbedded - 1) + MID$(Command.Line$, Imbedded + 2)
        Command.Line$ = RTRIM$(Command.Line$) ' trim command
        Command.Line$ = LTRIM$(Command.Line$) ' trim command
        Local.Mode = True ' reset local mode
    END IF ' end verify command switch
    IF LEN(Command.Line$) = False THEN ' check command
        IF Local.Mode OR Port.Override THEN ' check switches
            Node = "<n/a>" ' assign default node type
            EXIT SUB ' exit from command line routine
        END IF ' end check switches
    END IF ' end check command
    IF LEFT$(Command.Line$, 1) = "/" THEN ' check for command switch
        Command.Line$ = MID$(Command.Line$, 2) ' store command line
        SELECT CASE LEN(Command.Line$) ' get length of command line node
            CASE 1 ' command line length
                SELECT CASE Command.Line$ ' get command line node number
                    CASE "0" TO "9", "A" TO "Z" ' command line node
                        Node1 = ASC(Command.Line$) ' assign to node type 1 variable
                END SELECT ' end command line node number
            CASE 2 ' command line length
                SELECT CASE INT(VAL(Command.Line$)) ' select extended node number
                    CASE 10 TO 99 ' command line node
                        Node2 = INT(VAL(Command.Line$)) ' assign to node type 2 variable
                END SELECT ' end select node number
        END SELECT ' end select node number
    END IF ' end check command
    IF Node1 > False OR Node2 > False THEN ' verify valid node number
        EXIT SUB ' exit routine
    ELSE
        EXIT DO
    END IF ' end check node number
LOOP
COLOR 15, 0 ' color white on black
PRINT "Dnddoor usage:"
COLOR 14, 0 ' color yellow on black
PRINT "   Dnddoor [/#] [/L] [/P:x] [/B:<rate>] [/A:<base>]"
COLOR 15, 0 ' color white on black
PRINT "Where:"
COLOR 14, 0 ' color yellow on black
PRINT "   /# is door node number 0 - 9, A - Z, or 10 - 99."
PRINT "   /L to logon without door support."
PRINT "   /P:x  over-ride port number (1 to 16)."
PRINT "   /B:<rate>  over-ride connect baud rate."
COLOR 7, 0 ' color white on black
END ' halt program

' routine to convert string to value
Get.Numeric:
Value$ = Nul ' reset value
DO ' start convert loop
    Line.Value$ = LEFT$(Command.Line$, 1) ' get command line character
    IF Line.Value$ >= "0" AND Line.Value$ <= "9" THEN ' check ascii value
        Value$ = Value$ + Line.Value$ ' store value
        Command.Line$ = MID$(Command.Line$, 2) ' get next command character
    ELSE ' check ascii value
        EXIT DO ' exit routine loop
    END IF ' end check ascii value
LOOP ' end convert loop
RETURN ' exit form routine
END SUB ' end routine to read command line

REM * routine reads runtime arrays.

SUB Read.Arrays
' allocate runtime arrays
REDIM Alignment.Name1(1 TO 3) AS STRING * 7
REDIM Alignment.Name2(1 TO 3) AS STRING * 7
REDIM Class.Name(1 TO 10) AS STRING * 15
REDIM Direction(1 TO 12) AS STRING * 9
REDIM High.Class.Name(1 TO 10) AS STRING * 15
REDIM Race(1 TO 8) AS STRING * 8
REDIM Stat(1 TO 7) AS STRING * 12
REDIM Training.Stats(1 TO 10, 1 TO 4) AS INTEGER
REDIM WeaponTypeName(1 TO 4) AS STRING * 9
RESTORE ConfigDATA
FOR Array.Index1 = 1 TO 10 ' loop through ten class statistic increment array
    FOR Array.Index2 = 1 TO 4 ' loop through each four class statistic array
        READ Training.Stats(Array.Index1, Array.Index2) ' read statistics
    NEXT ' loop through
NEXT ' loop through
FOR Array.Index2 = 1 TO 10 ' loop through high class names
    READ High.Class.Name(Array.Index2) ' read high class names
NEXT ' loop through
FOR Array.Index2 = 1 TO 8 ' loop through race names
    READ Race(Array.Index2) ' read race names
NEXT ' loop through
FOR Array.Index2 = 1 TO 10 ' loop through class names
    READ Class.Name(Array.Index2) ' read class names
NEXT ' loop through
FOR Array.Index2 = 1 TO 7 ' loop through statistic names
    READ Stat(Array.Index2) ' read statistic names
NEXT ' loop through
FOR Array.Index2 = 1 TO 12 ' loop through direction names
    READ Direction(Array.Index2) ' read direction names
NEXT ' loop through
FOR Array.Index2 = 1 TO 4 ' loop through weapon proficiency names
    READ WeaponTypeName(Array.Index2) ' read weapons proficiency names
NEXT ' loop through
FOR Array.Index2 = 1 TO 3 ' loop through first set of alignment names
    READ Alignment.Name1(Array.Index2) ' read alignment names
NEXT ' loop through
FOR Array.Index2 = 1 TO 3 ' loop through second set of alignment names
    READ Alignment.Name2(Array.Index2) ' read alignment names
NEXT ' loop through
END SUB ' end routine to read arrays

REM * routine opens runtime file handles.

SUB Open.Files
CLOSE ' close all files
' files opened in random mode, opened in share mode for multi access, and each
' file record length specified as the length of the random record which will
' be used to get/put data to the file. 10 files, 2 mail files. Remaining unopened
' files include 2 temp files, 1 help file.
OPEN "actions.dat" FOR RANDOM SHARED AS #ActionFile LEN = LEN(ActionRecord)
OPEN "messages.dat" FOR RANDOM SHARED AS #MessageFile LEN = LEN(MessageRecord)
OPEN "montalk.dat" FOR RANDOM SHARED AS #MonTalkFile LEN = LEN(MonsterTalkRecord)
OPEN "monclass.dat" FOR RANDOM SHARED AS #MonClassFile LEN = LEN(MonclassRecord)
OPEN "monsters.dat" FOR RANDOM SHARED AS #MonsterFile LEN = LEN(MonsterRecord)
OPEN "msgtable.dat" FOR RANDOM SHARED AS #TableFile LEN = LEN(TableRecord)
OPEN "nonplyrs.dat" FOR RANDOM SHARED AS #NonPlayerFile LEN = LEN(MonsterRecord)
OPEN "objects.dat" FOR RANDOM SHARED AS #ObjectFile LEN = LEN(ObjectRecord)
OPEN "rooms.dat" FOR RANDOM SHARED AS #RoomFile LEN = LEN(RoomRecord)
OPEN "spells.dat" FOR RANDOM SHARED AS #SpellFile LEN = LEN(SpellRecord)
OPEN "treasure.dat" FOR RANDOM SHARED AS #TreasureFile LEN = LEN(TreasureRecord)
OPEN "users.dat" FOR RANDOM SHARED AS #UserFile LEN = LEN(UserRecord)
END SUB ' end routine to open files

REM * routine makes door filename.

SUB Read.Door
BBS.Name = Nul ' reset name of bbs
Color.Graphics = True ' default remote ansi color on
Door.Name = Nul ' reset name of user
Door.Time = 1800! ' reset time remaining
IF Node1 THEN ' check node type 1
    Node = CHR$(Node1) ' store node in string form
ELSE ' check door node
    IF Node2 THEN ' check node type 2
        Node = MID$(STR$(Node2), 2) ' store node in string form
    END IF ' end check node
END IF ' end check node type
IF Local.Mode THEN ' verify local command line switch
    BBS.Name = "SYSTEM" ' store name of bbs
    Port = False ' reset modem port
    Modem.Baud = 96 ' reset modem baud
    Door.Name = "LOCAL SYSOP" ' store name of user
    Color.Type = 2 ' store color value
    EXIT SUB ' exit door file routine
END IF ' end verify local switch
IF Node1 = False AND Node2 = False THEN ' check node type
    BBS.Name = "SYSTEM" ' store name of bbs
    Port = False ' reset modem port
    Modem.Baud = 96 ' reset modem baud
    Door.Name = "REMOTE USER" ' store name of user
    Color.Type = 2 ' store color value
    IF Port.Override THEN ' verify port number for no node
        Port = Port.Override - 1 ' store port value
    END IF ' end check port/node
    IF Baud.Override THEN ' check baud value for no node
        Modem.Baud = Baud.Override ' store baud value
    END IF ' end check baud/node
    CALL Open.Modem ' init baud rate
    EXIT SUB ' exit routine
END IF ' end check node type
IF LEN(Node) = 1 THEN ' check node length
    FileName = "DORINFO" + Node + ".DEF" ' make door info filename
    IF _FILEEXISTS(FileName) THEN ' check file exists
        CALL Read.Door.File ' routine to read door file
        EXIT SUB ' exit door file routine
    END IF ' end check file
END IF ' end check node length
FileName = "DINF" + Node + ".DEF" ' make door info filename
IF _FILEEXISTS(FileName) THEN ' check file exists
    CALL Read.Door.File ' routine to read door file
    EXIT SUB ' exit door file routine
END IF ' end check file
END SUB ' end routine to read door file

REM * routine reads door file.

SUB Read.Door.File
CLOSE #TempFile ' close work file
OPEN FileName FOR INPUT SHARED AS #TempFile ' open the door file for input
FOR Door.File.Line = 1 TO 13 ' loop through all lines in door file
    LINE INPUT #TempFile, Inpt ' read next text line from door file
    Inpt = UCASE$(Inpt) ' convert text string to upper case
    SELECT CASE Door.File.Line ' select which line to process
        CASE 1 ' line one
            BBS.Name = Inpt ' store name of bbs
        CASE 4 ' line four
            Port = VAL(MID$(Inpt, 4, 1)) ' set port to value of rightmost character
            IF Port = False THEN ' port zero speficies local console mode
                Local.Mode = True ' set local mode flag
            ELSE ' other port number
                Port = Port - 1 ' convert modem comm (1-8) to modem port (0-7)
                IF Port.Override THEN ' check over-ride switch
                    Port = Port.Override - 1 ' store value
                END IF ' end check switch
            END IF ' end compare port number
        CASE 5 ' line five
            Input.Length = INSTR(Inpt, " ") - 3 ' get baud rate
            Inpt = LEFT$(Inpt, Input.Length) ' store rate
            Modem.Baud = INT(VAL(Inpt)) ' store modem baud rate
            IF Baud.Override THEN ' check over-ride switch
                Modem.Baud = Baud.Override ' store value
            END IF ' end check switch
        CASE 7 ' line seven
            Door.Name = Inpt ' store name of user
        CASE 8 ' line eight
            IF LEN(Inpt) THEN ' compare length of user last name
                Door.Name = Door.Name + " " + Inpt ' append user last name
            END IF ' end compare user name length
        CASE 10 ' line ten
            Color.Type = INT(VAL(Inpt)) ' get value of text line
            IF Color.Type < 2 THEN ' compare graphics preference
                Color.Graphics = False ' value other than two sets graphics off
            END IF ' end compare graphcis
        CASE 12 ' line twelve
            Door.Time = VAL(Inpt) * 60! ' store value of text line of remaining time
    END SELECT ' end process line
NEXT ' end loop through file
CALL Open.Modem
END SUB ' end routine to read door file

REM * routine loads runtime file info.

SUB Read.Monclass
LOCATE , , 1 ' turn on cursor
Monclass.Max = LOF(MonClassFile) / LEN(MonclassRecord) ' compute number of records
IF Monclass.Max = False THEN ' monster class file, compare to minimum
    Monclass.Max = 1 ' and increment if none found
    FOR Monster.Classes = 1 TO 10 ' loop through all monsters in class
        CALL PutMonclassRecordMonsters(Monster.Classes, False) ' reset to empty
    NEXT ' end loop
    CALL Share.Record(MonClassFile, 1) ' write first monster class record if empty
END IF ' end compare length of file
' check upper bounds on monclass file
IF Monclass.Max > 1024 THEN ' check bounds
    Monclass.Max = 1024 ' reset maximum
END IF ' end check bounds
' allocate monster class, rate, and percent arrays
REDIM Monster.Class(1 TO Monclass.Max, 1 TO 10) AS INTEGER
REDIM Monster.Percent(1 TO Monclass.Max, 1 TO 10) AS INTEGER
REDIM Monster.Rate(1 TO Monclass.Max, 1 TO 10) AS INTEGER
' loop through all monster classes in file
FOR Monster.Classes = 1 TO Monclass.Max
    CALL Read.Record(MonClassFile, Monster.Classes) ' read monster class record
    FOR Monster.Class.Number = 1 TO 10 ' loop through all monsters in record
        ' get monster number in class
        Monster.Array.Number = GetMonclassRecordMonsters(Monster.Class.Number)
        ' store in monster class array
        Monster.Class(Monster.Classes, Monster.Class.Number) = Monster.Array.Number
        IF Monster.Array.Number > False AND Monster.Array.Number <= LOF(MonsterFile) / LEN(MonsterRecord) THEN
            ' read monster number from monster file
            CALL Read.Record(MonsterFile, Monster.Array.Number)
            ' store encounter rate
            Monster.Rate(Monster.Classes, Monster.Class.Number) = MonsterRecord.Rate
            ' store encounter rate percent
            Monster.Percent(Monster.Classes, Monster.Class.Number) = MonsterRecord.RatePercent
        END IF ' end compare file bounds
    NEXT ' end loop through ten monsters in class
NEXT ' end loop through all monster classes
Max.Spells = LOF(SpellFile) / LEN(SpellRecord) ' get number of spell records
' check bounds of spell file
IF Max.Spells > 1024 THEN ' check bounds
    Max.Spells = 1024 ' reste maximum
END IF ' end check bounds
Spell.Number = LEN(Learned.Spells) ' get length of learned spells array
IF Max.Spells > Spell.Number THEN ' compare spells length
    ' extended array
    Learned.Spells = Learned.Spells + STRING$(Max.Spells - Spell.Number, "0")
END IF ' end compare length
END SUB ' end configure routine

REM * routine matches online baud rate
REM * and opens modem as file.

SUB Open.Modem
IF Local.Mode THEN ' check online mode
    _TITLE "DNDDOOR - LOCAL"
    EXIT SUB ' return from routine
END IF ' end check online mode
Modem.Speed$ = "96" ' default baud
SELECT CASE Modem.Baud ' select baud rate to match
    CASE 3 ' 300 baud
        Modem.Speed$ = "3" ' calculate baud
    CASE 6 ' 600 baud
        Modem.Speed$ = "6" ' calculate baud
    CASE 12 ' 1200 baud
        Modem.Speed$ = "12" ' calculate baud
    CASE 24, 48 ' 2400 baud
        Modem.Speed$ = "24" ' calculate baud
    CASE 96, 144 ' multiple bauds
        Modem.Speed$ = "96" ' calculate baud
    CASE 192, 288 ' multiple bauds
        Modem.Speed$ = "192" ' calculate baud
    CASE 384, 576, 786 ' multiple bauds
        Modem.Speed$ = "192" ' calculate baud
    CASE 1152, 2304 ' multiple bauds
        Modem.Speed$ = "1152" ' calculate baud
END SELECT ' end calculate baud

' open modem as file
Modem.Access = True
Modem.Error = False
P$ = "COM" + MID$(STR$(Port + 1), 2) + ":"
FileName$ = P$ + Modem.Speed$ + "00,N,8,1,CD1"
CLOSE ModemFile
OPEN FileName$ FOR RANDOM AS #ModemFile
IF Modem.Error THEN
    Modem.Access = False
    CALL Lost.Carrier
    END
END IF
PUT #ModemFile, 0 ' test modem
Modem.Access = False
IF Modem.Error THEN
    CALL Lost.Carrier
    END
END IF
_TITLE "DNDDOOR - " + P$
END SUB

REM * standard output routine to send data to the screen and modem.
REM * input variables:
REM *   Allow.Break - set to true to allow control-k checking.
REM *   Carriage.Return - set to true to suppress cr/lf after output.
REM *   Outpt - contains text to output to screen/modem.
REM * output variables:
REM *   Break - set to true if user entered control-k.
REM *   Carriage.Return, Outpt - reset to zero/null.
REM * processing variables:
REM *   Char.Output$ - character to send to modem/screen.
REM *   Count - loop variable for character of output sent.

SUB IO.O
IF LEN(Outpt) THEN ' check length of output string
    CALL Out.ANSI ' routine to output ansi sequence to modem/color code locally
END IF ' end check output string
Count = False ' reset output character counter
DO ' output characters loop
    CALL Keyboard(Keyboard.Break) ' routine reads keyboard
    IF Break THEN ' check break flag
        EXIT DO ' exit output loop
    END IF ' end check break flag
    Count = Count + 1 ' increment output counter
    GOSUB IO.Wrap ' subroutine to word wrap
    IF Count > LEN(Outpt) THEN ' check length of output
        EXIT DO ' exit output loop
    END IF ' end check length of output
    Char.Output$ = MID$(Outpt, Count, 1) ' get single character to output
    CALL Put.Modem(Char.Output$) ' send character to modem
    CALL Scrn(Char.Output$) ' send character to screen
LOOP ' output character loop
IF Carriage.Return = False THEN ' check output cr/lf suppressed
    IF Break = False THEN ' check control-k flag
        CALL Line.Return ' routine to send cr/lf to modem/screen
    END IF ' end check control-k flag
END IF ' end check i/o cr/lf
Carriage.Return = False ' reset supressed cr/lf
Outpt = Nul ' reset output
EXIT SUB ' exit routine

' subroutine to word wrap
IO.Wrap:
IF User.LineLength = False THEN ' check linelength
    RETURN ' exit subroutine
END IF ' end check of linelength
SELECT CASE User.Wordwrap ' check word wrap preference
    CASE True ' word wrap toggled off
        IF POS(0) >= User.LineLength THEN ' check line length
            IF MID$(Outpt, Count, 1) = " " THEN ' check wrapped space
                Count = Count + 1 ' increment space from loop
            END IF ' end check wrapped space
            IF Count <= LEN(Outpt) THEN ' check end of line
                CALL Line.Return ' send cr/lf
            END IF ' check eol
        END IF ' end check line length
    CASE False ' word wrap toggled on
        Space.Wrap = INSTR(Count + 1, Outpt, " ") ' store next word
        IF Space.Wrap THEN ' find first imbedded space
            Word$ = MID$(Outpt, Count) ' truncate word
            Space.Wrap = INSTR(2, Word$, " ") ' store next word
            Word$ = LEFT$(Word$, Space.Wrap - 1) ' store remaining output of line
        ELSE ' store empty remaining word to wrap
            IF MID$(Outpt, Count, 1) = " " THEN ' verify last possible word
                Word$ = MID$(Outpt, Count) ' store last remaining word
            ELSE ' calculate last word
                Word$ = Nul ' store empty word
            END IF ' end calculate last word
        END IF ' end storing next word to send
        ' compute line position plus next word
        IF POS(0) + LEN(Word$) > User.LineLength THEN
            IF MID$(Outpt, Count, 1) = " " THEN ' check next word is space
                Count = Count + 1 ' increment space from loop
            END IF ' end check next word is space
            CALL Line.Return ' next word wraps, send cr/lf
        END IF ' end compute wrap length
END SELECT ' end select word wrap preference
RETURN ' exit subroutine
END SUB ' end routine to send a string to the modem and screen

REM * standard input routine to get data from the keyboard and modem.
REM * input variables:
REM *   Buffer - any previous user input stored during output.
REM *   Carriage.Return - set to true to suppress cr/lf after output.
REM *   Hide - flag to echo mask characters in place of input.
REM *   Line.Length - maximum characters allowed to input.
REM *   No.Echo - do not echo user input.
REM *   No.Input.Out - string to use if no input entered.
REM *   Outpt - contains input prompt.
REM *   Word.Wrap - allow words entered to wrap.
REM * output variables:
REM *   Inpt - contains user input string.
REM *   No.Input - flag set if return entered.
REM *   No - set if N was entered.
REM *   Yes - set if Y was entered.
REM *   Quit - set if Q was entered.
REM * processing variables:
REM *   Buffered.Input$ 0 stores buffered input to add to next input prompt.
REM *   Char$ - current character received from modem/keyboard.
REM *   Char - ascii code of character input.
REM *   Line.Limit - stores length of input line during input.
REM *   Time.Out! - current time character input loop entered.
REM *   Word - set if a word wrapped.
REM *   OutY$, Valid.Char$, Last.Wrap, Last.Word - local work variables.

SUB IO.I
STATIC Buffered.Input$ ' stores data input between calls to input routine
IF POS(0) > 1 THEN ' check io supressed cr/lf
    CALL Line.Return ' make cr/lf before input prompt
END IF ' end check previous supressed cr/lf
Carriage.Return = True ' set cr/lf flag
Line.Limit = LEN(Outpt) ' reset length of line for io.o
Outpt = Outpt + Buffered.Input$ ' add stored data input to input prompt
CALL IO.O ' output the input prompt
Inpt = Buffered.Input$ ' reset input string+data input between calls
Buffered.Input$ = Nul ' reset data input between calls
DO ' loop through input characters
    Char$ = Nul ' reset input character
    Time.Out! = TIMER ' set time loop entered
    DO WHILE Char$ = Nul ' loop until received a character
        GOSUB IO.Timing ' routine to calculate timing values
        CALL Keyboard(Keyboard.Break) ' routine to get any keyboard input
        ' keyboard input received is a function key/extended key
        IF Keyboard.Break THEN
            Buffer = Nul ' reset type ahead buffer
            Carriage.Return = False ' reset cr/lf flag
            No.Input.Out = Nul ' reset null input echo string
            EXIT SUB ' exit routine to process extended keyboard input
        END IF ' end check keyboard routine flag
        CALL Get.Modem(Input.Char) ' routine to get character from modem
        IF Input.Char THEN ' modem input in return variable
            ' append modem input to buffer variable
            Buffer = Buffer + CHR$(Input.Char)
        END IF ' end check modem for character
        IF LEN(Buffer) THEN ' check buffer contents
            Char$ = LEFT$(Buffer, 1) ' get first buffer character
            Buffer = MID$(Buffer, 2) ' store buffer minus first character
        END IF ' end check buffer
        'Call Release.Time(1) ' give up time slice
    LOOP ' end loop until character received
    Char = ASC(Char$) ' convert character string to ascii integer
    SELECT CASE Char ' process character ascii integer
        CASE 8 ' backspace
            IF LEN(Inpt) THEN ' check length of input
                Inpt = LEFT$(Inpt, LEN(Inpt) - 1) ' strip off last character from input
                CALL Back.Space ' routine to send backspace codes
            END IF ' end check string length
        CASE 13 ' return key
            IF Inpt = Nul THEN ' no input in string
                IF LEN(No.Input.Out) THEN ' check null input echo string
                    Inpt = No.Input.Out ' set input string to default input string
                    OutY$ = LCASE$(No.Input.Out) ' convert null input string
                    CALL Scrn(OutY$) ' send string to screen
                    IF User.Echo = False THEN ' check echo mode
                        CALL Put.Modem(OutY$) ' send string to modem
                    END IF ' end check echo mode
                END IF ' end check null input flag
            END IF ' end check null input
            CALL Scrn(CHR$(13)) ' send carriage return to screen
            IF User.Echo = False THEN ' check echo mode
                CALL Put.Modem(CHR$(13)) ' send carriage return to modem
            END IF ' end check echo mode
            EXIT DO ' end routine after return key prssed/return input string
        CASE 32 TO 127 ' remaining valid input ascii codes
            IF Inpt = Nul THEN ' check length of stored input
                IF Upper.Case THEN ' verify force uppercase
                    Char$ = UCASE$(Char$) ' change first character to uppercase
                END IF ' end verify force
            END IF ' end check length
            Valid.Char$ = Char$ ' store character
            IF No.Echo = False THEN ' check echo flag
                IF Hidden THEN ' check hide flag
                    Valid.Char$ = Mask$ ' echo mask character
                END IF ' end check hide flag
                IF User.Echo = False THEN ' check echo mode
                    CALL Put.Modem(Valid.Char$) ' echo character entered to modem
                END IF ' end check echo mode
                IF Word.Wrap = False THEN ' check word wrap flag
                    CALL Scrn(Valid.Char$) ' echo character entered to screen
                END IF ' end check word wrap flag
            END IF ' end check echo flag
            Inpt = Inpt + Char$ ' append character input to input string
            ' check echo and word wrap
            IF No.Echo = False AND Word.Wrap THEN
                SELECT CASE LEN(Inpt) + Line.Limit ' check length of input+line length
                    CASE 0 TO User.LineLength - 2 ' position to left, no word wrap
                        CALL Scrn(Valid.Char$) ' send echo character
                    CASE ELSE ' position to left, word wrap
                        Last.Word = False ' counter of characters at last word to wrap
                        Word = False ' set flag for word wrapped
                        ' loop through last word reverse
                        FOR Last.Wrap = LEN(Inpt) TO 1 STEP -1
                            IF MID$(Inpt, Last.Wrap, 1) = " " THEN ' locate space of word
                                FOR Word.Break = 1 TO Last.Word ' loop through length of word
                                    CALL Back.Space ' routine to backspace
                                NEXT ' end loop through word
                                ' store wrapped word for next i/o
                                Buffered.Input$ = MID$(Inpt, Last.Wrap + 1)
                                ' truncate word from input string
                                Inpt = LEFT$(Inpt, Last.Wrap)
                                CALL Scrn(CHR$(13)) ' send return
                                Word = True ' word has wrapped flag
                                EXIT FOR ' end loop for a space
                            END IF ' end check space location
                            Last.Word = Last.Word + 1 ' increment last word wrapped counter
                        NEXT
                        IF Word = False THEN ' check flag for word wrapped
                            CALL Scrn(CHR$(13)) ' send return to screen
                        END IF ' end check word wrapped flag
                        CALL Put.Modem(CHR$(13)) ' send return to modem
                        EXIT DO ' end input loop
                END SELECT ' end check length of input+line length
            ELSE ' word wrap flag not set
                IF Line.Length > False THEN ' check line length variable
                    IF LEN(Inpt) >= Line.Length THEN ' check input string length
                        CALL Scrn(CHR$(13)) ' send return to screen
                        CALL Put.Modem(CHR$(13)) ' send return to modem
                        EXIT DO ' exit input loop
                    END IF ' end check input string length
                ELSE ' check line length
                    ' check right column
                    IF LEN(Inpt) + Line.Limit > User.LineLength - 2 THEN
                        CALL Scrn(CHR$(13)) ' send return to screen
                        CALL Put.Modem(CHR$(13)) ' send return to modem
                        EXIT DO ' exit input loop
                    END IF ' end check line limit
                END IF ' end check line length
            END IF ' end check word wrap
    END SELECT ' end select character ascii code
LOOP ' end input loop
Carriage.Return = False ' reset output cr/lf flag
Outpt = Nul ' reset input prompt
No.Echo = False ' reset echoing mask character flag
No.Input = False ' reset null input flag
Line.Length = False ' reset length of input line
IF Inpt = Nul THEN ' check input string to null
    No.Input = True ' set null input flag
END IF ' end check input string length
No.Input.Out = Nul ' reset echo string for null input flag
IF User.LineFeeds = False THEN ' check linefeed mode
    IF User.Echo = False THEN ' check echo mode
        CALL Put.Modem(CHR$(10)) ' send linefeed
    END IF ' end check echo mode
END IF ' end check linefeed mode
Word.Char$ = UCASE$(LEFT$(Inpt, 1)) ' get uppercase of first character of input
No = (Word.Char$ = "N") ' set no flag (user entered N as first character)
Yes = (Word.Char$ = "Y") ' set yes flag (user entered Y as first character)
Quit = (Word.Char$ = "Q") ' set quit flag (user entered Q as first character)
Continue = (Word.Char$ = "C") ' set continuous flag
EXIT SUB ' exit routine to get input string in Inpt

REM * subroutine to verify time limits.

IO.Timing:
' routine to calculate input timing
CALL Second.Timer(Time.Expired, Time.Out!, 180!)
IF Time.Expired THEN ' timing has expired
    CALL Hang.Up(1) ' routine to hang up modem with message
END IF ' end check timing
' routine to calculate time on
CALL Second.Timer(Time.Expired, Timeon, Time.Left)
IF Time.Expired THEN ' time limit has expired
    CALL Hang.Up(2) ' routine to hang up modem with message
END IF ' end check time limit
IF Two.Minutes.Left = False THEN ' check two minutes message flag
    ' calculate two minutes time
    CALL Second.Timer(Time.Expired, Timeon, Time.Left - 120!)
    IF Time.Expired THEN ' time has expired, user has two minutes left
        Two.Minutes.Left = True ' set two minutes message flag
        Outpt = Nul ' reset output string
        CALL IO.O ' send null return
        CALL Put.Modem(CHR$(7)) ' send beep to modem
        Outpt = "Two Minutes Left!" ' send two minutes remaining message
        CALL IO.O ' to modem
    END IF ' end check two minutes
END IF ' end check two minutes flag
RETURN ' end timing subroutine
END SUB ' end routine to get input string in Inpt

REM * Routine to send a string to the modem.
REM * input variables:
REM *   Allow.Break - set if control-k checking allowed.
REM *   Local.Mode - flag if the console is operating program.
REM *   Output$ - string to send.
REM * output variables:
REM *   Break - flag set if control-k entered.
REM *   Buffer - any input from modem during output.
REM * processing variables:
REM *   Char - ascii code of character to send.
REM *   Count - loop variable of output string sent.
REM *   Input.Char - local work variable.

SUB Put.Modem (Output$)
IF Local.Mode = False THEN ' check if local mode is activated
    FOR Count = 1 TO LEN(Output$) ' loop through output string
        CALL Get.Modem(Input.Char) ' routine to get data from modem
        IF Input.Char THEN ' character received from modem
            IF Input.Char = 11 THEN ' check if character is control-k
                IF Allow.Break THEN ' check if break allowed
                    Break = True ' set break flag
                    EXIT SUB ' end routine
                END IF ' end check break allowed
            ELSE ' any other character
                Buffer = Buffer + CHR$(Input.Char) ' append to input buffer
            END IF ' end check character is control-k
        END IF ' end check character received
        ' convert output string character to ascii
        Char = ASC(MID$(Output$, Count, 1))
        Modem.Access = True
        Modem.Error = False
        PUT #ModemFile, Char
        Modem.Access = False
        IF Modem.Error THEN
            CALL Lost.Carrier
        END IF
    NEXT ' end loop through output string
END IF ' end check local mode
END SUB ' end routine to send output to modem

REM * Routine to get a character from the modem.
REM * input variables:
REM *   Local.Mode - flag if console is operating system.
REM * output variables:
REM *   Input.Char - character received from modem.

SUB Get.Modem (Input.Char)
IF Local.Mode THEN
    EXIT SUB
END IF
Input.Char = 0
Modem.Access = True
Modem.Error = False
IF LOC(ModemFile) THEN
    GET #ModemFile, Input.Char
END IF
Modem.Access = False
IF Modem.Error THEN
    CALL Lost.Carrier
END IF
END SUB

REM * subroutine to display lost carrier or modem not found message
REM * and terminate program.

SUB Lost.Carrier
CALL Status.Line(-2) ' routine to clear lower two console status areas
COLOR 15, 0 ' reset color
IF Modem.Error = 68 THEN ' device unavailable
    Outpt = CHR$(13) + "Dnddoor terminated (no modem found)." + CHR$(13)
ELSE
    Outpt = CHR$(13) + "Dnddoor terminated (lost carrier)." + CHR$(13)
END IF
CALL Scrn(Outpt) ' send message
Outpt = "Logoff: port" + STR$(Port) + " at " + clock$ + ", node " + Node + "." + CHR$(13)
CALL Scrn(Outpt) ' send message
COLOR 7, 0 ' reset color
SLEEP 5 ' pause for 5 seconds
SYSTEM ' terminate program
END SUB

REM * routine to calculate elapsed time
REM * input variables:
REM *   Start.Time! - equals stored start time from Timer function.
REM *   Wait.Time! - equals elapsed time to calculate in seconds.
REM * output variables:
REM *   Time.Expired - set to true if Wait.Time has elapsed.
REM * working variables:
REM *   Time.Elapsed! - contains time difference.

SUB Second.Timer (Time.Expired, Start.Time!, Wait.Time!)
Time.Expired = False ' set return flag time has not expired
Time.Elapsed! = TIMER - Start.Time! ' calculate time elapsed from start time
IF Time.Elapsed! < 0! THEN ' check if midnight has passed
    Time.Elapsed! = Time.Elapsed! + 86400! ' increment calculated time elapsed
END IF ' end check midnight passed
' compare calculated elapsed time to wait time
IF Time.Elapsed! >= Wait.Time! THEN
    Time.Expired = True ' set return flag time has expired
END IF ' end compare times
END SUB ' end routine to calculate time

REM * Routine to get and process keyboard entry.
REM * output variables:
REM *   Keyboard.Break - true to exit from IO.I when function key pressed.
REM * work variables:
REM *   Keyboard.Char$ - contains key pressed.

SUB Keyboard (Keyboard.Break)
Keyboard.Break = False ' reset extended/function key input flag
Keyboard.Char$ = INKEY$ ' reset next keyboard character
SELECT CASE LEN(Keyboard.Char$) ' process keyboard character type
    CASE 0 ' nothing entered
        EXIT SUB ' end routine
    CASE 1 ' single keystroke
        SELECT CASE ASC(Keyboard.Char$) ' process single key
            CASE 8, 13, 32 TO 127 ' valid keys entered during i/o
                Buffer = Buffer + Keyboard.Char$ ' append keystroke to buffer
            CASE 11 ' control-k
                IF Allow.Break THEN ' check break allowed
                    Break = True ' set break flag
                END IF ' end check break allowed
            CASE 27 ' escape key
                IF Chat THEN ' check in chat flag
                    Chat = False ' set chat exit flag
                    Keyboard.Break = True ' set function/extended key flag
                ELSE ' escape key at console
                    IF Logged.In THEN ' check player in game
                        CALL Status.Line(True) ' toggles status line area
                    END IF ' end check player logged in game
                END IF ' end check flag
        END SELECT ' end sigle keystroke
    CASE 2 ' function/extended key
        SELECT CASE ASC(RIGHT$(Keyboard.Char$, 1)) ' process extended key code
            CASE 79 ' end key/terminate key
                CALL Hang.Up(4) ' routine to hang up modem with message
                Keyboard.Break = True ' set function/extended key flag
            CASE 71 ' home key/chat key
                IF Chat = False THEN ' check if already in chat
                    IF Local.Mode = False THEN ' check local mode not activated
                        IF Logged.In THEN ' check user logged in flag
                            Chat = True ' set in chat flag
                            CALL Enter.Chat ' routine to chat with user
                            Chat = False ' reset in chat flag
                            Keyboard.Break = True ' set function/extended key flag
                        END IF ' end check user logged in
                    END IF ' end check local mode
                END IF ' end check chat flag
            CASE 59 TO 68 ' function key entered
                IF Logged.In THEN ' check user logged in flag
                    FunctionBuffer = RIGHT$(Keyboard.Char$, 1) ' store function key in buffer
                    Keyboard.Break = True ' set function/extended key flag
                END IF ' end check user logged in
        END SELECT ' end process extended key code
END SELECT ' end process keyboard character type
END SUB ' end routine to get/process keyboard entry

REM * Routine to send carriage return/linefeed (cr/lf) to screen/modem.

SUB Line.Return
CALL Scrn(CHR$(13)) ' send return character to screen
CALL Put.Modem(CHR$(13)) ' send cr to modem
IF User.LineFeeds = False THEN ' check linefeed mode
    CALL Put.Modem(CHR$(10)) ' send lf to modem
END IF ' end check linefeed mode
END SUB ' end routine to send cr/lf

REM * Routine to backspace over one previous character.

SUB Back.Space
' Store current screen location of cursor column minus one
Cursor.Column = POS(0) - 1
LOCATE CSRLIN, Cursor.Column, 0 ' locate back one space, cursor off
PRINT " "; ' erase character with space
LOCATE CSRLIN, Cursor.Column, 1 ' locate back one space, cursor on
IF User.Echo = False THEN ' check echo mode
    CALL Put.Modem(CHR$(8) + " " + CHR$(8)) ' send backspace to modem
END IF ' end check echo mode
END SUB ' end routine to backspace

REM * Routine to change color. ANSI to modem, COLOR statement to screen.

SUB Out.ANSI
Ansi.ColorCode = 37 ' default to ANSI white
IF Graphics.Off = False THEN ' check graphics color change flag
    Color.Code = Color.Code + 1 ' increment current ANSI color variable
    IF Color.Code < 31 OR Color.Code > 36 THEN ' check ANSI color bounds
        Color.Code = 31 ' reset to base ANSI color
    END IF ' end check color bounds
    Ansi.ColorCode = Color.Code ' store new color code
END IF ' end check graphics change flag
CALL Modem.ANSI(Ansi.ColorCode) ' send ANSI color code number
END SUB ' end routine to change color

REM * Routine to send ANSI color change code to modem/screen.
REM * input variables:
REM *   Ansi.ColorCode - contains the Ansi color code number.
REM * work variables:
REM *   Ansi.Output$ - stores the Ansi string to send.

SUB Modem.ANSI (Ansi.ColorCode)
CALL Convert.Color(Ansi.ColorCode) ' change screen color
IF Color.Graphics THEN ' check remote ANSI graphics flag
    ' construct ANSI code
    Ansi.Output$ = CHR$(27) + "[0;1;" + MID$(STR$(Ansi.ColorCode), 2) + "m"
    CALL Put.Modem(Ansi.Output$) ' send code to modem
END IF ' end check remote ANSI flag
END SUB ' end routine to send ANSI code

REM * Routine to reset ANSI color code to default.
REM * work variables:
REM *   Ansi.Output$ - stores the default Ansi color string.

SUB Reset.ANSI
IF Color.Graphics THEN ' check remote ANSI graphics flag
    Ansi.Output$ = CHR$(27) + "[0;37m" ' construct ANSI black on white code
    CALL Put.Modem(Ansi.Output$) ' send code to modem
END IF ' end check remote ANSI flag
COLOR 7, 0 ' reset local screen to black on white
PRINT Nul; ' display a zero length string to change color
END SUB ' end routine to reset ANSI color code

REM * Routine to change local screen color to ANSI color code number.
REM * input variables:
REM *   Ansi.ColorCode - the Ansi color code number.
REM * work variables:
REM *   Scrn.ColorCode - stores the CGA screen color number.

SUB Convert.Color (Ansi.ColorCode)
SELECT CASE Ansi.ColorCode ' choose ANSI color code number
    CASE 31 ' ANSI red
        Scrn.ColorCode = 12 ' COLOR statement red
    CASE 32 ' ANSI green
        Scrn.ColorCode = 10 ' COLOR statement green
    CASE 33 ' ANSI yellow
        Scrn.ColorCode = 14 ' COLOR statement yellow
    CASE 34 ' ANSI blue
        Scrn.ColorCode = 9 ' COLOR statement blue
    CASE 35 ' ANSI magenta
        Scrn.ColorCode = 13 ' COLOR statement magenta
    CASE 36 ' ANSI cyan
        Scrn.ColorCode = 11 ' COLOR statement cyan
    CASE 37 ' ANSI white
        Scrn.ColorCode = 15 ' COLOR statement white
    CASE ELSE ' default ANSI white
        Scrn.ColorCode = 15 ' COLOR statement white
END SELECT ' end choose ANSI color code
COLOR Scrn.ColorCode, 0 ' change screen color
END SUB ' end routine to change local screen color

REM * Routine to process stored function key pressed during remote play.
REM * input variables:
REM *   FunctionKey.Number - the ascii value of the extended function key.

SUB Function.Key (FunctionKey.Number)
IF Logged.In = False THEN ' check user is logged in
    EXIT SUB ' exit routine
END IF ' end check user logged in
IF Local.Mode THEN ' check console is logged in
    SELECT CASE FunctionKey.Number ' choose function key number
        CASE 59 ' F1 - !Edit
            CALL IO.O ' empty output
            CALL Editor ' editor routine
        CASE 60 ' F2 - !Status
            CALL IO.O ' empty output
            CALL Display.Memory ' routine to display memory stats
        CASE 61 ' F3 - !Discard
            Outpt = Action.Prompt + "!DISCARD " ' display action prompt plus command
            CALL IO.I ' get item to discard
            Parsed.Command1 = UCASE$(Inpt) ' store item name
            CALL Discard.Object ' routine to discard item from player inventory
        CASE 62 ' F4 - !Reduce
            Outpt = Action.Prompt + "!REDUCE " ' display action prompt plus command
            CALL IO.I ' get number
            Parsed.Command1 = Inpt ' store number
            CALL Reduce.Monsters ' routine to reduce monsters in room
        CASE 63 ' F5 - !Call
            Outpt = Action.Prompt + "!CALL " ' display action prompt plus command
            CALL IO.I ' get monster number
            Parsed.Command1 = Inpt ' store monster number
            CALL Summon.Monster ' routine to get a monster into room
        CASE 64 ' F6 - !Kill
            Outpt = Action.Prompt + "!KILL " ' display action prompt plus command
            CALL IO.I ' get monster name
            Parsed.Command1 = UCASE$(Inpt) ' store monster name
            Parsed.Command2 = Parsed.Command1 ' store monster name
            CALL Kill.Monster ' routine to kill monster name
        CASE 65 ' F7 - !Teleport
            Outpt = Action.Prompt + "!TELEPORT " ' display action prompt plus command
            CALL IO.I ' get room number
            Parsed.Command1 = Inpt ' store room number
            Parsed.Command2 = Parsed.Command1 ' store room number
            CALL Teleport.User ' routine to teleport user to a room number
        CASE 66 ' F8 - !Invisibility
            CALL IO.O ' empty output
            CALL Toggle.Invisible ' routine to toggle invisibility mode
        CASE 67 ' F9 - !Get
            Outpt = Action.Prompt + "!GET " ' display action prompt plus command
            CALL IO.I ' get item name
            Inpt = UCASE$(Inpt) ' store item name
            Stored.Parsed.Command2 = Inpt ' store item name
            CALL Drop.Object ' routine to get an item
        CASE 68 ' F9 - !Link
            CALL IO.O ' empty output
            CALL Link.Room ' routine to add/remove room links
    END SELECT ' end select function key number
    EXIT SUB ' exit routine
END IF ' end check console logged in
Inpt = Nul ' reset input string
Outpt = Nul ' reset output string
CALL IO.O ' send empty cr/lf
SELECT CASE FunctionKey.Number ' choose function key number
    CASE 59 ' F1 - !Edit
        Outpt = "System Operator is using the editor. Please wait.." ' send message
        CALL IO.O ' sysop will edit during game
        Local.Mode = True ' set local mode
        Outpt = "Sysop: Enter room number to edit, or press enter to start !Edit:"
        CALL IO.O ' write local sysop info message
        Outpt = Action.Prompt + "!EDIT " ' display action prompt plus command
        CALL IO.I ' get room number to edit(if any)
        IF Inpt <> Nul THEN ' determine edit room number
            Room.Number = INT(VAL(Inpt)) ' room number parameter to edit
            IF Room.Number > False AND Room.Number <= LOF(RoomFile) / LEN(RoomRecord) THEN ' range
                CALL Change.Room(Room.Number) ' edit room from number parameter
            END IF ' end compare room number range
        ELSE ' edit users
            CALL Editor ' editor routine
        END IF ' end determine edit prompt
        Local.Mode = False ' reset local mode
    CASE 60 ' F2 - !Status
        Local.Mode = True ' reset local mode
        CALL Display.Memory ' routine to display memory stats
        Local.Mode = False ' restore local mode
    CASE 61 ' F3 - !Discard
        Local.Mode = True ' reset local mode
        Outpt = "Sysop: Enter player inventory treasure name to discard:"
        CALL IO.O ' write local sysop info message
        Outpt = Action.Prompt + "!DISCARD " ' display action prompt plus command
        CALL IO.I ' get item to discard
        Parsed.Command1 = UCASE$(Inpt) ' store item name
        CALL Discard.Object ' routine to discard item from player inventory
        Local.Mode = False ' restore local mode
    CASE 62 ' F4 - !Reduce
        Local.Mode = True ' reset local mode
        Outpt = "Sysop: Enter number of monsters in room to reduce to:"
        CALL IO.O ' write local sysop info message
        Outpt = Action.Prompt + "!REDUCE " ' display action prompt plus command
        CALL IO.I ' get number
        Parsed.Command1 = Inpt ' store number
        CALL Reduce.Monsters ' routine to reduce monsters in room
        Local.Mode = False ' restore local mode
    CASE 63 ' F5 - !Call
        Local.Mode = True ' reset local mode
        Outpt = "Sysop: Enter name or number of monster to call:"
        CALL IO.O ' write local sysop info message
        Outpt = Action.Prompt + "!CALL " ' display action prompt plus command
        CALL IO.I ' get monster number
        Local.Mode = False ' restore local mode
        Parsed.Command1 = Inpt ' store monster number
        CALL Summon.Monster ' routine to get a monster into room
    CASE 64 ' F6 - !Kill
        Local.Mode = True ' reset local mode
        Outpt = "Sysop: Enter name of monster in room to kill off:"
        CALL IO.O ' write local sysop info message
        Outpt = Action.Prompt + "!KILL " ' display action prompt plus command
        CALL IO.I ' get monster name
        Local.Mode = False ' restore local mode
        Parsed.Command1 = UCASE$(Inpt) ' store monster name
        Parsed.Command2 = Parsed.Command1 ' store monster name
        CALL Kill.Monster ' routine to kill monster name
    CASE 65 ' F7 - !Teleport
        Local.Mode = True ' reset local mode
        Outpt = "Sysop: Enter number of room to teleport player to:"
        CALL IO.O ' write local sysop info message
        Outpt = Action.Prompt + "!TELEPORT " ' display action prompt plus command
        CALL IO.I ' get room number
        Local.Mode = False ' restore local mode
        Parsed.Command1 = Inpt ' store room number
        Parsed.Command2 = Parsed.Command1 ' store room number
        CALL Teleport.User ' routine to teleport user to a room number
    CASE 66 ' F8 - !Invisibility
        CALL Toggle.Invisible ' routine to toggle invisibility mode
    CASE 67 ' F9 - !Get
        Local.Mode = True ' reset local mode
        Outpt = "Sysop: Enter name of treasure item to get from treasure file:"
        CALL IO.O ' write local sysop info message
        Outpt = Action.Prompt + "!GET " ' display action prompt plus command
        CALL IO.I ' get item name
        Local.Mode = False ' restore local mode
        Inpt = UCASE$(Inpt) ' store item name
        Stored.Parsed.Command2 = Inpt ' store item name
        CALL Drop.Object ' routine to get an item
    CASE 68 ' F10 - !Link
        Outpt = "System Operator is using the editor. Please wait.." ' send message
        CALL IO.O ' sysop will edit during game
        Local.Mode = True ' reset local mode
        CALL Link.Room ' routine to add/remove room links
        Local.Mode = False ' restore local mode
END SELECT ' end choose function key number
Outpt = Nul ' reset output string
Inpt = Nul ' reset input string
END SUB ' end routine to process game function keys

REM * Routine to hang up modem with a message.
REM * input variables:
REM *   HangUp.Type - the hang up message to display.

SUB Hang.Up (HangUp.Type)
Allow.Break = False ' disable break
Break = False ' reset control-k flag
Buffer = Nul ' reset buffer
FunctionBuffer = Nul ' reset function key buffer
Graphics.Off = True ' turn off graphics color changing
Inpt = Nul ' set input string to null
Outpt = Nul ' set output string to null
Timeon = TIMER ' store current time in time on variable
Time.Left = 180 ' store 3 minutes remaining in variable
CALL IO.O ' empty output
Outpt = "Dnddoor terminating at " + clock$ + ", " + Node + "." ' make message
CALL IO.O ' send message
SELECT CASE HangUp.Type ' choose hang up message
    CASE 1 ' type 1
        Outpt = "Connect timeout"
    CASE 2 ' type 2
        Outpt = "Time limit exceeded"
    CASE 4 ' type 4
        Outpt = "Forced logoff"
    CASE 5 ' type 5
        Outpt = "Call limit exceeded"
    CASE 6 ' type 6
        CALL Restriction.Notice ' routine to display restricted time
    CASE 7 ' type 7
        Outpt = "Illegal login attempt"
    CASE 8 ' type 8
        Outpt = "Password verification!"
END SELECT ' end choose hang up message
Outpt = "Reason: " + Outpt + "."
CALL IO.O ' display message
CALL Status.Line(-2) ' clear status line areas
COLOR 7, 0 ' reset screen color
SLEEP 5 ' pause for 5 seconds
SYSTEM ' terminate program
END SUB ' end routine to hang up modem

REM * Routine to put a character on the screen at current row/column.
REM * input variables:
REM *   Scrn.Char$ - the character to display.

SUB Scrn (Scrn.Char$)
Row = CSRLIN - 1 ' calculate current row
Column = POS(0) - 1 ' calculate current column
IF Row = Max.Row THEN ' compare current row
    ' scroll at max.row (status area)
    IF Column = User.LineLength THEN ' verify column to scroll
        PRINT ; ' force screen scroll
        IF Scrn.Char$ = CHR$(13) THEN ' check cr/lf at corner
            EXIT SUB ' return from routine
        END IF ' end check corner cr/lf
    END IF ' end verify column
END IF ' end compare row
PRINT Scrn.Char$; ' display character at row, column
END SUB ' end routine to display a character on screen

REM * Routine to make users restricted time online output message.

SUB Restriction.Notice
Outpt = "Your calls are restricted from" ' start of message
Restrict.Time = UserRecord.FromHour ' store users hour of restricted time
IF Restrict.Time > 12 THEN ' compare 24 hour format time to standard time
    Outpt = Outpt + STR$(UserRecord.FromHour - 12) ' decrement to 24 hour format
    Inpt = "pm" ' 24 hour indicator
ELSE ' compare to standard time
    Outpt = Outpt + STR$(UserRecord.FromHour) ' store hour format
    Inpt = "am" ' 12 hour indicator
END IF ' end compare time type
' format string of restricted time from, plus minutes
Outpt = Outpt + ":" + RIGHT$(STR$(UserRecord.FromMin + 100), 2) + Inpt + " to"
Restrict.Time = UserRecord.ToHour ' store users hour of restricted time
IF Restrict.Time > 12 THEN ' compare 24 hour format time to standard time
    Outpt = Outpt + STR$(UserRecord.ToHour - 12) ' decrement to 24 huur format
    Inpt = "pm" ' 24 hour indicator
ELSE ' compare to standard time
    Outpt = Outpt + STR$(UserRecord.ToHour) ' store hour format
    Inpt = "am" ' 12 hour indicator
END IF ' end compare time type
' format string of restricted time to, plus minutes
Outpt = Outpt + ":" + RIGHT$(STR$(UserRecord.ToMin + 100), 2) + Inpt + "."
END SUB ' end routine to make restricted time message

REM * Routine to display a text file.
REM * input variables:
REM *   File.Output$ - filename of file to display.
REM * processing variables:
REM *   Allow.Break - flag to allow control-k.
REM *   Break - set to true if control-k pressed.
REM *   Continue - set to true for continuous output
REM *   Page.Length, Page.Break - paginating variables.

SUB Out.File (File.Output$)
Graphics.Off = True ' turn off graphics color changing
File.Page = User.PageLength ' store pagelength
IF File.Page = False THEN ' check pagelength
    File.Page = 24 ' reset pagelength
END IF ' end check pagelength
Allow.Break = True ' turn on allow break flag
Break = False ' reset control-k flag
Continue = False ' reset continuous flag
Page.Length = False ' reset page counter
CLOSE #TempFile ' close temporary file
IF _FILEEXISTS(File.Output$) = 0 THEN ' check file exists
    Outpt = "File " + File.Output$ + " not found." ' make output
    CALL IO.O ' send output
    Allow.Break = False ' reset allow break flag
    Break = False ' reset control-k flag
    Graphics.Off = False ' reset color change flag
    EXIT SUB ' exit routine
END IF ' end check file exists
OPEN File.Output$ FOR RANDOM SHARED AS #TempFile LEN = 1 ' open the file
IF LOF(TempFile) = False THEN ' check file length
    Outpt = "File " + File.Output$ + " length zero." ' make output
    CALL IO.O ' send output
    Allow.Break = False ' reset allow break flag
    Break = False ' reset control-k flag
    Graphics.Off = False ' reset color change flag
    EXIT SUB ' exit routine
END IF ' end check file length
CLOSE #TempFile ' close temporary file
OPEN File.Output$ FOR INPUT SHARED AS #TempFile ' open the file
DO WHILE NOT EOF(TempFile) ' loop through all the text in file
    LINE INPUT #TempFile, Outpt ' get the next text line
    CALL IO.O ' display the text line
    IF Break THEN ' check control-k pressed
        EXIT DO ' exit text input loop
    END IF ' end check control-k
    Page.Length = Page.Length + 1 ' increment line displayed counter
    IF Page.Length = File.Page THEN ' compare page counter
        Page.Length = False ' reset page counter
        IF Continue = False THEN ' check continuous flag
            CALL More.Prompt ' routine to prompt for more
            IF No THEN ' check more prompt returned no flag
                EXIT DO ' exit text input loop
            END IF ' end check more prompt no flag
        END IF ' end check page counter
    END IF ' end check continuous flag
LOOP ' end text file input loop
Allow.Break = False ' reset allow break flag
IF Break THEN ' check control-k flag
    Break = False ' reset control-k flag
    Outpt = Nul ' set output to null
    CALL IO.O ' send empty return
END IF ' end check control-k flag
Graphics.Off = False ' reset graphics color changer
IF Page.Length THEN ' check page counter
    CALL More.Prompt ' routine to prompt for more
END IF ' end check page counter
Allow.Break = False ' reset allow break flag
Break = False ' reset control-k flag
Graphics.Off = False ' reset color change flag
END SUB ' end routine to display a text file

REM * Routine to display the more prompt.
REM * work variables:
REM *   Allow.Type - store break flag type
REM *   Graphics.Type - stores graphics cycling variable.

SUB More.Prompt
Allow.Type = Allow.Break ' store break type
Allow.Break = False ' disable break during prompt
Graphics.Type = Graphics.Off ' store graphics changing flag
Graphics.Off = False ' turn off graphics changing
Outpt = More$ ' get the more prompt
Line.Length = 1 ' allow one key input
No.Echo = True ' do not echo any key
CALL IO.I ' input routine
Allow.Break = Allow.Type ' restore break type
Graphics.Off = Graphics.Type ' restore graphics changing flag
END SUB

REM * Routine to restrict a room direction to a players level.
REM * input variable:
REM *   Room.Direction - direction to restrict.
REM * output variable:
REM *   Direction.Restricted - set true to restrict player.
REM * work variables:
REM *   Action.Number - contains the room action number.

SUB Restrict (Room.Direction, Direction.Restricted)
Direction.Restricted = False ' reset return variable
Action.Number = RoomRecord.Action ' store room action number
IF Action.Number > False AND Action.Number <= LOF(ActionFile) / LEN(ActionRecord) THEN ' check file bounds
    CALL Read.Record(ActionFile, Action.Number) ' read action record
    SELECT CASE ActionRecord.Level ' determine player level
        CASE IS > False ' player at least room level
            IF UserRecord.Level < ActionRecord.Level THEN ' compare levels
                ' compare room direction
                IF ActionRecord.Restrictions AND 2 ^ Room.Direction THEN
                    Direction.Restricted = True ' direction restricted flag
                END IF ' end compare room direction
            END IF ' end compare levels
        CASE IS < False ' player at most room level
            IF UserRecord.Level > ABS(ActionRecord.Level) THEN ' compare levels
                ' compare room direction
                IF ActionRecord.Restrictions AND 2 ^ Room.Direction THEN
                    Direction.Restricted = True ' direction restricted flag
                END IF ' end compare room direction
            END IF ' end compare levels
    END SELECT ' end choose room/player level
END IF ' end check file bounds
END SUB ' end routine to restrict room entrance

REM * Routine to restrict a room direction to entry Type
REM * output variable:
REM *   Direction.Restricted - set true to restrict player.
REM * work variables:
REM *   Action.Number - contains the room action number.

SUB Restrict.Room.Type (Direction.Restricted)
CALL Read.Room.Record(Next.Room) ' read next room number record
Direction.Restricted = False ' reset return variable
Action.Number = RoomRecord.Action ' store room action number
IF Action.Number > False AND Action.Number <= LOF(ActionFile) / LEN(ActionRecord) THEN ' check file bounds
    CALL Read.Record(ActionFile, Action.Number) ' read action record
    IF ActionRecord.Attribute2 = Air THEN ' verify room is air
        IF Entry.Command <> Fly THEN ' verify player fly
            Direction.Restricted = True ' set restricted flag
        END IF ' end verify player fly command
    END IF ' end verify room air attribute
    IF ActionRecord.Attribute2 <> Air THEN ' verify room is air
        IF Entry.Command = Fly THEN ' verify player fly
            Direction.Restricted = True ' set restricted flag
        END IF ' end verify player fly command
    END IF ' end verify room air attribute
    IF ActionRecord.Attribute2 = Water THEN ' verify room water
        IF Entry.Command <> Swim THEN ' verify player swim
            Direction.Restricted = True ' set restricted flag
        END IF ' end verify player swim command
    END IF ' end verify room water attribute
    IF ActionRecord.Attribute2 <> Water THEN ' verify room water
        IF Entry.Command = Swim THEN ' verify player swim
            Direction.Restricted = True ' set restricted flag
        END IF ' end verify player swim command
    END IF ' end verify room water attribute
ELSE ' check file bounds
    IF Entry.Command = Fly OR Entry.Command = Swim THEN ' check room entry
        Direction.Restricted = True ' set restricted flag
    END IF ' end check room entry
END IF ' end check file bounds
CALL Read.Room.Record(Room) ' read current room record
END SUB ' end routine to restrict room entrance

REM * Routine to determine validity of a string.
REM * input variables:
REM *   Validate$ - string to verify.
REM *   Length - stores length string must be (an even number).
REM * output variables:
REM *   Validate$ - set to null if invalid.
REM * work variables:
REM *   String.Length - loop counter of string to validate.
REM *   Char - character in string to check.

SUB Valid (Validate$, Length)
Validate$ = LEFT$(Validate$, Length) ' truncate string to length
Validate$ = Validate$ + SPACE$(Length - LEN(Validate$)) ' extend string length
FOR String.Length = 1 TO LEN(Validate$) ' loop through all characters in string
    ' get ascii value of string character
    Char = ASC(MID$(Validate$, String.Length, 1))
    IF Char < 32 OR Char > 127 THEN ' verify bounds of character
        Validate$ = Nul ' return null if not valid
        EXIT SUB ' end routine
    END IF ' end verify ascii code
NEXT ' end loop
END SUB ' end routine to determine string validity

REM * Routine to encrypt a string.
REM * input variables:
REM *   Encrypted$ - the string to encrypt.
REM *   Add.Checksum - set to false to add byte checksum.
REM * output variables:
REM *   Encrypted$ - the encrypted string.
REM * work variables:
REM *   Encrypt.Work$ - variable containing encrypted string.
REM *   Encrypt.Count - loop variable.
REM *   Encrypted.Sum - calculated byte encryption variable.
REM *   Encrypt.Work1, Encrypt.Work2, Encrypt.Work3.

SUB Encrypt (Encrypted$, Add.Checksum)
Encrypt.Work$ = Nul ' reset encrypted work variable
' loop through string to encrypt in byte pairs
FOR Encrypt.Count = 1 TO LEN(Encrypted$) STEP 2
    ' get first byte substring of pair
    Encrypt.Work1 = ASC(MID$(Encrypted$, Encrypt.Count, 1))
    ' get second byte substring of pair
    Encrypt.Work2 = ASC(MID$(Encrypted$, Encrypt.Count + 1, 1))
    IF Add.Checksum THEN ' check checksum ordering
        Encrypted.Sum = 20000 ' checksum bit off
    ELSE ' checksum ordering on
        Encrypt.Work3 = Encrypt.Work1 + Encrypt.Work2 ' compute sum
        ' compute even sum of bytes
        IF INT(Encrypt.Work3 / 2) = Encrypt.Work3 / 2 THEN
            Encrypted.Sum = 10000 ' checksum bit for even parity
        ELSE ' checksum ordering
            Encrypted.Sum = False ' checksum bit for odd parity
        END IF ' end check checksum ordering
    END IF ' end verify checksum used
    ' convert checksum and bytes to integer
    Encrypted.Sum = Encrypted.Sum + (Encrypt.Work1 - 32) * 100 + (Encrypt.Work2 - 32)
    ' convert integer to string
    Encrypt.Work$ = Encrypt.Work$ + MKI$(Encrypted.Sum)
NEXT ' end loop through string to encrypt
Encrypted$ = Encrypt.Work$ ' set return variable to encrypted string
END SUB ' end routine to encrypt a string

REM * Routine to decrypt a string.
REM * input variables:
REM *   Decrypted$ - the string to decrypt.
REM * output variables:
REM *   Decrypted$ - the decrypted string, or null for checksum error.
REM * work variables:
REM *   Decrypt.Work$ - variable containing encrypted string.
REM *   Decrypt.Count - loop variable.
REM *   Decrypted.Sum - calculated byte encryption variable.
REM *   Decrypt.Work1, Decrypt.Work2, Decrypt.Work3.
REM *   Decrypted.Checksum - contains the bit parity.

SUB Decrypt (Decrypted$)
Decrypt.Work$ = Nul ' reset decrypted work variable
' loop through string to decrypt in byte pairs
FOR Decrypt.Count = 1 TO LEN(Decrypted$) STEP 2
    ' convert the two byte string to integer
    Decrypted.Sum = CVI(MID$(Decrypted$, Decrypt.Count, 2))
    ' divide two byte integer to one byte integer
    Decrypted.Checksum = Decrypted.Sum \ 100
    ' get low byte from pair
    Decrypt.Work1 = Decrypted.Sum - Decrypted.Checksum * 100
    ' adjust byte ascii value
    Decrypt.Work1 = Decrypt.Work1 + 32
    ' reset integer of string to high byte
    Decrypted.Sum = Decrypted.Checksum
    ' divide byte integer out leaving checksum bit
    Decrypted.Checksum = Decrypted.Sum \ 100
    ' get high byte from pair
    Decrypt.Work2 = Decrypted.Sum - Decrypted.Checksum * 100
    Decrypt.Work2 = Decrypt.Work2 + 32 ' adjust byte ascii value
    Decrypt.Work3 = Decrypt.Work1 + Decrypt.Work2 ' compute sum
    SELECT CASE Decrypted.Checksum ' select checksum parity
        CASE 0 ' verify the checksum bit for odd parity
            IF INT(Decrypt.Work3 / 2) = Decrypt.Work3 / 2 THEN ' check not odd parity
                Decrypted$ = Nul ' return null for checksum error
                EXIT SUB ' end routine
            END IF ' end check not odd parity
        CASE 1 ' verify the checksum bit for even parity
            IF INT(Decrypt.Work3 / 2) <> Decrypt.Work3 / 2 THEN ' check not even parity
                Decrypted$ = Nul ' return null for checksum error
                EXIT SUB ' end routine
            END IF ' end check not even parity
    END SELECT ' end verify checksum bit
    ' convert integer bytes to string
    Decrypt.Work$ = Decrypt.Work$ + CHR$(Decrypt.Work2) + CHR$(Decrypt.Work1)
NEXT ' end loop through string to decrypt
Decrypted$ = Decrypt.Work$ ' set return variable to decryptes string
END SUB ' end routine to decrypt a string

REM * Routine to store a file record
REM * input variables:
REM *   File.Number - number of the file to write record.
REM *   Record.Number - the number of the record to write.

SUB Share.Record (File.Number, Record.Number)
Share.Count = 0
LOCK File.Number, Record.Number ' lock record number
SELECT CASE File.Number ' select file number to write a record to
    CASE UserFile ' type 1
        PUT UserFile, Record.Number, UserRecord ' write user record
    CASE NonPlayerFile ' type 2
        PUT NonPlayerFile, Record.Number, MonsterRecord ' write nonplayers record
    CASE ObjectFile ' type 4
        PUT ObjectFile, Record.Number, ObjectRecord ' write object record
    CASE MonsterFile ' type 5
        PUT MonsterFile, Record.Number, MonsterRecord ' write monsters record
    CASE TreasureFile ' type 6
        PUT TreasureFile, Record.Number, TreasureRecord ' write treasure record
    CASE MonTalkFile ' type 7
        PUT MonTalkFile, Record.Number, MonsterTalkRecord ' write monster talk record
    CASE MonClassFile ' type 8
        PUT MonClassFile, Record.Number, MonclassRecord ' write monster class record
    CASE SpellFile ' type 9
        PUT SpellFile, Record.Number, SpellRecord ' write spell record
    CASE TableFile ' type 10
        PUT TableFile, Record.Number, TableRecord ' write message table record
    CASE ActionFile ' type 12
        PUT ActionFile, Record.Number, ActionRecord ' write action record
    CASE TempFile ' type 13
        PUT TempFile, Record.Number, TableRecord ' write temp table record
    CASE HelpFile ' type 15
        PUT HelpFile, Record.Number, HelpRecord ' write helpfile record
    CASE UserFile2 ' type 16
        PUT UserFile, Record.Number, UserRecord2 ' write user record
END SELECT ' end select file number
Share.Count = 0
UNLOCK File.Number, Record.Number ' lock record number
END SUB ' end routine to write file record using share

REM * Routine to read a file record
REM * input variables:
REM *   File.Number - number of the file to read record.
REM *   Record.Number - the number of the record to read.

SUB Read.Record (File.Number, Record.Number)
SELECT CASE File.Number ' select file number to read a record to
    CASE UserFile ' type 1
        GET UserFile, Record.Number, UserRecord ' read user record
    CASE NonPlayerFile ' type 2
        GET NonPlayerFile, Record.Number, MonsterRecord ' read nonplayers record
    CASE ObjectFile ' type 4
        GET ObjectFile, Record.Number, ObjectRecord ' read object record
    CASE MonsterFile ' type 5
        GET MonsterFile, Record.Number, MonsterRecord ' read monsters record
    CASE TreasureFile ' type 6
        GET TreasureFile, Record.Number, TreasureRecord ' read treasure record
    CASE MonTalkFile ' type 7
        GET MonTalkFile, Record.Number, MonsterTalkRecord ' read monster talk record
    CASE MonClassFile ' type 8
        GET MonClassFile, Record.Number, MonclassRecord ' read monster class record
    CASE SpellFile ' type 9
        GET SpellFile, Record.Number, SpellRecord ' read spell record
    CASE TableFile ' type 10
        GET TableFile, Record.Number, TableRecord ' read message table record
    CASE ActionFile ' type 12
        GET ActionFile, Record.Number, ActionRecord ' read action record
    CASE TempFile ' type 13
        GET TempFile, Record.Number, TableRecord ' read temp table record
    CASE HelpFile ' type 15
        GET HelpFile, Record.Number, HelpRecord ' read help record
    CASE UserFile2 ' type 16
        GET UserFile, Record.Number, UserRecord2 ' read user record
END SELECT ' end select file number
END SUB ' end routine to read file record using share

REM * Routine to store a message file record
REM * input variables:
REM *   File.Number - number of the file to write record.
REM *   Record.Number! - the number of the record to write.

SUB Share.Message (File.Number, Record.Number!)
Share.Count = 0
LOCK File.Number, Record.Number! ' lock record number
PUT File.Number, Record.Number!, MessageRecord ' write message record
Share.Count = 0
UNLOCK File.Number, Record.Number! ' lock record number
END SUB ' end routine to write file record using share

REM * Routine to read a message file record
REM * input variables:
REM *   File.Number - number of the file to read record.
REM *   Record.Number! - the number of the record to read.

SUB Read.Message.Record (File.Number, Record.Number!)
GET File.Number, Record.Number!, MessageRecord ' write message record
END SUB ' end routine to write file record using share

REM * Routine to store a room file record
REM * input variables:
REM *   Record.Number! - the number of the record to write.

SUB Share.Room.Record (Record.Number!)
Share.Count = 0
LOCK RoomFile, Record.Number! ' lock record number
PUT RoomFile, Record.Number!, RoomRecord ' write room record
Share.Count = 0
UNLOCK RoomFile, Record.Number! ' lock record number
END SUB ' end routine to write file record using share

REM * Routine to read a room file record
REM * input variables:
REM *   Record.Number! - the number of the record to read.

SUB Read.Room.Record (Record.Number!)
GET RoomFile, Record.Number!, RoomRecord ' write message record
END SUB ' end routine to write file record using share

REM * Filename: dnds1.bas Version: v5.0a r1.0a
REM * This subprogram contains most display and find routines.

REM * This routines displays information of the action file.
REM * input variables:
REM *   Number - the room file record.

SUB Display.Action (Number)
IF Number <= False THEN ' check number
    EXIT SUB ' exit subroutine
END IF ' end check number
Graphics.Off = False ' reset color
Outpt = "Action number" + STR$(Number) + ":" ' action number message
CALL IO.O ' send output message
Graphics.Off = True ' reset color
IF ActionRecord.MonsterTrigger THEN ' check action trigger
    ' get monster trigger record
    CALL Read.Record(MonsterFile, ActionRecord.MonsterTrigger)
    Outpt = MonsterRecord.MonsterName ' store monster name
    Outpt = RTRIM$(Outpt) ' trim name
    Inpt = STR$(ActionRecord.MonsterTrigger) + ", " + Outpt + "." ' make message
ELSE ' check trigger
    Inpt = " <none>" ' make default message
END IF ' end check trigger
Outpt = "[A]Monster trigger in effect for monster:" + Inpt ' make message
CALL IO.O ' send output message
IF ActionRecord.SpellTrigger THEN ' check action trigger
    CALL Read.Record(SpellFile, ActionRecord.SpellTrigger) ' get spell record
    Outpt = SpellRecord.SpellName ' store spell name
    Outpt = RTRIM$(Outpt) ' trim name
    Inpt = STR$(ActionRecord.SpellTrigger) + ", " + Outpt + "." ' make message
ELSE ' check trigger
    Inpt = " <none>" ' make default message
END IF ' end check trigger
Outpt = "[B]Spell trigger in effect for spell:" + Inpt
CALL IO.O ' send output message
IF ActionRecord.MonsterTalk THEN ' check action trigger
    CALL Read.Record(MonsterFile, ActionRecord.MonsterTalk) ' get monster record
    Outpt = MonsterRecord.MonsterName ' store monster name
    Outpt = RTRIM$(Outpt) ' trim name
    Inpt = STR$(ActionRecord.MonsterTalk) + ", " + Outpt + "." ' make message
ELSE ' check trigger
    Inpt = " <none>" ' make default message
END IF ' end check trigger
Outpt = "[C]Monster talk trigger in effect for monster:" + Inpt ' make message
CALL IO.O ' send output message
IF ActionRecord.Level > False THEN ' check action trigger
    Inpt = STR$(ActionRecord.Level) ' make message
ELSE ' check trigger
    Inpt = " <none>" ' make default message
END IF ' end check trigger
Outpt = "[D]High level room entry:" + Inpt ' message
IF ActionRecord.Level < False THEN ' check action trigger
    Inpt = STR$(ABS(ActionRecord.Level)) ' make message
ELSE ' check trigger
    Inpt = " <none>" ' make default message
END IF ' end check trigger
Outpt = Outpt + ", Low level room entry:" + Inpt ' message
CALL IO.O ' send output message
Outpt = "Restricted room directions:" ' make message
CALL IO.O ' send output message
IF ActionRecord.Restrictions > False THEN ' check action trigger
    Outpt = Nul ' reset message
    FOR Direction.Number = 1 TO 12 ' loop through directions
        ' check restriction trigger
        IF ActionRecord.Restrictions AND 2 ^ Direction.Number THEN
            Outpts = Direction(Direction.Number) ' make direction name
            Outpt = Outpt + Outpts + ", " ' append direction
        END IF ' end check trigger
    NEXT ' end loop
    IF ActionRecord.Restrictions AND 2 ^ 13 THEN ' check restriction trigger
        Outpt = Outpt + "enter portal, " ' append message
    END IF ' end check trigger
    IF LEN(Outpt) THEN ' check string length
        Outpt = LEFT$(Outpt, LEN(Outpt) - 2) ' truncate comma
    END IF ' end check string length
ELSE ' check trigger
    Outpt = "<none>" ' default message
END IF ' end check trigger
CALL IO.O ' send output message
Outpt = "[E]Action hits for:"
SELECT CASE ActionRecord.HitPoints ' check action trigger
    CASE IS > False
        Inpt = STR$(ActionRecord.HitPoints) + " fatigue" ' make message
    CASE IS < False
        Inpt = STR$(ABS(ActionRecord.HitPoints)) + " vitality" ' make message
    CASE ELSE ' check trigger
        Inpt = " <none>" ' default message
END SELECT ' end check trigger
Outpt = Outpt + Inpt
CALL IO.O ' send output message
IF ActionRecord.EncounterRate THEN ' check action trigger
    Inpt = STR$(ActionRecord.EncounterRate) ' make message
ELSE ' check trigger
    Inpt = " <none>" ' default message
END IF ' end check trigger
Outpt = "[F]Encounter rate:" + Inpt
IF ActionRecord.HealthRate THEN ' check action trigger
    Inpt = STR$(ActionRecord.HealthRate) ' make message
ELSE ' check trigger
    Inpt = " <none>" ' default message
END IF ' end check trigger
Outpt = Outpt + ", Health rate:" + Inpt
CALL IO.O ' send output message
Outpt = "[G]Inventory: "
SELECT CASE ActionRecord.Inventory ' make selection of inventory actions
    CASE 1 ' weapon action
        Inpt = "breaks weapons" ' make message
    CASE 2 ' shield action
        Inpt = "smashes shields" ' make message
    CASE 3 ' armor action
        Inpt = "wrecks armor" ' make message
    CASE 4 ' magic item action
        Inpt = "drains magic items" ' make message
    CASE ELSE ' check trigger
        Inpt = "<none>"
END SELECT ' end selection of inventory actions
Outpt = Outpt + Inpt
CALL IO.O
IF ActionRecord.Fumble THEN ' check action trigger
    Outpt = "[H]Action fumbles." ' make message
ELSE ' check trigger
    Outpt = "[H]Action does not fumble." ' make message
END IF ' end check trigger
CALL IO.O ' send output message
IF ActionRecord.Teleport THEN ' check action trigger
    Outpt = "[I]Action teleports to room" + STR$(ActionRecord.Teleport) + "."
ELSE ' check trigger
    Outpt = "[I]Action does not teleport." ' make message
END IF ' end check trigger
CALL IO.O ' send output message
IF ActionRecord.RustRate THEN ' check action trigger
    Inpt = STR$(ActionRecord.RustRate) ' make message
ELSE ' check trigger
    Inpt = " <none>" ' default message
END IF ' end check trigger
Outpt = "[J]Action has weapon rusting rate of" + Inpt + " rounds." ' message
CALL IO.O ' send output message
IF ActionRecord.StealRate THEN ' check action trigger
    Inpt = STR$(ActionRecord.StealRate) ' make message
ELSE ' check trigger
    Inpt = " <none>" ' default message
END IF ' end check trigger
Outpt = "[K]Action has monster stealing rate of" + Inpt + " rounds." ' message
CALL IO.O ' send output message
Outpt = "[L]Action attributes: " ' make edit display message
IF ActionRecord.Attribute1 = LitRoom THEN ' check attribute
    Outpt = Outpt + "Lit" ' append attribute message
ELSE ' check attribute
    IF ActionRecord.Attribute1 = UnlitRoom THEN ' check attribute
        Outpt = Outpt + "Unlit" ' append attribute message
    END IF ' end check attribute
END IF ' end check attribute
Outpt = Outpt + ", " ' append comma
SELECT CASE ActionRecord.Attribute2 ' select attribute
    CASE Land ' check attribute
        Outpt = Outpt + "Land" ' append attribute message
    CASE Air ' check attribute
        Outpt = Outpt + "Air" ' append attribute message
    CASE Water ' check attribute
        Outpt = Outpt + "Water" ' append attribute message
END SELECT ' end select attribute
CALL IO.O ' send attribute message
Outpt = "[X]Clear action" ' make alternate display message
CALL IO.O ' send alternate display message
END SUB

REM * This routines displays information on the monster file.
REM * input variables:
REM *   Number - the monster file record.

SUB Display.Monster (Number)
IF Number <= False THEN ' check number
    EXIT SUB ' exit subroutine
END IF ' end check number
Graphics.Off = False ' reset color
Outpt = "Monster number" + STR$(Number) + ":" ' monster number message
CALL IO.O ' send output message
Graphics.Off = True ' reset color
Outpt = "[A]Monster name: " + RTRIM$(MonsterRecord.MonsterName) ' message
CALL IO.O ' send output message
Outpt = "[B]Plural of monster name: " + MonsterRecord.PluralName ' message
CALL IO.O ' send output message
Outpt = "[C]Monster level:" + STR$(MonsterRecord.Level) ' message
CALL IO.O ' send output message
Outpt = "[D]Magical monster: " ' message
IF MonsterRecord.Magic THEN ' verify monster
    Outpt = Outpt + "Yes" ' add to message
ELSE ' verify
    Outpt = Outpt + "No" ' add to message
END IF ' end verify
CALL IO.O ' send output message
Outpt = "[E]Hit points:" + STR$(MonsterRecord.Hits) ' message
CALL IO.O ' send output message
Outpt = "[F]Experience points:" + STR$(MonsterRecord.Experience) ' message
CALL IO.O ' send output message
Outpt = "[G]Gold points:" + STR$(MonsterRecord.Gold) ' message
CALL IO.O ' send output message
Outpt = "[H]Number appearing:" + STR$(MonsterRecord.NumberAppearing) ' message
CALL IO.O ' send output message
Outpt = "[I]Poisonous monster: " ' message
IF MonsterRecord.Poison THEN ' verify monster
    Outpt = Outpt + "Yes(" + MID$(STR$(MonsterRecord.PoisonPercent), 2) + " percent)" ' message
ELSE ' verify
    Outpt = Outpt + "No" ' add to message
END IF ' end verify
CALL IO.O ' send output message
Outpt = "[J]Level draining monster: " ' message
IF MonsterRecord.LevelDrain THEN ' verify monster
    Outpt = Outpt + "Yes(" + MID$(STR$(MonsterRecord.DrainPercent), 2) + " percent)" ' message
ELSE ' verify
    Outpt = Outpt + "No" ' add to message
END IF ' end verify
CALL IO.O ' send output message
Outpt = "[K]Monster blocks exits: " ' message
IF MonsterRecord.Block THEN ' verify monster
    Outpt = Outpt + "Yes(" + MID$(STR$(MonsterRecord.BlockPercent), 2) + " percent)" ' message
ELSE ' verify
    Outpt = Outpt + "No" ' add to message
END IF ' end verify
CALL IO.O ' send output message
Outpt = "[L]Monster prevents treasure take: " ' message
IF MonsterRecord.Prevent THEN ' verify monster
    Outpt = Outpt + "Yes(" + MID$(STR$(MonsterRecord.PreventPercent), 2) + " percent)" ' message
ELSE ' verify
    Outpt = Outpt + "No" ' add to message
END IF ' end verify
CALL IO.O ' send output message
Outpt = "[M]Monster follows player: " ' message
IF MonsterRecord.Follow THEN ' verify monster
    Outpt = Outpt + "Yes(" + MID$(STR$(MonsterRecord.FollowPercent), 2) + " percent)(" + MID$(STR$(MonsterRecord.Teleport), 2) + "% teleport)"
ELSE ' verify
    Outpt = Outpt + "No" ' add to message
END IF ' end verify
CALL IO.O ' send output message
Outpt = "[N]Monster casts spells: " ' message
Spell.Number = MonsterRecord.Spell ' store spell number
IF Spell.Number = False THEN ' verify monster
    Outpt = Outpt + "No" ' add to message
ELSE ' verify
    ' file bounds
    IF Spell.Number > False AND Spell.Number <= LOF(SpellFile) / LEN(SpellRecord) THEN
        CALL Read.Record(SpellFile, Spell.Number) ' get spell record
        Outpt = Outpt + "Yes, " + RTRIM$(SpellRecord.SpellName) + "(" + MID$(STR$(MonsterRecord.SpellPercent), 2) + " percent)" ' message
    END IF ' end check file bounds
END IF ' end verify
CALL IO.O ' send output message
Outpt = "[O]Monster jails attacker: " ' message
IF MonsterRecord.Jail THEN ' verify monster
    Outpt = Outpt + "Yes" ' add to message
ELSE ' verify
    Outpt = Outpt + "No" ' add to message
END IF ' end verify
CALL IO.O ' send output message
Outpt = "[P]Encounter rate:"
IF MonsterRecord.Rate = False THEN ' verify rate
    Outpt = Outpt + " No"
ELSE ' verify rate
    Outpt = Outpt + STR$(MonsterRecord.Rate) + " (" + MID$(STR$(MonsterRecord.RatePercent), 2) + " percent)" ' message
END IF ' end verify rate
CALL IO.O ' send output message
Outpt = "[R]Permanent monster: " ' message
IF MonsterRecord.Permanent THEN ' verify monster
    Outpt = Outpt + "Yes" ' add to message
ELSE ' verify
    Outpt = Outpt + "No" ' add to message
END IF ' end verify
CALL IO.O ' send output message
Outpt = "[S]Monster uses psionics: " ' message
IF MonsterRecord.Psionic = False THEN ' verify monster
    Outpt = Outpt + "No" ' add to message
ELSE ' verify
    Spell.Number = MonsterRecord.PsionicSpell ' store psionic spell number
    ' file bounds
    IF Spell.Number > False AND Spell.Number <= LOF(SpellFile) / LEN(SpellRecord) THEN
        CALL Read.Record(SpellFile, Spell.Number) ' get spell record
        Outpt = Outpt + "Yes, " + SpellRecord.SpellName ' message
    END IF ' end check file bounds
END IF ' end verify
CALL IO.O ' send output message
Outpt = "[T]Inventory:" ' message
Number.Items = False ' reset counter
FOR Array.Number = 1 TO 5 ' loop through monster treasure
    ' get monster treasure number
    Treasure.Number = GetMonsterRecordTreasure(Array.Number)
    ' check file bounds
    IF Treasure.Number > False AND Treasure.Number <= LOF(TreasureFile) / LEN(TreasureRecord) THEN
        Number.Items = Number.Items + 1 ' increment counter
    END IF ' end check file bounds
NEXT ' end loop through monster treasure
IF Number.Items = 1 THEN ' check value
    Outpt = Outpt + STR$(Number.Items) + " item." ' make display message
ELSE ' check value
    Outpt = Outpt + STR$(Number.Items) + " items." ' make display message
END IF ' end check value
CALL IO.O ' send output message
END SUB ' end routine

REM * This routines displays information on the monster class file.
REM * input variables:
REM *   Number - the monster class file record.

SUB Display.Monster.Class (Number)
IF Number <= False THEN ' check number
    EXIT SUB ' exit subroutine
END IF ' end check number
Graphics.Off = False ' reset color
Outpt = "Monster class number" + STR$(Number) + ":"
CALL IO.O ' send output message
Graphics.Off = True ' reset color
FOR Array.Number = 1 TO 10 ' loop through the monster classes
    ' get monster number in class
    Monster.Index = GetMonclassRecordMonsters(Array.Number)
    ' file bounds
    IF Monster.Index > False AND Monster.Index <= LOF(MonsterFile) / LEN(MonsterRecord) THEN
        CALL Read.Record(MonsterFile, Monster.Index) ' get monster record
        Outpt = "Monster number" + STR$(Array.Number) + ":" + RTRIM$(MonsterRecord.MonsterName) ' make display message
        CALL IO.O ' send output message
    END IF ' end check file bounds
NEXT ' end loop through monster class
END SUB ' end routine to display monster class

REM * This routines displays information on the nonplayer file.
REM * input variables:
REM *   Number - the nonplayer file record.

SUB Display.Nonplayer (Number)
IF Number <= False THEN ' check number
    EXIT SUB ' exit subroutine
END IF ' end check number
Graphics.Off = False ' reset color
Outpt = "Nonplayer number" + STR$(Number) + ":" ' nonplayer number message
CALL IO.O ' send output message
Graphics.Off = True ' reset color
Outpt = "[A]Nonplayer name: " + RTRIM$(MonsterRecord.MonsterName) ' message
CALL IO.O ' send output message
Outpt = "[B]Nonplayer rooms: " + MonsterRecord.PluralName ' message
CALL IO.O ' send output message
Outpt = "[C]Nonplayer level:" + STR$(MonsterRecord.Level) ' message
CALL IO.O ' send output message
Outpt = "[D]Hit points:" + STR$(MonsterRecord.Hits) ' message
CALL IO.O ' send output message
Outpt = "[E]Experience points:" + STR$(MonsterRecord.Experience) ' message
CALL IO.O ' send output message
Outpt = "[F]Gold points:" + STR$(MonsterRecord.Gold) ' message
CALL IO.O ' send output message
Outpt = "[G]Poisonous nonplayer: " ' message
IF MonsterRecord.Poison THEN ' verify nonplayer
    Outpt = Outpt + "Yes(" + MID$(STR$(MonsterRecord.PoisonPercent), 2) + " percent)" ' message
ELSE ' verify
    Outpt = Outpt + "No" ' add to message
END IF ' end verify
CALL IO.O ' send output message
Outpt = "[H]Level draining nonplayer: " ' message
IF MonsterRecord.LevelDrain THEN ' verify nonplayer
    Outpt = Outpt + "Yes(" + MID$(STR$(MonsterRecord.DrainPercent), 2) + " percent)" ' message
ELSE ' verify
    Outpt = Outpt + "No" ' add to message
END IF ' end verify
CALL IO.O ' send output message
Outpt = "[I]Nonplayer blocks exits: " ' message
IF MonsterRecord.Block THEN ' verify nonplayer
    Outpt = Outpt + "Yes(" + MID$(STR$(MonsterRecord.BlockPercent), 2) + " percent)" ' message
ELSE ' verify
    Outpt = Outpt + "No" ' add to message
END IF ' end verify
CALL IO.O ' send output message
Outpt = "[J]Nonplayer prevents treasure take: " ' message
IF MonsterRecord.Prevent THEN ' verify nonplayer
    Outpt = Outpt + "Yes(" + MID$(STR$(MonsterRecord.PreventPercent), 2) + " percent)" ' message
ELSE ' verify
    Outpt = Outpt + "No" ' add to message
END IF ' end verify
CALL IO.O ' send output message
Outpt = "[K]Nonplayer follows player: " ' message
IF MonsterRecord.Follow THEN ' verify nonplayer
    Outpt = Outpt + "Yes(" + MID$(STR$(MonsterRecord.FollowPercent), 2) + " percent)(" + MID$(STR$(MonsterRecord.Teleport), 2) + "% teleport)" ' message
ELSE ' verify
    Outpt = Outpt + "No" ' add to message
END IF ' end verify
CALL IO.O ' send output message
Outpt = "[L]Nonplayer casts spells: " ' message
Spell.Number = MonsterRecord.Spell ' store spell number
IF Spell.Number = False THEN ' verify nonplayer
    Outpt = Outpt + "No" ' add to message
ELSE ' verify
    ' file bounds
    IF Spell.Number > False AND Spell.Number <= LOF(SpellFile) / LEN(SpellRecord) THEN
        CALL Read.Record(SpellFile, Spell.Number) ' get spell record
        Outpt = Outpt + "Yes, " + RTRIM$(SpellRecord.SpellName) + "(" + MID$(STR$(MonsterRecord.SpellPercent), 2) + " percent)" ' message
    END IF ' end check file bounds
END IF ' end verify
CALL IO.O ' send output message
Outpt = "[M]Nonplayer jails attacker: " ' message
IF MonsterRecord.Jail THEN ' verify nonplayer
    Outpt = Outpt + "Yes" ' add to message
ELSE ' verify
    Outpt = Outpt + "No" ' add to message
END IF ' end verify
CALL IO.O ' send output message
Outpt = "[N]Encounter rate:"
IF MonsterRecord.Rate = False THEN ' verify rate
    Outpt = Outpt + " No"
ELSE ' verify rate
    Outpt = Outpt + STR$(MonsterRecord.Rate) + "(" + MID$(STR$(MonsterRecord.RatePercent), 2) + " percent)" ' message
END IF ' end verify rate
CALL IO.O ' send output message
Outpt = "[O]Nonplayer uses psionics: " ' message
IF MonsterRecord.Psionic = False THEN ' verify nonplayer
    Outpt = Outpt + "No" ' add to message
ELSE ' verify
    Spell.Number = MonsterRecord.PsionicSpell ' store psionic spell number
    ' file bounds
    IF Spell.Number > False AND Spell.Number <= LOF(SpellFile) / LEN(SpellRecord) THEN
        CALL Read.Record(SpellFile, Spell.Number) ' get spell record
        Outpt = Outpt + "Yes, " + SpellRecord.SpellName ' message
    END IF ' end check file bounds
END IF ' end verify
CALL IO.O ' send output message
Outpt = "[P]Carries treasure:" ' message
CALL IO.O ' send output message
FOR Array.Number = 1 TO 5 ' loop through nonplayer treasure
    ' get nonplayer treasure number
    Treasure.Number = GetMonsterRecordTreasure(Array.Number)
    IF Treasure.Number > False AND Treasure.Number <= LOF(TreasureFile) / LEN(TreasureRecord) THEN ' file bounds
        CALL Read.Record(TreasureFile, Treasure.Number) ' get treasure record
        Outpt = TreasureRecord.TreasureName ' store treasure name
        CALL IO.O ' send output message
    END IF ' end check file bounds
NEXT ' end loop through nonplayer treasure
END SUB ' end routine

REM * This routines displays information on the object file.
REM * input variables:
REM *   Number - the object file record.

SUB Display.Object (Number)
IF Number <= False THEN ' check number
    EXIT SUB ' exit subroutine
END IF ' end check number
Graphics.Off = False ' reset color
Outpt = "Object number" + STR$(Number) + ":" ' object number message
CALL IO.O ' send output message
Graphics.Off = True ' reset color
Outpt = "[A]Object name: " + RTRIM$(ObjectRecord.ObjectName) ' message
CALL IO.O ' send output message
Outpt = "[B]Object identifier: " + LCASE$(RTRIM$(ObjectRecord.ShortName))
CALL IO.O ' send output message
Outpt = "[C]Room number link:" + STR$(ObjectRecord.RoomLink) ' message
CALL IO.O ' send output message
Outpt = "[D]Trapped portal: " ' message
SELECT CASE ObjectRecord.Trap ' selection of object trap type
    CASE False ' verify object
        Outpt = Outpt + "No" ' add to message
    CASE 1 ' verify object
        Outpt = Outpt + "poison needles" ' message
    CASE 2 ' berify object
        Outpt = Outpt + "teleport to" + STR$(ObjectRecord.Teleport) ' message
    CASE 3 ' verify object
        Outpt = Outpt + "hits for" + STR$(ABS(ObjectRecord.Teleport)) + " " ' message
        SELECT CASE Object.Teleport ' selection of object trap hit type
            CASE IS < False ' verify object
                Outpt = Outpt + "vitality" ' message
            CASE ELSE ' verify object
                Outpt = Outpt + "fatigue" ' message
        END SELECT ' end selection of object hit type
END SELECT ' end selection of object type
CALL IO.O ' send output message
Outpt = "[E]Long description:" ' message
CALL IO.O ' send output message
Outpt = RTRIM$(ObjectRecord.LongDesc) ' message
IF Outpt = Nul THEN ' check message
    Outpt = "<none>" ' reset
END IF ' end check message
CALL IO.O ' send output message
Outpt = "[F]Entry description:" ' message
CALL IO.O ' send output message
Outpt = RTRIM$(ObjectRecord.ShortDesc) ' message
IF Outpt = Nul THEN ' check message
    Outpt = "<none>" ' reset
END IF ' end check message
CALL IO.O ' send output message
Outpt = "[G]Hidden object: " ' message
IF ObjectRecord.Hidden THEN ' verify object
    Outpt = Outpt + "Yes" ' add to message
ELSE ' verify
    Outpt = Outpt + "No" ' add to message
END IF ' end verify
CALL IO.O ' send output message
Outpt = "[H]Invisible object: " ' message
IF ObjectRecord.Invisible THEN ' verify object
    Outpt = Outpt + "Yes" ' add to message
ELSE ' verify
    Outpt = Outpt + "No" ' add to message
END IF ' end verify
CALL IO.O ' send output message
Outpt = "[I]Jail attacker trap: " ' message
IF ObjectRecord.JailTrap THEN ' verify object
    Outpt = Outpt + "Yes" ' add to message
ELSE ' verify
    Outpt = Outpt + "No" ' add to message
END IF ' end verify
CALL IO.O ' send output message
Outpt = "[J]Locked portal: " ' message
IF ObjectRecord.DoorLock = 2 THEN ' verify object
    Outpt = Outpt + "Yes" ' add to message
ELSE ' verify
    Outpt = Outpt + "No" ' add to message
END IF ' end verify
CALL IO.O ' send output message
Outpt = "[K]Relocking portal: " ' message
IF ObjectRecord.Relocks THEN ' verify object
    Outpt = Outpt + "Yes" ' add to message
ELSE ' verify
    Outpt = Outpt + "No" ' add to message
END IF ' end verify
CALL IO.O ' send output message
Outpt = "[L]Key number:" + STR$(ObjectRecord.Keyed) ' message
CALL IO.O ' send output message
Outpt = "[M]Permanent object: " ' message
IF ObjectRecord.Permanent THEN ' verify object
    Outpt = Outpt + "Yes" ' add to message
ELSE ' verify
    Outpt = Outpt + "No" ' add to message
END IF ' end verify
CALL IO.O ' send output message
Outpt = "[N]Object is a light: " ' message
IF ObjectRecord.LightRoom = False THEN ' verify object
    Outpt = Outpt + "No" ' add to message
ELSE ' verify
    Outpt = Outpt + "Yes, Light Time: " ' message
    IF ObjectRecord.LightTime = False THEN ' verify object
        Outpt = Outpt + "<any>" ' message
    ELSE ' verify
        Outpt = Outpt + "From:" + RIGHT$(STR$(ObjectRecord.FromHour + 100), 2) + ":" + RIGHT$(STR$(ObjectRecord.FromMin + 100), 2) + " to " + RIGHT$(STR$(ObjectRecord.ToHour + 100), 2) + ":" + RIGHT$(STR$(ObjectRecord.ToMin + 100), 2) ' message
    END IF ' end verify
END IF ' end verify
CALL IO.O ' send output message
END SUB ' end routine

REM * This routines displays information on the room file.
REM * input variables:
REM *   Number! - the room file record.

SUB Display.Room.Desc (Number!)
IF Number! <= 0! THEN ' check number
    EXIT SUB ' exit subroutine
END IF ' end check number
Graphics.Off = False ' reset color
Outpt = "Room number" + STR$(Number!) + ": Action" + STR$(RoomRecord.Action) + ": Monster class:" + STR$(RoomRecord.MonsterClass) + "." ' make room message
CALL IO.O ' send output message
Outpt = "Short description:" ' message
CALL IO.O ' send output message
Graphics.Off = True ' reset color
Outpt = RTRIM$(RoomRecord.ShortDesc) ' message
IF INSTR(Outpt, CHR$(0)) THEN ' find any null characters from old structures
    Outpt = LEFT$(Outpt, INSTR(Outpt, CHR$(0)) - 1) ' truncate off nulls
END IF ' end find old nulls
CALL IO.O ' send output message
Graphics.Off = False ' reset color
Outpt = "Long description:" ' message
CALL IO.O ' send output message
Graphics.Off = True ' reset color
FOR Array.Number = 1 TO 4 ' loop through all four long room description lines
    Outpt = GetRoomRecordLongDesc$(Array.Number) ' get next room description line
    Outpt = RTRIM$(Outpt) ' trim room description line
    IF INSTR(Outpt, CHR$(0)) THEN ' find any nul characters from old files
        Outpt = LEFT$(Outpt, INSTR(Outpt, CHR$(0)) - 1) ' truncate off nulls
    END IF ' end find old nulls
    IF Outpt = Nul THEN ' check if long description ends
        EXIT FOR ' exit long description display loop
    END IF ' end check description end
    CALL IO.O ' send output message
NEXT ' end long room description loop
END SUB ' end routine

REM * This routines displays information on the spell file.
REM * input variables:
REM *   Number - the spell file record.

SUB Display.Spell (Number)
IF Number <= False THEN ' check number
    EXIT SUB ' exit subroutine
END IF ' end check number
Graphics.Off = False ' reset color
Outpt = "Spell number" + STR$(Number) + ":" ' spell number message
CALL IO.O ' send output message
Graphics.Off = True ' reset color
Outpt = "[A]Spell name: " + RTRIM$(SpellRecord.SpellName) ' message
CALL IO.O ' send output message
Outpt = "[B]Spell chant:" ' message
CALL IO.O ' send output message
Outpt = LCASE$(RTRIM$(SpellRecord.Chant)) ' message
CALL IO.O ' send output message
Outpt = "[C]Spell cast description:" ' message
CALL IO.O ' send output message
Outpt = RTRIM$(SpellRecord.Desc)
CALL IO.O ' send output message
Outpt = "[D]Spell level:" + STR$(SpellRecord.Level) ' message
CALL IO.O ' send output message
Outpt = "[E]" ' set to option
SELECT CASE SpellRecord.SpellType ' selection of spell type
    CASE Enchant
        Inpt = "Enchant" ' message
    CASE Offense
        Inpt = "Offense" ' message
    CASE Bless
        Inpt = "Bless" ' message
    CASE Wish
        Inpt = "Wish" ' message
    CASE Poison
        Inpt = "Poison" ' message
    CASE Vigor
        Inpt = "Vigor" ' message
    CASE Heal
        Inpt = "Heal" ' message
    CASE CurePoison
        Inpt = "Curepoison" ' message
    CASE LevelDrain
        Inpt = "Level Drain" ' message
    CASE Teleport
        Inpt = "Teleport to Room" + STR$(SpellRecord.Teleport)
    CASE Befuddled
        Inpt = "Befuddle" ' message
    CASE TurnUndead
        Inpt = "Turn Undead" ' message
    CASE PassDoor
        Inpt = "Pass Door" ' message
    CASE Conjure
        Inpt = "Conjure" ' message
    CASE Psionic
        Inpt = "Psionic" ' message
    CASE DetectLock
        Inpt = "Detect Lock" ' message
    CASE DetectEvil
        Inpt = "Detect Evil" ' message
    CASE DetectTrap
        Inpt = "Detect Trap" ' message
    CASE Intoxicate
        Inpt = "Intoxicate" ' message
    CASE SetTrap
        Inpt = "Set Trap" ' message
    CASE Hiding
        Inpt = "Hide" ' message
    CASE Search
        Inpt = "Search" ' message
    CASE Invisibility
        Inpt = "Invisibility" ' message
    CASE Identify
        Inpt = "Identify" ' message
    CASE Enlighten
        Inpt = "Enlighten" ' message
    CASE Illuminate
        Inpt = "Illuminate" ' message
    CASE Psyche
        Inpt = "Psyche" ' message
    CASE Telepathy
        Inpt = "Telepathy" ' message
    CASE ELSE ' other
        Inpt = "<none>" ' message
END SELECT ' end selection of spell type
Outpt = Outpt + "Spell type: " + Inpt ' message
CALL IO.O ' send output message
Outpt = "[F]Spell is psionic: " ' message
IF SpellRecord.Psionic = False THEN ' verify spell
    Outpt = Outpt + "No" ' add to message
ELSE ' verify
    Outpt = Outpt + "Yes, " ' message
    SELECT CASE SpellRecord.PsionicMode ' selection of spell psionic type
        CASE PsiAttack
            Outpt = Outpt + "Attack" ' message
        CASE PsiDefense
            Outpt = Outpt + "Defense" ' message
        CASE ELSE ' other
            Outpt = Outpt + "<none>" ' message
    END SELECT ' end selection of spell type
END IF ' end verify
CALL IO.O ' send output message
Outpt = "[G]Characters which can cast spell:" ' message
CALL IO.O ' send output message
Spell.Cast = False ' set spell cast found flag
FOR Class.Number = 1 TO 8 ' loop through spell cast flags
    IF SpellRecord.ClassType AND 2 ^ Class.Number THEN ' compare spell cast flag
        Class.Type$ = Class.Name(Class.Number) ' store class name
        Class.Type$ = RTRIM$(Class.Type$) ' trim name
        ' uppercase first word
        MID$(Class.Type$, 1, 1) = UCASE$(MID$(Class.Type$, 1, 1))
        Outpt = Outpt + Class.Type$ + ", " ' append to output string
        Spell.Cast = True ' set spell cast found flag
    END IF ' end compare spell cast flags
NEXT ' end loop through spell cast flags
IF Spell.Cast THEN ' check spell cast found flag
    Outpt = LEFT$(Outpt, LEN(Outpt) - 2) + "." ' message
ELSE ' check spell flag
    Outpt = "<none>" ' message
END IF ' end check spell cast found flag
CALL IO.O ' send output message
Outpt = "[H]Spell ingredients:" ' message
CALL IO.O ' send output message
Spell.Ingredient = False ' set spell ingredient found flag
FOR Array.Number = 1 TO 5 ' loop through spell ingerdients
    ' get spell ingredient number
    Treasure.Number = GetSpellRecordIngred(Array.Number)
    IF Treasure.Number > False AND Treasure.Number <= LOF(TreasureFile) / LEN(TreasureRecord) THEN ' file bounds
        CALL Read.Record(TreasureFile, Treasure.Number) ' get treasure record
        Ingredient.Name$ = TreasureRecord.TreasureName ' store treasure name
        Ingredient.Name$ = RTRIM$(Ingredient.Name$) ' trim name
        Ingredient.Name$ = LCASE$(Ingredient.Name$) ' lowercase name
        ' uppercase first word
        MID$(Ingredient.Name$, 1, 1) = UCASE$(MID$(Ingredient.Name$, 1, 1))
        Outpt = Outpt + Ingredient.Name$ + ", " ' message
        Spell.Ingredient = True ' set spell ingredient found flag
    END IF ' end check file bounds
NEXT ' end loop through spell ingredients
IF Spell.Ingredient THEN ' check spell ingredient found flag
    Outpt = LEFT$(Outpt, LEN(Outpt) - 2) + "." ' message
ELSE ' check spell ingredient
    Outpt = "<none>" ' message
END IF ' end check spell ingredient found flag
CALL IO.O ' send output message
Outpt = "[I]Spell casting type which requires ingredients:" ' message
CALL IO.O ' send output message
Spell.Requirement = False ' set ingredient requirement flag
IF SpellRecord.SpellFlag AND Use.Spell.Type THEN ' verify spell
    Outpt = "use command, " ' message
    Spell.Requirement = True ' set ingredient flag
END IF ' end verify
IF SpellRecord.SpellFlag AND Scroll.Spell.Type THEN ' verify spell
    Outpt = Outpt + "read scroll, " ' message
    Spell.Requirement = True ' set ingredient flag
END IF ' end verify
IF SpellRecord.SpellFlag AND Cast.Spell.Type THEN ' verify spell
    Outpt = Outpt + "cast spell, " ' message
    Spell.Requirement = True ' set ingredient flag
END IF ' end verify
IF Spell.Requirement THEN ' check ingredient flag
    Outpt = LEFT$(Outpt, LEN(Outpt) - 2) + "." ' message
ELSE ' check flag
    Outpt = "<none>" ' message
END IF ' end check ingredient flag
CALL IO.O ' send output message
END SUB ' end routine

REM * This routines displays information on the treasure file.
REM * input variables:
REM *   Number - the treasure file record.

SUB Display.Treasure (Number)
IF Number <= False THEN ' check number
    EXIT SUB ' exit subroutine
END IF ' end check number
Graphics.Off = False ' reset color
Outpt = "Treasure number" + STR$(Number) + ":" ' treasure number message
CALL IO.O ' send output message
Graphics.Off = True ' reset color
Outpt = "[A]Treasure name: " + TreasureRecord.TreasureName ' message
Outpt = LEFT$(Outpt, 40) ' truncate string
Outpt = Outpt + SPACE$(40 - LEN(Outpt)) ' pad with blanks
Outpt = Outpt + "[U]Treasure is coins: " ' message
IF TreasureRecord.Coin THEN ' verify treasure
    Outpt = Outpt + "Yes" ' add to message
ELSE ' verify
    Outpt = Outpt + "No" ' add to message
END IF ' end verify
CALL IO.O ' send output message
Outpt = "[B]Treasure identifier: " + LCASE$(TreasureRecord.ShortName) ' message
Outpt = LEFT$(Outpt, 40) ' truncate string
Outpt = Outpt + SPACE$(40 - LEN(Outpt)) ' pad with blanks
Outpt = Outpt + "[V]Treasure is a potion: " ' message
IF TreasureRecord.Potion THEN ' verify treasure
    Outpt = Outpt + "Yes" ' add to message
ELSE ' verify
    Outpt = Outpt + "No" ' add to message
END IF ' end verify
CALL IO.O ' send output message
Outpt = "[C]Weight:" + STR$(TreasureRecord.Weight) ' message
Outpt = LEFT$(Outpt, 40) ' truncate string
Outpt = Outpt + SPACE$(40 - LEN(Outpt)) ' pad with blanks
Outpt = Outpt + "[W]Treasure is a scroll: " ' message
IF TreasureRecord.Scroll THEN ' verify treasure
    Outpt = Outpt + "Yes" ' add to message
ELSE ' verify
    Outpt = Outpt + "No" ' add to message
END IF ' end verify
CALL IO.O ' send output message
Outpt = "[D]Gold coin value:" + STR$(TreasureRecord.Gold) ' message
Outpt = LEFT$(Outpt, 40) ' truncate string
Outpt = Outpt + SPACE$(40 - LEN(Outpt)) ' pad with blanks
Outpt = Outpt + "[X]Treasure is invisible: " ' message
IF TreasureRecord.Invisible THEN ' verify treasure
    Outpt = Outpt + "Yes" ' add to message
ELSE ' verify
    Outpt = Outpt + "No" ' add to message
END IF ' end verify
CALL IO.O ' send output message
Outpt = "[E]Treasure type: " ' message
IF TreasureRecord.Loadable THEN ' check treasure type
    Outpt = Outpt + "Loadable" ' append message
ELSE ' check next type
    IF TreasureRecord.Launchable THEN ' check treasure type
        Outpt = Outpt + "Launchable" ' append message
    ELSE ' check next type
        SELECT CASE TreasureRecord.Type ' selection of treasure type
            CASE False ' verify treasure type
                IF TreasureRecord.Plus = False THEN ' verify weapon type
                    Outpt = Outpt + "Treasure" ' message
                ELSE ' verify type
                    Outpt = Outpt + "Weapon" ' message
                END IF ' end verify treasure type
            CASE IS < False ' verify shield type
                Outpt = Outpt + "Shield" ' message
            CASE IS > False ' verify armor type
                Outpt = Outpt + "Armor" ' message
        END SELECT ' end selection of treasure type
    END IF ' end check treasure type
END IF ' end check treasure type
Outpt = LEFT$(Outpt, 40) ' truncate string
Outpt = Outpt + SPACE$(40 - LEN(Outpt)) ' pad with blanks
Outpt = Outpt + "[Y]Treasure is a container: " ' message
IF TreasureRecord.Container THEN ' verify treasure
    Outpt = Outpt + "Yes" ' add to message
ELSE ' verify
    Outpt = Outpt + "No" ' add to message
END IF ' end verify
CALL IO.O ' send output message
Outpt = "[F]Hit plus:" + STR$(ABS(TreasureRecord.Plus)) ' message
Outpt = LEFT$(Outpt, 40) ' truncate string
Outpt = Outpt + SPACE$(40 - LEN(Outpt)) ' pad with blanks
Outpt = Outpt + "[Z]Container is locked: " ' message
IF TreasureRecord.Locked = 1 AND TreasureRecord.Closed = 1 THEN ' verify treasure
    Outpt = Outpt + "Yes" ' add to message
ELSE ' verify
    Outpt = Outpt + "No" ' add to message
END IF ' end verify
CALL IO.O ' send output message
Outpt = "[G]Charges:" + STR$(TreasureRecord.Charges) ' message
Outpt = LEFT$(Outpt, 40) ' truncate string
Outpt = Outpt + SPACE$(40 - LEN(Outpt)) ' pad with blanks
Outpt = Outpt + "[1]Treasure key number:" + STR$(TreasureRecord.Keyed) ' message
CALL IO.O ' send output message
Outpt = "[H]Weapon class: " ' message
' selection of treasure proficiency type
SELECT CASE TreasureRecord.Proficiency
    CASE Blunt
        Outpt = Outpt + "Blunt" ' message
    CASE Pole
        Outpt = Outpt + "Pole" ' message
    CASE Sharp
        Outpt = Outpt + "Sharp" ' message
    CASE Thrusting
        Outpt = Outpt + "Thrusting" ' message
    CASE ELSE ' other
        Outpt = Outpt + "None" ' message
END SELECT ' end selection of weapon proficiency type
Outpt = LEFT$(Outpt, 40) ' truncate string
Outpt = Outpt + SPACE$(40 - LEN(Outpt)) ' pad with blanks
Outpt = Outpt + "[2]Treasure is fuel: " ' message
IF TreasureRecord.FuelType = False THEN ' verify treasure
    Outpt = Outpt + "No" ' add to message
ELSE ' verify
    Outpt = Outpt + "Yes:" + STR$(TreasureRecord.FuelCharges) + " charges" ' message
END IF ' end verify
CALL IO.O ' send output message
Outpt = "[I]Treasure is permanent: " ' message
IF TreasureRecord.Permanent THEN ' verify treasure
    Outpt = Outpt + "Yes" ' add to message
ELSE ' verify
    Outpt = Outpt + "No" ' add to message
END IF ' end verify
Outpt = LEFT$(Outpt, 40) ' truncate string
Outpt = Outpt + SPACE$(40 - LEN(Outpt)) ' pad with blanks
Outpt = Outpt + "[3]Weapon rusts: " ' message
IF TreasureRecord.Rustable THEN ' verify treasure
    Outpt = Outpt + "Yes:" + STR$(TreasureRecord.RustPercent) + "%"
ELSE ' verify
    Outpt = Outpt + "No" ' add to message
END IF ' end verify
CALL IO.O ' send output message
Outpt = "[J]Treasure is edible: " ' message
IF TreasureRecord.Edible THEN ' verify treasure
    Outpt = Outpt + "Yes" ' add to message
ELSE ' verify
    Outpt = Outpt + "No" ' add to message
END IF ' end verify
Outpt = LEFT$(Outpt, 40) ' truncate string
Outpt = Outpt + SPACE$(40 - LEN(Outpt)) ' pad with blanks
Outpt = Outpt + "[4]Weapon is stealable: " ' message
IF TreasureRecord.Stealable THEN ' verify treasure
    Outpt = Outpt + "Yes:" + STR$(TreasureRecord.StealPercent) + "%"
ELSE ' verify
    Outpt = Outpt + "No" ' add to message
END IF ' end verify
CALL IO.O ' send output message
Outpt = "[K]Treasure is magical: " ' message
Spell.Number = TreasureRecord.Spell ' store spell number
IF Spell.Number = False THEN ' verify treasure
    Outpt = Outpt + "No" ' add to message
ELSE ' verify
    ' file bounds
    IF Spell.Number > False AND Spell.Number <= LOF(SpellFile) / LEN(SpellRecord) THEN
        CALL Read.Record(SpellFile, Spell.Number) ' get spell record
        Outpt = Outpt + "Yes, " + RTRIM$(SpellRecord.SpellName) + " spell" ' message
    END IF ' end check file bounds
END IF ' end verify
CALL IO.O ' send output message
Outpt = "[L]Treasure is a ring: " ' message
SELECT CASE TreasureRecord.RingType
    CASE 1
        Outpt = Outpt + "protection from poison" ' message
    CASE 2
        Outpt = Outpt + "protection from level drain" ' message
    CASE 3
        Outpt = Outpt + "protection from spell: " ' message
        Spell.Number = TreasureRecord.RingSpell ' store ring spell number
        IF Spell.Number = True THEN ' verify treasure
            Outpt = Outpt + "generic spell" ' message
        ELSE ' verify
            ' file bounds
            IF Spell.Number > False AND Spell.Number <= LOF(SpellFile) / LEN(SpellRecord) THEN
                CALL Read.Record(SpellFile, Spell.Number) ' get spell record
                Outpt = Outpt + RTRIM$(LCASE$(SpellRecord.SpellName)) + " spell" ' message
            END IF ' end check file bounds
        END IF ' end verify
    CASE ELSE ' verify
        Outpt = Outpt + "No" ' add to message
END SELECT
CALL IO.O ' send output message
Outpt = "[M]Treasure is a light: " ' message
IF TreasureRecord.LightType = False THEN ' verify treasure
    Outpt = Outpt + "No" ' add to message
ELSE ' verify
    Outpt = Outpt + "Yes, Charges:" + STR$(TreasureRecord.LightCharges) ' message
END IF ' end verify
CALL IO.O ' send output message
Outpt = "[N]Treasure is a vehicle: " ' message
IF TreasureRecord.Vehicle = False THEN ' verify treasure
    Outpt = Outpt + "No" ' add to message
ELSE ' verify
    Outpt = Outpt + "Yes, hits:" + STR$(TreasureRecord.VehicleHits) ' message
END IF ' end verify
CALL IO.O ' send output message
Outpt = "[O]Treasure loads from devices: " ' message
IF TreasureRecord.Ammunition THEN ' verify treasure
    Outpt = Outpt + "Yes" ' add to message
ELSE ' verify
    Outpt = Outpt + "No" ' add to message
END IF ' end verify
CALL IO.O ' send output message
Outpt = "[P]Treasure is loadable: " ' message
IF TreasureRecord.Loadable THEN ' verify treasure
    Outpt = Outpt + "Yes: Loads(" ' add to message
    ' store ammunition treasure number
    Treasure.Number = TreasureRecord.AmmoLoads
    IF Treasure.Number > False AND Treasure.Number <= LOF(TreasureFile) / LEN(TreasureRecord) THEN ' file bounds
        CALL Share.Record(TreasureFile, Number) ' store current treasure record
        CALL Read.Record(TreasureFile, Treasure.Number) ' get treasure record
        Outpt = Outpt + RTRIM$(TreasureRecord.TreasureName) + ")" ' message
        CALL Read.Record(TreasureFile, Number) ' restore treasure record
    ELSE ' check file bounds
        Outpt = Outpt + "<none>)" ' append message
    END IF ' end check file bounds
ELSE ' verify
    Outpt = Outpt + "No" ' add to message
END IF ' end verify
CALL IO.O ' send output message
Outpt = "[R]Treasure launchs from devices: " ' message
IF TreasureRecord.LaunchAmmo THEN ' verify treasure
    Outpt = Outpt + "Yes" ' add to message
ELSE ' verify
    Outpt = Outpt + "No" ' add to message
END IF ' end verify
CALL IO.O ' send output message
Outpt = "[S]Treasure is launchable: " ' message
IF TreasureRecord.Launchable THEN ' verify treasure
    Outpt = Outpt + "Yes: Launchs(" ' add to message
    ' store ammunition treasure number
    Treasure.Number = TreasureRecord.LaunchLoads
    IF Treasure.Number > False AND Treasure.Number <= LOF(TreasureFile) / LEN(TreasureRecord) THEN ' file bounds
        CALL Share.Record(TreasureFile, Number) ' store current treasure record
        CALL Read.Record(TreasureFile, Treasure.Number) ' get treasure record
        Outpt = Outpt + RTRIM$(TreasureRecord.TreasureName) + ")" ' message
        CALL Read.Record(TreasureFile, Number) ' restore treasure record
    ELSE ' check file bounds
        Outpt = Outpt + "<none>)" ' append message
    END IF ' end check file bounds
ELSE ' verify
    Outpt = Outpt + "No" ' add to message
END IF ' end verify
CALL IO.O ' send output message
Outpt = "[T]Launchable device can be moved: " ' message
IF TreasureRecord.Movable THEN ' verify treasure
    Outpt = Outpt + "Yes" ' add to message
ELSE ' verify
    Outpt = Outpt + "No" ' add to message
END IF ' end verify
CALL IO.O ' send output message
END SUB ' end routine

REM * This routines displays information on the user file.
REM * input variables:
REM *   Number - the user file record.

SUB Display.User (Number)
IF Number <= False THEN ' check number
    EXIT SUB ' exit subroutine
END IF ' end check number
Graphics.Off = False ' reset color
Outpt = "User number" + STR$(Number) + "." ' user number message
CALL IO.O ' send output message
Graphics.Off = True ' reset color
Inpt = UserRecord.CodeName ' store player codename
CALL Decrypt(Inpt) ' decrypt codename
Inpt = RTRIM$(Inpt) ' trim codename
Inpt = LCASE$(Inpt) ' lowercase codename
MID$(Inpt, 1, 1) = UCASE$(MID$(Inpt, 1, 1)) ' uppercase first word
Outpt = "[A]Codename: " + Inpt + " " ' message
Inpt = STRING$(10, Mask$) ' mask passwword
Outpt = Outpt + "[B]Password: " + Inpt ' message
CALL IO.O ' send output message
Outpt = "[C]Level:" + STR$(UserRecord.Level) ' message
CALL IO.O ' send output message
Outpt = "[D]" ' set to option
Class.Number = UserRecord.ClassType ' store player class type
SELECT CASE Class.Number ' selection of class type
    CASE 1 TO 10 ' check class type
        Class.Type$ = Class.Name(Class.Number) ' store class name
    CASE ELSE ' check
        Class.Type$ = Class.Name(1) ' default class name
END SELECT ' end selection of class type
Outpt = Outpt + "Class: " + RTRIM$(Class.Type$) ' message
CALL IO.O ' send output message
Outpt = "[E]" ' set to option
Weapon.Number = UserRecord.Proficiency ' store player proficiency number
SELECT CASE Weapon.Number ' selection of proficiency
    CASE 1 TO 4 ' check type
        Weapon.Type$ = WeaponTypeName(Weapon.Number) ' store proficiency name
    CASE ELSE ' check
        Weapon.Type$ = WeaponTypeName(1) ' default name
END SELECT ' end seelction of proficiency
MID$(Weapon.Type$, 1, 1) = UCASE$(MID$(Weapon.Type$, 1, 1)) ' uppercase first letter
Outpt = Outpt + "Weapon proficiency: " + RTRIM$(Weapon.Type$) ' message
CALL IO.O ' send output message
Outpt = "[F]Blunt%:" + STR$(GetUserRecordWeapons(1)) + " " ' message
Outpt = Outpt + "[G]Pole%:" + STR$(GetUserRecordWeapons(2)) ' message
CALL IO.O ' send output message
Outpt = "[H]Sharp%:" + STR$(GetUserRecordWeapons(3)) + " " ' message
Outpt = Outpt + "[I]Thrusting%:" + STR$(GetUserRecordWeapons(4)) ' message
CALL IO.O ' send output message
Outpt = "[J]Classname: " ' message
Inpt = UserRecord.ClassName
CALL Decrypt(Inpt)
Inpt = RTRIM$(Inpt)
Outpt = Outpt + Inpt
CALL IO.O ' send output message
Outpt = "[K]Strength:" + STR$(GetUserRecordStats(1)) ' message
CALL IO.O ' send output message
Outpt = "[L]Intelligence:" + STR$(GetUserRecordStats(2)) ' message
CALL IO.O ' send output message
Outpt = "[M]Wisdom:" + STR$(GetUserRecordStats(3)) ' message
CALL IO.O ' send output message
Outpt = "[N]Dexterity:" + STR$(GetUserRecordStats(4)) ' message
CALL IO.O ' send output message
Outpt = "[O]Constitution:" + STR$(GetUserRecordStats(5)) ' message
CALL IO.O ' send output message
Outpt = "[P]Piety:" + STR$(GetUserRecordStats(6)) ' message
CALL IO.O ' send output message
Outpt = "[R]Charisma:" + STR$(GetUserRecordStats(7)) ' message
CALL IO.O ' send output message
Outpt = "[S]Experience: " + FormatString$(UserRecord.Experience) ' message
CALL IO.O ' send output message
Outpt = "[T]Gold: " + FormatString$(UserRecord.Gold) ' message
CALL IO.O ' send output message
Outpt = "[U]Room number:" + STR$(UserRecord.Room) ' message
CALL IO.O ' send output message
Outpt = "[V]Call restrictions." ' message
CALL IO.O ' send output message
Outpt = "[W]Inventory:" ' message
' count user inventory items
Number.Items = False ' reset counter
' loop through player objects
FOR Object.Index = 1 TO 20
    ' store object
    Object.Number = GetUserRecordObject(Object.Index)
    ' check object number
    IF Object.Number > False AND Object.Number <= LOF(ObjectFile) / LEN(ObjectRecord) THEN
        Number.Items = Number.Items + 1 ' increment number of items
    END IF ' end check object number
NEXT ' end count objects
' loop through inventory
FOR Treasure.Index = 1 TO 20
    ' store number
    Treasure.Number = GetUserRecordInv(Treasure.Index)
    ' check number
    IF Treasure.Number > False AND Treasure.Number <= LOF(TreasureFile) / LEN(TreasureRecord) THEN
        Number.Items = Number.Items + 1 ' increment number of items
    END IF ' end check object number
NEXT ' end count inventory
' loop through user containers
FOR Number.Containers = 1 TO 3 ' loop through user containers
    X = GetUserRecordContainer(Number.Containers) ' store user container into record
    IF RTRIM$(ContainerRec.ShortName) <> Nul THEN ' compare container name length
        Number.Items = Number.Items + 1 ' increment number of items
    END IF ' end check object number
NEXT ' end count inventory
IF Number.Items = 1 THEN ' check value
    Outpt = Outpt + STR$(Number.Items) + " item." ' make display message
ELSE ' check value
    Outpt = Outpt + STR$(Number.Items) + " items." ' make display message
END IF ' end check value
CALL IO.O ' send output message
Outpt = "[X]Special characters: " ' message
Special.Flag = False
IF UserRecord.Flags AND Special.Char1 THEN ' verify user
    Outpt = Outpt + "Town Mayor, " ' message
    Special.Flag = True
END IF ' end verify
IF UserRecord.Flags AND Special.Char2 THEN ' verify user
    Outpt = Outpt + "Governor, " ' message
    Special.Flag = True
END IF ' end verify
IF UserRecord.Flags AND Special.Char3 THEN ' verify user
    Outpt = Outpt + "Guild Master, " ' message
    Special.Flag = True
END IF ' end verify
IF UserRecord.Flags AND Special.Char4 THEN ' verify user
    Outpt = Outpt + "Sysop, " ' message
    Special.Flag = True
END IF ' end verify
IF Special.Flag THEN ' verify user
    Outpt = LEFT$(Outpt, LEN(Outpt) - 2) + "." ' message
ELSE ' verify
    Outpt = Outpt + "None." ' message
END IF ' end verify
CALL IO.O ' send output message
Outpt = "[Y]Training rooms" ' message
CALL IO.O ' send output message
Graphics.Off = False ' reset color
END SUB ' end routine

REM * routine used by the editor to find monster file number
REM * of a monster name.
REM * output variables:
REM *   Number - record number of monster.
REM * processing variables:
REM *   Inpt - monster name or number to search for.
REM *   Parsed.Value - monster number increment.

SUB Find.Monster (Number)
Outpt = "Monster name, or number" ' input prompt
Outpt = Outpt + "(1-" + MID$(STR$(LOF(MonsterFile) / LEN(MonsterRecord)), 2) + ")? "
CALL IO.I ' get input
IF Inpt = Nul THEN ' compare input length
    Number = False ' reset number found
    EXIT SUB ' exit routine
END IF ' end compare length
CALL Parse.Num(Inpt, Parsed.Value) ' parse number
Number.Count = False ' reset monster found counter
FOR Number = 1 TO LOF(MonsterFile) / LEN(MonsterRecord) ' loop through monsters
    CALL Read.Record(MonsterFile, Number) ' get monster record
    ' compare monster record name to input monster name,
    ' truncated to input length
    IF LEFT$(MonsterRecord.MonsterName, LEN(Inpt)) = Inpt THEN ' compare
        Number.Count = Number.Count + 1 ' increment monster found counter
        ' compare parsed number of monster to search for
        IF Parsed.Value = False OR Number.Count = Parsed.Value THEN ' compare
            EXIT SUB ' exit routine
        END IF ' end compare counters
    END IF ' end compare monster names
NEXT ' end loop through monster file
Number = INT(VAL(Inpt)) ' convert input to integer
IF Number <= False OR Number > LOF(MonsterFile) / LEN(MonsterRecord) THEN ' file bounds
    Outpt = Range$ ' make error message
    CALL IO.O ' send error message
    Number = False ' reset monster found
ELSE ' end check file bounds
    CALL Read.Record(MonsterFile, Number) ' get monster record number
END IF ' end check file bounds
END SUB ' end routine

REM * routine used by the editor to find nonplayer file number
REM * of a nonplayer name.
REM * output variables:
REM *   Number - record number of nonplayer.
REM * processing variables:
REM *   Inpt - nonplayer name or number to search for.
REM *   Parsed.Value - nonplayer number increment.

SUB Find.Nonplayer (Number)
Outpt = "Nonplayer name, or number" ' input prompt
Outpt = Outpt + "(1-" + MID$(STR$(LOF(NonPlayerFile) / LEN(MonsterRecord)), 2) + ")? "
CALL IO.I ' get input
IF Inpt = Nul THEN ' compare input length
    Number = False ' reset number found
    EXIT SUB ' exit routine
END IF ' end compare length
CALL Parse.Num(Inpt, Parsed.Value) ' parse number
Number.Count = False ' reset nonplayer counter
FOR Number = 1 TO LOF(NonPlayerFile) / LEN(MonsterRecord) ' loop through nonplayers
    CALL Read.Record(NonPlayerFile, Number) ' get nonplayer record
    ' compare nonplayer record name to input nonplayer name,
    ' truncated to input length
    IF LEFT$(MonsterRecord.MonsterName, LEN(Inpt)) = Inpt THEN ' compare
        Number.Count = Number.Count + 1 ' increment nonplayer found counter
        ' compare parsed number of object to search for
        IF Parsed.Value = False OR Number.Count = Parsed.Value THEN ' compare
            EXIT SUB ' exit routine
        END IF ' end compare counters
    END IF ' end compare nonplayer names
NEXT ' end loop through nonplayer file
Number = INT(VAL(Inpt)) ' convert input to integer
IF Number <= False OR Number > LOF(NonPlayerFile) / LEN(MonsterRecord) THEN ' file bounds
    Outpt = Range$ ' make error message
    CALL IO.O ' send error message
    Number = False ' reset nonplayer found
ELSE ' end check file bounds
    CALL Read.Record(NonPlayerFile, Number) ' get nonplayer record number
END IF ' end check file bounds
END SUB ' end routine

REM * routine used by the editor to find the object file number
REM * of an object name.
REM * output variables:
REM *   Number - record number of object.
REM * processing variables:
REM *   Inpt - object name or number to search for.
REM *   Parsed.Value - object number increment.

SUB Find.Objects (Number)
Outpt = "Object name, or number" ' make input prompt
Outpt = Outpt + "(1-" + MID$(STR$(LOF(ObjectFile) / LEN(ObjectRecord)), 2) + ")? "
CALL IO.I ' get input
IF Inpt = Nul THEN ' compare input length
    Number = False ' reset number found
    EXIT SUB ' exit routine
END IF ' end compare length
CALL Parse.Num(Inpt, Parsed.Value) ' get parsed number
Number.Count = False ' reset spell found counter
FOR Number = 1 TO LOF(ObjectFile) / LEN(ObjectRecord) ' loop through object file
    CALL Read.Record(ObjectFile, Number) ' get object record
    ' compare object record name to input object name,
    ' truncated to input length
    IF LEFT$(ObjectRecord.ObjectName, LEN(Inpt)) = Inpt THEN ' compare
        Number.Count = Number.Count + 1 ' increment object counter
        ' compare parsed number of object to search for
        IF Parsed.Value = False OR Number.Count = Parsed.Value THEN ' compare
            EXIT SUB ' exit routine
        END IF ' end compare counters
    END IF ' end compare object names
NEXT ' end loop through object file
Number = INT(VAL(Inpt)) ' convert input to integer
IF Number <= False OR Number > LOF(ObjectFile) / LEN(ObjectRecord) THEN ' file bounds
    Outpt = Range$ ' make error message
    CALL IO.O ' send error message
    Number = False ' reset object number to zero
ELSE ' end check file bounds
    CALL Read.Record(ObjectFile, Number) ' get object record number
END IF ' end check file bounds
END SUB ' end routine

REM * routine used by the editor to find the spell file number
REM * of a spell name.
REM * output variables:
REM *   Number - record number of spell.
REM * processing variables:
REM *   Inpt - spell name or number to search for.
REM *   Parsed.Value - spell number increment.

SUB Find.Spell (Number)
Outpt = "Spell name, or number" ' input prompt
Outpt = Outpt + "(1-" + MID$(STR$(LOF(SpellFile) / LEN(SpellRecord)), 2) + ")? "
CALL IO.I ' get input
IF Inpt = Nul THEN ' check length of input
    Number = False ' reset number found
    EXIT SUB ' exit routine
END IF ' end check length
CALL Parse.Num(Inpt, Parsed.Value) ' parse out number
Number.Count = False ' reset spell found counter
FOR Number = 1 TO LOF(SpellFile) / LEN(SpellRecord) ' loop through spell file
    CALL Read.Record(SpellFile, Number) ' get spell record
    ' compare spell record name to input spell name, truncated to input length
    IF LEFT$(SpellRecord.SpellName, LEN(Inpt)) = Inpt THEN ' compare
        Number.Count = Number.Count + 1 ' increment spell found
        ' compare parsed number of spell to search for
        IF Parsed.Value = False OR Number.Count = Parsed.Value THEN ' compare
            EXIT SUB ' exit routine
        END IF ' end compare counters
    END IF ' end compare spell names
NEXT ' end loop through spell file
Number = INT(VAL(Inpt)) ' convert input to integer
IF Number <= False OR Number > LOF(SpellFile) / LEN(SpellRecord) THEN ' compare file bounds
    Outpt = Range$ ' make error message
    CALL IO.O ' send error message
    Number = False ' reset return spell number to zero
ELSE ' end compare spell number
    CALL Read.Record(SpellFile, Number) ' get spell record number
END IF ' end check file bounds
END SUB ' end routine

REM * routine used by the editor to find the treasure file number
REM * of a treasure name.
REM * output variables:
REM *   Number - record number of treasure.
REM * processing variables:
REM *   Inpt - treasure name or number to search for.
REM *   Parsed.Value - treasure number increment.

SUB Find.Treasure (Number)
Outpt = "Treasure name, or number" ' input prompt
Outpt = Outpt + "(1-" + MID$(STR$(LOF(TreasureFile) / LEN(TreasureRecord)), 2) + ")? "
CALL IO.I ' get input
IF Inpt = Nul THEN ' compare input length
    Number = False ' reset number found
    EXIT SUB ' exit routine
END IF ' end compare length
CALL Parse.Num(Inpt, Parsed.Value) ' parse number
Number.Count = False ' reset treasure found counter
FOR Number = 1 TO LOF(TreasureFile) / LEN(TreasureRecord) ' loop through treasure
    CALL Read.Record(TreasureFile, Number) ' get treasure record
    ' compare treasure record name to input treasure name,
    ' truncated to input length
    IF LEFT$(TreasureRecord.TreasureName, LEN(Inpt)) = Inpt THEN ' compare
        Number.Count = Number.Count + 1 ' increment treasure found counter
        ' compare parsed number of treasure to search for
        IF Parsed.Value = False OR Number.Count = Parsed.Value THEN ' compare
            EXIT SUB ' exit routine
        END IF ' end compare counters
    END IF ' end compare treasure names
NEXT ' end loop through treasure file
Number = INT(VAL(Inpt)) ' convert input to integer
IF Number <= False OR Number > LOF(TreasureFile) / LEN(TreasureRecord) THEN ' file bounds
    Outpt = Range$ ' make error message
    CALL IO.O ' send error message
    Number = False ' reset treasure found
ELSE ' end check file bounds
    CALL Read.Record(TreasureFile, Number) ' get treasure record number
END IF ' end check file bounds
END SUB ' end routine

REM * routine for player to hold an item of inventory.

SUB Hold.Object
CALL Check.Inventory.Treasure ' check player inventory treasure mneomnic
IF Index.Number = False THEN ' found treasure file index
    Outpt = "You aren't carrying that!" ' make message
    CALL IO.O ' send message
    EXIT SUB ' exit routine
END IF ' end compare player inventory
IF TreasureRecord.Type > False THEN ' compare treasure type
    Outpt = "You can't hold that!" ' make message
    CALL IO.O ' send message
    EXIT SUB ' exit routine
END IF ' end compare treasure type
IF TreasureRecord.Loadable THEN ' verify treasure type
    Outpt = "You can't hold that!" ' make output message
    CALL IO.O ' send message
    EXIT SUB ' exit routine
END IF ' end verify treasure type
IF TreasureRecord.Launchable THEN ' verify treasure type
    Outpt = "You can't hold that!" ' make output message
    CALL IO.O ' send message
    EXIT SUB ' exit routine
END IF ' end verify treasure type
' compute treasure charges remaining
IF GetUserRecordCharges(Array.Number) = False THEN
    Outpt = "You can't, it has zero charges!" ' make message
    CALL IO.O ' send message
    EXIT SUB ' exit routine
END IF ' end compare treasure charges
Item.Plus = ABS(TreasureRecord.Plus)
SELECT CASE TreasureRecord.Type ' select treasure type
    CASE IS < False ' compare treasure type to shield
        Weapon3 = Item.Plus ' set variable of holding shield plus
        ' set variable of holding shield of inventory array index
        Weapon5 = Array.Number
    CASE False ' compare treasure type to weapon
        IF Item.Plus = False THEN ' verify weapon type
            Outpt = "You can't hold that!" ' make output message
            CALL IO.O ' send message
            EXIT SUB ' exit routine
        END IF ' end verify weapon type
        Index.Number = TreasureRecord.Proficiency ' store treasure proficiency
        IF UserRecord.ClassType = Cleric THEN ' compare player classtype to cleric
            SELECT CASE Index.Number ' select treasure proficiency type
                CASE 3 TO 4 ' compare to sharp or thrusting
                    Outpt = "Clerics may only use blunt or pole type weapons!" ' message
                    CALL IO.O ' send message
                    EXIT SUB ' exit routine
            END SELECT ' end compare weapon proficiency type
        END IF ' end compare to cleric
        Weapon2 = Item.Plus ' set variable of holding weapon plus
        ' set variable of holding weapon of inventory array index
        Weapon6 = Array.Number
        Weapon10 = Index.Number ' set variable of holding weapon proficiency type
END SELECT ' end select treasure type
Outpt = "You hold " + Outpts + "(+" + MID$(STR$(Item.Plus), 2) + ")!" ' format message
CALL IO.O ' send message
END SUB ' end routine to hold an item

REM * routine for player to wear an item of inventory.

SUB Wear.Object
CALL Check.Inventory.Treasure ' check player inventory treasure mnemonic
IF Index.Number = False THEN ' found treasure index
    Outpt = "You aren't carrying that!" ' make message
    CALL IO.O ' send message
    EXIT SUB ' exit routine
END IF ' end compare index
IF TreasureRecord.Type <= False THEN ' commpare treasure type
    IF TreasureRecord.RingType = False THEN ' compare ring type
        Outpt = "You can't wear that!" ' make message
        CALL IO.O ' send message
        EXIT SUB ' exit routine
    END IF ' end compare ring
END IF ' end compare treasure type
IF GetUserRecordCharges(Array.Number) = False THEN ' compare player charges array
    Outpt = "You can't, it has zero charges!" ' make message
    CALL IO.O ' send message
    EXIT SUB ' exit routine
END IF ' end compare charges
Item.Plus = ABS(TreasureRecord.Plus) ' store treasure item plus hits
SELECT CASE TreasureRecord.Type ' selecct treasure type
    CASE IS > False ' treasure type is armor
        Weapon1 = Item.Plus ' set variable of wearing item to plus
        ' set variable of wearing item to inventory array index
        Weapon4 = Array.Number
    CASE ELSE ' treasure type is ring
        Weapon7 = Array.Number ' set wearing ring item to inventory array index
        Weapon8 = TreasureRecord.RingType ' set wearing ring type variable
        Weapon9 = TreasureRecord.RingSpell ' set wearing ring spell type variable
        Weapon11 = Item.Plus ' set wearing ring plus
END SELECT ' end select treasure type
Outpt = "You wear " + Outpts + "(+" + MID$(STR$(Item.Plus), 2) + ")!" ' format message
CALL IO.O ' send message
END SUB ' end routine to wear an item

REM * routine to return an item from being held or worn.

SUB Return.Object
CALL Check.Inventory.Treasure ' check player inventory mnemonic
IF Index.Number = False THEN ' treasure index of item
    Outpt = "You aren't carrying that!" ' make message
    CALL IO.O ' send message
    EXIT SUB ' exit routine
END IF ' end compare treasure inventory
Outpt = "You return " + Outpts + "!" ' format message
SELECT CASE Array.Number ' select treasure inventory array variable
    CASE Weapon4 ' compare treasure inventory array to armor variable
        Weapon1 = False ' reset armor being worn holding variable
        Weapon4 = False ' reset armor being worn variable
        CALL IO.O ' send message
    CASE Weapon5 ' compare treasure inventory array to shield variable
        Weapon3 = False ' reset shield holding variable
        Weapon5 = False ' reset shield holding variable
        CALL IO.O ' send message
    CASE Weapon6 ' compare treasure inventory array to weapon variable
        Weapon2 = False ' reset weapon holding variable
        Weapon6 = False ' reset weapon holding variable
        Weapon10 = False ' reset weapon holding variable
        CALL IO.O ' send message
    CASE Weapon7 ' compare treasure inventory array to ring variable
        Weapon7 = False ' reset ring being worn variable
        Weapon8 = False ' reset ring being worn variable
        Weapon9 = False ' reset ring being worn variable
        Weapon11 = False ' reset ring worn variable
        CALL IO.O ' send message
    CASE ELSE ' no matching variables being held or worn
        Outpt = "Your aren't wearing that!" ' make message
        CALL IO.O ' send message
END SELECT ' end select treasure array variable
END SUB ' end routine to return item

REM * routine to reset an item with original charges if broken.

SUB Fix.Object
CALL Examine.Treasure ' check treasure mnemonic
IF Index.Number = False THEN ' treasure index
    Outpt = "You can't fix that!" ' make message
    CALL IO.O ' send message
    EXIT SUB ' exit routine
END IF ' end check treasure
IF TreasureRecord.Spell THEN ' treasure has spell
    Outpt = "You can't fix that!" ' make message
    CALL IO.O ' send message
    EXIT SUB ' exit routine
END IF ' end compare spell
IF TreasureRecord.Plus = False THEN ' treasure is not weapon, armor, shield
    Outpt = "You can't fix that!" ' make message
    CALL IO.O ' send message
    EXIT SUB ' exit routine
END IF ' end compare spell
IF GetUserRecordCharges(Array.Number) THEN ' check remaining treasure charges
    Outpt = "That's not broken!" ' make message
    CALL IO.O ' send message
    EXIT SUB ' exit routine
END IF ' end compare charges
IF GetUserRecordStats(3) < 7 THEN ' compute random chance
    Outpt = "You didn't fix it!" ' make message
    CALL IO.O ' send message
    EXIT SUB ' exit routine
END IF ' end random chance
IF ABS(TreasureRecord.Plus) > 10 THEN ' compare treasure plus
    Outpt = "You didn't fix it!" ' make message
    CALL IO.O ' send message
    EXIT SUB ' exit routine
END IF ' end compare plus
IF RND < .5 THEN ' random chance
    Outpt = "You didn't fix it!" ' make message
    CALL IO.O ' send message
    EXIT SUB ' exit routine
END IF ' end chance
Item.Charges = TreasureRecord.Charges ' store treasure maximum charges
IF Item.Charges > 250 THEN ' only repair item to maximum of 250 charges
    Item.Charges = 250 ' set storage variable to 250 charges
END IF ' end compare charges
SELECT CASE TypeNumber ' select room or inventory
    CASE 1 ' treasure fixed in room
        ' reset room treasure charges
        CALL PutRoomRecordTreCharges(Array.Number, Item.Charges)
        CALL Share.Room.Record(Room) ' write room record
    CASE 0 ' treasure fixed in inventory
        ' reset inventory treasure charges
        CALL PutUserRecordCharges(Array.Number, Item.Charges)
END SELECT ' end treasure fixed charges
Outpt = "You fix it!" ' make message
CALL IO.O ' send message
END SUB ' end fix routine

REM * routine to drink an item.

SUB Drink.Potion
CALL Check.Inventory.Treasure ' check treasure mnemonic
IF Index.Number = False THEN ' inventory treasure index
    Outpt = "You can't drink that!" ' make message
    CALL IO.O ' send message
    EXIT SUB ' exit routine
END IF ' end treasure index
IF TreasureRecord.Potion = False THEN ' compare treasure to potion
    Outpt = "That's not a potion!" ' make message
    CALL IO.O ' send message
    EXIT SUB ' exit routine
END IF ' end compare potion
' check inventory charges remaining
IF GetUserRecordCharges(Array.Number) = False THEN
    Outpt = "You can't, it's empty!" ' make message
    CALL IO.O ' send message
    EXIT SUB ' exit routine
END IF ' end compare charges
Spell.Number = TreasureRecord.Spell ' get treasure spell number
' check spell file
IF Spell.Number > False AND Spell.Number <= LOF(SpellFile) / LEN(SpellRecord) THEN
    CALL Read.Record(SpellFile, Spell.Number) ' get spell record
    ' reduce charges
    CALL PutUserRecordCharges(Array.Number, GetUserRecordCharges(Array.Number) - 1)
    Magic.Spell = SpellRecord.SpellType ' store spell type
    Multiplier = SpellRecord.Level ' store spell level
    Monster.Number = False ' monster not being attacked
    Parser = False ' no extra parameters
    Index.Number = False ' not cast on an object
    Outpt = "You drink " + Outpts + "!" ' format message
    CALL IO.O ' send message
    CALL Magic(True) ' cast the spell
    EXIT SUB ' exit routine
END IF ' end check spell file
Outpt = "You can't drink that!" ' make message
CALL IO.O ' send message
END SUB ' end drink potion

REM * routine to eat an item.

SUB Eat.Object
CALL Check.Inventory.Treasure ' check treasure mnemonic
IF Index.Number = False THEN ' inventory treasure index
    Outpt = "You can't eat that!" ' make message
    CALL IO.O ' send message
    EXIT SUB ' exit routine
END IF ' end treasure index
IF TreasureRecord.Edible = False THEN ' compare treasure to food
    Outpt = "That's not edible!" ' make message
    CALL IO.O ' send message
    EXIT SUB ' exit routine
END IF ' end compare food
' check inventory charges remaining
IF GetUserRecordCharges(Array.Number) = False THEN
    Outpt = "You can't, it's been eaten!" ' make message
    CALL IO.O ' send message
    EXIT SUB ' exit routine
END IF ' end compare charges
Spell.Number = TreasureRecord.Spell ' get treasure spell number
' check spell file
IF Spell.Number > False AND Spell.Number <= LOF(SpellFile) / LEN(SpellRecord) THEN
    CALL Read.Record(SpellFile, Spell.Number) ' get spell record
    ' reduce charges
    CALL PutUserRecordCharges(Array.Number, GetUserRecordCharges(Array.Number) - 1)
    Magic.Spell = SpellRecord.SpellType ' store spell type
    Multiplier = SpellRecord.Level ' store spell level
    Monster.Number = False ' monster not being attacked
    Parser = False ' no extra parameters
    Index.Number = False ' not cast on a target
    Outpt = "You eat " + Outpts + "!" ' format message
    CALL IO.O ' send message
    CALL Magic(True) ' cast the spell
    EXIT SUB ' exit routine
END IF ' end check spell file
Outpt = "You can't eat that!" ' make message
CALL IO.O ' send message
END SUB ' end eat routine

REM * routine to load device with charges from an item, loads device on
REM * ground or player inventory, removes item from from room or inventory.

SUB Load.Object
CALL Examine.Treasure ' check treasure mnemonic
IF Index.Number = False THEN ' found treasure index
    Outpt = "You can't load that!" ' make message
    CALL IO.O ' send message
    EXIT SUB ' exit routine
END IF ' end treasure index
Load.Type = False ' set load type flag
IF TreasureRecord.Loadable = True THEN ' treasure can be loaded
    ' store treasure index which loads device
    Device.Type = TreasureRecord.AmmoLoads
    Load.Type = True ' set load type flag
END IF ' end compare loaded treasure
IF TreasureRecord.Launchable = True THEN ' treasure can be launched
    ' store treasure index which launchs item
    Device.Type = TreasureRecord.LaunchLoads
    Load.Type = 1 ' set load type flag
END IF ' end compare launched treasure
IF Load.Type = False THEN ' compare load type flag
    Outpt = "You can't load that!" ' make message
    CALL IO.O ' send message
    EXIT SUB ' exit routine
END IF ' end compare load type
' check index bounds
IF Device.Type < False OR Device.Type > LOF(TreasureFile) / LEN(TreasureRecord) THEN
    Outpt = "There's no ammunition!" ' make message
    CALL IO.O ' send message
    EXIT SUB ' exit routine
END IF ' end check index bounds
Charges.Number = False ' set treasure ammunition flag
CALL Read.Record(TreasureFile, Device.Type) ' get treasure index of ammunition
IF Load.Type = True THEN ' check load type
    IF TreasureRecord.Ammunition THEN ' treasure index ammunition found
        Charges.Number = True ' set treasure ammunition flag
    END IF ' end compare ammunition index
END IF ' end load type
IF Load.Type = 1 THEN ' check load type
    IF TreasureRecord.LaunchAmmo THEN ' treasure index ammunition found
        Charges.Number = True ' set treasure ammunition flag
    END IF ' end compare ammunition index
END IF ' end load type
IF Charges.Number = False THEN ' check ammunition flag
    Outpt = "There's no ammunition!" ' make message
    CALL IO.O ' send message
    EXIT SUB ' exit routine
END IF ' end check ammunition
Ammo.Used = False ' set ammunition used flag
FOR Inventory.Number = 1 TO 20 ' loop through all player inventory
    ' compare inventory index to ammo index
    IF Device.Type = GetUserRecordInv(Inventory.Number) THEN
        ' remove ammo from inventory
        CALL Discard.Inventory(Inventory.Number, True)
        Ammo.Used = True ' set ammunition used flag
        EXIT FOR ' exit loop to find ammo index
    END IF ' end compare inventory index
NEXT ' end loop to find ammo index
IF Ammo.Used = False THEN ' check ammunition used flag
    FOR Inventory.Number = 1 TO 20 ' loop through all room inventory
        ' compare room inventory to
        IF Device.Type = GetRoomRecordTreasure(Inventory.Number) THEN
            ' ammo index, remove room ammo
            CALL Discard.Room.Treasure(Inventory.Number)
            Ammo.Used = True ' set ammunition used flag
            EXIT FOR ' exit loop to find ammo index
        END IF ' end compare inventory index
    NEXT ' end loop to find ammo index
END IF ' end ammunition used flag
IF Ammo.Used = False THEN ' check ammunition used flag
    Outpt = "There's no ammunition!" ' make message
    CALL IO.O ' send message
    EXIT SUB ' exit routine
END IF ' end check ammunition used flag
Device.Loaded = False ' set ammunition loaded flag
SELECT CASE TypeNumber ' select room or inventory
    CASE 0 ' treasure loaded in player inventory
        ' store increment charges
        Device.Charges = GetUserRecordCharges(Array.Number) + 1
        IF Device.Charges > 10 THEN ' compare to maximum charges
            Device.Loaded = True ' set ammunition loaded flag
        ELSE ' compare less than maximum
            CALL PutUserRecordCharges(Array.Number, Device.Charges) ' increment charges
        END IF ' end compare maximum charges
    CASE 1 ' treasure loaded in room inventory
        ' store increment charges
        Device.Charges = GetRoomRecordTreCharges(Array.Number) + 1
        IF Device.Charges > 10 THEN ' compare to maximum charges
            Device.Loaded = True ' set ammunition loaded flag
        ELSE ' compare less than maximum
            CALL PutRoomRecordTreCharges(Array.Number, Device.Charges) ' increment charges
        END IF ' end compare maximum charges
        CALL Share.Room.Record(Room) ' write room record
END SELECT ' end compare inventory to load
IF Device.Loaded THEN ' compare ammunition loaded flag
    Outpt = "The device can't load that much!" ' make message
ELSE ' ammunition loaded
    Outpt = "You load the device!" ' make message
END IF ' end compare ammunition loaded flag
CALL IO.O ' send message
END SUB ' end routine to load ammunition to an item

REM * routine to fire a device at a monster in room, device explodes in
REM * player by chance, device discharges harmlessly.

SUB Fire.Object
CALL Find.Inventory ' check player inventory for mnemonic of device to fire
IF Index.Number = False THEN ' found treasure index to player inventory
    Outpt = "You can't fire that!" ' make message
    CALL IO.O ' send message
    EXIT SUB ' exit routine
END IF ' end check treasure index
IF TreasureRecord.Loadable = False THEN ' check device can be loaded
    Outpt = "You can't fire that!" ' make message
    CALL IO.O ' send message
    EXIT SUB ' exit routine
END IF ' end compare device
IF TreasureRecord.AmmoLoads = False THEN ' check item which loads device
    Outpt = "It's not loaded with ammunition!" ' make message
    CALL IO.O ' send message
    EXIT SUB ' exit routine
END IF ' end compare device
CALL Read.Record(TreasureFile, TreasureRecord.AmmoLoads) ' get record ammo used
IF TreasureRecord.Ammunition = False THEN ' item which loads device is nto ammo
    Outpt = "It's not loaded with ammunition!" ' make message
    CALL IO.O ' send message
    EXIT SUB ' exit routine
END IF ' end compare device
' read treasure record index of device to fire
CALL Read.Record(TreasureFile, Index.Number)
IF Parser = False THEN ' check second parameter of monster to fire at
    Outpt = "Fire at what?" ' make message
    CALL IO.O ' send message
    EXIT SUB ' exit routine
END IF ' end compare monster
' compare remaining charges in device
IF GetUserRecordCharges(Array.Number) = False THEN
    Outpt = "It's not loaded!" ' make message
    CALL IO.O ' send message
    EXIT SUB ' exit routine
END IF ' end compare device
' decrement charges
CALL PutUserRecordCharges(Array.Number, GetUserRecordCharges(Array.Number) - 1)
Parsed.Command1 = Parsed.Command2 ' get second parameter of monster
CALL Numeric ' parse out temp variable
IF RND < .2 THEN ' compute random chance
    Outpt = "Bang! The ammunition explodes in your face!" ' device backfires
    CALL IO.O ' send message
    Outpt = "The device hits you for" ' string preceding points hit on player
    Hits# = CDBL(INT(RND * ABS(TreasureRecord.Plus) + 1)) ' calculate random hits
    CALL Hit.Player(Hits#) ' routine to get hits on player
    EXIT SUB ' exit routine
END IF ' end random chance
IF RND < .2 THEN ' random chance
    Outpt = "The device fires harmlessly!" ' make message
    CALL IO.O ' send message
    EXIT SUB ' exit routine
END IF ' end random chance
Treasure.Name$ = TreasureRecord.ShortName ' store treasure name being fired
Treasure.Name$ = RTRIM$(Treasure.Name$) ' strip trailing blanks
Treasure.Name$ = LCASE$(Treasure.Name$) ' format to lower case
CALL Check.Monster ' find monster being fired at in parsed string
IF Monster.Number THEN ' monster found from second parameter
    Outpts = MonsterArray(Monster.Number).MonsterName ' store monster name
    Outpts = RTRIM$(Outpts) ' strip trailing blanks
    Outpts = LCASE$(Outpts) ' format to lower case
    Outpt = "You fire the " + Treasure.Name$ + " at the " + Outpts + "!" ' format message
    CALL IO.O ' send message
    Multiplier = ABS(TreasureRecord.Plus) ' calculate hits on monster multiplier
    CALL Hit.Monster ' routine to attack monster in Monster.Number
    EXIT SUB ' exit routine
END IF ' end compare monster to fire at
Outpt = "The device fires into empty air!" ' make message
CALL IO.O ' send message
END SUB ' end routine to fire a device at a monster

REM * dm/sysop routine to get an item from treasure or object files and
REM * add to room, interactive function during remote logon also.

SUB Drop.Object
Parsed.Command1 = Stored.Parsed.Command2 ' get entire second half of command
CALL Numeric ' parse temp variable
Inpt = Parsed.Command1 ' put full name of item in variable
Inpt = LCASE$(Inpt) ' convert to lower case
CALL Drop(True) ' routine to put item name in room
END SUB ' end dm/sysop drop routine

REM * routine to bless an object by increasing the magical charges.

SUB Bless.Object
IF Normal.User THEN ' check player is normal player
    IF UserRecord.ClassType <> Cleric THEN ' compare player class type to cleric
        Outpt = "Only clerics can bless!" ' make message
        CALL IO.O ' send message
        EXIT SUB ' exit routine
    END IF ' end compare class type
END IF ' end compare player type
CALL Examine.Treasure ' check treasure mnemonic
IF Index.Number THEN ' found treasure index
    Charges.Number = TreasureRecord.Spell ' store treasure spell type
    IF Charges.Number = False THEN ' compare treasure spell type
        Outpt = "You can't bless that!" ' make message
        CALL IO.O ' send message
        EXIT SUB ' exit routine
    END IF ' end compare treasure type
    ' check file bounds
    IF Charges.Number > False AND Charges.Number < LOF(SpellFile) / LEN(SpellRecord) THEN
        ' read spell record from treasure index
        CALL Read.Record(SpellFile, Charges.Number)
        IF SpellRecord.SpellType = 4 THEN ' compare spell to wish
            Outpt = "You can't bless that!" ' make message
            CALL IO.O ' send message
            EXIT SUB ' exit routine
        END IF ' and compare spell
        Object.Blessed = False ' set object blessed flag
        Max.Charges = TreasureRecord.Charges ' store maximum charges
        SELECT CASE TypeNumber ' select room or inventory
            CASE 0 ' bless item in player inventory
                ' get incremented charges
                TypeCharges = GetUserRecordCharges(Array.Number) + 1
                IF TypeCharges > Max.Charges THEN ' compare maximum charges
                    Object.Blessed = True ' set bless flag
                ELSE ' compare charges
                    ' set incremented charges
                    CALL PutUserRecordCharges(Array.Number, TypeCharges)
                END IF ' end compare maximum charges
            CASE 1 ' bless item in room inventory
                ' get incremented charges
                TypeCharges = GetRoomRecordTreCharges(Array.Number) + 1
                IF TypeCharges > Max.Charges THEN ' compare maximum charges
                    Object.Blessed = True ' set bless flag
                ELSE ' compare charges
                    ' set incremented charges
                    CALL PutRoomRecordTreCharges(Array.Number, TypeCharges)
                    CALL Share.Room.Record(Room) ' write room record
                END IF ' end compare maximum charges
        END SELECT ' end inventory type
        IF Object.Blessed THEN ' compare bless flag
            Outpt = "Nothing happens!" ' make message
        ELSE ' compare bless flag
            Outpt = "You bless it!" ' make message
        END IF ' end compare flag
        CALL IO.O ' send message
        EXIT SUB ' exit routine
    END IF
END IF
Outpt = "You can't bless that!" ' make message
CALL IO.O ' send message
END SUB ' end routine to bless an object

REM * routine to curse a monster by decreasing hits.

SUB Curse.Object
IF Normal.User THEN ' check player type
    IF UserRecord.ClassType <> Cleric THEN ' check player class type to cleric
        Outpt = "Only clerics can curse!" ' make message
        CALL IO.O ' send message
        EXIT SUB ' exit routine
    END IF ' end compare class type
END IF ' end compare player type
CALL Check.Monster ' search for monster parameter
IF Monster.Number THEN ' index found to monster array
    Outpts = MonsterArray(Monster.Number).MonsterName ' store full monster name
    Outpts = RTRIM$(Outpts) ' strip trailing blanks
    Outpts = LCASE$(Outpts) ' convert to lower case
    Outpt = "You curse the " + Outpts + "!" ' format message
    CALL IO.O ' send message
    Multiplier = INT(GetUserRecordStats(6) / 2) ' compute hits on monster multiplier
    CALL Hit.Monster ' routine to attack a monster
    EXIT SUB ' exit routine
END IF ' end get monster index
Outpt = "You can't curse that!" ' make message
CALL IO.O ' send message
END SUB ' end routine to curse a monster

REM * routine to bless player (bless without parameter).

SUB Bless.Self
IF Normal.User THEN ' check player type
    IF UserRecord.ClassType <> Cleric THEN ' check player class type
        Outpt = "Only clerics can bless!" ' message for clerics only
        CALL IO.O ' send message
        EXIT SUB ' exit routine
    END IF ' end compare class type
END IF ' end compare player type
TypeCharges = GetUserRecordStats(6) + 1 ' store incremented piety
IF TypeCharges > MaxStat THEN ' compare to maximum piety
    Outpt = "Nothing happens!" ' make message
    CALL IO.O ' send message
    EXIT SUB ' exit routine
END IF ' end compare piety
CALL PutUserRecordStats(6, TypeCharges) ' update new piety
Outpt = "You feel a glow about you!" ' make message
CALL IO.O ' send message
Outpt = "Your piety is now" + STR$(TypeCharges) + "!" ' format message
CALL IO.O ' send message
END SUB ' end routine to bless self

REM * routine to curse player (curse without parameter).

SUB Curse.Self
IF Normal.User THEN ' check player type
    IF UserRecord.ClassType <> Cleric THEN ' check player class type
        Outpt = "Only clerics can curse!" ' make message
        CALL IO.O ' send message
        EXIT SUB ' exit routine
    END IF ' end compare class type
END IF ' end compare player type
TypeCharges = GetUserRecordStats(6) - 1 ' store decremented piety
IF TypeCharges <= False THEN ' compare to minimum piety
    Outpt = "Nothing happens!" ' make message
    CALL IO.O ' send message
    EXIT SUB ' exit routine
END IF ' end compare piety
CALL PutUserRecordStats(6, TypeCharges) ' set new piety
Outpt = "You feel a darkening about you!" ' make message
CALL IO.O ' send message
Outpt = "Your piety is now" + STR$(TypeCharges) + "!" ' format message
CALL IO.O ' send message
END SUB ' end routine to curse self

REM * routine to move a lanuch type object to a nearby room and move player
REM * to room also, verify room traps.

SUB Move.Object
CALL Parse ' parse out parameters
IF Parser THEN ' check for first parameter
    CALL Numeric '  parse temp variable from first parameter
END IF ' end check parameter
CALL Examine.Treasure ' check treasure mnemonic in inventory or room
IF Index.Number = False THEN ' locate treasure index
    Outpt = "You can't move that!" ' make message
    CALL IO.O ' send message
    EXIT SUB ' exit routine
END IF ' end compare treasure
IF TreasureRecord.Movable = False THEN ' compare treasure can be moved
    Outpt = "You can't move that!" ' make message
    CALL IO.O ' send message
    EXIT SUB ' exit routine
END IF ' end compare treasure
IF TreasureRecord.Launchable = False THEN ' compare treasure can be launched
    Outpt = "You can't move that!" ' make message
    CALL IO.O ' send message
    EXIT SUB ' exit routine
END IF ' end compare treasure
IF Parser = False THEN ' compare parameter parsed
    Outpt = "Move which direction?" ' no direction specified
    CALL IO.O ' send message
    EXIT SUB ' exit routine
END IF ' end check parse
Treasure.Name$ = Outpts ' store full treasure name to be moved
Item.Number = Array.Number ' store treasure index to array
Item.Charges = Charges.Number ' store treasure charges
Item.Index = Index.Number ' store treasure index to file
User.Command = Parsed.Command2 ' get second parameter
CALL Get.Direction(Direction.Number) ' compare moving parameter to direction
IF Direction.Number THEN ' direction found flag set
    Next.Room = GetRoomRecordDirect(Direction.Number) ' get room number direction
    Outpts = RTRIM$(Direction(Direction.Number)) ' store direction name
    IF Next.Room = False THEN ' compare room number
        Outpt = "You can't move it there!" ' make message
        CALL IO.O ' send message
        EXIT SUB ' exit routine
    END IF ' end compare room number
ELSE ' no direction flag set, check for moving through an object
    Parsed.Command1 = User.Command ' get second parameter
    CALL Check.Room.Objects ' check with mnemonic name of object in room
    IF Index.Number = False THEN ' check object index
        Outpt = "You can't move it there!" ' make message
        CALL IO.O ' send message
        EXIT SUB ' exit routine
    END IF ' end compare room object
    IF ObjectRecord.RoomLink = False THEN ' verify room object links to room
        Outpt = "You can't move it there!" ' make message
        CALL IO.O ' send message
        EXIT SUB ' exit routine
    END IF ' end compare object is a portal
    IF ObjectRecord.JailTrap THEN ' verify room is trapped
        Outpt = "Trapped portal!" ' make message
        CALL IO.O ' send message
        EXIT SUB ' exit routine
    END IF ' end compare room trap
    IF ObjectRecord.Closed THEN ' compare room closed
        IF Pass.Door = False THEN ' check is passdoor spell cast
            Outpt = "You can't, it's closed!" ' make message
            CALL IO.O ' send message
            EXIT SUB ' exit routine
        END IF ' end cmpare passdoor
    END IF ' end compare object closed
    IF ObjectRecord.Relocks THEN ' object portal closes after entry
        ObjectRecord.DoorLock = 2 ' set object portal lock
        ObjectRecord.Closed = True ' set object portal closed
        CALL Share.Record(ObjectFile, Index.Number) ' write object record
    END IF ' end compare relocking object portal
    Pass.Door = False ' reset passdoor spell
    Next.Room = ObjectRecord.RoomLink ' get room number to enter portal
    Outpts = "into a " + RTRIM$(LCASE$(ObjectRecord.ShortName)) ' store object name
END IF ' end compare move to a direction or to a portal
SWAP Room, Next.Room ' interchange current room with next room to move item
CALL Read.Room.Record(Room) ' read next room to move item to room file
' routine to add room item
CALL Add.Room.Treasure(Item.Index, Item.Charges, False, Item.Added)
SWAP Room, Next.Room ' interchange current room with next room
CALL Read.Room.Record(Room) ' read current room record
IF Item.Added THEN ' add item to room flag
    SELECT CASE TypeNumber ' select room or inventory
        CASE 0 ' treasure moved from player inventory
            CALL Discard.Inventory(Item.Number, True) ' remove item from inventory
        CASE 1 ' treasure moved from room
            CALL Discard.Room.Treasure(Item.Number) ' remove item from room
    END SELECT ' end compare treasure moved
    Outpt = "You move " + Treasure.Name$ + " " + Outpts + "!" ' format message
    CALL IO.O ' send message
    CALL Enter.Room ' move player to next room
    EXIT SUB ' exit routine
END IF ' end locate empty room inventory
Outpt = "You can't move it there!" ' make message
CALL IO.O ' send message
END SUB ' end routine to move an item

REM * routine to launch an item in a specific direction to smash through
REM * a random amount of object portal locks connected through successive
REM * rooms, or to launch a weapon at a monster. weapon ammunition lands
REM * in last room accessed.

SUB Launch.Object
CALL Parse ' get first parameter
IF Parser THEN ' found parameter
    CALL Numeric ' parse number after # sign
END IF ' end compare parse
CALL Examine.Treasure ' check treasure mnemonic to room or player inventory
IF Index.Number = False THEN ' treasure index not found
    Outpt = "You can't launch that!" ' make message
    CALL IO.O ' send message
    EXIT SUB ' exit routine
END IF ' end compare index
IF TreasureRecord.Launchable = False THEN ' check if treasure can be launched
    Outpt = "That's not a launchable device!" ' make message
    CALL IO.O ' send message
    EXIT SUB ' exit routine
END IF ' end compare item to launch
IF Parser = False THEN ' compare second parameter
    Outpt = "Launch where?" ' make message
    CALL IO.O ' send message
    EXIT SUB ' exit routine
END IF ' end compare second parameter
Item.Launched = False ' item was launched flag
Item.Type = TypeNumber ' store inventory type
Item.Index = Array.Number ' store launched item index to array
Item.Charges = Charges.Number ' store launched item charges
Launch.Item = TreasureRecord.LaunchLoads ' store item launched
' check item launch treasure record
IF Launch.Item > False AND Launch.Item <= LOF(TreasureFile) / LEN(TreasureRecord) THEN
    CALL Read.Record(TreasureFile, Launch.Item) ' read item launched record
    CALL TreasureCharges(Launch.Charges) ' store item launched charges
END IF ' end check item launched number
CALL Read.Record(TreasureFile, Index.Number) ' reread launch item
IF Charges.Number = False THEN ' compare remaining charges in device
    Outpt = "You can't, it's not loaded!" ' make message
    CALL IO.O ' send message
    EXIT SUB ' exit routine
END IF ' end compare device
IF Item.Charges > 10 THEN ' compare device charges
    Item.Charges = 10 ' reset maximum charges
END IF ' end compare device
Launch.Number = False ' variable of number of loads launched
Treasure.Name$ = TreasureRecord.ShortName ' store name of device
Treasure.Name$ = RTRIM$(Treasure.Name$) ' strip trailing blanks
Treasure.Name$ = LCASE$(Treasure.Name$) ' convert to lower case
User.Command = Parsed.Command2 ' get second parameter
Parsed.Command1 = User.Command ' store parameter
CALL Get.Direction(Direction.Number) ' determine direction to launch at
IF Direction.Number THEN ' returned variable contains direction number
    Store.Room! = GetRoomRecordDirect(Direction.Number) ' get next room direction
    IF Store.Room! = SFalse THEN ' compare room number
        Outpt = "You can't launch at that!" ' make message
        CALL IO.O ' send message
        EXIT SUB ' exit routine
    END IF ' end compare room number
    Item.Launched = True ' set item is launched flag
    Outpts = Direction(Direction.Number) ' store direction name
    Outpts = RTRIM$(Outpts) ' strip trailing blanks
    Outpts = LCASE$(Outpts) ' convert to lower case
    Outpt = "You launch the " + Treasure.Name$ + " " + Outpts + "!" ' format message
    CALL IO.O ' send message
    TempZ! = Room ' store current room.
    ' loop through all charges in device to launch
    FOR Launchs = 1 TO Item.Charges
        Graphics.Off = False ' color normal
        Outpt = "For launch" + STR$(Launchs) + ";" ' format message of launch number
        CALL IO.O ' send message
        Launch.Number = Launch.Number + 1 ' increment number of launches
        Graphics.Off = True ' color white
        Outpt = "The device launchs: " ' message
        Carriage.Return = True ' suppress cr/lf
        CALL IO.O ' send message
        Outpt = Outpts + ", " ' first direction device launches
        Carriage.Return = True ' suppress cr/lf
        CALL IO.O ' send direction message
        Locks.Smashed = False ' counter of number of locks smashed
        Chained.Rooms = False ' number of rooms chained to
        Next.Room = Store.Room! ' store room number
        CALL Chain.Rooms(Next.Room, Room) ' routine to send launched item
        Outpt = "and stopped!" ' trailing message
        CALL IO.O ' send message
        ' place launched item in last room
        Room = Last.Room ' reset current room
        CALL Read.Room.Record(Room) ' read room
        ' add ammunition
        CALL Add.Room.Treasure(Launch.Item, Launch.Charges, False, Added)
        Room = TempZ! ' restore current room
        CALL Read.Room.Record(Room) ' restore room record
    NEXT ' end loop through all device launch charges
    IF Locks.Smashed = False THEN ' compare number of locks smashed
        Outpt = "You didn't smash anything!" ' make message
    ELSE ' compare lock number
        Outpt = "You smashed open" ' format message
        Outpt = Outpt + STR$(Locks.Smashed) ' with number of locks smashed
        IF Locks.Smashed = 1 THEN ' compare number
            Outpt = Outpt + " lock.." ' append message
        ELSE ' compare number
            Outpt = Outpt + " locks!" ' append message
        END IF ' end compare
    END IF ' end compare number
    CALL IO.O ' send message
    Graphics.Off = False ' color normal
    CALL Read.Room.Record(Room) ' read current room record
ELSE ' verify monster to launch at
    CALL Check.Monster ' find monster mnemonic to launch device at
    IF Monster.Number THEN ' found monster variable indexed to array
        Multiplier = ABS(TreasureRecord.Plus) ' compute attack multiplier
        ' store full monster name
        Outpts = MonsterArray(Monster.Number).MonsterName
        Outpts = RTRIM$(Outpts) ' strip trailing blanks
        Outpts = LCASE$(Outpts) ' convert to lower case
        Outpt = "You launch " + Treasure.Name$ + " at the " + Outpts + "!" ' make message
        CALL IO.O ' send message
        Item.Launched = True ' set item is launched flag
        Graphics.Off = True ' color white
        ' loop through all charges in device to launch
        FOR Launchs = 1 TO Item.Charges
            ' format message of launch number
            Outpt = "For launch" + STR$(Launchs) + ";"
            CALL IO.O ' send message
            Launch.Number = Launch.Number + 1 ' increment number of launches
            CALL Attack.Monster ' routine to hit monster
            CALL Check.Monster ' reverify monster being attacked
            ' place launched item in room
            CALL Add.Room.Treasure(Launch.Item, Launch.Charges, False, Added)
            IF Monster.Number = False THEN ' monster is not found
                EXIT FOR ' exit device launch loop
            END IF ' end compare monster
        NEXT ' end loop through all device launch charges
        Graphics.Off = False ' color normal
    END IF ' end find monster
END IF ' end compare launch direction or monster
IF Item.Launched = False THEN ' device was launched flag
    Outpt = "You can't launch at that!" ' make message
    CALL IO.O ' send message
    EXIT SUB ' exit routine
END IF ' end compare if launched
IF Item.Type = False THEN ' device launched from player inventory
    ' compute loads launched decrement
    Total.Charges = GetUserRecordCharges(Item.Index) - Launch.Number
    IF Total.Charges < False THEN ' compare lower limit
        Total.Charges = False ' set lower limit
    END IF ' end compare limit
    CALL PutUserRecordCharges(Item.Index, Total.Charges) ' store inventory charges
ELSE ' device launched from room
    ' compute loads launched decrement
    Total.Charges = GetRoomRecordTreCharges(Item.Index) - Launch.Number
    IF Total.Charges < False THEN ' compare lower limit
        Total.Charges = False ' set lower limit
    END IF ' end compare limit
    CALL PutRoomRecordTreCharges(Item.Index, Total.Charges) ' store room charges
    CALL Share.Room.Record(Room) ' write room record
END IF ' end compare launched from
END SUB ' end routine to launch an item from a device

REM * routine to recursively enter rooms by nearby room links, smashing a
REM * random number of portals.

SUB Chain.Rooms (Chain.Room!, Previous.Room!)
' Chain.Room! is a passed parameter of the next room to access.
' Previous.Room! stores the called room number to be restored upon exit.
Last.Room = Chain.Room! ' store last room recursed into
CALL Read.Room.Record(Chain.Room!) ' get next chained to room
FOR Array.Number = 1 TO 20 ' loop through all objects in room
    Object.Number = GetRoomRecordObject(Array.Number) ' get object index to file
    ' check object file bounds
    IF Object.Number > False AND Object.Number <= LOF(ObjectFile) / LEN(ObjectRecord) THEN
        CALL Read.Record(ObjectFile, Object.Number) ' get the object record
        IF ObjectRecord.DoorLock = 2 THEN ' check if object is locked
            IF RND > .66 THEN ' compute random chance
                Locks.Smashed = Locks.Smashed + 1 ' increment locks smashed counter
                ObjectRecord.DoorLock = 1 ' set object unlocked
                ObjectRecord.Closed = False ' set object to open
                CALL Share.Record(ObjectFile, Object.Number) ' write object record
            END IF ' end random chance
        END IF ' end compare object lock
    END IF ' end end object bounds
NEXT ' end loop through all objects
FOR Direction.Number = 1 TO 12 ' loop through all directions in chained to room
    Next.Room = GetRoomRecordDirect(Direction.Number) ' get the room number of the
    IF Next.Room THEN ' direction and compare to
        IF Next.Room <> Previous.Room! THEN ' the current room in routine
            IF RND > .66 THEN ' compute random chance
                IF Chained.Rooms < 5 THEN ' compare to maximum of 5 recursive calls
                    Carriage.Return = True ' suppress cr/lf
                    Outpt = Direction(Direction.Number) ' get direction name
                    Outpt = RTRIM$(Outpt) + ", " ' format message
                    CALL IO.O ' send message
                    ' increment number of recursively chained rooms
                    Chained.Rooms = Chained.Rooms + 1
                    CALL Chain.Rooms(Next.Room, Chain.Room!) ' routine calls self
                    CALL Read.Room.Record(Chain.Room!) ' next room gets previous room
                    EXIT SUB ' recursively exit previous calls
                END IF ' end compare maximum
            END IF ' end random chance
        END IF ' end compare to current room
    END IF ' end compare next direction
NEXT ' end loop through all directions in called room (exit recursively)
END SUB ' end routine to recursively loop through all objects in neraby rooms

REM * routine for player to steal an item from a monster.

SUB Steal.Object
IF Normal.User THEN ' check player type
    IF UserRecord.ClassType <> Thief THEN ' check player class type
        Outpt = "Only thieves can steal!" ' make message
        CALL IO.O ' send message
        EXIT SUB ' exit routine
    END IF ' end compare class type
END IF ' end compare player type
CALL ParseX ' parse the second parameter
IF Parser THEN ' parse the
    CALL Numeric ' number after the # sign
END IF ' end parse
IF Parser = False THEN ' no second parameter
    Outpt = "Steal from whom?" ' make message
    CALL IO.O ' send message
    EXIT SUB ' exit routine
END IF ' end compare parse
CALL Check.Monster ' verify monster mnemonic
IF Monster.Number THEN ' monster index
    Parsed.Command1 = LCASE$(Parsed.Command2) ' get first parameter
    CALL Numeric ' parse number
    FOR Array.Number = 1 TO 5 ' loop through all monster treasure
        ' get monster treasure index
        Treasure.Number = GetMonsterRecordTreasure(Array.Number)
        ' check file bounds
        IF Treasure.Number > False AND Treasure.Number <= LOF(TreasureFile) / LEN(TreasureRecord) THEN
            CALL Read.Record(TreasureFile, Treasure.Number) 'read treasure record
            Inpt = TreasureRecord.ShortName ' store short treasure mnemonic
            Inpt = RTRIM$(Inpt) ' strip trailing blanks
            Inpt = LCASE$(Inpt) ' convert to lower case
            Inpt = LEFT$(Inpt, LEN(Parsed.Command1)) ' shorten to parsed length
            IF Inpt = Parsed.Command1 THEN ' compare treasure to parsed treasure
                ' compare monster to magical, or permanent
                IF MonsterArray(Monster.Number).Magic <= True OR MonsterArray(Monster.Number).Permanent <= True THEN
                    Outpt = "You can't steal from that monster!" ' make message
                    CALL IO.O ' send message
                    EXIT SUB ' exit routine
                END IF ' end compare monster
                ' compute random chance
                IF INT(RND * 5 + 5) > GetUserRecordStats(4) / 2 THEN
                    Outpt = "Didn't work!" ' make message
                    CALL IO.O ' send message
                    EXIT SUB ' exit routine
                END IF ' end random chance
                Inpt = TreasureRecord.TreasureName ' get full name of treasure
                Inpt = RTRIM$(Inpt) ' strip trailing blanks
                Inpt = LCASE$(Inpt) ' convert to lower case
                ' get monster name
                Outpts = MonsterArray(Monster.Number).MonsterName
                Outpts = RTRIM$(Outpts) ' strip trailing blanks
                Outpts = LCASE$(Outpts) ' convert to lower case
                Item.Charges = TreasureRecord.Charges ' store charges
                ' add to player inventory
                CALL Add.Inventory(Treasure.Number, Item.Charges, Item.Added)
                IF Item.Added THEN ' flag set to add inventory
                    Outpt = "You steal " + Inpt + " from the " + Outpts + "!" ' make message
                    CALL IO.O ' send message
                    ' clear element in monster inventory array
                    CALL PutMonsterRecordTreasure(Array.Number, False)
                    EXIT SUB ' exit routine
                END IF ' end compare flag
            END IF ' end compare treasure
        END IF ' end compare treasure
    NEXT ' end loop through all monster inventory
END IF ' end find monster number
Outpt = "You can't steal that!" ' make message
CALL IO.O ' send message
END SUB ' end routine to steal

REM * routine to throw an item of player inventory at monster, weapon falls
REM * into the room.

SUB Throw.Object
IF Normal.User THEN ' verify player type
    IF UserRecord.ClassType <> Thief THEN ' verify player class type
        Outpt = "Only thieves can throw!" ' make message
        CALL IO.O ' send message
        EXIT SUB ' exit routine
    END IF ' end compare class type
END IF ' end compare player type
CALL ParseX ' get second parameter
IF Parser THEN ' parse parameter
    CALL Numeric ' store number
END IF ' end compare parse
IF Parser = False THEN ' missing second parameter
    Outpt = "Throw at whom?" ' make message
    CALL IO.O ' send message
    EXIT SUB ' exit routine
END IF ' end compare parameter
CALL Check.Monster ' find monster in room
IF Monster.Number THEN ' found monster index
    Last.Monster = Parsed.Command1 ' reset last monster attacked
    Parsed.Command1 = UCASE$(Parsed.Command2) ' get first parameter
    CALL Numeric ' parse number sign
    CALL Check.Inventory.Treasure ' search player inventory for treasure
    IF Index.Number > False THEN ' found treasure index in player inventory
        ' weapon being held must match the weapon thrown
        IF Weapon6 = Array.Number THEN
            Multiplier = Weapon2 ' set attack multiplier
            Inpt = MonsterArray(Monster.Number).MonsterName ' get monster name
            Inpt = RTRIM$(Inpt) ' strip trailing blanks
            Inpt = LCASE$(Inpt) ' convert to lower case
            Outpt = "You throw " + Outpts + " at the " + Inpt + "!" ' format message
            CALL IO.O ' send message
            IF INT(RND * 5 + 2) > GetUserRecordStats(4) / 2 THEN ' compute random chance
                Outpt = "You missed!" ' make message
                CALL IO.O ' send message
            ELSE ' random chance
                Parsed.Command1 = Last.Monster ' reset parameter to last monster
                Last.Command.Number = 55 ' reset the command number to attack
                CALL Attack.Monster ' routine to hit monster
            END IF ' end random chance
            CALL Discard.Inventory(Weapon6, False) ' remove item from inventory
            EXIT SUB ' exit routine
        END IF ' end compare weapon being held
        Outpt = "You aren't holding the weapon!" ' make message
        CALL IO.O ' send message
        EXIT SUB ' exit routine
    END IF ' end search for weapon to throw
END IF ' end search for monster
Outpt = "You can't throw that!" ' make message
CALL IO.O ' send message
END SUB ' end routine to throw an object

REM * routine to fuel an item from the charges of another item.

SUB Fuel.Object
CALL Find.Inventory ' parse first parameter, search player inventory
Treasure.Name$ = TreasureRecord.ShortName ' store treasure short mnemonic name
Treasure.Name$ = RTRIM$(Treasure.Name$) ' strip trailing blanks
Treasure.Name$ = LCASE$(Treasure.Name$) ' convert to lower case
IF Index.Number = False THEN ' compare treasure index
    Outpt = "You can't fuel that!" ' make message
    CALL IO.O ' send message
    EXIT SUB ' exit routine
END IF ' end compare index
IF TreasureRecord.LightType = False THEN ' check treasure can be fueled
    Outpt = "You can't fuel that!" ' make message
    CALL IO.O ' send message
    EXIT SUB ' exit routine
END IF ' end compare treasure
IF TreasureRecord.LightCharges = False THEN ' check treasure can be fueled
    Outpt = "It has no light charges!" ' make message
    CALL IO.O ' send message
    EXIT SUB ' exit routine
END IF ' end compare treasure
IF GetUserRecordCharges(Array.Number) THEN ' check treasure already fueled
    Outpt = "It's already fueled!" ' make message
    CALL IO.O ' send message
    EXIT SUB ' exit routine
END IF ' end compare treasure
Fuel.Charges = TreasureRecord.LightCharges ' store treasure light charges
Item.Number = Array.Number ' store treasure array index
IF Parser = False THEN ' compare second parameter
    Outpt = "Fuel with what?" ' make message
    CALL IO.O ' send message
    EXIT SUB ' exit routine
END IF ' end compare parse
Parsed.Command1 = Parsed.Command2 ' get second parameter
CALL Numeric ' parse number
CALL Find.Inventory ' search player inventory
IF Index.Number = False THEN ' compare treasure index
    Outpt = "You can't fuel with that!" ' make message
    CALL IO.O ' send message
    EXIT SUB ' exit routine
END IF ' end compare index
IF TreasureRecord.FuelType = False THEN ' check treasure is a fuel
    Outpt = "You can't fuel with that!" ' make message
    CALL IO.O ' send message
    EXIT SUB ' exit routine
END IF ' end compare treasure
IF TreasureRecord.FuelCharges <= False THEN ' check treasure fuel amount
    Outpt = "It has no fuel charges!" ' make message
    CALL IO.O ' send message
    EXIT SUB ' exit routine
END IF ' end compare treasure
IF GetUserRecordCharges(Array.Number) = False THEN ' check fuel charges
    Outpt = "It has no fuel charges!" ' make message
    CALL IO.O ' send message
    EXIT SUB ' exit routine
END IF ' end compare treasure
' decrement fuel
CALL PutUserRecordCharges(Array.Number, GetUserRecordCharges(Array.Number) - 1)
CALL PutUserRecordCharges(Item.Number, Fuel.Charges) ' add fuel charges to treasure
Outpt = "You fuel the " + Treasure.Name$ + "!" ' format message
CALL IO.O ' send message
END SUB ' end routine to fuel an item

REM * routine to light an item.

SUB Light.Object
CALL Check.Inventory.Treasure ' search for player inventory treasure
IF Index.Number = False THEN ' compare treasure index
    Outpt = "You can't light that!" ' make message
    CALL IO.O ' send message
    EXIT SUB ' exit routine
END IF ' end compare treasure index
IF TreasureRecord.LightType = False THEN ' check treasure is a light
    Outpt = "That's not a light!" ' make message
    CALL IO.O ' send message
    EXIT SUB ' exit routine
END IF ' end compare treasure
IF GetUserRecordCharges(Array.Number) = False THEN ' check treasure is fueled
    Outpt = "You can't, it's not fueled!" ' make message
    CALL IO.O ' send message
    EXIT SUB ' exit routine
END IF ' end compare treasure
' check treasure is already lit
IF GetUserRecordCharges(Array.Number) < False THEN
    Outpt = "It's already lit!" ' make message
    CALL IO.O ' send message
    EXIT SUB ' exit routine
END IF ' end compare treasure
' set inventory charges
Var = GetUserRecordCharges(Array.Number): Var = -Var
CALL PutUserRecordCharges(Array.Number, Var)
Outpts = TreasureRecord.ShortName ' to negative to indicate treasure is a light
Outpts = RTRIM$(Outpts) ' store treasure name, strip trailing blanks
Outpts = LCASE$(Outpts) ' convert to lower case
Outpt = "You light the " + Outpts + "!" ' format message
CALL IO.O ' send message
END SUB ' end routine to light an item

REM * dm/sysop routine to discard an item of inventory to nowhere, can be
REM * used interactively during remote with function key.

SUB Discard.Object
CALL Check.Inventory.Treasure ' find treasure in player inventory
IF Index.Number THEN ' treasure index to player array
    ' routine to remove from inventory
    CALL Discard.Inventory(Array.Number, True)
    Outpt = "You discard " + Outpts + "!" ' format message
    CALL IO.O ' send message
    EXIT SUB ' exit routine
END IF ' end search treasure inventory
CALL Num ' decrement parse counter
CALL Check.Inventory.Objects ' find object in player inventory
IF Index.Number THEN ' object index to player array
    CALL Discard.Inventory.Object(Array.Number)
    Outpt = "You discard " + Outpts + "!" ' format message
    CALL IO.O ' send message
    EXIT SUB ' exit routine
END IF ' end search object inventory
CALL Num ' decrement parse temp
CALL Check.Inventory.Container ' check player inventory for container
IF Index.Number THEN ' returns index of container to treasure
    Outpts = RTRIM$(ContainerRec.ContainerName) ' message
    CALL Clear.Container(0, True) ' clear player container structure
    CALL PutUserRecordContainer(Array.Number) ' clear container record
    Outpt = "You discard " + Outpts + "!" ' format message
    CALL IO.O ' send output
    EXIT SUB ' return from routine
END IF ' end check container index
Outpt = "You can't discard that!" ' make message
CALL IO.O ' send message
END SUB ' end routine to discard an item

REM * Filename: dnds2.bas Version: v5.0a r1.0a
REM * This subprogram contains most login routines, and parse routines.

REM * routine to login user.
REM * output variables:
REM *   Time.On - contains time logged in.
REM *   Timeon - contains time logged in past midnight.
REM *   Time.Left - stores time limit in seconds past midnight.
REM *   User.Index - number of record in user file codename will use.
REM * processing variables:
REM *   Time.Left - during logging in.
REM *   Two.Minutes.Left - time remaining flag.
REM *   Login.Try - total login attempts.
REM *   New.User - flag indicates new user login.
REM *   Logged.User - flag indicates user exists in files.

SUB Login
Time.On = TIME$ ' store system time logged in (string form hh:mm:ss)
Timeon = TIMER ' store system time logged in (seconds past midnight)
Timelogged.On = TIME$ ' store time logged in (double format)
Time.Left = 600! ' set time limit during logging in
Two.Minutes.Left = False ' flag for two minute left display message
Logged.In = False ' set user logged in flag
' format initial login display line
Outpt = "The Adventure Door v" + Version$ + ", Node: " + Node + ", " + clock$
IF Local.Mode = False THEN
    Outpt = Outpt + ", baud" + STR$(Modem.Baud) + "00"
END IF
Outpt = Outpt + "."
CALL IO.O ' display initial message
' format prompt for login welcome
Outpt = "Press <enter> to display the welcome, or <space> to skip:"
No.Echo = True ' supress prompt input echo
Line.Length = 1 ' get only one keypress
CALL IO.I ' input routine
No.Echo = False ' reset echo
IF Inpt = Nul THEN ' check empty return
    CALL Out.File("welcome.dat") ' display welcome file
END IF ' end display welcome
DO ' main login processing loop
    Login.Try = False ' reset login attempts
    DO ' get codename processing loop
        New.User = False ' reset new user flag
        CALL Get.Codename ' prompt user for codename
        CALL Get.PassWord ' prompt user for password
        CALL Find.PassWord(Logged.User) ' find codename in user file
        IF Logged.User THEN ' compare user has entered an existing codename
            CALL Verify.PassWord(Logged.Pass) ' compare entered password
            IF Logged.Pass THEN ' check if password matches
                EXIT DO ' exit codename processing/entry loop
            END IF ' end check password match
            IF Login.Try >= 3 THEN ' compare login attempts
                CALL Hang.Up(7) ' routine to terminate program w/ message
                EXIT SUB ' exit login routine/return to main
            END IF ' end compare login attempts
            Login.Try = Login.Try + 1 ' increment login attempt
            Outpt = "Illegal password attempt!" ' display error message
            CALL IO.O ' send output/continue codename entry loop
        ELSE ' user has not entered an existing codename
            CALL Get.Newuser.Record ' find an empty user file record
            CALL Verify.Newuser(New.User) ' verify password entered
            IF New.User > False THEN ' user selected disconnect
                CALL Hang.Up(8) ' routine to terminate program w/ message
                EXIT SUB ' exit routine
            END IF ' end compare disconect
            IF New.User < False THEN ' user password verified
                CALL Init.Newuser ' initialize some new user variables
                CALL Verify.Newlogin(New.User, True) ' verify user is new
                IF New.User THEN ' user selects to continue as a new user
                    EXIT DO ' exit codename/password entry loop
                END IF ' end new user continue
            END IF ' end password verify
        END IF ' end compare existing codename
    LOOP ' codename/password entry loop
    CALL Update.Login ' intialize some login variables
    IF New.User = False THEN ' compare user is new user
        EXIT DO ' exit main login processing loop
    END IF ' end compare new user
    IF New.User THEN ' compare new user login is verified
        CALL Roll.Character ' get user selected character statistics
        CALL Verify.Newlogin(New.User, False) ' verify new user to continue
        IF New.User THEN ' new user is logged in
            EXIT DO ' exit main login processing loop
        END IF ' end compare new user login
    END IF ' end compare new user login
LOOP ' end main login processing loop
CALL Login.User ' routine to initialize some login variables
Outpt = Nul ' send empty output
CALL IO.O ' send output
Outpt = "Press <enter> to begin the adventure:" ' format message
No.Echo = True ' set flag to supress echo input
Line.Length = 1 ' get one keypress
CALL IO.I ' get input
No.Echo = False ' reset echo flag
Next.Room = Room ' store room number
CALL Enter.Room ' display room description
Logged.In = True ' set user logged in flag
FunctionBuffer = Nul ' reset function key flag
END SUB ' end routine to login user

REM * routine to find codename entered in user file.
REM * output variables:
REM *   PassWord.Found - flag if codename is in user file.

SUB Find.PassWord (PassWord.Found)
PassWord.Found = False ' set flag to false
Inpt = RTRIM$(Player.CodeName) ' store codename
FOR User.Index = 1 TO LOF(UserFile) / LEN(UserRecord) ' loop through entire user file
    CALL Read.Record(UserFile, User.Index) ' get next user file record
    Outpt = UserRecord.CodeName ' store user file codename
    CALL Decrypt(Outpt) ' decrypt user file codename
    Outpt = RTRIM$(Outpt) ' trim user file codename
    IF Outpt = Inpt THEN ' compare user file codename to codename entered
        PassWord.Found = True ' set return variable flag
        EXIT FOR ' exit user file loop
    END IF ' end check codenames
NEXT ' loop through user file
END SUB ' end routine to find user file codename

REM * routine to verify valid password of codename entered.
REM * output variables:
REM *   PassWord.Found - flag indicates password exists.

SUB Verify.PassWord (PassWord.Found)
PassWord.Found = False ' set flag to false
Outpt = UserRecord.PassWord ' store user file password
CALL Decrypt(Outpt) ' decrypt password
IF LEN(Outpt) = False THEN ' verify decrypt result
    Outpt = "Password has a checksum error!" ' format message
    CALL IO.O ' send message
    PassWord.Found = False ' set flag to verify password
    EXIT SUB ' exit check password routine
END IF ' end check password
Outpt = RTRIM$(Outpt) ' store trimmed user file password
Inpt = RTRIM$(Player.PassWord) ' store password entered
IF Outpt = Inpt THEN ' compare user file password to entered password
    PassWord.Found = True ' set flag to verify password
END IF ' end compare passwords
END SUB ' end routine to check valid password

REM * routine to verify new user is continuing.
REM * input variables:
REM *   Message.Type - message to select.
REM * output variables:
REM *   Response.Type - returns true to continue, false if not.

SUB Verify.Newlogin (Response.Type, Message.Type)
DO ' process input loop
    Graphics.Off = False ' reset color
    IF Message.Type THEN ' compare prompt
        Outpt = "Press <enter> to roll character, or <space> to reenter:"
    ELSE ' select prompt
        Outpt = "Press <enter> to use character, or <space> to reroll:"
    END IF ' end compare prompt
    No.Echo = True ' supress input echo
    Line.Length = 1 ' input one keypress
    CALL IO.I ' get user input
    No.Echo = False ' reset input echo
    IF Inpt = " " THEN ' selected space
        Response.Type = False ' set return flag
        EXIT DO ' exit routine
    END IF ' end compare select
    IF No.Input THEN ' compare empty input
        Response.Type = True ' set return flag
        EXIT DO ' exit routine
    END IF ' end compare select
LOOP ' process input loop
END SUB ' end routine to prompt to continue

REM * routine to get codename, check illegal character in codename, verify
REM * user has entered correct codename.
REM * output variables:
REM *   Player.CodeName - contains lowercased, trimmed codename entered.

SUB Get.Codename
DO ' main codename entry processing loop
    DO ' loop until valid codename entered
        Outpt = Nul ' empty output
        CALL IO.O ' send output
        Graphics.Off = True ' reset color
        Outpt = "         +---------+---------+---------+" ' make length bar
        CALL IO.O ' send output
        Outpt = "Codename? " ' codename prompt
        Line.Length = 30 ' set line length of codename
        Upper.Case = True ' reset uppercase flag
        CALL IO.I ' get codename input
        Upper.Case = False ' reset uppercase flag
        Graphics.Off = False ' reset color
        Inpt = LTRIM$(Inpt) ' trim blanks
        Inpt = RTRIM$(Inpt) ' trim blanks
        Inpt = UCASE$(Inpt) ' set input to uppercase
        Player.CodeName = Inpt ' store codename
        IF LEN(Player.CodeName) > False THEN ' check length of codename
            CALL Valid(Player.CodeName, 30) ' verify valid characters
            IF LEN(Player.CodeName) > False THEN ' check valid codename
                EXIT DO ' exit codename entry loop
            END IF ' end check valid characters
            Outpt = "Illegal characters in codename!" ' format message
            CALL IO.O ' send output
        END IF ' end check codename length
    LOOP ' continue entry loop
    Outpt = RTRIM$(Player.CodeName) ' store codename
    Outpt = LCASE$(Outpt) ' set to lowercase
    MID$(Outpt, 1, 1) = UCASE$(MID$(Outpt, 1, 1)) ' make first character uppercase
    Outpt = "You are " + CHR$(34) + Outpt + CHR$(34) + "(y/n)? " ' format prompt
    No.Input.Out = "Y" ' set default input to yes
    Line.Length = 1 ' reset line length
    CALL IO.I ' prompt for correct codename entered
    IF Yes THEN ' verify user has verified correct codename
        EXIT DO ' exit main codename entery loop
    END IF ' end verify user input
LOOP ' end codename entry loop
END SUB ' end routine to get and verify codename entry

REM * routine to prompt user for password, check valid characters in password.
REM * output variables:
REM *   Player.PassWord - contains lowercased, trimmed password entered.

SUB Get.PassWord
DO ' password entry loop
    Outpt = Nul ' empty output
    CALL IO.O ' send output
    Graphics.Off = True ' reset color
    Outpt = "         +---------+---------+" ' make length bar
    CALL IO.O ' send output
    Outpt = "Password? " ' password prompt
    Line.Length = 20 ' set line length of password
    Hidden = True ' set flag to echo mask characters
    CALL IO.I ' get input
    Hidden = False ' reset mask flag
    Graphics.Off = False ' reset color
    Inpt = LTRIM$(Inpt) ' trim blanks
    Inpt = RTRIM$(Inpt) ' trim blanks
    Inpt = UCASE$(Inpt) ' set input to uppercase
    Player.PassWord = Inpt ' store password
    IF LEN(Player.PassWord) > False THEN ' check length of password
        CALL Valid(Player.PassWord, 20) ' verify valid characters
        IF LEN(Player.PassWord) > False THEN ' check valid password
            EXIT DO ' exit password entry loop
        END IF ' end check valid characters
        Outpt = "Illegal characters in password!" ' format message
        CALL IO.O ' send output
    END IF ' end check password length
LOOP ' end password entry loop
END SUB ' end routine to enter password

REM * routine to find empty user file record.
REM * output variables:
REM *    User.Index - unused number of record in user file.

SUB Get.Newuser.Record
FOR User.Index = 1 TO LOF(UserFile) / LEN(UserRecord) ' loop through users
    CALL Read.Record(UserFile, User.Index) ' load the user record
    Outpt = UserRecord.CodeName ' store user file codename
    CALL Decrypt(Outpt) ' routine to decrypt codename
    IF LEFT$(Outpt, 9) = Deleted$ THEN ' compare to deleted record
        EXIT FOR ' end loop through user file
    END IF ' end compare deleted record
NEXT ' end loop through user file
' exit of loop w/o finding a deleted record will set User.Index to one
' record past the last in the user file, appending the next record.
CALL Read.Record(UserFile, User.Index) ' get empty user file record
Outpt = "Codename not found in files!" ' make message
CALL IO.O ' send message
END SUB ' end routine to find empty user file record

REM * routine to verify new user login.
REM * output variables:
REM *   Response.Type - flag set to 1 to disconnect,
REM *   0 if password not verified, -1 if password is verified.

SUB Verify.Newuser (Response.Type)
DO ' password verify loop
    Outpt = "Press <C>ontinue, <H>angup, or <R>estart:" ' make message
    No.Input.Out = "C" ' default empty input to continue
    Line.Length = 1 ' reset line length
    CALL IO.I ' get input
    Response.Type = False ' set return flag to unverified
    SELECT CASE UCASE$(Inpt) ' compare input selection
        CASE "C" ' continue selected
            Outpt = Nul ' set empty output
            CALL IO.O ' send output
            Outpt = "Verify password: " ' prompt for password
            Hidden = True ' set echo characters masked
            Line.Length = 20 ' line length of password
            CALL IO.I ' get user input
            Hidden = False ' reset echo mask flag
            Inpt = LTRIM$(Inpt) ' trim input
            Inpt = RTRIM$(Inpt) ' trim input
            Inpt = UCASE$(Inpt) ' set input uppercase
            Outpt = RTRIM$(Player.PassWord) ' store password entered
            IF Outpt <> Inpt THEN ' compare password entered to verify entry
                Outpt = "Passwords don't match!" ' make error message
                CALL IO.O ' send output
                Response.Type = False ' set flag to unverified
                EXIT DO ' exit routine
            END IF ' end compare password
            Outpt = "Memorize your password!" ' make message
            CALL IO.O ' send output
            CALL IO.O ' send empty line
            Response.Type = True ' set flag to verified
            EXIT DO ' exit routine
        CASE "H" ' hangup selected
            Response.Type = 1 ' set flag to hangup user
            EXIT DO ' exit routine
        CASE "R" ' restart selected
            Response.Type = False ' set flag to restart/unverified
            EXIT DO ' exit routine
    END SELECT ' end compare input selection
LOOP ' end password verify loop
END SUB ' end routine to verify new password

REM * routine to allow user to select new character statistics.
REM * processing variables:
REM *   Display.Help - flag to display login help messages.

SUB Roll.Character
Display.Help = False ' set flag to display help text
Outpt = "List help text during character logon(y/n)? " ' prompt for help
No.Input.Out = "N" ' default prompt
Line.Length = 1 ' reset line length
CALL IO.I ' get user input
IF Yes THEN ' check input
    Display.Help = True ' set help text flag
END IF ' end check input
IF Display.Help THEN ' check help flag
    CALL Logon.Help(1) ' display class help text
END IF ' end check help flag
CALL Modify.Class ' routine to select class type
IF Display.Help THEN ' check help flag
    CALL Logon.Help(2) ' display statistics entry help text
END IF ' end check help flag
CALL Modify.Stats ' routine to select character statistics
IF Display.Help THEN ' check help flag
    CALL Logon.Help(3) ' display race entry help text
END IF ' end check help flag
CALL Modify.Race ' routine to select character race
CALL Init.Race.Stats ' routine to initialize some race statistics
IF Display.Help THEN ' check help flag
    CALL Logon.Help(4) ' display weapon proficiency entry help text
END IF ' end check help flag
CALL Modify.Proficiency ' routine to select weapon proficiency
CALL Init.Proficiency.Stats ' routine to intialize proficiency statistics
CALL Init.Stats ' routine to intialize some character statistics
IF Display.Help THEN ' check help flag
    CALL Logon.Help(5) ' display alignment entry help text
END IF ' end check help flag
CALL Modify.Alignment ' routine to select character alignment
CALL Display.Init.Stats ' routine to display character statistics
END SUB ' end routine to get new character statistics

REM * routine to display help text.
REM * input variables:
REM *   Help.Number - range of help text file records to display.
REM * work variables:
REM *   Start.Help, End.Help, Help.Count.

SUB Logon.Help (Help.Number)
CLOSE #HelpFile ' close work file number
FileName = "logon.dat" ' store logon helptext filename
OPEN FileName FOR RANDOM SHARED AS #HelpFile LEN = LEN(HelpRecord) ' open file
Graphics.Off = True ' set color flag
Outpt = Nul ' send empty line
CALL IO.O ' send output
SELECT CASE Help.Number ' selection for logon help record ranges
    CASE 1 ' help records
        Start.Help = 2
        End.Help = 6
    CASE 2 ' help records
        Start.Help = 7
        End.Help = 15
    CASE 3 ' help records
        Start.Help = 16
        End.Help = 25
    CASE 4 ' help records
        Start.Help = 26
        End.Help = 32
    CASE 5 ' help records
        Start.Help = 33
        End.Help = 36
END SELECT ' end select record ranges
FOR Help.Count = Start.Help TO End.Help ' loop through help text file range
    CALL Read.Record(HelpFile, Help.Count) ' read help record
    Outpt = RTRIM$(HelpRecord.Text) ' format help text
    CALL IO.O ' send help output
NEXT 'end loop through help file
CALL IO.O ' send ampty output
CALL More.Prompt ' get keypress
Graphics.Off = False ' reset color flag
CLOSE #HelpFile
END SUB ' end routine to display help text

REM * routine to allow user to modify character alignment.

SUB Modify.Alignment
DO ' process modify alignment one loop
    Graphics.Off = False ' reset color
    Outpt = "Player Alignment:" ' make message
    CALL IO.O ' display message
    Outpt = "Press " + Enter$ + " for default." ' make message
    CALL IO.O ' display message
    Graphics.Off = True ' reset color
    FOR Align.Count = 1 TO 3 ' display alignment choices
        Outpt = MID$(STR$(Align.Count), 2) + "> " + RTRIM$(Alignment.Name1(Align.Count))
        CALL IO.O ' send output
    NEXT ' loop through alignment choices
    Outpt = "?" ' prompt for alignment number
    No.Input.Out = "2" ' default to neutral
    Line.Length = 1 ' reset line length
    CALL IO.I ' get input
    Player.Alignment = INT(VAL(Inpt)) ' convert to number
    IF Player.Alignment >= 1 AND Player.Alignment <= 3 THEN ' compare valid choice
        EXIT DO ' exit first loop
    END IF ' end compare choice
LOOP ' continue alignment loop
UserRecord.Align1 = Player.Alignment - 2 ' store alignment as -1/0/1
DO ' process modify alignment two loop
    Graphics.Off = False ' reset color
    Outpt = "Player Alignment:" ' make message
    CALL IO.O ' display message
    Outpt = "Press " + Enter$ + " for default." ' make message
    CALL IO.O ' display message
    Graphics.Off = True ' reset color
    FOR Align.Count = 1 TO 3 ' display alignment choices
        Outpt = MID$(STR$(Align.Count), 2) + "> " + RTRIM$(Alignment.Name2(Align.Count))
        CALL IO.O ' send output
    NEXT ' loop through alignment chocies
    Outpt = "?" ' prompt for alignment number
    No.Input.Out = "2" ' default to neutral
    Line.Length = 1 ' reset line length
    CALL IO.I ' get input
    Player.Alignment = INT(VAL(Inpt)) ' convert to number
    IF Player.Alignment >= 1 AND Player.Alignment <= 3 THEN ' compare valid choice
        EXIT DO ' exit second loop
    END IF ' end compare choice
LOOP ' continue alignment loop
UserRecord.Align2 = Player.Alignment - 2 ' store alignment as -1/0/1
END SUB ' end routine to modify character alignment

REM * routine to allow user to modify character class Type

SUB Modify.Class
Outpt = Nul ' make empty line
CALL IO.O ' send output
DO ' class entry process loop
    Graphics.Off = False ' reset color
    CALL IO.O ' send blank line
    Outpt = "Select your character class:" ' make message
    CALL IO.O ' send output
    Outpt = "Press " + Enter$ + " for default." ' make message
    CALL IO.O ' send output
    IF Local.Mode = False THEN ' console mode allows DM/Asst. DM entries
        Max.Class = 8 ' set number of class choices
    ELSE ' compare console mode
        Max.Class = 10 ' set number of class choices
    END IF ' end compare console mode
    Graphics.Off = True ' reset color
    FOR List.Counter = 1 TO Max.Class ' loop through class chocies
        Outpt = MID$(STR$(List.Counter), 2) ' store class number
        IF List.Counter = 10 THEN ' store DM class number
            Outpt = "#" ' choice ten is pound sign
        END IF ' end compare DM class number
        Outpt = Outpt + "> " + RTRIM$(Class.Name(List.Counter)) ' append class name
        CALL IO.O ' send output
    NEXT ' loop through class choices
    Outpt = "?" ' set input prompt
    No.Input.Out = "1" ' set default choice
    Line.Length = 1 ' reset line length
    CALL IO.I ' get user input
    Player.Class = INT(VAL(Inpt)) ' convert to number
    IF Inpt = "#" THEN ' compare DM selection
        Player.Class = 10 ' set to ten
    END IF ' end compare DM selection
    IF Player.Class >= 1 AND Player.Class <= Max.Class THEN ' check class range
        EXIT DO ' exit class type entry loop
    END IF ' end check class range
LOOP ' end class type entry loop
UserRecord.ClassType = Player.Class ' store class number in user record
Outpt = Class.Name(UserRecord.ClassType) ' get class name
CALL Valid(Outpt, 20) ' validate class name
IF Outpt = Nul THEN ' verify class name validity
    Outpt = "<checksum>" ' set error message
    CALL Valid(Outpt, 20) ' validate error
END IF ' end verify class name
CALL Encrypt(Outpt, True) ' encrypt class name
UserRecord.ClassName = Outpt ' store class name in user record
END SUB ' end routine to modify character class type

REM * routine to allow user to modify character statistics.

SUB Modify.Stats
DO ' loop until statistics selected are accepted
    DO ' loop until statistics are valid
        Graphics.Off = False ' reset color
        Outpt = "Enter character statistics, range from 8 to 18." ' message
        CALL IO.O ' send message
        Outpt = "Average less than or equal to 12." ' message
        CALL IO.O ' send message
        Outpt = "Press " + Enter$ + " for default." ' message
        CALL IO.O ' send message
        Stat.Total! = False ' reset total of selected statistics
        FOR Class.Number = 1 TO 7 ' loop through entry of all statistics
            DO ' loop until a valid statistic entered
                Graphics.Off = True ' reset color
                Outpt = RTRIM$(Stat(Class.Number)) + ">" ' make message w/ stat name
                No.Input.Out = "12" ' set default
                Line.Length = 2 ' reset line length
                CALL IO.I ' get input
                Stat = INT(VAL(Inpt)) ' convert to number
                IF Stat < 8 OR Stat > 18 THEN ' check range
                    Graphics.Off = False ' reset color
                    Outpt = "The average statistic must range from 8 to 18."
                    CALL IO.O ' send output message
                ELSE ' check range
                    Stat.Total! = Stat.Total! + Stat ' increment stat total
                    CALL PutUserRecordStats(Class.Number, Stat) ' store stat in user record
                    EXIT DO ' exit validity loop
                END IF ' end check range
            LOOP ' continue valid statistic loop
        NEXT ' loop through all statistics
        Stat.Total! = Stat.Total! / 7! ' calculate average of total statistics
        Stats$ = STR$(Stat.Total!) ' convert to string
        Stat.Delimit = INSTR(Stats$, ".") ' search string for decimal
        IF Stat.Delimit = False THEN ' compare decimal
            Inpt = Stats$ ' set output string to converted string
        ELSE ' check decimal, truncate to one place
            ' set string
            Inpt = LEFT$(Stats$, Stat.Delimit - 1) + "." + MID$(Stats$, Stat.Delimit + 1, 1)
        END IF ' end compare decimal
        Graphics.Off = False ' reset color
        IF Stat.Total! <= 12 THEN ' verify average
            EXIT DO ' exit validity loop
        END IF ' end verify average
        Outpt = "Average" + Inpt + " to high! Try again.." ' set message
        CALL IO.O ' send message
    LOOP ' end statistic validity loop
    Outpt = "Your average is" + Inpt + ". Change anything(y/n)? " ' make message
    No.Input.Out = "N" ' set default input
    Line.Length = 1 ' reset line length
    CALL IO.I ' get user input
    IF No THEN ' check no entered
        EXIT SUB ' exit routine
    END IF ' end check entry
LOOP ' end loop to verify statistics accepted
END SUB ' end routine to modify character statistics

REM * routine to allow user to modify character race.

SUB Modify.Race
DO ' loop until race entry is accepted
    Graphics.Off = False ' reset color
    Outpt = "Select your character race:" ' make message
    CALL IO.O ' send message
    Outpt = "Press " + Enter$ + " for default." ' make message
    CALL IO.O ' send message
    Graphics.Off = True ' reset color
    FOR Race.Count = 1 TO 8 ' loop through all race choices
        ' choice display
        Outpt = MID$(STR$(Race.Count), 2) + ">" + RTRIM$(Race(Race.Count))
        CALL IO.O ' send choice
    NEXT ' end race display loop
    Outpt = "?" ' set input prompt
    No.Input.Out = "1" ' set default
    Line.Length = 1 ' reset line length
    CALL IO.I ' get user input
    Player.Race = INT(VAL(Inpt)) ' convert to number
    IF Player.Race >= 1 AND Player.Race <= 8 THEN ' check race range
        UserRecord.Race = Player.Race ' store race in user record
        EXIT SUB ' exit routine
    END IF ' end compare race range
LOOP ' end loop to accept race entry
END SUB ' end routine to modify race

REM * routine to allow user to modify character weapon proficiency.

SUB Modify.Proficiency
DO ' loop until proficiency entry accepted
    Graphics.Off = False ' reset color
    Outpt = "Weapon Proficiency:" ' set message
    CALL IO.O ' send output
    Outpt = "Clerics may only use blunt or pole type weapons." ' message
    CALL IO.O ' send output
    Outpt = "Press " + Enter$ + " for default." ' message
    CALL IO.O ' send output
    Graphics.Off = True ' reset color
    FOR Prof.Count = 1 TO 4 ' loop through all weapon proficiencies
        Outpt = MID$(STR$(Prof.Count), 2) + "> " + RTRIM$(WeaponTypeName(Prof.Count))
        CALL IO.O ' send choice output
    NEXT ' end weapon choices
    Outpt = "?" ' set user prompt
    IF UserRecord.ClassType = Cleric THEN ' compare class to cleric
        No.Input.Out = "1" ' set default
    ELSE ' compare class
        No.Input.Out = "3" ' set default
    END IF ' end compare class
    Line.Length = 1 ' reset line length
    CALL IO.I ' get user input
    Player.Prof = INT(VAL(Inpt)) ' convert to number
    IF UserRecord.ClassType = Cleric THEN ' compare class to cleric
        ' compare valid choices for cleric
        IF Player.Prof = 1 OR Player.Prof = 2 THEN
            EXIT DO ' exit weapon input loop
        END IF ' end compare valid chocies
    ELSE ' compare to non cleric
        IF Player.Prof >= 1 AND Player.Prof <= 4 THEN ' compare valid choices
            EXIT DO ' exit weapon input loop
        END IF ' end compare valid choices
    END IF ' end compare class type
LOOP ' end loop to accept weapon proficiency
UserRecord.Proficiency = Player.Prof ' store character weapon selection
FOR Weapon.Number = 1 TO 4 ' loop through user record weapon proficiencies
    CALL PutUserRecordWeapons(Weapon.Number, False) ' reset to zero
NEXT ' end loop through weapon proficiencies
' set user record selected weapon profciency
CALL PutUserRecordWeapons(Player.Prof, 10)
END SUB ' end routine to modify character weapon proficinecy

REM * routine to initialize character proficiency statistics.

SUB Init.Proficiency.Stats
Graphics.Off = False ' reset color
SELECT CASE UserRecord.Race ' compare player character race
    CASE 3 ' gnome race
        CALL PutUserRecordWeapons(GetUserRecordProficiency, 15) ' increment proficiency
        Outpt = "Gnomes weapon proficiency is raised to 15%" ' make message
        CALL IO.O ' send message
    CASE 6 ' half-elf race
        CALL PutUserRecordWeapons(4, GetUserRecordWeapons(4) + 5) ' increment proficiency
        Outpt = "Half-elves thrusting weapon proficiency is raised by 5%" ' message
        CALL IO.O ' send message
    CASE 7 ' half-orc race
        CALL PutUserRecordWeapons(3, GetUserRecordWeapons(3) + 5) ' increment proficiency
        Outpt = "Half-orcs sharp weapon proficiency is raised by 5%" ' message
        CALL IO.O ' send message
END SELECT ' end compare race
END SUB ' end routine to initialize proficiency statistics

REM * routine to initialize character race statistics.

SUB Init.Race.Stats
Graphics.Off = False ' reset color
SELECT CASE UserRecord.Race ' compare player character race
    CASE 1 ' human race
        CALL PutUserRecordStats(1, GetUserRecordStats(1) + 1) ' increment statistics
        Outpt = "Humans strength is raised one point!" ' make message
        CALL IO.O ' send message
    CASE 2 ' elf race
        CALL PutUserRecordStats(4, GetUserRecordStats(4) + 1) ' increment statistics
        Outpt = "Elves dexterity is raised by one point!" ' make message
        CALL IO.O ' send message
    CASE 4 ' dwarf race
        CALL PutUserRecordStats(2, GetUserRecordStats(2) + 1) ' increment statistics
        Outpt = "Dwarves intelligence is raised by one point!" ' make message
        CALL IO.O ' send message
    CASE 5 ' halfling race
        CALL PutUserRecordStats(3, GetUserRecordStats(3) + 1) ' increment statistics
        Outpt = "Halflings wisdom is raised by one point!" ' make message
        CALL IO.O ' send message
    CASE 8 ' ogre race
        CALL PutUserRecordStats(1, GetUserRecordStats(1) + 1) ' increment statistics
        CALL PutUserRecordStats(4, GetUserRecordStats(4) + 1) ' increment statistics
        Outpt = "Ogres strength and dexterity are raised by one point!" ' message
        CALL IO.O ' send output
END SELECT ' end compare race
UserRecord.Beauty = INT(RND * 15 + 5) ' reset ladies beauty
UserRecord.Glamour = INT(RND * 15 + 5) ' reset ladies glamour
END SUB ' end routine to initialize character race statistics

REM * routine to initialize some character statistics.

SUB Init.Stats
User.Echo = False ' reset preference
User.LineFeeds = False ' reset preference
User.LineLength = 80 ' reset preference
User.PageLength = 24 ' reset preference
User.Wordwrap = False ' reset preference
UserRecord.Room = 1 ' reset user record room number
UserRecord.Level = 1 ' reset user record character level
UserRecord.Experience = 64 ' reset experience
UserRecord.Gold = 2048 ' reset user record gold
UserRecord.Bank = False ' reset user record amount of gold in bank
UserRecord.Borrow = False ' reset user record amount of gold borrowed from bank
UserRecord.Brief = False ' reset user record brief mode
UserRecord.Echo = False ' reset user echo mode
UserRecord.Linefeeds = False ' reset user linefeed mode
UserRecord.Linelength = 80 ' reset user linelength
UserRecord.Pagelength = 24 ' reset pagelength
UserRecord.Wordwrap = False ' reset user word wrap
UserRecord.FatigueMax = Training.Stats(UserRecord.ClassType, 1) ' reset user
UserRecord.VitalityMax = Training.Stats(UserRecord.ClassType, 2) ' record maximum
UserRecord.MagicMax = Training.Stats(UserRecord.ClassType, 3) ' statistic
UserRecord.PsionicMax = Training.Stats(UserRecord.ClassType, 4) ' points
UserRecord.Fatigue = UserRecord.FatigueMax ' reset user
UserRecord.Vitality = UserRecord.VitalityMax ' record working
UserRecord.Magic = UserRecord.MagicMax ' statistic
UserRecord.Psionic = UserRecord.PsionicMax ' points
UserRecord.MaxCalls = False ' reset user record maximum calls
UserRecord.FromHour = False ' reset user record time restrictions
UserRecord.FromMin = False ' reset user record time restrictions
UserRecord.ToHour = False ' reset user record time restrictions
UserRecord.ToMin = False ' reset user record time restrictions
UserRecord.Flags = False ' reset user record flags variable
CALL Clear.Container(0, True) ' routine to clear the container structure
FOR Container.Item = 1 TO 3 ' loop through user record containers
    CALL PutUserRecordContainer(Container.Item) ' reset container record
NEXT ' end loop through user record
' initialize training rooms
CALL PutUserRecordTrainRoom(1, 36)
CALL PutUserRecordTrainRoom(2, 4)
CALL PutUserRecordTrainRoom(3, 123)
CALL PutUserRecordTrainRoom(4, 163)
CALL PutUserRecordTrainRoom(5, 162)
CALL PutUserRecordTrainRoom(6, 426)
CALL PutUserRecordTrainRoom(7, 421)
CALL PutUserRecordTrainRoom(8, 417)
CALL PutUserRecordTrainRoom(9, 1)
CALL PutUserRecordTrainRoom(10, 1)
END SUB ' end routine to initialize character statistics

REM * routine to display login character statistics.

SUB Display.Init.Stats
Graphics.Off = False ' reset color
Outpt = "Your character statistics are:" ' message
CALL IO.O ' send output
Graphics.Off = True ' reset color
Outpt = "Level:" + STR$(UserRecord.Level) ' message
Outpt = Outpt + SPACE$(25 - LEN(Outpt)) ' append blanks
Alignment.Type$ = Alignment.Name1(UserRecord.Align1 + 2) ' message
Alignment.Type$ = RTRIM$(Alignment.Type$) ' trim blanks
' first character uppercase
MID$(Alignment.Type$, 1, 1) = UCASE$(MID$(Alignment.Type$, 1, 1))
Outpt = Outpt + "Align1: " + Alignment.Type$ ' combine message
CALL IO.O ' send output
Outpt = "Gold: " + STR$(UserRecord.Gold) ' message
Outpt = Outpt + SPACE$(25 - LEN(Outpt)) ' append blanks
Alignment.Type$ = Alignment.Name2(UserRecord.Align2 + 2) ' message
Alignment.Type$ = RTRIM$(Alignment.Type$) ' trim blanks
' first character uppercase
MID$(Alignment.Type$, 1, 1) = UCASE$(MID$(Alignment.Type$, 1, 1))
Outpt = Outpt + "Align2: " + Alignment.Type$ ' combine message
CALL IO.O ' send output
Outpt = "Room: " + STR$(UserRecord.Room) ' message
Outpt = Outpt + SPACE$(25 - LEN(Outpt)) ' append blanks
Player.Prof = UserRecord.Proficiency ' get statistic
Weapon.Type$ = WeaponTypeName(Player.Prof) ' message
Weapon.Type$ = RTRIM$(Weapon.Type$) ' trim blanks
' first character uppercase
MID$(Weapon.Type$, 1, 1) = UCASE$(MID$(Weapon.Type$, 1, 1))
Outpt = Outpt + "Prof:   " + Weapon.Type$ ' combine message
Weapon.Proficiency$ = STR$(GetUserRecordWeapons(Player.Prof)) ' message
Outpt = Outpt + ">" + Weapon.Proficiency$ + "%" ' combine message
CALL IO.O ' send output
Outpt = "Exp:  " + STR$(UserRecord.Experience) ' message
Outpt = Outpt + SPACE$(25 - LEN(Outpt)) ' append blanks
Outpt = Outpt + LEFT$(Stat(1), 3) + ":   " + STR$(GetUserRecordStats(1)) ' combine
CALL IO.O ' send output
Outpt = "Fat:  " + STR$(UserRecord.Fatigue) ' message
Outpt = Outpt + SPACE$(25 - LEN(Outpt)) ' append blanks
Outpt = Outpt + LEFT$(Stat(2), 3) + ":   " + STR$(GetUserRecordStats(2)) ' combine
CALL IO.O ' send output
Outpt = "Vit:  " + STR$(UserRecord.Vitality) ' message
Outpt = Outpt + SPACE$(25 - LEN(Outpt)) ' append blanks
Outpt = Outpt + LEFT$(Stat(3), 3) + ":   " + STR$(GetUserRecordStats(3)) ' combine
CALL IO.O ' send output
Outpt = "Mag:  " + STR$(UserRecord.Magic) ' message
Outpt = Outpt + SPACE$(25 - LEN(Outpt)) ' append blanks
Outpt = Outpt + LEFT$(Stat(4), 3) + ":   " + STR$(GetUserRecordStats(4)) ' combine
CALL IO.O ' send output
Outpt = "Psi:  " + STR$(UserRecord.Psionic) ' message
Outpt = Outpt + SPACE$(25 - LEN(Outpt)) ' append blanks
Outpt = Outpt + LEFT$(Stat(5), 3) + ":   " + STR$(GetUserRecordStats(5)) ' combine
CALL IO.O ' send output
Outpt = "Race:  " + Race(UserRecord.Race) ' message
Outpt = Outpt + SPACE$(25 - LEN(Outpt)) ' append blanks
Outpt = Outpt + LEFT$(Stat(6), 3) + ":   " + STR$(GetUserRecordStats(6)) ' combine
CALL IO.O ' send output
Class.Type$ = UserRecord.ClassName ' message
CALL Decrypt(Class.Type$) ' decrypt string
' first character uppercase
MID$(Class.Type$, 1, 1) = UCASE$(MID$(Class.Type$, 1, 1))
Class.Type$ = LEFT$(Class.Type$, 15) ' truncate to right
Outpt = "Class: " + Class.Type$ ' message
Outpt = Outpt + SPACE$(25 - LEN(Outpt)) ' append blanks
Outpt = Outpt + LEFT$(Stat(7), 3) + ":   " + STR$(GetUserRecordStats(7)) ' combine
CALL IO.O ' send output
IF UserRecord.ClassType = Lady THEN ' compare class to lady
    Outpt = "Lady stats:" ' message
    CALL IO.O ' send output
    Outpt = "Beauty:" + STR$(UserRecord.Beauty) ' message
    Outpt = Outpt + SPACE$(25 - LEN(Outpt)) ' append blanks
    Outpt = Outpt + "Glamour:" + STR$(UserRecord.Glamour) ' combine
    CALL IO.O ' send output
END IF ' end compare class type
CALL More.Prompt ' pause to continue
END SUB ' end routine to display character login statistics

REM * routine to initialize some variables after login.
REM * output variables:
REM *   Time.Left - player time left in seconds from login time.

SUB Update.Login
Outpt = UserRecord.DateOn ' get user's last login date
CALL Decrypt(Outpt) ' decrypt date
IF Outpt <> DATE$ THEN ' compare last login date to today
    UserRecord.NumCalls = False ' reset user's number of calls logged in per day
END IF ' end compare last login date
IF UserRecord.ClassType <= Lady THEN ' compare user to non DM status
    Calls.Exceeded = False ' set flag to maximum calls exceeded
    UserRecord.NumCalls = UserRecord.NumCalls + 1 ' increment user's max calls
    IF UserRecord.MaxCalls > False THEN ' compare maximum calls
        IF UserRecord.NumCalls > UserRecord.MaxCalls THEN ' compare calls
            Calls.Exceeded = True ' set maximum call flag
        END IF ' end compare calls to maximum calls
    ELSE ' end compare maximum calls
        IF UserRecord.NumCalls > 5 THEN ' compare maximum calls to default
            Calls.Exceeded = True ' set maximum call flag
        END IF ' end compare maximum default calls
    END IF ' end compare maximum calls
    IF Calls.Exceeded THEN ' check user has exceeded maximum calls
        CALL Share.Record(UserFile, User.Index) ' put user record
        CALL Hang.Up(5) ' routine to terminate program w/ message
        EXIT SUB ' exit routine
    END IF ' end check maximum calls
END IF ' end non DM status
CALL Restricted.Login ' check user is restricted to a time to login
IF UserRecord.ClassType < AsstDM THEN ' compare user is non DM status
    UserRecord.Invisible = False ' reset user invisibility
END IF ' end compare non DM status
SELECT CASE UserRecord.ClassType ' compare the user class type
    CASE IS >= AsstDM ' user is DM/Asst. DM
        Time.Left = 3600! ' user gets 60 minutes
    CASE ELSE ' user is non DM
        SELECT CASE UserRecord.Level ' select by user level
            CASE IS < 2 ' user level is less than two
                Time.Left = 900! ' user gets 15 mminutes
            CASE ELSE ' user level is two or more
                Time.Left = 1800! ' user gets 30 minutes
        END SELECT ' end user level
END SELECT ' end user DM status
IF Time.Left > Door.Time THEN ' compare user's time left to door file time left
    IF Door.Time > False THEN ' compare door file time
        Time.Left = Door.Time ' reset time left to door time left
    END IF ' end compare door file time
END IF ' end compare user time left
Two.Minutes.Left = False ' reset two minutes left message flag
User.Echo = UserRecord.Echo ' store user preference
User.LineFeeds = UserRecord.Linefeeds ' store user preference
User.LineLength = UserRecord.Linelength ' store user preference
User.PageLength = UserRecord.Pagelength ' store user preference
User.Wordwrap = UserRecord.Wordwrap ' store user preference
END SUB ' end routine to initialize some login variables

REM * routine to check user is restricted to specific login times.
REM * work variables:
REM *   Restrict.Start! - time in seconds to restrict logon.
REM *   Restrict.End! - time in seconds to restrict logon.

SUB Restricted.Login
' calculate time restrictions
Restrict.Start! = CSNG(UserRecord.FromHour * 3600! + UserRecord.FromMin * 60!)
Restrict.End! = CSNG(UserRecord.ToHour * 3600! + UserRecord.ToMin * 60!)
' compare any time restriction
IF Restrict.Start! > False OR Restrict.End! > False THEN
    IF TIMER < Restrict.Start! OR TIMER > Restrict.End! THEN ' compare time to now
        CALL Hang.Up(6) ' routine to terminate program w/ message
    END IF ' end compare time
END IF ' end compare time restriction
END SUB ' end routine to check restricted time login

REM * routine to intialize some new user variables in user file record.

SUB Init.Newuser
UserRecord.NumCalls = False ' reset maximum calls made today
UserRecord.ClassType = False ' reset class type
Outpt = Player.CodeName ' store codename
CALL Valid(Outpt, 30) ' validate codename
CALL Encrypt(Outpt, True) ' encrypt codename
UserRecord.CodeName = Outpt ' restore codename
Outpt = Player.PassWord ' store password
CALL Valid(Outpt, 20) ' validate password
CALL Encrypt(Outpt, False) ' encrypt password
UserRecord.PassWord = Outpt ' restore password
Outpt = Deleted$ ' set deleted
CALL Valid(Outpt, 20) ' validate deleted
CALL Encrypt(Outpt, True) ' encrypt deleted
UserRecord.ClassName = Outpt ' reset classname
Outpt = DATE$ ' store current date
CALL Valid(Outpt, 10) ' validate date
CALL Encrypt(Outpt, True) ' encrypt date
UserRecord.DateOn = Outpt ' reset date
UserRecord.MaxCalls = False ' reset maximum calls made
UserRecord.FromHour = False ' reset time restrictions
UserRecord.FromMin = False
UserRecord.ToHour = False
UserRecord.ToMin = False
END SUB ' end routine to intialize new user variables

REM * routine to initialize some user variables.

SUB Login.User
CALL Read.Room.Record(1!) ' get room record #1
Resurrection.Room = RoomRecord.MonsterClass ' get resurrection room number
Number.Monsters = False ' counter of monsters currently in the room
Monsters.Killed = False ' counter of monster killed by player during session
' allocate room monster arrays
REDIM MonsterArray(1 TO 20) AS MonsterType
REDIM MonsterIndex(1 TO 20) AS INTEGER
Max.Spells = LOF(SpellFile) / LEN(SpellRecord) ' compute number of spells in file
IF Max.Spells = False THEN ' compare empty file
    Max.Spells = 1 ' set number to at least one
    CALL Share.Record(SpellFile, 1) ' put default spell record
END IF ' end compare file
' check bounds of spell file
IF Max.Spells > 1024 THEN ' check bounds
    Max.Spells = 1024 ' reste maximum
END IF ' end check bounds
' make string of zeros length of spell file
Learned.Spells = STRING$(Max.Spells, "0")
IF UserRecord.Race <= False THEN ' check user race
    UserRecord.Race = 1 ' reset to one
END IF ' end check race
CALL Share.Record(UserFile, User.Index) ' put the user record
Room = UserRecord.Room ' store the room number
IF UserRecord.Level = False THEN ' check user level
    Outpt = "You are level zero. You can use the train command once free."
    CALL IO.O ' send output message
END IF ' end compare level
CALL Bank.Interest ' calculate bank interest for balance and loan
CALL Check.Mail ' routine to display number of new messages to player
Weapon1 = False ' reset working game weapon, shield, armor, and ring variables
Weapon2 = False ' reset variable
Weapon3 = False ' reset variable
Weapon4 = False ' reset variable
Weapon5 = False ' reset variable
Weapon6 = False ' reset variable
Weapon7 = False ' reset variable
Weapon8 = False ' reset variable
Weapon9 = False ' reset variable
Weapon10 = False ' reset variable
Weapon11 = False ' reset variable
CALL Get.User.Record ' read the user record
CALL Status.Line(1) ' initialize the console status lines
FunctionBuffer = Nul ' reset function key buffer
END SUB ' end routine to initialize some user variables

REM * routine to display treasure item.
REM * input variables:
REM *   Index.Number - treasure record number.
REM *   TypeNumber - room/inventory flag.

SUB Show.Treasure
IF TypeNumber THEN ' compare treasure in room
    Prefix1 = "It's " ' format prefix
ELSE ' compare treasure in player inventory
    Prefix1 = "You are carrying " ' format prefix
END IF ' end compare treasure
Graphics.Off = True ' reset color
IF TreasureRecord.Scroll THEN ' compare treasure to scroll
    IF TreasureRecord.Spell THEN ' check scroll spell number
        CALL Read.Record(SpellFile, TreasureRecord.Spell) ' get spell record
        Inpt = SpellRecord.Chant ' store spell chant
        Inpt = RTRIM$(Inpt) ' trim chant
        Inpt = LCASE$(Inpt) ' trim chant
        Outpt = "It reads: '" + Inpt + "'." ' display scroll chant
        CALL IO.O ' send message
        Outpt = "It disintegrated!" ' scroll vanished message
        CALL IO.O ' send output
        IF TypeNumber = False THEN ' compare treasure in room
            ' remove scroll from inventory
            CALL Discard.Inventory(Array.Number, True)
        ELSE ' compare treasure
            CALL Discard.Room.Treasure(Array.Number) ' remove scroll from room
        END IF ' end compare treasure
    END IF ' end compare scroll spell number
    EXIT SUB ' exit routine
END IF ' end compare treasure is scroll
Outpt = Prefix1 + Outpts ' format treasure name description
IF TreasureRecord.Keyed THEN ' append key number to treasure name
    Outpt = Outpt + "(#" + RIGHT$(STR$(TreasureRecord.Keyed + 100000!), 5) + ")"
END IF ' end compare treasure key number
IF TreasureRecord.Plus THEN ' append plus number to treasure name
    Outpt = Outpt + "(+" + MID$(STR$(ABS(TreasureRecord.Plus)), 2) + ")"
END IF ' end compare treasure plus
IF TreasureRecord.Spell THEN ' append spell plus to treasure name
    CALL Read.Record(SpellFile, TreasureRecord.Spell) ' get spell record
    Outpt = Outpt + "(+" + MID$(STR$(SpellRecord.Level), 2) + ")"
END IF ' end compare treasure spell plus
IF TreasureRecord.LightType THEN ' compare treasure to a light
    IF Charges.Number < False THEN ' check treasure is also lit
        Outpt = Outpt + "[lit]" ' append to treasure name
    END IF ' end check lit treasure
END IF ' end compare treasure to light
IF TreasureRecord.Invisible THEN ' compare treasure is invisible
    Outpt = Outpt + "[inv]" ' append to treasure name
ELSE ' compare treasure
    IF TypeNumber = 1 THEN ' check treasure is in room
        ' verify treasure in
        IF GetRoomRecordFlags(Array.Number) = Hidden.Object THEN
            Outpt = Outpt + "[inv]" ' room is invisible, append to name
        END IF ' end verify treasure in room was hidden
    END IF ' end check treasure in room
END IF ' end compare treasure is invisible
Outpt = Outpt + "." ' append message
CALL IO.O ' display treasure name message
IF TreasureRecord.Proficiency THEN ' compare treasure proficiency
    Outpt = WeaponTypeName(TreasureRecord.Proficiency) ' get proficiency
    Outpt = RTRIM$(Outpt) ' name and trim
    Outpt = "This is a " + Outpt + " weapon." ' make weapon type message
    CALL IO.O ' display weapon type message
END IF ' end compare treasure proficiency
IF Last.Command.Number = Identify.Command THEN ' check identify command used
    Outpt = "It's worth" + STR$(TreasureRecord.Gold) + " gold peices."
    CALL IO.O ' display treasure item gold value
    Outpt = "It weighs" + STR$(TreasureRecord.Weight) + " pounds."
    CALL IO.O ' display weight of item
    IF TreasureRecord.RingType THEN ' compare treasure ring type
        SELECT CASE TreasureRecord.RingType ' determine ring type
            CASE 1 ' ring type
                Outpt = "protection from poison." ' ring type message
            CASE 2 ' ring type
                Outpt = "protection from level drain." ' ring type message
            CASE 3 ' ring type
                Outpt = "protection from spells." ' ring type message
        END SELECT ' end dtermine ring type
        Outpt = "Its ring spell is " + Outpt ' make ring type message
        CALL IO.O ' send ring type message
    END IF ' end compare treasure to ring
    IF TreasureRecord.Spell THEN ' compare treasure spell type
        CALL Read.Record(SpellFile, TreasureRecord.Spell) ' get treasure spell
        Outpt = "Its magical spell is " + RTRIM$(SpellRecord.SpellName) + "."
        CALL IO.O ' display name of treasure spell
    END IF ' end compare treasure spell type
    ' compare treasure is loaded
    IF TreasureRecord.Loadable OR TreasureRecord.Launchable THEN
        IF Charges.Number <= False THEN ' compare treasure charges
            Outpt = "It's not loaded." ' display message
        ELSE ' compare treasure charges
            Outpt = "It's loaded with" + STR$(Charges.Number) + " charges." ' message
        END IF ' end compare loaded treasure charges
        CALL IO.O ' send message of charges in loaded treasure
    ELSE ' compare treasure item
        IF TreasureRecord.LightType THEN ' compare treasure is a light
            IF Charges.Number < False THEN ' compare light charges (is negative)
                Outpt = "It's fueled with" + STR$(ABS(Charges.Number)) + " charges."
                CALL IO.O ' send message of charges in light
            END IF ' end compare light charges
        ELSE ' compare other treasure plus
            IF TreasureRecord.RingType OR TreasureRecord.Spell OR TreasureRecord.Plus THEN ' treasure has charges
                IF Charges.Number <= False THEN ' compare charges
                    Outpt = "It's empty of charges." ' message
                ELSE ' compare charges
                    ' message of charges
                    Outpt = "It has" + STR$(Charges.Number) + " charges."
                END IF ' end compare charges
                CALL IO.O ' send message of charges
            END IF ' end compare treasure charges
        END IF ' end compare treasure plus
    END IF ' end compare treasure
    ' compare treasure is ammunition
    IF TreasureRecord.Ammunition OR TreasureRecord.LaunchAmmo THEN
        Outpt = "It's ammunition." ' treasure nessage
        CALL IO.O ' send message
    END IF ' end compare treasure
    IF TreasureRecord.Vehicle THEN ' compare treasure to vehicle
        Outpt = "It's a vehicle." ' display message
        CALL IO.O ' message
        Outpt = "It has" + STR$(Charges.Number) + " charges." ' display message
        CALL IO.O ' message
    END IF ' end compare treasure
    IF TreasureRecord.Potion THEN ' compare treasure to potion
        Outpt = "It's a potion." ' make message
        CALL IO.O ' send message
    END IF ' end compare to potion
    IF TreasureRecord.Edible THEN ' compare treasure to food
        Outpt = "It's edible." ' make message
        CALL IO.O ' send message
    END IF ' end compare to food
END IF ' end identify command
Graphics.Off = False ' reset color
END SUB ' end routine to display an itemof treasure

REM * routine to display object information.
REM * input variables:
REM *   Index.Number - object record number.
REM *   TypeNumber - object is in room/inventory.

SUB Show.Object
Hidden.Flag = False ' reset flag
IF TypeNumber THEN ' determine object in room
    Prefix1 = "It's " ' make prefix
    IF GetRoomRecordHiddenObj(Array.Number) THEN ' check object hidden
        Hidden.Flag = True ' reset flag
    END IF ' end check object
ELSE ' object in inventory
    Prefix1 = "You are carrying " ' make prefix
END IF ' end determine in room
Graphics.Off = True ' reset color
Outpt = Prefix1 + Outpts ' make message with object name
IF ObjectRecord.DoorLock > 1 THEN ' compare object is locked
    Outpt = Outpt + "[locked]" ' append to object name
END IF ' end compare locked object
IF ObjectRecord.DoorLock = 1 THEN ' compare object is unlocked
    IF ObjectRecord.Closed THEN ' compare object is closed
        Outpt = Outpt + "[closed]" ' append to object name
    END IF ' end compare closed object
END IF ' end compare object lock
IF ObjectRecord.Invisible OR Hidden.Flag THEN ' compare object is invisible
    Outpt = Outpt + "[inv]" ' append to object name
END IF ' end compare object invisible
IF ObjectRecord.Keyed THEN ' compare object key, append number to name
    Outpt = Outpt + "(#" + RIGHT$(STR$(ObjectRecord.Keyed + 100000!), 5) + ")"
END IF ' end compare object key number
Outpt = Outpt + "." ' append message
CALL IO.O ' display message with object name
Outpt = ObjectRecord.LongDesc ' store object additional description
Outpt = RTRIM$(Outpt) ' trim description
IF Outpt <> Nul THEN ' compare length of description
    CALL IO.O ' display additional object description
END IF ' end compare object description length
Graphics.Off = False ' reset color
END SUB ' end routine to display object information

REM * routine to display information on a monster.
REM * input variables:
REM *   Monster.Number - number of monster array.

SUB Show.Monster
Graphics.Off = True ' reset color
CALL The.Or.An ' routine for monster name prefix (a, an, the)
Level = MonsterArray(Monster.Number).Level ' store monster level
Outpt = "It's " + Prefix1 + Outpts ' make message of monster name
' append monster level (range of player's level capable to kill monster)
Outpt = Outpt + "(level" + STR$((Level - 1) * 2 + 1) + " to" + STR$(Level * 2) + ")."
CALL IO.O ' send message with monster name and level range
IF Last.Command.Number = Identify.Command THEN ' compare identify command
    Gold.Points# = MonsterArray(Monster.Number).Gold ' store monster gold
    IF Gold.Points# <= False THEN ' compare monster gold
        Gold.Points# = 10 ' set to minimum
    END IF ' end compare monster gold
    Outpt = "It has" + STR$(MonsterArray(Monster.Number).Hits) + " hits," + STR$(MonsterArray(Monster.Number).Experience) + " experience, and" + STR$(Gold.Points#) + " gold."
    CALL IO.O ' display message of monster gold
    Outpt = "It carries the following treasure:" ' make message
    CALL IO.O ' send message of treasure carried by monster
    Inventory.Count = False ' reset number of monster inventory items displayed
    FOR Array.Count = 1 TO 5 ' loop through all monster inventory
        ' get treasure
        Treasure.Number = GetMonsterRecordTreasure(Array.Count)
        ' number and check range in treasure file
        IF Treasure.Number > False AND Treasure.Number <= LOF(TreasureFile) / LEN(TreasureRecord) THEN
            Carriage.Return = True ' flag to disable return/linefeed
            CALL IO.O ' send output of previous item
            CALL Read.Record(TreasureFile, Treasure.Number) 'get treasure record
            Outpts = TreasureRecord.TreasureName ' store treasure name
            Outpt = RTRIM$(Outpts) + ", " ' trim name, append comma
            Inventory.Count = Inventory.Count + 1 ' increment items displayed flag
            IF Inventory.Count = 1 THEN ' compare item to first displayed
                MID$(Outpt, 1, 1) = UCASE$(MID$(Outpt, 1, 1)) ' uppercase first item
            END IF ' end compare first item
        END IF ' end check treasure file range
    NEXT ' end loop through monster inventory
    IF Inventory.Count = False THEN ' check if any items displayed
        Outpt = "Nothing at all." ' make message for none
    ELSE ' check items displayed
        Outpt = LEFT$(Outpt, LEN(Outpt) - 2) + "." ' trim comma, append period
        IF Inventory.Count > 1 THEN ' check more than one item displayed
            Outpt = "and " + Outpt ' append to last item
        END IF ' end check items
    END IF ' end check item
    CALL IO.O ' send output for last item
    ' compare monster spell ability
    Spell.Number = MonsterArray(Monster.Number).Spell
    ' check spell
    IF Spell.Number > False AND Spell.Number <= LOF(SpellFile) / LEN(SpellRecord) THEN
        CALL Read.Record(SpellFile, Spell.Number) ' range and get spell record
        Outpt = "It can cast " + RTRIM$(SpellRecord.SpellName) + " spells!"
        CALL IO.O ' send message of spell name monster can cast
    END IF ' end compare monster spell
    IF MonsterArray(Monster.Number).Poison THEN ' compare monster poisonous
        Outpt = "It can poison!" ' make message
        CALL IO.O ' send output message
    END IF ' end compare monster poisonous
    IF MonsterArray(Monster.Number).LevelDrain THEN ' compare monster undead
        Outpt = "It can drain levels!" ' make message
        CALL IO.O ' send output message
    END IF ' end compare monster undead
    IF MonsterArray(Monster.Number).Psionic THEN ' compare monster astral
        Outpt = "It can cast psi spells!" ' make message
        CALL IO.O ' send output message
    END IF ' end compare monster astral
END IF ' end check identify command used
Graphics.Off = False ' reset color
END SUB ' end routine to display monster information

REM * routine to determine validity of room number.
REM * input variables:
REM *   Room - contains room number to check.

SUB Check.Next.Room
DO ' loop until room is valid, room is created, or nondescriptive hangup
    IF Room > False AND Room <= LOF(RoomFile) / LEN(RoomRecord) THEN ' check room range
        CALL Read.Room.Record(Room) ' valid range, get room record
        EXIT SUB ' exit routine
    END IF ' end check valid range
    IF Room > LOF(RoomFile) / LEN(RoomRecord) THEN ' compare room number range
        IF NOT Normal.User THEN ' check non DM status
            CALL Add.Room(False, Room.Created) ' routine to create new room
            IF Room.Created THEN ' return variable indicates new room created
                EXIT SUB ' exit routine
            END IF ' end create new room
        END IF ' end check normal user
    END IF ' end compare room number range
    ' otherwise, any room number out of range will be changed to room 1, or
    ' changed to the resurrection room number.
    IF LOF(RoomFile) / LEN(RoomRecord) >= 1 THEN ' check for room
        Graphics.Off = False ' reset color
        Outpt = "Nondescriptive room number" + STR$(Room) + "!" ' make error message
        CALL IO.O ' display room number error message
        Room = 1 ' reset room number to resurrection room, continue loop
    ELSE ' room file is invalid, room file length is zero
        Graphics.Off = False ' reset color
        Room = 1 ' reset room number
        CALL Clear.Room(1) ' add first room
        EXIT SUB ' exit routine
    END IF ' end check room file length
LOOP ' end loop until valid room number
END SUB ' end routine to check room number validity

REM * routine to initialize some room variables, check next room number, and
REM * display next room description.
REM * input variables:
REM *   Room - room number to move to.
REM * output variables:
REM *   Room.Rust.Rate - number of prompts to check weapon rusting.
REM *   Room.Steal.Rate - number of prompts to check monster stealing.
REM *   Room.Monster.Rate - number of prompts to check monster encounter.
REM *   Room.Health.Rate - number of prompts to check health increases.

SUB Display.Room
CALL Check.Next.Room ' routine to verify next room number
Room.Rust.Rate = False ' store room rust rate
Room.Steal.Rate = False ' store room steal rate
Room.Monster.Rate = 6 ' store default room encounter rate
Room.Health.Rate = 6 ' store default room health rate
Room.Action = RoomRecord.Action
IF Room.Action > False AND Room.Action <= LOF(ActionFile) / LEN(ActionRecord) THEN
    CALL Read.Record(ActionFile, Room.Action)
    IF ActionRecord.RustRate > False THEN ' check room record action rust rate
        Room.Rust.Rate = ActionRecord.RustRate ' store room rust rate
    END IF ' end check action
    IF ActionRecord.StealRate > False THEN ' check room record action steal rate
        Room.Steal.Rate = ActionRecord.StealRate ' store room steal rate
    END IF ' end check action
    ' check room record action encounter rate
    IF ActionRecord.EncounterRate THEN
        ' store action encounter rate
        Room.Monster.Rate = ActionRecord.EncounterRate
    END IF ' end check action
    IF ActionRecord.HealthRate THEN ' check room record action health rate
        Room.Health.Rate = ActionRecord.HealthRate ' store action health rate
    END IF ' end check action
END IF
CALL Show.Room ' routine to display room
END SUB ' end routine to process next room

REM * routine to determine if a room is unlit.
REM * return variables:
REM *   Lit.Room - true if room is unlit.

SUB Check.Lit.Room (Lit.Room)
Lit.Room = False ' room is lit by default
CALL Read.Room.Record(Room) ' get room record
Action.Number = RoomRecord.Action ' store action number
' check action number
IF Action.Number < 1 OR Action.Number > LOF(ActionFile) / LEN(ActionRecord) THEN
    Lit.Room = False ' room is lit
    EXIT SUB ' exit lit check routine
END IF ' end check action number
CALL Read.Record(ActionFile, Action.Number) ' get action record number
IF ActionRecord.Attribute1 = LitRoom THEN ' compare lit flag
    Lit.Room = False ' room is lit
    EXIT SUB ' exit check lit routine
END IF ' end check lit flag
Lit.Room = True ' flag for unlit room
FOR Array.Index = 1 TO 20 ' loop through all user inventory
    Treasure.Number = GetUserRecordInv(Array.Index) ' get inventory number
    IF Treasure.Number THEN ' compare user treasure number
        CALL Read.Record(TreasureFile, Treasure.Number) ' get treasure record
        IF TreasureRecord.LightType THEN ' check treasure item is a light
            ' check light is charged/lit
            IF GetUserRecordCharges(Array.Index) < False THEN
                Lit.Room = False ' set flag for lit room
                EXIT SUB ' exit routine
            END IF ' end check charged light
        END IF ' end check treasure is a light
    END IF ' end compare treasure number
NEXT ' end loop through user inventory
FOR Array.Index = 1 TO 20 ' loop through all treasure in room
    ' get room treasure number
    Treasure.Number = GetRoomRecordTreasure(Array.Index)
    IF Treasure.Number THEN ' compare treasure number
        CALL Read.Record(TreasureFile, Treasure.Number) ' get treasure record
        IF TreasureRecord.LightType THEN ' compare treasure is a light
            ' compare light is charged
            IF GetRoomRecordTreCharges(Array.Index) < False THEN
                Lit.Room = False ' set flag for lit room
                EXIT SUB ' exit routine
            END IF ' end compare charged light
        END IF ' end compare treasure is a light
    END IF ' end compare treasure number
NEXT ' end loop through room treasure
FOR Array.Index = 1 TO 20 ' loop through all room objects
    IF GetRoomRecordObject(Array.Index) THEN ' compare room object number
        CALL Read.Record(ObjectFile, GetRoomRecordObject(Array.Index)) 'get record
        IF ObjectRecord.LightRoom THEN ' check object is a light
            IF ObjectRecord.LightTime = False THEN ' object lights at any time
                Lit.Room = False ' set flag for lit room
                EXIT SUB ' exit routine
            ELSE ' light has light time restriction
                ' calculate seconds light from/to
                Start.Time! = CSNG(ObjectRecord.FromHour * 3600! + ObjectRecord.FromMin * 60!)
                End.Time! = CSNG(ObjectRecord.ToHour * 3600! + ObjectRecord.ToMin * 60!)
                ' check valid light time
                IF Start.Time! > False OR End.Time! > False THEN
                    ' compare times
                    IF TIMER >= Start.Time! AND TIMER <= End.Time! THEN
                        Lit.Room = False ' set room lit flag
                        EXIT SUB ' exit routine
                    END IF ' end compare times
                END IF ' end check valid light time
            END IF ' end check object light type
        END IF ' end check object is a light
    END IF ' end compare object number
NEXT ' end loop through room objects
FOR Array.Index = 1 TO 20 ' loop through all user object inventory
    IF GetUserRecordObject(Array.Index) THEN ' compare user object number
        CALL Read.Record(ObjectFile, GetUserRecordObject(Array.Index)) 'get object
        IF ObjectRecord.LightRoom THEN ' compare object is a light
            IF ObjectRecord.LightTime = False THEN ' check object lights any time
                Lit.Room = False ' set flag for lit room
                EXIT SUB ' exit routine
            ELSE ' compare object light time restriction
                ' calculate seconds light from/to
                Start.Time! = CSNG(ObjectRecord.FromHour * 3600! + ObjectRecord.FromMin * 60!)
                End.Time! = CSNG(ObjectRecord.ToHour * 3600! + ObjectRecord.ToMin * 60!)
                ' check valid light time
                IF Start.Time! > False OR End.Time! > False THEN
                    ' compare times
                    IF TIMER >= Start.Time! AND TIMER <= End.Time! THEN
                        Lit.Room = False ' set lit room flag
                        EXIT SUB ' exit routine
                    END IF ' end compare times
                END IF ' end check valid light times
            END IF ' end compare object light type
        END IF ' end compare object is a light
    END IF ' end compare object number
NEXT ' end loop through user objects
END SUB ' end routine to determine lit room

REM * routine to display all the player character statistics.

SUB Display.Stats
Graphics.Off = True ' reset color
Outpt = UserRecord.CodeName ' get user codename
CALL Decrypt(Outpt) ' decrypt codename
Outpt = RTRIM$(Outpt) ' trim codename
Outpt = LCASE$(Outpt) ' set codename to lowercase
MID$(Outpt, 1, 1) = UCASE$(MID$(Outpt, 1, 1)) ' uppercase first letter
Outpt = "Information: " + Outpt + ". " + clock$ + "." ' make information message
CALL IO.O ' send message
CALL Show.Align ' routine to display player alignment
CALL More.Prompt ' pause display
IF No THEN ' check more prompt response
    EXIT SUB ' exit info display
END IF ' end check more prompt
CALL Show.Health ' routine to display player statistics
CALL More.Prompt ' pause display
IF No THEN ' check more prompt response
    EXIT SUB ' exit info display
END IF ' end check more prompt
CALL Display.Info ' routine to display additional player information
CALL More.Prompt ' pause display
IF No THEN ' check more prompt response
    EXIT SUB ' exit info display
END IF ' end check more prompt
CALL Display.Inventory ' routine to display player inventory
CALL More.Prompt ' pause display
IF No THEN ' check more prompt response
    EXIT SUB ' exit info display
END IF ' end check more prompt
CALL Display.Experience ' routine to display player experience and gold
CALL More.Prompt ' pause display
IF No THEN ' check more prompt response
    EXIT SUB ' exit info display
END IF ' end check more prompt
END SUB ' end routine to display all player character statistics

REM * routine to display player character experience, and gold.
REM * routine notes:
REM *    although gold and experience required to reach the next level double
REM *    each player level, the experience and gold required for players over
REM *    level 10 only increase by 10,000 points per level over 10.

SUB Display.Experience
Var1$ = FormatString$(UserRecord.Gold)
Var2$ = FormatString$(UserRecord.Experience)
Outpt = "You have " + Var1$ + " gold and " + Var2$ + " experience." ' make gold/experience message
Graphics.Off = True ' reset color
CALL IO.O ' send output message
Level = UserRecord.Level ' store player level
IF Level > False AND Level < MaxInt THEN ' compare level maximum
    Level = Level + 1 ' increment next level needed
    ' routine to calculate gold required for next level
    CALL Gold(Gold.Required#)
    ' routine to calculate experience needed
    CALL Experience(Exp.Required#)
    Var1$ = FormatString$(Gold.Required#)
    Var2$ = FormatString$(Exp.Required#)
    Outpt = "You need " + Var1$ + " gold and " + Var2$ + " experience to reach level" + STR$(Level) + "."
    CALL IO.O ' send output message
    EXIT SUB
END IF
Outpt = "There is no gold or experience at your level."
CALL IO.O
END SUB

REM * routine to display player characteristics, and weapons, shields, armor,
REM * and rings being held/worn.

SUB Display.Info
Graphics.Off = True ' reset color
IF Sysop THEN ' verify user is a sysop
    Outpt = "You are a Sysop!" ' make message
    CALL IO.O ' send message
END IF ' end verify sysop
IF Dungeon.Master THEN ' verify user is a DM
    Outpt = "You are a Dungeon Master!" ' make message
    CALL IO.O ' send message
END IF ' end verify DM
IF Dungeon.Master.Assistant THEN ' verify user is an Asst. DM
    Outpt = "You are an Assistant Dungeon Master!" ' make message
    CALL IO.O ' send message
END IF ' end verify Asst. DM
IF Town.Mayor THEN ' verify user is the mayor
    Outpt = "You are the Town Mayor!" ' make message
    CALL IO.O ' send message
END IF ' end verify mayor
IF Governor THEN ' verify user is governor
    Outpt = "You are the Governor!" ' make message
    CALL IO.O ' send message
END IF ' end verify governor
IF Guild.Master THEN ' verify user is guild master
    Outpt = "You are the Guild Master!" ' make message
    CALL IO.O ' send message
END IF ' end verify guild master
IF UserRecord.Invisible OR Invisible THEN ' check invisibility
    Outpt = "You are invisible!" ' make message
    CALL IO.O ' send message
END IF ' end check invisibility
IF UserRecord.Poison THEN ' check poisoned
    Outpt = "You are poisoned!" ' make message
    CALL IO.O ' send message
END IF ' end check poisoned
IF Weapon4 = False THEN ' check wearing armor
    IF Weapon7 = False THEN ' check wearing ring
        Outpt = "You are wearing nothing." ' make message
        CALL IO.O ' send message
    END IF ' end check ring
END IF ' end check armor
IF Weapon5 = False THEN ' check holding weapon
    IF Weapon6 = False THEN ' check holding shield
        Outpt = "You are holding nothing." ' make message
        CALL IO.O ' send message
    END IF ' end check shield
END IF ' end check weapon
Outpt = Nul ' clear display string
IF Weapon4 OR Weapon7 THEN ' check either armor or ring being worn
    Outpt = "You are wearing " ' initialize display string
    IF Weapon4 THEN ' check armor being worn
        CALL Read.Record(TreasureFile, ABS(GetUserRecordInv(Weapon4))) ' get armor
        Outpt = Outpt + RTRIM$(TreasureRecord.TreasureName) ' append name
        Outpt = Outpt + "(+" + MID$(STR$(Weapon1), 2) + ")" ' append plus
    END IF ' end check armor worn
END IF ' end check either being worn
IF Weapon7 THEN ' check ring being worn
    CALL Read.Record(TreasureFile, ABS(GetUserRecordInv(Weapon7))) ' ring treasure
    IF Weapon4 THEN ' check armor worn again
        Outpt = Outpt + " and " ' append
    ELSE ' armor not worn
        Outpt = "You are wearing "
    END IF ' end check armor worn
    Outpt = Outpt + RTRIM$(TreasureRecord.TreasureName) ' append name
    Outpt = Outpt + "(+" + MID$(STR$(Weapon11), 2) + ")." ' append plus
    CALL IO.O ' items being worn
ELSE ' end check ring worn
    IF Weapon4 THEN ' check armor worn, ring not
        Outpt = Outpt + "." ' append period
        CALL IO.O ' display only armor worn
    END IF ' end check armor, ring
END IF ' end check ring worn
Outpt = Nul ' clear display string
IF Weapon5 OR Weapon6 THEN ' check either weapon or shield being held
    Outpt = "You are holding " ' initialize display string
    IF Weapon6 THEN ' check weapon being held
        CALL Read.Record(TreasureFile, ABS(GetUserRecordInv(Weapon6))) ' get weapon
        Outpt = Outpt + RTRIM$(TreasureRecord.TreasureName) ' append name
        Outpt = Outpt + "(+" + MID$(STR$(Weapon2), 2) + ")" ' append plus
    END IF ' end check weapon held
END IF ' end check either being held
IF Weapon5 THEN ' check shield being held
    CALL Read.Record(TreasureFile, ABS(GetUserRecordInv(Weapon5))) ' shield record
    IF Weapon6 THEN ' check weapon held again
        Outpt = Outpt + " and " ' append
    ELSE ' weapon not held
        Outpt = "You are holding "
    END IF ' end check weapon held
    Outpt = Outpt + RTRIM$(TreasureRecord.TreasureName) ' append name
    Outpt = Outpt + "(+" + MID$(STR$(Weapon3), 2) + ")." ' append plus
    CALL IO.O ' items being held
ELSE ' end check shield held
    IF Weapon6 THEN ' check weapon held, not shield
        Outpt = Outpt + "." ' append period
        CALL IO.O ' display only weapon held
    END IF ' end check weapon, shield
END IF ' end check shield held
END SUB ' end routine to display player characteristics

REM * routine to display player character alignment and health statistics.

SUB Display.Health
Graphics.Off = True ' reset color
CALL Show.Align ' routine to display alignment
CALL Show.Health ' routine to display health statistics
END SUB ' end routine to display alignment/health

REM * routine to display health statistics, and weapon, shield, armor plus.

SUB Show.Health
Graphics.Off = True ' reset color
' display vital health statistics in percentage form
Outpt = "Vitals: " ' make output message
Var1# = CDBL(UserRecord.FatigueMax) ' calculate total hits
Var1# = Var1# + CDBL(UserRecord.VitalityMax) ' calculate total hits
Var2# = CDBL(UserRecord.Fatigue) ' calculate hits remaining
Var2# = Var2# + CDBL(UserRecord.Vitality) ' calculate hits remaining
Outpt = Outpt + "Body" ' append health message
IF Var1# = 0# THEN ' check divide by zero
    Temp# = 0# ' reset health hits
ELSE ' check divide
    Temp# = ((Var2# / Var1#) * .40#) * 100# ' compute health percentage
    Temp# = INT(Temp#) ' compute health integer
    IF Temp# < 1.0# THEN ' check percentage
        Temp# = 1.0# ' reset percentage
    END IF ' end check percentage
END IF ' end check divide by zero
IF Temp# < 0# THEN ' check overflow
    Temp# = 0# ' reset percentage
END IF ' end check overflow
Outpt = Outpt + STR$(Temp#) + "%" ' append health percentage
Outpt = Outpt + " Arms" ' append health message
IF Var1# = 0# THEN ' check divide by zero
    Temp# = 0# ' reset health hits
ELSE ' check divide
    Temp# = ((Var2# / Var1#) * .25#) * 100# ' compute health percentage
    Temp# = INT(Temp#)
    IF Temp# < 1.0# THEN ' check percentage
        Temp# = 1.0# ' reset percentage
    END IF ' end check percentage
END IF ' end check divide by zero
IF Temp# < 0# THEN ' check overflow
    Temp# = 0# ' reset percentage
END IF ' end check overflow
Outpt = Outpt + STR$(Temp#) + "%" ' append health percentage
Outpt = Outpt + " Legs" ' append health message
IF Var1# = 0# THEN ' check divide by zero
    Temp# = 0# ' reset health hits
ELSE ' check divide
    Temp# = ((Var2# / Var1#) * .25#) * 100# ' compute health percentage
    Temp# = INT(Temp#)
    IF Temp# < 1.0# THEN ' check percentage
        Temp# = 1.0# ' reset percentage
    END IF ' end check percentage
END IF ' end check divide by zero
IF Temp# < 0# THEN ' check overflow
    Temp# = 0# ' reset percentage
END IF ' end check overflow
Outpt = Outpt + STR$(Temp#) + "%" ' append health percentage
Outpt = Outpt + " Head" ' append health message
IF Var1# = 0# THEN ' check divide by zero
    Temp# = 0# ' reset health hits
ELSE ' check divide
    Temp# = ((Var2# / Var1#) * .10#) * 100# ' cmpute health percentage
    Temp# = INT(Temp#)
    IF Temp# < 1.0# THEN ' check percentage
        Temp# = 1.0# ' reset percentage
    END IF ' end check percentage
END IF ' end check divide by zero
IF Temp# < 0# THEN ' check overflow
    Temp# = 0# ' reset percentage
END IF ' end check overflow
Outpt = Outpt + STR$(Temp#) + "%" ' append health percentage
CALL IO.O ' send output
' health statistics line one contains vitals in form:
' Vitals: Fat 10/10(+10) Vit 10/10 Mag 10/10 Psi 10/10
IF Weapon4 OR Weapon5 THEN ' check armor, shield
    ' add pluses to message
    Weapon.Plus$ = "(+" + MID$(STR$(Weapon1 + Weapon3), 2) + ")"
ELSE ' neither armor, shield
    Weapon.Plus$ = Nul ' clear plusses message
END IF ' end check armor, shield
Outpt = "Vital hits:" ' initialize health one line
' append current fatigue and maximum fatigue
Outpt = Outpt + " Fat" ' append stat name
Outpt = Outpt + STR$(UserRecord.Fatigue) + "/" + MID$(STR$(UserRecord.FatigueMax), 2)
Outpt = Outpt + Weapon.Plus$ ' append plusses message
' append current vitality and maximum vitality
Outpt = Outpt + " Vit" ' append stat name
Outpt = Outpt + STR$(UserRecord.Vitality) + "/" + MID$(STR$(UserRecord.VitalityMax), 2)
' append current magic points and maximum magic points
Outpt = Outpt + " Mag" ' append stat name
Outpt = Outpt + STR$(UserRecord.Magic) + "/" + MID$(STR$(UserRecord.MagicMax), 2)
' append current psionic points and maximum psionic points
Outpt = Outpt + " Psi" ' append stat name
Outpt = Outpt + STR$(UserRecord.Psionic) + "/" + MID$(STR$(UserRecord.PsionicMax), 2)
CALL IO.O ' display vitals message line
' health statistics line two contains vitals in form:
' Stats: Str 10(+10) Int 10 Wis 10 Dex 10 Con 10 Pie 10 Cha 10
Outpt = "Stats: " ' initialize vitals message
FOR Array.Index = 1 TO 7 ' loop through all health statistics
    ' append first three letters of statistics name and player statistic value
    Outpt = Outpt + LEFT$(Stat(Array.Index), 3) + STR$(GetUserRecordStats(Array.Index))
    IF Array.Index = 1 THEN ' check strength selected
        IF Weapon6 THEN ' verify weapon being held
            Outpt = Outpt + "(+" + MID$(STR$(Weapon2), 2) + ")" ' append weapon plus
        END IF ' end check weapon
    END IF ' end check strength
    Outpt = Outpt + " " ' append one space
NEXT ' end loop through health statistics
CALL IO.O ' display vitals message line
' health statistics line three contains vitals in form:
' Weapons: Blunt> 0% Pole> 0% Sharp> 10% Thrusting> 0%
Outpt = "Weapons: " ' initialize vitals message
FOR Weapon.Number = 1 TO 4 ' loop through all weapon classes
    Weapon$ = RTRIM$(WeaponTypeName(Weapon.Number)) ' make weapon name
    MID$(Weapon$, 1, 1) = UCASE$(MID$(Weapon$, 1, 1)) ' make weapon name
    Outpt = Outpt + Weapon$ ' append weapon class name
    ' append player weapon class percentage value
    Outpt = Outpt + STR$(GetUserRecordWeapons(Weapon.Number)) + "% "
NEXT ' loop through weapon classes
CALL IO.O ' display vitals message line
' display lady statistics
IF UserRecord.ClassType = Lady THEN ' compare user class type to lady
    ' make message for lady statistics
    Outpt = "Lady stats: Beauty " + STR$(UserRecord.Beauty) ' append beauty value
    Outpt = Outpt + " Glamour " + STR$(UserRecord.Glamour) ' append glamour value
    CALL IO.O ' send lady statistics message
END IF ' end compare user class type
END SUB ' end routine to display health statistics

REM * routine to display player character alignment.

SUB Show.Align
Graphics.Off = True ' reset color
IF UserRecord.Level <= False THEN ' check user level
    Outpt = "You are dead!" ' make user level message
    CALL IO.O ' send output message
    EXIT SUB ' exit routine
END IF ' end check user level
Outpt = "You are a level" + STR$(UserRecord.Level) ' make user level message
IF UserRecord.Race <= False THEN ' check valid user race
    UserRecord.Race = 1 ' reset user race
END IF ' end check valid user race
Outpt = Outpt + " " + RTRIM$(Race(UserRecord.Race)) + " " ' append user race name
Inpt = UserRecord.ClassName ' store user class name
CALL Decrypt(Inpt) ' decrypt class name
Outpt = Outpt + RTRIM$(Inpt) + "." ' append classname
CALL IO.O ' send user type message
Outpt = "You are aligned " ' make aligned message
' append player alignment type name 1 through 3 (-1,0,1 plus 2) number 1
Outpt = Outpt + RTRIM$(Alignment.Name1(UserRecord.Align1 + 2)) + " "
' append player alignment type name 1 through 3 (-1,0,1 plus 2) number 2
Outpt = Outpt + RTRIM$(Alignment.Name2(UserRecord.Align2 + 2)) + "."
CALL IO.O ' send player alignment message
END SUB ' end routine to display player character alignment

REM * DM routine to display status of system.

SUB Display.Memory
Graphics.Off = False ' reset color
Outpt = "Dnddoor Author: " + Author$ ' get author name
CALL IO.O ' display author name string
Outpt = "Author email: " + Email$ ' get email address
CALL IO.O ' display email address
Outpt = _OS$ ' get system type
Graphics.Off = True ' reset color
Outpt = "This System: " + Outpt
CALL IO.O ' display system status message
END SUB ' end DM routine to display system memory

REM * routine to display extended information on an object, treasure,
REM * monster, or container using the identify command.

SUB Identify.Object
IF Normal.User THEN ' check non DM status
    IF UserRecord.Level <= 4 THEN ' check player level
        Outpt = "You are not high enough level!" ' make message
        CALL IO.O ' send message
        EXIT SUB ' exit routine
    END IF ' end check level
END IF ' end check DM status
CALL Display.Information ' routine to display information on an item
END SUB ' end identify routine

REM * routine to display list of weapons for sale, the first 15 items in the
REM * treasure file.

SUB Weapon.List
Graphics.Off = False ' reset color
Outpt = "The Blacksmith says: Here's a list of my inventory." ' message
CALL IO.O ' send output message
Graphics.Off = True ' reset color
Outpt = "To purchase, enter number to buy, for example: Buy 15." ' make message
CALL IO.O ' send output message
Graphics.Off = False ' reset color
Outpt = "Number Weapon                        Weight Plus   Gold" ' make output
CALL IO.O ' send output message
Allow.Break = True ' allow control-k breaking
Break = False ' reset control-k flag
Graphics.Off = True ' reset color
FOR List.Count = 1 TO 20 ' loop through the first 20 treasure items
    CALL Read.Record(TreasureFile, List.Count) ' get the next treasure record
    Item.Weight = TreasureRecord.Weight ' store the treasure item weight
    Gold.Value# = TreasureRecord.Gold ' store the treasure item gold value
    ' store the treasure name
    WeaponList.Output$ = TreasureRecord.TreasureName
    ' set first character uppercase
    MID$(WeaponList.Output$, 1, 1) = UCASE$(MID$(WeaponList.Output$, 1, 1))
    Weapon.Plus = False ' reset plus
    IF TreasureRecord.Spell THEN ' compare item to spell
        CALL Read.Record(SpellFile, TreasureRecord.Spell) ' get spell record
        Weapon.Plus = SpellRecord.Level ' store spell level of item
    ELSE ' end compare spell item
        IF TreasureRecord.Plus THEN ' compare treasure item plus
            Weapon.Plus = ABS(TreasureRecord.Plus) ' store item plus
        END IF ' end compare treasure item plus
    END IF ' end compare item spell plus
    ' combine the treasure weight, gold value, and name with blanks imbedded
    Outpt = MID$(STR$(List.Count), 2) ' append item value
    Outpt = Outpt + SPACE$(8 - LEN(STR$(List.Count))) ' pad blanks
    Outpt = Outpt + WeaponList.Output$ ' append item value
    Outpt = Outpt + SPACE$(21 - LEN(WeaponList.Output$)) ' pad blanks
    Outpt = Outpt + MID$(STR$(Item.Weight), 2) ' append item value
    Outpt = Outpt + SPACE$(8 - LEN(STR$(Item.Weight))) ' pad blanks
    Outpt = Outpt + MID$(STR$(Weapon.Plus), 2) ' append item value
    Outpt = Outpt + SPACE$(8 - LEN(STR$(Weapon.Plus))) ' pad blanks
    Outpt = Outpt + MID$(STR$(Gold.Value#), 2) ' append item value
    CALL IO.O ' send message output
    IF Break THEN ' check break
        EXIT FOR ' exit treasure file loop
    END IF ' end compare break
NEXT ' end treasure file item display loop
Allow.Break = False ' reset control-k breaking
IF Break THEN ' check control-k flag
    Break = False ' reset control-k flag
    Outpt = Nul ' set output to null
    CALL IO.O ' send empty return
END IF ' end check control-k flag
END SUB ' end routine to list treasure items for sale

REM * routine to allow user to change password.

SUB Change.PassWord
Graphics.Off = True ' reset color
Outpt = "Change your password(y/n)? " ' input prompt
No.Input.Out = "N" ' default input
CALL IO.I ' get user input
IF Yes THEN ' compare input
    Outpt = "Type in old password for verification:" ' input prompt
    Line.Length = 20 ' line length for password
    Hidden = True ' echo mask characters
    CALL IO.I ' get user input
    Hidden = False ' reset echo mask flag
    Inpt = LTRIM$(Inpt) ' trim entry password
    Inpt = RTRIM$(Inpt) ' trim entry password
    Inpt = UCASE$(Inpt) ' set entry password to uppercase
    Outpt = UserRecord.PassWord ' get user's current password
    CALL Decrypt(Outpt) ' decrypt user password
    IF Outpt = Nul THEN ' verify password validity
        Outpt = "This password has a checksum error!" ' make error message
        CALL IO.O ' send output message
        EXIT SUB ' exit routine
    END IF ' end compare password validity
    Outpt = RTRIM$(Outpt) ' trim password
    IF Outpt <> Inpt THEN ' compare entered password to user password
        Outpt = "Passwords don't match!" ' make error message
        CALL IO.O ' send output message
        EXIT SUB ' exit routine
    END IF ' end compare passwords
    Outpt = "Type in new password(20 char. max.)" ' format input message
    CALL IO.O ' send output message
    Line.Length = 20 ' set line length of new password
    Outpt = "?" ' set input prompt
    Hidden = True ' set echo mask character flag
    CALL IO.I ' get user input
    Hidden = False ' reset echo mask flag
    IF No.Input THEN ' check length of input
        Outpt = "Password not changed." ' make error message
        CALL IO.O ' send output message
        EXIT SUB ' exit routine
    END IF ' end compare length of input
    Inpt = LTRIM$(Inpt) ' trim new password
    Inpt = RTRIM$(Inpt) ' trim new password
    Inpt = UCASE$(Inpt) ' convert to uppercase
    CALL Valid(Inpt, 20) ' check validity of new password
    IF Inpt = Nul THEN ' compare validity of new password
        Outpt = "Illegal characters in password!" ' make error message
        CALL IO.O ' send output message
        EXIT SUB ' exit routine
    END IF ' end compare password validity
    CALL Encrypt(Inpt, False) ' encrypt new password
    UserRecord.PassWord = Inpt ' store new password in user record
    Outpt = "Password changed." ' make message
    CALL IO.O ' send output message
    EXIT SUB ' exit routine
END IF ' end compare input
Outpt = "Password not changed." ' make message
CALL IO.O ' send output message
END SUB ' end routine to change password

REM * routine to change alignment once per player character.

SUB Align
IF Normal.User THEN ' compare to non DM
    IF UserRecord.Flags AND Alignmented THEN ' compare user record flag
        Outpt = "You've already changed alignment once!" ' message
        CALL IO.O ' send output
        EXIT SUB ' exit routine
    END IF ' end compare user record flag
END IF ' end compare normal user
Outpt = "Change alignment(y/n)? " ' input prompt
No.Input.Out = "Y" ' default input
CALL IO.I ' get user input
IF Yes THEN ' compare yes entered
    UserRecord.Flags = UserRecord.Flags OR Alignmented ' set user record flag
    CALL Modify.Alignment ' routine to change alignment
    Outpt = "Alignment is now " ' message with new alignment
    Outpt = Outpt + RTRIM$(Alignment.Name1(UserRecord.Align1 + 2)) + " " ' message
    Outpt = Outpt + RTRIM$(Alignment.Name2(UserRecord.Align2 + 2)) ' message
    CALL IO.O ' send message
    EXIT SUB ' exit routine
END IF ' end compare input
Outpt = "Alignment not changed!" ' make output message
CALL IO.O ' send output message
END SUB ' end routine to change alignment

REM * routine to allow user to change all statistics once per character.

SUB Reroll.Character
IF Normal.User THEN ' compare to non DM
    IF UserRecord.Flags AND Rerolled THEN ' check user record flag
        Outpt = "You've already rerolled your character!" ' message
        CALL IO.O ' send output
        EXIT SUB ' exit routine
    END IF ' end compare user flag
END IF ' end compare normal user
Outpt = "Reroll character(y/n)? " ' prompt user to reroll
No.Input.Out = "Y" ' set default input
CALL IO.I ' get user input
IF Yes THEN ' compare yes entered
    UserRecord.Flags = UserRecord.Flags OR Rerolled ' set user record flag
    DO ' loop until changes completed
        Outpt = "Character reroll:" ' message
        CALL IO.O ' send output
        Outpt = "Change class type/name(y/n)? " ' input prompt
        No.Input.Out = "Y" ' set default input
        CALL IO.I ' get user input
        IF Yes THEN ' compare yes entered
            CALL Modify.Class ' routine to modify class type
        END IF ' end compare yes entered
        Outpt = "Character reroll:" ' message
        CALL IO.O ' send output
        Outpt = "Change vital statistics(y/n)? " ' input prompt
        No.Input.Out = "Y" ' set default input
        CALL IO.I ' get user input
        IF Yes THEN ' compare yes entered
            CALL Modify.Stats ' routine to modify statistics
        END IF ' end compare yes entered
        Outpt = "Character reroll:" ' message
        CALL IO.O ' send output
        Outpt = "Change character race type/name(y/n)? " ' input prompt
        No.Input.Out = "Y" ' set default input
        CALL IO.I ' get user input
        IF Yes THEN ' compare yes entered
            CALL Modify.Race ' routine to modify race
        END IF ' end compare yes entered
        Outpt = "Character reroll:" ' message
        CALL IO.O ' send output
        Outpt = "Change weapon proficiency(y/n)? " ' input prompt
        No.Input.Out = "Y" ' set default input
        CALL IO.I ' get user input
        IF Yes THEN ' compare yes entered
            CALL Modify.Proficiency ' routine to modify weapon proficiency
        END IF ' end compare yes entered
        Outpt = "Character reroll:" ' message
        CALL IO.O ' send output
        Outpt = "Change character alignment(y/n)? " ' input prompt
        No.Input.Out = "Y" ' set default input
        CALL IO.I ' get user input
        IF Yes THEN ' compare yes entered
            CALL Modify.Alignment ' routine to modify alignment
        END IF ' end compare yes entered
        DO ' loop until changes finished prompt
            Outpt = "All changes finished(y/n)? " ' input prompt
            No.Input.Out = "Y" ' set default input
            CALL IO.I ' get user input
            IF Yes THEN ' compare yes entered
                EXIT SUB ' exit routine
            END IF ' end compare
            IF No THEN ' compare no entered
                EXIT DO ' exit changes loop
            END IF ' end compare
        LOOP ' loop until yes or no entered
    LOOP ' end loop until changes completed
    EXIT SUB ' exit routine
END IF ' end compare yes entered
Outpt = "Your character has not been rerolled!" ' make output message
CALL IO.O ' send output message
END SUB ' end routine to modify all statistics

REM * routine returns a prefix for monster name.
REM * output variables:
REM *   Prefix1 - monster name prefix.

SUB The.Or.An
IF MonsterArray(Monster.Number).Permanent < True THEN ' check for nonplayer
    Prefix1 = "the " ' make prefix
ELSE ' check monster type
    Prefix$ = MonsterArray(Monster.Number).MonsterName ' get monster name
    Prefix$ = LEFT$(Prefix$, 1) ' get first letter of monster name
    IF INSTR("aeiou", Prefix$) THEN ' check monster name vowel
        Prefix1 = "an " ' set prefix
    ELSE ' check vowel
        Prefix1 = "a " ' set prefix
    END IF ' end check monster name vowel
END IF ' end check nonplayer
END SUB ' end routine to get monster name prefix

REM * routine for parsing numeric value from parameter.
REM * input variables:
REM *   Parsed.Command1 - string with imbedded pound sign to check.
REM * output variables:
REM *   Parse.Number - value of number after pound sign.
REM * work variables:
REM *   Delimit - position of # sign.

SUB Numeric
Parse.Number = False ' reset numeric value
Parse.Delimit = INSTR(Parsed.Command1, "#") ' search parameter for # sign
IF Parse.Delimit THEN ' check # sign in string
    ' store numeric value after #
    Parse.Number = INT(VAL(MID$(Parsed.Command1, Parse.Delimit + 1)))
    ' trim # from string
    Parsed.Command1 = LEFT$(Parsed.Command1, Parse.Delimit - 1)
END IF ' end check for # sign in string
END SUB ' end routine to parse part of parameter

REM * routine decrements parameter # value after calls to search routines.
REM * input variables:
REM *   Parse.Count - counter for search routines.
REM * output variables:
REM *   Parse.Number - decremented # sign value counter.

SUB Num
IF Parse.Number > False THEN ' check counter
    ' decrement search routine value from counter
    Parse.Number = Parse.Number - Parse.Count
    IF Parse.Number < False THEN ' check counter
        Parse.Number = False ' reset counter
    END IF ' end check counter
END IF ' end check counter
END SUB ' end routine to decrement # sign value counter

REM * routine to separate two parameters after command input.
REM * input variables:
REM *   Parsed.Command2 - first/second parameters combined.
REM * output variables:
REM *   Parsed.Command1 - first parsed parameter.
REM *   Parsed.Command2 - second parsed parameter.
REM * work variables:
REM *   Delimit - position of # sign.

SUB Parse
' find imbedded space in command parameter
Parse.Delimit = INSTR(Parsed.Command2, " ")
Parser = False ' reset position of space
IF Parse.Delimit THEN ' check imbedded space
    ' store first parameter
    Parsed.Command1 = LEFT$(Parsed.Command2, Parse.Delimit - 1)
    ' store second parameter
    Parsed.Command2 = MID$(Parsed.Command2, Parse.Delimit + 1)
    Parser = Parse.Delimit ' store parsed space position
END IF ' end check for space
END SUB ' end routine to separate parameters

REM * routine to separate two parameters after command input in reverse order.
REM * input variables:
REM *   Parsed.Command2 - first/second parameters combined.
REM * output variables:
REM *   Parsed.Command1 - second parsed parameter.
REM *   Parsed.Command2 - first parsed parameter.
REM * work variables:
REM *   Delimit - position of # sign.

SUB ParseX
' find imbedded space in command parameter
Parse.Delimit = INSTR(Parsed.Command2, " ")
Parser = False ' reset position of space
IF Parse.Delimit THEN ' check imbedded space
    ' store second parameter
    Parsed.Command1 = MID$(Parsed.Command2, Parse.Delimit + 1)
    ' store first parameter
    Parsed.Command2 = LEFT$(Parsed.Command2, Parse.Delimit - 1)
    Parser = Parse.Delimit ' storeparsed space position
END IF ' end check for space
END SUB ' end routine to separate parameters in reverse order

REM * routine computes gold player needs for next training level.
REM * output variables:
REM *   Gold.Required# - gold points.

SUB Gold (Gold.Required#)
IF UserRecord.Level <= 10 THEN ' check player level
    Gold.Required# = 2 ^ (UserRecord.Level + 5) ' calculate gold
ELSE ' player level over ten
    Gold.Required# = 2 ^ 15 + (UserRecord.Level - 10) * 10000! ' calculate gold
END IF ' end check player level
END SUB ' end routine to calculate gold

REM * routine computes experience player needs for next training level.
REM * output variables:
REM *   Exp.Required# - experience points.

SUB Experience (Exp.Required#)
IF UserRecord.Level <= 10 THEN ' check player level
    Exp.Required# = 2 ^ (UserRecord.Level + 6) ' calculate experience
ELSE ' player level over ten
    Exp.Required# = 2 ^ 16 + (UserRecord.Level - 10) * 10000! ' calculate experience
END IF ' end check player level
END SUB ' end routine to calculate experience

REM * routine returns range of numbers.
REM * input variables:
REM *   Upper.Range - contains upper range.
REM * output variables:
REM *   Start.Range - start of range.
REM *   End.Range - end of range.

SUB Get.Range (Upper.Range, Start.Range, End.Range)
Range.Type$ = MID$(STR$(Upper.Range), 2) ' convert upper range to string
Outpt = "From(1-" + Range.Type$ + ")? " ' make input prompt
No.Input.Out = "1" ' default input
CALL IO.I ' get input
Start.Range = INT(VAL(Inpt)) ' convert input to integer
IF Start.Range < 1 THEN ' check bounds of input
    Start.Range = 1 ' reset input
END IF ' end check bounds
IF Start.Range > Upper.Range THEN ' check bounds of input
    Start.Range = Upper.Range ' reset input
END IF ' end check bounds
Outpt = "To(" + MID$(STR$(Start.Range), 2) + "-" + Range.Type$ + ")? " ' input prompt
No.Input.Out = Range.Type$ ' default input
CALL IO.I ' get input
End.Range = INT(VAL(Inpt)) ' convert input to integer
IF End.Range < Start.Range THEN ' check bounds
    End.Range = Start.Range ' reset input
END IF ' end check bounds
IF End.Range > Upper.Range THEN ' check bounds
    End.Range = Upper.Range ' reset input
END IF ' end check bounds
END SUB ' end routine to get range of numbers

REM * routine returns range of numbers.
REM * input variables:
REM *   Start.Range - starting of range.
REM *   End.Range - end of range.
REM * output variables:
REM *   Upper.Range - contains upper range.

SUB Get.Range2 (Start.Range, End.Range, Upper.Range)
Start.Range$ = MID$(STR$(Start.Range), 2) ' convert starting range to string
End.Range$ = MID$(STR$(End.Range), 2) ' convert upper range to string
Outpt = Outpt + "(" + Start.Range$ + "-" + End.Range$ + ")? " ' make input prompt
No.Input.Out = Start.Range$ ' default input
CALL IO.I ' get input
Upper.Range = INT(VAL(Inpt)) ' convert input to integer
IF Upper.Range < Start.Range THEN ' check bounds of input
    Upper.Range = Start.Range ' reset input
END IF ' end check bounds
IF Upper.Range > End.Range THEN ' check bounds of input
    Upper.Range = End.Range ' reset input
END IF ' end check bounds
END SUB ' end routine to get range of numbers

REM * routine returns range of single precision numbers.
REM * input variables:
REM *   Upper.Range! - contains upper range.
REM * output variables:
REM *   Start.Range! - start of range.
REM *   End.Range! - end of range.

SUB Get.Room.Range (Upper.Range!, Start.Range!, End.Range!)
Range.Type$ = MID$(STR$(Upper.Range!), 2) ' convert upper range to string
Outpt = "From(1-" + Range.Type$ + ")? " ' make input prompt
No.Input.Out = "1" ' default input
CALL IO.I ' get input
Start.Range! = INT(VAL(Inpt)) ' convert input to integer
IF Start.Range! < 1! THEN ' check bounds of input
    Start.Range! = 1! ' reset input
END IF ' end check bounds
IF Start.Range! > Upper.Range! THEN ' check bounds of input
    Start.Range! = Upper.Range! ' reset input
END IF ' end check bounds
Outpt = "To(" + MID$(STR$(Start.Range!), 2) + "-" + Range.Type$ + ")? " ' input prompt
No.Input.Out = Range.Type$ ' default input
CALL IO.I ' get input
End.Range! = INT(VAL(Inpt)) ' convert input to integer
IF End.Range! < Start.Range! THEN ' check bounds
    End.Range! = Start.Range! ' reset input
END IF ' end check bounds
IF End.Range! > Upper.Range! THEN ' check bounds
    End.Range! = Upper.Range! ' reset input
END IF ' end check bounds
END SUB ' end routine to get range of numbers

REM * routine returns range of single precision numbers.
REM * input variables:
REM *   Start.Range - starting of range.
REM *   End.Range - end of range.
REM * output variables:
REM *   Upper.Range - contains upper range.

SUB Get.Room.Range2 (Start.Range!, End.Range!, Upper.Range!)
Start.Range$ = MID$(STR$(Start.Range!), 2) ' convert starting range to string
End.Range$ = MID$(STR$(End.Range!), 2) ' convert upper range to string
Outpt = Outpt + "(" + Start.Range$ + "-" + End.Range$ + ")? " ' make input prompt
No.Input.Out = Start.Range$ ' default input
CALL IO.I ' get input
Upper.Range! = INT(VAL(Inpt)) ' convert input to integer
IF Upper.Range! < Start.Range! THEN ' check bounds of input
    Upper.Range! = Start.Range! ' reset input
END IF ' end check bounds
IF Upper.Range! > End.Range! THEN ' check bounds of input
    Upper.Range! = End.Range! ' reset input
END IF ' end check bounds
END SUB ' end routine to get range of numbers

REM * routine returns the charges of an item of treasure.
REM * output variables:
REM *   Charges.Amount - stores treasure type charges.

SUB TreasureCharges (Charges.Amount)
Charges.Amount = TreasureRecord.Charges ' store treasure charges
IF TreasureRecord.FuelType THEN ' compare treasure to fuel
    Charges.Amount = TreasureRecord.FuelCharges ' reset treasure charges
END IF ' end compare fuel charges
IF TreasureRecord.LightType THEN ' compare treasure to light
    Charges.Amount = False ' reset treasure charges
END IF ' compare charges
IF TreasureRecord.Vehicle THEN ' compare treasure to vehicle
    Charges.Amount = TreasureRecord.VehicleHits ' reset treasure charges
END IF ' compare charges
END SUB ' end routine to return charges

REM * routine to wish for an item
REM * input variables:
REM *   Inpt - stores name of item

SUB Wish.Item
Outpt = "The Ghods Thunder.." ' make output message
CALL IO.O ' send message
Outpt = "   What Do You Wish For?" ' make input prompt
CALL IO.I ' get input
Stored.Parsed.Command2 = Inpt ' store input
Parsed.Command1 = Stored.Parsed.Command2 ' store input
CALL Numeric ' parse number
Inpt = Parsed.Command1 ' restore input
Inpt = LCASE$(Inpt) ' convert to lowercase
CALL Drop(False) ' call routine to get item
END SUB

REM * routine to wish for points or an item, or get an object or treasure.
REM * input variables:
REM *   Drop.Type - false to use normal drop routine, true for extended drop.

SUB Drop (Drop.Type)
Wish.Points = 1 ' store points number to wish for
IF RIGHT$(Inpt, 7) = " points" THEN ' compare wish for two points
    Inpt = LEFT$(Inpt, LEN(Inpt) - 7) ' truncate wish parameter
    Wish.Points = 2 ' store points number to wish for
END IF ' end compare points wish
FOR Stat.Number = 1 TO 7 ' loop through statistic names
    Outpts = Stat(Stat.Number) ' get statistic name
    Outpts = RTRIM$(Outpts) ' trim name
    Outpts = LCASE$(Outpts) ' lowercase name
    IF Inpt = Outpts THEN ' compare wish item to statistic name
        IF Normal.User THEN ' check non DM
            ' check point already wished for
            IF (UserRecord.Flags AND 2 ^ Stat.Number) THEN
                GOTO Wish.Denied ' jump to wish denied subroutine
            END IF ' end check point wished for
        END IF ' end check normal player
        ' add player wish bitflag
        UserRecord.Flags = (UserRecord.Flags OR 2 ^ Stat.Number)
        Wish.Points = Wish.Points * INT(RND * 3 + 1) ' calculate points to add
        ' calculate new statistic
        New.Stat# = GetUserRecordStats(Stat.Number) + Wish.Points
        IF New.Stat# > MaxInt THEN ' compare maximum integer
            New.Stat# = MaxInt ' reset to maximum integer
        END IF ' end check maximum integer
        New.Stat = CINT(New.Stat#) ' store in integer
        IF Normal.User THEN ' check non DM
            IF New.Stat > MaxStat THEN ' check maximum statistic allowed
                GOTO Wish.Denied ' jump to wish denied subroutine
            END IF ' end check maximum stat
        END IF ' end check normal player
        CALL PutUserRecordStats(Stat.Number, New.Stat) ' increment point wished for
        Graphics.Off = True ' reset color
        Outpt = "The Ghods Thunder..." ' make ghod message
        CALL IO.O ' send message
        Outpt = "   Your " + Outpts + " Has Been Raised!" ' make stat message
        CALL IO.O ' send update stat message
        Graphics.Off = False ' reset color
        EXIT SUB ' exit routine
    END IF ' end compare point wish
NEXT ' end loop through statistic names
IF Drop.Type = False THEN ' check drop type
    IF Normal.User THEN ' check normal player
        IF UserRecord.Flags AND Wished THEN ' check player has already wished
            GOTO Wish.Denied ' jump to wish denied subroutine
        END IF ' end check already wished
    END IF ' end check normal user
END IF ' end check drop type
UserRecord.Flags = UserRecord.Flags OR Wished ' set player wish bitflag
Parse.Value = False ' item counter
Wish.Charges = False ' item charges
Wish.Index = False ' item index
' loop through treasure file
FOR Treasure.Number = 1 TO LOF(TreasureFile) / LEN(TreasureRecord)
    CALL Read.Record(TreasureFile, Treasure.Number) ' get next record
    Outpts = TreasureRecord.TreasureName ' store treasure name
    Outpts = LEFT$(Outpts, LEN(Inpt)) ' truncate name
    IF Inpt = Outpts THEN ' compare treasure name to wish item name
        Parse.Value = Parse.Value + 1 ' increment item counter
        ' compare counters
        IF Parse.Number = False OR Parse.Value = Parse.Number THEN
            Wish.Index = Treasure.Number ' store treasure file number
            CALL TreasureCharges(Wish.Charges) ' routine to get treasure charges
            EXIT FOR ' exit loop through treasure file
        END IF ' end compare counters
    END IF ' end compare names
NEXT ' end loop through treasure file
IF Wish.Index = False THEN ' check no treasure match found
    IF Normal.User = False OR Drop.Type THEN ' compare DM/Sysop or drop type
        ' loop through object file
        FOR Object.Number = 1 TO LOF(ObjectFile) / LEN(ObjectRecord)
            CALL Read.Record(ObjectFile, Object.Number) ' get object record
            Outpts = ObjectRecord.ObjectName ' store object name
            Outpts = LEFT$(Outpts, LEN(Inpt)) ' truncate object name
            IF Inpt = Outpts THEN ' compare object name to wish name
                Parse.Value = Parse.Value + 1 ' increment counter
                ' compare counters
                IF Parse.Number = False OR Parse.Value = Parse.Number THEN
                    ' store negation of object file index
                    Wish.Index = -Object.Number
                    Wish.Charges = False ' clear charges
                    EXIT FOR ' exit loop through object file
                END IF ' end compare counters
            END IF ' end compare names
        NEXT ' end loop through object file
    END IF ' end compare drop type/DM, Sysop
END IF ' end check treasure found
IF Drop.Type = False THEN ' check drop type
    IF Wish.Index > False THEN ' check treasure found
        IF Normal.User THEN ' check normal player/not DM
            IF TreasureRecord.Container THEN ' check treasure container
                Wish.Index = False ' clear treasure found
            ELSE ' check treasure
                IF TreasureRecord.Vehicle THEN ' check treasure vehicle
                    Wish.Index = False ' clear treasure found
                ELSE ' check treasure
                    Spell.Number = TreasureRecord.Spell ' get treasure spell
                    ' check spell file bounds
                    IF Spell.Number > False AND Spell.Number <= LOF(SpellFile) / LEN(SpellRecord) THEN
                        CALL Read.Record(SpellFile, Spell.Number) ' get spell record
                        IF SpellRecord.SpellType = 4 THEN ' check spell type wish
                            Wish.Index = False ' clear treasure found
                        END IF ' end check wish spell
                    END IF ' end check spell file bounds
                END IF ' end check treasure
            END IF ' end check treasure
        END IF ' end check normal player
    END IF ' and check treasure found to drop
END IF ' end check drop type
Drop.Type = False ' clear drop flag
SELECT CASE Wish.Index ' selection of item type to drop
    CASE IS < False ' check object being dropped
        ' add object to room
        CALL Add.Room.Object(ABS(Wish.Index), False, Drop.Type)
    CASE IS > False ' check treasure being dropped
        SELECT CASE TreasureRecord.Container ' selection of container dropped
            CASE False ' check treasure container
                ' add treasure to room
                CALL Add.Room.Treasure(Wish.Index, Wish.Charges, False, Drop.Type)
            CASE True ' check container
                ' check container name
                IF RTRIM$(RoomRecord.Container.ShortName) = Nul THEN
                    Drop.Type = True ' set drop flag
                    ' store container variables
                    ContainerRec.Closed = TreasureRecord.Closed
                    ContainerRec.ContainerName = TreasureRecord.TreasureName
                    ContainerRec.Locked = TreasureRecord.Locked
                    ContainerRec.Keyed = TreasureRecord.Keyed
                    ContainerRec.ShortName = TreasureRecord.ShortName
                    ContainerRec.Permanent = TreasureRecord.Permanent
                    FOR Container.Count = 1 TO 5 ' loop through container contents
                        ' clear container contents
                        CALL Clear.Container(Container.Count, False)
                    NEXT ' end loop through container
                    RoomRecord.Container = ContainerRec ' add container record to room
                    CALL Share.Room.Record(Room) ' write room record
                END IF ' end check container
        END SELECT ' end select container
END SELECT ' end select treasure
IF Drop.Type = False THEN ' check drop flag
    GOTO Wish.Denied ' jump to wish denied subroutine
END IF ' end check drop flag
Graphics.Off = True ' reset color
Outpt = "A Dark Cloud Passes Overhead..." ' make ghod message
CALL IO.O ' send message
Outpt = "   Some Treasure Falls From The Sky..." ' make ghod message
CALL IO.O ' send message
Outpt = "The Cloud Disappears..." ' make ghod message
CALL IO.O ' send message
Graphics.Off = False ' reset color
EXIT SUB ' exit routine

Wish.Denied:
Graphics.Off = True ' reset color
Outpt = "The Ghods Thunder..." ' make ghod message
CALL IO.O ' send message
Outpt = "   Your Wish Is Denied!" ' make ghod message
CALL IO.O ' send message
Graphics.Off = False ' reset color
END SUB ' end routine to drop item to ground

REM * Filename: dnds3.bas Version: v5.0a r1.0a
REM * This subprogram contains most of the attack and player death routines.

REM * routine to attack a monster, or object.
REM * input variables:
REM *   Parsed.Command1 - contains the attack target.
REM *   Room - contains room number user is in.
REM * work variables:
REM *   Prefix1 - contains prefix to target (he/the/it).
REM *   (constants) - command numbers.

SUB Attack.Monster
Prefix1 = Nul ' reset target prefix
Monster.Number = False ' reset monster target number
IF Room = 1! THEN ' compare room number
    Outpt = "This is a safe haven!" ' make message
    CALL IO.O ' send message
    EXIT SUB ' exit routine
END IF ' end compare room number
CALL Check.Room.Objects ' verify target is an object in room
IF Index.Number THEN ' compare object index
    IF ObjectRecord.JailTrap THEN ' check object jails attacker
        Jail.Room.Number = ObjectRecord.RoomLink
        CALL Jail ' routine to send player to jail room
    ELSE ' compare check jail object
        Outpt = "Nothing happens.." ' make message
        CALL IO.O ' send message
    END IF ' end compare jail object
    EXIT SUB ' exit routine
END IF ' end compare object index
Previous.Command = False ' reset number of previous attack command
IF Intoxicated > False THEN ' compare player is drunk
    Outpt = "You are too drunk to fight!" ' make message
    CALL IO.O ' send message
    EXIT SUB ' exit routine
END IF ' end compare drunk player
CALL Check.Monster ' verify target is monster
IF Monster.Number = False THEN ' check monster index
    IF Parser = False THEN ' verify command type
        CALL Get.Last.Monster(Monster.Found) ' routine to get default monster
        IF Monster.Found THEN ' check monster exists
            Parsed.Command1 = Last.Monster ' store monster name
        END IF ' end check monster
    END IF ' end verufy command
    IF Monster.Number = False THEN ' verify target found
        Outpt = "You can't attack that!" ' make message
        CALL IO.O ' send message
        EXIT SUB ' exit routine
    END IF ' end verify target
END IF ' end compare monster target
IF MonsterArray(Monster.Number).Jail THEN ' verify monster jails attacker
    Jail.Room.Number = MonsterRecord.Teleport ' store jail room number
    CALL Jail ' routine to send player to jail room
    EXIT SUB ' exit routine
END IF ' end verify jail monster
CALL The.Or.An ' routine to get prefix
Outpts = MonsterArray(Monster.Number).MonsterName ' store monster target name
Outpts = RTRIM$(Outpts) ' trim monster name
Outpts = LCASE$(Outpts) ' lowercase monster name
SELECT CASE Last.Command.Number ' make selection of attack command number
    CASE Charm ' charm
        IF UserRecord.ClassType = Lady THEN ' compare class type of player
            IF MonsterArray(Monster.Number).Magic <= True THEN ' check magic monster
                Outpt = "Didn't work!" ' make message
                CALL IO.O ' send message
                EXIT SUB ' exit routine
            END IF ' end check magic monster
            IF INT(RND * 10 + 4) > UserRecord.Beauty THEN ' calculate lady hits
                Outpt = "Didn't work!" ' make message
                CALL IO.O ' send message
                EXIT SUB ' exit routine
            END IF ' end calculate hits
            MonsterArray(Monster.Number).Magic = 5 ' update monster befuddle rounds
            Outpt = "It worked!" ' make message
            CALL IO.O ' send message
            EXIT SUB ' exit routine
        END IF ' end compare class type
        Outpt = "Your class does not allow that!" ' make message
        CALL IO.O ' send message
        EXIT SUB ' exit routine
    CASE Beguile ' beguile
        IF UserRecord.ClassType = Lady THEN ' compare class type of player
            IF INT(RND * 12 + 5) > (UserRecord.Beauty + UserRecord.Glamour) THEN ' calculate
                Outpt = "Didn't work!" ' make message
                CALL IO.O ' send message
                EXIT SUB ' exit routine
            END IF ' end calculate monster death
            CALL Monster.Died ' calculated monster died
            EXIT SUB ' exit routine
        END IF ' end compare class type
        Outpt = "Your class does not allow that!" ' make message
        CALL IO.O ' send message
        EXIT SUB ' exit routine
    CASE Circled ' circle
        IF MonsterArray(Monster.Number).Magic <= True THEN ' compare magic monster
            Outpt = "Didn't work!" ' make message
            CALL IO.O ' send message
            EXIT SUB ' exit routine
        END IF ' end compare magic monster
        ' compute
        Calculate# = (RND * 5 + UserRecord.Level) > MonsterArray(Monster.Number).Level
        IF Calculate# THEN ' compare circle monster
            Previous.Command = True ' reset previous attack command number
            MonsterArray(Monster.Number).Magic = 4 ' update befuddle monster rounds
            Outpt = "It worked!" ' make message
            CALL IO.O ' send message
            EXIT SUB ' exit routine
        END IF ' end compare circled monster
        Outpt = "Didn't work!" ' make message
        CALL IO.O ' send message
        EXIT SUB ' exit routine
    CASE Feint ' feint
        IF MonsterArray(Monster.Number).Magic <= True THEN ' compare magic monster
            Outpt = "Didn't work!" ' make message
            CALL IO.O ' send message
            EXIT SUB ' exit routine
        END IF ' end compare magic monster
        Calculate# = GetUserRecordStats(4) > (RND * 5 + MonsterArray(Monster.Number).Level / 2)
        IF Calculate# THEN ' compare calculated befuddle
            Previous.Command = Last.Command.Number ' store last command number
            MonsterArray(Monster.Number).Magic = 3 ' update monster befuddle rounds
            Outpt = "It worked!" ' make message
            CALL IO.O ' send message
            EXIT SUB ' exit routine
        END IF ' end compare calculation
        Outpt = "Didn't work!" ' make message
        CALL IO.O ' send message
        EXIT SUB ' exit routine
    CASE Resist, Parry ' parry, resist
        Previous.Command = Last.Command.Number ' store last command number
        Outpt = "It worked!" ' make message
        CALL IO.O ' send message
        EXIT SUB ' exit routine
    CASE Turn ' turn
        Magic.Spell = 12 ' force turn spell cast
    CASE Bemuse ' bemuse
        IF UserRecord.ClassType = Lady THEN ' compare player class type
            IF MonsterArray(Monster.Number).LevelDrain THEN ' check monster type
                Outpt = "Didn't work!" ' make message
                CALL IO.O ' send message
                EXIT SUB ' exit routine
            END IF ' end compare class type
            IF INT(RND * 8 + 5) > UserRecord.Glamour THEN ' calculate lady level drain
                Outpt = "Didn't work!" ' make message
                CALL IO.O ' send message
                EXIT SUB ' exit routine
            END IF ' end calculate level drain
            MonsterArray(Monster.Number).Level = MonsterArray(Monster.Number).Level - 1
            Outpt = "You drain a level from the " + Outpts + "!" ' decrement monster level
            CALL IO.O ' send message
            IF MonsterArray(Monster.Number).Level <= False THEN ' check monster level
                Outpt = "You killed the " + Outpts + "!" ' make message
                CALL IO.O ' send message
                CALL Monster.Died ' routine for dead monster
                EXIT SUB ' exit routine
            END IF ' end compare dead monster
        END IF ' end compare lady class
        Outpt = "Your class does not allow that!" ' make message
        CALL IO.O ' send message
        EXIT SUB ' exit routine
    CASE Befuddle ' befuddle
        IF MonsterArray(Monster.Number).Magic <= True THEN ' compare magic monster
            Outpt = "Didn't work!" ' make message
            CALL IO.O ' send message
            EXIT SUB ' exit routine
        END IF ' end compare magic monster
        IF (GetUserRecordStats(3) + GetUserRecordStats(4)) > (RND * 10 + MonsterArray(Monster.Number).Level / 2) THEN ' calculate befuddle
            Previous.Command = True ' reset previous command number
            MonsterArray(Monster.Number).Magic = 6 ' update monster befuddle rounds
            Outpt = "It worked!" ' make message
            CALL IO.O ' send message
            EXIT SUB ' exit routine
        END IF ' end calculate befuddle
        Outpt = "Didn't work!" ' make message
        CALL IO.O ' send message
        EXIT SUB ' exit routine
    CASE Lunge, Dodge ' lunge, dodge
        IF Weapon5 THEN ' verify player holding shield
            Weapon3 = False ' reset shield
            Weapon5 = False ' reset shield
            Outpt = "You return your shield!" ' make message
            CALL IO.O ' send message
        END IF ' end verify shield
    CASE Beat, Punch ' beat, punch
        IF Weapon6 THEN ' verify holding weapon
            Weapon2 = False ' reset weapon
            Weapon6 = False ' reset weapon
            Weapon10 = False ' reset weapon
            Outpt = "You return your weapon!" ' make message
            CALL IO.O ' send message
        END IF ' end verify weapon
    CASE Shield, Guard ' shield, guard
        IF Weapon3 THEN ' verify holding shield
            Previous.Command = True ' reset previous command
            Outpt = "It worked!" ' make message
            CALL IO.O ' send message
            EXIT SUB ' exit routine
        END IF ' end verify shield
        Previous.Command = False ' reset previous command
        Outpt = "You are not holding a shield!" ' make error message
        CALL IO.O ' send message
        EXIT SUB ' exit routine
END SELECT ' end select command number
SELECT CASE Magic.Spell ' select attack of spell number
    CASE Poison ' poison spell
        IF MonsterArray(Monster.Number).Poison THEN ' check monster is poisoned
            Outpt = "Didn't work!" ' make message
            CALL IO.O ' send message
            EXIT SUB ' exit routine
        END IF ' end check monster
        IF RND > .5 THEN ' random chance
            Outpt = "Didn't work!" ' make message
            CALL IO.O ' send message
            EXIT SUB ' exit routine
        END IF ' end random chance
        ' poison monster rounds
        MonsterArray(Monster.Number).Poison = UserRecord.Level
        Outpt = "You poison the " + Outpts + "!" ' make message
        CALL IO.O ' send message
        EXIT SUB ' exit routine
    CASE LevelDrain ' leveldrain spell
        IF MonsterArray(Monster.Number).LevelDrain THEN ' check monster is undead
            Outpt = "Didn't work!" ' make message
            CALL IO.O ' send message
            EXIT SUB ' exit routine
        END IF ' end check monster
        IF RND > .5 THEN ' random chance
            Outpt = "Didn't work!" ' make message
            CALL IO.O ' send message
            EXIT SUB ' exit routine
        END IF ' end check random chance
        MonsterArray(Monster.Number).Level = MonsterArray(Monster.Number).Level - 1
        Outpt = "You drain a level from the " + Outpts + "!" ' decrement monster level
        CALL IO.O ' send message
        IF MonsterArray(Monster.Number).Level <= False THEN ' check monster level
            Outpt = "You killed the " + Outpts + "!" ' make message
            CALL IO.O ' send message
            CALL Monster.Died ' routine for dead monster
        END IF ' end check monster level
        EXIT SUB ' exit routine
    CASE Befuddled ' befuddle spell
        IF MonsterArray(Monster.Number).Magic <= True THEN ' check magic monster
            Outpt = "Didn't work!" ' make message
            CALL IO.O ' send message
            EXIT SUB ' exit routine
        END IF ' end check magic monster
        IF RND > .5 THEN ' random chance
            Outpt = "Didn't work!" ' make message
            CALL IO.O ' send message
        END IF ' end chance
        MonsterArray(Monster.Number).Magic = 4 ' update monster befuddle rounds
        Outpt = "It worked!" ' make message
        CALL IO.O ' send message
        EXIT SUB ' exit routine
    CASE TurnUndead ' turn undead spell
        ' check monster undead
        IF MonsterArray(Monster.Number).LevelDrain = False THEN
            Outpt = "That's not undead!" ' make message
            CALL IO.O ' send message
            EXIT SUB ' exit routine
        END IF ' end check monster
        Calculate# = (RND * UserRecord.Level + 2) > MonsterArray(Monster.Number).Level
        IF Calculate# THEN ' calculate cast spell
            Outpt = "You damned the " + Outpts + "!" ' make message
            CALL IO.O ' send message
            MonsterArray(Monster.Number).Level = MonsterArray(Monster.Number).Level - 1
            Outpt = "You drain a level from the " + Outpts + "!" ' decrement monster level
            CALL IO.O ' send message
            IF MonsterArray(Monster.Number).Level <= False THEN ' check monster level
                Outpt = "You killed the " + Outpts + "!" ' make message
                CALL IO.O ' send message
                CALL Monster.Died ' routine for dead monster
                EXIT SUB ' exit routine
            END IF ' end check monster level
        END IF ' end calculate undead spell
        Outpt = "Didn't work!" ' make message
        CALL IO.O ' send message
        EXIT SUB ' exit routine
    CASE Intoxicate ' intoxicate spell
        IF MonsterArray(Monster.Number).Magic <= True THEN ' check magic monster
            Intoxicated = UserRecord.Level ' store intoxicated rounds counter
            Outpt = "You just became very drunk!" ' make message
            CALL IO.O ' send message
            EXIT SUB ' exit routine
        END IF ' end check magic monster
        IF RND > .5 THEN ' random chance
            Outpt = "Didn't work!" ' make message
            CALL IO.O ' send message
            EXIT SUB ' exit routine
        END IF ' end check chance
        MonsterArray(Monster.Number).Magic = 3 ' update monster befuddle rounds
        Outpt = "It worked!" ' make message
        CALL IO.O ' send message
        EXIT SUB ' exit routine
END SELECT ' end selection of magic spell number
IF Magic.Spell = False THEN ' compare spell being cast
    IF UserRecord.Fatigue <= False THEN ' check player fatigue
        UserRecord.Fatigue = False ' reset fatigue
        Outpt = "You are too exhausted to fight!" ' make message
        CALL IO.O ' send message
        EXIT SUB ' exit routine
    END IF ' end check player fatigue
    ' normal attack can't hit magical monsters, only spells can
    IF MonsterArray(Monster.Number).Magic <= True THEN ' check magic monster
        Outpt = "A magical force prevents your attack!" ' make message
        CALL IO.O ' send message
        EXIT SUB ' exit routine
    END IF ' end check magic monster
END IF ' end compare spell being cast
' nonplayers can't be attacked by normal weapons or spells.
' check permanent nonplayer
IF MonsterArray(Monster.Number).Permanent < True THEN
    Outpt = "A mysterious force prevents your attack!" ' make message
    CALL IO.O ' send message
    EXIT SUB ' exit routine
END IF ' end check nonplayer
' calculate miss on monster
Calculate# = (RND * 10 + MonsterArray(Monster.Number).Level / 10) > GetUserRecordStats(4)
IF Calculate# THEN ' player missed
    IF Magic.Spell THEN ' check spell being cast
        Outpt = "You missed!" ' spell missed message
        CALL IO.O ' send output message
        EXIT SUB ' exit routine
    END IF ' end check spell being cast
    ' attacking with a weapon
    Calculate# = (RND * 10) > GetUserRecordStats(1) ' calculate fumble
    IF Calculate# THEN ' fumbled
        CALL Fumble ' routine to drop weapon, shield
    END IF ' end fumble check
    Outpt = "You missed the " + Outpts + "!" ' make miss message
    CALL IO.O ' send output message
    EXIT SUB ' exit routine
END IF ' end check player missed
' calculate attack weapon bonus multiplier
IF Magic.Spell = False THEN ' verify attack by weapon, not spell
    Outpt = Nul ' reset output string
    Multiplier = 1 ' reset multiplier
    SELECT CASE Weapon6 ' select type of weapon being held
        CASE False ' no weapon being held
            Outpt = "Punch!" ' punch with fists
        CASE ELSE ' weapon is being held
            SELECT CASE RND ' select random chance
                CASE IS > .96 ' highest probability
                    Outpt = "Deathly damage!" ' make damage message
                    Multiplier = 4 ' set multiplier
                CASE IS > .91 ' median probability
                    Outpt = "Triple damage!" ' make damage message
                    Multiplier = 3 ' set multiplier
                CASE IS > .86 ' lowest probability
                    Outpt = "Double damage!" ' make damage message
                    Multiplier = 2 ' set multiplier
            END SELECT ' end selection of random chance for weapon
    END SELECT ' end selection of weapon type
END IF ' end verify weapon attack
SELECT CASE Last.Command.Number ' selection of attack command number
    CASE Killed ' kill
        IF Outpt <> Nul THEN ' check bonus hit string
            CALL IO.O ' send bonus output message
        END IF ' end check bonus flag
    CASE Lunge, Dodge ' lunge, dodge
        Multiplier = 2 ' store lunge/dodge multiplier
        Previous.Command = Last.Command.Number ' store last command attack number
    CASE Backstab ' backstab
        IF Hidden.Player THEN ' verify player hidden for backstab
            Multiplier = Multiplier + 1 ' increment hit multiplier
        ELSE ' player not hidden
            Outpt = "The " + Outpts + " sees you!" ' make message
            CALL IO.O ' send message
            Multiplier = Multiplier - 1 ' decrement multiplier
        END IF ' end verify hidden player
    CASE Thrust ' thrust
        Multiplier = Multiplier + 1 ' increment hit multiplier for thrust
        Previous.Command = Last.Command.Number ' store last command attack number
    CASE Charge ' charge
        Outpt = "Argh! You charge towards the " + Outpts + "!" ' make hit message
        CALL IO.O ' send hit message
        Multiplier = Multiplier + 1 ' increment hit multiplier
        Previous.Command = Last.Command.Number ' store last command attack number
    CASE Counter ' counter
        Multiplier = Multiplier + 1 ' increment multiplier for counter attack
    CASE Pummel ' pummel
        Outpt = "Your fists fly at the monster!" ' make hit message
        CALL IO.O ' send hit message
        Multiplier = 2 ' set hit multiplier
    CASE Beat, Punch ' beat, punch
        Multiplier = 2 ' set hit multiplier
        Previous.Command = Last.Command.Number ' store last command attack number
    CASE Bewitch ' bewitch
        Multiplier = Multiplier + 1 ' increment hit multiplier for bewitch attack
        Previous.Command = Last.Command.Number ' store last command attack number
    CASE Bewilder ' bewilder
        Multiplier = Multiplier + 1 ' increment hit multiplier for bewilder attack
        Previous.Command = Last.Command.Number ' store last command attack number
END SELECT ' end selection of attack command number
IF Magic.Spell = False THEN ' verify weapon being used, not spell cast
    IF Weapon6 THEN ' check player holding weapon
        ' decrement weapon strikes remaining.
        ' store weapon strikes remaining
        Charges.Number = GetUserRecordCharges(Weapon6)
        IF Charges.Number > False THEN ' check weapon strikes remaining
            Charges.Number = Charges.Number - 1 ' decrement weapon strikes
        END IF ' end check strikes remaining
        ' store weapon strikes in user record
        CALL PutUserRecordCharges(Weapon6, Charges.Number)
        IF Charges.Number = False THEN ' verify weapon strikes
            CALL Read.Record(TreasureFile, GetUserRecordInv(Weapon6)) ' get record
            Inpt = TreasureRecord.ShortName ' store weapon mnemonic
            Inpt = RTRIM$(Inpt) ' trim mnemonic
            Inpt = LCASE$(Inpt) ' lowercase mnemonic
            Outpt = "Your " + Inpt + " breaks in half!" ' make weapon message
            CALL IO.O ' send weapon message
            CALL Discard.Inventory(Weapon6, False) ' weapon falls to ground
        END IF ' end verify remaining weapon strikes
    END IF ' end check player holding weapon
END IF ' end verify weapon being used
Magic.Spell = True ' reset spell cast flag
CALL Hit.Monster ' get actual hits on monster
END SUB ' end routine to attack monster

REM * routine to send player to jail room number.

SUB Jail
Outpt = "You were just thrown in jail!" ' make message
CALL IO.O ' send jail message
IF Jail.Room.Number = 0! THEN ' check jail room number
    Next.Room = 1! ' reset player room number
ELSE ' check jail room number
    Next.Room = Jail.Room.Number ' store new room number
END IF ' end check jail room number
CALL Enter.Room ' send player to jail room
END SUB ' end routine to jail player

REM * routine to decrease the number of monsters in the room.

SUB Reduce.Monsters
Inpt = Parsed.Command1 ' store command parameter
Inpt = RTRIM$(Inpt) ' trim command parameter
New.Monsters = INT(VAL(Inpt)) ' convert parameter to integer
' compare number to reduce
IF New.Monsters >= False AND New.Monsters < Number.Monsters THEN
    Number.Monsters = New.Monsters ' reduce monsters in room
    Outpt = "Number of Monsters reduced to:" + STR$(Number.Monsters) + ".."
    CALL IO.O ' send monsters reduced message
END IF ' end compare number to reduce
END SUB ' end routine to decrease monsters in room

REM * routine to kill off a monster.

SUB Kill.Monster
Monster.Number = False ' reset monster number
CALL Check.Monster ' get monster number from command parameter
IF Monster.Number THEN ' compare monster name found to kill
    Graphics.Off = True ' reset color
    Outpt = "Evil Laughter Sounds From Above..." ' make message
    CALL IO.O ' send message
    Outpt = "   A Bolt Of Lightning Strikes..." ' make message
    CALL IO.O ' send message
    Graphics.Off = False ' reset color
    Outpt = "The " + Outpts + " was just struck dead!" ' make monster died message
    CALL IO.O ' send monster died message
    CALL Monster.Died ' kill monster
    EXIT SUB ' exit routine
END IF ' end compare monster name
Outpt = "You can't kill that!" ' make error message
CALL IO.O ' send error message
END SUB ' end routine to kill monster

REM * routine to attack monster.
REM * input variables:
REM *   Monster.Number - number of monster in array to attack.
REM *   Outpts - name of monster being attacked.
REM * processing variables:
REM *   Calculate# - maximum possible hits on monster.

SUB Hit.Monster
Hidden.Player = False ' reset hidden player
IF Last.Command.Number = PsiMode THEN ' compare attack type
    ' calculate hits on monster for psionic attack
    Calculate# = CDBL(INT(RND * GetUserRecordStats(2) + GetUserRecordStats(3) * (Psi.Attack.Mode + Multiplier)))
ELSE ' compare attack type
    ' calculate hits on monster for normal attack
    Calculate# = CDBL(INT((RND * GetUserRecordStats(1) + UserRecord.Level / 2) * (Multiplier + (GetUserRecordWeapons(UserRecord.Proficiency) / 100)) + Weapon2))
END IF ' end compare attack type
Magic.Spell = False ' reset magic spell being used to attack
IF Calculate# <= False THEN ' check hits are zero
    Outpt = "You missed!" ' make missed message
    CALL IO.O ' send missed message
    EXIT SUB ' exit routine
END IF ' end check hits
Outpt = "You hit the " + Outpts + " for" + STR$(Calculate#) + " hits!" ' hits message
CALL IO.O ' send hits message
CALL Monster.Flees(Ran.Away) ' check if monster flees
IF Ran.Away THEN ' compare flee
    CALL Monster.Died ' routine for dead monster
    EXIT SUB ' exit routine
END IF ' end compare flee
' magical monsters don't attack until hit.
' check magical monster attacked
IF MonsterArray(Monster.Number).Magic = True THEN
    MonsterArray(Monster.Number).Magic = -2 ' set magical monster to attack also
END IF ' end check attacking a magical monster
' decrement the hits on the monster
MonsterArray(Monster.Number).Hits = MonsterArray(Monster.Number).Hits - Calculate#
IF MonsterArray(Monster.Number).Hits <= False THEN ' check monster still alive
    Outpt = "You just killed the " + Outpts + "!" ' make message
    CALL IO.O ' send message
    CALL Monster.Died ' routine for dead monster
END IF ' end check live monster
END SUB ' end routine to attack monster
 
REM * routine to calculate hits causes monster to run.
REM * output variables:
REM *   Ran.Away - true if monster flees.
REM * input variables:
REM *   Outpts - name of monster.

SUB Monster.Flees (Ran.Away)
Ran.Away = False ' reset return variable
IF MonsterArray(Monster.Number).Magic >= False THEN ' nonmagical monster
    IF MonsterArray(Monster.Number).Permanent = False THEN ' nonpermanent monster
        ' calculate hits on monster
        IF MonsterArray(Monster.Number).Hits - Calculate# > False THEN
            ' calculate 25% of hits
            IF Calculate# > (MonsterArray(Monster.Number).Hits * .75) THEN
                Outpt = "The " + Outpts + " flees from your attack!" ' make message
                CALL IO.O ' send message
                MonsterArray(Monster.Number).Experience = INT(MonsterArray(Monster.Number).Experience / 2) ' half experience
                Ran.Away = True ' reset return variable
            END IF ' end calculate hits on monster is 25%
        END IF ' end check if monster has hits remaining after attack
    END IF ' end verify permanent monster
END IF ' end verify magical monster
END SUB ' end routine to check monster flees

REM * routine for dead monster.
REM * input variables:
REM *   Monster.Number - index of monster to array.
REM * processing variables:
REM *   Monsters.Killed - total monsters killed by player this session.
REM *   Choice - index of monster file.
REM *   Weapon6, Weapon10 - proficiency calculations.

SUB Monster.Died
' check permanent nonplayer
IF MonsterArray(Monster.Number).Permanent < True THEN
    CALL Remove.Monster ' nonplayer only leaves room
    EXIT SUB ' exit routine
END IF ' end check nonplayer
Graphics.Off = True ' reset color
Last.Command.Number = False ' reset last attack command number
Outpt = "On it you find " ' format message prefix
Carriage.Return = True ' disable cr/lf
CALL IO.O ' send message prefix
Number.Items = False ' number of items
' loop through monster array for all treasure to leave on ground
FOR Treasure.Index = 1 TO 5 ' loop through monster inventory
    ' get next inventory
    Treasure.Number = GetMonsterRecordTreasure(Treasure.Index)
    IF Treasure.Number > False AND Treasure.Number <= LOF(TreasureFile) / LEN(TreasureRecord) THEN ' file bounds
        CALL Read.Record(TreasureFile, Treasure.Number) ' get treasure record
        CALL TreasureCharges(Charges.Amount) ' routine to get treasure charges
        ' add inventory to ground
        CALL Add.Room.Treasure(Treasure.Number, Charges.Amount, False, Item.Added)
        IF Item.Added THEN ' compare return variable to add to room
            Carriage.Return = True ' disable cr/lf
            CALL IO.O ' send message of previous treasure name
            ' format treasure name
            Outpt = RTRIM$(TreasureRecord.TreasureName) + ", "
            Number.Items = Number.Items + 1 ' increment number of items
        END IF ' end compare reutnr variable
    END IF ' end compare file bounds
NEXT ' end loop through monster inventory
IF Number.Items = False THEN ' compare number of items
    Outpt = "nothing.." ' format message
ELSE ' compare items
    Outpt = LEFT$(Outpt, LEN(Outpt) - 2) + "." ' strip comma, add period
    IF Number.Items > 1 THEN ' compare more than one item
        Outpt = "and " + Outpt ' append to message
    END IF ' end compare number of items
END IF ' end compare number of items
CALL IO.O ' send last part of message
CALL Share.Room.Record(Room) ' write the room record
Monsters.Killed = Monsters.Killed + 1 ' increment player monsters killed session
' increment player record
UserRecord.MonstersKilled = UserRecord.MonstersKilled + 1
Choice = MonsterIndex(Monster.Number) ' store monster record number
IF MonsterArray(Monster.Number).Permanent = True THEN ' check permanent monster
    CALL Read.Record(MonsterFile, Choice) ' get permanent monster record
    MonsterRecord.Permanent = False ' clear permanent monster flag
    CALL Share.Record(MonsterFile, Choice) ' put permanent monster record
END IF ' end check permanent monster
IF Weapon6 AND Weapon10 THEN ' check player is holding a weapon
    ' calculate difference between player and monster level
    Level = MonsterArray(Monster.Number).Level - UserRecord.Level
    IF Level > False THEN ' check level is positive
        ' store player weapon proficiency number
        Proficient.Weapon = UserRecord.Proficiency
        ' check player is holding a weapon in his weapon proficiency
        IF Weapon10 = Proficient.Weapon THEN
            Level = Level * 10 ' compute proficiency for kill
        ELSE ' compare weapon proficiency for weapon held
            Level = Level * 5 ' compute proficiency for kill
        END IF ' end compare weapon proficiency held
        IF Level > 100 THEN ' check calculated proficiency over 100 percent
            Level = 100 ' reset to 100 percent
        END IF ' end check maximum proficiency percent
        ' compare to current
        IF Level > GetUserRecordWeapons(Proficient.Weapon) THEN
            ' proficiency, store new percent
            CALL PutUserRecordWeapons(Proficient.Weapon, Level)
            Outpt = "Your " + RTRIM$(WeaponTypeName(Proficient.Weapon)) + " weapon proficiency is now" + STR$(Level) + " percent!" ' make message
            CALL IO.O ' send weapon message
        END IF ' end check maximum percent
    END IF ' end check attack level
END IF ' end check player holding weapon
Exp.Points# = MonsterArray(Monster.Number).Experience ' store monster experience
Gold.Points# = MonsterArray(Monster.Number).Gold ' store monster gold
IF Gold.Points# <= False THEN ' compare monster gold
    Gold.Points# = 10 ' set to minimum
END IF ' end compare monster gold
Outpt = "You gained" + STR$(Exp.Points#) + " experience points" ' make message
IF Gold.Points# > False THEN ' compare gold
    Outpt = Outpt + " and" + STR$(Gold.Points#) + " gold" ' append gold message
END IF ' end compare gold
Outpt = Outpt + "!" ' append message
CALL IO.O ' send experience and gold message
UserRecord.Gold = UserRecord.Gold + Gold.Points# ' increment player gold
' increment player experience
UserRecord.Experience = UserRecord.Experience + Exp.Points#
CALL Remove.Monster ' take monster out of active monster array
IF UserRecord.Level > False THEN ' verify player level
    ' calculate experience needed to reach next level
    CALL Experience(Exp.Required#)
    IF UserRecord.Experience >= Exp.Required# THEN ' compare player experience
        CALL Gold(Gold.Required#) ' routine calculates gold needed for level
        IF UserRecord.Gold >= Gold.Required# THEN ' compare to player gold
            CALL Is.Training.Room(In.Room) ' check training room
            IF In.Room THEN ' check flag
                CALL Train.Stats ' routine to train for next level
            END IF ' end check flag
        END IF ' end check player gold required
    END IF ' end check player experience required
END IF ' end check player level
END SUB ' end monster death routine

REM * routine to remove a monster from the monster arrays.
REM * input variables:
REM *   Monster.Number - number of monster to remove.
REM * output variables:
REM *   Number.Monsters - number of monster currently in room.

SUB Remove.Monster
' loop through monster array from monster to remove
FOR Array.Index = Monster.Number TO 19 ' pack array
    ' move next array element
    MonsterIndex(Array.Index) = MonsterIndex(Array.Index + 1)
    ' move next array element
    MonsterArray(Array.Index) = MonsterArray(Array.Index + 1)
NEXT ' end loop through all monsters in arrays
Number.Monsters = Number.Monsters - 1 ' decrement number of monsters in room
IF Number.Monsters < False THEN ' compare number of monsters in room
    Number.Monsters = False ' reset numberof monsters to zero
END IF ' end compare number of monsters
END SUB ' end routine to remove a monster

REM * routine for monsters to attack player.

SUB Monster.Attack
Monster.Cycle = Monster.Cycle + 1 ' increment attack counter
IF Monster.Cycle >= Room.Monster.Rate THEN ' compare attack counter to
    Monster.Cycle = False ' room monster attack rate counter, reset to zero
END IF ' end compare rate counters
IF Number.Monsters = False THEN ' check monsters in room
    EXIT SUB ' exit routine if no monsters
END IF ' end check monsters in room
IF Room = 1! THEN ' check player in safe room
    EXIT SUB ' exit routine for safe room
END IF ' end check player in safe room
FOR Monster.Number = 1 TO Number.Monsters ' loop through all monsters in room
    ' get monster befuddle rounds
    Inactive.Rounds = MonsterArray(Monster.Number).Magic
    IF Inactive.Rounds >= False THEN ' compare number of befuddle rounds
        IF GetUserRecordStats(6) <= 1 THEN ' check player piety
            Inactive.Rounds = False ' monsters attack on sight for low piety
        END IF ' end check player piety
        ' compare inactive befuddle rounds for monster
        IF Inactive.Rounds > False THEN
            Inactive.Rounds = Inactive.Rounds - 1 ' decrement befuddle rounds
        END IF ' end compare monster befuddle rounds
        ' store monster befuddle rounds
        MonsterArray(Monster.Number).Magic = Inactive.Rounds
    END IF ' end compare number of monster befuddle rounds
    IF Inactive.Rounds < True THEN ' check if magical monster is attacking
        ' reset befuddle rounds remaining for monster to zero
        Inactive.Rounds = False
    END IF ' end check magical monster is attacking
    ' routine for monster hit regeneration
    IF MonsterArray(Monster.Number).Magic <= True THEN ' check magical monster
        IF MonsterArray(Monster.Number).Level >= 15 THEN ' check monster level
            IF Monster.Cycle = False THEN ' compare room monster update rate
                Calculate# = MonsterArray(Monster.Number).Hits ' store monster hits
                ' regenrate hits by ten percent
                Calculate# = CDBL(INT(Calculate# + Calculate# * .1))
                IF Calculate# > MaxInt THEN ' check max integer
                    ' store new monster hits
                    MonsterArray(Monster.Number).Hits = MaxInt
                END IF ' end check integer
                IF Calculate# <= MaxInt THEN ' check max integer
                    ' store new monster hits
                    MonsterArray(Monster.Number).Hits = Calculate#
                END IF ' end check integer
            END IF ' end compare room monster update rate
        END IF ' end check monster level
    END IF ' end check magical monster
    ' continue with monster attack if player is not hidden,
    ' and monster is not befuddled.
    ' continue with monster attack
    IF Hidden.Player = False AND Inactive.Rounds = False THEN
        Outpts = MonsterArray(Monster.Number).MonsterName ' store monster name
        Outpts = RTRIM$(Outpts) ' trim monster name
        Outpts = LCASE$(Outpts) ' lowercase monster name
        ' compute monster attack multiplier is monster level plus one
        Multiplier = MonsterArray(Monster.Number).Level + 1 ' compute multiplier
        IF MonsterArray(Monster.Number).Poison THEN ' check monster is poisonous
            ' compute random chance of monster poison percent
            IF RND < (MonsterArray(Monster.Number).PoisonPercent / 100) THEN
                Outpt = "The " + Outpts + " casts a poison spell!" ' make message
                CALL IO.O ' send poison message
                ' check player is wearing antipoison ring
                IF Weapon8 = 1 THEN ' check for ring
                    Ring.Charges = GetUserRecordCharges(Weapon7) ' store ring strikes
                    ' decrement strikes
                    Ring.Charges = Ring.Charges - MonsterArray(Monster.Number).Level
                    IF Ring.Charges < False THEN ' compare strikes remaining
                        Ring.Charges = False ' store zero strikes
                    END IF ' end compare strikes
                    ' store strikes in record
                    CALL PutUserRecordCharges(Weapon7, Ring.Charges)
                    Outpt = "Your ring absorbs the poison spell!" ' make message
                    CALL IO.O ' send ring protection message
                    IF Ring.Charges = False THEN ' compare ring charges remaining
                        Outpt = "Your ring disintegrated!" ' make message
                        CALL IO.O ' send ring message
                        CALL Discard.Inventory(Weapon7, True) ' ring falls to ground
                    END IF ' end compare ring charges
                ELSE ' check for ring
                    UserRecord.Poison = True ' reset user poisoned flag
                    Outpt = "You've been poisoned!" ' make message
                    CALL IO.O ' send poisoned message
                END IF ' end compare ring
            END IF ' end compute monster poisons
        END IF ' end compare monster is poisonous
        ' check monster is undead
        IF MonsterArray(Monster.Number).LevelDrain THEN
            ' compute random chance vs. undead monster leveldrain percent
            IF RND < (MonsterArray(Monster.Number).DrainPercent / 100) THEN ' compute
                Outpt = "The " + Outpts + " casts a level drain spell!" ' message
                CALL IO.O ' send level drain message
                ' check player is wearing anti level drain ring
                IF Weapon8 = 2 THEN ' check for ring
                    Ring.Charges = GetUserRecordCharges(Weapon7) ' store ring strikes
                    ' decrement ring charges
                    Ring.Charges = Ring.Charges - MonsterArray(Monster.Number).Level
                    IF Ring.Charges < False THEN ' compare strikes
                        Ring.Charges = False ' reset ring strikes
                    END IF ' end compare strikes
                    ' store srikes in record
                    CALL PutUserRecordCharges(Weapon7, Ring.Charges)
                    Outpt = "Your ring absorbs the level drain!" ' make message
                    CALL IO.O ' send ring message
                    IF Ring.Charges = False THEN ' check ring charges
                        Outpt = "Your ring disintegrated!" ' make message
                        CALL IO.O ' send ring charges message
                        CALL Discard.Inventory(Weapon7, True) ' ring falls to ground
                    END IF ' end check remaining ring charges
                ELSE ' check player wearing ring
                    UserRecord.Level = UserRecord.Level - 1 ' decrement player level
                    CALL New.Stats ' get updated statistics
                    Outpt = "You've been drained a level!" ' make message
                    CALL IO.O ' send leveldrain message
                    IF UserRecord.Level <= False THEN ' compare player level to dead
                        UserRecord.Level = 1 ' reset player level
                        CALL The.Or.An ' get monster prefix
                        ' store message for death routine
                        Message1 = "You were just killed by " + Prefix1 + " " + Outpts + "!"
                        CALL Player.Died ' routine for dead player
                        EXIT FOR ' exit monster attack loop
                    END IF ' end compare player level
                END IF ' end check player has ring
            END IF ' end compute random chance
        END IF ' end check monster is undead
        Magic.Spell = False ' reset magic spell being cast
        ' get monster spell cast number
        Spell.Number = MonsterArray(Monster.Number).Spell
        ' file bounds
        IF Spell.Number > False AND Spell.Number <= LOF(SpellFile) / LEN(SpellRecord) THEN
            CALL Read.Record(SpellFile, Spell.Number) ' get spell record
            Magic.Spell = SpellRecord.SpellType ' store monster spell cast type
            Multiplier = INT(MonsterArray(Monster.Number).Level) ' calculate spell
            Multiplier = Multiplier + INT(SpellRecord.Level / 2 + .5) ' attack multiplier
            IF Magic.Spell = Offense THEN ' compare spell type to offense attack
                ' compute random chance to monster spell percent
                IF RND < (MonsterArray(Monster.Number).SpellPercent / 100) THEN
                    ' make message of spell monster casts
                    Outpt = "The " + Outpts + " casts a " + RTRIM$(SpellRecord.SpellName) + " spell!" ' message
                    CALL IO.O ' send cast message
                    ' check player is wearing antispell ring
                    IF Weapon8 = 3 THEN ' check ring
                        ' check ring is generic antispell or the spell cast
                        IF Weapon9 = True OR Weapon9 = Spell.Number THEN ' check ring
                            ' store ring strikes
                            Ring.Charges = GetUserRecordCharges(Weapon7)
                            ' subtract spell level
                            Ring.Charges = Ring.Charges - SpellRecord.Level
                            IF Ring.Charges < False THEN ' compare strikes
                                Ring.Charges = False ' reset strikes
                            END IF ' end compare strikes
                            CALL PutUserRecordCharges(Weapon7, Ring.Charges) ' store strikes
                            Outpt = "Your ring absorbs the spell!" ' make message
                            CALL IO.O ' send ring message
                            ' check remaining ring charges
                            IF Ring.Charges = False THEN ' compare
                                Outpt = "Your ring disintegrated!" ' make message
                                CALL IO.O ' send ring message
                                CALL Discard.Inventory(Weapon7, True) ' falls to ground
                            END IF ' end compare ring charges
                        END IF ' end ring type
                    END IF ' end check ring
                END IF ' end computer random chance
            END IF ' end compare spell attack type
        END IF ' end check spell file bounds
        ' check if monster is poisoned
        IF MonsterArray(Monster.Number).Poison > False THEN ' check poisoned
            IF Monster.Cycle = False THEN ' compare rounds counter
                ' decrement monster poisoned rounds counter
                MonsterArray(Monster.Number).Poison = MonsterArray(Monster.Number).Poison - 1 ' decrement
                Outpt = "Poison drains the " + Outpts + "!" ' make message
                CALL IO.O ' send poisoned message
                ' calculate hits on monster from poison.
                ' store monster hits
                Monster.Hits# = MonsterArray(Monster.Number).Hits
                ' store monster level
                Monster.Level# = MonsterArray(Monster.Number).Level
                ' compute poison
                Poison.Hits# = INT(UserRecord.Level * GetUserRecordStats(7))
                ' compute poison
                Monster.Hits# = INT(Monster.Hits# - Poison.Hits# / Monster.Level#)
                ' store monster hits
                MonsterArray(Monster.Number).Hits = Monster.Hits#
                ' check monster hits
                IF MonsterArray(Monster.Number).Hits <= False THEN
                    Outpt = "The " + Outpts + " died from poison!" ' make message
                    CALL IO.O ' send died message
                    CALL Monster.Died ' routine for dead monster
                    EXIT FOR ' exit monster attack routine
                END IF ' end check monster hits
            END IF ' end compare rounds counter
        END IF ' end check monster is poisoned
        ' calculate monster misses
        Calculate# = (GetUserRecordStats(4) / 2) > (RND * 10 + MonsterArray(Monster.Number).Level / 10) ' calculate miss
        IF Calculate# THEN ' compare miss equation
            Outpt = "The " + Outpts + " fumbled!" ' make message
            CALL IO.O ' send missed message
        ELSE ' compare miss
            IF Previous.Command = True THEN ' compare previous attack command type
                Multiplier = Multiplier - 1 ' decrement monster multiplier
                Previous.Command = False ' reset last attack command type
            END IF ' end compare previous attack command type
            ' compare psionic monster
            IF MonsterArray(Monster.Number).Psionic = False THEN ' compare
                ' calculate normal hits on player
                Calculate# = CDBL(INT(RND * (MonsterArray(Monster.Number).Hits / 10 - (Weapon1 + Weapon3) / 2) * Multiplier))
            ELSE ' calculate psionic hits on player
                ' calculate psionic hits on player
                Calculate# = CDBL(INT(RND * (MonsterArray(Monster.Number).Hits / 10 - Psi.Defense.Mode) * MonsterArray(Monster.Number).Psionic))
            END IF ' end compare attack type
            CALL The.Or.An ' get prefix of monster name
            Prefix1 = "The " ' reset prefix
            CALL Get.Hits(Calculate#) ' routine to hit player
            IF Calculate# = True THEN ' return variable for dead player
                EXIT FOR ' exit monster attack loop
            END IF ' end check dead player
        END IF ' end compare monster miss equation
    END IF ' end compare monster befuddle rounds
NEXT ' end loop through all monsters in room
END SUB ' end routine for monster attacks

REM * routine to hit player.
REM * input variables:
REM *   Calculate# - actual hits.
REM * output variables:
REM *   Calculate# - true if player died.

SUB Get.Hits (Calculate#)
' selecting the previous attack command player used modifies the actual
' hits by the type of attack, and reduces or increases the actual hits
SELECT CASE Previous.Command ' select the previous attack command
    CASE Lunge ' lunge
        Calculate# = Calculate# - GetUserRecordStats(4) / 2 - GetUserRecordStats(1) / 2
    CASE Dodge ' dodge
        Calculate# = Calculate# - (GetUserRecordStats(4) + GetUserRecordStats(1)) / 2
    CASE Feint ' feint
        Calculate# = Calculate# - GetUserRecordStats(4) / 2 - GetUserRecordStats(1)
    CASE Parry ' parry
        Calculate# = Calculate# - GetUserRecordStats(4) - GetUserRecordStats(1)
    CASE Thrust ' thrust
        Calculate# = Calculate# + GetUserRecordStats(4) / 2 + GetUserRecordStats(1)
    CASE Charge ' charge
        Calculate# = Calculate# + GetUserRecordStats(4) + GetUserRecordStats(1)
    CASE Resist ' resist
        Calculate# = Calculate# - GetUserRecordStats(1) / 2
    CASE Beat ' beat
        Calculate# = Calculate# + GetUserRecordStats(4) / 2 + GetUserRecordStats(1) / 2
    CASE Punch ' punch
        Calculate# = Calculate# + (GetUserRecordStats(4) + GetUserRecordStats(1)) / 2
    CASE Bewitch ' bewitch
        Calculate# = INT(Calculate# / 2) - UserRecord.Beauty
    CASE Bewilder ' bewilder
        Calculate# = INT(Calculate# / 2) - UserRecord.Glamour
END SELECT ' end selection of previous attack command
IF Calculate# <= False THEN ' compare hits on player to miss
    Outpt = Prefix1 + Outpts + " missed!" ' make message
    CALL IO.O ' send miss message
    Calculate# = False ' return value for live player
    EXIT SUB ' exit routine
END IF ' end compare hits to miss
IF Monster.Number THEN ' check monster is attacking
    IF Magic.Spell = False THEN ' check monster using spell
        Outpt = Prefix1 + Outpts + " attacks you" ' make attack message
        ' check monster using psionic
        IF MonsterArray(Monster.Number).Psionic THEN
            Outpt = Outpt + " with psionics" ' append psionic attack message
        END IF ' end compare monster using psionics
        Outpt = Outpt + "!" ' append exclamation
        CALL IO.O ' send attack message
    END IF ' end compare monster using spell
END IF ' end compare monster attacking
IF UserRecord.Invisible THEN ' check player is invisible
    Outpt = "Your invisibility protects you!" ' make message
    CALL IO.O ' send invisible message
    Calculate# = False ' return value for live player
    EXIT SUB ' exit routine
END IF ' end compare invisible player
IF Vehicle2 THEN ' check if player is using a vehicle
    IF RND > .75 THEN ' random chance vehicle is hit
        CALL Read.Record(TreasureFile, Vehicle3) ' get vehicle treasure record
        Inpt = TreasureRecord.ShortName ' store vehicle name
        Inpt = RTRIM$(Inpt) ' trim name
        Inpt = LCASE$(Inpt) ' lowercase name
        Outpt = Prefix1 + Outpts + " hits your " + Inpt + " for" + STR$(Calculate#) + " hits!"
        CALL IO.O ' send message of hits on vehicle
        Vehicle2 = Vehicle2 - Calculate# ' decrement vehicle hits
        CALL PutRoomRecordTreCharges(Vehicle1, Vehicle2) ' put vehicle in room
        IF Vehicle2 <= False THEN ' compare remaining vehicle hits
            CALL PutRoomRecordTreasure(Vehicle1, Vehicle3) ' put vehicle in room
            CALL PutRoomRecordTreCharges(Vehicle1, False) ' put vehicle in room
            Vehicle1 = False ' remove vehicle from player
            Vehicle2 = False ' remove vehicle from player
            Vehicle3 = False ' remove vehicle from player
            Outpt = Prefix1 + Outpts + " damaged your " + Inpt + "!" ' make message
            CALL IO.O ' send vehicle message
        END IF ' end compare vehicle hits
    END IF ' end random chance to hit vehicle
    Calculate# = False ' return value for live player
    CALL Share.Room.Record(Room) ' write room record
    EXIT SUB ' exit routine
END IF ' end check player using vehicle
Outpt = Prefix1 + Outpts + " hits you for"
CALL Hit.Player(Calculate#) ' routine to decrement hits on player statistics
IF UserRecord.Vitality = False THEN ' check player vitality remaining
    Calculate# = True ' set player died flag
    EXIT SUB ' exit routine
END IF ' end check player died
Calculate# = False ' return value for live player
IF Weapon5 THEN ' check player holding shield
    ' store shield strikes remaining
    Charges.Number = GetUserRecordCharges(Weapon5)
    IF Charges.Number > False THEN ' check shield strikes
        Charges.Number = Charges.Number - 1 ' decrement shield strikes
    END IF ' end check shield strikes
    ' store shield strikes in user record
    CALL PutUserRecordCharges(Weapon5, Charges.Number)
    IF Charges.Number = False THEN ' compare shield charges
        ' get shield treasure record
        CALL Read.Record(TreasureFile, GetUserRecordInv(Weapon5))
        Inpt = TreasureRecord.ShortName ' store treasure name
        Inpt = RTRIM$(Inpt) ' trim name
        Inpt = LCASE$(Inpt) ' lowercase name
        Outpt = "Your " + Inpt + " breaks in half!" ' make message
        CALL IO.O ' send weapon message
        CALL Discard.Inventory(Weapon5, False) ' shield falls to ground
    END IF ' end compare remaining shield charges
END IF ' end check player holding shield
IF Weapon4 THEN ' check player is holding a armor
    Charges.Number = GetUserRecordCharges(Weapon4) ' store armor strikes remaining
    IF Charges.Number > False THEN ' compare armor strikes
        Charges.Number = Charges.Number - 1 ' decrement armor strikes
    END IF ' end compare armor strikes
    ' store armor strikes in user record
    CALL PutUserRecordCharges(Weapon4, Charges.Number)
    IF Charges.Number = False THEN ' check armor strikes remaining
        ' get treasure armor record
        CALL Read.Record(TreasureFile, GetUserRecordInv(Weapon4))
        Inpt = TreasureRecord.ShortName ' store treasure name
        Inpt = RTRIM$(Inpt) ' trim name
        Inpt = LCASE$(Inpt) ' lowercase name
        Outpt = "Your " + Inpt + " crumbles!" ' make message
        CALL IO.O ' send armor message
        CALL Discard.Inventory(Weapon4, False) ' falls to ground
    END IF ' end compare remaining armor charges
END IF ' end check player holding armor
END SUB ' end routine to get hits on player

REM * routine to decrement hits on player from fatigue and vitality.
REM * input variables:
REM *   Hit.Points - hits to subtract, positive from fatigue,
REM *   negative from vitality.

SUB Hit.Player (Hit.Points#)
IF Hit.Points# > False THEN ' check decrement from fatigue
    IF UserRecord.Fatigue - Hit.Points# >= False THEN ' compare the decrement
        ' subtract from fatigue
        UserRecord.Fatigue = UserRecord.Fatigue - Hit.Points#
        Outpt = Outpt + STR$(Hit.Points#) + " fatigue points!" ' make message
        CALL IO.O ' send hits message
        EXIT SUB ' exit routine
    END IF ' end compare the decrement
    ' get the difference from hits on fatigue
    Hit.Points# = Hit.Points# - UserRecord.Fatigue
    IF UserRecord.Fatigue > False THEN ' compare hits to fatigue remaining
        Outpt = Outpt + STR$(UserRecord.Fatigue) + " fatigue and" ' make message
    END IF ' end compare fatigue message, remaining difference hits vitlaity
    UserRecord.Fatigue = False ' reset player fatigue to zero
END IF ' end decrement from fatigue
Hit.Points# = ABS(Hit.Points#) ' convert hits on vitality to positive integer
IF Hit.Points# > False THEN ' compare vitiality hits
    Outpt = Outpt + STR$(Hit.Points#) + " vitality points!" ' make message
    CALL IO.O ' send hits message
    IF UserRecord.Vitality - Hit.Points# <= False THEN ' compare the decrement
        UserRecord.Vitality = False ' reset vitality
        Message1 = "You were just killed!" ' make died routine message
        CALL Player.Died ' routine for dead player
        EXIT SUB ' exit routine
    END IF ' end compare vitality decrement
    ' decrement player vitality
    UserRecord.Vitality = UserRecord.Vitality - Hit.Points#
END IF ' end compare vitality hits
END SUB ' end routine to decrement hits on player

REM * routine to fumble weapon and shield from player to ground.

SUB Fumble
IF Weapon5 OR Weapon6 THEN ' check if holding weapon or shield
    Outpt = "You fumbled! " ' make first message
    Inpt = "You dropped your " ' make second part of message
    Graphics.Off = True ' reset color
    Carriage.Return = True ' disable cr/lf
    CALL IO.O ' send first fumble message
    IF Weapon5 THEN ' check shield being held
        ' get shield treasure record
        CALL Read.Record(TreasureFile, GetUserRecordInv(Weapon5))
        Treasure.Name$ = TreasureRecord.ShortName ' get treasure name
        Treasure.Name$ = RTRIM$(Treasure.Name$) ' trim name
        Treasure.Name$ = LCASE$(Treasure.Name$) ' lowercase name
        CALL Discard.Inventory(Weapon5, False) ' falls to ground
        IF Weapon6 THEN ' check weapon also being held
            Outpt = " and " + Treasure.Name$ ' make next part of message
        ELSE ' check weapon
            Outpt = Inpt + Treasure.Name$ + "!" ' make message for shield only
        END IF ' end check weapon being held
    END IF ' end check shield held
    IF Weapon6 THEN ' check weapon being held
        ' get weapon treasure record
        CALL Read.Record(TreasureFile, GetUserRecordInv(Weapon6))
        Treasure.Name$ = TreasureRecord.ShortName ' get treasure name
        Treasure.Name$ = RTRIM$(Treasure.Name$) ' trim name
        Treasure.Name$ = LCASE$(Treasure.Name$) ' lowercase name
        Outpt = Inpt + Treasure.Name$ + Outpt + "!" ' make last part of message
        CALL Discard.Inventory(Weapon6, False) ' falls to ground
    END IF ' end check weapon held
    CALL IO.O ' send fumble message
    Graphics.Off = False ' reset color
END IF
END SUB ' end routine to fumble weapon and shield

REM * routine to get permanent monster encounters in a room.
REM * input variables:
REM *   Room - number of room player is currently in.

SUB Encounter.Permanent
' loop through all nonplayer records
FOR Monster.Index = 1 TO LOF(NonPlayerFile) / LEN(MonsterRecord)
    CALL Read.Record(NonPlayerFile, Monster.Index) ' get nonplayer record
    ' store nonplayer room
    Monster.Rooms$ = "," + RTRIM$(MonsterRecord.PluralName) + ","
    Player.Room$ = "," + MID$(STR$(Room), 2) + "," ' store current room
    ' verify nonplayer in current room
    IF INSTR(Monster.Rooms$, Player.Room$) THEN
        IF Number.Monsters < 20 THEN ' check maximum monsters in room
            Number.Monsters = Number.Monsters + 1 ' increment monsters in room
            MonsterIndex(Number.Monsters) = Monster.Index ' store nonplayer index
            MonsterArray(Number.Monsters) = MonsterRecord ' store nonplayer record
            MonsterArray(Number.Monsters).Magic = True ' set magical monster
            MonsterArray(Number.Monsters).Permanent = -2 ' set nonplayer flag
        END IF ' end check maximum monsters
    END IF ' end verify current room
NEXT ' end loop through nonplayer records
IF RoomRecord.MonsterClass THEN ' check room monster class number
    FOR Monsters = 1 TO 10 ' loop through room monsterclass monsters
        ' get monster number
        Choice = Monster.Class(RoomRecord.MonsterClass, Monsters)
        IF Choice > False AND Choice <= LOF(MonsterFile) / LEN(MonsterRecord) THEN ' bounds
            CALL Read.Record(MonsterFile, Choice) ' get monster record
            IF MonsterRecord.Permanent = True THEN ' check if monster permanent
                Number.Appearing = 1 ' set number of monsters to appear to one
                CALL Get.Monster.Stats ' get the monster
            END IF ' end check permanent monster
        END IF ' end check file bounds
    NEXT ' end loop through room monsterclass
END IF ' end check room monster class number
END SUB ' end routine to get permanent monsters

REM * routine to get one monster.
REM * input variables:
REM *   Parsed.Command1 - name or number of monster.
REM * working variables:
REM *   Choice - is the monster number.

SUB Summon.Monster
IF Room = 1! THEN ' check player in safe room
    Outpt = "This is a safe haven!" ' display message
    CALL IO.O ' send message
    EXIT SUB ' exit routine for safe room
END IF ' end check player in safe room
Choice = False ' reset monster number
Inpt = Parsed.Command1 ' store command parameter
CALL Parse.Num(Inpt, Parse.Value) ' parse number
Parse.Count = False ' reset monster found counter
FOR Monster.Index = 1 TO LOF(MonsterFile) / LEN(MonsterRecord) 'loop through file
    CALL Read.Record(MonsterFile, Monster.Index) ' get monster record
    ' compare monster record name to input monster name,
    ' truncated to input length
    IF LEFT$(MonsterRecord.MonsterName, LEN(Inpt)) = Inpt THEN ' compare
        Parse.Count = Parse.Count + 1 ' increment monster found counter
        ' compare parsed number of monster to search for
        IF Parse.Value = False OR Parse.Count = Parse.Value THEN ' compare
            EXIT FOR ' exit routine
        END IF ' end compare counters
    END IF ' end compare monster names
NEXT ' end loop through monster file
IF Parse.Count = False THEN ' check monster found
    Monster.Index = INT(VAL(Inpt)) ' convert input to integer
    ' file bounds
    IF Monster.Index > False AND Monster.Index <= LOF(MonsterFile) / LEN(MonsterRecord) THEN
        Choice = Monster.Index ' store monster number
    END IF ' end check file bounds
ELSE ' check file bounds
    Choice = Monster.Index ' store monster number
END IF ' end check file bounds
IF Choice > False THEN ' check monster number found
    CALL Get.Monster ' get the monster
END IF ' end check monster number found
END SUB ' end routine to get a monster

REM * routine to get a random monster.
REM * processing variables:
REM *   Choice - is the monster number.

SUB Call.Monster
IF Room = 1! THEN ' check player in safe room
    Outpt = "This is a safe haven!" ' display message
    CALL IO.O ' send message
    EXIT SUB ' exit routine for safe room
END IF ' end check player in safe room
Monster.Class = RoomRecord.MonsterClass ' store room monster class
' check monster class bounds
IF Monster.Class > False AND Monster.Class <= Monclass.Max THEN
    ' calculate a random monster from the room monsterclass
    Choice = Monster.Class(Monster.Class, INT(RND * 10 + 1)) ' random
    CALL Get.Monster ' get the monster
END IF ' end check monsterclass bounds
END SUB ' end routine to get a random monster

REM * routine to get a random monster from the room monster class, computed
REM * from the monster encounter round rate counter, and the monster
REM * percentage.
REM * input variables:
REM *   Room - number of the current room.
REM *   Monster.Rate1 - counter of rounds from file.
REM *   Monster.Rate2 - counter of rounds for monster rate.

SUB Encounter.Monster
IF Room = 1! THEN ' check safe room
    EXIT SUB ' exit routine
END IF ' end check safe room
Monster.Class = RoomRecord.MonsterClass ' store room monster class
' check monster class bounds
IF Monster.Class > False AND Monster.Class <= Monclass.Max THEN
    IF Monster.Rate1 = MaxInt THEN ' check rate counter maximum integer
        Monster.Rate1 = 1 ' reset rate counter
    ELSE ' check rate
        Monster.Rate1 = Monster.Rate1 + 1 ' increment rate
    END IF ' end check room monster counter rate
    FOR Monsters = 1 TO 10 ' loop through monsters with specified percentages
        IF Monster.Rate(Monster.Class, Monsters) THEN ' check monster rate
            ' verify monster rate equals room rate
            IF Monster.Rate1 / Monster.Rate(Monster.Class, Monsters) = INT(Monster.Rate1 / Monster.Rate(Monster.Class, Monsters)) THEN
                ' verify random monster rate percentage
                IF RND < (Monster.Percent(Monster.Class, Monsters) / 100) THEN ' verify
                    ' store monster number
                    Choice = Monster.Class(Monster.Class, Monsters)
                    CALL Get.Monster ' get the monster
                END IF ' end verify random rate
            END IF ' end verify monster rate percent
        END IF ' end check monster rate
    NEXT ' end loop through monster percentages
    Monster.Rate2 = Monster.Rate2 + 1 ' increment monster rate for room
    IF Monster.Rate2 >= Room.Monster.Rate THEN ' check monster rate
        Monster.Rate2 = False ' reset monster rate
        Choice = Monster.Class(Monster.Class, INT(RND * 10 + 1)) ' get random monster
        IF MonsterRecord.Rate = False THEN ' verify monster has specific rate
            IF RND < .5 THEN ' random chance for encounter
                CALL Get.Monster ' get the monster
            END IF ' end random chance
        END IF ' end verify monster specific rate
    END IF ' end monster rate
END IF ' end check room has monster class
END SUB ' end routine to random monster from rates and percentages

REM * routine to get monsters.
REM * input variables:
REM *   Choice - the number of the monster record.
REM * processing variables:
REM *   Number.Appearing - number of monsters to encounter.

SUB Get.Monster
IF Choice > False AND Choice <= LOF(MonsterFile) / LEN(MonsterRecord) THEN ' file bounds
    CALL Read.Record(MonsterFile, Choice) ' get the monster record
    IF MonsterRecord.Permanent = False THEN ' verify monster not permanent
        ' calculate random number of monsters to encounter
        Number.Appearing = INT(RND * MonsterRecord.NumberAppearing) + 1 ' calculate
        WHILE Number.Appearing + Number.Monsters > 20 ' loop to compare maximum
            Number.Appearing = Number.Appearing - 1 ' monsters in room, decrementing
        WEND ' number appearing while room maximum is not exceeded
        SELECT CASE Number.Appearing ' selection of number encountered
            CASE 0 ' number appearing is greater than room can hold
                EXIT SUB ' exit routine
            CASE 1 ' format message for one monster
                ' compare monster name starting with a vowel
                IF INSTR("aeiou", LEFT$(MonsterRecord.MonsterName, 1)) THEN
                    Outpt = "an " ' format prefix to monster name
                ELSE ' check vowel
                    Outpt = "a " ' format prefix to monster name
                END IF ' end compare monster name
                Outpt = " " + Outpt + RTRIM$(MonsterRecord.MonsterName)
            CASE ELSE ' format message for many monsters
                Outpt = STR$(Number.Appearing) + " " + RTRIM$(MonsterRecord.PluralName)
        END SELECT ' end selection of monsters appearing
        Outpt = "You encounter" + Outpt + "!" ' make message
        CALL IO.O ' send encounter message
        CALL Get.Monster.Stats ' add the monster to the arrays
    END IF ' end verify not permanent
END IF ' end check file bounds
END SUB ' end routine to get monsters

REM * routine to store monster statistics for encountered monster number.
REM * input variables:
REM *   Choice - is the monster number to the monster file.
REM *   Number.Appearing - the number of monsters of type choice to encounter.

SUB Get.Monster.Stats
IF Number.Monsters >= 20 THEN ' check room is full of monsters
    EXIT SUB ' exit routine
END IF ' end check room full
FOR Monsters = 1 TO Number.Appearing ' loop through all number appearing
    IF Number.Monsters < 20 THEN ' check room full of monsters
        Number.Monsters = Number.Monsters + 1 ' increment the number of monsters
        MonsterIndex(Number.Monsters) = Choice ' store the monster record choice
        MonsterArray(Number.Monsters) = MonsterRecord ' store the monster record
        IF MonsterArray(Number.Monsters).Psionic THEN ' verify psionic monster
            ' get monster psionic spell number
            Spell.Number = MonsterArray(Number.Monsters).PsionicSpell
            MonsterArray(Number.Monsters).Psionic = False ' spell from array and
            IF Spell.Number > False AND Spell.Number <= LOF(SpellFile) / LEN(SpellRecord) THEN ' reset with
                ' psionic spell number from spell file
                CALL Read.Record(SpellFile, Spell.Number)
                IF SpellRecord.Psionic THEN ' check spell record is psionic
                    IF SpellRecord.PsionicMode = 1 THEN ' check psionic is offense
                        ' store the psionic spell level in the monster array
                        MonsterArray(Number.Monsters).Psionic = SpellRecord.Level
                    END IF ' end check spell psionic type
                END IF ' end check spell is psionic
            END IF ' end check spell file bounds
        END IF ' end check monster is psionic
        IF MonsterArray(Number.Monsters).Permanent = False THEN ' check permanent
            FOR Treasure.Index = 1 TO 5 ' loop through all monster inventory
                IF RND > .75 THEN ' random chance
                    ' clear monster treasure element
                    CALL PutMonsterRecordTreasure(Treasure.Index, False)
                END IF ' end random chance
            NEXT ' end loop through monster inventory
        END IF ' end check nonpermanent monster
    END IF ' end check maximum room monsters
NEXT ' end loop through number appearing of monster type choice
Action.Number = RoomRecord.Action ' store room action number
' check file bounds
IF Action.Number > False AND Action.Number <= LOF(ActionFile) / LEN(ActionRecord) THEN
    CALL Read.Record(ActionFile, Action.Number) ' read action record
    ' check room action to monster choice
    IF ActionRecord.MonsterTrigger = Choice THEN
        ' check room action not talk activated
        IF ActionRecord.MonsterTalk = False THEN
            Action1$ = "As you step forward," ' make action message one
            Action2$ = "The monster hits you for" ' make action message two
            ' routine to activate actions by specific trigger
            CALL Actions(Action1$, Action2$)
        END IF ' end check room action
    END IF ' end check room has monster trigger
END IF ' end check file bounds
END SUB ' end routine to store encountered monster statistics

REM * routine for dead player.

SUB Player.Died
UserRecord.Poison = False ' reset player poisoned
Vehicle2 = False ' reset vehicle
Vehicle3 = False ' reset vehicle
Weapon1 = False ' reset weapons, shield, armor, and rings being held/worn
Weapon2 = False ' reset weapons, shield, armor, and rings being held/worn
Weapon3 = False ' reset weapons, shield, armor, and rings being held/worn
Weapon4 = False ' reset weapons, shield, armor, and rings being held/worn
Weapon5 = False ' reset weapons, shield, armor, and rings being held/worn
Weapon6 = False ' reset weapons, shield, armor, and rings being held/worn
Weapon7 = False ' reset weapons, shield, armor, and rings being held/worn
Weapon8 = False ' reset weapons, shield, armor, and rings being held/worn
Weapon9 = False ' reset weapons, shield, armor, and rings being held/worn
Weapon10 = False ' reset weapons, shield, armor, and rings being held/worn
Weapon11 = False ' reset weapons, shield, armor, and rings being held/worn
UserRecord.MonstersKilled = False ' reset user score
IF Normal.User = False THEN ' check player is Asst. DM or DM or Sysop
    Outpt = Message1 ' copy death message
    CALL IO.O ' send death message
    Outpt = "You were resurrected!" ' make message
    CALL IO.O ' send message
    CALL Read.Room.Record(1!) ' get room record #1
    Resurrection.Room = RoomRecord.MonsterClass ' get resurrection room number
    Next.Room = CSNG(Resurrection.Room) ' reset room number
    IF Next.Room = 0! THEN ' check room number
        Next.Room = 1! ' reset room
    END IF ' end check room number
    Number.Monsters = False ' reset number of monsters in room
    CALL Enter.Room ' send player to room one
    EXIT SUB ' exit routine
END IF ' end compare normal user type
' rest of death routine for normal users
Stat.Amount = GetUserRecordStats(5) ' store player constitution
IF Stat.Amount > 1 THEN ' check constitution positive
    CALL PutUserRecordStats(5, Stat.Amount - 1) ' store decremented player constitution
END IF ' end check constitution
Stat.Amount = GetUserRecordStats(6) ' store player piety
IF Stat.Amount > 1 THEN ' check piety positive
    CALL PutUserRecordStats(6, Stat.Amount - 1) ' store player decremented piety
END IF ' end check piety
IF RND > .5 THEN ' random chance for statistic decrement
    Stat.Number = INT(RND * 4 + 1) ' choose random statistic to decrement
    Stat.Amount = GetUserRecordStats(Stat.Number) ' store random statistic
    IF Stat.Amount > 1 THEN ' compare to positive value
        ' store player decremented statistic
        CALL PutUserRecordStats(Stat.Number, Stat.Amount - 1)
    END IF ' end compare value
END IF ' end random chance
Number.Monsters = False ' reset room monsters
UserRecord.Poison = False ' reset player poisoned flag
Weight = False ' reset player inventory weight
FOR Array.Index = 1 TO 20 ' loop through player inventory
    Charges.Number = GetUserRecordCharges(Array.Index) ' store inventory charges
    Index.Number = GetUserRecordInv(Array.Index) ' store inventory number
    IF Index.Number THEN ' verify inventory
        ' falls to ground
        CALL Add.Room.Treasure(Index.Number, Charges.Number, False, Item.Added)
        IF Item.Added = False THEN ' return variable for full room
            EXIT FOR ' exit inventory loop
        END IF ' end check return variable
    END IF ' end check inventory
NEXT ' end loop through player inventory
FOR Array.Index = 1 TO 20 ' loop through player inventory
    CALL PutUserRecordInv(Array.Index, False) ' set inventory to zero
    CALL PutUserRecordCharges(Array.Index, False) ' set inventory to zero
NEXT ' end loop through inventory
CALL Read.Room.Record(1!) ' get room record #1
Resurrection.Room = RoomRecord.MonsterClass ' get resurrection room number
Next.Room = CSNG(Resurrection.Room) ' reset room number
IF Next.Room = 0! THEN ' check room number
    Next.Room = 1! ' reset room
END IF ' end check room number
Room = Next.Room ' reset user room
Outpt = Message1 ' copy death message
CALL IO.O ' send death message
' calculate player constitution roll
Level = UserRecord.Level ' store user level
Calculate# = (INT(RND * 10 + 10) + GetUserRecordStats(5)) > 20 ' calculate
IF Calculate# = False THEN ' compare constitution roll
    Outpt = "Constitution roll failed!" ' make message
    CALL IO.O ' send fail message
    Outpt = "You were not resurrected!" ' make message
    CALL IO.O ' send fail message
    Level = INT(Level / 2 - .5) ' player loses half levels
ELSE ' end compare constitution roll
    ' player survives constitution roll
    Outpt = "You were resurrected!" ' make message
    CALL IO.O ' send message
    IF GetUserRecordStats(6) <= 1 THEN ' compare piety
        Outpt = "Except your piety failed!" ' player loses roll anyway
        CALL IO.O ' send fail message
        Level = INT(Level / 2 - .5) ' player loses half levels
    ELSE ' compare piety
        Level = Level - 1 ' decrement player level by one
    END IF ' end compare piety
END IF ' end compare constitution roll
IF Level < False THEN ' compare level
    Level = False ' reset level
END IF ' end compare level
UserRecord.Level = Level ' store user level
CALL More.Prompt ' pause for next screen
CALL Enter.Room ' routine to move player to room number
CALL Zero.Stats ' routine to clear player statistics
END SUB ' end routine for dead player

REM * routine to clear player statistics for death.

SUB Zero.Stats
CALL Experience(Calculate#) ' calculate new experience
Calculate# = Calculate# - INT(Calculate# / 10) ' calculate decremented experience
IF Calculate# < UserRecord.Experience THEN ' compare new experience
    UserRecord.Experience = Calculate# ' reset experience
END IF ' end compare new experience
CALL Gold(Calculate#) ' calculate new gold
Calculate# = Calculate# - INT(Calculate# / 10) ' calculate decremented gold
IF Calculate# < UserRecord.Gold THEN ' compare new gold
    UserRecord.Gold = Calculate# ' reset player gold
END IF ' end compare new gold
CALL New.Stats ' get new statistics for level
UserRecord.Flags = UserRecord.Flags AND NOT Wished ' clear player flags
UserRecord.Flags = UserRecord.Flags AND NOT Rerolled ' clear player flags
UserRecord.Flags = UserRecord.Flags AND NOT Alignmented ' clear player flags
FOR Stats = 1 TO 7 ' loop through player statistics
    IF GetUserRecordStats(Stats) <= False THEN ' compare statistcis to zero
        CALL PutUserRecordStats(Stats, 1) ' reset statistic
    END IF ' end compare player statistics
NEXT ' end loop through player statistics
CALL Put.User.Record ' store player record
END SUB ' end routine to clear dead player

REM * routine for player to learn a spell chant.
REM * processing variables:
REM *   Learned.Spells - array of 0s and 1s of learned spells.
REM * input variables:
REM *   Parsed.Command1 - contains name of spell to learn.

SUB Learn.Spell
Spell.Found = False ' flag for spell name found
FOR Spell.Number = 1 TO LOF(SpellFile) / LEN(SpellRecord) ' loop through spells
    CALL Read.Record(SpellFile, Spell.Number) ' get next spell record
    ' compare spell name with command parameter
    IF RTRIM$(SpellRecord.SpellName) = LCASE$(Parsed.Command1) THEN ' compare
        Spell.Found = True ' set spell flag found
        EXIT FOR ' exit spell file loop
    END IF ' end compare spell names
NEXT ' end loop through spell file
IF Spell.Found = False THEN ' compare spell flag
    Outpt = "You can't learn that spell!" ' make message
    CALL IO.O ' send error message
    EXIT SUB ' exit routine
END IF ' end compare spell flag
IF Spell.Number > 1024 THEN ' compare spell maximum
    Outpt = "You can't learn that spell!" ' make message
    CALL IO.O ' send error message
    EXIT SUB ' exit routine
END IF ' end compare spell flag
Outpt = "Enter chant to learn: " ' format input prompt
CALL IO.I ' get input spell chant
Inpt = UCASE$(Inpt) ' uppercase spell chant input
IF RTRIM$(SpellRecord.Chant) <> Inpt THEN ' compare spell chant entered
    Outpt = "Wrong spell chant!" ' make message
    CALL IO.O ' send error message
    EXIT SUB ' exit routine
END IF ' end compare spell chants
' check length of learned spell flags
IF Spell.Number > LEN(Learned.Spells) THEN
    ' append 0s to learned spells
    Learned.Spells = Learned.Spells + STRING$(Spell.Number - LEN(Learned.Spells), "0")
END IF ' end check length of learned spells flags
MID$(Learned.Spells, Spell.Number, 1) = "1" ' set flag for spell learned
Outpt = "You now memorize the spell!" ' make message
CALL IO.O ' send spell message
END SUB ' end routine to learn spell

REM * routine for preprocessing spell casting.
REM * input variables:
REM *   Require.Chant - set to true if entering the spell chant is required.
REM * output variables:
REM *   Spell.Cast.Type - set to 1=use command, 2=scroll used, 4=cast command.
REM *   Spell.Number - number of the spell being used.
REM *   Spell.Chant - true if spell being used w/o chant.

SUB Cast.Spell (Require.Chant)
IF Weapon6 THEN ' check player is holding a weapon
    Outpt = "You must return your weapon first!" ' make message
    CALL IO.O ' send message
    EXIT SUB ' exit routine
END IF ' end check weapon held
Spell.Cast.Type = False ' reset casting variables
Spell.Number = False ' reset casting variables
Spell.Chant = False ' reset casting variables
IF Require.Chant THEN ' check if chant required
    IF UserRecord.Magic = False THEN ' verify spell points remaining
        Outpt = "You have no more spell points!" ' make message
        CALL IO.O ' send message
        EXIT SUB ' exit routine
    END IF ' end verify spell points
    Outpt = "Chant? " ' format input prompt
    CALL IO.I ' get player input
    Inpt = UCASE$(Inpt) ' uppercase input
    Chant.Found = False ' set spell chant found flag
    ' loop through all spell records
    FOR Spell.Number = 1 TO LOF(SpellFile) / LEN(SpellRecord)
        CALL Read.Record(SpellFile, Spell.Number) ' get spell record
        Spell.Chant$ = SpellRecord.Chant ' store spell chant
        Spell.Chant$ = RTRIM$(Spell.Chant$) ' trim chant
        Spell.Chant$ = UCASE$(Spell.Chant$) ' uppercase chant
        IF Spell.Chant$ = Inpt THEN ' compare chants
            IF SpellRecord.Psionic = False THEN ' check for psionic spell
                Chant.Found = True ' set spell chant found flag
                EXIT FOR ' exit spell file loop
            END IF ' end check psionic spell
        END IF ' end compare chants
        Spell.Chant$ = SpellRecord.SpellName ' store spell name
        Spell.Chant$ = RTRIM$(Spell.Chant$) ' trim name
        Spell.Chant$ = UCASE$(Spell.Chant$) ' uppercase name
        IF Spell.Chant$ = Inpt THEN ' compare spell names
            IF SpellRecord.Psionic = False THEN ' check for psionic spell
                ' check learned spells bounds
                IF Spell.Number > 1024 THEN ' compare spell maximum
                    EXIT FOR ' exit spell file loop
                END IF ' end compare spell maximum
                IF Spell.Number <= LEN(Learned.Spells) THEN
                    ' check learned flag
                    IF MID$(Learned.Spells, Spell.Number, 1) = "1" THEN
                        Chant.Found = True ' set spell chant found flag
                        EXIT FOR ' exit spell file loop
                    END IF ' end check learned flag
                END IF ' end check bounds
            END IF ' end check for psionic spell
        END IF ' end compare spell names
    NEXT ' end loop through spell file
    IF Chant.Found = False THEN ' check spell chant found flag
        Outpt = "Wrong spell chant!" ' make message
        CALL IO.O ' send message
        EXIT SUB ' exit routine
    END IF ' end check spell chant flag
    Spell.Cast.Type = Cast.Spell.Type ' store spell type being used
ELSE ' compare chant required
    CALL Find.Inventory ' check magic device being used
    IF Index.Number = False THEN ' verify device used
        Outpt = "You can't use that!" ' make message
        CALL IO.O ' send message
        EXIT SUB ' exit routine
    END IF ' end verify device
    IF TreasureRecord.Potion THEN ' check device type
        Outpt = "You can't use that!" ' make message
        CALL IO.O ' send message
        EXIT SUB ' exit routine
    END IF ' end check device type
    IF TreasureRecord.Edible THEN ' check device type
        Outpt = "You can't use that!" ' make message
        CALL IO.O ' send message
        EXIT SUB ' exit routine
    END IF ' end check device
    IF TreasureRecord.Scroll THEN ' check device type
        Outpt = "The scroll disintegrated!" ' make message
        CALL IO.O ' send message
        CALL Discard.Inventory(Array.Number, True) ' remove item from inventory
        Spell.Cast.Type = Scroll.Spell.Type ' store spell type being used
    ELSE ' check device type
        IF TreasureRecord.Spell = False THEN ' check device type spell type
            Outpt = "That's not a magical device!" ' make message
            CALL IO.O ' send message
            EXIT SUB ' exit routine
        END IF ' end check device/spell type
        IF GetUserRecordCharges(Array.Number) = False THEN ' compare spell charges
            Outpt = "You can't, it's empty!" ' make message
            CALL IO.O ' send message
            EXIT SUB ' exit routine
        END IF ' end check spell charges
        Charges.Number = GetUserRecordCharges(Array.Number) ' store spell charges
        Charges.Number = Charges.Number - 1 ' decrement charges
        IF Charges.Number < False THEN ' compare zero
            Charges.Number = False ' set to zero
        END IF ' end compare zero
        CALL PutUserRecordCharges(Array.Number, Charges.Number) ' store spell charges
        Spell.Cast.Type = Use.Spell.Type ' store spell type being used
    END IF ' end compare device type
    Spell.Number = TreasureRecord.Spell ' store spell type
    Spell.Chant = True ' no chant required flag
END IF ' end compare chant required
CALL Read.Record(SpellFile, Spell.Number) ' get the spell record being used
Magic.Spell = SpellRecord.SpellType ' store the spell type being used
Multiplier = SpellRecord.Level ' store the spell level
Index.Number = False ' reset target variable
IF Parser THEN ' check for second command parameter
    Parsed.Command1 = Parsed.Command2 ' store second parameter
    CALL Numeric ' get pound sign number
    CALL Examine.Treasure ' check for treasure target
    IF Index.Number = False THEN ' treasure target found
        CALL Num ' decrement target number
        CALL Examine.Objects ' check for object target
        IF Index.Number > False THEN ' check object target found
            Outpt = "Wrong spell target for chant" ' make message
            CALL IO.O ' send message
            EXIT SUB ' exit sub
        END IF ' end check object target
    END IF ' end check target
END IF ' end check second parameter
CALL Magic(Spell.Chant) ' routine to use the magic spell
END SUB ' end spell preprocessing routine

REM * routine to cast a magic spell.
REM * input variables:
REM *   Magic.Spell - number of the spell Type
REM *   Chant.Used - true if spell being used w/o chant
REM *   Spell.Cast.Type - set to 1=use command, 2=scroll used, 4=cast command.
REM *   Spell.Number - number of the spell being used.
REM *   Index.Number - type of spell target.

SUB Magic (Chant.Used)
SELECT CASE Index.Number ' selection of the target type
    CASE IS > False ' target is an item of treasure
        SELECT CASE Magic.Spell ' selection of spell type for target of treasure
            CASE Enchant ' enchant
                IF TreasureRecord.Spell = False THEN ' check treasure target spell type
                    Outpt = "Wrong spell target for chant!" ' make message
                    CALL IO.O ' send message
                    EXIT SUB ' exit routine
                END IF ' end check treasure target spell type
            CASE Vigor ' vigor
                IF TreasureRecord.Vehicle = False THEN ' check treasure target vehicle
                    Outpt = "Wrong spell target for chant!" ' make message
                    CALL IO.O ' send message
                    EXIT SUB ' exit routine
                END IF ' end check treasure target vehicle type
            CASE ELSE ' check target type
                IF Magic.Spell <> Hiding THEN ' check spell type on treasure target
                    Outpt = "Wrong spell target for chant!" ' make message
                    CALL IO.O ' send message
                    EXIT SUB ' exit routine
                END IF ' end check treasure target type
        END SELECT ' end selection of target spell type
    CASE False ' target is not an item of treasure
        IF Parser THEN ' check target parameter flag
            SELECT CASE Magic.Spell ' selection of spells w/ targets
                CASE Offense, Poison, LevelDrain, Befuddled, TurnUndead, Intoxicate
                    Index.Number = False
                CASE ELSE ' spell w/o target
                    Outpt = "Wrong spell target for chant!" ' make message
                    CALL IO.O ' send message
                    EXIT SUB ' exit routine
            END SELECT ' end spell type selection
        END IF ' end check target parameter
END SELECT ' end target selection type
IF Chant.Used = False THEN ' check spell usage type
    IF Multiplier > UserRecord.Level THEN ' compare spell level to player level
        Outpt = "You are not high enough level to cast the spell!" ' message
        CALL IO.O ' send message
        EXIT SUB ' exit routine
    END IF ' end check spell/player levels
    IF UserRecord.Magic = False THEN ' check player magic points remaining
        Outpt = "You have no more spell points!" ' make message
        CALL IO.O ' send message
        EXIT SUB ' exit routine
    END IF ' end check player magic points
    ' verify spell can be cast by player class type
    IF (SpellRecord.ClassType AND 2 ^ UserRecord.ClassType) = False THEN ' verify
        IF Normal.User = True THEN ' check player is Asst. DM/DM/Sysop
            Outpt = "Your class can't cast that spell!" ' make message
            CALL IO.O ' send message
            EXIT SUB ' exit routine
        END IF ' end check player type
    END IF ' end verify spell type
    Charges.Number = UserRecord.Magic ' get player magic points
    ' decrement magic points by spell level
    Charges.Number = Charges.Number - Multiplier
    IF Charges.Number < False THEN ' check zero
        Charges.Number = False ' set to zero
    END IF ' end compare zero
    UserRecord.Magic = Charges.Number ' store player magic points remaining
END IF ' end check spell usage type
' check spell ingredients.
' verify spell type uses ingredients
IF SpellRecord.SpellFlag AND Spell.Cast.Type THEN
    FOR Spell.Index = 1 TO 5 ' loop through spell ingredients
        ' get treasure ingredient number
        Ingredient.Number = GetSpellRecordIngred(Spell.Index)
        IF Ingredient.Number > False AND Ingredient.Number <= LOF(TreasureFile) / LEN(TreasureRecord) THEN ' bounds
            Player.Ingredient = False ' player has ingredient flag
            FOR Array.Index = 1 TO 20 ' loop through player inventory
                ' check player inventory
                IF GetUserRecordInv(Array.Index) = Ingredient.Number THEN
                    Player.Ingredient = True ' set ingredient flag
                    EXIT FOR ' exit player inventory loop
                END IF ' end check player inventory is ingredient
            NEXT ' end loop through player inventory
            ' get treasure ingredient record
            CALL Read.Record(TreasureFile, Ingredient.Number)
            IF Player.Ingredient = False THEN ' check player has ingredient flag
                Outpt = "You don't have the correct spell ingredients!" ' message
                CALL IO.O ' send message
                Inpt = TreasureRecord.TreasureName ' get treasure name
                Inpt = RTRIM$(Inpt) ' trim name
                Inpt = LCASE$(Inpt) ' lowercase name
                Outpt = "You need " + Inpt + "!" ' make message
                CALL IO.O ' send ingredient message
                EXIT SUB ' exit routine
            END IF ' end check player ingredient flag
            ' check ingredient charges
            IF GetUserRecordCharges(Array.Index) = False THEN
                Inpt = TreasureRecord.ShortName ' get treasure name
                Inpt = RTRIM$(Inpt) ' trim name
                Inpt = LCASE$(Inpt) ' lowercase name
                Outpt = "The " + Inpt + " are used up!" ' make message
                CALL IO.O ' send ingredient message
                EXIT SUB ' exit routine
            END IF ' end check inrgedient charges
            ' decrement player ingredient charges
            CALL PutUserRecordCharges(Array.Index, GetUserRecordCharges(Array.Index) - 1)
        END IF ' end check file bounds
    NEXT ' end loop through spell ingredients
END IF ' end verify spell requires ingredients
Spell.Name$ = SpellRecord.SpellName ' get spell name being cast
Spell.Name$ = RTRIM$(Spell.Name$) ' trim name
Spell.Name$ = LCASE$(Spell.Name$) ' lowercase name
IF INSTR("aeiou", LEFT$(Spell.Name$, 1)) THEN ' check spell vowel
    Spell.Name$ = " an " + Spell.Name$ ' append vowel
ELSE ' check vowel
    Spell.Name$ = " a " + Spell.Name$ ' append vowel
END IF ' end check vowel spelling
Outpt = SpellRecord.Desc ' get spell long cast description
Outpt = RTRIM$(Outpt) ' trim description
IF Outpt = Nul THEN ' check description length
    Outpt = "You cast" + Spell.Name$ + " spell!" ' make default cast description
END IF ' end check description length
CALL IO.O ' send spell cast message
CALL Read.Room.Record(Room) ' get current room record
Action.Number = RoomRecord.Action ' get room action number
' check file bounds
IF Action.Number > False AND Action.Number <= LOF(ActionFile) / LEN(ActionRecord) THEN
    CALL Read.Record(ActionFile, Action.Number) ' read action record
    IF ActionRecord.SpellTrigger = Spell.Number THEN ' check for spell trigger
        Action1$ = "As the spell is cast," ' action message
        Action2$ = "You are hit for" ' action message
        CALL Actions(Action1$, Action2$) ' routine for activated action
    END IF ' end check room for action triggered
END IF ' end check file bounds
' check for magic traps in room
FOR Array.Index = 1 TO 20 ' loop through room treasure
    IF GetRoomRecordFlags(Array.Index) = Magic.Trap THEN ' check room for trap
        IF GetRoomRecordTreasure(Array.Index) = True THEN ' check trap triggered
            IF GetUserRecordStats(2) < INT(RND * 6 + 10) THEN ' random chance for trap
                ' hits on player for magical trap
                Calculate# = CDBL(INT(RND * 10 + UserRecord.Level) * GetRoomRecordTreCharges(Array.Index))
                CALL PutRoomRecordTreasure(Array.Index, False) ' reset room trap
                CALL PutRoomRecordTreCharges(Array.Index, False) ' reset room trap
                CALL PutRoomRecordFlags(Array.Index, False) ' reset room trap
                CALL Share.Room.Record(Room) ' write new room record
                Outpt = "You triggered a magical trap!" ' message
                CALL IO.O ' send trap message
                Outpt = "The trap hit you for" ' hits message
                CALL Hit.Player(Calculate#) ' routine to hit player
                EXIT FOR ' exit room trap loop
            END IF ' end random chance
        END IF ' end trap trigger
    END IF ' end room trap
NEXT ' end loop through room
' selections of the actual spell cast subroutines
SELECT CASE Magic.Spell ' select the spell type number
    CASE Offense, Poison, LevelDrain, Befuddled, TurnUndead, Intoxicate
        CALL Attack.Monster ' spell cast on monster target
    CASE Enchant ' enchant (heal magic points)
        IF Index.Number > False THEN ' enchant on a target
            CALL Read.Record(SpellFile, TreasureRecord.Spell) ' get target spell
            IF SpellRecord.SpellType = 4 THEN ' compare target to wish device
                Outpt = "You can't recharge that!" ' make message
                CALL IO.O ' send message
                EXIT SUB ' exit routine
            END IF ' end compare wish item
            Calculate# = GetUserRecordCharges(Array.Number) ' get charges of item
            Calculate# = Calculate# + UserRecord.Level ' calculate new item charges
            IF Calculate# > MaxInt THEN ' compare maximum integer
                Calculate# = MaxInt ' reduce to maximum integer
            END IF ' end compare maximum integer
            ' compare maximum treasure charges
            IF Calculate# > TreasureRecord.Charges THEN
                ' reduce to maximum treasure charges
                Calculate# = TreasureRecord.Charges
            END IF ' end compare charges
            CALL PutUserRecordCharges(Array.Number, CINT(Calculate#)) ' store new charges
            Outpt = "You recharge it!" ' make message
            CALL IO.O ' send message
        ELSE ' compare enchant on a target
            ' calculate player enchant
            Calculate# = UserRecord.Magic + UserRecord.Level + Multiplier ' calculate
            IF Calculate# > MaxInt THEN ' compare maximum integer
                Calculate# = MaxInt ' set to maximum integer
            END IF ' end compare maximum integer
            UserRecord.Magic = CINT(Calculate#) ' store player magic points
            CALL New.Stats ' update player statistics
            Outpt = "You now have" + STR$(UserRecord.Magic) + " magic points!" ' message
            CALL IO.O ' send message
        END IF ' end compare enchant target
    CASE Bless ' bless (heal piety)
        Outpt = "Nothing happened.." ' default bless message
        IF GetUserRecordStats(6) < MaxStat THEN ' compare player piety points
            CALL PutUserRecordStats(6, GetUserRecordStats(6) + 1) ' increment player piety
            Outpt = "You feel a magical glow about you!" ' message
        END IF ' end compare piety
        CALL IO.O ' send piety message
    CASE Wish ' wish
        ' compare spell cast type
        IF Spell.Cast.Type = Use.Spell.Type OR Spell.Cast.Type = Scroll.Spell.Type THEN ' compare
            ' reset player wish flag
            UserRecord.Flags = UserRecord.Flags AND NOT Wished ' reset
        END IF ' end compare spell cast type
        Graphics.Off = True ' reset color
        Outpt = "The Ghods thunder.." ' message
        CALL IO.O ' send ghod message
        Outpt = "   What do you wish for?" ' format input prompt
        Graphics.Off = False ' reset color
        CALL IO.I ' get player input
        Stored.Parsed.Command2 = Inpt ' store item wish name
        Parsed.Command1 = Stored.Parsed.Command2 ' store item wish name
        CALL Numeric ' parse out pound sign
        Inpt = Parsed.Command1 ' store parsed item wish name
        Inpt = LCASE$(Inpt) ' lowercase item
        CALL Drop(False) ' routine to get item from treasure file
    CASE Vigor ' vigor (heal fatigue)
        IF Index.Number > False THEN ' check treasure target
            CALL Read.Room.Record(Room) ' get current room
            Calculate# = GetRoomRecordTreCharges(Array.Number) ' get vehicle hits
            Calculate# = Calculate# + UserRecord.Level * 2 ' increment vehicle hits
            IF Calculate# > MaxInt THEN ' compare maximum integer
                Calculate# = MaxInt ' reset to maximum integer
            END IF ' end compare maximum integer
            Vehicle.Hits = CINT(Calculate#) ' convert to integer
            ' compare maximum vehicle hits
            IF Vehicle.Hits > TreasureRecord.VehicleHits THEN
                ' reset to maximum vehicle hits
                Vehicle.Hits = TreasureRecord.VehicleHits
            END IF ' end compare vehicle hits
            ' store new vehicle hits
            CALL PutRoomRecordTreCharges(Array.Number, Vehicle.Hits)
            CALL Share.Room.Record(Room) ' write room record
            Outpts = TreasureRecord.ShortName ' get vehicle name
            Outpts = RTRIM$(Outpts) ' trim name
            Outpts = LCASE$(Outpts) ' lowercase name
            Outpt = "The " + Outpts + " now has" + STR$(Vehicle.Hits) + " hits!" ' message
            CALL IO.O ' send healed vehicle message
        ELSE ' compare target
            ' calculate player fatigue increase
            Calculate# = UserRecord.Fatigue + UserRecord.Level + Multiplier ' calculate
            IF Calculate# > MaxInt THEN ' compare to max integer
                Calculate# = MaxInt ' set to max integer
            END IF ' end compare max integer
            UserRecord.Fatigue = CINT(Calculate#) ' set player fatigue to new value
            CALL New.Stats ' routine to update player statistics
            Outpt = "You now have" + STR$(UserRecord.Fatigue) + " fatigue points!"
            CALL IO.O ' send vigor message
        END IF ' end compare target
    CASE Heal ' (heal vitality)
        ' calculate player vitality increase
        Calculate# = UserRecord.Vitality + UserRecord.Level + Multiplier ' calculate
        IF Calculate# > MaxInt THEN ' compare to max integer
            Calculate# = MaxInt ' set to max integer
        END IF ' end compare max integer
        UserRecord.Vitality = CINT(Calculate#) ' store new vitality
        CALL New.Stats ' routine to update player statistics
        Outpt = "You now have" + STR$(UserRecord.Vitality) + " vitality points!"
        CALL IO.O ' send new vitality message
    CASE CurePoison ' cure poison
        Outpt = "Nothing happened.." ' default message
        IF UserRecord.Poison THEN ' check player is poisoned
            UserRecord.Poison = False ' clear player poisoned flag
            Outpt = "The poison disappeared!" ' message
        END IF ' end check player poisoned
        CALL IO.O ' send poison message
    CASE Teleport ' teleport spell
        Next.Room = SpellRecord.Teleport ' copy teleportation room ro next room
        Teleported = True ' set teleporting flag
        CALL Enter.Room ' routine to move player to new room
    CASE PassDoor ' open nearby doors
        Pass.Door = True ' set passdoor flag
        Outpt = "You hear a click!" ' passdoor message
        CALL IO.O ' send passdoor message
    CASE Conjure ' call up random monster
        FOR Array.Index = 1 TO Monclass.Max ' loop through all monster classes
            ' get a random monster choice
            Choice = Monster.Class(Array.Index, INT(RND * 10 + 1))
            IF Choice THEN ' verify choice exists
                CALL Read.Record(MonsterFile, Choice) ' get monster record
                ' compare monster to player level
                IF MonsterRecord.Level = UserRecord.Level THEN ' compare
                    CALL Get.Monster ' get the monster
                    EXIT SUB ' exit routine
                END IF ' end compare levels
            END IF ' end verify choice
        NEXT ' end loop through monster classes
        Outpt = "Nothing conjures.." ' default message
        CALL IO.O ' send message
    CASE Psionic ' psionics
        Outpt = "You can't cast a psi spell!" ' error message
        CALL IO.O ' send error message
    CASE DetectLock ' detect hidden or invisible doors
        FOR Array.Index = 1 TO 20 ' loop through room objects
            Object.Number = GetRoomRecordObject(Array.Index) ' get room object nummber
            IF Object.Number > False AND Object.Number <= LOF(ObjectFile) / LEN(ObjectRecord) THEN ' file bounds
                CALL Read.Record(ObjectFile, Object.Number) ' get object record
                ' compare object invisible or hidden
                IF ObjectRecord.Invisible OR ObjectRecord.Hidden THEN ' compare
                    Outpt = "You detect hidden objects here!" ' display message
                    CALL IO.O ' send message
                    EXIT SUB ' exit routine
                END IF ' end compare object type
            END IF ' end check file bounds
        NEXT ' end loop through room objects
        Outpt = "You detect nothing.." ' default message
        CALL IO.O ' send message
    CASE DetectEvil ' determine which direction the highest class monsters are
        High.Class = False ' highest monster class so far
        Monster.Direction = False ' direction of monster class
        FOR Direction.Number = 1 TO 12 ' loop through all room directions
            CALL Read.Room.Record(Room) ' get current room record
            ' get next room direction
            Next.Direction! = GetRoomRecordDirect(Direction.Number)
            IF Next.Direction! THEN ' compare room direction
                CALL Read.Room.Record(Next.Direction!) ' get next room record
                ' compare monster class higher
                IF RoomRecord.MonsterClass > High.Class THEN ' compare
                    High.Class = RoomRecord.MonsterClass ' store higher monster class
                    Monster.Direction = Direction.Number ' store direction
                END IF ' end compare monster class
            END IF ' end compare room direction
        NEXT ' end loop through room directions
        CALL Read.Room.Record(Room) ' get current room number
        IF Monster.Direction THEN ' verify room direction found w/ monsters
            ' make display message with room direction
            Outpt = "You detect evil going " + RTRIM$(Direction(Monster.Direction)) + "!"
        ELSE ' verify direction
            Outpt = "You detect no evil presence.." ' make display message
        END IF ' end verify room direction
        CALL IO.O ' send display message
    CASE DetectTrap ' find traps in room objects
        FOR Array.Index = 1 TO 20 ' loop through room objects
            Object.Number = GetRoomRecordObject(Array.Index) ' get object number
            IF Object.Number > False AND Object.Number <= LOF(ObjectFile) / LEN(ObjectRecord) THEN ' file bounds
                CALL Read.Record(ObjectFile, Object.Number) ' get object record
                IF ObjectRecord.Trap THEN ' verify object is a trap
                    Outpt = "You detect traps here!" ' make display message
                    CALL IO.O ' send display message
                    EXIT SUB ' exit routine
                END IF ' end verify object trap
            END IF ' end check file bounds
        NEXT ' end loop through room objects
        Outpt = "You detect nothing.." ' make default display message
        CALL IO.O ' send default message
    CASE SetTrap ' sets a magical trap in room
        Trap.Set = False ' room full flag
        FOR Treasure.Number = 1 TO 20 ' loop through room treasure
            ' check empty treasure in room
            IF GetRoomRecordTreasure(Treasure.Number) = False THEN
                Trap.Set = True ' set room full flag
                EXIT FOR ' exit loop
            END IF ' end check empty treasure
        NEXT ' end loop room
        IF Trap.Set = False THEN ' check room full flag
            Outpt = "You didn't set the trap!" ' make display message
            CALL IO.O ' send message
            EXIT SUB ' exit routine
        END IF ' end check room flag
        IF GetUserRecordStats(2) < INT(RND * 5 + 5) THEN ' calculate random percent
            Outpt = "You didn't set the trap!" ' make display message
            CALL IO.O ' send message
            EXIT SUB ' exit routine
        END IF ' end calculate percent
        ' calculate trap percent
        IF GetUserRecordStats(2) + GetUserRecordStats(3) < INT(RND * 10 + 5) THEN ' calculate
            Outpt = "The trap explodes in your face!" ' make display message
            CALL IO.O ' send message
            Calculate# = CDBL(INT(RND * 10 + 2) * UserRecord.Level) ' get hits on player
            Outpt = "The trap hit you for" ' death message
            CALL Hit.Player(Calculate#) ' routine to hit player
            EXIT SUB ' exit routine
        END IF ' end trap percent
        CALL PutRoomRecordTreasure(Treasure.Number, True) ' set the trap activated
        CALL PutRoomRecordTreCharges(Treasure.Number, UserRecord.Level) ' set trap level
        CALL PutRoomRecordFlags(Treasure.Number, Magic.Trap) ' set trap flag
        CALL Share.Room.Record(Room) ' write room record
        Outpt = "You set a magical trap!" ' make display message
        CALL IO.O ' send message
    CASE Hiding ' spell to hide an item in room
        Item.Hidden = False ' room full flag
        FOR Array.Index = 1 TO 20 ' loop through room treasure
            ' found empty room treasure
            IF GetRoomRecordTreasure(Array.Index) = False THEN
                Item.Hidden = True ' set room full flag
                EXIT FOR ' exit loop
            END IF ' end check empty treasure
        NEXT ' end loop through room
        IF Item.Hidden = False THEN ' check room full flag
            Outpt = "You didn't hide the object!" ' make display message
            CALL IO.O ' send message
            EXIT SUB ' exit routine
        END IF ' end check room full flag
        IF GetUserRecordStats(2) < INT(RND * 5 + 4) THEN ' calculate random percent
            Outpt = "You didn't hide the object!" ' make display message
            CALL IO.O ' send message
            EXIT SUB ' exit routine
        END IF ' end random percent
        ' remove item from player inventory
        CALL Discard.Inventory(Array.Number, True)
        CALL PutRoomRecordTreasure(Array.Index, Index.Number) ' add item to room
        ' add item charges to room
        CALL PutRoomRecordTreCharges(Array.Index, Charges.Number)
        CALL PutRoomRecordFlags(Array.Index, Magically.Hidden) ' set magically hidden flag
        CALL Share.Room.Record(Room) ' write room record
        Outpt = "You hide the object magically!" ' make display message
        CALL IO.O ' send message
    CASE Search ' displays magically hidden objects in room
        Graphics.Off = True ' reset color
        Outpt = "You find " ' make first string part
        Carriage.Return = True ' disable cr/lf
        CALL IO.O ' send output
        Items.Displayed = False ' items displayed counter
        Item.Displayed = False ' item displayed flag
        Outpt = Nul ' reset output string
        FOR Array.Index = 1 TO 20 ' loop through room treasure
            Inpt = Nul ' next item string
            ' check hidden flag
            IF GetRoomRecordFlags(Array.Index) = Magically.Hidden THEN
                ' store treasure number
                Treasure.Number = GetRoomRecordTreasure(Array.Index)
                IF Treasure.Number > False AND Treasure.Number <= LOF(TreasureFile) / LEN(TreasureRecord) THEN ' bounds
                    CALL Read.Record(TreasureFile, Treasure.Number) ' get treasure
                    Inpt = TreasureRecord.TreasureName ' store treasure name
                    Inpt = RTRIM$(Inpt) + "[inv]" ' append treasure name
                END IF ' end check file bounds
            END IF ' end check treasure magically hidden flag
            IF Outpt <> Nul AND Inpt <> Nul THEN ' check next output item
                Carriage.Return = True ' disable cr/lf
                CALL IO.O ' send item to output
                ' increment items displayed counter
                Items.Displayed = Items.Displayed + 1
            END IF ' end check output item
            IF Inpt <> Nul THEN ' check last item output
                Item.Displayed = True ' set item displayed flag
                Outpt = Inpt + ", " ' append comma
            END IF ' end check items
        NEXT ' end loop through room
        IF Item.Displayed = False THEN ' check item displayed flag
            Outpt = "nothing special.." ' default display message
        END IF ' end check item flag
        IF Item.Displayed AND Outpt <> Nul THEN ' check item flag
            Outpt = LEFT$(Outpt, LEN(Outpt) - 2) + "." ' trim comma, add period
            IF Items.Displayed > 1 THEN ' check number of items displayed
                Outpt = "and " + Outpt ' append to string
            END IF ' end check item number
        END IF ' end check items
        CALL IO.O ' send output
        Graphics.Off = False ' reset color
    CASE Invisibility ' normal player invisibility spell
        Outpt = "Didn't work!" ' default display message
        Level = INT((UserRecord.Level + 4) / 2) ' calculate level
        IF Level > False THEN ' check level for invisibility
            Invisible = Level ' set invisibility counter
            UserRecord.Invisible = True ' set player record invisible flag
            Outpt = "You are now invisible!" ' make display message
        END IF ' end check invisible percent
        CALL IO.O ' send display message
    CASE Identify ' spell to display monster stattistics
        Parsed.Command1 = Last.Monster ' get parameter in monster name
        CALL Check.Monster ' check monster name
        IF Monster.Number = False THEN ' compare monster number found
            Outpt = "You are not fighting a monster now!" ' make error message
            CALL IO.O ' send message
            EXIT SUB ' exit routine
        END IF ' end compare monster number
        Graphics.Off = True ' reset color
        CALL The.Or.An ' get prefix
        Outpts = MonsterArray(Monster.Number).MonsterName ' store monster name
        Outpts = RTRIM$(Outpts) ' trim name
        Outpts = LCASE$(Outpts) ' lowercase name
        Level = MonsterArray(Monster.Number).Level ' store monster level
        ' format message with monster to player level
        Outpt = "You are fighting " + Prefix1 + Outpts + "(level" + STR$((Level - 1) * 2 + 1) + " to" + STR$(Level * 2) + ")"
        CALL IO.O ' send level message
        Outpt = "It has" + STR$(MonsterArray(Monster.Number).Hits) + " hits."
        CALL IO.O ' display message of monster hit points
        Outpt = "It has" + STR$(MonsterArray(Monster.Number).Experience) + " experience."
        CALL IO.O ' display message of monster experience
        Outpt = "It has" + STR$(MonsterArray(Monster.Number).Gold) + " gold."
        CALL IO.O ' display message of monster gold
        Spell.Number = MonsterArray(Monster.Number).Spell ' get monster spell type
        ' file bounds
        IF Spell.Number > False AND Spell.Number <= LOF(SpellFile) / LEN(SpellRecord) THEN
            CALL Read.Record(SpellFile, Spell.Number) ' get spell record
            ' format message with spell monster can cast
            Outpt = "It can cast " + RTRIM$(SpellRecord.SpellName) + " spells."
            CALL IO.O ' send spell message
        END IF ' end check file bounds
        IF MonsterArray(Monster.Number).Poison THEN ' check poisonous monster
            Outpt = "It can poison." ' make message
            CALL IO.O ' send message
        END IF ' end check poisonous monster
        IF MonsterArray(Monster.Number).LevelDrain THEN ' check undead monster
            Outpt = "It can drain levels." ' make message
            CALL IO.O ' send message
        END IF ' end check undead monster
        IF MonsterArray(Monster.Number).Psionic THEN ' check psionic monster
            Outpt = "It can cast psi spells." ' make message
            CALL IO.O ' send message
        END IF ' end check psionic monster
        IF MonsterArray(Monster.Number).Magic THEN ' compare magical monster
            Outpt = "It's magical." ' make message
            CALL IO.O ' send message
        END IF ' end check magical monster
        IF MonsterArray(Monster.Number).Block THEN ' check monster blocks
            Outpt = "It blocks exits." ' make message
            CALL IO.O ' send message
        END IF ' end check monster blocks
        IF MonsterArray(Monster.Number).Prevent THEN ' check monster takes
            Outpt = "It prevents treasure take." ' make message
            CALL IO.O ' send message
        END IF ' end check monster takes
        IF MonsterArray(Monster.Number).Follow THEN ' check monster follows
            Outpt = "It follows users." ' make message
            CALL IO.O ' send message
        END IF ' end check monster follows
        IF MonsterArray(Monster.Number).Jail THEN ' check monster jails
            Outpt = "It jails attacker." ' make message
            CALL IO.O ' send message
        END IF ' end check monster jails
        IF MonsterArray(Monster.Number).Teleport THEN ' check monster teleports
            Outpt = "It can teleport." ' make message
            CALL IO.O ' send message
        END IF ' end check monster teleports
        Graphics.Off = False ' reset color
    CASE Enlighten ' locates hidden objects in room
        Graphics.Off = True ' reset color
        Outpt = "You see " ' make first output string
        Carriage.Return = True ' disable cr/lf
        CALL IO.O ' output first string
        Items.Displayed = False ' items displayed counter
        Item.Displayed = False ' item displayed flag
        Outpt = Nul ' reset output string
        FOR Array.Index = 1 TO 20 ' loop through room treasure
            Inpt = Nul ' reset last item string
            ' compare item hidden
            IF GetRoomRecordFlags(Array.Index) = Hidden.Object THEN
                ' store treasure number
                Treasure.Number = GetRoomRecordTreasure(Array.Index)
                IF Treasure.Number > False AND Treasure.Number <= LOF(TreasureFile) / LEN(TreasureRecord) THEN ' bounds
                    CALL Read.Record(TreasureFile, Treasure.Number) ' get treasure
                    Inpt = TreasureRecord.TreasureName ' get treasure name
                    Inpt = RTRIM$(Inpt) + "[inv]" ' append to name
                END IF ' end check file bounds
            END IF ' end compare hidden item
            IF Outpt <> Nul AND Inpt <> Nul THEN ' check next item display
                Carriage.Return = True ' disable cr/lf
                CALL IO.O ' send item display
                ' increment items displayed counter
                Items.Displayed = Items.Displayed + 1
            END IF ' end check items
            IF Inpt <> Nul THEN ' check last item displayed
                Item.Displayed = True ' set item displayed flag
                Outpt = Inpt + ", " ' append comma
            END IF ' end check item
        NEXT ' end loop through room
        IF Item.Displayed = False THEN ' check item displayed flag
            Outpt = "nothing special.." ' make message
        END IF ' end check item display flag
        IF Item.Displayed AND Outpt <> Nul THEN ' check items displayed
            Outpt = LEFT$(Outpt, LEN(Outpt) - 2) + "." ' trim comma, add period
            IF Items.Displayed > 1 THEN ' check number of items
                Outpt = "and " + Outpt ' append to string
            END IF ' end check number of items
        END IF ' end check items
        CALL IO.O ' send last output string
        Graphics.Off = False ' reset color
    CASE Illuminate ' searches room for magic traps
        Graphics.Off = True ' reset color
        Outpt = "You see " ' set first output string
        Carriage.Return = True ' disable cr/lf
        CALL IO.O ' send first string
        Items.Displayed = False ' set items displayed counter
        Item.Displayed = False ' set item displayed flag
        Outpt = Nul ' reset first item string
        FOR Array.Index = 1 TO 20 ' loop through all room treasure
            Inpt = Nul ' set next item displayed
            ' compare treasure magic trap
            IF GetRoomRecordFlags(Array.Index) = Magic.Trap THEN
                IF GetRoomRecordTreasure(Array.Index) = True THEN ' check trap active
                    Inpt = "a magical trap " ' make display message
                    ' append magic trap hits to message
                    Inpt = Inpt + "[" + MID$(STR$(GetRoomRecordTreCharges(Array.Index)), 2) + " hits]"
                END IF ' end check active trap
            END IF ' end compare magic trap
            IF Outpt <> Nul AND Inpt <> Nul THEN ' check next item display
                Carriage.Return = True ' disable cr/lf
                CALL IO.O ' send item output
                ' increment items displayed counter
                Items.Displayed = Items.Displayed + 1
            END IF ' end check next item
            IF Inpt <> Nul THEN ' check items
                Item.Displayed = True ' set item displayed flag
                Outpt = Inpt + ", " ' append comma
            END IF ' end check items
        NEXT ' end loop through room
        IF Item.Displayed = False THEN ' compare item displayed flag
            Outpt = "nothing special.." ' make display message
        END IF ' end check item displayed flag
        IF Outpt <> Nul THEN ' check last item
            Outpt = LEFT$(Outpt, LEN(Outpt) - 2) + "." ' trim comma, add period
            IF Items.Displayed > 1 THEN ' check number of items displayed
                Outpt = "and " + Outpt ' append to string
            END IF ' end check number of items
        END IF ' end check items
        CALL IO.O ' send last string
        Graphics.Off = False ' reset color
    CASE Psyche ' heal psionic points
        ' calculate psionic points
        Calculate# = UserRecord.Psionic + UserRecord.Level + Multiplier ' calculate
        IF Calculate# > MaxInt THEN ' check max integer
            Calculate# = MaxInt ' set to max integer
        END IF ' end check max integer
        UserRecord.Psionic = CINT(Calculate#) ' store players new psionic points
        CALL New.Stats ' routine to update player statistics
        Outpt = "You now have" + STR$(UserRecord.Psionic) + " psionic points!" ' message
        CALL IO.O ' send display message
    CASE ELSE ' any other spell type
        Outpt = "You can't cast that spell!" ' make error message
        CALL IO.O ' send message
END SELECT ' end selection of spell type number
END SUB ' end routine to cast magic spell

REM * routine to use psionic spell.
REM * processing variables:
REM *   Psi.Mode.Type - psi mode 1 or 2 (attack/defense).
REM * output variables:
REM *   Psi.Attack.Mode - level of psi spell.
REM *   Psi.Defense.Mode - level of psi spell.

SUB Psi.Mode
SELECT CASE LCASE$(Parsed.Command1) ' selection of command parameter
    CASE "attack" ' attack command
        Psi.Mode.Type = PsiAttack ' store psi attack
    CASE "defense" ' defense command
        Psi.Mode.Type = PsiDefense ' store psi defense
    CASE ELSE ' other
        Outpt = "You can only use psi mode attack or defense!" ' make message
        CALL IO.O ' send error message
        EXIT SUB ' exit routine
END SELECT ' end psi type selection
Outpt = "Psi spell? " ' format input prompt
CALL IO.I ' get input
Inpt = UCASE$(Inpt) ' uppercase input
Inpt = RTRIM$(Inpt) ' trim input
Spell.Found = False ' psi spell found flag
' loop through all spell records
FOR Spell.Number = 1 TO LOF(SpellFile) / LEN(SpellRecord)
    CALL Read.Record(SpellFile, Spell.Number) ' get spell record
    Outpt = SpellRecord.Chant ' store spell chant
    Outpt = RTRIM$(Outpt) ' trim chant
    IF Outpt = Inpt THEN ' compare chant to input chant
        IF SpellRecord.Psionic THEN ' check spell is psi spell
            Spell.Found = True ' set psi spell flag found
            EXIT FOR ' exit spell file loop
        END IF ' end check psi spell
    END IF ' end check chants
NEXT ' end spell file loop
IF Spell.Found = False THEN ' check psi spell found flag
    Outpt = "Unknown psi spell!" ' make message
    CALL IO.O ' send error message
    EXIT SUB ' exit routine
END IF ' end check psi spell flag
Spell.Name$ = SpellRecord.SpellName ' store psi spell name
Spell.Name$ = RTRIM$(Spell.Name$) ' trim name
Spell.Name$ = LCASE$(Spell.Name$) ' lowercase name
IF Psi.Mode.Type <> SpellRecord.PsionicMode THEN ' verify psi spell mode
    Outpt = "Incorrect spell for psi mode!" ' make message
    CALL IO.O ' send error message
    EXIT SUB ' exit routine
END IF ' end verify psi spell mode
Level = SpellRecord.Level ' store psi spell level
IF UserRecord.Psionic <= False THEN ' check player psionic points remaining
    Outpt = "You have no more psi points!" ' make message
    CALL IO.O ' send error message
    EXIT SUB ' exit routine
END IF ' end check player psionic points
Charges.Number = UserRecord.Psionic ' store player psionic points remaining
' decrement player psionic points by level of psi spell
Charges.Number = Charges.Number - Level
IF Charges.Number < False THEN ' compare to zero
    Charges.Number = False ' reset to zero
END IF ' end compare zero
UserRecord.Psionic = Charges.Number ' store player psionic points remaining
Outpt = "You cast a " + Spell.Name$ + " psi spell!" ' make message
CALL IO.O ' send psi spell cast message
SELECT CASE Psi.Mode.Type ' selection of psi spell type
    CASE PsiAttack ' attack mode
        Psi.Attack.Mode = Level ' set psi attack mode to psi spell level
        Parsed.Command1 = Last.Monster ' store last monster attacked
        CALL Attack.Monster ' attack monster
    CASE PsiDefense ' defense mode
        Psi.Defense.Mode = Level ' set psi defense mode to psi spell level
        Outpt = "Your psi defense mode is now active!" ' make message
        CALL IO.O ' send psi message
END SELECT ' end selection of psi spell type
END SUB ' end routine to cast a psi spell

REM * routine to get default monster name
REM * output variables:
REM *   Monster.Found - true for the first monster
REM *   Monster.Number - index of monster
REM *   Last.Monster - name of monster

SUB Get.Last.Monster (Monster.Found)
Monster.Found = False ' reset return flag
FOR Array.Number = 1 TO Number.Monsters ' search through monsters in room
    IF MonsterArray(Array.Number).Magic >= False THEN ' check monster magical
        IF MonsterArray(Array.Number).Permanent >= True THEN ' check permanent
            Word.Parse1 = False ' reset parsing flag
            Word.Parse2 = False ' reset parsing flag
            Outpts = MonsterArray(Array.Number).MonsterName ' get monster name
            Outpts = RTRIM$(Outpts) ' trim name
            Outpts = UCASE$(Outpts) ' uppercase name
            Word.Parse1 = INSTR(Word.Parse1 + 1, Outpts, " ") ' search for space
            WHILE Word.Parse1 ' loop while spaces
                Word.Parse2 = Word.Parse1 ' store monster name parse
                Word.Parse1 = INSTR(Word.Parse1 + 1, Outpts, " ") ' check spaces
            WEND ' end loop while spaces
            Monster.Found = True ' set return flag
            Monster.Number = Array.Number ' store monster array index
            Last.Monster = MID$(Outpts, Word.Parse2 + 1) ' store monster name
            EXIT SUB ' exit routine
        END IF ' end check permanent monster
    END IF ' end check magic monster
NEXT ' end search through room
END SUB ' end routine to get first monster name

REM * Filename: dnds4.bas Version: v5.0a r1.0a
REM * This subprogram contains room edit routines, user list routines,
REM * shopkeeper routines, and some main commands.

REM * routine to toggle player sort flag.

SUB Sort.Inventory
SELECT CASE Sorting ' toggle sort method
    CASE -1 ' now sort by charges
        Sorting = 0 ' change to no sort
        UserRecord.Sort = 0 ' store in user record
        Outpt = "Inventory sorting off."
    CASE 0 ' now not sorting
        Sorting = 1 ' change to sorting by plus
        UserRecord.Sort = 1 ' store in user record
        Outpt = "Inventory plus sorting on."
    CASE 1 ' now sort by plus
        Sorting = -1 ' change to sorting by charges
        UserRecord.Sort = -1 ' store in user record
        Outpt = "Inventory charges sorting on."
    CASE ELSE ' not within range
        Sorting = 0 ' default sorting off
        UserRecord.Sort = 0 ' store in user record
        Outpt = "Inventory sorting off."
END SELECT ' end toggle sort flag
CALL IO.O ' send display message
END SUB ' end routine to toggle sort flag

REM * routine to sort player inventory.
REM * input variables:
REM *   Sorting - flag for sort toggle.

SUB Sorter
IF Sorting = False THEN ' check sorting on
    EXIT SUB ' exit routine
END IF ' end check sorting on
' perform a simple 'bubble sort' while swapping indexes of weapons variables
FOR Sort1 = 1 TO 20 ' loop through top bubble of player inventory
    FOR Sort2 = Sort1 + 1 TO 20 ' loop through bottom bubble of player inventory
        IF Sorting = -1 THEN ' sort by charges
            '  make the comparison of charges left of two inventory items
            IF GetUserRecordCharges(Sort1) < GetUserRecordCharges(Sort2) THEN
                ' swap the two items if the second one is greater
                Var1 = GetUserRecordInv(Sort1): Var2 = GetUserRecordInv(Sort2)
                CALL PutUserRecordInv(Sort1, Var2)
                CALL PutUserRecordInv(Sort2, Var1)
                Var1 = GetUserRecordCharges(Sort1): Var2 = GetUserRecordCharges(Sort2)
                CALL PutUserRecordCharges(Sort1, Var2)
                CALL PutUserRecordCharges(Sort2, Var1)
                GOSUB Swap.Weapons ' check weapons being held
            END IF ' end compare value of two items
        ELSE
            IF Sorting = 1 THEN ' sort by plus
                Inventory1 = GetUserRecordInv(Sort1) ' get treasure index
                Inventory2 = GetUserRecordInv(Sort2) ' get treasure index
                IF Inventory1 > 0 AND Inventory2 > 0 THEN ' compare index values
                    CALL Read.Record(TreasureFile, Inventory1) ' get record
                    Inventory.Plus1 = TreasureRecord.Plus ' store treasure plus
                    CALL Read.Record(TreasureFile, Inventory2) ' get record
                    Inventory.Plus2 = TreasureRecord.Plus ' store treasure plus
                    IF Inventory.Plus1 < Inventory.Plus2 THEN ' compare inventory pluses
                        ' swap the two items if the second one is greater
                        Var1 = GetUserRecordInv(Sort1): Var2 = GetUserRecordInv(Sort2)
                        CALL PutUserRecordInv(Sort1, Var2)
                        CALL PutUserRecordInv(Sort2, Var1)
                        Var1 = GetUserRecordCharges(Sort1): Var2 = GetUserRecordCharges(Sort2)
                        CALL PutUserRecordCharges(Sort1, Var2)
                        CALL PutUserRecordCharges(Sort2, Var1)
                        GOSUB Swap.Weapons ' check weapons being held
                    END IF
                END IF
            END IF
        END IF
    NEXT ' end loop through bubble sort
NEXT ' end loop through bubble sort
EXIT SUB ' exit sort routine
Swap.Weapons:
' check if any of the weapons/armor/shield/rings held/worn are
' equal to one of the two items being swapped so their indexes
' still point to the correct player inventory array elements.
SELECT CASE Weapon4 ' select an index (armor)
    CASE Sort1 ' swap 1
        Weapon4 = Sort2 ' switch index
    CASE Sort2 ' swap 2
        Weapon4 = Sort1 ' switch index
END SELECT ' end index selection
SELECT CASE Weapon5 ' select an index (shield)
    CASE Sort1 ' swap 1
        Weapon5 = Sort2 ' switch index
    CASE Sort2 ' swap 2
        Weapon5 = Sort1 ' switch index
END SELECT ' end index selection
SELECT CASE Weapon6 ' select an index (weapon)
    CASE Sort1 ' swap 1
        Weapon6 = Sort2 ' switch index
    CASE Sort2 ' swap 2
        Weapon6 = Sort1 ' switch index
END SELECT ' end index selection
SELECT CASE Weapon7 ' select an index (ring)
    CASE Sort1 ' swap 1
        Weapon7 = Sort2 ' switch index
    CASE Sort2 ' swap 2
        Weapon7 = Sort1 ' switch index
END SELECT ' end index selection
RETURN
END SUB ' end routine to sort player inventory

REM * routine to page sysop at console by beeping.
REM * processing variables:
REM *   Chat - contains chat mode toggle.

SUB Page.Sysop
IF Normal.User THEN ' check non DM
    IF UserRecord.Level <= 1 THEN ' check player level
        Outpt = "The Sysop is not answering pages now." ' make message
        CALL IO.O ' send message
        EXIT SUB ' exit routine
    END IF ' end check player level
END IF ' end check normal user
Beep.Count = False ' reset beep counter
Graphics.Off = True ' reset color
Outpt = "Hit <control-k> to return to prompt." ' make message
CALL IO.O ' send message
Outpt = "Sysop press <escape> to enter chat.." ' make message
CALL IO.O ' send message
Outpt = "Paging Sysop:" ' make paging message
Carriage.Return = True ' disable cr/lf
CALL IO.O ' send paging message
Chat = True ' store chat flag toggle off
Allow.Break = True ' enable control-k checking
Break = False ' reset control-k flag
Beep.Time! = TIMER ' store current time
DO WHILE Chat ' loop until chat entered, 10 beeps, or control-k break entered
    IF Break THEN ' check control-k entered
        EXIT DO ' exit chat loop
    END IF ' end check control-k entered
    ' routine to compute time elapsed
    CALL Second.Timer(Time.Elapsed, Beep.Time!, 2!)
    IF Time.Elapsed THEN ' check two seconds elapsed
        Outpt = CHR$(7) + Mask$ ' make remote beep plus character
        Carriage.Return = True ' disable cr/lf
        CALL IO.O ' send page message
        Beep.Time! = TIMER ' store current time
        Beep.Count = Beep.Count + 1 ' increment beep counter
        IF Beep.Count = 10 THEN ' check 10 beeps
            EXIT DO ' exit chat loop
        END IF ' end check beep exit
    END IF ' end check time elapsed
LOOP ' end chat loop
Allow.Break = False ' reset control-k checking off
IF Break THEN ' check control-k flag
    Break = False ' reset control-k flag
    Outpt = Nul ' set output to null
    CALL IO.O ' send empty return
END IF ' end check control-k flag
IF Chat = False THEN ' compare chat flag on
    Chat = True ' reset chat flag
    CALL Enter.Chat ' routine to chat with player
END IF ' end compare chat flag
Chat = False ' clear chat flag
CALL IO.O ' send empty cr/lf
END SUB

REM * routine to chat with remote player.
REM * input variables:
REM *   Chat - is true, false to quit.
REM * processing variables:
REM *   Logged.On! - time player logged on (seconds from midnight).
REM *   Chat.Start! - time chat started (seconds from midnight).
REM *   Time.Remaining! - time player had remaining before chat in seconds.

SUB Enter.Chat
Graphics.Off = True ' reset graphics
Logged.On! = Timeon ' store timeon
Chat.Start! = TIMER ' store time now
Time.Remaining! = Time.Left ' store time left
Timeon = TIMER ' reset time on to now
Time.Left = 60! ' reset time left to 60 seconds
Allow.Break = False ' disable control-k checking
Break = False ' reset control-k flag
Outpt = Nul ' format empty string
CALL IO.O ' send empty cr/lf
Outpt = "Chat Mode.." ' make chat mode message
CALL IO.O ' send chat mode message
User.Word.Wrap = User.Wordwrap ' store word wrap
User.Wordwrap = False ' reset word wrap
Word.Wrap = True ' enable 80 column word wrap
DO WHILE Chat ' chat in an input loop
    Timeon = TIMER ' reset time on (disables timeout messages)
    Time.Left = 600! ' reset time left (disables timeout messages)
    CALL IO.I ' continually process input (from keyboard and modem)
LOOP ' loop until chat toggla flag is reset
' end chat, restore time variables
Timeon = Logged.On! + FIX(TIMER - Chat.Start!) ' recalculate time on
IF Timeon > 86400! THEN ' check chatted past midnight
    Timeon = Timeon - 86400! ' decrement midnight
END IF ' end check midnight
Time.Left = Time.Remaining! ' restore time left
Allow.Break = False ' disable control-k checking
Break = False ' reset control-k flag
Buffer = Nul ' clear buffer
FunctionBuffer = Nul ' clear buffer
Outpt = Nul ' clear buffer
Word.Wrap = False ' disable word wrap
User.Wordwrap = User.Word.Wrap ' restore word wrap
IF LEN(Inpt) THEN ' check last input
    CALL IO.O ' output last input
END IF ' end check last input
Inpt = Nul ' reset input buffer
END SUB ' end routine to chat with remote player

REM * routine to display help text for DMs.
REM * input variables:
REM *   Stored.Parsed.Command1 - command to look up help text for.

SUB DM.Help
Help.Command$ = Stored.Parsed.Command1 ' get command parameter
' compare first character to DM command prefix
IF LEFT$(Help.Command$, 1) <> "!" THEN
    Outpt = "Enter DM command, form: !Help !<command>" ' make error message
    CALL IO.O ' send error message
    EXIT SUB ' exit routine
END IF ' end compare DM command prefix
' store next part of parameter for command lookup
Help.Command$ = MID$(Help.Command$, 2)
Stored.Parsed.Command1 = Help.Command$ ' store into parsed command variable
CALL Read.Help(True) ' routine to read help text
END SUB ' end routine for DM help text

REM * routine displays help text for a command.
REM * input variables:
REM *   Help.Type is 1=!edit help, 0=normal command, -1=DM command lookup.

SUB Read.Help (Help.Type)
Help.Command$ = Stored.Parsed.Command1 ' store parameter of command to lookup
Help.Command$ = RTRIM$(Help.Command$) ' trim command
Help.Command$ = UCASE$(Help.Command$) ' uppercase command
CLOSE #HelpFile ' close work file
SELECT CASE Help.Type ' selection of help file
    CASE True ' check DM command
        FileName = "dmhelp.dat" ' store DM help filename
        Help.Command$ = LEFT$(Help.Command$, 8) ' truncate command
    CASE False ' check normal command
        FileName = "help.dat" ' store normal command help filename
        Help.Command$ = LEFT$(Help.Command$, 8) ' truncate command
    CASE 1 ' check !edit help command
        FileName = "edithelp.dat" ' store !edit help filename
END SELECT ' end check command type
OPEN FileName FOR RANDOM SHARED AS #HelpFile LEN = LEN(HelpRecord) ' open file
DO
    Allow.Break = True ' enable control-k checking
    Break = False ' reset control-k flag
    Continue = False ' reset continuous flag
    Graphics.Off = True ' reset color
    Help.Displayed = False ' reset help text displayed flag
    Page.Length = False ' reset page length counter
    FOR Record.Count = 2 TO LOF(HelpFile) / LEN(HelpRecord) ' loop through help records
        CALL Read.Record(HelpFile, Record.Count) ' get next help record
        Command.Name$ = HelpRecord.CName ' store command name of help record
        Command.Name$ = RTRIM$(Command.Name$) ' trim command
        Command.Name$ = UCASE$(Command.Name$) ' uppercase command
        IF Help.Type <= False THEN ' check help file type
            Command.Name$ = LEFT$(Command.Name$, 8) ' truncate command
        END IF ' end check command
        IF Help.Displayed = True THEN ' check help topic found already
            IF Help.Command$ <> Command.Name$ THEN ' compare against topic selected
                EXIT FOR ' exit help file display loop
            END IF ' end compare topic
        END IF ' end check help topic displayed
        IF Help.Command$ = Command.Name$ THEN ' compare help record command
            IF Help.Type = 1 THEN ' check displaying !edit help
                IF Help.Displayed = False THEN ' verify topic already being displayed
                    IF Help.Command$ <> "CONTENTS" THEN ' compare help type
                        Count.Store = Record.Count ' save help file record counter
                        Topic.Name$ = RTRIM$(HelpRecord.CName) ' get command being displayed
                        FOR Topic.Count = 2 TO LOF(HelpFile) / LEN(HelpRecord) ' loop through help records
                            CALL Read.Record(HelpFile, Topic.Count) ' get next help record
                            Topic.Number$ = HelpRecord.Text ' store command name of help record
                            Topic.Number$ = RTRIM$(Topic.Number$) ' trim command
                            Topic.Space = INSTR(Topic.Number$, " ") ' store imbdded topic number
                            IF Topic.Space THEN ' remove topic sequence from topic
                                Topic.Number$ = LEFT$(Topic.Number$, Topic.Space - 1) ' remove topic
                            END IF ' end check for topic sequence
                            IF Topic.Number$ = Topic.Name$ THEN ' compare to topic
                                Page.Length = Page.Length + 1 ' increment text displayed
                                Outpt = RTRIM$(HelpRecord.Text) ' store topic number/name
                                CALL IO.O ' send command name message
                                EXIT FOR ' exit loop search
                            END IF ' end compare topic
                        NEXT ' end loop search
                        Record.Count = Count.Store ' restore help file record counter
                    END IF ' end compare help type
                END IF ' end verify topic displayed
            END IF ' end check displaying
            CALL Read.Record(HelpFile, Record.Count) ' get help record
            Outpt = HelpRecord.Text ' get command help text
            Outpt = RTRIM$(Outpt) ' trim help text
            Outpt = LTRIM$(Outpt) ' trim help text
            CALL IO.O ' display help text
            Help.Displayed = True ' set text displayed flag
            Page.Length = Page.Length + 1 ' increment page length counter
            IF Page.Length = User.PageLength THEN ' compare page length
                Page.Length = False ' reset page length counter
                IF Continue = False THEN ' check continuous flag
                    CALL More.Prompt ' routine to pause for more
                    IF No THEN ' check no more entered
                        EXIT FOR
                    END IF ' end check no  entered
                END IF ' end check continuous flag
            END IF ' end check page length
        END IF ' end compare help commands
    NEXT ' end loop through all help file records
    IF Help.Type = 1 THEN ' check if !edit help topics are being displayed
        IF Record.Count < LOF(HelpFile) / LEN(HelpRecord) THEN ' verify eof
            Graphics.Off = False ' reset color
            Outpt = "Continue search(y/n)? " ' prompt to continue to next topic
            No.Input.Out = "y" ' default input
            CALL IO.I ' get user input
            IF Yes THEN ' verify to continue display
                Help.Command$ = Command.Name$ ' store last help topic
            ELSE
                EXIT DO
            END IF ' end verify to continue
        END IF ' end verify file end
    ELSE
        EXIT DO
    END IF ' end check help display type
LOOP
Graphics.Off = False ' reset color
Allow.Break = False ' clear control-k flag
IF Break THEN ' check control-k flag
    Break = False ' reset control-k flag
    Outpt = Nul ' set output to null
    CALL IO.O ' send empty return
END IF ' end check control-k flag
IF Page.Length THEN ' check page length
    CALL More.Prompt ' routine for more
END IF ' end check page length
IF Help.Displayed = False THEN ' check help text displayed flag
    Outpt = "No help found on '" + LCASE$(Help.Command$) + "'." ' make error message
    CALL IO.O ' send error message
END IF ' end check help text flag
CLOSE #HelpFile ' close work file
END SUB ' end routine to display help text

REM * routine trains player for next level.

SUB Train.Stats
IF UserRecord.Level < False THEN ' check negative player level
    UserRecord.Level = False ' set player level
END IF ' end check negative level
IF UserRecord.Level >= MaxInt THEN ' check player level to maximum integer
    Outpt = "Nothing happens.." ' make error message
    CALL IO.O ' send message
    EXIT SUB ' exit routine
END IF ' end check maximum integer
IF UserRecord.Level > False THEN ' check player level again
    CALL Gold(Required.Gold#) ' get gold required to train for next level
    UserRecord.Gold = UserRecord.Gold - Required.Gold# ' subtract gold from player
END IF ' end check player level
Stat = INT(RND * 7 + 1) ' get random statistic to increment
' verify statistic below maximum statistic or player is DM type
IF GetUserRecordStats(Stat) < MaxStat OR Normal.User = False THEN ' verify
    CALL PutUserRecordStats(Stat, GetUserRecordStats(Stat) + 1) ' increment statistic
END IF ' end verify player type
IF UserRecord.Level <= 10 THEN ' compare player level
    IF GetUserRecordStats(6) < MaxStat THEN ' check statistic below maximum stat
        CALL PutUserRecordStats(6, GetUserRecordStats(6) + 1) ' increment piety
    END IF ' end check statistic
END IF ' end compare level
UserRecord.Level = UserRecord.Level + 1 ' increment the player level
CALL New.Stats ' routine to update statistics based on level
Outpt = "After many hours of training and meditation..." ' make train message
CALL IO.O ' send train message
CALL Display.Health ' routine to display player statistics
CALL Display.Experience ' routine to display player requirements
END SUB ' end routine to train player for next level

REM * routine recalculates player statistics based on player level.

SUB New.Stats
IF UserRecord.Level < False THEN ' check player level
    UserRecord.Level = False ' reset player level
END IF ' end check player level
' compute player maximum fatigue points
New.Stat# = CDBL(Training.Stats(UserRecord.ClassType, 1)) * CDBL(UserRecord.Level)
IF New.Stat# < 0# THEN ' compare fatigue points
    New.Stat# = 0# ' reset fatigue points
END IF ' end compare points
IF New.Stat# > MaxInt THEN ' compare fatigue points
    New.Stat# = MaxInt ' reset fatigue points
END IF ' end compare points
' store new maximum fatigue points in player record
UserRecord.FatigueMax = CINT(New.Stat#) ' convert points to integer
' compute player maximum vitality points
New.Stat# = CDBL(Training.Stats(UserRecord.ClassType, 2)) * CDBL(UserRecord.Level)
IF New.Stat# < 0# THEN ' compare vitality points
    New.Stat# = 0# ' reset vitality points
END IF ' end compare points
IF New.Stat# > MaxInt THEN ' compare vitality points
    New.Stat# = MaxInt ' reset vitality points
END IF ' end compare points
' store new maximum vitality points in player record
UserRecord.VitalityMax = CINT(New.Stat#) ' convert points to integer
' compute player maximum magic points
New.Stat# = CDBL(Training.Stats(UserRecord.ClassType, 3)) * CDBL(UserRecord.Level)
IF New.Stat# < 0# THEN ' compare magic points
    New.Stat# = 0# ' reset magic points
END IF ' end compare points
IF New.Stat# > MaxInt THEN ' compare magic points
    New.Stat# = MaxInt ' reset magic points
END IF ' end compare points
' store new maximum magic points in player record
UserRecord.MagicMax = CINT(New.Stat#) ' convert points to integer
' compute player maximum psionic points
New.Stat# = CDBL(Training.Stats(UserRecord.ClassType, 4)) * CDBL(UserRecord.Level)
IF New.Stat# < 0# THEN ' compare psionic points
    New.Stat# = 0# ' reset psionic points
END IF ' end compare points
IF New.Stat# > MaxInt THEN ' compare psionic points
    New.Stat# = MaxInt ' reset psionic points
END IF ' end compare points
' store new maximum psionic points in player record
UserRecord.PsionicMax = CINT(New.Stat#) ' convert points to integer
Stat = UserRecord.Fatigue ' store player fatigue points
Stat.Max = UserRecord.FatigueMax ' store player maximum fatigue points
IF Stat < 0 OR Stat > Stat.Max THEN ' compare fatigue points range
    Stat = Stat.Max ' reset fatigue to maximum fatigue points
END IF ' end compare points
UserRecord.Fatigue = Stat ' store new fatigue points
Stat = UserRecord.Vitality ' store player vitality points
Stat.Max = UserRecord.VitalityMax ' store player maximum vitality points
IF Stat < 0 OR Stat > Stat.Max THEN ' compare vitality points range
    Stat = Stat.Max ' reset vitality to maximum vitality points
END IF ' end compare points
UserRecord.Vitality = Stat ' store new vitality points
Stat = UserRecord.Magic ' store player magic points
Stat.Max = UserRecord.MagicMax ' store player maximum magic points
IF Stat < 0 OR Stat > Stat.Max THEN ' compare magic points range
    Stat = Stat.Max ' reset magic points to maximum magic points
END IF ' end compare points
UserRecord.Magic = Stat ' store new magic points
Stat = UserRecord.Psionic ' store player psionic points
Stat.Max = UserRecord.PsionicMax ' store player maximum psionic points
IF Stat < 0 OR Stat > Stat.Max THEN ' compare spionic points range
    Stat = Stat.Max ' reset psionic points to maximum psionic points
END IF ' end compare points
UserRecord.Psionic = Stat ' store new psionic points
' routine for maximum statistic comparison
IF Normal.User THEN ' check non DM
    FOR Stats = 1 TO 7 ' loop through all player statistic points
        IF GetUserRecordStats(Stats) > MaxStat THEN ' compare statistic to maximum
            CALL PutUserRecordStats(Stats, MaxStat) ' reset statistic to maximum
        END IF ' end compare points
    NEXT ' end loop through statistics
END IF
' routine for zero stats
IF Normal.User THEN
    Low.Stats = False ' reset low stats flag
    FOR Stats1 = 1 TO 7 ' loop through statistics again
        IF GetUserRecordStats(Stats1) <= False THEN ' check low statistic
            CALL PutUserRecordStats(Stats1, 1) ' reset low statistic
            ' loop through statistics again to prevent death loop
            FOR Stats2 = 1 TO 7
                IF GetUserRecordStats(Stats2) <= False THEN ' check low statistic
                    CALL PutUserRecordStats(Stats2, 1) ' reset statistic
                END IF ' end check low stat
            NEXT ' end loop through stats
            Low.Stats = True ' set low stats flag
            EXIT FOR ' exit loop so low stats don't death loop
        END IF ' end check low stat
    NEXT ' end loop through stats
    IF Low.Stats THEN ' check low stats flag
        ' make death message
        Outpts = LCASE$(RTRIM$(Stat(Stats1)))
        Message1 = "Your " + Outpts + " is zero! You have died!"
        CALL Player.Died ' routine for dead player
        EXIT SUB ' exit low stats loop
    END IF ' end check low stats flag
END IF ' end check normal player
' routine to assign upper class name to player
IF UserRecord.Level >= 10 THEN ' check player level
    Class.Number = UserRecord.ClassType ' get player class type
    IF Class.Number <= 0 OR Class.Number > 10 THEN ' verify bounds of class type
        Class.Number = 1 ' reset class type to fighter
        UserRecord.ClassType = 1 ' reset class type to fighter
    END IF ' end verify class type bounds
    Inpt = High.Class.Name(Class.Number) ' get player level 10 class name
    CALL Valid(Inpt, 20) ' validate name
    IF LEN(Inpt) THEN ' compare name length
        CALL Encrypt(Inpt, True) ' encrypt name
        UserRecord.ClassName = Inpt ' assign class name to player record
    END IF ' end compare name length
END IF ' end check player level
CALL Get.User.Stats ' routine to assign more player stats
END SUB ' end routine to recalculate player stats

REM * routine adds and edits new room.
REM * input variables:
REM *   Next.Room - number of new room number to add.
REM *   Last.Direction - last direction entered.
REM * output variables:
REM *   Room.Added - true for a new room added, false if not.

SUB Add.Room (Last.Direction, Room.Added)
Graphics.Off = True ' reset color
Outpt = "Add new room(y/n)? " ' make input prompt
No.Input.Out = "N" ' default input
CALL IO.I ' get input
Room.Added = False ' set return variable
IF No THEN ' check response
    EXIT SUB ' exit routine
END IF ' end check response
Room.Added = True ' set return variable
Next.Room = LOF(RoomFile) / LEN(RoomRecord) + 1 ' store last room record
CALL Clear.Room(Next.Room) ' routine to clear room record
' routine to edit room description and monster class
CALL Change.Room(Next.Room)
Outpt = "Add room link(y/n)? " ' input prompt
No.Input.Out = "Y" ' default input
CALL IO.I ' get input
IF Yes THEN ' check response
    IF Last.Direction THEN ' check direction entered
        Graphics.Off = True ' reset color
        Outpt = "Press <enter> for entry link:" ' make message
        CALL IO.O ' send message
    END IF ' end check direction entered
    ' routine to add link to room
    CALL Add.Link(Room, Next.Room, Last.Direction)
END IF ' end check response
Graphics.Off = False ' reset color
Outpt = "New room" + STR$(Next.Room) + " added." ' make display message
CALL IO.O ' send display message
Room = Next.Room ' update current room number to added room number
END SUB ' end routine to add new room

REM * routine clears room record variables.
REM * input variables:
REM *   Room.Number! - room number.

SUB Clear.Room (Room.Number!)
RoomRecord.ShortDesc = Nul ' set short description
FOR Array.Index = 1 TO 4 ' loop through long description
    CALL PutRoomRecordLongDesc(Array.Index, Nul) ' set long description
NEXT ' end loop through long description
RoomRecord.Action = False ' clear variable
RoomRecord.MonsterClass = False ' clear variable
FOR Array.Index = 1 TO 12 ' loop through room directions
    CALL PutRoomRecordDirect(Array.Index, False) ' clear variable
NEXT ' end loop through directions
FOR Array.Index = 1 TO 20 ' loop through room objects and treasure
    CALL PutRoomRecordObject(Array.Index, False) ' clear variable
    CALL PutRoomRecordHiddenObj(Array.Index, False) ' clear variable
    CALL PutRoomRecordTreasure(Array.Index, False) ' clear variable
    CALL PutRoomRecordTreCharges(Array.Index, False) ' clear variable
    CALL PutRoomRecordFlags(Array.Index, False) ' clear variable
NEXT ' end loop through room objects and treasure
CALL Clear.Container(0, True) ' routine to clear container record
RoomRecord.Container = ContainerRec ' clear variable
CALL Share.Room.Record(Room.Number!) ' write room record number
END SUB ' end routine to clear room record

REM * routine to add, delete, and list room links.

SUB Link.Room
DO ' input entry loop
    Graphics.Off = False ' reset color
    Outpt = "Room link edit:" ' make option message
    CALL IO.O ' send option message
    Graphics.Off = True ' reset color
    Outpt = "[A]dd" ' make message
    CALL IO.O ' send message
    Outpt = "[D]elete" ' make message
    CALL IO.O ' send message
    Outpt = "[L]ist" ' make message
    CALL IO.O ' send message
    Graphics.Off = False ' reset color
    Outpt = "Enter room link option(q to quit)? " ' make input prompt
    No.Input.Out = "Q" ' default input
    CALL IO.I ' get input
    Graphics.Off = True ' reset color
    SELECT CASE UCASE$(Inpt) ' make selection of input
        CASE "A" ' add link
            Outpt = "Enter room number" ' make range prompt
            Max.Rooms! = LOF(RoomFile) / LEN(RoomRecord) ' make range number
            ' routine to get number from range
            CALL Get.Room.Range2(0!, Max.Rooms!, Room.From!)
            IF Room.From! THEN ' check range
                Outpt = "Enter link room number" ' make range prompt
                ' routine to get number from range
                CALL Get.Room.Range2(0!, Max.Rooms!, Room.To!)
                IF Room.To! THEN ' check range
                    ' routine to link two room numbers
                    CALL Add.Link(Room.From!, Room.To!, False)
                END IF ' end check range
            END IF ' end check range
        CASE "D" ' delete link
            Outpts = "Link not deleted." ' make default response
            Outpt = "Enter room number" ' make range prompt
            Max.Rooms! = LOF(RoomFile) / LEN(RoomRecord) ' make range number
            ' routine to get number from range
            CALL Get.Room.Range2(0!, Max.Rooms!, Room.Delete!)
            IF Room.Delete! THEN ' check range
                Outpt = "Enter direction(N/E/S/W/NE/SE/SW/NW/U/D/I/O)? "
                CALL IO.I ' get input
                ' routine to get room link number
                CALL Find.Link(Inpt, Room.Link, False)
                IF Room.Link >= 1 AND Room.Link <= 12 THEN ' check link number
                    CALL Read.Room.Record(Room.Delete!) ' get room record
                    CALL PutRoomRecordDirect(Room.Link, False) ' clear room link number
                    CALL Share.Room.Record(Room.Delete!) ' write room record
                    Outpts = "Room" + STR$(Room.Delete!) + ", " + RTRIM$(Direction(Room.Link)) + " link removed."
                END IF ' end check link number
            END IF ' end check range
            Outpt = Outpts ' store response
            CALL IO.O ' send message
        CASE "L" ' list links
            Graphics.Off = False ' reset color
            Outpt = "Enter range of room numbers:" ' make display message
            CALL IO.O ' send message
            Graphics.Off = True ' reset color
            Max.Rooms! = LOF(RoomFile) / LEN(RoomRecord) ' store length of room file
            ' get range of rooms to list
            CALL Get.Room.Range(Max.Rooms!, Room.List1!, Room.List2!)
            Allow.Break = True ' set allow break flag
            Break = False ' reset control-k flag
            Continue = False ' set continuous flag
            Page.Length = False ' reset page length counter
            FOR Room.Number! = Room.List1! TO Room.List2! ' loop through rooms to list
                IF Page.Length + 4 >= User.PageLength THEN ' compare page length
                    Page.Length = False ' clear page length
                    IF Continue = False THEN ' check continuous flag
                        CALL More.Prompt ' routine to pause
                        IF No THEN ' check more promtp response
                            EXIT FOR ' exit room link display loop
                        END IF ' end check response
                    END IF ' end check continuouu flag
                END IF ' end compare page length
                CALL Read.Room.Record(Room.Number!) ' get next room record
                CALL Display.Room.Links(Room.Number!) ' display room links
                IF Break OR No THEN ' check break flag
                    EXIT FOR ' exit display loop
                END IF ' end check break flag
                Page.Length = Page.Length + 4 ' increment page length counter
            NEXT ' end loop through rooms
            Allow.Break = False ' reset allow break flag
            IF Break THEN ' check control-k flag
                Break = False ' reset control-k flag
                Outpt = Nul ' set output to null
                CALL IO.O ' send empty return
            END IF ' end check control-k flag
            IF Page.Length THEN ' check page length counter
                CALL More.Prompt ' pause prompt
            END IF ' end check page length
        CASE "Q" ' quit
            EXIT DO ' exit input loop
    END SELECT ' end selection of input
LOOP ' end input loop
CALL Read.Room.Record(Room) ' get current room record
END SUB ' end DM link routine

REM * routine adds links between two room numbers.
REM * input variables:
REM *   Room.Number1! - room to link.
REM *   Room.Number2! - room to link.
REM *   Entry.Link - default entry link.

SUB Add.Link (Room.Number1!, Room.Number2!, Entry.Link)
' make direction link prompt
Outpt = "Enter direction(N/E/S/W/NE/SE/SW/NW/U/D/I/O)? "
CALL IO.I ' get input
' routine converts direction to link number
CALL Find.Link(Inpt, Direction.Number, Entry.Link)
IF Direction.Number = False OR Direction.Number = 13 THEN ' check link number
    Outpt = "Link not added." ' make error message
    CALL IO.O ' send message
    EXIT SUB ' exit routine
END IF ' end check link number
CALL Read.Room.Record(Room.Number1!) ' get room number one to link
CALL PutRoomRecordDirect(Direction.Number, Room.Number2!) ' add link to room
CALL Share.Room.Record(Room.Number1!) ' write room record
' make link message
Outpt = "Room" + STR$(Room.Number1!) + " link added to room" + STR$(Room.Number2!) + "."
CALL IO.O ' send message
' make link back prompt
Outpt = "Link room" + STR$(Room.Number2!) + " back to room" + STR$(Room.Number1!) + "(y/n)? "
No.Input.Out = "Y" ' default input
CALL IO.I ' get input
IF Yes THEN ' check response
    CALL Find.Back.Link(Direction.Number, Return.Link) ' get return link
    IF Return.Link THEN ' check return link
        CALL Read.Room.Record(Room.Number2!) ' get room record
        ' add return link to first room
        CALL PutRoomRecordDirect(Return.Link, Room.Number1!)
        CALL Share.Room.Record(Room.Number2!) ' write room record
        ' make return link message
        Outpt = "Room" + STR$(Room.Number2!) + " link added back to room" + STR$(Room.Number1!) + "."
        CALL IO.O ' send message
    END IF ' end check return link
END IF ' end check response
END SUB ' end routine to link two rooms

REM * routine converts direction string to number.
REM * input variables:
REM *   Direction.Name$ - string of direction name.
REM *   Entry.Link - default entry link number.
REM * output variables:
REM *   Direction.Number - direction number.

SUB Find.Link (Direction.Name$, Direction.Number, Entry.Link)
SELECT CASE UCASE$(Direction.Name$) ' select direction string
    CASE "N" ' north
        Direction.Number = 1 ' direction number
    CASE "E" ' east
        Direction.Number = 2 ' direction number
    CASE "S" ' south
        Direction.Number = 3 ' direction number
    CASE "W" ' west
        Direction.Number = 4 ' direction number
    CASE "NE" ' northeast
        Direction.Number = 5 ' direction number
    CASE "SE" ' southeast
        Direction.Number = 6 ' direction number
    CASE "SW" ' southwest
        Direction.Number = 7 ' direction number
    CASE "NW" ' northwest
        Direction.Number = 8 ' direction number
    CASE "U" ' up
        Direction.Number = 9 ' direction number
    CASE "D" ' down
        Direction.Number = 10 ' direction number
    CASE "I" ' in
        Direction.Number = 11 ' direction number
    CASE "O" ' out
        Direction.Number = 12 ' direction number
    CASE "G" ' go to portal
        Direction.Number = 13 ' go to direction number
    CASE ELSE ' otherwise
        Direction.Number = Entry.Link ' no direction found
END SELECT ' end selection of direction
END SUB ' end routine to convert direction

REM * routine determines direction opposite to input direction.
REM * input variables:
REM *   Direction.Number - direction number.
REM * output variables:
REM *   Return.Direction - opposite direction number.

SUB Find.Back.Link (Direction.Number, Return.Direction)
SELECT CASE Direction.Number ' selection of direction number
    CASE 1 ' north
        Return.Direction = 3 ' south
    CASE 2 ' east
        Return.Direction = 4 ' west
    CASE 3 ' south
        Return.Direction = 1 ' north
    CASE 4 ' west
        Return.Direction = 2 ' east
    CASE 5 ' northeast
        Return.Direction = 7 ' southwest
    CASE 6 ' southeast
        Return.Direction = 8 ' northwest
    CASE 7 ' southwest
        Return.Direction = 5 ' northeast
    CASE 8 ' northwest
        Return.Direction = 6 ' southeast
    CASE 9 ' up
        Return.Direction = 10 ' down
    CASE 10 ' down
        Return.Direction = 9 ' up
    CASE 11 ' in
        Return.Direction = 12 ' out
    CASE 12 ' out
        Return.Direction = 11 ' in
    CASE ELSE ' default
        Return.Direction = 0 ' no direction
END SELECT ' end selection of direction
END SUB ' end routine to find return direction

REM * routine displays room number links.
REM * input variables:
REM *   Room.Number! - room number to display.

SUB Display.Room.Links (Room.Number!)
IF Room.Number! <= 0! THEN ' check number
    EXIT SUB ' exit subroutine
END IF ' end check number
Graphics.Off = False ' reset color
Outpt = "Room number" + STR$(Room.Number!) + ":" ' make room number message
CALL IO.O ' send message
Graphics.Off = True ' reset color
Outpt = Nul ' clear output string
FOR Link.Number = 1 TO 4 ' loop through room directions
    ' make display line containing room directions
    Inpt = MID$("NESW", Link.Number, 1) + "  " + STR$(GetRoomRecordDirect(Link.Number))
    Outpt = Outpt + Inpt + SPACE$(10 - LEN(Inpt)) ' pad with blanks
NEXT ' end loop through room directions
CALL IO.O ' send line one of room directions
Outpt = Nul ' clear output string
FOR Link.Number = 5 TO 8 ' loop through room directions
    ' make display line containing room directions
    Inpt = MID$("NESESWNW", (Link.Number - 4) * 2 - 1, 2) + " " + STR$(GetRoomRecordDirect(Link.Number))
    Outpt = Outpt + Inpt + SPACE$(10 - LEN(Inpt)) ' pad with blanks
NEXT ' end loop through room directions
CALL IO.O ' send line one of room directions
Outpt = Nul ' clear output string
FOR Link.Number = 9 TO 12 ' loop through room directions
    ' make display line containing room directions
    Inpt = MID$("UDIO", Link.Number - 8, 1) + "  " + STR$(GetRoomRecordDirect(Link.Number))
    Outpt = Outpt + Inpt + SPACE$(10 - LEN(Inpt)) ' pad with blanks
NEXT ' end loop through room directions
CALL IO.O ' send line two of room directions
Outpt = Nul ' clear output string
END SUB ' end routine to display room links

REM * routine moves player in a certain direction.
REM * input variables:
REM *   Direction.Number - direction to go.
REM * processing variables:
REM *   New.Room - true if room can be entered, false if not.

SUB Go.Direction (Direction.Number)
' change last command to direction
Entry.Command = Last.Command.Number ' store room entry command
Last.Command = UCASE$(Direction(Direction.Number))
Last.Command.Number = True ' reset last command number
' routine to check if next room can be entered
CALL Verify.Room(Direction.Number)
IF New.Room THEN ' check next room variable
    CALL Enter.Room ' routine to move the player
END IF ' end check next room
END SUB ' end routine to move player

REM * routine compares player input to direction.
REM * input variables:
REM *   User.Command - original command input.
REM * output variables:
REM *   Direction.Number - contains direction number.

SUB Get.Direction (Direction.Number)
FOR Direction.Number = 1 TO 12 ' loop through direction names
    Outpts = Direction(Direction.Number) ' store direction name
    Outpts = RTRIM$(Outpts) ' trim name
    Outpts = UCASE$(Outpts) ' uppercase name
    IF User.Command = Outpts THEN ' compare to player direction
        EXIT SUB ' exit routine
    END IF ' end check directions
NEXT ' end loop through direction names
' routine to get direction
CALL Find.Link(User.Command, Direction.Number, False)
IF Direction.Number = 13 THEN ' check enter portal link
    Direction.Number = False ' reset direction
END IF ' end check direction
END SUB ' end routine to compare direction

REM * routine to add item of treasure to player inventory.
REM * input variables:
REM *   Treasure.Number - treasure file index number.
REM *   Treasure.Charges - treasure charges.
REM * output variables:
REM *   Item.Added - true if item added, false if not.

SUB Add.Inventory (Treasure.Number, Treasure.Charges, Item.Added)
Item.Added = False ' clear return variable
FOR Array.Index = 1 TO 20 ' loop through all player inventory
    IF GetUserRecordInv(Array.Index) = False THEN ' check for empty inventory
        CALL PutUserRecordInv(Array.Index, Treasure.Number) ' add treasure index
        CALL PutUserRecordCharges(Array.Index, Treasure.Charges) ' add treasure charges
        Weight = Weight + TreasureRecord.Weight ' increment player weight
        Item.Added = True ' set return flag
        EXIT SUB ' exit routine
    END IF ' end check empty inventory
NEXT ' end loop through player inventory
END SUB ' end routine to add item of treasure to player inventory

REM * routine to add an object to player inventory.
REM * input variables:
REM *   Object.Number - object file index number.
REM * output variables:
REM *   Item.Added - true if item added, false if not.

SUB Add.Object (Object.Number, Item.Added)
Item.Added = False ' clear return variable
FOR Array.Index = 1 TO 20 ' loop through all player inventory
    IF GetUserRecordObject(Array.Index) = False THEN ' check for empty inventory
        CALL PutUserRecordObject(Array.Index, Object.Number) ' add object index
        Item.Added = True ' set return flag
        EXIT SUB ' exit routine
    END IF ' end check empty inventory
NEXT ' end loop through player inventory
END SUB ' end routine to add item of treasure to player inventory

REM * routine removes an item of treasure from player inventory.
REM * input variables:
REM *   Inventory.Number - number of inventory.
REM *   Leave.Item - false to leave inventory in room, true to discard.

SUB Discard.Inventory (Inventory.Number, Leave.Item)
' store player treasure index
Inventory.Index = GetUserRecordInv(Inventory.Number)
' store player treasure charges
Inventory.Charges = GetUserRecordCharges(Inventory.Number)
CALL Read.Record(TreasureFile, ABS(Inventory.Index)) ' get treasure record of item
Weight = Weight - TreasureRecord.Weight ' subtract weight
IF Weight < False THEN ' compare weight
    Weight = False ' clear weight
END IF ' end check weight
CALL Discard.Inventory.Treasure(Inventory.Number) ' remove inventory item
IF GetUserRecordInv(1) = False THEN ' check player inventory empty
    Weight = False ' clear weight
END IF ' end check player inventory
SELECT CASE Inventory.Number ' select weapon
    CASE Weapon4 ' check armor being dropped
        Weapon1 = False ' clear item
        Weapon4 = False ' clear item
    CASE Weapon5 ' check shield being dropped
        Weapon3 = False ' clear item
        Weapon5 = False ' clear item
    CASE Weapon6 ' check weapon being dropped
        Weapon2 = False ' clear item
        Weapon6 = False ' clear item
        Weapon10 = False ' clear item
    CASE Weapon7 ' check ring being dropped
        Weapon7 = False ' clear item
        Weapon8 = False ' clear item
        Weapon9 = False ' clear item
        Weapon11 = False ' clear item
END SELECT ' end check item
' check inventory number
IF Inventory.Number < Weapon4 THEN ' check armor index
    Weapon4 = Weapon4 - 1 ' shift item index
END IF ' end check inventory
' check inventory number
IF Inventory.Number < Weapon5 THEN ' check shield index
    Weapon5 = Weapon5 - 1 ' shift item index
END IF ' end check inventory
' check inventory number
IF Inventory.Number < Weapon6 THEN ' check weapon index
    Weapon6 = Weapon6 - 1 ' shift item index
END IF ' end check inventory
' check inventory number
IF Inventory.Number < Weapon7 THEN ' check ring index
    Weapon7 = Weapon7 - 1 ' shift item index
END IF ' end check inventory
IF Leave.Item = False THEN ' verify drop item in room
    ' routine to add item to room
    CALL Add.Room.Treasure(Inventory.Index, Inventory.Charges, False, Item.Added)
END IF ' end check room
END SUB ' end routine to remove player item

REM * routine adds item to room.
REM * input variables:
REM *   Treasure.Number - treasure file index.
REM *   Treasure.Charges - treasure charges.
REM *   Treasure.Flags - flags
REM * output variables:
REM *   Item.Added - return true if item added to room, false if not.

SUB Add.Room.Treasure (Treasure.Number, Treasure.Charges, Treasure.Flags, Item.Added)
Item.Added = False ' clear return flag
FOR Array.Index = 1 TO 20 ' loop through room treasure inventory
    ' check empty room inventory
    IF GetRoomRecordTreasure(Array.Index) = False THEN
        CALL PutRoomRecordTreasure(Array.Index, Treasure.Number) ' add treasure index
        CALL PutRoomRecordTreCharges(Array.Index, Treasure.Charges) ' add treasure charges
        CALL PutRoomRecordFlags(Array.Index, Treasure.Flags) ' add treasure flags
        CALL Share.Room.Record(Room) ' write room record
        Item.Added = True ' set return flag
        EXIT SUB ' exit routine
    END IF ' end check empty inventory
NEXT ' end loop through room treasure inventory
END SUB ' end routine to add item to room

REM * routine removes item of treasure from room.
REM * input variables:
REM *   Inventory.Number - room inventory number to remove.

SUB Discard.Room.Treasure (Inventory.Number)
CALL PutRoomRecordTreasure(Inventory.Number, False) ' clear treasure items
CALL PutRoomRecordTreCharges(Inventory.Number, False) ' clear treasure items
CALL PutRoomRecordFlags(Inventory.Number, False) ' clear treasure items
CALL Share.Room.Record(Room) ' write room record
END SUB ' end routine to remove item from room

REM * routine adds object to room.
REM * input variables:
REM *   Object.Number - object index.
REM *   Hide.Item - true to hide object.
REM * output variables:
REM *   Item.Added - true if item added, false if not.

SUB Add.Room.Object (Object.Number, Hide.Item, Item.Added)
Item.Added = False ' clear return flag
FOR Array.Index = 1 TO 20 ' loop through room object inventory
    ' check empty object inventory
    IF GetRoomRecordObject(Array.Index) = False THEN
        CALL PutRoomRecordObject(Array.Index, Object.Number) ' add object index
        CALL PutRoomRecordHiddenObj(Array.Index, Hide.Item) ' add hidden object
        CALL Share.Room.Record(Room) ' write room record
        Item.Added = True ' set return flag
        EXIT SUB ' exit routine
    END IF ' end check empty object inventory
NEXT ' end loop through room object inventory
END SUB ' end routine to add item to room

REM * routine removes an object from room.
REM * input variables:
REM *   Inventory.Number - room inventory number to remove.

SUB Discard.Room.Object (Inventory.Number)
CALL PutRoomRecordObject(Inventory.Number, False) ' clear object item
CALL PutRoomRecordHiddenObj(Inventory.Number, False) ' clear object item
CALL Share.Room.Record(Room) ' write room record
END SUB ' end routine to remove item from room

REM * routine removes an object from inventory.
REM * input variables:
REM *   Inventory.Number - player inventory number to remove.

SUB Discard.Inventory.Object (Inventory.Number)
FOR Inventory.Count = Inventory.Number TO 19 ' loop packer
    ' shift object items
    CALL PutUserRecordObject(Inventory.Count, GetUserRecordObject(Inventory.Count + 1))
NEXT ' end packing
CALL PutUserRecordObject(20, False) ' clear last object
END SUB ' end routine to remove item from room

REM * routine removes a treasure from inventory,
REM * and packs user inventory.
REM * input variables:
REM *   Inventory.Number - player inventory number to remove.

SUB Discard.Inventory.Treasure (Inventory.Number)
FOR Inventory.Count = Inventory.Number TO 19 ' loop packer
    ' shift treasure items
    CALL PutUserRecordInv(Inventory.Count, GetUserRecordInv(Inventory.Count + 1))
    ' shift treasure items
    CALL PutUserRecordCharges(Inventory.Count, GetUserRecordCharges(Inventory.Count + 1))
NEXT ' end packing
CALL PutUserRecordInv(20, False) ' clear last treasure
CALL PutUserRecordCharges(20, False) ' clear last treasure
END SUB ' end routine to remove item from room

REM * routine removes treasure from room after player leaves to new room.
REM * input variables:
REM *   Room - number of room to clean.

SUB Clean.Room
IF Room > False AND Room <= LOF(RoomFile) / LEN(RoomRecord) THEN ' check file bounds
    CALL Read.Room.Record(Room) ' get room record
    IF RoomRecord.Container.Permanent = False THEN ' check permanent container
        CALL Clear.Container(0, True) ' routine to clear container record
        RoomRecord.Container = ContainerRec ' store container record in room
        CALL Share.Room.Record(Room) ' write room record
    END IF ' end check permanent container
    FOR Array.Index = 1 TO 20 ' loop through room treasure
        Treasure.Number = GetRoomRecordTreasure(Array.Index) ' store treasure index
        IF Treasure.Number > False AND Treasure.Number <= LOF(TreasureFile) / LEN(TreasureRecord) THEN ' file bounds
            CALL Read.Record(TreasureFile, Treasure.Number) ' get treasure record
            IF TreasureRecord.Permanent = False THEN ' check permanent treasure
                IF TreasureRecord.Invisible = False THEN ' check invisible treasure
                    ' check treasure flags
                    IF GetRoomRecordFlags(Array.Index) = False THEN
                        ' remove item from room
                        CALL Discard.Room.Treasure(Array.Index)
                    END IF ' end check flags
                END IF ' end check invisible
            END IF ' end check permanent
        END IF ' end check file bounds
    NEXT ' end loop through room treasure
END IF ' end check room number range
END SUB ' end routine to clean room

REM * routine lists users.

SUB User.List
Graphics.Off = True ' reset color
GOSUB Heading ' subroutine to display heading
Allow.Break = True ' enable control-k checking
Break = False ' reset control-k flag
Continue = False ' reset continuous flag
Page.Length = 3 ' set page length
FOR User.Number = 1 TO LOF(UserFile) / LEN(UserRecord) ' loop through all users
    CALL Read.Record(UserFile2, User.Number) ' get next user record
    Inpt = UserRecord2.CodeName ' store user codename
    CALL Decrypt(Inpt) ' routine to decrypt codename
    Inpt = LCASE$(Inpt) ' lowercase codename,
    List.User = True ' set list user flag
    IF LEFT$(Inpt, 9) = Deleted$ THEN ' check user record exists
        List.User = False ' reset list user flag
    ELSE ' check user
        IF UserRecord2.Flags AND Locked.User THEN ' check locked user
            IF Normal.User THEN ' compare DM or Sysop
                List.User = False ' reset list user flag
            END IF ' end compare normal user
        END IF ' end check locked mailbox
    END IF ' end check user
    IF List.User THEN ' check list user flag
        Outpt = MID$(STR$(User.Number), 2) ' make output line
        Outpt = Outpt + SPACE$(9 - LEN(Outpt)) ' with user number,
        MID$(Inpt, 1, 1) = UCASE$(MID$(Inpt, 1, 1)) ' with codename,
        Outpt = Outpt + Inpt + " " ' make output line
        Inpt = UserRecord2.ClassName ' with user class name,
        CALL Decrypt(Inpt) ' decrypt classname
        Outpt = Outpt + Inpt + " " ' make output line
        IF UserRecord2.Race <= False THEN ' verify race in bounds
            UserRecord2.Race = 1 ' reset race
        END IF ' end verify race
        Inpt = Race(UserRecord2.Race) ' with user race name
        Inpt = RTRIM$(Inpt) ' make output line
        Inpt = Inpt + SPACE$(8 - LEN(Inpt)) ' append blanks
        Outpt = Outpt + Inpt ' with race name,
        IF UserRecord2.Level <= False THEN ' check player level
            Inpt = " " + Dead$ ' player is dead, append
        ELSE ' check level
            Inpt = STR$(UserRecord2.Level) ' add player level,
        END IF ' end check level
        Inpt = Inpt + SPACE$(7 - LEN(Inpt)) ' make output line
        IF UserRecord2.ClassType >= AsstDM THEN ' check special class type,
            Inpt = Inpt + "*" ' add an asterick for DMs
        END IF ' end check class type
        IF UserRecord2.Flags AND Locked.User THEN ' check locked user
            Inpt = Inpt + Mask$ ' add mask character for locked mailbox
        END IF ' end check class type
        Outpt = Outpt + Inpt ' add to output line
        CALL IO.O ' send output line
        IF Break THEN ' check control-k entered
            EXIT FOR ' exit loop through user file
        END IF ' end check control-k
        Page.Length = Page.Length + 1 ' increment page length
        IF Page.Length = UserRecord.Pagelength THEN ' compare page length
            Page.Length = 3 ' reset page length
            IF Continue = False THEN ' check continuous flag
                CALL More.Prompt ' pause for more
                IF No THEN ' more response
                    EXIT FOR ' exit loop through user file
                END IF ' end check response
                GOSUB Heading ' subroutine to display heading
            END IF ' end check continuous flag
        END IF ' end compare page length
    END IF ' end check valid user
NEXT ' end loop through user file
Allow.Break = False ' disable control-k checking
IF Break THEN ' check control-k flag
    Break = False ' reset control-k flag
    Outpt = Nul ' set output to null
    CALL IO.O ' send empty return
END IF ' end check control-k flag
IF Page.Length > 3 THEN ' recheck page length
    CALL More.Prompt ' display more prompt
END IF ' end check last page length
EXIT SUB ' exit routine

Heading:
' make heading message
Outpt = "The Adventure Door v" + Version$ + " User List For " + clock$ + "."
CALL IO.O ' send message
Outpt = Nul ' empty cr/lf
CALL IO.O ' send empty cr/lf
' make heading message
Outpt = "Number   User Name                      Class Name           Race     Level DM"
CALL IO.O ' send heading message
Outpt = STRING$(78, "-") ' make heading
CALL IO.O ' send heading
RETURN ' exit subroutine
END SUB ' end routine to display users

REM * routine reads player userfile record, sets some variables.
REM * input variables:
REM *   User.Index - number of user file record.

SUB Get.User.Record
CALL Read.Record(UserFile, User.Index) ' read user file record
Weight = False ' clear inventory weight
FOR Inventory.Number = 1 TO 20 ' loop through player inventory
    Treasure.Number = GetUserRecordInv(Inventory.Number) ' store treasure number
    IF Treasure.Number > False AND Treasure.Number <= LOF(TreasureFile) / LEN(TreasureRecord) THEN ' file bounds
        CALL Read.Record(TreasureFile, Treasure.Number) ' get treasur record
        Weight = Weight + TreasureRecord.Weight ' add weight
    END IF ' end check file bounds
NEXT ' end loop through player userfile record
User.Echo = UserRecord.Echo ' store user preference
User.LineFeeds = UserRecord.Linefeeds ' store user preference
User.LineLength = UserRecord.Linelength ' store user preference
User.PageLength = UserRecord.Pagelength ' store user preference
User.Wordwrap = UserRecord.Wordwrap ' store user preference
CALL Get.User.Stats ' routine to set any special player statistics
Room = UserRecord.Room ' store player room number
Hidden.Player = False ' reset hidden player flag
IF UserRecord.Brief THEN ' check player brief mode
    Action.Prompt = "Next?" ' store command prompt
ELSE ' check player
    Action.Prompt = "Command? " ' store command prompt
END IF ' end compare player brief mode
IF UserRecord.Beauty <= 1 OR UserRecord.Beauty > MaxStat THEN ' check lady player
    UserRecord.Beauty = INT(RND * 15 + 5) ' recalculate lady stats
END IF ' end check lady stats
' check lady player
IF UserRecord.Glamour <= 1 OR UserRecord.Glamour > MaxStat THEN
    UserRecord.Glamour = INT(RND * 15 + 5) ' recalculate lady stats
END IF ' end check lady stats
Sorting = UserRecord.Sort ' get user sort preference
END SUB ' end routine to read user record

REM * routine sets any special player statistics and attributes.

SUB Get.User.Stats
IF UserRecord.Flags AND Special.Char1 THEN ' check player is town mayor
    Town.Mayor = True ' set town mayor flag
ELSE ' check player
    Town.Mayor = False ' set town mayor flag
END IF ' end check player special stats
IF UserRecord.Flags AND Special.Char2 THEN ' check player is governor
    Governor = True ' set governor flag
ELSE ' check player
    Governor = False ' set governor flag
END IF ' end check player special stats
IF UserRecord.Flags AND Special.Char3 THEN ' check player is guild master
    Guild.Master = True ' set guild master flag
ELSE ' check player
    Guild.Master = False ' set guild master flag
END IF ' end check player special stats
IF UserRecord.Flags AND Special.Char4 THEN ' check player is sysop
    Sysop = True ' set sysop flag
ELSE ' check player
    Sysop = False ' set sysop flag
END IF ' end check player special stats
IF UserRecord.ClassType = AsstDM THEN ' check player is assistant DM
    Dungeon.Master.Assistant = True ' set asst. dm flag
ELSE ' check player
    Dungeon.Master.Assistant = False ' set asst. dm flag
END IF ' end check player
IF UserRecord.ClassType = DM THEN ' check player is dungeon master
    Dungeon.Master = True ' set dm flag
ELSE ' check player
    Dungeon.Master = False ' set dm flag
END IF ' end check player
' check special player types
IF Dungeon.Master OR Dungeon.Master.Assistant OR Sysop THEN
    Normal.User = False ' set normal player mode off
ELSE ' check player
    Normal.User = True ' set normal player mode on
END IF ' end check player
END SUB ' end routine to set player special statistics/attributes

REM * routine writes user record.

SUB Put.User.Record
UserRecord.Echo = User.Echo ' write user preference
UserRecord.Linefeeds = User.LineFeeds ' write user preference
UserRecord.Linelength = User.LineLength ' write user preference
UserRecord.Pagelength = User.PageLength ' write user preference
UserRecord.Wordwrap = User.Wordwrap ' write user preference
UserRecord.Sort = Sorting ' user sort preference
UserRecord.Room = Room ' store current room
CALL Share.Record(UserFile, User.Index) ' write user record
END SUB ' end routine to write user record

REM * routine updates player health, room lights, drunkeness, and poison.
REM * input variables:
REM *   Room.Rate - stores rounds counter.
REM *   Room.Health.Rate - stores health update rate.

SUB Health.Update
Graphics.Off = False ' reset color
Room.Rate = Room.Rate + 1 ' increment room health counter
IF Room.Rate < Room.Health.Rate THEN ' compare health rate
    EXIT SUB ' exit routine
END IF ' end check counter
Room.Rate = False ' reset room health counter
' determine any lights in player inventory go out
FOR Inventory.Number = 1 TO 20 ' loop through player inventory
    ' get player inventory charges
    Charges.Number = GetUserRecordCharges(Inventory.Number)
    IF Charges.Number < False THEN ' check for light on
        Charges.Number = Charges.Number + 1 ' decrement negatively light charges
        CALL PutUserRecordCharges(Inventory.Number, Charges.Number) ' store new charges
        IF Charges.Number = False THEN ' compare charges
            ' get treasure record
            CALL Read.Record(TreasureFile, GetUserRecordInv(Inventory.Number))
            Outpts = TreasureRecord.ShortName ' get light name
            Outpts = RTRIM$(Outpts) ' trim name
            Outpts = LCASE$(Outpts) ' lowercase name
            Outpt = "The " + Outpts + " went out!" ' make light out message
            CALL IO.O ' send message
        END IF ' end compare charges remaining
    END IF ' end check light on
NEXT ' end loop through player inventory
IF Intoxicated > False THEN ' verify player drunk
    Intoxicated = Intoxicated - 1 ' decrement drunkeness
    IF Intoxicated <= False THEN ' compare drunkeness counter
        Intoxicated = False ' reset counter
        Outpt = "Your drunk is over.." ' make message
    ELSE ' check drunk
        UserRecord.Fatigue = UserRecord.Fatigue - 2 ' decrement fatigue for drunk
        IF UserRecord.Fatigue <= False THEN ' compare fatigue
            UserRecord.Fatigue = False ' reset fatigue
            Intoxicated = False ' reset drunkeness
            Outpt = "Your drunk is over.." ' make message
        ELSE ' compare still drunk
            Outpt = "You feel drunk!" ' make message
        END IF ' end compare drunk
    END IF ' end check drunk
    CALL IO.O ' send drunkeness message
ELSE ' verify drunk player
    New.Stat! = UserRecord.Fatigue + 4 ' increment player fatigue
    IF New.Stat! > MaxInt THEN ' check maximum fatigue
        New.Stat! = MaxInt ' reduce to maximum integer
    END IF ' end check maximum fatigue
    UserRecord.Fatigue = CINT(New.Stat!) ' store new fatigue
END IF ' end verify drunk player
IF UserRecord.Poison THEN ' verify player poisoned
    UserRecord.Vitality = UserRecord.Vitality - 2 ' decrement player vitality
    Outpt = "You feel poison running through your veins!" ' make message
    CALL IO.O ' send poisoned message
    IF UserRecord.Vitality <= False THEN ' check vitality
        UserRecord.Vitality = False ' reset vitality
        Message1 = "You finally died from your poisonous wounds!" ' message
        CALL Player.Died ' routine for dead player
    END IF ' end check vitality
ELSE ' verify poisoned player
    New.Stat! = UserRecord.Vitality + 3 ' increment player vitality
    IF New.Stat! > MaxInt THEN ' check maximum integer
        New.Stat! = MaxInt ' reset to maximum integer
    END IF ' end check maximum integer
    UserRecord.Vitality = CINT(New.Stat!) ' store new vitality
END IF ' end verify poisoned player
New.Stat! = UserRecord.Magic + 2 ' increment player magic points
IF New.Stat! > MaxInt THEN ' compare magic points to maximum integer
    New.Stat! = MaxInt ' reset to maximum integer
END IF ' end check maximum integer
UserRecord.Magic = CINT(New.Stat!) ' store new magic points
New.Stat! = UserRecord.Psionic + 1 ' increment psionic points
IF New.Stat! > MaxInt THEN ' check maximum psionic points
    New.Stat! = MaxInt ' reset to maximum integer
END IF ' end check maximum integer
UserRecord.Psionic = CINT(New.Stat!) ' store new psionic points
CALL New.Stats ' routine to update statistics
IF Invisible > False THEN ' check invisible counter
    Invisible = Invisible - 1 ' decrement invisible counter
    IF Invisible <= False THEN ' compare counter
        UserRecord.Invisible = False ' reset invisible
        Invisible = False ' reset invisible
        Outpt = "You are no longer invisible!" ' make message
        CALL IO.O ' send invisible message
    END IF ' end compare counter
END IF ' end check counter
END SUB ' end health update routine

REM * routine searches current room, displays hidden items.

SUB Search.Room
Outpt = "You search the room.." ' make message
CALL IO.O ' send search message
Graphics.Off = True ' reset color
Outpt = "You find " ' make first display message
Carriage.Return = True ' disable cr/lf
CALL IO.O ' send first message
Items.Displayed = False ' items displayed counter
FOR Room.Objects = 1 TO 20 ' loop through room objects
    Object.Number = GetRoomRecordObject(Room.Objects) ' get room object index
    ' check object file bounds
    IF Object.Number > False AND Object.Number <= LOF(ObjectFile) / LEN(ObjectRecord) THEN
        CALL Read.Record(ObjectFile, Object.Number) ' get object record
        Display.Item = False ' set display flag
        Hidden.Flag = GetRoomRecordHiddenObj(Room.Objects) ' reset hidden flag
        IF ObjectRecord.Invisible OR Hidden.Flag THEN ' check object invisible
            IF RND < .5 THEN ' random chance
                Display.Item = True ' set display flag
            END IF ' end random chance
        END IF ' end check invisible object
        IF ObjectRecord.Hidden THEN ' check object hidden
            IF Normal.User THEN ' verify non DM
                Display.Item = False ' set display flag
            END IF ' end verify normal player
        END IF ' end check hidden object
        IF Display.Item THEN ' check display flag
            Carriage.Return = True ' disable cr/lf
            CALL IO.O ' send previous string
            Outpt = RTRIM$(ObjectRecord.ObjectName) + ", " ' store object name
            Items.Displayed = Items.Displayed + 1 ' increment item displayed counter
        END IF ' end check display flag
    END IF ' end check file bounds
NEXT ' end loop through room objects
FOR Room.Treasure = 1 TO 20 ' loop through room treasure
    Treasure.Number = GetRoomRecordTreasure(Room.Treasure) ' get treasure index
    IF Treasure.Number > False AND Treasure.Number <= LOF(TreasureFile) / LEN(TreasureRecord) THEN ' file bounds
        CALL Read.Record(TreasureFile, Treasure.Number) ' get treasure record
        Display.Item = False ' set display flag
        ' check treasure hidden
        IF GetRoomRecordFlags(Room.Treasure) = Hidden.Object THEN
            Hidden.Flag = True ' reset flag
        ELSE ' check treasure
            Hidden.Flag = False ' reset flag
        END IF ' end check treasure
        ' check treasure invisible
        IF TreasureRecord.Invisible OR Hidden.Flag THEN
            IF RND < .5 THEN ' random chance
                Display.Item = True ' set display flag
            END IF ' end random chance
        END IF ' end check hidden treasure
        IF Display.Item THEN ' check display flag
            Carriage.Return = True ' disable cr/lf
            CALL IO.O ' send previous string
            Outpt = RTRIM$(TreasureRecord.TreasureName) + ", " ' store treasure name
            Items.Displayed = Items.Displayed + 1 ' increment displayed counter
        END IF ' end check display flag
    END IF ' end check file bounds
NEXT ' end loop through room treasure
IF Items.Displayed = False THEN ' check items displayed counter
    Outpt = "nothing.." ' make message
ELSE ' check item counter
    Outpt = LEFT$(Outpt, LEN(Outpt) - 2) + "." ' trim comma, add period
    IF Items.Displayed > 1 THEN ' check counter
        Outpt = "and " + Outpt ' append string
    END IF ' end check counter
END IF ' end check counter
CALL IO.O ' send message
END SUB ' end routine to search room

REM * routine processes actions, routine is entered when a trigger is
REM * activated.
REM * input variables:
REM *   Activate.Action$ - the display string which activated the action.
REM *   Trigger.Action$ - prefix string of what hit player.

SUB Actions (Activate.Action$, Trigger.Action$)
IF Room = 1 THEN ' check resurrected room
    EXIT SUB ' exit routine
END IF ' end check room
Inpt = Nul ' reset output string
Graphics.Off = False ' reset color
SELECT CASE ActionRecord.Inventory ' selection of room action
    CASE 1 ' inventory action 1 breaks all weapons
        Item.Broke = False ' weapon broke flag
        Weapon2 = False ' reset weapon held
        Weapon6 = False ' reset weapon held
        Weapon10 = False ' reset weapon held
        FOR Inventory.Number = 1 TO 20 ' loop through all player inventory
            Treasure.Number = GetUserRecordInv(Inventory.Number) ' get treasure index
            IF Treasure.Number > False AND Treasure.Number <= LOF(TreasureFile) / LEN(TreasureRecord) THEN 'file bounds
                CALL Read.Record(TreasureFile, Treasure.Number) ' get treasure record
                IF TreasureRecord.Plus THEN ' check weapon plus
                    IF TreasureRecord.Type = False THEN ' check weapon
                        ' check weapon charges
                        IF GetUserRecordCharges(Inventory.Number) THEN
                            ' clear weapon charges
                            CALL PutUserRecordCharges(Inventory.Number, False)
                            Item.Broke = True ' set weapon broke flag
                        END IF ' end check charges
                    END IF ' end check weapon
                END IF ' end check weapon plus
            END IF ' end check file bounds
        NEXT ' end loop through player inventory
        IF Item.Broke THEN ' compare weapon broke flag
            Outpt = Activate.Action$ ' copy routine string
            CALL IO.O ' send string
            Outpt = "All your weapons break!" ' make message
            CALL IO.O ' send weapon message
        END IF ' end compare broke flag
    CASE 2 ' inventory action 2 breaks all shields
        Item.Broke = False ' set shield broke flag
        Weapon3 = False ' clear shield held
        Weapon5 = False ' clear shield held
        FOR Inventory.Number = 1 TO 20 ' loop through all player inventory
            Treasure.Number = GetUserRecordInv(Inventory.Number) ' get treasure index
            IF Treasure.Number > False AND Treasure.Number <= LOF(TreasureFile) / LEN(TreasureRecord) THEN 'file bounds
                CALL Read.Record(TreasureFile, Treasure.Number) ' get treasure record
                IF TreasureRecord.Type < False THEN ' check shield type
                    IF GetUserRecordCharges(Inventory.Number) THEN ' check charges
                        ' clear shield charges
                        CALL PutUserRecordCharges(Inventory.Number, False)
                        Item.Broke = True ' set shield broke flag
                    END IF ' end check charges
                END IF ' end check shield
            END IF ' end check file bounds
        NEXT ' end loop through player inventory
        IF Item.Broke THEN ' compare shield broke flag
            Outpt = Activate.Action$ ' copy routine string
            CALL IO.O ' send string
            Outpt = "All your shields break!" ' make message
            CALL IO.O ' send shield message
        END IF ' end compare broke flag
    CASE 3 ' inventory action 3 breaks all armor
        Item.Broke = False ' set armor broke flag
        Weapon1 = False ' clear armor worn
        Weapon4 = False ' clear armor worn
        FOR Inventory.Number = 1 TO 20 ' loop through all player inventory
            Treasure.Number = GetUserRecordInv(Inventory.Number) ' get treasure index
            IF Treasure.Number > False AND Treasure.Number <= LOF(TreasureFile) / LEN(TreasureRecord) THEN 'file bounds
                CALL Read.Record(TreasureFile, Treasure.Number) ' get treasure record
                IF TreasureRecord.Type > False THEN ' check armor
                    IF GetUserRecordCharges(Inventory.Number) THEN ' check charges
                        ' clear armor charges
                        CALL PutUserRecordCharges(Inventory.Number, False)
                        Item.Broke = True ' set armor broke flag
                    END IF ' end check charges
                END IF ' end check armor
            END IF ' end check file bounds
        NEXT ' end loop through player inventory
        IF Item.Broke THEN ' compare armor broke flag
            Outpt = Activate.Action$ ' copy routine message
            CALL IO.O ' send message
            Outpt = "All your armor breaks!" ' make message
            CALL IO.O ' send armor message
        END IF ' end compare broke flag
    CASE 4 ' inventory action 4 breaks all magic items
        Item.Broke = False ' set magic item broke flag
        Weapon7 = False ' clear ring held
        Weapon8 = False ' clear ring held
        FOR Inventory.Number = 1 TO 20 ' loop through all player inventory
            Treasure.Number = GetUserRecordInv(Inventory.Number) ' get treasure index
            IF Treasure.Number > False AND Treasure.Number <= LOF(TreasureFile) / LEN(TreasureRecord) THEN 'file bounds
                CALL Read.Record(TreasureFile, Treasure.Number) ' get treasure record
                IF TreasureRecord.Spell THEN ' check magic item
                    IF GetUserRecordCharges(Inventory.Number) THEN ' check charges
                        ' clear magic item charges
                        CALL PutUserRecordCharges(Inventory.Number, False)
                        Item.Broke = True ' set magic item broke flag
                    END IF ' end check charges
                END IF ' end check magic item
            END IF ' end check file bounds
        NEXT ' end loop through player inventory
        IF Item.Broke THEN ' compare magic item broke flag
            Outpt = Activate.Action$ ' copy routine string
            CALL IO.O ' send string
            Outpt = "All your magic items break!" ' make message
            CALL IO.O ' send magic item message
        END IF ' end compare broke flag
END SELECT ' end selection of action
IF ActionRecord.Fumble THEN ' verify room fumble action
    CALL Fumble ' routine to fumble weapon/shield
END IF ' end verify fumble action
Teleport.Number = ActionRecord.Teleport
IF Teleport.Number > False AND Teleport.Number <> Room THEN
    Outpt = Activate.Action$ ' copy routine string
    CALL IO.O ' send string
    Outpt = "You are teleported elsewhere!" ' make message
    CALL IO.O ' send teleport message
    Next.Room = Teleport.Number ' store new room number
    Teleported = True ' set teleporting flag
    CALL Enter.Room ' routine to move player to new room
END IF ' end verify teleport action
Room.Hits# = CDBL(INT(ActionRecord.HitPoints)) ' store room hit action
IF Room.Hits# > False THEN ' verify hit points action
    Outpt = Activate.Action$ ' copy routine string
    CALL IO.O ' send string
    Outpt = Trigger.Action$ ' copy second routine string
    CALL Hit.Player(Room.Hits#) ' routine to hit player
END IF ' end verify hit points action
END SUB ' end routine to activate room actions

REM * routine to verify monster blocking exits, then check valid direction.
REM * input variables:
REM *   Direction.Number - direction to go.
REM * output variables:
REM *   Next.Room - number of new room to enter.

SUB Verify.Room (Direction.Number)
New.Room = False ' enter new room flag
IF UserRecord.ClassType < Lady THEN ' check normal player class
    FOR Array.Index = 1 TO Number.Monsters ' loop through monsters in room
        IF MonsterArray(Array.Index).Block THEN ' check monster blocks exits
            ' random percent
            IF RND < (MonsterArray(Array.Index).BlockPercent / 100) THEN
                Inpt = MonsterArray(Array.Index).MonsterName ' store monster name
                Inpt = RTRIM$(Inpt) ' trim name
                Outpt = "The " + Inpt + " blocks your way!" ' make block message
                CALL IO.O ' send block message
                EXIT SUB ' exit routine
            END IF ' end check random percentage
        END IF ' end check monster blocks exits
    NEXT ' end loop through monsters
END IF ' end check normal user
Next.Room = GetRoomRecordDirect(Direction.Number) ' get room number of direction
IF Next.Room = False THEN ' check next room number
    IF Normal.User THEN ' check DM status
        Outpt = "You can't go in that direction!" ' make entry message
        CALL IO.O ' send entry message
    ELSE ' check DM status
        CALL Add.Room(Direction.Number, Room.Added) ' routine to add new room
        New.Room = Room.Added ' store new room flag
    END IF ' end check DM
    EXIT SUB ' exit routine
ELSE ' check next room number
    ' routine to verify room direction restricted
    CALL Restrict(Direction.Number, Restricted)
    IF Restricted THEN ' check room restricted flag
        Outpt = "Your level does not permit entrance to that room!" ' message
        CALL IO.O ' send restricted message
        EXIT SUB ' exit routine
    END IF ' end check room restricted
    ' routine to verify room entry type restricted
    CALL Restrict.Room.Type(Restricted)
    IF Restricted THEN ' check room restricted flag
        Outpt = "You can't walk to that room!" ' message
        CALL IO.O ' send restricted message
        EXIT SUB ' exit routine
    END IF ' end check room restricted
END IF ' end check next room number
New.Room = True ' store next room valid flag
END SUB ' end routine to verify room entry, direction

REM * routine to exit a room with the Out command.

SUB Exit.Room
Entry.Command = Last.Command.Number ' store room entry command
User.Command = "O" ' store out command
Last.Command = "OUT" ' store out command
Last.Command.Number = True ' reset command type
CALL Verify.Room(5) ' routine to verify valid room
IF New.Room THEN ' check valid room flag
    CALL Enter.Room ' routine to move player to room
END IF ' end check valid room
END SUB ' end routine to go Out

REM * routine to use the Up direction.

SUB Climb
Entry.Command = Last.Command.Number ' store room entry command
User.Command = "U" ' store up command
Last.Command = "UP" ' store up command
Last.Command.Number = True ' store command type
CALL Verify.Room(6) ' routine to check valid room
IF New.Room THEN ' check valid room flag
    CALL Enter.Room ' routine to move player to room
END IF ' end check valid room
END SUB ' end routine to go Up

REM * routine processes room traps.

SUB Traps
New.Room = False ' reset next room flag
IF RND < .5 THEN ' random chance
    Outpt = "It's trapped! " ' make trap message
    SELECT CASE ObjectRecord.Trap ' selection of room trap type
        CASE 1 ' type 1
            Outpt = Outpt + "Poison needles!" ' make trap message
            CALL IO.O ' send trap message
            UserRecord.Poison = True ' set player poison flag
        CASE 2 ' type 2
            New.Room = ObjectRecord.Teleport ' store teleport room number
            ' file bounds
            IF New.Room > False AND New.Room <= LOF(RoomFile) / LEN(RoomRecord) THEN
                IF New.Room <> Room THEN ' check destination room for recursion
                    Outpt = Outpt + "Falling door!" ' make trap message
                    CALL IO.O ' send trap message
                    Pass.Door = False ' clear pass door flag
                    Number.Monsters = False ' clear number of monsters
                    Next.Room = New.Room ' store room trap teleport number
                    Teleported = True ' set teleporting flag
                    CALL Enter.Room ' routine to move player to a room
                END IF ' end check destination room
            END IF ' end check file bounds
        CASE 3 ' type 3
            Outpt = Outpt + "Deadly spears!" ' make trap message
            CALL IO.O ' send trap message
            Outpt = "You are hit for" ' make hits message
            ' get object trap hits on player
            Room.Hits# = CDBL(INT(ObjectRecord.Teleport))
            IF Room.Hits# > False THEN ' check hits
                CALL Hit.Player(Room.Hits#) ' routine to hit player
            END IF ' end check hits
    END SELECT ' end selection of trap type
END IF ' end check random chance
END SUB ' end routine for room traps

REM * routine to hide player.

SUB Hide.User
IF Hidden.Player THEN ' check player already hidden
    Outpt = "You hide in the shadows!" ' make hide message
    CALL IO.O ' send hide message
    EXIT SUB ' exit routine
END IF ' end check player hidden
Hide.Flag = False ' set hide flag
IF Number.Monsters = False THEN ' check number of monsters in room
    Hide.Flag = True ' set hide flag
ELSE ' check monsters
    IF RND > .66 THEN ' random chance
        Hide.Flag = True ' set hide flag
    END IF ' end check random chance
END IF ' end check monsters in room
IF Hide.Flag THEN ' check hide flag
    Hidden.Player = True ' set player hide flag
    Outpt = "You hide in the shadows!" ' make hide message
ELSE ' check hide flag
    Outpt = "Didn't work!" ' make hide message
END IF ' end check hide flag
CALL IO.O ' send hide message
END SUB ' end routine to hide player

REM * routine for player to use a vehicle.

SUB Enter.Vehicle
CALL Check.Room.Treasure ' routine finds vehicle name in room
IF Index.Number = False THEN ' check room vehicle found
    Outpt = "That's not a vehicle!" ' make error message
    CALL IO.O ' send error message
    EXIT SUB ' exit routine
END IF ' end check vehicle found
IF TreasureRecord.Vehicle = False THEN ' check item is vehicle
    Outpt = "That's not a vehicle!" ' make error message
    CALL IO.O ' send error message
    EXIT SUB ' exit routine
END IF ' end check vehicle
Outpts = TreasureRecord.ShortName ' store treasure name
Outpts = RTRIM$(Outpts) ' trim name
Outpts = LCASE$(Outpts) ' lowercase name
IF Charges.Number = False THEN ' check vehicle hits
    Outpt = "The " + Outpts + " is damaged!" ' make error message
    CALL IO.O ' send error message
    EXIT SUB ' exit routine
END IF ' end chekc vehicle hits
Vehicle1 = Array.Number ' store vehicle variable
Vehicle2 = Charges.Number ' store vehicle variable
Vehicle3 = Index.Number ' store vehicle variable
Vehicle4 = TreasureRecord.VehicleType ' store vehicle variable
Outpt = "You enter the " + Outpts + "." ' make vehicle message
CALL IO.O ' send vehicle message
END SUB ' end routine to use vehicle

REM * routine to move player and vehicle in a direction or through a portal.

SUB Ride.Vehicle
IF Vehicle3 = False THEN ' check player using a vehicle
    Outpt = "You're not riding a vehicle!" ' make error message
    CALL IO.O ' send error message
    EXIT SUB ' exit routine
END IF ' end check player vehicle
User.Command = Parsed.Command1 ' store direction parameter
CALL Get.Direction(Direction.Number) ' routine verifies direction
IF Direction.Number THEN ' compare direction number
    Next.Room = GetRoomRecordDirect(Direction.Number) ' get room direction
    IF Next.Room = False THEN ' check room direction number
        Outpt = "You can't travel in that direction!" ' make error message
        CALL IO.O ' send error message
        EXIT SUB ' exit routine
    END IF ' end check room direction number
    CALL Vehicle.Type ' routine compares vehicle to room type
    IF Next.Room = False THEN ' check room type flag
        Outpt = "You can't travel in that direction!" ' make error message
        CALL IO.O ' send error message
        EXIT SUB ' exit routine
    END IF ' end compare vehicle to room type
    Outpts = Direction(Direction.Number) ' store dirction name
    Outpts = RTRIM$(Outpts) ' trim name
    Outpts = LCASE$(Outpts) ' lowercase name
    Outpt = "You ride " + Outpts + "!" ' make vehicle message
    CALL IO.O ' send vehicle message
    CALL Enter.Room ' routine moves player to room
    EXIT SUB ' exit routine
END IF ' end compare direction number
CALL Check.Room.Objects ' routine searches room for portal name
IF Index.Number = False THEN ' check room portal number
    Outpt = "You can't travel in that direction!" ' make error message
    CALL IO.O ' send error message
    EXIT SUB ' exit routine
END IF ' end check room portal
IF ObjectRecord.RoomLink = False THEN ' check portal goes to room
    Outpt = "You can't travel there!" ' make error message
    CALL IO.O ' send error message
    EXIT SUB ' exit routine
END IF ' end check portal to room number
IF ObjectRecord.JailTrap THEN ' check room portal is a jail trap
    Outpt = "Trapped portal!" ' make error message
    CALL IO.O ' send error message
    EXIT SUB ' exit routine
END IF ' end check room portal type
IF ObjectRecord.Closed THEN ' check roomportal is locked
    IF Pass.Door = False THEN ' check pass door spell in effect
        Outpt = "You can't, it's closed!" ' make entry error message
        CALL IO.O ' send entry error message
        EXIT SUB ' exit routine
    END IF ' end check pass door spell
END IF ' end check room portal locked
IF ObjectRecord.Relocks THEN ' check room portal relocks
    ObjectRecord.DoorLock = 2 ' reset room portal lock
    ObjectRecord.Closed = True ' reset room portal lock
    CALL Share.Record(ObjectFile, Index.Number) ' write object record
END IF ' end check room portal relock
Outpt = ObjectRecord.ShortDesc ' store room entry display description
IF Outpt <> STRING$(80, 0) THEN ' check description to nulls
    Outpt = RTRIM$(Outpt) ' trim description
    IF Outpt <> Nul THEN ' compare length of description
        CALL IO.O ' send room entry description message
    END IF ' end compare description length
END IF ' end check description
Pass.Door = False ' reset pass door spell
Number.Monsters = False ' reset number of monsters in room
' store room number of object portal destination
Next.Room = ObjectRecord.RoomLink ' store
CALL Vehicle.Type ' routine to verify vehicle to room type
IF Next.Room = False THEN ' check verified room number
    Outpt = "You can't travel in that direction!" ' make error message
    CALL IO.O ' send error message
    EXIT SUB ' exit routine
END IF ' end check vehicle to room type
Outpts = ObjectRecord.ShortName ' store object name
Outpts = RTRIM$(Outpts) ' trim name
Outpts = LCASE$(Outpts) ' lowercase name
Outpt = "You ride to the " + Outpts + "!" ' make vehicle message
CALL IO.O ' send message
CALL Enter.Room ' routine moves player to room
END SUB ' end routine to move player and vehicle

REM * routine remove player from vehicle.

SUB Exit.Vehicle
CALL Check.Room.Treasure ' routine finds vehicle name in room
IF Array.Number = Vehicle1 THEN ' compare treasure number to vehicle number
    Outpts = TreasureRecord.ShortName ' store treasure name
    Outpts = RTRIM$(Outpts) ' trim name
    Outpts = LCASE$(Outpts) ' lowercase name
    Vehicle1 = False ' reset vehicle variable
    Vehicle2 = False ' reset vehicle variable
    Vehicle3 = False ' reset vehicle variable
    Outpt = "You exit the " + Outpts + "!" ' make vehicle message
ELSE ' compare vehicle numbers
    Outpt = "You can't exit that!" ' make error message
END IF ' end compare vehicle
CALL IO.O ' send message
END SUB ' end routine to exit vehicle

REM * routine verifies vehicle type can enter room Type
REM * output variables:
REM *   Next.Room - false for invalid vehicle to room Type

SUB Vehicle.Type
IF Next.Room <= False OR Next.Room > LOF(RoomFile) / LEN(RoomRecord) THEN ' file bounds
    Next.Room = False ' return next room number
    EXIT SUB ' exit routine
END IF ' end check file bounds
CALL Read.Room.Record(Next.Room) ' get destination room
IF Vehicle4 <> 3 THEN ' check all terrain vehicle
    Action.Number = RoomRecord.Action ' store room action number
    IF Action.Number > False AND Action.Number <= LOF(ActionFile) / LEN(ActionRecord) THEN
        CALL Read.Record(ActionFile, Action.Number) ' read action record
        IF ActionRecord.Attribute2 <> Vehicle4 THEN ' check vehicle terrain type
            Next.Room = False ' reset next room number
        END IF ' end check vehicle terrain type
    END IF ' end check action number range
END IF ' end check all terrain vehicle
CALL Read.Room.Record(Room) ' restore room record
END SUB ' end routine to compare vehicle to room type

REM * routine determines if player can train for next level.

SUB Train
IF UserRecord.Level <= False THEN ' verify player level
    CALL Train.Stats ' train player
    EXIT SUB ' exit routine
END IF ' end verify player level
' calculate experience needed to reach next level
CALL Experience(Exp.Required#)
IF UserRecord.Experience < Exp.Required# THEN ' compare player experience
    Outpt = "You don't have enough experience to train!" ' train error message
    CALL IO.O ' send train error message
    EXIT SUB ' exit routine
END IF ' end compare player experience
CALL Gold(Gold.Required#) ' routine calculates gold needed for level
IF UserRecord.Gold < Gold.Required# THEN ' compare to player gold
    Outpt = "You don't have enough Gold to train!" ' make train error message
    CALL IO.O ' send train error message
    EXIT SUB ' exit routine
END IF ' end compare gold
CALL Is.Training.Room(In.Room) ' check training room
IF In.Room = False THEN ' check flag
    Outpt = "You are not in the training room for your class!" ' message
    CALL IO.O ' output message
ELSE ' check room number
    CALL Train.Stats ' routine to train for next level
END IF ' end check room number
END SUB ' end train routine

REM * routine determines player is in correct training room.
REM * output variables:
REM *   In.Room - true for in room

SUB Is.Training.Room (In.Room)
In.Room = False ' reset flag
Player.Class = UserRecord.ClassType ' store class number
' check for no specific training room
IF GetUserRecordTrainRoom(Player.Class) = SFalse THEN
    In.Room = True ' reset flag
    EXIT SUB ' exit routine
END IF ' end check room number
' compare room numbers
IF GetUserRecordTrainRoom(Player.Class) = Room THEN
    In.Room = True ' reset flag
END IF ' end compare rooms
END SUB

REM * routine teleports player to room.
REM * input variables:
REM *   Parsed.Command1 - parameter containing room number.

SUB Teleport.User
Next.Room = INT(VAL(Parsed.Command1)) ' convert parameter to room number
Graphics.Off = True ' reset color
Outpt = "A Dark Cloud Passes Overhead..." ' make ghod message
CALL IO.O ' send ghod message
Outpt = "  A Bolt Of Lightning Strikes..." ' make ghod message
CALL IO.O ' send ghod message
Outpt = "The Cloud Disappears..." ' make ghod message
CALL IO.O ' send ghod message
Graphics.Off = False ' reset color
Teleported = True ' set teleporting flag
CALL Enter.Room ' routine moves player to room
END SUB ' end routine to teleport player

REM * routine moves player to room number in an object.
REM * input variables:
REM *   Parsed.Command1 - contains the object name.

SUB Enter.Object
User.Command = Parsed.Command1 ' store command parameter
CALL Get.Direction(Direction.Number) ' compare name to direction to go to
IF Direction.Number THEN ' check direction flag
    Entry.Command = Last.Command.Number ' store room entry command
    CALL Verify.Room(Direction.Number) ' routine verifies room number
    IF New.Room THEN ' check room flag
        CALL Enter.Room ' routine moves player
    END IF ' end check room flag
    EXIT SUB ' exit routine
END IF ' end check direction
CALL Check.Room.Treasure ' compare treasure name
IF Index.Number THEN ' check treasure flag
    IF TreasureRecord.Vehicle THEN ' object to move to is vehicle
        CALL Enter.Vehicle ' routine to enter vehicle
        EXIT SUB ' exit routine
    END IF ' end check object name
END IF ' end check treasure flag
CALL Check.Room.Objects ' compare room object
IF Index.Number = False THEN ' check object flag
    CALL Num ' count next object
    CALL Check.Inventory.Objects ' compare user object
    IF Index.Number = False THEN ' check object flag
        Outpt = "You can't go there!" ' make error message
        CALL IO.O ' send error message
        EXIT SUB ' exit routine
    END IF ' end check user object
END IF ' end check room object
IF ObjectRecord.RoomLink = False THEN ' check object portal room number
    Outpt = "You can't go there!" ' make error message
    CALL IO.O ' send error message
    EXIT SUB ' exit routine
END IF ' end check portal room number
CALL Restrict(12, Restricted) ' routine checks enter command restricted
IF Restricted THEN ' compare restrict flag
    Outpt = "Your level does not permit entrance to that room!" ' message
    CALL IO.O ' send error message
    EXIT SUB ' exit routine
END IF ' end compare restricted room
IF ObjectRecord.JailTrap THEN ' check object is jailed
    Outpt = "Trapped portal!" ' make error message
    CALL IO.O ' send error message
    EXIT SUB ' exit routine
END IF ' end check jailed object
IF ObjectRecord.Closed THEN ' check object lock
    IF Pass.Door = False THEN ' check pass door spell in effect
        Outpt = "You can't, it's closed!" ' make error message
        CALL IO.O ' send error message
        EXIT SUB ' exit routine
    END IF ' end check pass door spell
END IF ' end check object lock
IF ObjectRecord.Relocks THEN ' check object relocks after entry
    ObjectRecord.DoorLock = 2 ' relock object
    ObjectRecord.Closed = True ' relock object
    CALL Share.Record(ObjectFile, Index.Number) ' write object record
END IF ' end check relocking object
Outpt = ObjectRecord.ShortDesc ' store entry description
IF Outpt <> STRING$(80, 0) THEN ' compare description to nulls
    Outpt = RTRIM$(Outpt) ' trim description
    IF Outpt <> Nul THEN ' compare length of description
        CALL IO.O ' send entry description message
    END IF ' end compare description length
END IF ' end compare description
IF ObjectRecord.Trap THEN ' verify object has trap
    CALL Traps ' routine to activate object trap
    IF New.Room THEN ' check teleporting trap
        EXIT SUB ' exit routine
    END IF ' end check teleporting trap
END IF ' end verify object trap
Pass.Door = False ' reset pass door spell
Number.Monsters = False ' set monsters in room to zero
Next.Room = ObjectRecord.RoomLink ' store object portal room number
CALL Enter.Room ' routine to move player to new room
END SUB ' end routine to move player through an object to room

REM * routine moves player to new room number.
REM * input variables:
REM *   Next.Room - contains new room number to move player to.

SUB Enter.Room
New.Room = True ' set room entry flag
CALL Clean.Room ' routine to remove treasure from old room
IF Next.Room > False AND Next.Room <= LOF(RoomFile) / LEN(RoomRecord) THEN ' file bounds
    SWAP Room, Next.Room ' store new room, saving old room number
END IF ' end check room file bounds
CALL Status.Line(False) ' routine updates status line
Monster.Rate1 = False ' reset room monster encounter rate
CALL Read.Room.Record(Room) ' get the new room record
IF Vehicle1 > False THEN ' verify vehicle variable used
    Move.Vehicle = False ' move vehicle flag
    Entry.Command = Last.Command.Number ' store command number to enter room
    ' vehicle entered room
    IF Entry.Command = RideVehicle OR Entry.Command = DriveVehicle THEN
        FOR Treasure.Number = 1 TO 20 ' loop through room treasure
            ' check empty treasure
            IF GetRoomRecordTreasure(Treasure.Number) = False THEN
                ' store vehicle inventory
                CALL PutRoomRecordTreCharges(Treasure.Number, Vehicle2)
                ' store vehicle inventory
                CALL PutRoomRecordTreasure(Treasure.Number, Vehicle3)
                CALL Share.Room.Record(Room) ' write new room record
                CALL Read.Room.Record(Next.Room) ' get previous room
                CALL PutRoomRecordTreasure(Vehicle1, False) ' reset vehicle inventory
                CALL PutRoomRecordTreCharges(Vehicle1, False) ' reset vehicle inventory
                CALL Share.Room.Record(Next.Room) ' write room record
                CALL Read.Room.Record(Room) ' get current room record
                Move.Vehicle = True ' set move vehicle flag
                Vehicle1 = Treasure.Number ' store new vehicle treasure number
                EXIT FOR ' exit loop through treasure in room
            END IF ' end check for empty treasure in room
        NEXT ' end loop through room treasure inventory
    END IF ' end check command used to enter room
    IF Move.Vehicle = False THEN ' verify vehicle moves to new room
        CALL Read.Record(TreasureFile, Vehicle3) ' get treasure record
        Outpts = TreasureRecord.ShortName ' store treasure name
        Outpts = RTRIM$(Outpts) ' trim name
        Outpts = LCASE$(Outpts) ' lowercase name
        Outpt = "You exit the " + Outpts + "." ' make vehicle message
        CALL IO.O ' send vehicle message
        Vehicle1 = False ' reset vehicle varible
        Vehicle2 = False ' reset vehicle varible
        Vehicle3 = False ' reset vehicle varible
        Vehicle4 = False ' reset vehicle varible
    END IF ' end verify vehicle moves
END IF ' end verify vehicle used
FOR Array.Index = 1 TO Number.Monsters ' loop through room monsters
    ' compare permanent monster
    IF MonsterArray(Array.Index).Permanent = True THEN
        ' store permanent monster file index
        Monster.Number = MonsterIndex(Array.Index)
        CALL Read.Record(MonsterFile, Monster.Number) ' get monster file record
        ' store permanent level
        MonsterRecord.Level = MonsterArray(Array.Index).Level
        MonsterRecord.Hits = MonsterArray(Array.Index).Hits ' stor permanent hits
        CALL Share.Record(MonsterFile, Monster.Number) ' write monster record
    END IF ' end compare permanent monster
NEXT ' end loop through room monsters
IF Room = 1 THEN ' check safe room
    Monster.Follow = False ' monsters followed flag
ELSE ' end check safe room
    IF UserRecord.ClassType < Lady THEN ' check class number
        Monster.Follow = True ' monsters followed flag
    ELSE ' check class number
        Monster.Follow = False ' monsters followed flag
    END IF ' end check class number
END IF ' end check safe room
Teleported.Flag = Teleported ' store teleporting flag
Teleported = False ' reset teleporting flag
IF Monster.Follow THEN ' compare number of monsters followed
    IF Teleported.Flag THEN ' compare teleporting flag
        Outpts = " teleports with you!" ' make follow message
    ELSE ' check teleporting flag
        Outpts = " follows you!" ' make follow message
    END IF ' end check teleporting flag
    Monsters.Followed = False ' number of monsters which followed counter
    FOR Array.Index = 1 TO Number.Monsters ' loop through all monsters in room
        Monster.Followed = False ' set followed flag
        IF MonsterArray(Array.Index).Follow THEN ' check monster follows
            IF Teleported.Flag THEN ' check player teleported
                ' random chance monster teleports with player
                IF RND < (MonsterArray(Array.Index).Teleport / 100) THEN
                    Monster.Followed = True ' set followed flag
                END IF ' end random chance
            ELSE ' check player teleported
                ' random chance monster follows player
                IF RND < (MonsterArray(Array.Index).FollowPercent / 100) THEN
                    Monster.Followed = True ' set followed flag
                END IF ' end random chance
            END IF ' end check player teleported
            IF Monster.Followed THEN ' verify followed flag
                ' permanent monster
                IF MonsterArray(Array.Index).Permanent = False THEN
                    ' increment number of monsters
                    Monsters.Followed = Monsters.Followed + 1
                    ' store monster
                    MonsterArray(Monsters.Followed) = MonsterArray(Array.Index)
                    ' store monster
                    MonsterIndex(Monsters.Followed) = MonsterIndex(Array.Index)
                    ' store monster name
                    Inpt = MonsterArray(Monsters.Followed).MonsterName
                    Inpt = RTRIM$(Inpt) ' trim name
                    Inpt = LCASE$(Inpt) ' lowercase name
                    Outpt = "The " + Inpt + Outpts ' make followed message
                    CALL IO.O ' send message
                END IF ' end check permanent monster
            END IF ' end verify followed flag
        END IF ' end check monster follows
    NEXT ' end loop through monsters in room
END IF ' end compare number of monsters following
Monster.Rate1 = False ' reset room monster encounter rate
Number.Monsters = Monsters.Followed ' store number of monsters followed
Room.Rate = False ' reset room rate counter
Rust.Rate = False ' reset room rate counter
Steal.Rate = False ' reset room rate counter
Teleported = False ' reset teleporting flag
CALL Check.Next.Room ' routine to get new room record
CALL Encounter.Permanent ' routine to get permanent monsters
CALL Display.Room ' routine displays room description
Action.Number = RoomRecord.Action ' store room action number
' check file bounds
IF Action.Number > False AND Action.Number <= LOF(ActionFile) / LEN(ActionRecord) THEN
    CALL Read.Record(ActionFile, Action.Number) ' read action record
    IF ActionRecord.SpellTrigger = False THEN ' check room spell action
        IF ActionRecord.MonsterTrigger = False THEN ' check room monster action
            ' check room monster talk action
            IF ActionRecord.MonsterTalk = False THEN
                Action1$ = "As you enter the room," ' make action message
                Action2$ = "You are hit for" ' make action message
                CALL Actions(Action1$, Action2$) ' routine for room actions
            END IF ' end check room action
        END IF ' end check room action
    END IF ' end check room action
END IF ' end check file bounds
END SUB ' end routine to move player to new room

REM * routine toggles invisible mode.

SUB Toggle.Invisible
UserRecord.Invisible = NOT UserRecord.Invisible ' negate player invisible mode
IF UserRecord.Invisible THEN ' check invisible
    Outpt = "You are invisible!" ' make message
ELSE ' check invisible
    Outpt = "You are no longer invisible!" ' make message
END IF ' end check invisible
CALL IO.O ' send message
END SUB ' end routine to toggle invisible mode

REM * routine toggles linefeed mode.

SUB Toggle.Linefeeds
User.LineFeeds = NOT User.LineFeeds ' negate player linefeed mode
IF User.LineFeeds THEN ' check linefeeds
    Outpt = "Linefeeds toggled off." ' make message
ELSE ' check linefeeds
    Outpt = "Linefeeds toggled on." ' make message
END IF ' end check linefeeds
CALL IO.O ' send message
END SUB ' end routine to toggle linefeed mode

REM * routine toggles echo mode.

SUB Toggle.Echo
User.Echo = NOT User.Echo ' negate player echo mode
IF User.Echo THEN ' check echo
    Outpt = "Echo toggled off." ' make message
ELSE ' check echo
    Outpt = "Echo toggled on." ' make message
END IF ' end check echo
CALL IO.O ' send message
END SUB ' end routine to toggle echo mode

REM * routine toggles word wrap mode.

SUB Toggle.Wordwrap
User.Wordwrap = NOT User.Wordwrap ' negate player word wrap mode
IF User.Wordwrap THEN ' check word wrap
    Outpt = "Word wrap toggled off." ' make message
ELSE ' check word wrap
    Outpt = "Word wrap toggled on." ' make message
END IF ' end check word wrap
CALL IO.O ' send message
END SUB ' end routine to toggle word wrap mode

REM * routine to toggle player Ansi mode.

SUB Toggle.ANSI
Color.Graphics = NOT Color.Graphics ' negate ansi color flag
IF Color.Graphics THEN ' check ansi toggle
    Outpt = "ANSI codes enabled." ' make message
ELSE ' check ansi
    Outpt = "ANSI codes disabled." ' make message
END IF ' end check ansi toggle
CALL IO.O ' send message
END SUB ' end routine to toggle ansi

REM * routine to toggle brief mode.

SUB Brief.Mode
IF Normal.User THEN ' verify non DM
    IF UserRecord.Level <= 1 THEN ' check player level
        Outpt = "Brief mode not allowed until level two!" ' make error message
        CALL IO.O ' send message
        EXIT SUB ' exit routine
    END IF ' end check player level
END IF ' end check normal player
UserRecord.Brief = NOT UserRecord.Brief ' negate brief mode
IF UserRecord.Brief THEN ' check brief mode
    Action.Prompt = "Next?" ' store new command prompt
    Outpt = "Brief mode on." ' make message
ELSE ' check brief mode
    Action.Prompt = "Command? " ' store new command prompt
    Outpt = "Brief mode off." ' make message
END IF ' end check brief mode
CALL IO.O ' send message
END SUB ' end routine to toggle brief mode

REM * routine to change pagelength.

SUB Change.Pagelength
Outpt = "Enter pagelength(1-50)? " ' make input prompt
No.Input.Out = "24" ' default input
CALL IO.I ' get user input
Page.Length = INT(VAL(Inpt)) ' convert input to integer
IF Page.Length >= 1 AND Page.Length <= 50 THEN ' check pagelength bounds
    User.PageLength = Page.Length ' store new pagelength
    Outpt = "Pagelength now" + STR$(Page.Length) + " lines." ' make output message
ELSE ' check bounds
    Outpt = "Pagelength not changed." ' make output message
END IF ' end check bounds
CALL IO.O ' send output
END SUB ' end routine to change pagelength

REM * routine to change linelength.

SUB Change.Linelength
Outpt = "Enter linelength(1-132)? " ' make input prompt
No.Input.Out = "80" ' default input
CALL IO.I ' get user input
Line.Length = INT(VAL(Inpt)) ' convert input to integer
IF Line.Length >= 1 AND Line.Length <= 132 THEN ' check linelength bounds
    User.LineLength = Line.Length ' store new linelength
    Outpt = "Linelength now" + STR$(Line.Length) + " characters." ' make output message
ELSE ' check bounds
    Outpt = "Linelength not changed." ' make output message
END IF ' end check bounds
CALL IO.O ' send output
END SUB ' end routine to change linelength

REM * routine to display preferences.

SUB Display.Prefs
Graphics.Off = True ' reset color
IF User.LineFeeds THEN ' check linefeeds
    Outpt = "Linefeeds toggled off." ' make message
ELSE ' check linefeeds
    Outpt = "Linefeeds toggled on." ' make message
END IF ' end check linefeeds
CALL IO.O ' send message
IF User.Echo THEN ' check echo
    Outpt = "Echo toggled off." ' make message
ELSE ' check echo
    Outpt = "Echo toggled on." ' make message
END IF ' end check echo
CALL IO.O ' send message
IF User.Wordwrap THEN ' check word wrap
    Outpt = "Word wrap toggled off." ' make message
ELSE ' check word wrap
    Outpt = "Word wrap toggled on." ' make message
END IF ' end check word wrap
CALL IO.O ' send message
IF Color.Graphics THEN ' check ansi toggle
    Outpt = "ANSI codes enabled." ' make message
ELSE ' check ansi
    Outpt = "ANSI codes disabled." ' make message
END IF ' end check ansi toggle
CALL IO.O ' send message
IF UserRecord.Brief THEN ' check brief mode
    Outpt = "Brief mode on." ' make message
ELSE ' check brief mode
    Outpt = "Brief mode off." ' make message
END IF ' end check brief mode
CALL IO.O ' send message
Outpt = "Pagelength now" + STR$(User.PageLength) + " lines." ' make message
CALL IO.O ' send output
Outpt = "Linelength now" + STR$(User.LineLength) + " characters." ' make message
CALL IO.O ' send output
SELECT CASE UserRecord.Sort ' check sorting preference
    CASE -1 ' check value
        Outpt = "Inventory charges sorting on." ' make display message
    CASE 0 ' check value
        Outpt = "Inventory sorting off." ' make display message
    CASE 1 ' check value
        Outpt = "Inventory plus sorting on." ' make display message
    CASE ELSE ' check sorting
        Outpt = "Inventory sorting off." ' make display message
END SELECT ' end check sorting
CALL IO.O ' send display message
' check player locked mailbox
IF UserRecord.Flags AND Locked.User THEN
    Outpt = "Mailbox locked." ' make locked message
ELSE ' check player locked
    Outpt = "Mailbox unlocked." ' make locked message
END IF ' end check player locked flag
CALL IO.O ' send message
Graphics.Off = False ' reset color
END SUB ' end display preferences routine

REM * routine to have the blacksmith repair some item of treasure.

SUB Weapons.Shop
CALL Check.Inventory.Treasure ' routine to find treasure name
IF Index.Number = False THEN ' check treasure index
    Outpt = "The Blacksmith says: You can't repair that!" ' make error message
    CALL IO.O ' send error message
    EXIT SUB ' exit routine
END IF ' end check treasure index
IF TreasureRecord.Spell THEN ' compare treasure is magic
    Outpt = "The Blacksmith says: Can't fix that here!" ' make error message
    CALL IO.O ' send error message
    EXIT SUB ' exit routine
END IF ' end compare magical treasure
IF TreasureRecord.Type = False THEN ' check treasure is weapon, shield, armor
    IF TreasureRecord.Plus = False THEN ' check weapon plus
        Outpt = "The Blacksmith says: Can't fix that here!" ' make error message
        CALL IO.O ' send error message
        EXIT SUB ' exit routine
    END IF ' end check weapon plus
END IF ' end check treasuer type
IF GetUserRecordCharges(Array.Number) <> False THEN ' compare treasure charges
    Outpt = "The Blacksmith says: That isn't broken!" ' make error message
    CALL IO.O ' send error message
    EXIT SUB ' exit routine
END IF ' end compare treasure charges
Item.Cost# = INT(TreasureRecord.Gold * .5) ' calculate price to repair item
IF Item.Cost# > UserRecord.Gold THEN ' compare price to player gold
    Outpt = "The Blacksmith says: You don't have enough gold!" ' make message
    CALL IO.O ' send message
    EXIT SUB ' exit routine
END IF ' end compare price to gold
' make input prompt
Outpt = "The Blacksmith asks: How about" + STR$(Item.Cost#) + " gold(y/n)? "
No.Input.Out = "Y" ' default input
CALL IO.I ' get input
IF Yes THEN ' compare response
    UserRecord.Gold = UserRecord.Gold - Item.Cost# ' decrement player gold
    ' store repaired charges
    CALL PutUserRecordCharges(Array.Number, TreasureRecord.Charges)
    Outpt = "The Blacksmith says: There, it's repaired!" ' make message
ELSE ' compare yresponse
    Outpt = "The Blacksmith says: It ain't repaired!" ' make message
END IF ' end compare response
CALL IO.O ' send message
END SUB ' end routine to repair item

REM * routine to repair an item of magical treasure.

SUB Alchemist
CALL Check.Inventory.Treasure ' routine to find treasure name
IF Index.Number = False THEN ' check treasure index found
    Outpt = "The Alchemist says: That can't be recharged here!" ' make message
    CALL IO.O ' send error message
    EXIT SUB ' exit routine
END IF ' end check treasure index
Spell.Number = TreasureRecord.Spell ' store treasure spell number
' file bounds
IF Spell.Number <= False OR Spell.Number > LOF(SpellFile) / LEN(SpellRecord) THEN
    Outpt = "The Alchemist says: You can't recharge that here!" ' make message
    CALL IO.O ' send error message
    EXIT SUB ' exit routine
END IF ' end check file bounds
CALL Read.Record(SpellFile, Spell.Number) ' get spell record of magic item
IF SpellRecord.SpellType = Wish THEN ' compare to wish item
    Outpt = "The Alchemist says: I won't recharge that item!" ' make message
    CALL IO.O ' send error message
    EXIT SUB ' exit routine
END IF ' end compare wish item
IF GetUserRecordCharges(Array.Number) <> False THEN ' check item charges
    Outpt = "The Alchemist says: That's not discharged!" ' make message
    CALL IO.O ' send error message
    EXIT SUB ' exit routine
END IF ' end check charges remaining
Item.Cost# = INT(TreasureRecord.Gold * .5) ' calculate price to repair
IF Item.Cost# > UserRecord.Gold THEN ' compare price to player gold
    Outpt = "The Alchemist says: You don't have enough gold!" ' make message
    CALL IO.O ' send error message
    EXIT SUB ' exit routine
END IF ' end compare price to gold
' make input prompt
Outpt = "The Alchemist says: How about" + STR$(Item.Cost#) + " gold(y/n)? "
No.Input.Out = "Y" ' default input
CALL IO.I ' get input
IF Yes THEN ' compare response
    UserRecord.Gold = UserRecord.Gold - Item.Cost# ' decrement player gold
    ' store item charges
    CALL PutUserRecordCharges(Array.Number, TreasureRecord.Charges)
    Outpt = "The Alchemist chants an invocation!" ' make message
ELSE ' compare response
    Outpt = "The Alchemist says: Didn't repair it!" ' make message
END IF ' end compare response
CALL IO.O ' send message
END SUB ' end routine to repair magic item

REM * routine to sell treasure item from list.
REM * input variables:
REM *   Parsed.Command1 - number of item to purchase.

SUB Weapons.Shoppe
Treasure.Number = INT(VAL(Parsed.Command1)) ' convert parameter to integer
IF Treasure.Number <= False OR Treasure.Number > 20 THEN ' compare integer bounds
    Outpt = "The Blacksmith says: You can't buy that!" ' make error message
    CALL IO.O ' send error message
    EXIT SUB ' exit routine
END IF ' end compare range
IF Treasure.Number > LOF(TreasureFile) / LEN(TreasureRecord) THEN ' check bounds
    Outpt = "The Blacksmith says: You can't buy that!" ' make error message
    CALL IO.O ' send error message
    EXIT SUB ' exit routine
END IF ' end compare file bounds
CALL Read.Record(TreasureFile, Treasure.Number) ' get treasure record
IF UserRecord.Gold - TreasureRecord.Gold < False THEN ' compute price
    Outpt = "The Blacksmith says: You don't have enough gold!" ' make message
    CALL IO.O ' send error message
    EXIT SUB ' exit routine
END IF ' end compare price to player gold
IF Weight + TreasureRecord.Weight > GetUserRecordStats(1) * 10 THEN ' compute weight
    Outpt = "The Blacksmith says: You can't carry any more!" ' make message
    CALL IO.O ' send error message
    EXIT SUB ' exit routine
END IF ' end compare weight of new item
CALL TreasureCharges(Charges.Amount) ' routine to get treasure charges
' routine to add item to player inventory
CALL Add.Inventory(Treasure.Number, Charges.Amount, Item.Added)
IF Item.Added THEN ' check return variable for added inventory
    UserRecord.Gold = UserRecord.Gold - TreasureRecord.Gold ' decrement gold
    Outpt = "The Blacksmith says: There, sold!" ' make message
ELSE ' check inventory added
    Outpt = "The Blacksmith says: You can't carry any more!" ' make message
END IF ' end check inventory added flag
CALL IO.O ' send output message
END SUB ' end routine to purchase item for sale

REM * routine displays a sorted list of the top ten players, writes the
REM * top ten ranking bulletin file.

SUB Top.Ten
Max.Users = LOF(UserFile) / LEN(UserRecord) ' store length of user file
REDIM Temp.Array1(1 TO Max.Users) AS INTEGER
REDIM Temp.ArrayZ(1 TO Max.Users) AS DOUBLE ' dimension working arrays
CLOSE #TempFile ' close work file
OPEN "ranklist.dat" FOR OUTPUT AS #TempFile ' open to work file
Outpt = "The Adventure Door v" + Version$ + " Top Ten Player Rankings For " + clock$ + "."
PRINT #TempFile, Outpt ' write to file
Outpt = Nul ' make empty string
PRINT #TempFile, Outpt ' write to file
Player.Count = False ' reset player counter
FOR User.Number = 1 TO Max.Users ' loop through user file
    CALL Read.Record(UserFile2, User.Number) ' get next user file record
    Outpt = UserRecord2.CodeName ' store player codename
    CALL Decrypt(Outpt) ' decrypt codename
    IF LEFT$(Outpt, 9) <> Deleted$ THEN ' compare deleted user record
        IF UserRecord2.Level > False THEN ' check user level
            IF (UserRecord2.Flags AND Locked.User) = False THEN ' check locked user
                Score# = UserRecord2.MonstersKilled * UserRecord2.Level ' compute score
                IF Score# > False THEN ' compare score
                    ' increment high score player counter
                    Player.Count = Player.Count + 1
                    Temp.Array1(Player.Count) = User.Number ' store record index
                    Temp.ArrayZ(Player.Count) = INT(Score#) ' store score
                END IF ' end compare score
            END IF ' end check locked user record
        END IF ' end check user level
    END IF ' end compare deleted user
NEXT ' end loop through user file
' bubble sort
FOR Sort1 = 1 TO Player.Count ' loop through all items to sort
    FOR Sort2 = Sort1 + 1 TO Player.Count ' loop through remaining items
        IF Temp.ArrayZ(Sort1) < Temp.ArrayZ(Sort2) THEN ' compare scores
            SWAP Temp.Array1(Sort1), Temp.Array1(Sort2) ' swap lower array
            SWAP Temp.ArrayZ(Sort1), Temp.ArrayZ(Sort2) ' swap lower array
        END IF ' end compare scores
    NEXT ' end loop through array
NEXT ' end loop through array
IF Player.Count > 10 THEN ' check maximum number of users
    Player.Count = 10 ' reset to top ten
END IF ' end check maximum scoring players
' make header
Outpt = "Username                       Level Classname            Ranking"
PRINT #TempFile, Outpt ' write to file
Outpt = STRING$(65, "-") ' make header line
PRINT #TempFile, Outpt ' write to file
FOR Array.Number = 1 TO Player.Count ' loop through high scoring players
    User.Number = Temp.Array1(Array.Number) ' get user file record number
    CALL Read.Record(UserFile2, User.Number) ' get user file record
    Outpt = UserRecord2.CodeName ' store player codename
    CALL Decrypt(Outpt) ' decrypt codename
    Outpt = LCASE$(Outpt) ' lowercase codename
    MID$(Outpt, 1, 1) = UCASE$(MID$(Outpt, 1, 1)) ' uppercase first word
    Outpt = Outpt + STR$(UserRecord2.Level) ' add player level
    Outpt = Outpt + SPACE$(7 - LEN(STR$(UserRecord2.Level))) ' pad blanks
    Inpt = UserRecord2.ClassName ' store player class name
    CALL Decrypt(Inpt) ' decrypt class name
    Outpt = Outpt + Inpt ' append class name
    Outpt = Outpt + STR$(Temp.ArrayZ(Array.Number)) ' add score
    PRINT #TempFile, Outpt ' write to file
NEXT ' end loop through top ten players
IF Player.Count = False THEN ' compare number of players
    Outpt = "No users have top scores." ' make score message
    PRINT #TempFile, Outpt ' write to file
END IF ' end compare number of players
CLOSE #TempFile ' close work file
User.Line.Length = User.LineLength
User.LineLength = 80
CALL Out.File("ranklist.dat")
User.LineLength = User.Line.Length
REDIM Temp.Array1(1) AS INTEGER
REDIM Temp.ArrayZ(1) AS DOUBLE ' remove temporary arrays
END SUB ' end routine to display and write top ten list

REM * routine deletes a user record.

SUB Delete.User
Outpt = Deleted$ ' store deleted string
CALL Valid(Outpt, 30) ' validate string
CALL Encrypt(Outpt, True) ' encrypt string
UserRecord.CodeName = Outpt ' store string in codename
Outpt = Deleted$ ' store deleted string
CALL Valid(Outpt, 20) ' validate string
CALL Encrypt(Outpt, False) ' encrypt string
UserRecord.PassWord = Outpt ' store string in password
Outpt = Deleted$ ' store deleted string
CALL Valid(Outpt, 20) ' validate string
CALL Encrypt(Outpt, True) ' encrypt string
UserRecord.ClassName = Outpt ' store string in class name
UserRecord.ClassType = False ' reset class number
UserRecord.Flags = False ' reset user flags
UserRecord.FromHour = False ' reset time restriction
UserRecord.FromMin = False ' reset time restriction
UserRecord.Level = False ' reset player level
UserRecord.MaxCalls = False ' reset maximum calls allowed
UserRecord.MonstersKilled = False ' reset score counter
UserRecord.ToHour = False ' reset time restriction
UserRecord.ToMin = False ' reset time restriction
FOR Array.Index = 1 TO 20 ' loop through inventory
    CALL PutUserRecordInv(Array.Index, False) ' reset inventory
    CALL PutUserRecordCharges(Array.Index, False) ' reset inventory
NEXT ' end loop through inventory
FOR Array.Index = 1 TO 20 ' loop through object inventory
    CALL PutUserRecordObject(Array.Index, False) ' reset inventory
    CALL PutUserRecordTrainRoom(Array.Index, False) ' reset empty array
NEXT ' end loop through inventory
CALL Clear.Container(0, True) ' clear container record
FOR Array.Index = 1 TO 3 ' loop through all containers
    CALL PutUserRecordContainer(Array.Index) ' store container record
NEXT ' end loop through containers
END SUB ' end routine to delete a user record

REM * Filename: dnds5.bas Version: v5.0a r1.0a
REM * This subprogram contains all container, bank, and some display
REM * and main routines.

REM * routine to search player and room containers for parameter name
REM * input variables:
REM *   Parsed.Command1 - name of container.
REM * output variables:
REM *   Index.Number - true if container name found.
REM *   TypeNumber - 0 for container in inventory, 1 for container in room.

SUB Examine.Container
TypeNumber = False ' store container flag
CALL Check.Inventory.Container ' search player inventory containers
IF Index.Number = False THEN ' check player container found
    CALL Num ' decrement counters
    TypeNumber = 1 ' store container flag
    CALL Check.Room.Container ' search room inventory containers
END IF ' end check player container found
END SUB ' end routine to search inventory for container name

REM * routine to remove an item from player inventory and add to a container.

SUB Drop.Into.Container
CALL Parse ' get first parameter
CALL Numeric ' parse parameter number from # sign
CALL Check.Inventory.Treasure ' search for inventory to drop
IF Index.Number = False THEN ' index to treasure file not found
    Outpt = "You can't drop that!" ' message
    CALL IO.O ' send output
    EXIT SUB ' return from routine
END IF ' end check treasure index
Item.Number = Array.Number ' store treasure index to array
Item.Charges = Charges.Number ' store treasure charges
Item.Index = Index.Number ' store treasure index to file
Parsed.Command1 = Parsed.Command2 ' get second parameter
CALL Numeric ' parse parameter number from # sign
CALL Examine.Container ' check player inventory and room for container
IF Index.Number = False THEN ' index to treasure file not found
    Outpt = "You can't drop that!" ' message
    CALL IO.O ' send output
    EXIT SUB ' return from routine
END IF ' end check index
IF ContainerRec.Locked > False THEN ' check container locked
    Outpt = "You can't, it's locked!" ' message
    CALL IO.O ' send output
    EXIT SUB ' return from routine
END IF ' end check container
IF ContainerRec.Closed > False THEN ' check container closed
    Outpt = "You can't, it's closed!" ' message
    CALL IO.O ' send output
    EXIT SUB ' return from routine
END IF ' end check container
Container.Number = Array.Number ' store index of inventory array of container
FOR Array.Counter = 1 TO 5 ' loop through container elements
    ' find empty container element
    IF GetContainerRecInventory(Array.Counter) = False THEN
        Container.Name$ = ContainerRec.ContainerName ' store container name
        Container.Name$ = RTRIM$(Container.Name$) ' strip trailing blanks
        IF Last.Command.Number = Hide.Command THEN ' last command was hide
            ' set container record hide flag
            CALL PutContainerRecInvisible(Array.Counter, True)
        ELSE ' not hide
            ' set container record hide
            CALL PutContainerRecInvisible(Array.Counter, False)
        END IF ' end check last command
        ' store container record index
        CALL PutContainerRecInventory(Array.Counter, Item.Index)
        ' store container record charges
        CALL PutContainerRecCharges(Array.Counter, Item.Charges)
        SELECT CASE TypeNumber ' select room or inventory
            CASE 1 ' drop item into container in room
                RoomRecord.Container = ContainerRec ' set room container record
                CALL Share.Room.Record(Room) ' put room record
            CASE 0 ' dropped item in container in player inventory
                ' update player container
                CALL PutUserRecordContainer(Container.Number)
        END SELECT ' end select container drop
        Outpts = TreasureRecord.TreasureName ' store name of treasure dropped
        Outpts = RTRIM$(Outpts) ' strip trailing blanks
        IF Last.Command.Number = Hide.Command THEN ' last command hide
            Inpt = " hide " ' set equal to hide
        ELSE ' not hide
            Inpt = " drop " ' set equal to drop
        END IF ' end check last command
        Outpt = "You" + Inpt + Outpts + " in " + Container.Name$ + "!" ' format message
        CALL IO.O ' send output
        CALL Discard.Inventory(Item.Number, True) ' remove item from inventory
        EXIT SUB ' return from routine
    END IF ' end check empty container record
NEXT ' loop through container records
Outpt = "You can't, it's full!" ' message
CALL IO.O ' send output
END SUB ' end drop into container routine

REM * routine to take an item of treasure from a container and add to
REM * the player inventory.

SUB Take.From.Container
CALL ParseX ' parse second parameter
CALL Numeric ' parse parameter number from # sign
CALL Examine.Container ' check container mnemonic
IF Index.Number = False THEN ' compare treasure index
    Outpt = "You can't get that!" ' make message
    CALL IO.O ' send message
    EXIT SUB ' exit routine
END IF ' end compare container index
IF ContainerRec.Locked > False THEN ' check container locked
    Outpt = "You can't, it's locked!" ' message
    CALL IO.O ' send message
    EXIT SUB ' exit routine
END IF ' end check container locked
IF ContainerRec.Closed > False THEN ' check container closed
    Outpt = "You can't, it's closed!" ' message
    CALL IO.O ' send message
    EXIT SUB ' exit routine
END IF ' end check container closed
Container.Number = Array.Number ' store container index of player inventory
Container.Name$ = ContainerRec.ContainerName ' store container name
Container.Name$ = RTRIM$(Container.Name$) ' edit container name
Parsed.Command1 = Parsed.Command2 ' get first parameter
CALL Numeric ' parse parameter number from # sign
Item.Counter = False ' parsed number counter
FOR Array.Counter = 1 TO 5 ' loop through all container treasure
    ' get treasure name from container
    Index.Number = GetContainerRecInventory(Array.Counter) ' store treasure index
    ' check treasure index range
    IF Index.Number > False AND Index.Number <= LOF(TreasureFile) / LEN(TreasureRecord) THEN
        CALL Read.Record(TreasureFile, Index.Number) ' get treasure record
        Inpt = RTRIM$(TreasureRecord.ShortName)
        IF LEN(Inpt) THEN ' check treasure name length
            Inpt = LEFT$(Inpt, LEN(Parsed.Command1)) ' truncate treasure name
            IF Inpt = Parsed.Command1 THEN ' compare to treasure to take
                Item.Counter = Item.Counter + 1 ' increment parsed counter
                ' compare to parameter number
                IF Parse.Number = False OR Item.Counter = Parse.Number THEN
                    Outpts = TreasureRecord.TreasureName ' format treasure name
                    Outpts = RTRIM$(Outpts) ' trim name
                    ' compute weight player carrying
                    New.Weight# = CDBL(INT(Weight + TreasureRecord.Weight))
                    IF New.Weight# > MaxInt THEN ' check maximum integer
                        New.Weight# = MaxInt ' reduce to maximum integer
                    END IF ' end check maximum integer
                    Total.Weight# = CDBL(GetUserRecordStats(1)) ' store player strength
                    Total.Weight# = Total.Weight# * 100 ' multiply weight player can carry
                    IF New.Weight# > Total.Weight# THEN ' compare wieght player can carry
                        Outpt = "You can't carry any more!" ' weight and make message
                        CALL IO.O ' send message
                        EXIT SUB ' exit routine
                    END IF ' end compare weight
                    ' get treasure charges
                    Charges.Number = GetContainerRecCharges(Array.Counter)
                    ' add treasure to inventory
                    CALL Add.Inventory(Index.Number, Charges.Number, Item.Added)
                    IF Item.Added THEN ' flag set to add
                        CALL Clear.Container(Array.Counter, False) ' clear container item
                        SELECT CASE TypeNumber ' select room or inventory
                            CASE 1 ' flag indicating container in room or inventory
                                RoomRecord.Container = ContainerRec ' set room container record
                                CALL Share.Room.Record(Room) ' write room record
                            CASE 0 ' flag indicates player record
                                ' update player container
                                CALL PutUserRecordContainer(Container.Number)
                        END SELECT ' end compare room or inventory
                        Outpt = "You take " + Outpts + " from " + Container.Name$ + "!" ' message
                        CALL IO.O ' send message
                        EXIT SUB ' exit routine
                    END IF ' end compare add flag
                END IF ' end check treasure index
            END IF ' end compare parsed number
        END IF ' end compare treasure name
    END IF ' end check treaure name length
NEXT ' end loop through player container inventory
Outpt = "You can't get that!" ' make message
CALL IO.O ' send message
END SUB ' end take from container routine

REM * sets container elements to zero.
REM * input variables:
REM *   Item.Number - container item to clear.
REM *   Clear.Items - true to clear container, false to clear container item.

SUB Clear.Container (Item.Number, Clear.Items)
IF Clear.Items = False THEN ' passed parameter to clear one container item
    CALL PutContainerRecCharges(Item.Number, False) ' container element to zero
    CALL PutContainerRecInventory(Item.Number, False) ' container element to zero
    CALL PutContainerRecInvisible(Item.Number, False) ' container element to zero
ELSE ' passed parameter to clear entire container
    ContainerRec.Closed = False ' container to zero
    ContainerRec.ContainerName = Nul ' container to zero
    ContainerRec.Keyed = False ' container to zero
    ContainerRec.Locked = False ' container to zero
    ContainerRec.ShortName = Nul ' container to zero
    ContainerRec.Permanent = False ' container to zero
    FOR Container.Item = 1 TO 5 ' loop through all container elements
        CALL PutContainerRecInventory(Container.Item, False) ' reset container element
        CALL PutContainerRecCharges(Container.Item, False) ' reset container element
        CALL PutContainerRecInvisible(Container.Item, False) ' reset container element
    NEXT ' end loop all container elements
END IF ' end passed parameter
END SUB ' end routine to clear container

REM * routine to display container information.
REM * input variables:
REM *   ContainerRec - container record to display.
REM *   TypeNumber - item is in room/inventory.

SUB Show.Container
IF TypeNumber THEN ' container in room
    Prefix1 = "It's " ' format prefix
ELSE ' container in inventory
    Prefix1 = "You are carrying " ' format prefix
END IF ' end container check
Graphics.Off = True ' reset color
Outpt = Prefix1 + RTRIM$(ContainerRec.ContainerName) ' make container name
IF ContainerRec.Keyed THEN ' check container has a lock number
    Outpt = Outpt + "(#" + RIGHT$(STR$(ContainerRec.Keyed + 100000!), 5) + ")"
END IF ' end append container lock number
Outpt = Outpt + "." ' append to message
CALL IO.O ' send message of container name
IF ContainerRec.Locked > False THEN ' verify container is locked
    Outpt = "It's locked." ' make message
    CALL IO.O ' send output
    EXIT SUB ' exit routine
END IF ' end verify locked container
IF ContainerRec.Closed > False THEN ' verify container is closed
    Outpt = "It's closed." ' make message
    CALL IO.O ' send output
    EXIT SUB ' exit routine
END IF ' end verify closed container
Outpt = "It contains the following treasure:" ' message
CALL IO.O ' send prefix message
Container.Count = False ' reset number of container items displayed
Outpt = Nul ' reset message of container item
FOR Array.Index = 1 TO 5 ' loop through all container items
    ' store container invisibility flag
    Invisible.Container = GetContainerRecInvisible(Array.Index)
    ' store container inventory number
    Container.Number = GetContainerRecInventory(Array.Index)
    Display.Item = True ' reset flag to display container item
    IF Container.Number THEN ' check container item
        IF Invisible.Container = True THEN ' verify item is invisible
            IF Normal.User THEN ' determine non DM
                Display.Item = False ' set flag not to display item
            END IF ' end normal user
        END IF ' end invisible item
        IF Display.Item = True THEN ' check flag to display item
            Carriage.Return = True ' set flag to disable return/linefeed
            CALL IO.O ' send message of previous item
            ' get treasure record of container item
            CALL Read.Record(TreasureFile, Container.Number)
            Outpts = TreasureRecord.TreasureName ' store treasure name
            Outpts = RTRIM$(Outpts) ' trim name
            IF Invisible.Container = True THEN ' compare invisible item
                Outpts = Outpts + "[inv]" ' sppend invisible message
            END IF ' end compare invisible
            Outpt = Outpts + ", " ' append comma to item name
            ' increment number of items displayed
            Container.Count = Container.Count + 1
            IF Container.Count = 1 THEN ' check if first item displayed
                MID$(Outpt, 1, 1) = UCASE$(MID$(Outpt, 1, 1)) ' uppercase first item
            END IF ' end compare to first item
        END IF ' end check display flag
    END IF ' end check container item
NEXT ' end loop through all five items
IF Container.Count = False THEN ' check if any container items displayed
    Outpt = "Nothing at all." ' make message
ELSE ' check any container items displayed
    Outpt = LEFT$(Outpt, LEN(Outpt) - 2) + "." ' trim comma, append period
    IF Container.Count > 1 THEN ' more than one item displayed
        Outpt = "and " + Outpt ' append to message
    END IF ' end check item number
END IF ' end check item number
CALL IO.O ' send message of last contaier item
Graphics.Off = False ' reset color
END SUB ' end routine to display container information

REM * routine to search player inventory for container name
REM * input variables:
REM *   Parsed.Command1 - name of container to search for.
REM * output variables:
REM *   Array.Number - container number.
REM *   Index.Number - true if container found.
REM *   Outpts - name of container.
REM * processing variables:
REM *   Parse.Count - increment counter.
REM *   Container.Name$ - container name.

SUB Check.Inventory.Container
Index.Number = False ' reset container found flag
Parse.Count = False ' reset counter
IF Parsed.Command1 <> Nul THEN ' check search string length
    FOR Array.Number = 1 TO 3 ' loop through player inventory containers
        ' store container record
        X = GetUserRecordContainer(Array.Number)
        ' store container name
        Container.Name$ = RTRIM$(ContainerRec.ShortName)
        Outpts = Container.Name$ ' store container name
        IF Container.Name$ <> Nul THEN ' check length of container name
            ' trim length of container name to length of search string
            Container.Name$ = LEFT$(Container.Name$, LEN(Parsed.Command1)) ' trim
            IF Container.Name$ = Parsed.Command1 THEN ' compare container names
                Parse.Count = Parse.Count + 1 ' increment counter
                ' check increment counter not specified,
                ' or counter equals increment counter.
                IF Parse.Number = False OR Parse.Count = Parse.Number THEN
                    Index.Number = True ' set container found flag
                    EXIT FOR ' exit loop through player containers
                END IF ' end check counters
            END IF ' end compare container names
        END IF ' end check container name length
    NEXT ' end loop through player containers
END IF ' end check search string length
END SUB ' end routine to search player inventory for container name

REM * routine to compare room container to search container name
REM * input variables:
REM *   Parsed.Command1 - name of container to ccompare.
REM * output variables:
REM *   Index.Number - room container matched.
REM *   Outpts - name of container.
REM * processing variables:
REM *   Parse.Count - increment counter.
REM *   Container.Name$ - container name.

SUB Check.Room.Container
Index.Number = False ' reset container found flag
Parse.Count = False ' reset counter
IF Parsed.Command1 <> Nul THEN ' check search string length
    ' get room container name
    Container.Name$ = RTRIM$(RoomRecord.Container.ShortName)
    Outpts = Container.Name$ ' store container name
    IF Container.Name$ <> Nul THEN ' check length of container name
        ' trim length of container name to length of search string
        Container.Name$ = LEFT$(Container.Name$, LEN(Parsed.Command1)) ' trim
        IF Container.Name$ = Parsed.Command1 THEN ' compare container names
            Parse.Count = Parse.Count + 1
            ' check increment counter not specified,
            ' or counter equals increment counter.
            IF Parse.Number = False OR Parse.Count = Parse.Number THEN
                Index.Number = True ' set container match flag
                ContainerRec = RoomRecord.Container ' store container record
            END IF ' end check counters
        END IF ' end compare names
    END IF ' end check container name
END IF ' end check search string length
END SUB

REM * routine to remove an item of player inventory and add to room.

SUB Drop.Item
CALL Find.Object ' check inventory for object by mnemonic name
IF Index.Number THEN ' returns index to objects file
    IF Last.Command.Number = Hide.Command THEN ' compare command to hide
        Outpt = "You hide " + Outpts + "!" ' format hide message
        Hide.Item = True ' set hide flag
    ELSE ' not hide command
        Outpt = "You drop " + Outpts + "!" ' format message
        Hide.Item = False ' set hide flag
    END IF ' end compare command
    CALL Add.Room.Object(Index.Number, Hide.Item, Item.Added) ' add item
    IF Item.Added THEN ' check object added flag
        CALL Discard.Inventory.Object(Array.Number) ' remove object
    ELSE ' check item added to room
        Outpt = "You can't drop that!" ' message
    END IF ' end check item added to room objects
    CALL IO.O ' send output
    EXIT SUB ' return from routine
END IF ' end check inventory for object
CALL Num ' decrement parse temp
CALL Find.Inventory ' check inventory for treasure by mnemonic name
IF Index.Number THEN ' returns index to treasure file
    IF Last.Command.Number = Hide.Command THEN ' compare command to hide
        Outpt = "You hide " + Outpts + "!" ' format hide message
        Hide.Item = Hidden.Object ' set hide flag
    ELSE ' not hide command
        Outpt = "You drop " + Outpts + "!" ' format message
        Hide.Item = False ' set hide flag
    END IF ' end compare command
    ' add item to room
    CALL Add.Room.Treasure(Index.Number, Charges.Number, Hide.Item, Item.Added)
    IF Item.Added THEN ' added to room flag
        CALL Discard.Inventory(Array.Number, True) ' remove from player inventory
    ELSE ' not added
        Outpt = "You can't drop that!" ' message
    END IF ' end check room flag
    CALL IO.O ' send output
    EXIT SUB ' return from routine
END IF ' end check inventory for treasure
CALL Num ' decrement parse temp
CALL Check.Inventory.Container ' check player inventory for container
IF Index.Number THEN ' returns index of container to treasure
    IF Last.Command.Number = Hide.Command THEN ' compare to hide command
        Outpt = "You can't hide that!" ' message
        CALL IO.O ' send output
        EXIT SUB ' return from routine
    END IF ' end compare to hide command
    IF RTRIM$(RoomRecord.Container.ShortName) <> Nul THEN ' room container empty
        Outpt = "You can't put more than one container in a room!" ' message
        CALL IO.O ' output message
        EXIT SUB ' exit routine
    END IF
    IF RTRIM$(RoomRecord.Container.ShortName) = Nul THEN ' room container empty
        ' store room container
        X = GetUserRecordContainer(Array.Number)
        RoomRecord.Container = ContainerRec
        CALL Share.Room.Record(Room) ' record structure, put room record,
        Outpts = RTRIM$(ContainerRec.ContainerName) ' message
        CALL Clear.Container(0, True) ' clear player container structure
        CALL PutUserRecordContainer(Array.Number) ' clear container record
        Outpt = "You drop " + Outpts + "!" ' format message
        CALL IO.O ' send output
        EXIT SUB ' return from routine
    END IF ' end compare room container
END IF ' end check container index
Outpt = "You can't drop that!" ' message
CALL IO.O ' send output
END SUB ' end drop item routine

REM * routine to take an object or item of treasure from the room and
REM * add to the player inventory.

SUB Take.Object
CALL Check.Room.Objects ' find the mnemonic of object to take
IF Index.Number THEN ' verify object file index
    Object.Flag = False ' reset flag
    IF Normal.User = False THEN ' check DM/Sysop
        Object.Flag = True ' reset flag
    ELSE ' end check user type
        IF ObjectRecord.Permanent = False THEN ' verify object permanent
            Object.Flag = True ' reset flag
        END IF ' end check object
    END IF ' end check player type
    IF Object.Flag THEN ' check take flag
        ' routine to add object to user
        CALL Add.Object(Index.Number, Item.Added)
        IF Item.Added THEN ' an empty object element
            ' routine to remove item from room
            CALL Discard.Room.Object(Array.Number)
            Outpt = "You take " + Outpts + "!" ' format message
            CALL IO.O ' display message
        ELSE ' check empty object found
            Outpt = "You can't carry any more!" ' format message
            CALL IO.O ' display message
        END IF ' end compare empty object element
        EXIT SUB ' exit routine
    END IF ' end find object to take
END IF ' end normal player
CALL Num ' decrement parse temp
CALL Check.Room.Treasure ' find mnemonic of treasure to take
IF Index.Number THEN ' treasure index found in room to take
    ' calculate weight player is carrying
    New.Weight# = CDBL(INT(Weight + TreasureRecord.Weight))
    IF New.Weight# > MaxInt THEN ' check maximum integer
        New.Weight# = MaxInt ' reduce to maximum integer
    END IF ' end check maximum integer
    Total.Weight# = CDBL(GetUserRecordStats(1)) ' store player strength
    Total.Weight# = Total.Weight# * 100 ' multiply to weight player can carry
    IF New.Weight# > Total.Weight# THEN ' compare weight player can carry
        Outpt = "You can't carry any more!" ' weight ten times player strength
        CALL IO.O ' format message
        EXIT SUB ' exit routine
    END IF ' end compare weight
    IF UserRecord.ClassType < AsstDM THEN ' normal class prevented to take item
        FOR Array.Counter = 1 TO Number.Monsters ' loop through all monsters
            ' check monster can prevent
            IF MonsterArray(Array.Counter).Prevent THEN
                ' compute random percent
                IF RND < (MonsterArray(Array.Counter).PreventPercent / 100) THEN
                    Inpt = MonsterArray(Array.Counter).MonsterName ' store name
                    Inpt = RTRIM$(Inpt) ' format message
                    Outpt = "The " + Inpt + " prevents you from getting it!"
                    CALL IO.O ' send message
                    EXIT SUB ' exit routine
                END IF ' end compare random calculation
            END IF ' end compare monster to prevent taking item
        NEXT ' end loop through all monsters in room
    END IF ' end compare player class
    IF TreasureRecord.Coin THEN ' check item is coins
        Outpt = "You take " + Outpts + "!" ' format message
        CALL IO.O ' send message
        UserRecord.Gold = UserRecord.Gold + TreasureRecord.Gold ' increment gold
        Outpt = "You now have" + FormatString$(UserRecord.Gold) + " Gold!" ' message
        CALL IO.O ' send message
        CALL Discard.Room.Treasure(Array.Number) ' remove item from room
        EXIT SUB ' exit routine
    END IF ' end compare item if coins
    ' add room item to inventory
    CALL Add.Inventory(Index.Number, Charges.Number, Item.Added)
    IF Item.Added THEN ' return flag indicates successful addition to inventory
        Outpt = "You take " + Outpts + "!" ' format message
        CALL IO.O ' send message
        CALL Discard.Room.Treasure(Array.Number) ' remove item from room
    ELSE ' compare flag
        Outpt = "You can't carry any more!" ' make message
        CALL IO.O ' send message
    END IF ' end compare flag
    EXIT SUB ' exit routine
END IF ' end item is treasure in room
CALL Num ' decrement parse temp
CALL Check.Room.Container ' verify item to take is a container
IF Index.Number THEN ' found container index to treasure file
    ' loop through player inventory to find empty container
    FOR Array.Number = 1 TO 3
        ' compare container name
        X = GetUserRecordContainer(Array.Number)
        IF RTRIM$(ContainerRec.ShortName) = Nul THEN
            ' store container record
            ContainerRec = RoomRecord.Container
            CALL PutUserRecordContainer(Array.Number)
            Outpts = RTRIM$(ContainerRec.ContainerName) ' format message
            Outpt = "You get " + Outpts + "!" ' make message
            CALL IO.O ' send message
            CALL Clear.Container(0, True) ' clear entire container record
            RoomRecord.Container = ContainerRec ' clear room container
            CALL Share.Room.Record(Room) ' write room record
            EXIT SUB ' exit routine
        END IF ' end compare empty container mnemonic name
    NEXT ' loop through inventory
    Outpt = "You can't carry more than three containers!" ' message
    CALL IO.O ' send message
    EXIT SUB ' exit routine
END IF ' end verify taking container
Outpt = "You can't get that!" ' message
CALL IO.O ' send message
END SUB ' end take item routine

REM * routine to lock an object or container.
REM * Note: Allows DM/Sysop to add locks.

SUB Lock.Object
CALL Examine.Container ' check container mnemonic
IF Index.Number THEN ' container index to treasure file
    Container.Number = Array.Number ' store container number
    IF ContainerRec.Locked > False THEN ' container lock variable is already set
        Outpt = "It's already locked!" ' make message
        CALL IO.O ' send message
        EXIT SUB ' exit routine
    END IF ' end compare container lock
    IF ContainerRec.Locked = False THEN ' container has no lock setting
        IF Normal.User THEN ' check DM/Sysop
            Outpt = "You can't lock that!" ' make message
            CALL IO.O ' send message
            EXIT SUB ' exit routine
        ELSE ' check DM/Sysop
            Outpt = "Container has no lock. Add one(y/n)? " ' make input prompt
            No.Input.Out = "N" ' default input
            CALL IO.I ' get input
            IF No THEN ' compare response
                EXIT SUB ' exit routine
            END IF ' end compare response
            Outpt = "Add a key to the lock(y/n)? " ' make message
            No.Input.Out = "N" ' default input
            CALL IO.I ' get input
            IF Yes THEN ' check response
                Outpt = "Key number(1-32767)? " ' make input prompt
                No.Input.Out = "1" ' default input
                CALL IO.I ' get user input
                ContainerRec.Keyed = INT(VAL(Inpt)) ' set object key number
            END IF ' end check response
        END IF ' check DM/Sysop
    END IF ' end compare container lock
    ContainerRec.Locked = 1 ' set container variable to locked
    ContainerRec.Closed = 1 ' set container variable to locked
    SELECT CASE TypeNumber ' select room or inventory
        CASE 1 ' container is locked in room
            RoomRecord.Container = ContainerRec
            CALL Share.Room.Record(Room) ' write room record
        CASE 0 ' container is locked in player inventory
            ' update player container
            CALL PutUserRecordContainer(Container.Number)
    END SELECT ' end compare container locked
    Outpt = "You lock it!" ' make message
    CALL IO.O ' send message
    EXIT SUB ' exit routine
END IF ' end check container
CALL Num ' decrement parse temp
CALL Examine.Objects ' check object mnemonic
IF Index.Number THEN ' found object index
    IF ObjectRecord.RoomLink = False THEN ' check object is a portal
        Outpt = "You can't lock that!" ' make message
        CALL IO.O ' send message
        EXIT SUB ' exit routine
    END IF ' end compare object portal
    IF ObjectRecord.DoorLock = 2 THEN ' object is already locked
        Outpt = "It's already locked!" ' make message
        CALL IO.O ' send message
        EXIT SUB ' exit routine
    END IF ' end compare object lock
    IF ObjectRecord.DoorLock = False THEN ' object has no lock setting
        IF Normal.User THEN ' check DM/Sysop
            Outpt = "You can't lock that!" ' make message
            CALL IO.O ' send message
            EXIT SUB ' exit routine
        ELSE ' check DM/Sysop
            Outpt = "Object has no lock. Add one(y/n)? " ' make input prompt
            No.Input.Out = "N" ' default input
            CALL IO.I ' get input
            IF No THEN ' compare response
                EXIT SUB ' exit routine
            END IF ' end compare response
            Outpt = "Add a key to the lock(y/n)? " ' make message
            No.Input.Out = "N" ' default input
            CALL IO.I ' get input
            IF Yes THEN ' check response
                Outpt = "Key number(1-32767)? " ' make input prompt
                No.Input.Out = "1" ' default input
                CALL IO.I ' get user input
                ObjectRecord.Keyed = INT(VAL(Inpt)) ' set object key number
            END IF ' end check response
        END IF ' check DM/Sysop
    END IF ' end compare object lock
    ObjectRecord.DoorLock = 2 ' set object lock variable
    ObjectRecord.Closed = True ' set object lock variable
    CALL Share.Record(ObjectFile, Index.Number) ' write object record
    Outpt = "You lock it!" ' make message
    CALL IO.O ' send message
    EXIT SUB ' exit routine
END IF ' end check object
Outpt = "You can't lock that!" ' make message
CALL IO.O ' send message
END SUB ' end routine to lock an item

REM * routine to unlock or picklock an object or container.

SUB Unlock.Object
CALL Examine.Container ' check container mnemonic
IF Index.Number THEN ' found container treasure index
    Container.Number = Array.Number ' store container number
    IF ContainerRec.Locked = False THEN ' container has no lock setting
        Outpt = "You can't unlock that!" ' make message
        CALL IO.O ' send message
        EXIT SUB ' exit routine
    END IF ' end compare container lock
    IF ContainerRec.Locked < False THEN ' container lock variable already set
        Outpt = "It's already unlocked!" ' make message
        CALL IO.O ' send message
        EXIT SUB ' exit routine
    END IF ' end compare container lock
    IF ContainerRec.Keyed = False THEN ' container has no key number
        ContainerRec.Locked = True ' set container lock variable
        ContainerRec.Closed = 1 ' set container lock variable
        SELECT CASE TypeNumber ' select room or inventory
            CASE 1 ' container unlocked in room
                RoomRecord.Container = ContainerRec ' set room container record
                CALL Share.Room.Record(Room) ' write room record
            CASE 0 ' container is locked in player inventory
                ' update player container
                CALL PutUserRecordContainer(Container.Number)
        END SELECT ' end store container record
        Outpt = "You unlock it!" ' make message
        CALL IO.O ' send message
        EXIT SUB ' exit routine
    END IF ' end compare container key number
    IF Last.Command.Number = Picklock.Command THEN ' picklock container
        IF UserRecord.ClassType <> Thief THEN ' compare player class to thief
            IF Normal.User THEN ' compare player type to normal
                Outpt = "You can't picklock doors!" ' make message
                CALL IO.O ' send message
                EXIT SUB ' exit routine
            END IF ' end compare player type
        END IF ' end compare class to thief
        IF INT(RND * GetUserRecordStats(1)) + 3 < 7 THEN ' compute random chance
            Outpt = "You didn't picklock it!" ' make message
            CALL IO.O ' send message
            EXIT SUB ' exit routine
        END IF ' end random chance
        IF INT(RND * GetUserRecordStats(2)) + 3 < 7 THEN ' compute random chance
            Outpt = "You didn't picklock it!" ' make message
            CALL IO.O ' send message
            EXIT SUB ' exit routine
        END IF ' end random chance
        IF RND < .2 THEN ' random chance
            Outpt = "You didn't picklock it!" ' make message
            CALL IO.O ' send message
            EXIT SUB ' exit routine
        END IF ' end chance
        ContainerRec.Locked = True ' set container record to unlocked
        ContainerRec.Closed = 1 ' set container record to unlocked
        SELECT CASE TypeNumber ' select room or inventory
            CASE 1 ' container unlocked in room
                RoomRecord.Container = ContainerRec ' set room container record
                CALL Share.Room.Record(Room) ' write room record
            CASE 0 ' container unlocked in player inventory
                ' update player container
                CALL PutUserRecordContainer(Container.Number)
        END SELECT ' end set container record
        Outpt = "You picklock it!" ' make message
        CALL IO.O ' send message
        EXIT SUB ' exit routine
    END IF ' end check picklock container
    ' compare player inventory key numbers to container key number
    Key.Number = ContainerRec.Keyed ' store container key number to unlock
    Inpt = RIGHT$(STR$(Key.Number + 100000!), 5) ' format key number to string
    FOR Inventory.Number = 1 TO 20 ' loop through all player inventory
        ' player inventory treasure index
        IF GetUserRecordInv(Inventory.Number) THEN
            ' get treasure record
            CALL Read.Record(TreasureFile, GetUserRecordInv(Inventory.Number))
            IF TreasureRecord.Keyed THEN ' compare inventory treasure to key
                Container.Locked = True ' container unlocked flag
                ' format key number in inventory to string
                Outpts = RIGHT$(STR$(TreasureRecord.Keyed + 100000!), 5)
                FOR Digit.Counter = 1 TO 5 ' loop through all key number digits
                    ' get value of key digit
                    Key.Digit = VAL(MID$(Outpts, Digit.Counter, 1))
                    ' compare player inventory key digit to container key digit
                    ' digits unequal
                    IF Key.Digit <> VAL(MID$(Inpt, Digit.Counter, 1)) THEN
                        ' player inventory key digit 0 matches
                        IF Key.Digit > False THEN
                            ' container is not unlocked flag
                            Container.Locked = False
                            EXIT FOR ' exit loop, digit mismatch
                        END IF ' end compare key digit 0 matches any key number
                    END IF ' end compare key digit matches specific key number
                NEXT ' end loop through all key digits in player inventory key
                ' key in player inventory found to match container
                IF Container.Locked THEN
                    ContainerRec.Locked = True ' set container unlock
                    ContainerRec.Closed = 1 ' set container unlock
                    SELECT CASE TypeNumber ' select room or inventory
                        CASE 1 ' container unlocked with key in room
                            RoomRecord.Container = ContainerRec ' set room container
                            CALL Share.Room.Record(Room) ' write room record
                        CASE 0 ' container unlocked with key in player inventory
                            ' update player container
                            CALL PutUserRecordContainer(Container.Number)
                    END SELECT ' end set container record
                    Outpt = "You unlock it!" ' make message
                    CALL IO.O ' send message
                    EXIT SUB ' exit routine
                END IF ' end compare key match
            END IF ' end compare player inventory to a key
        END IF ' end compare player inventory
    NEXT ' end loop through all keys in player inventory
    Outpt = "You don't have the key!" ' make message
    CALL IO.O ' send message
    EXIT SUB ' exit routine
END IF ' end check container
CALL Num ' decrement parse temp
CALL Examine.Objects ' check object mnemonic
IF Index.Number THEN ' found object index
    IF ObjectRecord.RoomLink = False THEN ' check object is a portal
        Outpt = "You can't unlock that!" ' make message
        CALL IO.O ' send message
        EXIT SUB ' exit routine
    END IF ' end object portal
    IF ObjectRecord.DoorLock = False THEN ' object does not lock
        Outpt = "You can't unlock that!" ' make message
        CALL IO.O ' send message
        EXIT SUB ' exit routine
    END IF ' end object lock
    IF ObjectRecord.DoorLock = 1 THEN ' object is already unlocked
        Outpt = "It's already unlocked!" ' make message
        CALL IO.O ' send message
        EXIT SUB ' exit routine
    END IF ' end object lock
    IF ObjectRecord.Keyed = False THEN ' object has no key number
        ObjectRecord.DoorLock = 1 ' set object unlocked
        ObjectRecord.Closed = True ' set object unlocked
        CALL Share.Record(ObjectFile, Index.Number) ' write object record
        Outpt = "You unlock it!" ' make message
        CALL IO.O ' send message
        EXIT SUB ' exit routine
    END IF ' end object key number
    IF Last.Command.Number = Picklock.Command THEN ' picklock object
        IF ObjectRecord.Hidden THEN ' object is hidden
            Outpt = "You can't picklock hidden doors!" ' make message
            CALL IO.O ' send message
            EXIT SUB ' exit routine
        END IF ' end object hidden
        IF UserRecord.ClassType <> Thief THEN ' check player class to thief
            IF Normal.User THEN ' check player is normal
                Outpt = "You can't picklock doors!" ' make message
                CALL IO.O ' send message
                EXIT SUB ' exit routine
            END IF ' end compare normal player
        END IF ' end compare class type
        IF INT(RND * GetUserRecordStats(1)) + 3 < 7 THEN ' compute random chance
            Outpt = "You didn't picklock it!" ' make message
            CALL IO.O ' send message
            EXIT SUB ' exit routine
        END IF ' end random chance
        IF INT(RND * GetUserRecordStats(2)) + 3 < 7 THEN ' compute random chance
            Outpt = "You didn't picklock it!" ' make message
            CALL IO.O ' send message
            EXIT SUB ' exit routine
        END IF ' end random chance
        IF RND < .2 THEN ' random chance
            Outpt = "You didn't picklock it!" ' make message
            CALL IO.O ' send message
            EXIT SUB ' exit routine
        END IF ' end chance
        ObjectRecord.DoorLock = 1 ' set object unlocked
        ObjectRecord.Closed = True ' set object unlocked
        CALL Share.Record(ObjectFile, Index.Number) ' write object record
        Outpt = "You picklock it!" ' make message
        CALL IO.O ' send message
        EXIT SUB ' exit routine
    END IF ' end picklock
    ' compare player inventory key numbers to object key number
    Inpt = RIGHT$(STR$(ObjectRecord.Keyed + 100000!), 5) ' format key number
    FOR Inventory.Number = 1 TO 20 ' loop through all player inventory
        ' player inventory treasure index
        IF GetUserRecordInv(Inventory.Number) THEN
            ' get treasure record
            CALL Read.Record(TreasureFile, GetUserRecordInv(Inventory.Number))
            IF TreasureRecord.Keyed THEN ' compare treasure to key
                Container.Locked = True ' set object unlocked flag
                ' format player inventory key number to string
                Outpts = RIGHT$(STR$(TreasureRecord.Keyed + 100000!), 5)
                FOR Digit.Counter = 1 TO 5 ' loop through all key number digits
                    ' get inventory key digit
                    Key.Digit = VAL(MID$(Outpts, Digit.Counter, 1))
                    ' compare player inventory key digit to container key digit
                    ' compare to object
                    IF Key.Digit <> VAL(MID$(Inpt, Digit.Counter, 1)) THEN
                        IF Key.Digit > False THEN ' zero digit matches any key digit
                            Container.Locked = False ' set object is not unlocked flag
                            EXIT FOR ' exit loop, found unmatching digit
                        END IF ' end compare matching zero digit
                    END IF ' end compare both digits
                NEXT ' end loop through all digits
                IF Container.Locked THEN ' object key number matched
                    ObjectRecord.DoorLock = 1 ' set object unlocked
                    ObjectRecord.Closed = True ' set object unlocked
                    CALL Share.Record(ObjectFile, Index.Number) ' write object record
                    Outpt = "You unlock it!" ' make message
                    CALL IO.O ' send message
                    EXIT SUB ' exit routine
                END IF ' end matching key number
            END IF ' end locate player inventory key
        END IF ' end find player inventory treasure
    NEXT ' end loop through all player inventory for keys
    Outpt = "You don't have the key!" ' make message
    CALL IO.O ' send message
    EXIT SUB ' exit routine
END IF ' end unlock object
Outpt = "You can't unlock that!" ' make message
CALL IO.O ' send message
END SUB ' end routine to unlock/picklock an item or container

REM * routine to close an object.

SUB Close.Object
CALL Examine.Container ' check container mnemonic
IF Index.Number THEN ' found container index to treasure file
    Container.Number = Array.Number ' store container number
    IF ContainerRec.Locked = False THEN ' container has no lock to set
        Outpt = "You can't close that!" ' make message
        CALL IO.O ' send message
        EXIT SUB ' exit routine
    END IF ' end container lock
    IF ContainerRec.Locked > False THEN ' check if container already locked
        Outpt = "It's already closed!" ' make message
        CALL IO.O ' send message
        EXIT SUB ' exit routine
    END IF ' end container lock
    IF ContainerRec.Closed = False THEN ' container has no close setting
        Outpt = "You can't close that!" ' make message
        CALL IO.O ' send message
        EXIT SUB ' exit routine
    END IF ' end compare closed
    IF ContainerRec.Closed > False THEN ' container is already closed
        Outpt = "It's already closed!" ' make message
        CALL IO.O ' send message
        EXIT SUB ' exit routine
    END IF ' end compare closed
    ContainerRec.Closed = 1 ' set container record closed variable
    SELECT CASE TypeNumber ' select room or inventory
        CASE 1 ' container closed in room
            RoomRecord.Container = ContainerRec ' set room container record
            CALL Share.Room.Record(Room) ' write room record
        CASE 0 ' container closed in player inventory
            ' update player container
            CALL PutUserRecordContainer(Container.Number)
    END SELECT ' end reset container
    Outpt = "You close it!" ' make message
    CALL IO.O ' send message
    EXIT SUB ' exit routine
END IF ' end compare container index
CALL Num ' decrement parse temp
CALL Examine.Objects ' check object mnemonic
IF Index.Number THEN ' found object index to object file
    IF ObjectRecord.DoorLock <> 1 THEN ' compare object already locked
        Outpt = "You can't close that!" ' make message
        CALL IO.O ' send message
        EXIT SUB ' exit routine
    END IF ' end compare lock
    IF ObjectRecord.Closed THEN ' compare container already closed
        Outpt = "It's already closed!" ' make message
        CALL IO.O ' send message
        EXIT SUB ' exit routine
    END IF ' end compare closed
    ObjectRecord.Closed = True ' set object record closed variable
    CALL Share.Record(ObjectFile, Index.Number) ' write object record
    Outpt = "You close it!" ' make message
    CALL IO.O ' send message
    EXIT SUB ' exit routine
END IF ' end object search
Outpt = "You can't close that!" ' make message
CALL IO.O ' send message
END SUB ' end routine to closed an item

REM * routine to open an item.

SUB Open.Object
CALL Examine.Container ' check container mnemonic
IF Index.Number THEN ' found container index
    Container.Number = Array.Number ' store container number
    IF ContainerRec.Locked = False THEN ' container has no lock to set
        Outpt = "You can't open that!" ' make message
        CALL IO.O ' send message
        EXIT SUB ' exit routine
    END IF ' end compare container lock
    IF ContainerRec.Locked > False THEN ' compare container lock variable
        Outpt = "You can't, it's locked!" ' make message
        CALL IO.O ' send message
        EXIT SUB ' exit routine
    END IF ' end compare lock
    IF ContainerRec.Closed = False THEN ' container has no close variable to set
        Outpt = "You can't open that!" ' make message
        CALL IO.O ' send message
        EXIT SUB ' exit routine
    END IF ' end compare close
    IF ContainerRec.Closed < False THEN ' container is already open
        Outpt = "It's already open!" ' make message
        CALL IO.O ' send message
        EXIT SUB ' exit routine
    END IF ' end compare close
    ContainerRec.Closed = True ' set container open variable
    SELECT CASE TypeNumber ' select room or inventory
        CASE 1 ' container is opened in room
            RoomRecord.Container = ContainerRec ' reset room container
            CALL Share.Room.Record(Room) ' write room record
        CASE 0 ' container is open in inventory
            ' update player container
            CALL PutUserRecordContainer(Container.Number)
    END SELECT ' end reset container
    Outpt = "You open it!" ' make message
    CALL IO.O ' send message
    EXIT SUB ' exit routine
END IF ' end compare container
CALL Num ' reset parse temp
CALL Examine.Objects ' check object mnemonic
IF Index.Number THEN ' found object index
    IF ObjectRecord.DoorLock <> 1 THEN ' compare object closed variable
        Outpt = "You can't open that!" ' make message
        CALL IO.O ' send message
        EXIT SUB ' exit routine
    END IF ' end compare closed
    IF ObjectRecord.Closed = False THEN ' compare object open variable
        Outpt = "It's already open!" ' make message
        CALL IO.O ' send message
        EXIT SUB ' exit routine
    END IF ' end compare opened
    ObjectRecord.Closed = False ' set object record open variable
    CALL Share.Record(ObjectFile, Index.Number) ' write object record
    Outpt = "You open it!" ' make message
    CALL IO.O ' send message
    EXIT SUB ' exit routine
END IF ' end object index
Outpt = "You can't open that!" ' make message
CALL IO.O ' send message
END SUB ' end routine to open an item

REM * routine to smash open a locked item.

SUB Smash.Object
CALL Examine.Container ' check container mnemonic
IF Index.Number THEN ' found container index to treasure file
    Container.Number = Array.Number ' store container number
    IF ContainerRec.Locked = False THEN ' container has no available lock
        Outpt = "You can't smash that!" ' make message
        CALL IO.O ' send message
        EXIT SUB ' exit routine
    END IF ' end compare container lock
    IF ContainerRec.Locked < False THEN ' lock variable is already open
        Outpt = "It's already unlocked!" ' make message
        CALL IO.O ' send message
        EXIT SUB ' exit routine
    END IF ' end compare lock
    IF ContainerRec.Keyed > False THEN ' container has a key number
        Outpt = "You can't smash that!" ' make message
        CALL IO.O ' send message
        EXIT SUB ' exit routine
    END IF ' end compare key number
    IF INT(RND * GetUserRecordStats(2)) + 1 < 7 THEN ' compute smash chance
        Outpt = "You didn't smash it open!" ' make message
        CALL IO.O ' send message
        EXIT SUB ' exit routine
    END IF ' end chance
    IF INT(RND * GetUserRecordStats(1)) + 1 < 7 THEN ' compare smash chance
        Outpt = "You didn't smash it open!" ' make message
        CALL IO.O ' send message
        EXIT SUB ' exit routine
    END IF ' end chance
    IF RND < .2 THEN ' random chance to smash lock
        Outpt = "You didn't smash it open!" ' make message
        CALL IO.O ' send message
        EXIT SUB ' exit routine
    END IF ' end chance
    ContainerRec.Locked = True ' set container to unlocked
    ContainerRec.Closed = 1 ' set container to closed
    SELECT CASE TypeNumber ' select room or inventory
        CASE 1 ' container unlocked in room
            RoomRecord.Container = ContainerRec ' set new container record
            CALL Share.Room.Record(Room) ' write room record
        CASE 0 ' container unlocked in inventory
            ' update player container
            CALL PutUserRecordContainer(Container.Number)
    END SELECT ' end reset container
    Outpt = "You smash it open!" ' make message
    CALL IO.O ' send message
    EXIT SUB ' exit routine
END IF ' end compare container index
CALL Num ' decrement parse temp
CALL Examine.Objects ' check object mnemonic
IF Index.Number THEN ' found object index
    IF ObjectRecord.Hidden THEN ' object type is hidden
        Outpt = "You can't smash that!" ' make message
        CALL IO.O ' send message
        EXIT SUB ' exit routine
    END IF ' end object type
    IF ObjectRecord.RoomLink = False THEN ' object is not a portal
        Outpt = "You can't smash that!" ' make message
        CALL IO.O ' send message
        EXIT SUB ' exit routine
    END IF ' end portal
    IF ObjectRecord.Keyed THEN ' object has a key number
        Outpt = "You can't smash that!" ' make message
        CALL IO.O ' send message
        EXIT SUB ' exit routine
    END IF ' end object key number
    IF ObjectRecord.DoorLock = 1 THEN ' object lock is already open
        Outpt = "It's already unlocked!" ' make message
        CALL IO.O ' send message
        EXIT SUB ' exit routine
    END IF ' end object lock
    IF INT(RND * GetUserRecordStats(2)) + 1 < 7 THEN ' compute random chance
        Outpt = "You didn't smash it open!" ' make message
        CALL IO.O ' send message
        EXIT SUB ' exit routine
    END IF ' end chance
    IF INT(RND * GetUserRecordStats(1)) + 1 < 7 THEN ' compute random chance
        Outpt = "You didn't smash it open!" ' make message
        CALL IO.O ' send message
        EXIT SUB ' exit routine
    END IF ' end chance
    IF RND < .2 THEN ' random chance
        Outpt = "You didn't smash it open!" ' make message
        CALL IO.O ' send message
        EXIT SUB ' exit routine
    END IF ' end chance
    ObjectRecord.DoorLock = 1 ' set object record lock to open
    CALL Share.Record(ObjectFile, Index.Number) ' write object record
    Outpt = "You smash it open!" ' make message
    CALL IO.O ' send message
    IF ObjectRecord.Trap THEN ' check object is trapped
        CALL Traps ' initiate any object traps
    END IF ' end object trap
    EXIT SUB ' exit routine
END IF ' end object index
Outpt = "You can't smash that!" ' make message
CALL IO.O ' send message
END SUB ' end routine to smash open lock

REM * routine to search a container.

SUB Search.Object
CALL Examine.Container ' check player and room inventory for container name
IF Index.Number THEN ' found container index, placed into container record
    IF ContainerRec.Locked > False THEN ' compare container lock
        Outpt = "You can't, it's locked!" ' make message
        CALL IO.O ' send message
        EXIT SUB ' exit routine
    END IF ' end compare container
    IF ContainerRec.Closed > False THEN ' compare container closed
        Outpt = "You can't, it's closed!" ' make message
        CALL IO.O ' send message
        EXIT SUB ' exit routine
    END IF ' end compare container
    Outpt = "You search the container.." ' make message
    CALL IO.O ' send message
    Graphics.Off = True ' color white
    Outpt = "In it you find " ' preceding message
    Item.Counter = False ' container contents counter
    FOR Array.Number = 1 TO 5 ' loop through all container contents
        IF GetContainerRecInvisible(Array.Number) THEN ' compare invisible item
            ' get container treasure index
            Inventory.Number = GetContainerRecInventory(Array.Number)
            IF Inventory.Number > False AND Inventory.Number <= LOF(TreasureFile) / LEN(TreasureRecord) THEN
                ' read index from treasure file
                CALL Read.Record(TreasureFile, Inventory.Number)
                IF RND > .5 THEN ' compute random chance
                    Carriage.Return = True ' supress cr/lf
                    CALL IO.O ' send message
                    Outpt = TreasureRecord.TreasureName ' get treasure name found
                    Outpt = RTRIM$(Outpt) + ", " ' format treasure name
                    ' increment container contents counter
                    Item.Counter = Item.Counter + 1
                END IF ' end random chance
            END IF ' end check treasure file bounds
        END IF ' end compare container contents invisible
    NEXT ' end loop through all container contents
    IF Item.Counter = False THEN ' compare no contents found
        Outpt = "In it you find nothing.." ' make message
        CALL IO.O ' send message
        EXIT SUB ' exit routine
    END IF ' end compare container content counter
    Outpt = LEFT$(Outpt, LEN(Outpt) - 2) + "." ' format message, strip trailing comma
    IF Item.Counter > 1 THEN ' compare container content counter
        Outpt = "and " + Outpt ' format message with last contents found
    END IF ' end compare content counter
    CALL IO.O ' send message
    EXIT SUB ' exit routine
END IF ' end find container to search
Outpt = "You can't search that!" ' make message
CALL IO.O ' send message
END SUB ' end routine to search container

REM * routine to display verbose text on an item entered on the game prompt,
REM * including a room number, an object, an item of treasure, monster, and
REM * contents of a container.

SUB Display.Information
IF NOT Normal.User THEN ' check non DM
    ' convert parameter to number (i.e. Look 12)
    Room.Number! = INT(VAL(Parsed.Command1))
    ' check room bounds
    IF Room.Number! > False AND Room.Number! <= LOF(RoomFile) / LEN(RoomRecord) THEN
        SWAP Room.Number!, Room ' store current room number
        CALL Read.Room.Record(Room) ' get room to display
        CALL Show.Room ' routine to display room
        SWAP Room.Number!, Room ' restore current room number
        CALL Read.Room.Record(Room) ' get current room
        EXIT SUB ' exit routine
    END IF ' end check room number
END IF ' end check normal user
Outpt = UserRecord.CodeName ' get user codename
CALL Decrypt(Outpt) ' decrypt codename
Outpt = LEFT$(Outpt, LEN(Parsed.Command1)) ' store parameter
IF Outpt = Parsed.Command1 THEN ' compare look command to user codename
    CALL Display.Stats ' display player statistics
    EXIT SUB ' exit routine
END IF ' end check command parameter
CALL Examine.Treasure ' routine to compare parameter to treasure name
IF Index.Number THEN ' compare returned index to treasure file
    CALL Show.Treasure ' routine to display information on an item of treasure
    EXIT SUB ' exit routine
END IF ' end compare display treasure item
CALL Num ' reduce number of item
CALL Examine.Container ' compare parameter to container name
IF Index.Number THEN ' check container to display
    CALL Show.Container ' routine to display information on a container
    EXIT SUB ' exit routine
END IF ' end display container
CALL Num ' reduce number of item
CALL Examine.Objects ' compare parameter to object
IF Index.Number THEN ' check index to object record
    CALL Show.Object ' routine to display information on an object
    EXIT SUB ' exit routine
END IF ' end check object
CALL Num ' reduce number of item
CALL Check.Monster ' compare parameter to monster name in room
IF Monster.Number THEN ' check index of monster array in room
    CALL Show.Monster ' routine to display information on a monster
    EXIT SUB ' exit routine
END IF ' end check monster name
Outpt = "You can't examine that!" ' make error message
CALL IO.O ' send error message
END SUB ' end routine to display item

REM * routine to display room description (short or long), objects in room,
REM * monsters in room, containers in room, and directions/exits from room.
REM * working variables:
REM *   Lit.Room - returned true if room is unlit.

SUB Show.Room
CALL Check.Lit.Room(Lit.Room) ' routine to determine if room is unlit
IF Lit.Room THEN ' check unlit room
    Outpt = "It's too dark to see!" ' make message
    CALL IO.O ' send message output
    EXIT SUB ' exit routine
END IF ' end check unlit room
Graphics.Off = True ' reser color
' check if user is in brief mode, or room has no long description
Outpt = GetRoomRecordLongDesc(1) ' store room short one line description
Outpt = RTRIM$(Outpt) ' trim brief description
' find any null characters from old structures
IF INSTR(Outpt, CHR$(0)) THEN
    Outpt = LEFT$(Outpt, INSTR(Outpt, CHR$(0)) - 1) ' truncate off nulls
END IF ' end find old nulls
IF UserRecord.Brief OR Outpt = Nul THEN
    IF Normal.User = False THEN ' compare non DM status
        Outpt = "(room:" + STR$(Room) + ", monclass:" ' display DM room statistics
        IF Room = 1! THEN ' check safe haven room
            Outpt = Outpt + " <safe haven>" ' append room message
        ELSE ' check safe haven
            Outpt = Outpt + STR$(RoomRecord.MonsterClass) ' display room number
        END IF ' end check safe haven room number
        Outpt = Outpt + ", action:" + STR$(RoomRecord.Action) + ")"
        IF Room = 1! THEN ' recheck safe haven room
            Outpt = Outpt + " (resurrection room:" ' display resurrection room
            IF RoomRecord.MonsterClass = False THEN ' check room number
                Outpt = Outpt + " <none>)" ' append default
            ELSE ' check room number
                Outpt = Outpt + STR$(RoomRecord.MonsterClass) + ")" ' append room number
            END IF ' end check room number
        END IF ' end check safe haven room
        CALL IO.O ' display room monster class
    END IF ' end compare normal user in brief description
    Outpt = RoomRecord.ShortDesc ' store room short one line description
    Outpt = RTRIM$(Outpt) ' trim brief description
    ' find any null characters from old structures
    IF INSTR(Outpt, CHR$(0)) THEN
        Outpt = LEFT$(Outpt, INSTR(Outpt, CHR$(0)) - 1) ' truncate off nulls
    END IF ' end find old nulls
    CALL IO.O ' display room short description
ELSE ' display room four line long description
    FOR Desc.Line = 1 TO 4 ' loop through all four long room description lines
        Outpt = GetRoomRecordLongDesc(Desc.Line) ' get next room description line
        Outpt = RTRIM$(Outpt) ' trim room description line
        IF INSTR(Outpt, CHR$(0)) THEN ' find any nul characters from old files
            Outpt = LEFT$(Outpt, INSTR(Outpt, CHR$(0)) - 1) ' truncate off nulls
        END IF ' end find old nulls
        IF Outpt = Nul THEN ' check if long description ends
            EXIT FOR ' exit long description display loop
        END IF ' end check description end
        CALL IO.O ' display room description line
    NEXT ' end long room description loop
END IF ' end check room description to display
Exit.Displayed = False ' set flag for an exit displayed
IF UserRecord.Brief THEN ' check user is in brief mode
    Outpt = Nul ' assign empty display string
ELSE ' user is not in brief mode
    Outpt = "The exits are " ' assign initial display string
END IF ' end check brief mode
FOR Direction.Number = 1 TO 12 ' loop through all room directions
    ' compare room structure for direction
    IF GetRoomRecordDirect(Direction.Number) THEN
        ' append exit to string
        Outpt = Outpt + RTRIM$(Direction(Direction.Number)) + ", "
        Exit.Displayed = True ' set flag for an exit displayed
    END IF ' end compare room direction exists
NEXT ' end loop through room directions
IF Exit.Displayed THEN ' compare flag for an exit displayed
    Outpt = LEFT$(Outpt, LEN(Outpt) - 2) + "." ' truncate trailing comma, add period
    IF UserRecord.Brief THEN ' check user in brief mode
        MID$(Outpt, 1, 1) = UCASE$(MID$(Outpt, 1, 1)) ' set first letter uppercase
    END IF ' end check brief mode
    CALL IO.O ' send room exit display
END IF ' end compare exit displayed flag
' following displays all objects, treasure, monsters, and container in room
IF UserRecord.Brief = False THEN ' check user is in brief mode
    Outpt = "You see " ' assign intial string
    Carriage.Return = True ' set flag to disable return/linefeed
    CALL IO.O ' send initial string
END IF ' end check brief mode
Outpt = Nul ' assign empty display string
Items.Displayed = False ' reset number of items displayed
Object.Displayed = False ' reset flag indicating an item has been displayed
FOR Array.Index = 1 TO 20 ' loop through all 20 objects in room
    DisplayObject = False ' set flag to show object
    Object.Number = GetRoomRecordObject(Array.Index) ' store room object number
    ' compare range
    IF Object.Number > False AND Object.Number <= LOF(ObjectFile) / LEN(ObjectRecord) THEN
        CALL Read.Record(ObjectFile, Object.Number) ' get object record
        IF ObjectRecord.Invisible THEN ' check object is invisible
            IF Normal.User THEN ' verify non DM
                DisplayObject = True ' set flag to show object off
            END IF ' end verify normal user
        END IF ' end check invisible object
        IF GetRoomRecordHiddenObj(Array.Index) THEN ' check object is hidden
            IF Normal.User THEN ' verify non DM
                DisplayObject = True ' set flag to show object off
            END IF ' end verify normal user
        END IF ' end check invisible object
        IF ObjectRecord.Hidden THEN ' check object is hidden
            IF Normal.User THEN ' verify non DM
                DisplayObject = True ' set flag to show object off
            END IF ' end verify normal user
        END IF ' end check hidden object
        IF DisplayObject = False THEN ' check display flag
            Carriage.Return = True ' set flag to disable return/linefeed
            CALL IO.O ' send display of previous item
            ' increment number of items displayed
            Items.Displayed = Items.Displayed + 1
            IF Items.Displayed = 1 THEN ' verify first item being displayed
                IF UserRecord.Brief THEN ' check user in brief mode
                    Outpt = "You see " ' send initial display string
                    Carriage.Return = True ' set flag to disable return/linefeed
                    CALL IO.O ' send previous string
                END IF ' end check brief mode
            END IF ' end verify first item
            Outpt = ObjectRecord.ObjectName ' store object name
            Outpt = RTRIM$(Outpt) ' trim object name
            IF ObjectRecord.DoorLock > 1 THEN ' check object is locked
                Outpt = Outpt + "[locked]" ' append message
            ELSE ' compare unlocked object
                IF ObjectRecord.DoorLock THEN ' check object has lock
                    IF ObjectRecord.Closed THEN ' check object is closed
                        Outpt = Outpt + "[closed]" ' append message
                    END IF ' end check closed object
                END IF ' end check object is lock
            END IF ' end check object is locked
            Hidden.Flag = GetRoomRecordHiddenObj(Array.Index) ' store hidden flag
            IF ObjectRecord.Hidden THEN ' check object is hidden
                Outpt = Outpt + "[hidden]" ' append message
            ELSE ' check object not hidden
                ' check object is invisible/hidden
                IF ObjectRecord.Invisible OR Hidden.Flag THEN
                    Outpt = Outpt + "[inv]" ' append message
                END IF ' end check invisible object
            END IF ' end check object hidden
            Outpt = Outpt + ", " ' append comma
            Object.Displayed = True ' set flag indicating an item displayed
        END IF ' end check display flag
    END IF ' end compare object file record range
NEXT ' end loop through objects in room
FOR Array.Index = 1 TO 20 ' loop through all room treasure
    DisplayTreasure = False ' set flag to display item
    Treasure.Number = GetRoomRecordTreasure(Array.Index) ' store treasure number
    IF Treasure.Number > False AND Treasure.Number <= LOF(TreasureFile) / LEN(TreasureRecord) THEN ' compare range
        CALL Read.Record(TreasureFile, Treasure.Number) ' get treasure record
        ' compare invisible treasure flag
        IF GetRoomRecordFlags(Array.Index) > False THEN
            IF Normal.User THEN ' check non DM
                DisplayTreasure = True ' set flag to display item
            END IF ' end check normal user
        END IF ' end compare invisible treasure
        IF TreasureRecord.Invisible THEN ' check treasure is invisible
            IF Normal.User THEN ' check non DM
                DisplayTreasure = True ' set display flag
            END IF ' end check normal user
        END IF ' end check invisible treasure
        IF DisplayTreasure = False THEN ' compare display flag
            Carriage.Return = True ' set flag to disable return/linefeed
            CALL IO.O ' display previous item
            ' increment number of items displayed
            Items.Displayed = Items.Displayed + 1
            IF Items.Displayed = 1 THEN ' check first item
                IF UserRecord.Brief THEN ' check user is in brief mode
                    Outpt = "You see " ' make initial string
                    Carriage.Return = True ' set flag to disable return/linefeed
                    CALL IO.O ' send initial string
                END IF ' end check brief mode
            END IF
            Outpt = TreasureRecord.TreasureName ' get treasure name
            Outpt = RTRIM$(Outpt) ' trim treasure name
            IF TreasureRecord.Keyed THEN ' append key number to treasure name
                Outpt = Outpt + "(#" + RIGHT$(STR$(TreasureRecord.Keyed + 100000!), 5) + ")"
            END IF ' end compare treasure key number
            IF TreasureRecord.Plus THEN ' append plus number to treasure name
                Outpt = Outpt + "(+" + MID$(STR$(ABS(TreasureRecord.Plus)), 2) + ")"
            END IF ' end compare treasure plus
            IF TreasureRecord.Spell THEN ' append spell plus to treasure name
                CALL Read.Record(SpellFile, TreasureRecord.Spell) 'get spell record
                Outpt = Outpt + "(+" + MID$(STR$(SpellRecord.Level), 2) + ")"
            END IF ' end compare treasure spell plus
            IF TreasureRecord.Locked > False THEN ' check treasure is locked
                Outpt = Outpt + "[locked]" ' append message
            ELSE ' check treasure locks
                IF TreasureRecord.Locked < False THEN ' check treasure lock
                    IF TreasureRecord.Closed > False THEN ' check treasure is closed
                        Outpt = Outpt + "[closed]" ' append message
                    END IF ' end check closed treasure
                END IF ' end check treasure lock
            END IF ' end check locked treasure
            IF TreasureRecord.Invisible THEN ' check treasure is invisible
                Outpt = Outpt + "[inv]" ' append message
            ELSE ' treasure is not invisible
                ' check invisible
                IF GetRoomRecordFlags(Array.Index) = Hidden.Object THEN
                    Outpt = Outpt + "[inv]" ' treasure flag, append message
                END IF ' end check invisible flag
            END IF ' end check invisible treasure
            IF GetRoomRecordFlags(Array.Index) = Magic.Trap THEN ' check magic trap
                Outpt = Outpt + "[trap]" ' treasure flag, append message
            END IF ' end check magic trap flag
            Outpt = Outpt + ", " ' append comma
            Object.Displayed = True ' set item displayed flag
        END IF ' end compare display flag
    END IF ' end compare treasure file record range
NEXT ' end loop through treasure
ContainerRec = RoomRecord.Container ' store room container into record
IF RTRIM$(ContainerRec.ShortName) <> Nul THEN ' compare container name length
    Carriage.Return = True ' set flag to disable return/linefeed
    CALL IO.O ' send previous string
    Items.Displayed = Items.Displayed + 1 ' increment number of items displayed
    IF Items.Displayed = 1 THEN ' check first item displayed
        IF UserRecord.Brief THEN ' check user is in brief mode
            Outpt = "You see " ' make initial string
            Carriage.Return = True ' set flag to disable return/linefeed
            CALL IO.O ' send first string
        END IF ' end check brief mode
    END IF ' end check first display item
    Outpt = RTRIM$(ContainerRec.ContainerName) ' store container name
    IF ContainerRec.Locked > False THEN ' check container is locked
        Outpt = Outpt + "[locked]" ' append message
    ELSE ' container is not locked
        IF ContainerRec.Closed > False THEN ' check container is closed
            Outpt = Outpt + "[closed]" ' append message
        END IF ' end check closed container
    END IF ' end check locked container
    IF ContainerRec.Keyed THEN ' check container has a key number
        Outpt = Outpt + "(#" + RIGHT$(STR$(ContainerRec.Keyed + 100000!), 5) + ")"
    END IF ' append key number to container name
    Outpt = Outpt + ", " ' append comma
    Object.Displayed = True ' set an item displayed flag
END IF ' end compare container name length
FOR Monster.Number = 1 TO Number.Monsters ' loop through all room monsters
    Carriage.Return = True ' set flag to disable return/linefeed
    CALL IO.O ' send previous string
    Items.Displayed = Items.Displayed + 1 ' increment number of items displayed
    IF Items.Displayed = 1 THEN ' check first item
        IF UserRecord.Brief THEN ' check user is in brief mode
            Outpt = "You see " ' make initial string
            Carriage.Return = True ' set flag to disable return/linefeed
            CALL IO.O ' send initial string
        END IF ' end check brief mode
    END IF ' end check first item
    Outpts = MonsterArray(Monster.Number).MonsterName ' store monster name
    Outpts = RTRIM$(Outpts) ' trim monster name
    CALL The.Or.An ' routine to get prefix (a, an, the)
    Outpt = Outpt + Prefix1 + Outpts + ", " ' make message of monster
    Object.Displayed = True ' set an item displayed flag
NEXT ' end loop through monsters in room
IF Object.Displayed = False THEN ' compare flag indicating an item is displayed
    IF UserRecord.Brief = False THEN ' check user is in brief mode
        Outpt = "nothing special." ' make last item string
    ELSE ' user is in brief mode
        Outpt = "You see nothing." ' make entire message
    END IF ' end check brief mode
ELSE ' an item was displayed
    IF Outpt <> Nul THEN ' compare string length
        Outpt = LEFT$(Outpt, LEN(Outpt) - 2) + "." ' trim comma, append period
        IF Items.Displayed > 1 THEN ' check more than one item was displayed
            Outpt = "and " + Outpt ' append to last item
        END IF ' end check item numbers
    END IF ' end check valid last item string
END IF ' end check flag of items displayed
CALL IO.O ' send last display item
END SUB ' end routine to show room description, etc.

REM * routine to display player character inventory.

SUB Display.Inventory
Treasure.Counter = False ' reset counter
FOR Array.Index = 1 TO 20 ' loop through all player treasure inventory
    Treasure.Number = GetUserRecordInv(Array.Index) ' get treasure number
    IF Treasure.Number THEN ' compare number
        Treasure.Counter = Treasure.Counter + 1 ' increment counter
    END IF ' end check compare
NEXT ' end loop counting
FOR Array.Index = 1 TO 20 ' loop through all player treasure inventory
    Treasure.Number = GetUserRecordObject(Array.Index) ' get treasure number
    IF Treasure.Number THEN ' compare number
        Treasure.Counter = Treasure.Counter + 1 ' increment counter
    END IF ' end check compare
NEXT ' end loop counting
Container.Count = False ' reset counter
FOR Array.Index = 1 TO 3 ' loop through all player character containers
    ' store container into record
    X = GetUserRecordContainer(Array.Index)
    ' compare container name length
    IF RTRIM$(ContainerRec.ShortName) <> Nul THEN
        Treasure.Counter = Treasure.Counter + 1 ' increment counter
        Container.Count = Container.Count + 1 ' increment counter
    END IF ' end check compare
NEXT ' end loop counting
Graphics.Off = True ' reset color
IF Treasure.Counter = 1 THEN ' check counter
    Outpt = "You are carrying" + STR$(Treasure.Counter) + " item" ' weight message
ELSE ' check counter
    Outpt = "You are carrying" + STR$(Treasure.Counter) + " items" ' weight message
END IF ' end check counter
Outpt = Outpt + " weighing" + STR$(Weight + Container.Count) + " pounds:" ' weight message
CALL IO.O ' send output message
Items.Displayed = False ' number of items displayed
FOR Array.Index = 1 TO 20 ' loop through all player treasure inventory
    Treasure.Number = GetUserRecordInv(Array.Index) ' get treasure number
    IF Treasure.Number THEN ' compare number
        Carriage.Return = True ' disable return/linefeed
        CALL IO.O ' send initial item
        CALL Read.Record(TreasureFile, Treasure.Number) ' get treasure record
        Outpts = TreasureRecord.TreasureName ' store treasure name
        Outpts = RTRIM$(Outpts) ' trim name
        IF TreasureRecord.Keyed THEN ' append key number to treasure name
            Outpts = Outpts + "(#" + RIGHT$(STR$(TreasureRecord.Keyed + 100000!), 5) + ")"
        END IF ' end compare treasure key number
        IF TreasureRecord.Plus THEN ' append plus number to treasure name
            Outpts = Outpts + "(+" + MID$(STR$(ABS(TreasureRecord.Plus)), 2) + ")"
        END IF ' end compare treasure plus
        IF TreasureRecord.Spell THEN ' append spell plus to treasure name
            CALL Read.Record(SpellFile, TreasureRecord.Spell) ' get spell record
            Outpts = Outpts + "(+" + MID$(STR$(SpellRecord.Level), 2) + ")"
        END IF ' end compare treasure spell plus
        IF TreasureRecord.LightType THEN ' check treasure is a light
            IF GetUserRecordCharges(Array.Index) < False THEN ' check light charges
                Outpts = Outpts + "[lit]" ' append message
            END IF ' end check charges
        END IF ' end check light
        IF TreasureRecord.Locked > False THEN ' check treasure is locked
            Outpts = Outpts + "[locked]" ' append message
        ELSE ' check treasure locks
            IF TreasureRecord.Locked < False THEN ' check treasure lock
                IF TreasureRecord.Closed > False THEN ' check treasure is closed
                    Outpts = Outpts + "[closed]" ' append message
                END IF ' end check closed treasure
            END IF ' end check treasure lock
        END IF ' end check locked treasure
        IF TreasureRecord.Invisible THEN ' check treasure is invisible
            Outpts = Outpts + "[inv]" ' append message
        END IF ' end check invisible
        Outpt = Outpts + ", " ' append comma
        Items.Displayed = Items.Displayed + 1 ' increment item number
        IF Items.Displayed = 1 THEN ' check first item
            MID$(Outpt, 1, 1) = UCASE$(MID$(Outpt, 1, 1)) ' uppercase first letter
        END IF ' end check first item
    END IF ' end compare treasure number
NEXT ' end loop through inventory treasure
FOR Array.Index = 1 TO 20 ' loop through player object inventory
    Object.Number = GetUserRecordObject(Array.Index) ' get object number
    IF Object.Number THEN ' compare object number
        Carriage.Return = True ' disable return/linefeed
        CALL IO.O ' send previous item
        CALL Read.Record(ObjectFile, Object.Number) ' get object record
        Outpts = ObjectRecord.ObjectName ' store object name
        Outpts = RTRIM$(Outpts) ' trim object name
        IF ObjectRecord.Invisible THEN ' compare object is invisible
            Outpts = Outpts + "[inv]" ' append to object name
        END IF ' end compare object invisible
        IF ObjectRecord.Keyed THEN ' compare object key, append number to name
            Outpts = Outpts + "(#" + RIGHT$(STR$(ObjectRecord.Keyed + 100000!), 5) + ")"
        END IF ' end compare object key number
        Outpt = Outpts + ", " ' append comma
        Items.Displayed = Items.Displayed + 1 ' increment item number
        IF Items.Displayed = 1 THEN ' check first item
            MID$(Outpt, 1, 1) = UCASE$(MID$(Outpt, 1, 1)) ' uppercase first letter
        END IF ' end check first item
    END IF ' end check object number
NEXT ' end loop through object inventory
FOR Array.Index = 1 TO 3 ' loop through all player character containers
    ' store container into record
    X = GetUserRecordContainer(Array.Index)
    ' compare container name length
    IF RTRIM$(ContainerRec.ShortName) <> Nul THEN
        Carriage.Return = True ' disable retuen/linefeed
        CALL IO.O ' send previous item
        Outpt = RTRIM$(ContainerRec.ContainerName) ' store container name
        IF ContainerRec.Locked > False THEN ' check container locked
            Outpt = Outpt + "[locked]" ' append message
        ELSE ' container not locked
            IF ContainerRec.Closed > False THEN ' check container closed
                Outpt = Outpt + "[closed]" ' append message
            END IF ' end check closed container
        END IF ' end check locked container
        IF ContainerRec.Keyed THEN ' check container key number
            Outpt = Outpt + "(#" + RIGHT$(STR$(ContainerRec.Keyed + 100000!), 5) + ")"
        END IF ' append container key number to message
        Outpt = Outpt + ", " ' append comma
        Items.Displayed = Items.Displayed + 1 ' increment item number
        IF Items.Displayed = 1 THEN ' check first item
            MID$(Outpt, 1, 1) = UCASE$(MID$(Outpt, 1, 1)) ' uppercase first letter
        END IF ' end check first item
    END IF ' end compare container name length
NEXT ' end loop through player containers
IF Items.Displayed = False THEN ' compare items displayed
    Outpt = "Nothing at all." ' display message
ELSE ' some items displayed
    Outpt = LEFT$(Outpt, LEN(Outpt) - 2) + "." ' trim comma, append period
    IF Items.Displayed > 1 THEN ' check more than one item displayed
        Outpt = "and " + Outpt ' add to message
    END IF ' end check item numbers
END IF ' end compare items
CALL IO.O ' send last item
END SUB ' end routine to display player inventory

REM * routine to sell item of treasure, object, or container.

SUB Pawn.Shop
Sell.Container = False ' item to sell is container flag
CALL Check.Inventory.Treasure ' routine to find treasure name
IF Index.Number = False THEN ' check treasure index
    CALL Num ' reduce counter
    CALL Check.Inventory.Container ' routine to find container name
    Sell.Container = True ' set container for sale flag
    Container.Number = Array.Number ' store container number
    IF Index.Number = False THEN ' check treasure index
        Outpt = "The Broker says: You can't sell that!" ' make error message
        CALL IO.O ' send error message
        EXIT SUB ' exit routine
    END IF ' end check index
END IF ' end check index
Outpts = TreasureRecord.ShortName ' store treasure name being sold
IF Sell.Container THEN ' check container flag
    Item.Cost# = 0# ' reset price of container contents
    Outpts = ContainerRec.ShortName ' store container name
    IF ContainerRec.Locked > False THEN ' check container lock
        Outpt = "The Broker tries the lock and says: Arrghh!! Can't open it!"
        CALL IO.O ' send lock message
        EXIT SUB ' exit routine
    END IF ' end check container lock
    FOR Array.Index = 1 TO 5 ' loop through container contents
        ' store treasure number
        Treasure.Number = GetContainerRecInventory(Array.Index)
        ' verify file bounds
        IF Treasure.Number > False AND Treasure.Number < LOF(TreasureFile) / LEN(TreasureRecord) THEN
            CALL Read.Record(TreasureFile, Treasure.Number) ' get treasure record
            ' verify item plus is not loadable or launchable object
            IF TreasureRecord.Loadable = False AND TreasureRecord.Launchable = False THEN
                ' check item plus or spell
                IF TreasureRecord.Plus OR TreasureRecord.Spell THEN
                    ' compare charges
                    IF GetContainerRecCharges(Array.Index) = False THEN
                        Item.Cost# = Item.Cost# + 1# ' add 1 gold for discharged item
                    ELSE ' compare charges
                        ' add treasure gold
                        Item.Cost# = Item.Cost# + CDBL(INT(TreasureRecord.Gold * .95))
                    END IF ' end compare charges
                ELSE ' compare item plus or spell
                    ' add treasure gold
                    Item.Cost# = Item.Cost# + CDBL(INT(TreasureRecord.Gold * .9))
                END IF ' end compare item plus or spell
            ELSE ' compare item plus type
                ' add treasure gold
                Item.Cost# = Item.Cost# + CDBL(INT(TreasureRecord.Gold))
            END IF ' end check item plus type
        END IF ' end verify file bounds
    NEXT ' end loop through container contents
    IF Item.Cost# = 0# THEN ' compare container content price
        Item.Cost# = 1# ' reset price
        Sell.Container = 1 ' sell the container
    ELSE ' compare contents
        Outpts = "contents of the " + Outpts ' make purchase message
    END IF ' end compare container contents
ELSE ' check container
    Item.Cost# = CDBL(INT(TreasureRecord.Gold * .9)) ' calculate price of treasure item
    ' verify item plus is not loadable or launchable object
    IF TreasureRecord.Loadable = False AND TreasureRecord.Launchable = False THEN
        IF TreasureRecord.Plus OR TreasureRecord.Spell THEN ' verify charges
            IF GetUserRecordCharges(Array.Number) = False THEN ' compare charges
                Item.Cost# = 1# ' set price to 1
                Outpts = "broken " + Outpts ' make broken item message
            END IF ' end compare charges
        END IF ' end compare charges left
    END IF ' end verify charges
END IF ' end check container being sold
Outpt = "The Broker says: Well, I'll give you" + STR$(Item.Cost#) + " gold for the " + RTRIM$(LCASE$(Outpts)) + "." ' message of price for items
CALL IO.O ' send purchase message
Outpt = "The Broker asks: Is that a deal(y/n)? " ' make input prompt
No.Input.Out = "Y" ' default input
CALL IO.I ' get input
IF Yes THEN ' compare response
    UserRecord.Gold = UserRecord.Gold + Item.Cost# ' increment player gold
    Outpt = "The Broker says: Great!" ' make purchase message
    IF Sell.Container = True THEN ' check container
        ' sell contents of container
        FOR Container.Item = 1 TO 5 ' loop through container contents
            ' routine resets container item
            CALL Clear.Container(Container.Item, False)
        NEXT ' end loop through container contents
        ' store container record
        CALL PutUserRecordContainer(Container.Number)
    ELSE ' check container
        ' sell container
        IF Sell.Container = 1 THEN ' check container
            CALL Clear.Container(0, True) ' clear player container structure
            CALL PutUserRecordContainer(Container.Number) ' clear container record
        ELSE ' check sell item
            ' routine removes item from player
            CALL Discard.Inventory(Array.Number, True)
        END IF ' end check sell item/container
    END IF ' end check container sold
ELSE ' compare repsonse
    Outpt = "The Broker says: Oh well!" ' make purchase message
END IF ' end compare response
CALL IO.O ' send purchase message
END SUB ' end routine to sell items

REM * routine for monsters to talk.
REM * input variables:
REM *   Parsed.Command1 - command parameter of monster name.

SUB Talk.To.Monster
CALL Check.Monster ' get monster number
IF Monster.Number = False THEN ' check monster found
    Outpt = "You can't talk to that!" ' make message
    CALL IO.O ' send error message
    EXIT SUB ' exit routine
END IF ' end check monster number
' get random number of monster variable of monster talk record file
Talk.Number = GetMonsterRecordTalk(INT(RND * 5 + 1))
Outpt = Nul ' reset talk response string
' check bounds
IF Talk.Number > False AND Talk.Number <= LOF(MonTalkFile) / LEN(MonsterTalkRecord) THEN
    CALL Read.Record(MonTalkFile, Talk.Number) ' get the random talk string
    Outpt = RTRIM$(MonsterTalkRecord.TalkMessage) ' trim talk response
END IF ' end check monster talk file bounds
IF Outpt = Nul THEN ' compare to length of string
    Outpt = "The monster doesn't reply!" ' set string
END IF ' end check string length
CALL IO.O ' send response
CALL Read.Room.Record(Room) ' get current room record
' compare monster number to room record talk action number
Action.Number = RoomRecord.Action ' store room action number
' check file bounds
IF Action.Number > False AND Action.Number <= LOF(ActionFile) / LEN(ActionRecord) THEN
    CALL Read.Record(ActionFile, Action.Number) ' read action record
    IF MonsterIndex(Monster.Number) = ActionRecord.MonsterTalk THEN ' compare
        Action1$ = "As you begin to speak," ' format action string one
        Action2$ = "The monster hits you for" ' format action string two
        ' routine to activate actions by specific trigger
        CALL Actions(Action1$, Action2$)
    END IF ' end compare room monster talk action
END IF ' end check file bounds
END SUB ' end routine to talk to monster

REM * routine to rust some item of player.
REM * input variables:
REM *   Room.Rust.Rate - number of rounds for rusting.
REM *   Rust.Rate - counter of rounds for rusting.

SUB Rust.Weapon
IF Room.Rust.Rate = False THEN ' check room has rust rate
    EXIT SUB ' exit routine
END IF ' end check room rust rate
Rust.Rate = Rust.Rate + 1 ' increment rust rate counter
IF Rust.Rate < Room.Rust.Rate THEN ' check rust rate counter
    EXIT SUB ' exit routine
END IF ' end check rust rate counter
Rust.Rate = False ' reset rust rate counter
CALL Rust.Inventory(Weapon4) ' routine to rust armor
CALL Rust.Inventory(Weapon5) ' routine to rust shield
CALL Rust.Inventory(Weapon6) ' routine to rust weapon
CALL Rust.Inventory(Weapon7) ' routine to rust ring
END SUB ' end rust routine

REM * routine to rust item.
REM * input variables:
REM *   Weapon.Number - array index to treasure file record.

SUB Rust.Inventory (Weapon.Number)
IF Weapon.Number > False THEN ' check treasure index
    ' store index to treasure file
    Treasure.Number = GetUserRecordInv(Weapon.Number)
    IF Treasure.Number > False AND Treasure.Number <= LOF(TreasureFile) / LEN(TreasureRecord) THEN 'compare bounds
        CALL Read.Record(TreasureFile, Treasure.Number) ' get treasure record
        IF TreasureRecord.Rustable = True THEN ' verify item can rust
            IF RND < (TreasureRecord.RustPercent / 100) THEN ' check rust percent
                Outpt = TreasureRecord.ShortName ' store treasure name
                Outpt = RTRIM$(Outpt) ' trim name
                Outpt = LCASE$(Outpt) ' lowercase name
                Outpt = "Your " + Outpt + " rusts!" ' make message
                CALL IO.O ' send rust message
                ' remove item from inventory
                CALL Discard.Inventory(Weapon.Number, True)
            END IF ' end check random rust percent
        END IF ' end verify item rusts
    END IF ' end compare file bounds
END IF ' end check treasure index
END SUB ' end routine to rust items

REM * routine for monster to steal item from player inventory.
REM * input variables:
REM *   Room.Steal.Rate - number of rounds for stealing.
REM *   Steal.Rate - counter of rounds for stealing.

SUB Steal.Treasure
IF Room.Steal.Rate = False THEN ' check room has steal rate
    EXIT SUB ' exit routine
END IF ' end check room steal rate
Steal.Rate = Steal.Rate + 1 ' increment steal rate counter
IF Steal.Rate < Room.Steal.Rate THEN ' compare steal rates
    EXIT SUB ' exit routine
END IF ' end compare steal rates
Steal.Rate = False ' reset steal rate counter
IF Number.Monsters = False THEN ' check monsters in room
    EXIT SUB ' exit routine
END IF ' end compare monsters in room
FOR Array.Index = 1 TO 20 ' loop through player inventory
    IF GetUserRecordInv(Array.Index) THEN ' check player has inventory
        CALL Read.Record(TreasureFile, GetUserRecordInv(Array.Index)) ' get record
        IF TreasureRecord.Stealable = True THEN ' verify item can be stolen
            ' check random percentage item can be stolen
            IF RND < (TreasureRecord.StealPercent / 100) THEN
                FOR Monster.Number = 1 TO Number.Monsters ' loop through monsters
                    ' check permanent
                    IF MonsterArray(Monster.Number).Permanent = False THEN
                        FOR Array.Number = 1 TO 5 ' loop through monsters inventory
                            ' locate empty inventory in monster
                            IF GetMonsterRecordTreasure(Array.Number) = False THEN
                                ' store player inventory in monster inventory
                                CALL PutMonsterRecordTreasure(Array.Number, GetUserRecordInv(Array.Index)) ' store
                                ' monster name
                                Outpt = MonsterArray(Monster.Number).MonsterName
                                Outpt = RTRIM$(Outpt) ' trim name
                                Inpt = TreasureRecord.ShortName ' store treasure name
                                Inpt = RTRIM$(Inpt) ' trim name
                                Inpt = LCASE$(Inpt) ' lowercase name
                                Outpt = "The " + Outpt + " steals your " + Inpt + "!" ' message
                                CALL IO.O ' send stolen message
                                ' remove inventory from player
                                CALL Discard.Inventory(Array.Index, True) ' remove
                                EXIT SUB ' exit routine
                            END IF ' end locate emtpt monster inventory
                        NEXT ' end loop through monster inventory array
                    END IF ' end check permanent monster
                NEXT ' end loop through monsters in room
            END IF ' end random percentage for stealing inventory
        END IF ' end verify item can be stolen
    END IF ' end check player inventory
NEXT ' end loop through player inventory
END SUB ' end routine to steal item from player

REM * routine to offer treasure or gold to a monster to leave the room.
REM * processing variables:
REM *   Monster.Number - number of monster.
REM *   UserRecord.Gold - for offering gold.

SUB Offer
CALL ParseX ' get first command parameter
IF Parser = False THEN ' check parameter
    Outpt = "Offer to whom?" ' make message
    CALL IO.O ' send error message
    EXIT SUB ' exit routine
END IF ' end check parameter
CALL Numeric ' parse parameter number from # sign
CALL Check.Monster ' get monster number
IF Monster.Number = False THEN ' check monster
    Outpt = "That's not here!" ' make message
    CALL IO.O ' send error message
    EXIT SUB ' exit routine
END IF ' end check monster number
Parsed.Command1 = Parsed.Command2 ' get second parameter
CALL Numeric ' parse parameter number from # sign
Accept.Offer = False ' flag for monster to accept offer
Player.Gold# = False ' stores amount of gold offered
IF MonsterArray(Monster.Number).Magic = False THEN ' check magical monster
    IF MonsterArray(Monster.Number).Permanent = False THEN ' permanent monster
        CALL Check.Inventory.Treasure ' parameter is item of player inventory
        IF Index.Number THEN ' player offers item of inventory
            CALL TreasureCharges(Charges.Amount) ' get treasure item charges
            IF Charges.Amount > False THEN ' compare remaining charges
                IF TreasureRecord.Spell THEN ' check item is magical
                    CALL Read.Record(SpellFile, TreasureRecord.Spell) ' get record
                    IF SpellRecord.Level >= 10 THEN ' verify item spell level
                        Accept.Offer = True ' set accept flag
                    END IF ' end compare spell level
                ELSE ' check magical item
                    IF TreasureRecord.Plus >= 10 THEN ' compare plus of item
                        ' set accept flag for plus greater than ten
                        Accept.Offer = True
                    END IF ' end check item plus
                END IF ' end check magical item
            END IF ' end compare charges
        ELSE ' compare parameter to offer
            ' convert offer to integer
            Offer.Amount# = CDBL(INT(VAL(Parsed.Command1)))
            IF Offer.Amount# > False THEN ' check value of offer
                ' compare value offered is greater than monster level
                Monster.Amount# = CDBL(INT(MonsterArray(Monster.Number).Level * 10))
                IF Offer.Amount# >= Monster.Amount# THEN ' compare
                    ' compute player gold
                    Player.Gold# = UserRecord.Gold - Offer.Amount#
                    IF Player.Gold# >= False THEN ' check player gold
                        Accept.Offer = True ' set accept flag
                    END IF ' end compare player has gold
                END IF ' end compare accept gold value offer
            END IF ' end check offer value
        END IF ' end item offered
    END IF ' end check parmanent monster
END IF ' end check magical monster
IF Accept.Offer = False THEN ' compare accept flag
    Outpt = "The monster ignores your offer!" ' make message
    CALL IO.O ' send message
    EXIT SUB ' exit routine
END IF ' end compare accept flag
CALL Remove.Monster ' routine to take monster out of array
IF Player.Gold# THEN ' check gold offered
    UserRecord.Gold = Player.Gold# ' decrement player gold value
    Outpt = "The monster takes your offer and leaves!" ' make message
    CALL IO.O ' send accept message
    EXIT SUB ' exit routine
END IF ' end check gold offered
CALL Discard.Inventory(Array.Number, True) ' remove item from player inventory
Outpt = "The monster trades with you and leaves!" ' make message
CALL IO.O ' send accept message
END SUB ' end routine to offer to monster

REM * routine to determine room direction for player to move to in panic.

SUB Panic
Directions = False ' room direction counter
FOR Direction.Number = 1 TO 12 ' loop through all room directions
    IF GetRoomRecordDirect(Direction.Number) THEN ' get next room direction
        CALL Restrict(Direction.Number, Restricted) ' routine checks valid room
        IF Restricted = False THEN ' compare restriction flag
            Directions = Directions + 1 ' increment direction counter
        END IF ' end compare restricted room
    END IF ' end check next room direction
NEXT ' end loop through room directions
IF Directions = False THEN ' compare direction counter
    Outpt = "There is nowhere to run! Try Appeal.." ' make panic error message
    CALL IO.O ' send panic error message
    EXIT SUB ' exit routine
END IF ' end compare direction counter
' calculate random number of directions to search from direction counter
New.Direction = INT(RND * Directions + 1) ' calculate
Direction.Counter = False ' reset search counter
FOR Direction.Number = 1 TO 12 ' loop through room directions
    IF GetRoomRecordDirect(Direction.Number) THEN ' compare room direction
        CALL Restrict(Direction.Number, Restricted) ' check room restriction
        IF Restricted = False THEN ' check restriction flag
            Direction.Counter = Direction.Counter + 1 ' increment search counter
            IF Direction.Counter = New.Direction THEN ' compare counters
                ' make panic message
                Outpt = "You run away like a screaming madman!"
                CALL IO.O ' send panic message
                CALL Fumble ' routine for inventory fumble
                ' store next room direction
                Next.Room = GetRoomRecordDirect(Direction.Number)
                CALL Enter.Room ' routine to move player to room
                EXIT SUB ' exit loop through room directions
            END IF ' end compare counters
        END IF ' end check restriction flag
    END IF ' end compare room direction
NEXT ' end loop through room directions
Outpt = "Nothing happens.." ' default message
CALL IO.O ' send message
END SUB ' end routine for player panic

REM * routine to move player with low statistics or in a room with no exits.

SUB Appeal
IF UserRecord.Fatigue < (UserRecord.FatigueMax * .25) THEN ' compare stats
    Outpt = "Your fatigue is very low.." ' make message
    CALL IO.O ' send message
    Outpt = "You are teleported elsewhere!" ' make teleport message
    CALL IO.O ' send teleport message
    CALL Read.Room.Record(1!) ' get room record #1
    Resurrection.Room = RoomRecord.MonsterClass ' get resurrection room number
    Next.Room = CSNG(Resurrection.Room) ' store resurrection room
    IF Next.Room = 0! THEN ' check room number
        Next.Room = 1! ' store new room number
    END IF ' end check room
    Teleported = True ' set teleporting flag
    CALL Enter.Room ' routine moves player to room
    EXIT SUB ' exit routine
END IF ' end compare low statistics
FOR Direction.Number = 1 TO 12 ' loop through room directions
    IF GetRoomRecordDirect(Direction.Number) THEN ' compare room direction
        CALL Restrict(Direction.Number, Restricted) ' routine checks valid room
        IF Restricted = False THEN ' compare restriction flag
            Outpt = "There are exits in the room!" ' make error message
            CALL IO.O ' send error message
            EXIT SUB ' exit routine
        END IF ' end compare restriction flag
    END IF ' end compare room directions
NEXT ' end loop through room directions
FOR Array.Index = 1 TO 20 ' loop through room objects
    Object.Number = GetRoomRecordObject(Array.Index) ' get object record
    ' file bounds
    IF Object.Number > False AND Object.Number <= LOF(ObjectFile) / LEN(ObjectRecord) THEN
        CALL Read.Record(ObjectFile, Object.Number) ' get object record
        IF ObjectRecord.RoomLink > False THEN ' compare object room number
            IF ObjectRecord.JailTrap = False THEN ' check object jail trap
                CALL Restrict(12, Restricted) ' routine checks valid room to go to
                IF Restricted = False THEN ' compare restriction flag
                    Outpt = "There are objects with exits in the room!" ' make error message
                    CALL IO.O ' send error message
                    EXIT SUB ' exit routine
                END IF ' end check restriction flag
            END IF ' end check object jail trap
        END IF ' end compare object room link number
    END IF ' end check file bounds
NEXT ' end loop through objects in room
Outpt = "You are teleported elsewhere!" ' make teleport message
CALL IO.O ' send teleport message
CALL Read.Room.Record(1!) ' get room record #1
Resurrection.Room = RoomRecord.MonsterClass ' get resurrection room number
Next.Room = CSNG(Resurrection.Room) ' store resurrection room
IF Next.Room = 0! THEN ' check room number
    Next.Room = 1! ' store new room number
END IF ' end check room
Number.Monsters = False ' reset number of monsters
Teleported = True ' set teleporting flag
CALL Enter.Room ' routine moves player to room
END SUB ' end routine to appeal player to new room

REM * routine to calculate bank interest.

SUB Bank.Interest
Outpt = UserRecord.DateOn ' get player last date online
CALL Decrypt(Outpt) ' decrypt date
DATE1$ = LEFT$(Outpt, 10) ' store date
IF ValidDate(DATE1$) THEN ' compare date validity
    IF DATE$ <> DATE1$ THEN ' compare date to today
        ' calculate difference
        Interest.Days = Difference(DATE1$, DATE$)
        IF Interest.Days > False THEN ' check difference of dates
            IF UserRecord.Bank > False THEN ' verify players bank balance
                Total.Interest# = False ' reset total interest
                ' loop through days since last online
                FOR Total.Days = 1 TO Interest.Days
                    ' compute interest for each day
                    Interest# = INT(UserRecord.Bank * .065)
                    ' add interest to account
                    UserRecord.Bank = UserRecord.Bank + Interest#
                    Total.Interest# = Total.Interest# + Interest# ' add interest to total
                NEXT ' end loop through dates
                IF Total.Interest# > False THEN ' compare total interest
                    ' make new balance message
                    Outpt = "The bank reports interest posting of" + STR$(Total.Interest#) + " gold to your account!"
                    CALL IO.O ' send display message
                END IF ' end compare interest
            END IF ' end verify bank balance
            IF UserRecord.Borrow > False THEN ' verify players bank loan
                Total.Interest# = False ' reset total interest
                ' loop through days since last online
                FOR Total.Days = 1 TO Interest.Days
                    ' compaute interest for each day
                    Interest# = INT(UserRecord.Borrow * .075)
                    ' add interest to loan
                    UserRecord.Borrow = UserRecord.Borrow + Interest#
                    Total.Interest# = Total.Interest# + Interest# ' add interest to total
                NEXT ' end loop through dates
                IF Total.Interest# > False THEN ' compare total interest
                    ' make new balance message
                    Outpt = "The bank reports interest posting of" + STR$(Total.Interest#) + " gold to your loan!"
                    CALL IO.O ' send display message
                END IF ' end compare interest
            END IF ' end verify loan balance
        END IF ' end check difference of dates
    END IF ' end compare todays date
END IF ' end compare validity
Outpt = DATE$ ' store todays date
CALL Valid(Outpt, 10) ' validate date string
CALL Encrypt(Outpt, True) ' encrypt date string
UserRecord.DateOn = Outpt ' store new date in player record
CALL Share.Record(UserFile, User.Index) ' routine to write player record
END SUB ' end routine to calculate bank balances

REM * get days difference between two dates.

FUNCTION Difference (startdate$, enddate$)

Year1 = VAL(MID$(startdate$, 7, 4))
Year2 = VAL(MID$(enddate$, 7, 4))

dayselapsed1 = 0
FOR x = 1980 TO Year1 - 1
    dayselapsed1 = dayselapsed1 + 365 + IsLeapYear(x)
NEXT

Month1 = VAL(MID$(startdate$, 1, 2))
Month2 = VAL(MID$(enddate$, 1, 2))

FOR x = 1 TO Month1 - 1
    SELECT CASE x
        CASE 1
            dayselapsed1 = dayselapsed1 + 31
        CASE 2
            dayselapsed1 = dayselapsed1 + 28 + IsLeapYear(Year1)
        CASE 3
            dayselapsed1 = dayselapsed1 + 31
        CASE 4
            dayselapsed1 = dayselapsed1 + 30
        CASE 5
            dayselapsed1 = dayselapsed1 + 31
        CASE 6
            dayselapsed1 = dayselapsed1 + 30
        CASE 7
            dayselapsed1 = dayselapsed1 + 31
        CASE 8
            dayselapsed1 = dayselapsed1 + 31
        CASE 9
            dayselapsed1 = dayselapsed1 + 30
        CASE 10
            dayselapsed1 = dayselapsed1 + 31
        CASE 11
            dayselapsed1 = dayselapsed1 + 30
        CASE 12
            dayselapsed1 = dayselapsed1 + 31
    END SELECT
NEXT

Day1 = VAL(MID$(startdate$, 4, 2))
Day2 = VAL(MID$(enddate$, 4, 2))

dayselapsed1 = dayselapsed1 + Day1

dayselapsed2 = 0
FOR x = 1980 TO Year2 - 1
    dayselapsed2 = dayselapsed2 + 365 + IsLeapYear(x)
NEXT

FOR x = 1 TO Month2 - 1
    SELECT CASE x
        CASE 1
            dayselapsed2 = dayselapsed2 + 31
        CASE 2
            dayselapsed2 = dayselapsed2 + 28 + IsLeapYear(Year2)
        CASE 3
            dayselapsed2 = dayselapsed2 + 31
        CASE 4
            dayselapsed2 = dayselapsed2 + 30
        CASE 5
            dayselapsed2 = dayselapsed2 + 31
        CASE 6
            dayselapsed2 = dayselapsed2 + 30
        CASE 7
            dayselapsed2 = dayselapsed2 + 31
        CASE 8
            dayselapsed2 = dayselapsed2 + 31
        CASE 9
            dayselapsed2 = dayselapsed2 + 30
        CASE 10
            dayselapsed2 = dayselapsed2 + 31
        CASE 11
            dayselapsed2 = dayselapsed2 + 30
        CASE 12
            dayselapsed2 = dayselapsed2 + 31
    END SELECT
NEXT

dayselapsed2 = dayselapsed2 + Day2
Difference = dayselapsed2 - dayselapsed1
END FUNCTION

REM * function returns 1 if year is leap year.

FUNCTION IsLeapYear (Var)
leap = 0
IF (Var MOD 4) = 0 THEN
    leap = -1
END IF
IF (Var MOD 100) = 0 THEN
    leap = 0
END IF
IF (Var MOD 400) = 0 THEN
    leap = -1
END IF
IF leap THEN
    IsLeapYear = 1
END IF
END FUNCTION

' Def FNclock$=Format$(Now,"ddd dd mmm yy\, hh:mma/p")
FUNCTION clock$
' mm-dd-yyyy hh:mm:ss

M = VAL(LEFT$(DATE$, 2))
D = VAL(MID$(DATE$, 4, 2))
Y = VAL(RIGHT$(DATE$, 2))
IF M < 3 THEN M = M + 12: Y = Y - 1
W = ((13 * M + 3) \ 5 + D + Y + Y \ 4 - Y \ 100 + Y \ 400 + 1) MOD 7
clock$ = MID$("SunMonTueWedThuFriSat", W * 3 + 1, 3)
clock$ = clock$ + " " + MID$(DATE$, 4, 2) ' dd

T = (VAL(MID$(DATE$, 1, 2)) - 1) * 3 + 1
clock$ = clock$ + " " + MID$("JanFebMarAprMayJunJulAugSepOctNovDec", T, 3)

clock$ = clock$ + " " + MID$(DATE$, 9, 2) ' yy
T = VAL(LEFT$(TIME$, 2))
IF T > 11 THEN T = T - 12
IF T = 0 THEN T = 12
clock$ = clock$ + ", " + RIGHT$("00" + LTRIM$(STR$(T)), 2) ' hh
clock$ = clock$ + ":" + MID$(TIME$, 4, 2) ' mm

T = VAL(LEFT$(TIME$, 2))
IF T < 12 THEN
    clock$ = clock$ + "a"
ELSE
    clock$ = clock$ + "p"
END IF
END FUNCTION

REM * routine to process bank commands.

SUB Bank
CALL Put.User.Record ' routine to write user record
DO ' loop through bank menu
    Graphics.Off = False ' reset color
    Outpt = "Bank menu options:" ' make display message
    CALL IO.O ' send message
    Graphics.Off = True ' reset color
    Outpt = "[A]ccount" ' make display message
    CALL IO.O ' send message
    Outpt = "[B]orrow" ' make display message
    CALL IO.O ' send message
    Outpt = "[D]eposit" ' make display message
    CALL IO.O ' send message
    Outpt = "[P]ayback" ' make display message
    CALL IO.O ' send message
    Outpt = "[T]ransfer" ' make display message
    CALL IO.O ' send message
    Outpt = "[W]ithdraw" ' make display message
    CALL IO.O ' send message
    Graphics.Off = False ' reset color
    Outpt = "The broker asks: What can I do for you(q to quit)? " ' input prompt
    No.Input.Out = "Q" ' default input
    CALL IO.I ' get user input
    SELECT CASE UCASE$(Inpt) ' selection of bank option
        CASE "A" ' option to display bank balances
            Outpt = "He fumbles with his ledgers and says:" ' make display message
            CALL IO.O ' send message
            Graphics.Off = True ' reset color
            Outpt = "The current account interest rate is 6.5 percent." ' message
            CALL IO.O ' send message
            Outpt = "The current lending interest rate is 7.5 percent." ' message
            CALL IO.O ' send message
            ' make display message
            Outpt = "You have " + FormatString$(UserRecord.Gold) + " gold and " + FormatString$(UserRecord.Bank) + " in the bank."
            CALL IO.O ' send message
            Outpt = "You have borrowed " + FormatString$(UserRecord.Borrow) + " gold from the bank." ' make display message
            CALL IO.O ' send message
        CASE "B" ' option to borrow gold from bank
            Outpt = "How much will you borrow(1-32767)? " ' make input prompt
            CALL IO.I ' get user input
            Borrow.Amount# = CDBL(INT(VAL(Inpt))) ' convert input to double
            ' compare input range
            IF Borrow.Amount# <= False OR Borrow.Amount# > MaxInt THEN
                Outpt = "The broker says: You can't borrow that much!" ' make message
                CALL IO.O ' send message
            ELSE ' check input range
                Outpt = "The broker hands you the gold." ' make display message
                CALL IO.O ' send message
                ' add gold to player gold
                UserRecord.Gold = UserRecord.Gold + Borrow.Amount#
                ' add gold to player loan
                UserRecord.Borrow = UserRecord.Borrow + Borrow.Amount#
            END IF ' end check input range
        CASE "D" ' option to deposit gold in bank
            IF UserRecord.Gold <= False THEN ' check player gold
                Outpt = "You have no gold to deposit!" ' make display message
                CALL IO.O ' send message
            ELSE ' check player gold
                Outpt = "How much(1-" + MID$(STR$(UserRecord.Gold), 2) + ")? " ' prompt
                CALL IO.I ' get user input
                Deposit.Amount# = CDBL(INT(VAL(Inpt))) ' convert input to double
                ' compare input range
                IF Deposit.Amount# <= False OR Deposit.Amount# > UserRecord.Gold THEN
                    Outpt = "You don't have that much gold!" ' make display message
                    CALL IO.O ' send message
                ELSE ' check input range
                    ' add gold to bank account
                    UserRecord.Bank = UserRecord.Bank + Deposit.Amount#
                    ' subtract gold from player
                    UserRecord.Gold = UserRecord.Gold - Deposit.Amount#
                    Outpt = "You hand him the gold." ' make display message
                    CALL IO.O ' send message
                END IF ' end check input range
            END IF ' end check player gold
        CASE "P" ' option to pay back loan
            IF UserRecord.Borrow <= False THEN ' check player loan
                Outpt = "You don't have any loan with the bank!" ' make message
                CALL IO.O ' send message
            ELSE ' check loan
                IF UserRecord.Bank > False THEN ' check player bank account
                    Outpt = "The broker asks: Pay back with your current bank account? "
                    No.Input.Out = "Y" ' default input
                    CALL IO.I ' get user input
                    IF Yes THEN ' check input response
                        ' compute difference between player bank account and loan
                        IF UserRecord.Borrow - UserRecord.Bank <= False THEN ' compute
                            ' subtract loan from player bank account
                            UserRecord.Bank = UserRecord.Bank - UserRecord.Borrow ' subtract
                            UserRecord.Borrow = False ' reset loan amount
                            Outpt = "The broker says: Your loan is paid off!" ' message
                            CALL IO.O ' send message
                        ELSE ' check computation
                            ' subtract player bank account from loan
                            UserRecord.Borrow = UserRecord.Borrow - UserRecord.Bank
                            UserRecord.Bank = False ' reset player bank account
                            Outpt = "The broker says: Your balance paid part of the loan!"
                            CALL IO.O ' send display message
                        END IF ' end check loan/bank account difference
                    END IF ' end check user input
                END IF ' end check player bank account
                IF UserRecord.Borrow > False THEN ' check player loan amount
                    IF UserRecord.Gold > False THEN ' check player gold
                        Outpt = "The broker asks: Pay back with your current gold? "
                        No.Input.Out = "Y" ' default input
                        CALL IO.I ' get user input
                        IF Yes THEN ' check input response
                            ' compute difference between player gold and loan
                            IF UserRecord.Borrow - UserRecord.Gold <= False THEN ' compute
                                ' subtract loan from player gold
                                UserRecord.Gold = UserRecord.Gold - UserRecord.Borrow
                                UserRecord.Borrow = False ' reset loan
                                Outpt = "The broker says: Your loan is paid off!" ' message
                                CALL IO.O ' send message
                            ELSE ' check computation
                                ' subtract player gold from loan
                                UserRecord.Borrow = UserRecord.Borrow - UserRecord.Gold
                                UserRecord.Gold = False ' reset player gold
                                Outpt = "The broker says: Your gold paid part of the loan!"
                                CALL IO.O ' send display message
                            END IF ' end loan/player gold difference
                        END IF ' end check user input
                    END IF ' end check player gold
                END IF ' end check player loan
            END IF ' end check player loan
        CASE "T" ' option to tranfer bank funds to another player
            IF UserRecord.Bank <= False THEN ' check player bank account
                Outpt = "You have nothing in your account to transfer!" ' make message
                CALL IO.O ' send message
            ELSE ' check player bank account
                Outpt = "How much(1-" + MID$(STR$(UserRecord.Bank), 2) + ")? " ' prompt
                CALL IO.I ' get user input
                Transfer.Amount# = CDBL(INT(VAL(Inpt))) ' convert input to double
                ' check input range
                IF Transfer.Amount# <= False OR Transfer.Amount# > UserRecord.Bank THEN
                    Outpt = "You don't have that much gold in the bank!" ' make message
                    CALL IO.O ' send message
                ELSE ' check input range
                    CALL Share.Record(UserFile, User.Index) ' write current user record
                    Outpt = "Transfer to what player? " ' make input prompt
                    CALL IO.I ' get user input
                    Inpt = UCASE$(Inpt) ' uppercase input
                    Inpt = RTRIM$(Inpt) ' trim input
                    Player.Found = False ' reset player found flag
                    FOR User.Number = 1 TO LOF(UserFile) / LEN(UserRecord) ' loop file
                        CALL Read.Record(UserFile, User.Number) ' read user record
                        Outpt = UserRecord.CodeName ' store codename
                        CALL Decrypt(Outpt) ' decrypt name
                        Outpt = RTRIM$(Outpt) ' trim name
                        IF Outpt <> Deleted$ THEN ' compare deleted codename
                            IF Outpt <> Dead$ THEN ' compare dead codename
                                IF Outpt = Inpt THEN ' compare codename to transfer
                                    IF User.Index <> User.Number THEN ' check indexes
                                        IF (UserRecord.Flags AND Locked.User) = False THEN
                                            Player.Found = True ' set player found flag
                                            ' add amount to tranfer to player account
                                            UserRecord.Bank = UserRecord.Bank + Transfer.Amount#
                                            CALL Share.Record(UserFile, User.Number) ' write
                                            EXIT FOR ' exit loop through user file
                                        END IF ' end check locked user record
                                    END IF ' end check indexes
                                END IF ' end check codename
                            END IF ' end check dead codename
                        END IF ' end check deleted codename
                    NEXT ' end loop through user file records
                    IF Player.Found THEN ' check player found flag
                        Outpt = "The broker works with his ledgers for a while,"
                        CALL IO.O ' send display message
                        Outpt = "And says: " + MID$(STR$(Transfer.Amount#), 2) + " gold transferred to his account."
                        CALL IO.O ' send display message
                    ELSE ' check player found flag
                        Transfer.Amount# = False ' reset gold tranfered
                        Outpt = "There's nobody in my ledgers with that name!"
                        CALL IO.O ' send display message
                    END IF ' end check player found flag
                    CALL Read.Record(UserFile, User.Index) ' read current user record
                    ' subtract amount tranfered
                    UserRecord.Bank = UserRecord.Bank - Transfer.Amount#
                END IF ' end check input range
            END IF ' end check player bank account
        CASE "W" ' option to withdraw gold from bank account
            IF UserRecord.Bank <= False THEN ' check bank account
                Outpt = "You have nothing to withdraw!" ' make display message
                CALL IO.O ' send message
            ELSE ' check bank account
                IF UserRecord.Borrow > False THEN ' check bank loan
                    Outpt = "You must pay back your loan first!" ' make display message
                    CALL IO.O ' send message
                ELSE ' check bank loan
                    Outpt = "How much(1-" + MID$(STR$(UserRecord.Bank), 2) + ")? " ' prompt
                    CALL IO.I ' get user input
                    Withdraw.Amount# = CDBL(INT(VAL(Inpt))) ' convert input to double
                    IF Withdraw.Amount# <= False OR Withdraw.Amount# > UserRecord.Bank THEN ' check input range
                        Outpt = "You don't have that much gold!" ' make display message
                        CALL IO.O ' send message
                    ELSE ' check input range
                        ' subtract from bank
                        UserRecord.Bank = UserRecord.Bank - Withdraw.Amount#
                        UserRecord.Gold = UserRecord.Gold + Withdraw.Amount# ' add to gold
                        Outpt = "The broker hands you the gold." ' make message
                        CALL IO.O ' send message
                    END IF ' end check input range
                END IF ' end check bank loan
            END IF '  end check bank account
        CASE "Q" ' option to exit bank menu
            EXIT DO ' exit bank menu
    END SELECT ' end selection of bank option
LOOP ' end loop through bank menu
END SUB ' end bank routine

REM * Filename: dnds6.bas Version: v5.0a r1.0a
REM * This subprogram contains mail and some find routines.

REM * main routine to enter mail and process mail selections.

SUB Mail
REDIM Array(1 TO 19) AS STRING ' dimension message array
DO ' loop through mail commands
    Graphics.Off = True ' reset color
    Outpt = "[E]nter" ' make display message
    CALL IO.O ' send message
    Outpt = "[K]ill" ' make display message
    CALL IO.O ' send message
    Outpt = "[L]ock" ' make display message
    CALL IO.O ' send message
    Outpt = "[N]ew" ' make display message
    CALL IO.O ' send message
    Outpt = "[R]ead" ' make display message
    CALL IO.O ' send message
    Graphics.Off = False ' reset color
    Outpt = "Mail command(q to quit)? " ' input prompt
    No.Input.Out = "Q" ' default input
    CALL IO.I ' get player input
    SELECT CASE UCASE$(Inpt) ' selection of input
        CASE "E" ' option to enter message
            CALL Enter.Message ' routine to enter message
        CASE "K" ' option to kill message
            CALL Kill.Message ' routine to kill message
        CASE "L" ' option to toggle locked mailbox
            CALL Lock.Mailbox ' routine to toggle locked mailbox
        CASE "N" ' option to read new messages
            CALL New.Messages ' routine to read new messages
        CASE "R" ' option to read messages
            CALL Read.Messages ' routine to read messages
        CASE "Q" ' option to exit mail menu
            EXIT DO ' exit mail menu
    END SELECT ' end mail option selection
LOOP ' end loop through mail menu
END SUB ' end mail routine

REM * routine to enter a message.
REM * output variables:
REM *   Array - array containing message.
REM *   Message.Length - number of message text lines.
REM *   Message.To - codename or null of player message is to.
REM *   Private.Message - true if message is private.
REM *   Subject - contains subject of message.

SUB Enter.Message
IF LOF(TableFile) / LEN(TableRecord) = MaxInt THEN ' check table size
    Outpt = "The message table is full." ' make error output message
    CALL IO.O ' send error output
    EXIT SUB ' exit routine
END IF ' end check table size
CALL Share.Record(UserFile, User.Index) ' routine to store player record
DO ' loop until message to stored
    Graphics.Off = True ' reset color
    Private.Message = False ' reset private message flag
    Outpt = "Message to(Press <enter> for ALL)? " ' input prompt
    CALL IO.I ' get player input
    Message.To = UCASE$(Inpt) ' uppercase and store input
    IF Inpt = Nul THEN ' compare null input
        EXIT DO ' exit loop with message to all
    END IF ' end compare input length
    FOR User.Number = 1 TO LOF(UserFile) / LEN(UserRecord) ' loop through user file
        CALL Read.Record(UserFile, User.Number) ' read next player record
        Outpt = UserRecord.CodeName ' store codename
        CALL Decrypt(Outpt) ' routine to decrypt codename
        Outpt = RTRIM$(Outpt) ' trim codename
        Outpt = UCASE$(Outpt) ' uppercase codename
        IF Message.To = Outpt THEN ' compare codename to input
            IF User.Index <> User.Number THEN ' check not self
                IF (UserRecord.Flags AND Locked.User) = False THEN ' check locked
                    Outpt = "Private message(y/n)? " ' input prompt
                    No.Input.Out = "N" ' default input
                    CALL IO.I ' get player input
                    Private.Message = Yes ' store private message flag
                    EXIT DO ' exit loop until message to found
                END IF ' end check locked player mailbox
            END IF ' end check player indexes
        END IF ' end check codename
    NEXT ' end loop through player file
    Outpt = "There is no such user." ' make display message
    CALL IO.O ' send message
LOOP ' end loop until message to stored
CALL Read.Record(UserFile, User.Index) ' reread player record
DO ' loop until subject entered
    Outpt = "Subject? " ' input prompt
    CALL IO.I ' get player input
    Inpt = LTRIM$(Inpt) ' trim input
    Inpt = RTRIM$(Inpt) ' trim input
    Inpt = LCASE$(Inpt) ' lowercase input
    IF Inpt <> Nul THEN ' check input length
        Subject = Inpt ' store input in subject string
        EXIT DO ' exit subject loop
    END IF ' end check input length
    Outpt = "Abort message(y/n)? " ' input prompt
    No.Input.Out = "N" ' default input
    CALL IO.I ' get player input
    IF Yes THEN ' check input response
        EXIT SUB ' exit message enter routine
    END IF ' end check input response
LOOP ' end loop until subject entered
Message.Reply = False
Outpt = "Enter message." ' make message
CALL IO.O ' send message
CALL Write.Message ' routine to enter and store message
END SUB ' end routine to enter a message

REM * routine to write message.

SUB Write.Message
Message.Length = False ' reset message length entered
User.Word.Wrap = UserRecord.Wordwrap ' store user word wrap
UserRecord.Wordwrap = False ' reset word wrap
DO ' loop until message entered
    Outpt = "Press <enter> on a blank line to edit." ' make message
    CALL IO.O ' send message
    DO ' loop until blank line entered
        Graphics.Off = True ' reset color
        IF Message.Length = 19 THEN ' compare length of message entered
            Outpt = "Message buffer full." ' make buffer message
            CALL IO.O ' send message
            EXIT DO ' exit message entry loop
        END IF ' end compare message length
        Word.Wrap = True ' set word wrap flag
        Outpt = "?" ' input prompt
        CALL IO.I ' get player input
        IF No.Input THEN ' check blank line entered
            EXIT DO ' exit message entry loop
        END IF ' end check blank line
        Message.Length = Message.Length + 1 ' increment message length counter
        Inpt = RTRIM$(Inpt) ' trim input
        Inpt = LEFT$(Inpt, 79) ' truncate input
        Array(Message.Length) = Inpt ' store next message line
    LOOP ' end loop until blank line entered
    Word.Wrap = False ' reset word wrap flag
    UserRecord.Wordwrap = User.Word.Wrap ' restore user word wrap
    CALL Edit.Message(Message.Edit) ' routine to edit message text
    ' selection of return variable from message editing
    SELECT CASE Message.Edit
        CASE True ' true returned
            Outpt = "Continue editing." ' make display message
            CALL IO.O ' send message
        CASE False ' false returned
            EXIT SUB ' message aborted
        CASE UnTrue ' other returned
            CALL Store.Message ' message stored
            EXIT SUB ' exit message entry routine
    END SELECT ' end selection of edit return variable
LOOP ' end loop until message entered
END SUB

REM * routine to lock/unlock player mailbox.

SUB Lock.Mailbox
' check player locked mailbox
IF (UserRecord.Flags AND Locked.User) = False THEN
    Outpt = "Mailbox locked." ' make locked message
    UserRecord.Flags = UserRecord.Flags OR Locked.User ' set locked player flag
ELSE ' check player locked
    Outpt = "Mailbox unlocked." ' make locked message
    UserRecord.Flags = UserRecord.Flags AND NOT Locked.User ' reset locked flag
END IF ' end check player locked flag
CALL IO.O ' send message
CALL Share.Record(UserFile, User.Index) ' routine to wite player record
END SUB

REM * routine to delete message.

SUB Kill.Message
Max.Messages = LOF(TableFile) / LEN(TableRecord) ' store length of table file
Outpt = "Enter message number to delete" ' make range input prompt
CALL Get.Range2(0, Max.Messages, Message.Number) ' routine to get number
IF Message.Number > False THEN ' check number bounds
    CALL Read.Record(TableFile, Message.Number) ' read table record
    Outpt = UserRecord.CodeName ' store codename
    CALL Decrypt(Outpt) ' decrypt codename
    Outpt = RTRIM$(Outpt) ' trim codename
    Outpt = UCASE$(Outpt) ' uppercase codename
    Inpt = TableRecord.ToUser ' store message from codename
    Inpt = RTRIM$(Inpt) ' trim message codename
    Inpt = UCASE$(Inpt) ' uppercase from codename
    IF Outpt <> Inpt THEN ' compare codenames
        Outpt = "Message" + STR$(Message.Number) + " is not a private message to you."
        CALL IO.O ' send message
        EXIT SUB ' exit routine
    END IF ' end compare codenames
    IF TableRecord.Private = False THEN
        Outpt = "Message" + STR$(Message.Number) + " is not a private message to you."
        CALL IO.O ' send message
        EXIT SUB ' exit routine
    END IF ' end compare codenames
    IF TableRecord.Killed THEN ' compare deleted message
        Outpt = "Message" + STR$(Message.Number) + " is already deleted." ' error message
        CALL IO.O ' send message
        EXIT SUB ' exit routine
    END IF ' end compare deleted message
    TableRecord.Killed = True ' set deleted message flag in table record
    CALL Share.Record(TableFile, Message.Number) ' routine to write table record
    Outpt = "Message" + STR$(Message.Number) + " deleted." ' make deleted message
    CALL IO.O ' send message
END IF ' end check number bounds
END SUB ' end routine to delete a message

REM * routine to display message.
REM * input variables:
REM *   Message.Number - contains number of message to display.
REM *   ListLines - true to list line numbers

SUB Read.Message (Message.Number, ListLines)
CALL Message.Header(Message.Number) ' routine to display message header
Allow.Break = True ' set control-k checking flag
Break = False ' reset control-k flag
Continue = False ' reset continuous flag
Page.Length = 5 ' reset page length counter
' store beginning of message record flag
Message.Start! = TableRecord.Start
' store ending of message record flag
Message.End! = Message.Start! + CSNG(TableRecord.Length - 1)
' loop through message contents
Line.Number = False ' reset line counter
FOR Message.Record! = Message.Start! TO Message.End!
    CALL Read.Message.Record(MessageFile, Message.Record!) ' read message text line
    Outpt = MessageRecord.Message ' store message text
    CALL Decrypt(Outpt) ' decrypt message text
    Outpt = RTRIM$(Outpt) ' trim message
    Line.Number = Line.Number + 1 ' increment line counter
    IF ListLines THEN ' check line number list flag
        Outpt = MID$(STR$(Line.Number), 2) + ":" + Outpt ' make line number
    END IF ' end check line number flag
    Outpt = LEFT$(Outpt, 79) ' truncate message text
    CALL IO.O ' send message text line
    IF Break THEN ' check control-k pressed flag
        EXIT FOR ' exit message display loop
    END IF ' end check control-k pressed
    Page.Length = Page.Length + 1 ' increment page length counter
    IF Page.Length = UserRecord.Pagelength THEN ' check page length
        Page.Length = False ' reset page length
        IF Continue = False THEN ' check continuous flag
            CALL More.Prompt ' routine to pause
            IF No THEN ' check pause response
                EXIT FOR ' exit message display loop
            END IF ' end check pause response
        END IF ' end check continuous flag
    END IF ' end check page length
NEXT ' end loop through message file
Allow.Break = False ' reset control-k checking flag
IF Break THEN ' check control-k flag
    Break = False ' reset control-k flag
    Outpt = Nul ' set output to null
    CALL IO.O ' send empty return
END IF ' end check control-k flag
END SUB ' end routine to display a message

REM * routine to display message header.
REM * input variables:
REM *   Message.Number - contains number of message.

SUB Message.Header (Message.Number)
' make header
CALL Sub.Header(Message.Number) ' routine displays first part of header
Outpt = "Rcvd:  " ' make header
IF TableRecord.Received THEN ' check message
    Outpt = Outpt + "Yes" ' append header
ELSE ' check message
    Outpt = Outpt + "No" ' append header
END IF ' end check message
Outpt = Outpt + SPACE$(35 - LEN(Outpt)) ' pad spaces
Outpt = Outpt + "Read:" + STR$(TableRecord.TimesRead) + " times"
CALL IO.O ' send more header
Outpt = "Reply: " ' make header
IF TableRecord.Reply THEN ' check message type
    Outpt = Outpt + "Yes" ' append header
ELSE ' check message
    Outpt = Outpt + "No" ' append header
END IF ' end check message type
Outpt = Outpt + SPACE$(35 - LEN(Outpt)) ' pad spaces
Outpt = Outpt + "Thread:" ' append more header
IF TableRecord.Reply THEN ' check message type again
    IF TableRecord.Thread = False THEN ' check message thread number
        Outpt = Outpt + "(forward)" ' append default header
    ELSE ' check thread number
        Outpt = Outpt + STR$(TableRecord.Thread) ' append thread number
    END IF ' end check message type
ELSE ' check message type
    Outpt = Outpt + " None" ' append more header
END IF ' end check message
CALL IO.O ' send header
Outpt = "Stat:  " ' make more header
Class.Number = TableRecord.ClassType ' store class type from
SELECT CASE Class.Number ' select class type
    CASE 1 TO 10 ' check class type range
        Outpt = Outpt + RTRIM$(Class.Name(Class.Number)) ' store class name
END SELECT ' end check class type range
Outpt = LEFT$(Outpt, 35) ' trunacte more header
Outpt = Outpt + SPACE$(35 - LEN(Outpt)) ' pad spaces
Outpt = Outpt + "Type: " ' append header
Inpt = Nul ' reset message type from
IF TableRecord.Flags AND Special.Char1 THEN ' check message from
    Inpt = Inpt + "Town Mayor " ' append class type from
END IF ' end check class type message from
IF TableRecord.Flags AND Special.Char2 THEN ' check message from
    Inpt = Inpt + "Governor " ' append class type from
END IF ' end check class type message from
IF TableRecord.Flags AND Special.Char3 THEN ' check message from
    Inpt = Inpt + "Guild Master " ' append class type from
END IF ' end check class type message from
IF TableRecord.Flags AND Special.Char4 THEN ' check message from
    Inpt = Inpt + "Sysop " ' append class type from
END IF ' end check class type message from
IF Inpt = Nul THEN ' check message from
    Inpt = "None" ' reset class type from
END IF ' end check class type message from
Outpt = Outpt + Inpt ' append more header
Outpt = LEFT$(Outpt, 79) ' truncate header
Outpt = RTRIM$(Outpt) ' trim header
CALL IO.O ' send more header
END SUB ' end routine to display message header

REM * routine displays first part of message header
REM * input variables:
REM *   Message.Number - number of message being read.

SUB Sub.Header (Message.Number)
Graphics.Off = False ' reset color
Outpt = "Msg#:" + STR$(Message.Number) + " of" + STR$(LOF(TableFile) / LEN(TableRecord))
IF TableRecord.Private THEN ' check private message
    Outpt = Outpt + "(private)" ' append to header
END IF ' end check message
IF TableRecord.Killed THEN ' check deleted message
    Outpt = Outpt + "(deleted)" ' append to header
END IF ' end check message
CALL IO.O ' send header
Graphics.Off = True ' reset color
Inpt = TableRecord.From ' make header
Inpt = LCASE$(Inpt) ' lowercase header
MID$(Inpt, 1, 1) = UCASE$(MID$(Inpt, 1, 1)) ' uppercase first character
Outpt = "From:  " + Inpt ' make header
Outpt = LEFT$(Outpt, 35) ' truncate hader
Outpt = Outpt + SPACE$(35 - LEN(Outpt)) ' pad spaces
Inpt = RTRIM$(TableRecord.Subject) ' make more header
MID$(Inpt, 1, 1) = UCASE$(MID$(Inpt, 1, 1)) ' uppercase more header
Outpt = Outpt + "Subj: " + Inpt ' append header
CALL IO.O ' send more header
Inpt = TableRecord.ToUser ' make more header
Inpt = RTRIM$(Inpt) ' trim header
IF Inpt = Nul THEN ' check null message to
    Inpt = "ALL" ' default to all
END IF ' end check message to
Inpt = LCASE$(Inpt) ' lowercase header
MID$(Inpt, 1, 1) = UCASE$(MID$(Inpt, 1, 1)) ' uppercase header
Outpt = "To:    " + Inpt ' make more header
Outpt = LEFT$(Outpt, 35) ' truncate more header
Outpt = Outpt + SPACE$(35 - LEN(Outpt)) ' pad spaces
Outpt = Outpt + "Time: " + TableRecord.Clock ' append header
CALL IO.O ' send more header
END SUB ' end routine to display subheader

REM * routine to read range of messages.

SUB Read.Messages
Code.Name$ = UserRecord.CodeName ' store codename
CALL Decrypt(Code.Name$) ' decrypt codename
Code.Name$ = RTRIM$(Code.Name$) ' trim codename
Code.Name$ = UCASE$(Code.Name$) ' uppercase codename
Max.Messages = LOF(TableFile) / LEN(TableRecord) ' store length of table file
Outpt = "Starting message number" ' make range input prompt
CALL Get.Range2(1, Max.Messages, Message.Number) ' routine to get number
DO ' message read loop
    IF Message.Number > LOF(TableFile) / LEN(TableRecord) THEN ' compare table end
        EXIT DO ' exit message read loop
    END IF ' end compare table
    CALL Read.Record(TableFile, Message.Number) ' read next table record
    Inpt = TableRecord.ToUser ' store message from
    Inpt = RTRIM$(Inpt) ' trim message from
    Inpt = UCASE$(Inpt) ' uppercase message from
    IF TableRecord.Private AND Inpt <> Code.Name$ THEN ' compare private message
        Outpt = "Message #" + MID$(STR$(Message.Number), 2) + " is private." ' message
        CALL IO.O ' send message
    ELSE ' compare private message, codenames
        IF TableRecord.Killed THEN ' compare deleted message
            Outpt = "Message #" + MID$(STR$(Message.Number), 2) + " was deleted."
            CALL IO.O ' send message
        ELSE ' compare deleted message
            CALL Read.Message(Message.Number, False) ' routine to display message
            TableRecord.Received = True ' set message recieved flag
            TableRecord.TimesRead = TableRecord.TimesRead + 1 ' increment times read
            CALL Share.Record(TableFile, Message.Number) ' write table record
        END IF ' end compare deleted message
    END IF ' end compare private message
    Graphics.Off = False ' reset color
    IF Message.Number > UserRecord.LastMessage THEN ' check last message pointer
        UserRecord.LastMessage = Message.Number ' store new last message read
    END IF ' end check last message read
    Outpt = "Read more (y)es/(n)o/(r)eply/(t)hread, msg number? " ' make prompt
    No.Input.Out = "Y" ' default input
    CALL IO.I ' get player input
    SELECT CASE UCASE$(Inpt) ' selection of prompt response
        CASE "N", "Q" ' check input response
            EXIT DO ' exit message read loop
        CASE "R" ' reply to message
            Outpt = "Private message(y/n)? " ' prompt
            No.Input.Out = "N"
            CALL IO.I ' get input
            Private.Message = Yes ' set message type
            Message.Reply = True ' set message type
            Message.Thread = Message.Number ' store message number
            Message.To = TableRecord.From ' store message header
            Subject = TableRecord.Subject ' store message header
            Outpt = "Enter message reply." ' make message
            CALL IO.O ' send message
            CALL Write.Message ' routine to enter and write message
            CALL Read.Record(TableFile, Message.Number) ' read current table
            TableRecord.Reply = True ' set reply
            CALL Share.Record(TableFile, Message.Number) ' write table record
            Message.Number = Message.Number - 1 ' adjust loop variable
        CASE "T" ' thread selection
            Outpt = "Direction (b)ackward/(f)oward/(s)earch? " ' input prompt
            No.Input.Out = "F" ' default input
            CALL IO.I ' get input
            SELECT CASE UCASE$(Inpt) ' selection of thread direction
                CASE "B" ' backward thread
                    IF TableRecord.Thread = False THEN ' check thread
                        Outpt = "Message has no backward reply thread." ' make message
                        CALL IO.O ' send message
                        CALL More.Prompt ' pause prompt
                        Message.Number = Message.Number - 1 ' reset loop variable
                    ELSE ' check thread
                        ' change loop variable to thread number
                        Message.Number = TableRecord.Thread - 1
                    END IF ' end check thread number
                CASE "F" ' forward thread
                    ' loop through table
                    FOR Message.Search = Message.Number TO LOF(TableFile) / LEN(TableRecord)
                        CALL Read.Record(TableFile, Message.Search) ' read table record
                        ' compare thread numbers
                        IF TableRecord.Thread = Message.Number THEN
                            Message.Number = Message.Search - 1 ' change loop variable
                            EXIT FOR ' exit search loop
                        END IF ' end compare thread numbers
                    NEXT ' end loop through table
                    IF Message.Search > LOF(TableFile) / LEN(TableRecord) THEN ' check bounds
                        Outpt = "Message has no forward thread." ' make message
                        CALL IO.O ' send message
                        CALL More.Prompt ' pause prompt
                        Message.Number = Message.Number - 1 ' reset loop variable
                    END IF ' end check loop variable
                CASE "S" ' search forward thread
                    ' loop through table
                    FOR Message.Search = Message.Number TO LOF(TableFile) / LEN(TableRecord)
                        CALL Read.Record(TableFile, Message.Search) ' read table record
                        ' compare thread numbers
                        IF TableRecord.Thread = Message.Number THEN
                            CALL Sub.Header(Message.Search) ' routine displays subheader
                            Graphics.Off = False ' reset color
                            Outpt = "Message thread:(c)ontinue search/(r)ead? " ' prompt
                            No.Input.Out = "R" ' default input
                            CALL IO.I ' get user input
                            IF UCASE$(Inpt) = "R" THEN ' compare input
                                Message.Number = Message.Search - 1 ' change loop variable
                                EXIT FOR ' exit search loop
                            END IF ' end compare input
                        END IF ' end compare thread numbers
                    NEXT ' end loop through table
                    IF Message.Search > LOF(TableFile) / LEN(TableRecord) THEN ' check bounds
                        Outpt = "There are no more forward threads." ' make message
                        CALL IO.O ' send message
                        CALL More.Prompt ' pause prompt
                        Message.Number = Message.Number - 1 ' reset loop variable
                    END IF ' end check loop variable
            END SELECT ' end thread selection
        CASE ELSE ' other selection
            Next.Message = INT(VAL(Inpt)) ' convert input to integer
            ' check input range
            IF Next.Message > False AND Next.Message <= LOF(TableFile) / LEN(TableRecord) THEN
                Message.Number = Next.Message - 1 ' reset message loop variable
            END IF ' end check input range
    END SELECT ' end prompt selection
    Message.Number = Message.Number + 1 ' increment table number
LOOP ' end loop through message table
Outpt = "End of messages." ' make message
CALL IO.O ' send message
END SUB ' end routine to read messages

REM * routine to read range of new messages.

SUB New.Messages
IF UserRecord.LastMessage >= LOF(TableFile) / LEN(TableRecord) THEN ' compare last
    Outpt = "No new messages." ' make message
    CALL IO.O ' send message
    EXIT SUB ' exit routine
END IF ' end compare last message read variable to length of table
Code.Name$ = UserRecord.CodeName ' store codename
CALL Decrypt(Code.Name$) ' decrypt codename
Code.Name$ = RTRIM$(Code.Name$) ' trim codename
Code.Name$ = UCASE$(Code.Name$) ' uppercase codename
Message.Number = UserRecord.LastMessage + 1 ' store last message read variable
Last.Message = LOF(TableFile) / LEN(TableRecord) ' store length of table
DO ' message read loop
    IF Message.Number > Last.Message THEN ' compare message range
        EXIT DO ' exit message loop
    END IF ' end compare message range
    CALL Read.Record(TableFile, Message.Number) ' read next table record
    Inpt = TableRecord.ToUser ' store message to
    Inpt = RTRIM$(Inpt) ' trim message to
    Inpt = UCASE$(Inpt) ' uppercase message to
    Message.Read = True ' set read message flag
    IF TableRecord.Private THEN ' check private message flag
        IF Inpt <> Code.Name$ THEN ' compare codenames
            Message.Read = False ' reset read message flag
        END IF ' end compare codenames
    END IF ' end check private message flag
    IF TableRecord.Killed THEN ' check deleted message
        Message.Read = False ' reset read message flag
    END IF ' end check deleted message
    IF Message.Read THEN ' verify read message flag
        CALL Read.Message(Message.Number, False) ' routine to display message
    END IF ' end verify read message flag
    Graphics.Off = False ' reset color
    ' update player last message variable
    UserRecord.LastMessage = Message.Number
    Outpt = "Read more (y)es/(n)o/(r)eply? " ' make more prompt
    No.Input.Out = "Y" ' default input
    CALL IO.I ' get player input
    IF UCASE$(Inpt) = "R" THEN ' check input
        Outpt = "Private message(y/n)? " ' input prompt
        CALL IO.I ' get player input
        Private.Message = Yes ' set message type
        Message.Reply = True ' set message type
        Message.Thread = Message.Number ' store message number
        Message.To = TableRecord.From ' store message header
        Subject = TableRecord.Subject ' store message header
        Outpt = "Enter message reply." ' make message
        CALL IO.O ' send message
        CALL Write.Message ' routine to enter and write message
        CALL Read.Record(TableFile, Message.Number) ' read current table
        TableRecord.Reply = True ' set reply
        CALL Share.Record(TableFile, Message.Number) ' write table record
        Message.Number = Message.Number - 1 ' adjust read loop variable
    END IF ' end check input response
    IF No OR Quit THEN ' check input response
        EXIT DO ' exit new message read loop
    END IF ' end check response
    Message.Number = Message.Number + 1 ' increment message variable
LOOP ' end loop through new messages
END SUB ' end routine to read new messages

REM * routine containing status line commands. status line toggle,
REM * initializing, clearing, and updating.
REM * input variables:
REM *   Status.Display -
REM *       -2 clear both status lines (rows 24, 25),
REM *       -1 toggle status line displaying remote user statistics, or
REM *          interactive console function key list,
REM *        0 update remote user status line statistics,
REM *        1 initialize both status lines.
REM * processing variables:
REM *   CursorX - temporary variable containing the cursor row position.
REM *   CursorY - temporary variable containing the cursor column position.
REM *   Statusline.Mode - static variable saved between calls containing the
REM *      toggle mode of the status line.
REM * notes on routine:
REM *   since the status line can be toggled while an online user is
REM *   entering input, the color of the screen is restored upon exit
REM *   from the routine. i.e. the status line toggles during any
REM *   character of i/o.

SUB Status.Line (Status.Display)
STATIC Statusline.Mode ' status line toggle (variable saved between calls)
IF Status.Display = -1 THEN ' compare to toggle command
    IF Local.Mode THEN ' compare local console logged in
        EXIT SUB ' exit routine w/o toggling status line
    END IF ' end compare local mode
    Statusline.Mode = NOT Statusline.Mode ' toggle status line
END IF ' end compare toggle command
IF Status.Display = -2 THEN ' compare to clear status line command
    FOR FunctionKeys = 1 TO 10 ' loop through all ten function keys
        KEY FunctionKeys, Nul ' reset/disable function key
    NEXT ' end loop through function keys
    CursorX = CSRLIN ' store current cursor row
    CursorY = POS(0) ' store current cursor column
    COLOR 7, 0 ' set color white on black non-intensity
    LOCATE 25, 1 ' locate at bottom status line
    PRINT SPACE$(79); ' clear status line w/ blanks
    LOCATE 24, 1 ' locate at second to bottom status line
    PRINT SPACE$(79); ' clear status line w/ blanks
    LOCATE CursorX, CursorY, 1 ' relocate at stored cursor row, column
    CALL Restore.Color ' subroutine to restore screen color
    EXIT SUB ' exit routine
END IF ' end compare clear status line command
IF Status.Display > False THEN ' compare to initialize status line command
    Statusline.Mode = False ' set the default status line mode
    IF Local.Mode THEN ' check console logged in
        IF Normal.User = False THEN ' verify user logged in is not DM/Sysop
            Statusline.Mode = True ' reset the default status line mode
        END IF ' end verify DM/Sysop
    END IF ' end check local mode
    'Call Door.Status.Line ' initialize the door information status line
END IF ' end compare initialize status line command
IF Statusline.Mode = False THEN ' compare status line mode
    CALL Make.Status.Line ' routine to update status line w/ player stats
ELSE ' status line mode
    IF Status.Display <> False THEN ' any status line command than update
        CALL Sysop.Status.Line ' routine for sysop function key status line
    END IF ' end compare status line command
END IF ' end compare status line command
END SUB ' end status line commands routine

REM * routine to make status line containing player character statistics.
REM * processing variables:
REM *   CursorX - contains the current cursor row.
REM *   CursorY - contains the current cursor column.

SUB Make.Status.Line
Status$ = UserRecord.CodeName ' store player codename
CALL Decrypt(Status$) ' decrypt codename
Status$ = LCASE$(Status$) ' set codename lowercase
IF LEFT$(Status$, 9) = Deleted$ THEN ' check codename is invalid/deleted
    EXIT SUB ' exit routine
END IF ' end check codename
MID$(Status$, 1, 1) = UCASE$(MID$(Status$, 1, 1)) ' uppercase first codename letter
IF UserRecord.ClassType = MagicUser THEN ' compare player class type to MU
    Status2$ = "MU" ' set status line player class name to MU abbreviation
ELSE ' compare to non MU
    ' set status line player class name to left part of entire class name
    Status2$ = LEFT$(Class.Name(UserRecord.ClassType), 8)
END IF ' end compare MU
IF Dungeon.Master.Assistant THEN ' compare player class name to Asst. DM
    Status2$ = "ADM" ' set status line to Asst. DM abbreviation
END IF ' end compare ADM
IF Dungeon.Master THEN ' compare player class name to DM
    Status2$ = "DM" ' set status line to DM abbreviation
END IF ' end compare DM
IF Sysop THEN ' compare player to sysop
    Status2$ = "SYS" ' set status line to sysop abbreviation
END IF ' end compare sysop
Status$ = Status$ + " " + Status2$ ' combine player codename with class name
Status$ = LEFT$(Status$, 39) ' truncate to left half
Status$ = Status$ + SPACE$(39 - LEN(Status$)) ' append trailing blanks
Status.Value = UserRecord.Fatigue ' store current player fatigue
IF Status.Value < False THEN ' check validity of fatigue
    Status.Value = False ' reset to zero
END IF ' end check validity
Status$ = Status$ + " Fat:" + MID$(STR$(Status.Value), 2) ' append fatigue message
Status.Value = UserRecord.Vitality ' store current player vitality
IF Status.Value < False THEN ' check validity of vitality
    Status.Value = False ' reset to zero
END IF ' end check validity
Status$ = Status$ + " Vit:" + MID$(STR$(Status.Value), 2) ' append vitality message
Status.Value = UserRecord.Magic ' store current player magic points
IF Status.Value < False THEN ' check magic points validity
    Status.Value = False ' reset to zero
END IF ' end check validity
' append magic points message
Status$ = Status$ + " Mag:" + MID$(STR$(Status.Value), 2)
Status.Value = UserRecord.Psionic ' store current player psionic points
IF Status.Value < False THEN ' check psionic points validity
    Status.Value = False ' reset to zero
END IF ' end check validity
' append psionic points message
Status$ = Status$ + " Psi:" + MID$(STR$(Status.Value), 2)
Status.Value = UserRecord.Level ' store player level
IF Status.Value < False THEN ' compare level validity
    Status.Value = False ' reset to zero
END IF ' end compare validity
Status2$ = " Lvl:" + MID$(STR$(Status.Value), 2) ' append level message
' verify string appended to status line
IF LEN(Status$) + LEN(Status2$) <= 79 THEN
    ' string is less than screen line length, and append
    Status$ = Status$ + Status2$
END IF ' end verify status line length
Status.Value = Room ' store current room number
IF Status.Value < False THEN ' check validity of room number
    Status.Value = False ' reset to zero
END IF ' end check validity
Status2$ = " Rm:" + MID$(STR$(Status.Value), 2) ' append room message
' verify string appended to status line
IF LEN(Status$) + LEN(Status2$) <= 79 THEN
    ' string is less than screen line length, and append
    Status$ = Status$ + Status2$
END IF ' end verify status line length
Status$ = LEFT$(Status$, 79) ' truncate status string length
Status$ = Status$ + SPACE$(79 - LEN(Status$)) ' append blanks to status string
CursorX = CSRLIN ' store current cursor row
CursorY = POS(0) ' store current cursor column
LOCATE 25, 1 ' position cursor at row 25
COLOR 14, 1 ' color hi-intensity yellow on blue
PRINT Status$; ' display the combined status string
LOCATE CursorX, CursorY, 1 ' restore cursor position
CALL Restore.Color ' routine to restore screen color
END SUB ' end routine to make and display status line

REM * routine to initialize the console function keys, initialize the
REM * DM/sysop status line with the function key names.
REM * processing variables:
REM *   CursorX, CursorY - contain the current cursor position.

SUB Sysop.Status.Line
FOR FunctionKeys = 1 TO 10 ' loop through all ten function keys
    KEY FunctionKeys, Nul ' clear the function key
NEXT ' loop through keys
IF Local.Mode THEN ' check console logged in
    IF Normal.User = False THEN ' check user is DM/sysop
        KEY 1, "!EDIT" + CHR$(13) ' assign key 1
        KEY 2, "!STA" + CHR$(13) ' assign key 2
        KEY 3, "!DIS " ' assign key 3
        KEY 4, "!REDU " ' assign key 4
        KEY 5, "!CALL" ' assign key 5
        KEY 6, "!KILL " ' assign key 6
        KEY 7, "!TELE " ' assign key 7
        KEY 8, "!INV" + CHR$(13) ' assign key 8
        KEY 9, "!GET " ' assign key 9
        KEY 10, "!LINK" + CHR$(13) ' assign key 10
    END IF ' end check DM
END IF ' end check local mode
CursorX = CSRLIN ' store cursor row
CursorY = POS(0) ' store cursor column
COLOR 14, 1 ' set color to hi-intensity yellow on blue
FOR FunctionKeys = 1 TO 10 ' loop through the ten function key numbers
    ' position the cursor at the function key column
    LOCATE 25, FunctionKeys * 8 - 7
    ' display the function key and number
    PRINT "F" + RIGHT$(STR$(FunctionKeys + 10), 1);
NEXT ' end loop through function key numbers
COLOR 15, 1 ' set color to hi-intensity white on blue
' store function key names
FunctionKeys$ = "EDITSTA DIS REDUCALLKILLTELEINV GET LINK"
FOR FunctionKeys = 1 TO 10 ' loop through ten function key names
    ' position the cursor at the status function key name
    LOCATE 25, FunctionKeys * 8 - 5
    ' make the function key name with DM prefix from the function string
    Status$ = "!" + MID$(FunctionKeys$, (FunctionKeys - 1) * 4 + 1, 4)
    IF FunctionKeys < 10 THEN ' check for last key
        Status$ = Status$ + " " ' append space
    END IF ' end check last key
    PRINT Status$; ' display the function key name
NEXT ' end loop through function key names
LOCATE CursorX, CursorY, 1 ' restore cursor position
CALL Restore.Color ' routine to restore screen color
END SUB ' end routine to initialize function keys and DM status line

REM * routine to restore current ansi color after status line is displayed.
REM * input variables:
REM *   Color.Code - contains current color in cycling.

SUB Restore.Color
IF Graphics.Off THEN ' check color cycling
    CALL Convert.Color(37) ' restore color to white
ELSE ' color check
    CALL Convert.Color(Color.Code) ' restore color
END IF ' end check color cycling on
END SUB ' end routine to restore color

REM * routine displays spell types.

SUB Spell.Types
Graphics.Off = True ' reset color
Outpt = "[A]Enchant      [O]Psionic"
CALL IO.O ' send output
Outpt = "[B]Offense      [P]Detect Lock"
CALL IO.O ' send output
Outpt = "[C]Bless        [R]Detect Evil"
CALL IO.O ' send output
Outpt = "[D]Wish         [S]Detect Trap"
CALL IO.O ' send output
Outpt = "[E]Poison       [T]Intoxicate"
CALL IO.O ' send output
Outpt = "[F]Vigor        [U]Set Trap"
CALL IO.O ' send output
Outpt = "[G]Heal         [V]Hide"
CALL IO.O ' send output
Outpt = "[H]Curepoison   [W]Search"
CALL IO.O ' send output
Outpt = "[I]Level Drain  [X]Invisibility"
CALL IO.O ' send output
Outpt = "[J]Teleport     [Y]Identify"
CALL IO.O ' send output
Outpt = "[K]Befuddle     [Z]Enlighten"
CALL IO.O ' send output
Outpt = "[L]Turn Undead  [1]Illuminate"
CALL IO.O ' send output
Outpt = "[M]Pass Door    [2]Psyche"
CALL IO.O ' send output
Outpt = "[N]Conjure      [3]Telepathy"
CALL IO.O ' send output
END SUB

REM * routine changes message.

SUB Change.Message
DO ' loop through message changing
    Graphics.Off = False ' reset color
    Max.Messages = LOF(TableFile) / LEN(TableRecord) 'store length of message table
    Outpt = "Message number to edit" ' make range input prompt
    CALL Get.Range2(0, Max.Messages, Message.Number) ' routine to get number
    IF Message.Number > False THEN ' check number bounds
        CALL Read.Record(TableFile, Message.Number) ' read table record
        Graphics.Off = False ' reset color
        Outpt = "Edit message header(y/n)? " ' make input prompt
        No.Input.Out = "N" ' default input
        CALL IO.I ' get user input
        IF Yes THEN ' check nput response
            CALL Message.Header(Message.Number) ' routine to display header
            Graphics.Off = False ' reset color
            Outpt = "Enter new message header:" ' make display message
            CALL IO.O ' send message
            Graphics.Off = True ' reset color
            Outpt = "From(press <enter> for Sysop)? " ' make input prompt
            No.Input.Out = "Sysop" ' default input
            CALL IO.I ' get user input
            TableRecord.From = Inpt ' store message from
            Outpt = "To(press <enter> for ALL)? " ' make input prompt
            CALL IO.I ' get user input
            TableRecord.ToUser = UCASE$(Inpt) ' store message to
            Outpt = "Subject? " ' make input prompt
            CALL IO.I ' get user input
            TableRecord.Subject = LCASE$(Inpt) ' store message subject
            CALL Share.Record(TableFile, Message.Number) ' routine to write table record
        END IF ' end check response
        DO ' loop through message text editing
            Graphics.Off = False ' reset color
            Outpt = "Edit more message text(y/n)? " ' make input prompt
            No.Input.Out = "N" ' default input
            CALL IO.I ' get user input
            IF No THEN ' check input response
                EXIT DO ' exit message editing loop
            END IF ' end check response
            CALL Read.Message(Message.Number, True) ' routine to display message
            Graphics.Off = False ' rset color
            MessageRec.Length = TableRecord.Length ' store message length
            Outpt = "Line number to edit" ' make range input prompt
            ' routine to get number
            CALL Get.Range2(0, MessageRec.Length, Message.Line)
            IF Message.Line > False THEN ' check range
                Outpt = "New message text:" ' make display message
                CALL IO.O ' send message
                Graphics.Off = True ' reset color
                Outpt = "?" ' make input prompt
                CALL IO.I ' get user input
                CALL Valid(Inpt, 80) ' validate input
                CALL Encrypt(Inpt, True) ' encrypt input
                MessageRecord.Message = Inpt ' store new message text
                MessageRec.Number! = CSNG(TableRecord.Start + Message.Line - 1)
                CALL Share.Message(MessageFile, MessageRec.Number!) ' write message record
            END IF ' end check range
        LOOP ' end loop through editing
    END IF ' end check number bounds
    Graphics.Off = False ' reset color
    Outpt = "Edit another message(y/n)? " ' make input prompt
    No.Input.Out = "Y" ' default input
    CALL IO.I ' get user input
    IF No THEN ' check input response
        EXIT SUB ' exit routine
    END IF ' end check response
LOOP ' end loop through editing
END SUB ' end message editing routine

REM * routine to delete message.

SUB Delete.Message
DO ' loop through delete routine
    Max.Messages = LOF(TableFile) / LEN(TableRecord) ' store length of table file
    Outpt = "Message number to delete" ' make range input prompt
    CALL Get.Range2(0, Max.Messages, Message.Number) ' routine to get number
    IF Message.Number > False THEN ' check number bounds
        CALL Read.Record(TableFile, Message.Number) ' read table record
        IF TableRecord.Killed THEN ' check deleted message flag
            Outpt = "Message" + STR$(Message.Number) + " is already deleted." ' message
        ELSE ' check deleted message
            TableRecord.Killed = True ' set deleted message flag
            CALL Share.Record(TableFile, Message.Number) ' routine to write table record
            Outpt = "Message" + STR$(Message.Number) + " deleted." ' make deleted message
        END IF ' end check deleted message flag
        CALL IO.O ' send message
    END IF ' end check number bounds
    Graphics.Off = False ' reset color
    Outpt = "Delete more messages(y/n)? " ' make input prompt
    No.Input.Out = "Y" ' default input
    CALL IO.I ' get user input
    IF No THEN ' check input response
        EXIT SUB ' exit routine
    END IF ' end check response
LOOP ' end loop through deleting
END SUB ' end delete message routine

REM * routine to list range of messages.

SUB List.Messages
Messages.Max = LOF(TableFile) / LEN(TableRecord) ' store length of table
' routine to get range of numbers
CALL Get.Range(Messages.Max, Messages1, Messages2)
FOR Message.List = Messages1 TO Messages2 ' loop through range of messages
    CALL Read.Record(TableFile, Message.List) ' read table record
    CALL Read.Message(Message.List, False) ' routine to display message
    Graphics.Off = False ' reset color
    Outpt = "Read more(y/n)? " ' make prompt
    No.Input.Out = "Y" ' default input
    CALL IO.I ' get player input
    IF No THEN ' check input response
        EXIT SUB ' exit routine
    END IF ' end check response
NEXT ' end message display loop
END SUB ' end message list routine

REM * routine to undelete messages.

SUB Undelete.Message
DO ' loop through undeleting
    Max.Messages = LOF(TableFile) / LEN(TableRecord) ' store table length
    Outpt = "Message number to undelete" ' make range input prompt
    CALL Get.Range2(0, Max.Messages, Message.Number) ' routine to get number
    IF Message.Number > False THEN ' check number bounds
        CALL Read.Record(TableFile, Message.Number) ' read table record
        IF TableRecord.Killed = False THEN ' check deleted message
            Outpt = "Message" + STR$(Message.Number) + " is not deleted." ' make message
        ELSE ' check deleted message
            TableRecord.Killed = False ' set deleted message flag
            CALL Share.Record(TableFile, Message.Number) ' routine to write table record
            Outpt = "Message" + STR$(Message.Number) + " undeleted." ' make message
        END IF ' end check undeleted message
        CALL IO.O ' send message
    END IF ' end check number bounds
    Graphics.Off = False ' reset color
    Outpt = "Undelete more messages(y/n)? " ' make input prompt
    No.Input.Out = "Y" ' default input
    CALL IO.I ' get user input
    IF No THEN ' check input response
        EXIT SUB ' exit routine
    END IF ' end check input
LOOP ' end undeleting loop
END SUB ' end undelete routine

REM * routine to pack messages files.

SUB Pack.Messages
Outpt = "Pack messages(y/n)? " ' make input prompt
No.Input.Out = "N" ' default input
CALL IO.I ' get user input
IF Yes THEN ' check response
    Outpt = "Packing messages.." ' make message
    CALL IO.O ' send message
    CALL Share.Record(UserFile, User.Index) ' store user record
    Max.Users = LOF(UserFile) / LEN(UserRecord) ' store length of user file
    ' dimension working array
    REDIM ArrayX(1 TO Max.Users) AS INTEGER
    ' packing pass 1
    FOR User.Number = 1 TO Max.Users ' loop through user file
        CALL Read.Record(UserFile, User.Number) ' get next user file record
        ArrayX(User.Number) = UserRecord.LastMessage ' store last message number
    NEXT ' end user file loop
    FOR Message.Number = 1 TO LOF(TableFile) / LEN(TableRecord) 'loop through table
        CALL Read.Record(TableFile, Message.Number) ' read table record
        IF TableRecord.Killed THEN ' check for deleted message
            FOR User.Number = 1 TO Max.Users ' loop through user array
                Last.Message = ArrayX(User.Number) ' store last message
                IF Last.Message <= Message.Number THEN ' check last message
                    Last.Message = Last.Message - 1 ' decrement last message read
                    ArrayX(User.Number) = Last.Message ' store last message read
                END IF ' end check last message
            NEXT ' end user array loop
        END IF ' end check deleted message
    NEXT ' end message pack loop
    FOR User.Number = 1 TO Max.Users ' loop through user file
        CALL Read.Record(UserFile, User.Number) ' get next user file record
        UserRecord.LastMessage = ArrayX(User.Number) ' store last message number
        CALL Share.Record(UserFile, User.Number) ' write user record
    NEXT ' end user file loop
    CALL Read.Record(UserFile, User.Index) ' reread current player record
    ' packing pass 2
    Max.Messages = LOF(TableFile) / LEN(TableRecord) ' store maximum table records
    ' dimension working array
    REDIM ArrayX(1 TO Max.Messages) AS INTEGER
    FOR Message.Number = 1 TO Max.Messages ' loop through table file
        CALL Read.Record(TableFile, Message.Number) ' read table record
        ArrayX(Message.Number) = TableRecord.Thread ' store thread number
    NEXT ' end table file rad loop
    FOR Message.Number = 1 TO Max.Messages ' loop through table file
        CALL Read.Record(TableFile, Message.Number) ' read table record
        IF TableRecord.Killed THEN ' check for deleted message
            Thread.Number = ArrayX(Message.Number) ' store thread number
            ' search for forward thread
            FOR Message.Search = Message.Number + 1 TO Max.Messages
                ' verify forward message thread
                IF ArrayX(Message.Search) = Message.Number THEN
                    ArrayX(Message.Search) = Thread.Number ' store thread number
                    IF Thread.Number = False THEN ' check for thread start
                        ' search for end of thread
                        FOR Thread.Search = Message.Search + 1 TO Max.Messages
                            ' compare thread has another ending thread
                            IF ArrayX(Thread.Search) = Message.Search THEN
                                EXIT FOR ' exit thread end loop
                            END IF ' end compare for end thread
                        NEXT ' end end thread search loop
                        IF Thread.Search > Max.Messages THEN ' compare end thread
                            CALL Read.Record(TableFile, Message.Search) ' read table
                            TableRecord.Reply = False ' reset thread reply start
                            CALL Share.Record(TableFile, Message.Search) 'write table
                        END IF ' end compare end thread
                    END IF ' end check thread start
                END IF ' end verify forward message thread
            NEXT ' end forward message thread search loop
            ' search through all forward thread numbers
            FOR Message.Search = Message.Number + 1 TO Max.Messages
                Thread.Number = ArrayX(Message.Search) ' store thread number
                IF Thread.Number >= Message.Number THEN ' verify forward thread
                    Thread.Number = Thread.Number - 1 ' decrement thread number
                    ArrayX(Message.Search) = Thread.Number ' store thread number
                END IF ' end verify forward message thread
            NEXT ' end forward message thread search loop
        END IF ' end check deleted message
    NEXT ' end message pack loop
    FOR Message.Number = 1 TO Max.Messages ' loop through table file
        CALL Read.Record(TableFile, Message.Number) ' read table record
        TableRecord.Thread = ArrayX(Message.Number) ' store thread number
        CALL Share.Record(TableFile, Message.Number) ' write table record
    NEXT ' end table file read loop
    ' packing pass 3
    CLOSE #TempFile ' close temporary file
    FileName = "msgtable.bak" ' get table filename
    KILL FileName ' remove temporary file
    ' open temporary backup file
    OPEN FileName FOR RANDOM SHARED AS #TempFile LEN = LEN(TableRecord)
    FOR Message.Number = 1 TO LOF(TableFile) / LEN(TableRecord) ' loop through table
        CALL Read.Record(TableFile, Message.Number) ' read table record
        CALL Share.Record(TempFile, Message.Number) ' store table record
    NEXT ' end message pack loop
    CLOSE #TempFile ' close temporary file
    FileName = "messages.bak" ' get table filename
    KILL FileName ' remove temporary file
    ' open temporary backup file
    OPEN FileName FOR RANDOM SHARED AS #TempFile LEN = LEN(MessageRecord)
    FOR Message.Number! = 1 TO LOF(MessageFile) / LEN(MessageRecord) ' loop message
        CALL Read.Message.Record(MessageFile, Message.Number!) ' read record
        CALL Share.Message(TempFile, Message.Number!) ' store message record
    NEXT ' end message pack loop
    CLOSE #TableFile, #MessageFile, #TempFile, #TempFile2 ' close temp, mail files
    FileName = "messages.dat" ' get mail filename
    KILL FileName ' remove mail file
    FileName = "msgtable.dat" ' get mail filename
    KILL FileName ' remove mail file
    OPEN "msgtable.dat" FOR RANDOM SHARED AS #TableFile LEN = LEN(TableRecord)
    OPEN "messages.dat" FOR RANDOM SHARED AS #MessageFile LEN = LEN(MessageRecord)
    OPEN "msgtable.bak" FOR RANDOM SHARED AS #TempFile LEN = LEN(TableRecord)
    OPEN "messages.bak" FOR RANDOM SHARED AS #TempFile2 LEN = LEN(MessageRecord)
    Table.Number = False ' reset table record number
    New.Message.Record! = False ' reset message file record counter
    FOR Message.Number = 1 TO LOF(TempFile) / LEN(TableRecord) 'loop through table
        CALL Read.Record(TempFile, Message.Number) ' read table record
        IF TableRecord.Killed = False THEN ' check for deleted message
            Message.Start! = TableRecord.Start ' store beginning
            Message.End! = Message.Start! + CSNG(TableRecord.Length - 1) ' store end
            Table.Number = Table.Number + 1 ' increment next table record
            TableRecord.Start = New.Message.Record! + 1 ' store message beginning
            CALL Share.Record(TableFile, Table.Number) ' write new table file record
            FOR Message.Record! = Message.Start! TO Message.End! ' message loop
                CALL Read.Message.Record(TempFile2, Message.Record!) 'read message
                New.Message.Record! = New.Message.Record! + 1! ' store next message
                CALL Share.Message(MessageFile, New.Message.Record!) ' write message
            NEXT ' end loop through message text array
        END IF ' end check deleted message
    NEXT ' end table file loop
    CLOSE #TempFile, #TempFile2 ' close temporary files
END IF ' end check response
END SUB ' end routine to pack messages

REM * routine to store message.

SUB Store.Message
CALL Read.Record(UserFile, User.Index) ' read user file
Outpt = UserRecord.CodeName ' store codename
CALL Decrypt(Outpt) ' decrypt codename
Inpt = RTRIM$(Outpt) ' store codename
Inpt = UCASE$(Inpt) ' uppercase ccodename
Outpts = Message.To ' store message header
Outpts = RTRIM$(Outpts) ' trim header
Outpts = UCASE$(Outpts) ' uppercase header
IF Inpt = Outpts THEN ' compare codenames
    Message.To = Nul ' reset message header
END IF ' end compare codenames
TableRecord.ClassType = UserRecord.ClassType ' store user class type
TableRecord.Clock = clock$ ' store message creation time
TableRecord.Date = DATE$ ' store system date
TableRecord.Flags = UserRecord.Flags ' store user flags
TableRecord.From = Outpt ' store user codename
TableRecord.Killed = False ' reset deleted message flag
TableRecord.Length = Message.Length ' store length of message
TableRecord.Private = Private.Message ' store private message flag
TableRecord.Received = False ' store flag for message received
TableRecord.Reply = Message.Reply ' store message reply flag
Message.Record.Number! = CSNG(LOF(MessageFile) / LEN(MessageRecord))
TableRecord.Start = Message.Record.Number! + 1! ' store message beginning
TableRecord.Subject = Subject ' store message subject
TableRecord.Time = TIME$ ' store system time
TableRecord.Timer = TIMER ' store system time in seconds
TableRecord.TimesRead = False ' reset number of times message read
TableRecord.ToUser = Message.To ' store message to
IF Message.Reply THEN ' check message type
    TableRecord.Thread = Message.Thread ' store message thread number
ELSE ' check message
    TableRecord.Thread = False ' store thread number
END IF ' end check message
Table.Record.Number = LOF(TableFile) / LEN(TableRecord) + 1 'store next table record
CALL Share.Record(TableFile, Table.Record.Number) ' routine to write table record
FOR Message.Number = 1 TO Message.Length ' loop through message text array
    Outpt = Array(Message.Number) ' store message text line
    CALL Valid(Outpt, 80) ' routine to validate text
    CALL Encrypt(Outpt, True) ' encrypt text
    MessageRecord.Message = Outpt ' store text line
    Message.Record.Number! = Message.Record.Number! + 1! ' store next message
    CALL Share.Message(MessageFile, Message.Record.Number!) ' write message text
NEXT ' end loop through message text array
Outpt = "Message stored." ' make message
CALL IO.O ' send message
END SUB ' end routine to store message

REM * routine to check for new messages.

SUB Check.Mail
New.Message = False ' reset number of new messages
Outpt = UserRecord.CodeName ' store codename
CALL Decrypt(Outpt) ' decrypt codename
Outpt = RTRIM$(Outpt) ' trim codename
Outpt = LCASE$(Outpt) ' lowercase codename
Last.Message = UserRecord.LastMessage ' store last message number read
Table.Number = LOF(TableFile) / LEN(TableRecord) ' store length of message table
' compare range of messages
IF Last.Message > False AND Last.Message <= Table.Number THEN
    FOR Table.Index = Last.Message TO Table.Number ' loop through new messages
        CALL Read.Record(TableFile, Table.Index) ' read table record
        IF TableRecord.Killed = False THEN ' check deleted message
            Inpt = TableRecord.ToUser ' store message to
            Inpt = RTRIM$(Inpt) ' trim message to
            Inpt = LCASE$(Inpt) ' lowercase message to
            IF Inpt <> Nul THEN ' check public message to
                IF Inpt = Outpt THEN ' compare codenames
                    New.Message = New.Message + 1 ' increment number of new message
                END IF ' end compare codenames
            END IF ' end check public message
        END IF ' end check deleted message
    NEXT ' end loop through new messages
END IF ' end compare new message range
Graphics.Off = False ' reset color
IF New.Message > False THEN ' check new messages
    Outpt = "You have" + STR$(New.Message) + " new messages." ' make message
ELSE ' check new message variable
    Outpt = "You have no new messages." ' make output message
END IF ' end check new message variable
CALL IO.O ' send message
Graphics.Off = False ' reset color
END SUB ' end routine to check for new mail

REM * routine to replace line of message text.

SUB Replace.Line
Messages.Max = Message.Length ' store message length
Outpt = "Line number" ' store input range prompt
CALL Get.Range2(0, Messages.Max, Message.Number) ' routine to get number
IF Message.Number > False THEN ' check number bounds
    Outpt = "Replacement line:" ' input prompt
    CALL IO.O ' send prompt
    Outpt = "?" ' input prompt
    CALL IO.I ' get user input
    Inpt = RTRIM$(Inpt) ' trim input
    Inpt = LEFT$(Inpt, 79) ' truncate input
    Array(Message.Number) = Inpt ' store new message line
    Outpt = "Line number" + STR$(Message.Number) + " replaced." ' make display message
    CALL IO.O ' send message
END IF ' end check number bounds
END SUB ' end routine to replace line of message text

REM * routine to edit a line of message text.

SUB Edit.Line
Messages.Max = Message.Length ' store length of message
Outpt = "Line number" ' make input range prompt
CALL Get.Range2(0, Messages.Max, Message.Number) ' routine to get number
IF Message.Number > False THEN ' check number bounds
    Outpt = "Replace what word? " ' make input prompt
    CALL IO.I ' get user input
    IF Inpt = Nul THEN ' check input length
        Outpt = "No replacements made." ' make display message
        CALL IO.O ' send message
        EXIT SUB ' exit routine
    END IF ' end check input length
    Message.Number$ = Inpt ' store input
    Outpt = "Replace with what word? " ' make input prompt
    CALL IO.I ' get user input
    IF Inpt = Nul OR Inpt = Message.Number$ THEN ' check input length, compare inputs
        Outpt = "No replacements made." ' make display message
        CALL IO.O ' send message
        EXIT SUB ' exit routine
    END IF ' end check inputs
    Edit.Replace$ = Inpt ' store input
    Outpt = "Replace all occurences? " ' make input prompt
    No.Input.Out = "Y" ' default input
    CALL IO.I ' get user input
    Replace.Word = 1 ' reset string search index
    Replace.All = Yes ' store input response
    Replacements = False ' reset replacement flag
    EditLine$ = RTRIM$(Array(Message.Number)) ' store message line to edit
    ' routine loop replaces Message.Number$ in EditLine$ with Edit.Replace$
    DO ' loop until replacements finished
        IF EditLine$ = Nul THEN ' check length of message line
            EXIT DO ' exit loop if line length reduced to null
        END IF ' end check length of message line
        ' get position of search string
        Replace = INSTR(Replace.Word, EditLine$, Message.Number$)
        IF Replace = False THEN ' check search string exists
            EXIT DO ' exit loop if search string not found
        END IF ' end check search string exists
        ' replace search string with replacement string
        EditLine$ = LEFT$(EditLine$, Replace - 1) + Edit.Replace$ + MID$(EditLine$, Replace + LEN(Message.Number$))
        EditLine$ = LEFT$(EditLine$, 79) ' truncate message line
        ' recalculate next position index for search
        Replace.Word = Replace + LEN(Edit.Replace$)
        Replacements = Replacements + 1 ' increment number of replacements made
        IF Replacements = 1 THEN ' check first replacement
            IF Replace.All = False THEN ' check flag to replace all searches
                EXIT DO ' exit loop after only one replacement made
            END IF ' end check replacement flag
        END IF ' end check replacement number
    LOOP ' end replacement loop
    Array(Message.Number) = EditLine$ ' store edited message line
    SELECT CASE Replacements ' selectionn of number of replacements made
        CASE 0 ' no replacements
            Outpt = "No replacements made." ' make display message
        CASE 1 ' one replacement
            Outpt = "One replacement made." ' make display message
        CASE ELSE ' more than one replacement
            Outpt = MID$(STR$(Replacements), 2) + " replacements made." ' make message
    END SELECT ' end selection of number of replacements
    CALL IO.O ' send display message
END IF ' end check number bounds
END SUB ' end routine to edit message line

REM * routine to delete range of lines from message text.

SUB Delete.Line
Outpt = "Enter line numbers:" ' make display message
CALL IO.O ' send message
Messages.Max = Message.Length ' store length of message
' routine to get range of numbers
CALL Get.Range(Messages.Max, Message.Line1, Message.Line2)
' input prompt
Outpt = "Delete lines" + STR$(Message.Line1) + " to" + STR$(Message.Line2) + "(y/n)? "
No.Input.Out = "Y" ' default input
CALL IO.I ' get user input
IF No THEN ' check input response
    EXIT SUB ' exit routine
END IF ' end check input response
' loop through the total number of lines to delete
FOR Message.Index = 1 TO Message.Line2 - Message.Line1 + 1
    Message.Length = Message.Length - 1 ' decrement the length of message
    ' loop from the first line to delete
    FOR Array.Index = Message.Line1 TO Message.Length
        Array(Array.Index) = Array(Array.Index + 1) ' packing the remaining message
    NEXT ' end loop through deleted lines
NEXT ' end loop through number of lines to delete
IF Message.Length = False THEN ' check length of message
    Outpt = "No message left." ' make error message
    CALL IO.O ' send message
    EXIT SUB ' exit routine
END IF ' end check message length
Outpt = "Line numbers" + STR$(Message.Line1) + " to" + STR$(Message.Line2) + " deleted." ' make message
CALL IO.O ' send message
END SUB ' end routine to delete range of lines

REM * routine to insert lines into message text.

SUB Insert.Lines
IF Message.Length = 64 THEN ' check length of message
    Outpt = "Message buffer full." ' make message
    CALL IO.O ' send message
    EXIT SUB ' exit routine
END IF ' end check message length
Messages.Max = Message.Length ' store message length
Outpt = "Before line number" ' make input range prompt
CALL Get.Range2(0, Messages.Max, Message.Line) ' routine to get number
IF Message.Line > False THEN ' check number bounds
    Graphics.Off = True ' reset color
    User.Word.Wrap = UserRecord.Wordwrap ' store user word wrap
    UserRecord.Wordwrap = False ' reset word wrap
    DO WHILE Message.Length < 64 ' loop while message length range
        Word.Wrap = True ' set word wrap flag
        Outpt = "?" ' make input prompt
        CALL IO.I ' get user input
        Word.Wrap = False ' reset word wrap flag
        IF No.Input THEN ' check length of input flag
            EXIT DO ' exit message netry loop
        END IF ' end check input length
        ' loop backwards through message
        FOR Array.Index = Message.Length TO Message.Line STEP -1
            ' pack lines before inserted line
            Array(Array.Index + 1) = Array(Array.Index)
        NEXT ' end backward loop
        Inpt = RTRIM$(Inpt) ' trim input
        Inpt = LEFT$(Inpt, 79) ' truncate input
        Array(Message.Line) = Inpt ' store new inserted message line
        Message.Line = Message.Line + 1 ' increment line number to insert before
        Message.Length = Message.Length + 1 ' increment length of message
    LOOP ' end message insert loop
    UserRecord.Wordwrap = User.Word.Wrap ' restore word wrap
    Graphics.Off = False ' reset color
    IF Message.Length = 64 THEN ' compare length of message
        Outpt = "Message buffer full." ' make buffer message
        CALL IO.O ' send message
    END IF ' end check length of message
END IF ' end check number bounds
END SUB ' end insert routine

REM * routine to list range of message text lines.

SUB List.Lines
Outpt = "Enter line numbers:" ' make display message
Messages.Max = Message.Length ' store length of message
' routine to get range of numbers
CALL Get.Range(Messages.Max, Message.Line1, Message.Line2)
Outpt = "Display line numbers(y/n)? " ' make input prompt
No.Input.Out = "Y" ' default input
CALL IO.I ' get user input
ListLines = Yes ' store response
Allow.Break = True ' turn on allow break flag
Break = False ' reset control-k flag
Continue = False ' reset continuous flag
Page.Length = False ' reset page counter
Graphics.Off = True ' reset color
' loop through range of message lines
User.Line.Length = UserRecord.Linelength ' store user line length
UserRecord.Linelength = False ' store line length
FOR Message.Line = Message.Line1 TO Message.Line2
    IF ListLines THEN ' check line number list flag
        Outpt = MID$(STR$(Message.Line), 2) + ":" ' make line number
    ELSE ' check line number flag
        Outpt = Nul ' reset line number string
    END IF ' end check line number flag
    Outpt = Outpt + Array(Message.Line) ' make line display
    Outpt = LEFT$(Outpt, 79) ' truncate line
    CALL IO.O ' send line display
    IF Break THEN ' check control-k pressed
        EXIT FOR ' exit text input loop
    END IF ' end check control-k
    Page.Length = Page.Length + 1 ' increment page counter
    IF Page.Length = UserRecord.Pagelength THEN ' check page counter
        Page.Length = False ' reset page counter
        IF Continue = False THEN ' check continuous flag
            CALL More.Prompt ' routine to pause
            IF No THEN ' check pause response
                EXIT FOR ' exit routine
            END IF ' end check response
        END IF ' end check continuous flag
    END IF ' end check page counter
NEXT ' end loop through range of lines
UserRecord.Linelength = User.Line.Length ' restore line length
Allow.Break = False ' reset allow break flag
IF Break THEN ' check control-k flag
    Break = False ' reset control-k flag
    Outpt = Nul ' set output to null
    CALL IO.O ' send empty return
END IF ' end check control-k flag
IF Page.Length THEN ' recheck page counter
    CALL More.Prompt ' pause routine
END IF ' end recheck counter
END SUB ' end list lines routine

REM * routine to edit rooms.

SUB Edit.Room
CALL Share.Room.Record(Room) ' store current room
DO ' loop through room edit menu
    Graphics.Off = False ' reset color
    Outpt = "Room edit:" ' make output display
    CALL IO.O ' send output
    Graphics.Off = True ' reset color
    Outpt = "[A]dd" ' make output display
    CALL IO.O ' send output
    Outpt = "[C]hange" ' make output display
    CALL IO.O ' send output
    Outpt = "[L]ist" ' make output display
    CALL IO.O ' send output
    Graphics.Off = False ' reset color
    Outpt = "Enter room edit option(q to quit)? " ' make input prompt
    No.Input.Out = "Q" ' default input
    CALL IO.I
    SELECT CASE UCASE$(Inpt) ' selection of input
        CASE "A" ' option to add new room
            CALL Add.Room(False, Room.Added) ' routine to add room
        CASE "C" ' option to select another room number to edit
            Outpt = "Enter room number" ' prompt
            Max.Rooms! = LOF(RoomFile) / LEN(RoomRecord) ' store length of room records
            ' routine to get number from range
            CALL Get.Room.Range2(1!, Max.Rooms!, Room.Number!)
            CALL Change.Room(Room.Number!) ' routine to edit room descriptions
        CASE "L" ' display room description
            Max.Rooms! = LOF(RoomFile) / LEN(RoomRecord) ' store length of room record
            ' routine to get range to display
            CALL Get.Room.Range(Max.Rooms!, Start.Room!, End.Room!)
            ' loop through range of room numbers
            Allow.Break = True ' set allow break flag
            Break = False ' reset control-k flag
            Continue = False ' set continuous flag
            FOR Room.Number! = Start.Room! TO End.Room!
                CALL Read.Room.Record(Room.Number!) ' get room record
                CALL Display.Room.Desc(Room.Number!) ' routine to display room
                IF Break THEN ' check break flag
                    EXIT FOR ' exit display loop
                END IF
                Graphics.Off = False ' reset color
                IF Continue = False THEN ' check continuous flag
                    CALL More.Prompt ' pause prompt
                    IF No THEN ' compare continue
                        EXIT FOR ' exit loop through rooms
                    END IF ' end compare
                END IF ' end check continuous flag
            NEXT ' end loop through rooms
            Allow.Break = False ' reset allow break flag
            IF Break THEN ' check control-k flag
                Break = False ' reset control-k flag
                Outpt = Nul ' set output to null
                CALL IO.O ' send empty return
            END IF ' end check control-k flag
        CASE "Q" ' option to exit menu
            CALL Read.Room.Record(Room) ' get current room
            EXIT DO ' exit edit menu
    END SELECT ' end input selection
LOOP ' end room edit menu
END SUB ' end room edit routine

REM * routine to read !edit help.

SUB Edit.Help
DO ' help menu loop
    Graphics.Off = False ' reset color
    Outpt = "Edit help:" ' make output message
    CALL IO.O ' send output message
    Graphics.Off = True ' reset color
    Outpt = "[C]ontents" ' make output message
    CALL IO.O ' send output message
    Outpt = "[T]opic" ' make output message
    CALL IO.O ' send output message
    Graphics.Off = False ' reset color
    No.Input.Out = "Q" ' default input
    Outpt = "Enter help option(q to quit)? " ' make input prompt
    CALL IO.I ' get user input
    SELECT CASE UCASE$(Inpt) ' selection of input
        CASE "C" ' display !edit help contents
            Stored.Parsed.Command1 = "contents" ' store help topic
            CALL Read.Help(1) ' routine to read !edit help
        CASE "T" ' select help topic number
            Outpt = "Enter help topic number sequence? " ' make input prompt
            CALL IO.I ' get user input
            Stored.Parsed.Command1 = Inpt ' store help topic
            CALL Read.Help(1) ' routine to read !edit help
        CASE "Q" ' exit menu loop
            EXIT DO ' exit loop
    END SELECT ' end input selection
LOOP ' end menu loop
END SUB ' end routine to read !edit help

REM * routine to edit container.
REM * input variables:
REM *   Container.Type     -1 for room/0 for user
REM *   ContainerRec    structure of container to edit

SUB Change.Container (Container.Type)
DO ' loop through room treasure edit menu
    Graphics.Off = False ' reset color
    IF Container.Type THEN ' check call type
        Outpt = "Room container edit options:" ' make display message
    ELSE ' check type
        Outpt = "User container edit options:" ' make display message
    END IF ' end check type
    CALL IO.O ' send message
    Graphics.Off = True ' reset color
    Outpt = "[A]container name" ' make display message
    CALL IO.O ' send message
    Outpt = "[B]container lock" ' make display message
    CALL IO.O ' send message
    Outpt = "[C]permanent" ' make display message
    IF ContainerRec.Permanent THEN ' check container value
        Outpt = Outpt + ": yes" ' append to message
    ELSE ' check value
        Outpt = Outpt + ": no" ' append to message
    END IF ' end check value
    CALL IO.O ' send message
    Outpt = "[D]inventory" ' make display message
    Number.Items = False ' reset counter
    FOR Treasure.Number = 1 TO 5 ' loop through items
        Treasure.Item = GetContainerRecInventory(Treasure.Number) ' store item index
        ' compare item range
        IF Treasure.Item > False AND Treasure.Item <= LOF(TreasureFile) / LEN(TreasureRecord) THEN
            Number.Items = Number.Items + 1 ' increment counter
        END IF ' end check range
    NEXT ' end item loop
    Outpt = Outpt + ":" + STR$(Number.Items)
    IF Number.Items = 1 THEN ' check counter
        Outpt = Outpt + " item." ' append message
    ELSE ' check counter
        Outpt = Outpt + " items." ' append message
    END IF ' end check counter
    CALL IO.O ' send message
    Graphics.Off = False ' reset color
    IF Container.Type THEN ' check call type
        Outpt = "Room container edit option(q to quit)? " ' make input prompt
    ELSE ' check type
        Outpt = "User container edit option(q to quit)? " ' make input prompt
    END IF ' end check type
    No.Input.Out = "Q" ' default input
    CALL IO.I ' get user input
    SELECT CASE UCASE$(Inpt) ' selection of room container option
        CASE "A" ' option to change container name
            Outpt = "Container name?" ' container input
            CALL IO.I ' get response
            Inpt = RTRIM$(Inpt) ' trim value
            IF Inpt <> Nul THEN ' compare value
                ContainerRec.ContainerName = LCASE$(Inpt) ' container name from treasure.dat file
            END IF ' end compare value
            Outpt = "Container mnemonic name?" ' container input
            CALL IO.I ' get response
            Inpt = RTRIM$(Inpt) ' trim value
            IF Inpt <> Nul THEN ' compare value
                ContainerRec.ShortName = UCASE$(Inpt) ' mnemonic of container name from treasure.dat file
            END IF ' end compare value
        CASE "B" ' option to change container lock/key
            Outpt = "Container locked(y/n)?" ' container input
            CALL IO.I ' get input
            IF Yes THEN ' check value
                ContainerRec.Locked = 1 ' locked flag 1=locked, 0=n/a, -1=unlocked
                ContainerRec.Closed = 1 ' closed flag 1=closed, 0=n/a, -1=open
            ELSE ' check value
                ContainerRec.Locked = 0 ' locked flag 1=locked, 0=n/a, -1=unlocked
                ContainerRec.Closed = 0 ' closed flag 1=closed, 0=n/a, -1=open
            END IF ' end check value
            Outpt = "Container has key(y/n)?" ' container input
            CALL IO.I ' get input
            IF Yes THEN ' check value
                Outpt = "Container key number(1-32767)?" ' input value
                CALL IO.I ' get input
                Key.Number = INT(VAL(Inpt)) ' store input
                IF Key.Number > False AND Key.Number <= MaxInt THEN ' check value
                    ContainerRec.Keyed = Key.Number ' container key number 1-32767
                END IF ' end check value
            ELSE ' check value
                ContainerRec.Keyed = False ' container key number 1-32767
            END IF ' end check value
        CASE "C" ' option to change permanent container
            Outpt = "Container is permanent(y/n)?" ' container input
            CALL IO.I ' get input
            ContainerRec.Permanent = Yes ' flag for permanent container (added 05/25/01)
        CASE "D" ' option to edit container contents
            DO ' loop through container treasure edit menu
                Graphics.Off = False ' reset color
                IF Container.Type THEN ' check call type
                    Outpt = "Room container treasure edit options:" ' make display message
                ELSE ' check type
                    Outpt = "User container treasure edit options:" ' make display message
                END IF ' end check type
                CALL IO.O ' send message
                Graphics.Off = True ' reset color
                Outpt = "[A]dd" ' make display message
                CALL IO.O ' send message
                Outpt = "[D]elete" ' make display message
                CALL IO.O ' send message
                Outpt = "[L]ist" ' make display message
                CALL IO.O ' send message
                Graphics.Off = False ' reset color
                IF Container.Type THEN ' check call type
                    Outpt = "Room container treasure edit option(q to quit)? " ' make input prompt
                ELSE ' check type
                    Outpt = "User container treasure edit option(q to quit)? " ' make input prompt
                END IF ' end check type
                No.Input.Out = "Q" ' default input
                CALL IO.I ' get user input
                SELECT CASE UCASE$(Inpt) ' selection of room container option
                    CASE "A" ' option to add container treasure
                        Outpt = "Enter item number(1-5)?" ' get item number
                        CALL IO.I ' get input
                        Container.Slot = INT(VAL(Inpt)) ' store input
                        IF Container.Slot > False AND Container.Slot <= 5 THEN ' check range
                            Treasure.Item = GetContainerRecInventory(Container.Slot) ' store item index
                            IF Treasure.Item = False THEN ' compare value
                                CALL Find.Treasure(Treasure.Number)
                                IF Treasure.Number > False THEN ' check number
                                    CALL TreasureCharges(Treasure.Charges) ' add treasure
                                    CALL PutContainerRecInventory(Container.Slot, Treasure.Number) ' index to treasure.dat of container contents
                                    CALL PutContainerRecCharges(Container.Slot, Treasure.Charges) ' treasure charges of container contents
                                    Outpt = "Item is invisible(y/n)?" ' container input
                                    CALL IO.I ' get input
                                    CALL PutContainerRecInvisible(Container.Slot, Yes) ' invisible flag -1/0 of container contents
                                END IF ' end check number
                            END IF ' end check treasure
                        END IF ' end check range
                    CASE "D" ' option to delete container treasure
                        Outpt = "Enter item number(1-5)?" ' get item number
                        CALL IO.I ' get input
                        Treasure.Number = INT(VAL(Inpt)) ' store input
                        IF Treasure.Number > False AND Treasure.Number <= 5 THEN ' check range
                            Treasure.Item = GetContainerRecInventory(Treasure.Number) ' store item index
                            IF Treasure.Item THEN ' compare value
                                CALL PutContainerRecInventory(Treasure.Number, False) ' reset container
                                CALL PutContainerRecCharges(Treasure.Number, False) ' reset container
                                CALL PutContainerRecInvisible(Treasure.Number, False) ' reset container
                                Outpt = "Treasure item deleted." ' format output
                                CALL IO.O ' send output
                            END IF ' end check value
                        END IF ' end check range
                    CASE "L" ' option to list container treasure
                        Graphics.Off = True ' reset color
                        FOR Treasure.Number = 1 TO 5 ' loop through items
                            Treasure.Item = GetContainerRecInventory(Treasure.Number) ' store item index
                            ' compare item range
                            IF Treasure.Item > False AND Treasure.Item <= LOF(TreasureFile) / LEN(TreasureRecord) THEN
                                ' display treasure name and number
                                CALL Read.Record(TreasureFile, Treasure.Item) ' get treasure name
                                Outpt = RTRIM$(TreasureRecord.TreasureName) ' store treasure name
                                MID$(Outpt, 1, 1) = UCASE$(MID$(Outpt, 1, 1)) ' uppercase first letter
                                Outpt = "#" + MID$(STR$(Treasure.Number), 2) + ":" + Outpt ' prepend container number
                                CALL IO.O ' send output
                            END IF ' end check range
                        NEXT ' end item loop
                    CASE "Q" ' option to exit room container edit menu
                        EXIT DO ' exit room container treasure edit menu
                END SELECT ' end selection of room container treasure edit menu
            LOOP ' end loop through room container treasure edit option menu
        CASE "Q" ' option to exit room container treasure edit menu
            EXIT DO ' exit room treasure edit menu
    END SELECT ' end selection of room treasure menu
LOOP ' end loop through room container edit option menu
END SUB ' end routine to edit container

REM * routine to edit room description and monster class.
REM * input variables:
REM *   Room.Number! - room number to edit.

SUB Change.Room (Room.Number!)
CALL Read.Room.Record(Room.Number!) ' get room record to edit
DO ' loop while edit
    CALL Display.Room.Desc(Room.Number!) ' routine to display room
    Graphics.Off = False ' reset color
    Outpt = "Room edit options:" ' make display output
    CALL IO.O ' send output
    Graphics.Off = True ' reset color
    Outpt = "[A]ction" ' make display message
    CALL IO.O ' send message
    Outpt = "[C]ontainer" ' make display message
    ContainerRec = RoomRecord.Container ' store room container into record
    IF RTRIM$(ContainerRec.ShortName) = Nul THEN ' compare container name length
        Outpt = Outpt + ": no" ' append message
    ELSE ' check container name
        Outpt = Outpt + ": yes" ' append message
    END IF ' end check container name
    CALL IO.O ' send message
    Outpt = "[D]escription" ' make display message
    CALL IO.O ' send message
    IF Room.Number! > 1! THEN ' check safe haven room
        Outpt = "[M]onster class" ' make display message
        CALL IO.O ' send message
    END IF ' end check room number
    IF Room.Number! = 1! THEN ' check safe haven room
        Outpt = "[R]esurrection room" ' make message
        CALL IO.O ' send message
    END IF ' end check room number
    Outpt = "[O]bjects:" ' make display message
    Number.Items = False ' reset counter
    FOR Array.Index = 1 TO 20 ' loop through room objects
        Object.Number = GetRoomRecordObject(Array.Index) ' store object index
        IF Object.Number > False AND Object.Number <= LOF(ObjectFile) / LEN(ObjectRecord) THEN ' bounds
            Number.Items = Number.Items + 1 ' increment counter
        END IF ' end check object bounds
    NEXT ' end count objects
    IF Number.Items = 1 THEN ' check value
        Outpt = Outpt + STR$(Number.Items) + " item." ' make display message
    ELSE ' check value
        Outpt = Outpt + STR$(Number.Items) + " items." ' make display message
    END IF ' end check value
    CALL IO.O ' send message
    Outpt = "[T]reasure:" ' make display message
    Number.Items = False ' reset counter
    FOR Array.Index = 1 TO 20 ' loop through room treasure
        ' store room treasure number
        Treasure.Number = GetRoomRecordTreasure(Array.Index)
        IF Treasure.Number > False AND Treasure.Number <= LOF(TreasureFile) / LEN(TreasureRecord) THEN
            Number.Items = Number.Items + 1 ' increment counter
        END IF ' end check treasure bounds
    NEXT ' end treasure objects
    IF Number.Items = 1 THEN ' check value
        Outpt = Outpt + STR$(Number.Items) + " item." ' make display message
    ELSE ' check value
        Outpt = Outpt + STR$(Number.Items) + " items." ' make display message
    END IF ' end check value
    CALL IO.O ' send message
    Graphics.Off = False ' reset color
    Outpt = "Room edit option(q to quit)? " ' make input prompt
    No.Input.Out = "Q" ' default input
    CALL IO.I ' get user input
    SELECT CASE UCASE$(Inpt) ' selection of room edit option
        CASE "A" ' option to change room action number
            Outpt = "Enter action number" ' prompt
            Max.Action = LOF(ActionFile) / LEN(ActionRecord) ' store length of room
            ' routine to get number from range
            CALL Get.Range2(0, Max.Action, Action.Number)
            RoomRecord.Action = Action.Number ' store action number
        CASE "C" ' option to change room container
            DO ' loop through room container edit menu
                Graphics.Off = False ' reset color
                Outpt = "Room container edit options:" ' make display message
                CALL IO.O ' send message
                Graphics.Off = True ' reset color
                Outpt = "[A]dd container" ' make option message
                CALL IO.O ' send option message
                Outpt = "[C]hange container" ' make option message
                CALL IO.O ' send option message
                Outpt = "[D]elete container" ' make option message
                CALL IO.O ' send option message
                Outpt = "[L]ist container" ' make option message
                CALL IO.O ' send option message
                Graphics.Off = False ' reset color
                Outpt = "Enter room container edit option(q to quit)? " ' make option prompt
                No.Input.Out = "Q" ' store default input
                CALL IO.I ' get option input
                SELECT CASE UCASE$(Inpt) ' make selection of input option
                    CASE "A" ' add container
                        Treasure.Added = False ' set container added flag
                        ContainerRec = RoomRecord.Container ' store room container into record
                        IF RTRIM$(ContainerRec.ShortName) = Nul THEN ' compare container name length
                            ' routine to get treasure number
                            CALL Find.Treasure(Treasure.Number)
                            IF Treasure.Number > False THEN ' check treasure number
                                IF TreasureRecord.Container THEN ' check treasure type
                                    ' store container variables
                                    ContainerRec.Closed = TreasureRecord.Closed
                                    ContainerRec.ContainerName = TreasureRecord.TreasureName
                                    ContainerRec.Locked = TreasureRecord.Locked
                                    ContainerRec.Keyed = TreasureRecord.Keyed
                                    ContainerRec.ShortName = TreasureRecord.ShortName
                                    ContainerRec.Permanent = TreasureRecord.Permanent
                                    FOR Container.Count = 1 TO 5 ' loop through container contents
                                        ' clear container contents
                                        CALL Clear.Container(Container.Count, False)
                                    NEXT ' end loop through container
                                    RoomRecord.Container = ContainerRec ' add container record to room
                                    Treasure.Added = True ' set container added flag
                                END IF ' end check treasure type
                            END IF ' end check treasure number
                        END IF ' end check container name length
                        IF Treasure.Added THEN ' check added flag
                            Outpt = "Container added to room." ' make message
                        ELSE ' check added flag
                            Outpt = "Container not added to room." ' make message
                        END IF ' end check added flag
                        CALL IO.O ' display message
                    CASE "C" ' change container
                        ContainerRec = RoomRecord.Container ' store room container into record
                        CALL Change.Container(-1) ' call container edit menu
                        RoomRecord.Container = ContainerRec ' store container into room record
                    CASE "D" ' delete container
                        Treasure.Deleted = False ' set container deleted flag
                        ContainerRec = RoomRecord.Container ' store room container into record
                        IF RTRIM$(ContainerRec.ShortName) <> Nul THEN ' compare container name length
                            CALL Clear.Container(0, True) ' clear entire container record
                            RoomRecord.Container = ContainerRec ' clear room container
                            Treasure.Deleted = True ' reset container deleted flag
                        END IF ' end compare container in room
                        IF Treasure.Deleted THEN ' check deleted flag
                            Outpt = "Container deleted from room." ' make message
                        ELSE ' check added flag
                            Outpt = "Container not deleted from room." ' make message
                        END IF ' end check added flag
                        CALL IO.O ' display message
                    CASE "L" ' list container
                        Graphics.Off = True ' reset color
                        ContainerRec = RoomRecord.Container ' store room container into record
                        IF RTRIM$(ContainerRec.ShortName) <> Nul THEN ' compare container name length
                            Outpt = RTRIM$(ContainerRec.ContainerName) ' store container name
                            MID$(Outpt, 1, 1) = UCASE$(MID$(Outpt, 1, 1)) ' uppercase first letter
                            IF ContainerRec.Locked > False THEN ' check container is locked
                                Outpt = Outpt + "[locked]" ' append message
                            ELSE ' container is not locked
                                IF ContainerRec.Closed > False THEN ' check container is closed
                                    Outpt = Outpt + "[closed]" ' append message
                                END IF ' end check closed container
                            END IF ' end check locked container
                            IF ContainerRec.Keyed THEN ' check container has a key number
                                Outpt = Outpt + "(#" + RIGHT$(STR$(ContainerRec.Keyed + 100000!), 5) + ")"
                            END IF ' append key number to container name
                            Number.Items = False ' reset counter
                            FOR Treasure.Number = 1 TO 5 ' loop through items
                                Treasure.Item = GetContainerRecInventory(Treasure.Number) ' store item index
                                ' compare item range
                                IF Treasure.Item > False AND Treasure.Item <= LOF(TreasureFile) / LEN(TreasureRecord) THEN
                                    Number.Items = Number.Items + 1 ' increment counter
                                END IF ' end check range
                            NEXT ' end item loop
                            Outpt = Outpt + ":" + STR$(Number.Items)
                            IF Number.Items = 1 THEN ' check counter
                                Outpt = Outpt + " item." ' append message
                            ELSE ' check counter
                                Outpt = Outpt + " items." ' append message
                            END IF ' end check counter
                        ELSE ' check for container
                            Outpt = "<no container>" ' make message
                        END IF ' end compare container name length
                        CALL IO.O ' display message
                    CASE "Q" ' quit container menu
                        EXIT DO
                END SELECT
            LOOP ' end room container loop
        CASE "D" ' option to edit room descriptions
            DO ' loop through room description edit menu
                Graphics.Off = True ' reset color
                Outpt = "[L]ong description" ' make option message
                CALL IO.O ' send option message
                Outpt = "[S]hort description" ' make option message
                CALL IO.O ' send option message
                Graphics.Off = False ' reset color
                Outpt = "Enter room edit option(q to quit)? " ' make option prompt
                No.Input.Out = "Q" ' store default input
                CALL IO.I ' get option input
                SELECT CASE UCASE$(Inpt) ' make selection of input option
                    CASE "L" ' edit long description
                        Graphics.Off = False ' reset color
                        Outpt = "Edit room long description(y/n)? " ' input prompt
                        No.Input.Out = "N" ' default input
                        CALL IO.I ' get input
                        IF Yes THEN ' compare input
                            Graphics.Off = False ' reset color
                            Outpt = "Enter four lines for long description:" ' make message
                            CALL IO.O ' send edit message
                            Outpt = "Press <enter> when done." ' make edit message
                            CALL IO.O ' send edit message
                            Graphics.Off = True ' reset color
                            FOR Room.Desc = 1 TO 4 ' loop through room long description
                                ' clear room long description
                                CALL PutRoomRecordLongDesc(Room.Desc, Nul)
                            NEXT ' end loop through room
                            User.Word.Wrap = UserRecord.Wordwrap ' store user word wrap
                            UserRecord.Wordwrap = False ' reset word wrap
                            Word.Wrap = True ' enable word wrap
                            FOR Room.Desc = 1 TO 4 ' loop through input for long description
                                Outpt = "?" ' make input prompt
                                IF Room.Desc = 4 THEN ' check last long description line
                                    Word.Wrap = False ' disable word wrap
                                END IF ' end check last input
                                CALL IO.I ' get input
                                IF No.Input THEN ' check empty cr/lf entered
                                    EXIT FOR ' exit description edit loop
                                END IF ' end check empty input
                                ' store long description
                                CALL PutRoomRecordLongDesc(Room.Desc, Inpt)
                            NEXT ' end loop through input
                            Word.Wrap = False ' disable word wrap
                            UserRecord.Wordwrap = User.Word.Wrap ' restore word wrap
                        END IF ' end compare input
                    CASE "S" ' edit short description
                        Graphics.Off = False ' reset color
                        Outpt = "Edit room short decription(y/n)? " ' input prompt
                        No.Input.Out = "N" ' default input
                        CALL IO.I ' get input
                        IF Yes THEN ' compare input
                            Graphics.Off = False ' reset color
                            Outpt = "Enter short description(78 characters):" ' make message
                            CALL IO.O ' send message
                            Outpt = "Press <enter> to leave unchanged." ' make message
                            CALL IO.O ' send message
                            Graphics.Off = True ' reset color
                            Line.Length = 78 ' set length of input
                            Outpt = "?" ' set input prompt
                            CALL IO.I ' get input
                            IF No.Input = False THEN ' check length of input
                                RoomRecord.ShortDesc = Inpt ' store room short description
                            END IF ' end check input length
                        END IF ' end compare input
                    CASE "Q" ' quit
                        EXIT DO ' exit prompt loop
                END SELECT ' end selection of input
            LOOP ' end loop through selection input
        CASE "R" ' check resurrection room number
            IF Room.Number! = 1! THEN ' check safe haven room
                Outpt = "Enter resurrection room number(1-32767)?" ' make message
                CALL IO.I ' get input
                RoomRecord.MonsterClass = INT(VAL(Inpt)) ' reset resurrection room
                Resurrection.Room = INT(VAL(Inpt)) ' store resurrection room
            END IF ' end check room number
        CASE "M" ' edit monster class
            IF Room.Number! > 1! THEN ' check safe haven room
                Outpt = "Enter monster class" ' make input prompt
                CALL Get.Range2(0, Monclass.Max, Monclass.Number) ' get number from range
                RoomRecord.MonsterClass = Monclass.Number ' store new monster class
                Outpt = "Monster class" + STR$(Monclass.Number) + " added to room" + STR$(Room.Number!) + "."
                CALL IO.O ' send display message
            END IF ' end check room number
        CASE "O" ' option to edit room objects
            DO ' loop through room object editing
                Graphics.Off = False ' reset color
                Outpt = "Room object options:" ' make display message
                CALL IO.O ' send message
                Graphics.Off = True ' reset color
                Outpt = "[A]dd" ' make display message
                CALL IO.O ' send message
                Outpt = "[D]elete" ' make display message
                CALL IO.O ' send message
                Outpt = "[L]ist" ' make display message
                CALL IO.O ' send message
                Graphics.Off = False ' reset color
                Outpt = "Room object edit option(q to quit)? " ' make input prompt
                No.Input.Out = "Q" ' default input
                CALL IO.I ' get user input
                SELECT CASE UCASE$(Inpt) ' selection of room object edit option
                    CASE "A" ' option to add room object
                        Object.Added = False ' object added flag
                        CALL Find.Objects(Object.Number) ' routine to get object number
                        IF Object.Number > False THEN ' check object number
                            FOR Array.Index = 1 TO 20 ' loop through room object inventory
                                ' check empty object inventory
                                IF GetRoomRecordObject(Array.Index) = False THEN
                                    CALL PutRoomRecordObject(Array.Index, Object.Number) ' add object index
                                    Object.Added = True ' set return flag
                                    EXIT FOR ' exit room object loop
                                END IF ' end check empty object inventory
                            NEXT ' end loop through room object inventory
                        END IF ' end check object number
                        IF Object.Added THEN ' check object added flag
                            Outpt = "Object added to room." ' make message
                        ELSE ' check object added flag
                            Outpt = "Object not added to room." ' make message
                        END IF ' end check object added flag
                        CALL IO.O ' send message
                    CASE "D" ' option to delete room object
                        Outpt = "Object number to delete" ' make range prompt
                        CALL Get.Range2(0, 10, Object.Number) ' get number from range
                        IF Object.Number > False THEN ' check number bounds
                            CALL PutRoomRecordObject(Object.Number, False) ' clear treasure items
                            Outpt = "Object " + STR$(Object.Number) + " deleted from room." ' make message
                            CALL IO.O ' send message
                        END IF ' end check number bounds
                    CASE "L" ' option to list objects in room
                        Objects.Counted = False ' reset counter
                        Graphics.Off = True ' reset color
                        FOR Array.Index = 1 TO 20 ' loop through room objects
                            Object.Number = GetRoomRecordObject(Array.Index) ' store object index
                            IF Object.Number > False AND Object.Number <= LOF(ObjectFile) / LEN(ObjectRecord) THEN ' bounds
                                Objects.Counted = True ' set counter
                                CALL Read.Record(ObjectFile, Object.Number) ' read object record
                                Outpt = "[" + MID$(STR$(Array.Index), 2) + "]" + RTRIM$(ObjectRecord.ObjectName) ' make object name display
                                CALL IO.O ' send object message
                            END IF ' end check object file bounds
                        NEXT ' end loop through room objects
                        IF Objects.Counted THEN ' check counter
                            CALL More.Prompt ' display more prompt
                        ELSE ' check counter
                            Outpt = "<no objects>" ' make display message
                            CALL IO.O ' send message
                        END IF ' end check counter
                    CASE "Q" ' option to exit room object edit menu
                        EXIT DO ' exit room object edit menu
                END SELECT ' end select room object edit options
            LOOP ' end loop through room object option menu
        CASE "T" ' option to edit room treasure
            DO ' loop through room treasure edit menu
                Graphics.Off = False ' reset color
                Outpt = "Room treasure options:" ' make display message
                CALL IO.O ' send message
                Graphics.Off = True ' reset color
                Outpt = "[A]dd" ' make display message
                CALL IO.O ' send message
                Outpt = "[D]elete" ' make display message
                CALL IO.O ' send message
                Outpt = "[L]ist" ' make display message
                CALL IO.O ' send message
                Graphics.Off = False ' reset color
                Outpt = "Room treasure edit option(q to quit)? " ' make input prompt
                No.Input.Out = "Q" ' default input
                CALL IO.I ' get user input
                SELECT CASE UCASE$(Inpt) ' selection of room treasure option
                    CASE "A" ' option to add room treasure
                        Treasure.Added = False ' set treasure added flag
                        ' routine to get treasure number
                        CALL Find.Treasure(Treasure.Number)
                        IF Treasure.Number > False THEN ' check treasure number
                            IF TreasureRecord.Container = False THEN ' check treasure type
                                FOR Array.Index = 1 TO 20 ' loop through room treasure inventory
                                    ' check empty room inventory
                                    IF GetRoomRecordTreasure(Array.Index) = False THEN
                                        CALL PutRoomRecordTreasure(Array.Index, Treasure.Number) ' add treasure index
                                        ' add treasure charges
                                        CALL TreasureCharges(Charges.Number)
                                        CALL PutRoomRecordTreCharges(Array.Index, Charges.Number)
                                        CALL PutRoomRecordFlags(Array.Index, False) ' add treasure flags
                                        Treasure.Added = True ' set return flag
                                        EXIT FOR ' exit room inventory loop
                                    END IF ' end check empty inventory
                                NEXT ' end loop through room treasure inventory
                            END IF ' end check treasure type
                        END IF ' end check treasure number
                        IF Treasure.Added THEN ' check treasure added flag
                            Outpt = "Treasure added to room." ' make message
                        ELSE ' check flag
                            Outpt = "Treasure not added to room." ' make message
                        END IF ' end check treasure added flag
                        CALL IO.O ' send message
                    CASE "D" ' option to delete treasure number
                        Outpt = "Treasure number to delete" ' make range prompt
                        CALL Get.Range2(0, 10, Treasure.Number) ' get number from range
                        IF Treasure.Number > False THEN ' check number bounds
                            CALL PutRoomRecordTreasure(Treasure.Number, False) ' clear treasure items
                            CALL PutRoomRecordTreCharges(Treasure.Number, False) ' clear treasure items
                            CALL PutRoomRecordFlags(Treasure.Number, False) ' clear treasure items
                            Outpt = "Treasure " + STR$(Treasure.Number) + " deleted from room." ' make message
                            CALL IO.O ' send message
                        END IF ' end check number bounds
                    CASE "L" ' option to list treasure in room
                        Treasure.Counted = False ' reset counter
                        Graphics.Off = True ' reset color
                        FOR Array.Index = 1 TO 20 ' loop through room treasure
                            ' store room treasure number
                            Treasure.Number = GetRoomRecordTreasure(Array.Index)
                            IF Treasure.Number > False AND Treasure.Number <= LOF(TreasureFile) / LEN(TreasureRecord) THEN
                                Treasure.Counted = True ' increment counter
                                CALL Read.Record(TreasureFile, Treasure.Number) ' read treasure
                                Outpt = "[" + MID$(STR$(Array.Index), 2) + "]" + RTRIM$(TreasureRecord.TreasureName) ' make treasure name
                                CALL IO.O ' send treasure name message
                            END IF ' end check file bounds
                        NEXT ' end loop through room treasure
                        IF Treasure.Counted THEN ' check counter
                            CALL More.Prompt ' display more prmpt
                        ELSE ' check counter
                            Outpt = "<no treasure>" ' make display message
                            CALL IO.O ' send message
                        END IF ' end check counter
                    CASE "Q" ' option to exit room treasure edit menu
                        EXIT DO ' exit room treasure edit menu
                END SELECT ' end selection of room treasure menu
            LOOP ' end loop through room treasure edit option menu
        CASE "Q" ' option to exit room edit menu
            CALL Share.Room.Record(Room.Number!) ' write current room number
            EXIT DO ' exit room edit menu
    END SELECT ' end selection of room edit menu
LOOP ' end loop through room edit option menu
END SUB ' end routine to edit room number

REM * routine to edit message text.
REM * output variables:
REM *   Message.Edit - true to continue, false to abort, 1 to store message.

SUB Edit.Message (Message.Edit)
DO ' loop through message edit menu
    Graphics.Off = False ' reset color
    Outpt = "Editing options:" ' make dislay message
    CALL IO.O ' send message
    Graphics.Off = True ' reset color
    Outpt = "[A]bort" ' make dislay message
    CALL IO.O ' send message
    Outpt = "[C]ontinue" ' make dislay message
    CALL IO.O ' send message
    IF Message.Length > False THEN ' check length of message
        Outpt = "[D]elete" ' make dislay message
        CALL IO.O ' send message
    END IF ' end check length
    IF Message.Length > False THEN ' check length of message
        Outpt = "[E]dit" ' make dislay message
        CALL IO.O ' send message
    END IF ' end check length
    IF Message.Length > False THEN ' check length of message
        Outpt = "[I]nsert" ' make dislay message
        CALL IO.O ' send message
    END IF ' end check length
    IF Message.Length > False THEN ' check length of message
        Outpt = "[L]ist" ' make dislay message
        CALL IO.O ' send message
    END IF ' end check length
    IF Message.Length > False THEN ' check length of message
        Outpt = "[R]eplace" ' make dislay message
        CALL IO.O ' send message
    END IF ' end check length
    IF Message.Length > False THEN ' check length of message
        Outpt = "[S]tore" ' make dislay message
        CALL IO.O ' send message
    END IF ' end check length
    Graphics.Off = False ' reset color
    Outpt = "Edit command? " ' make input prompt
    CALL IO.I ' get user input
    SELECT CASE UCASE$(Inpt) ' selection of edit option
        CASE "C" ' option to continue message
            Message.Edit = True ' set return variable
            EXIT SUB ' exit routine
        CASE "R" ' option to replace line
            IF Message.Length > False THEN ' check length of message
                CALL Replace.Line ' replace routine
            END IF ' end check length
        CASE "E" ' option to edit line
            IF Message.Length > False THEN ' check length of message
                CALL Edit.Line ' line edit routine
            END IF ' end check length
        CASE "D" ' option to delete line
            IF Message.Length > False THEN ' check length of message
                CALL Delete.Line ' delete routine
            END IF ' end check length
        CASE "I" ' option to insert lines
            IF Message.Length > False THEN ' check length of message
                CALL Insert.Lines ' insert routine
            END IF ' end check length
        CASE "A" ' option to abort message
            Outpt = "Are you sure you want to abort(y/n)? " ' make input prompt
            No.Input.Out = "N" ' default input
            CALL IO.I ' get user input
            IF Yes THEN ' check input response
                Message.Edit = False ' set return variable
                EXIT SUB ' exit routine
            END IF ' end check response
        CASE "L" ' option to list lines
            IF Message.Length > False THEN ' check length of message
                CALL List.Lines ' line list routine
            END IF ' end check length
        CASE "S" ' option to store message
            IF Message.Length > False THEN ' check length of message
                Message.Edit = UnTrue ' set return variable
                EXIT SUB ' exit routine
            END IF ' end check length
    END SELECT ' end selection of input
LOOP ' end loop through edit menu
END SUB ' end edit menu routine

REM * routine to edit messages files.

SUB Edit.Mail
DO ' loop through mail edit menu
    Graphics.Off = False ' reset color
    Outpt = "Mail edit:" ' make display message
    CALL IO.O ' send message
    Graphics.Off = True ' reset color
    Outpt = "[C]hange" ' make display message
    CALL IO.O ' send message
    Outpt = "[D]elete" ' make display message
    CALL IO.O ' send message
    Outpt = "[L]ist" ' make display message
    CALL IO.O ' send message
    Outpt = "[P]ack" ' make display message
    CALL IO.O ' send message
    Outpt = "[U]ndelete" ' make display message
    CALL IO.O ' send message
    Graphics.Off = False ' reset color
    Outpt = "Message edit option(q to quit)? " ' make input prompt
    No.Input.Out = "Q" ' default input
    CALL IO.I ' get user input
    SELECT CASE UCASE$(Inpt) ' select option
        CASE "C" ' change option
            CALL Change.Message ' routine to change message
        CASE "D" ' delete option
            CALL Delete.Message ' routine to delete message
        CASE "L" ' list option
            CALL List.Messages ' routine to list messages
        CASE "P" ' pack option
            CALL Pack.Messages ' routine to pack messages
        CASE "U" ' undelete option
            CALL Undelete.Message ' routine to undelete message
        CASE "Q" ' exit mail edit menu option
            EXIT DO ' exit edit mail menu
    END SELECT ' end mail edit option selection
LOOP ' end mail edit menu loop
END SUB ' end mail edit menu routine

REM * routine to parse out number after pound sign in a string
REM * input variables:
REM *   Parsed.Input$ - string to check.
REM * output variables:
REM *   Parsed.Input$ - string truncated before pound sign, lowercased.
REM *   Parsed.Value - number parsed after pound sign.
REM * work variables:
REM *   Parsed.Token - contains position of # sign.

SUB Parse.Num (Parsed.Input$, Parsed.Value)
Parsed.Input$ = LCASE$(Parsed.Input$) ' lowercase string
Parsed.Value = False ' set return variable
Parsed.Token = INSTR(Parsed.Input$, "#") ' search string for pound sign
IF Parsed.Token > False THEN ' check string search
    ' set return variable
    Parsed.Value = INT(VAL(MID$(Parsed.Input$, Parsed.Token + 1)))
    Parsed.Input$ = LEFT$(Parsed.Input$, Parsed.Token - 1) ' truncate string
END IF ' end check string
END SUB ' end routine

REM * routine to search player and room treasure for parameter name
REM * input variables:
REM *   Parsed.Command1 - name of treasure.
REM * output variables:
REM *   Charges.Number - charges of treasure.
REM *   Index.Number - index of treasure to file.
REM *   TypeNumber - 0 for treasure in inventory, 1 for treasure in room.

SUB Examine.Treasure
TypeNumber = False ' store treasure flag
CALL Check.Inventory.Treasure ' routine to search player inventory treasure
IF Index.Number = False THEN ' check player treasure found
    CALL Num ' decrement counters
    TypeNumber = 1 ' store treasure flag
    CALL Check.Room.Treasure ' routine to search room inventory treasure
END IF ' end check player treasure found
END SUB ' end routine to search inventory treasure for treasure name

REM * routine to search player and room objects for parameter name
REM * input variables:
REM *   Parsed.Command1 - name of object.
REM * output variables:
REM *   Charges.Number - charges of object.
REM *   Index.Number - index of object to file.
REM *   TypeNumber - 0 for object in inventory, 1 for object in room.

SUB Examine.Objects
TypeNumber = False ' store object flag
CALL Check.Inventory.Objects ' routine to search player inventory objects
IF Index.Number = False THEN ' check player inventory object found
    CALL Num ' decrement counters
    TypeNumber = 1 ' store object flag
    CALL Check.Room.Objects ' routine to search room inventory objects
END IF ' end check player inventory object found
END SUB ' end routine to search inventory objects for object name

REM * routine to search player inventory for treasure name
REM * input variables:
REM *   Parsed.Command2 - contains command parameter.
REM * output variables:
REM *   Charges.Number - treasure charges.
REM *   Index.Number - treasure index to file.

SUB Find.Inventory
CALL Parse ' parse command parameter
IF Parser THEN ' check for parsed command
    CALL Numeric ' store # sign counter
END IF ' end check parsed command
CALL Check.Inventory.Treasure ' routine to search player treasure inventory
END SUB ' end routine to search player treasure inventory

REM * routine to search player inventory for object name
REM * input variables:
REM *   Parsed.Command2 - contains command parameter.
REM * output variables:
REM *   Charges.Number - object charges.
REM *   Index.Number - object index to file.

SUB Find.Object
CALL Parse ' parse command parameter
IF Parser THEN ' check for parsed command
    CALL Numeric ' store # sign counter
END IF ' end check parsed command
CALL Check.Inventory.Objects ' routine to search player object inventory
END SUB ' end routine to search player object inventory

REM * routine searches for treasure name in room treasure inventory
REM * input variables:
REM *   Parsed.Command1 - name of treasure to search for.
REM *   Parse.Number - number increment of treasure.
REM * output variables:
REM *   Charges.Number - treasure charges.
REM *   Index.Number - treasure file index.
REM *   Outpts - treasure name.
REM * processing variables:
REM *   Parse.Count - counter of treasure found in search.

SUB Check.Room.Treasure
Charges.Number = False ' reset treasure charges
Index.Number = False ' reset treasure index
Parse.Count = False ' reset treasure counter
IF Parsed.Command1 <> Nul THEN ' compare search string length
    FOR Array.Number = 1 TO 20 ' loop through room inventory
        IF GetRoomRecordTreasure(Array.Number) THEN ' check room inventory index
            ' get treasure record
            CALL Read.Record(TreasureFile, GetRoomRecordTreasure(Array.Number))
            Outpts = TreasureRecord.ShortName ' store treasure name
            ' trim length of treasure name to length of search string
            Outpts = LEFT$(Outpts, LEN(Parsed.Command1)) ' trim
            ' compare treasure name to search name
            IF Outpts = Parsed.Command1 THEN
                Parse.Count = Parse.Count + 1 ' increment counter
                ' check increment counter not specified,
                ' or counter equals increment counter.
                IF Parse.Number = False OR Parse.Count = Parse.Number THEN
                    Outpts = TreasureRecord.TreasureName ' get treasure name
                    Outpts = RTRIM$(Outpts) ' trim name
                    ' store treasure index
                    Index.Number = GetRoomRecordTreasure(Array.Number)
                    ' store treasure charges
                    Charges.Number = GetRoomRecordTreCharges(Array.Number)
                    EXIT FOR ' exit loop through room treasure inventory
                END IF ' end check counters equal
            END IF ' end compare treasure names
        END IF ' end check room inventory index
    NEXT ' end loop through room treasure inventory
END IF ' end compare search string length
END SUB ' end room treasure inventory search

REM * routine searches for object name in room object inventory
REM * input variables:
REM *   Parsed.Command1 - name of object to search for.
REM *   Parse.Number - number increment of object.
REM * output variables:
REM *   Charges.Number - object charges.
REM *   Index.Number - object file index.
REM *   Outpts - object name.
REM * processing variables:
REM *   Parse.Count - counter of object found in search.

SUB Check.Room.Objects
Charges.Number = False ' reset object charges
Index.Number = False ' reset object index
Parse.Count = False ' reset object counter
IF Parsed.Command1 <> Nul THEN ' check search string length
    FOR Array.Number = 1 TO 20 ' loop through room object inventory
        IF GetRoomRecordObject(Array.Number) THEN ' check room object index
            ' get room object record
            CALL Read.Record(ObjectFile, GetRoomRecordObject(Array.Number))
            Outpts = ObjectRecord.ShortName ' store room object name
            ' trim length of object name to length of search string
            Outpts = LEFT$(Outpts, LEN(Parsed.Command1)) ' trim
            ' compare object name to search string
            IF Outpts = Parsed.Command1 THEN
                Parse.Count = Parse.Count + 1 ' increment counter
                ' check increment counter not specified,
                ' or counter equals increment counter.
                IF Parse.Number = False OR Parse.Count = Parse.Number THEN
                    Outpts = ObjectRecord.ObjectName ' store object name
                    Outpts = RTRIM$(Outpts) ' trim name
                    ' store object index
                    Index.Number = GetRoomRecordObject(Array.Number)
                    EXIT FOR ' exit loop through room object inventory
                END IF ' end check counters
            END IF ' end compare object names
        END IF ' end check room object index
    NEXT ' end loop through room object inventory
END IF ' end check search string length
END SUB ' end routine to search room object inventory

REM * routine searches for treasure name in player treasure inventory
REM * input variables:
REM *   Parsed.Command1 - name of treasure to search for.
REM *   Parse.Number - number increment of treasure.
REM * output variables:
REM *   Charges.Number - treasure charges.
REM *   Index.Number - treasure file index.
REM *   Outpts - treasure name.
REM * processing variables:
REM *   Parse.Count - counter of treasure found in search.

SUB Check.Inventory.Treasure
Charges.Number = False ' reset treasure charges
Index.Number = False ' reset treasure index
Parse.Count = False ' reset treasure counter
IF Parsed.Command1 <> Nul THEN ' compare search string length
    FOR Array.Number = 1 TO 20 ' loop through player treasure inventory
        IF GetUserRecordInv(Array.Number) THEN ' check player inventory index
            ' get treasure record
            CALL Read.Record(TreasureFile, GetUserRecordInv(Array.Number))
            Outpts = TreasureRecord.ShortName ' store treasure name
            ' trim length of treasure name to length of search string
            Outpts = LEFT$(Outpts, LEN(Parsed.Command1)) ' trim
            ' compare treasure name to search string
            IF Outpts = Parsed.Command1 THEN
                Parse.Count = Parse.Count + 1 ' increment counter
                ' check increment counter not specified,
                ' or counter equals increment counter.
                IF Parse.Number = False OR Parse.Count = Parse.Number THEN
                    Outpts = TreasureRecord.TreasureName ' store treasure name
                    Outpts = RTRIM$(Outpts) ' trim name
                    ' store treasure index
                    Index.Number = GetUserRecordInv(Array.Number)
                    ' store treasure charges
                    Charges.Number = GetUserRecordCharges(Array.Number)
                    EXIT FOR ' exit loop through player treasure inventory
                END IF ' end check counters
            END IF ' end compare treasure names
        END IF ' end check player treasure inventory index
    NEXT ' end loop through player treasure inventory
END IF ' end check search string length
END SUB ' end routine to search player treasure inventory for treasure name

REM * routine searches for object name in player object inventory
REM * input variables:
REM *   Parsed.Command1 - name of object to search for.
REM *   Parse.Number - number increment of object.
REM * output variables:
REM *   Charges.Number - object charges.
REM *   Index.Number - object file index.
REM *   Outpts - object name.
REM * processing variables:
REM *   Parse.Count - counter of object found in search.

SUB Check.Inventory.Objects
Charges.Number = False ' reset object charges
Index.Number = False ' reset object index
Parse.Count = False ' reset counter
IF Parsed.Command1 <> Nul THEN ' check search strig length
    FOR Array.Number = 1 TO 20 ' loop through player object inventory
        IF GetUserRecordObject(Array.Number) THEN ' check player object index
            ' get object record
            CALL Read.Record(ObjectFile, GetUserRecordObject(Array.Number))
            Outpts = ObjectRecord.ShortName ' store object name
            ' trim length of object name to length of search string
            Outpts = LEFT$(Outpts, LEN(Parsed.Command1)) ' trim
            ' compare object name to search string
            IF Outpts = Parsed.Command1 THEN
                Parse.Count = Parse.Count + 1 ' increment counter
                ' check increment counter not specified,
                ' or counter equals increment counter.
                IF Parse.Number = False OR Parse.Count = Parse.Number THEN
                    Outpts = ObjectRecord.ObjectName ' store objecct name
                    Outpts = RTRIM$(Outpts) ' trim name
                    ' store object index
                    Index.Number = GetUserRecordObject(Array.Number)
                    EXIT FOR ' exit loop through player object inventory
                END IF ' end check counters
            END IF ' end compare object names
        END IF ' end check player object index
    NEXT ' end loop through player object inventory
END IF ' end check search string length
END SUB ' end routine to search player object inventory

REM * routine to search for monster name in room monsters
REM * input variables:
REM *   Parsed.Command1 - name of monster to search for.
REM * output variables:
REM *   Monster.Number - number of monster array.
REM *   Last.Monster - name of monster found.
REM * work variables:
REM *   Word.Parse1, Word.Parse2.

SUB Check.Monster
Array.Number = False ' reset monster number loop counter
Parse.Count = False ' reset increment counter
Monster.Number = False ' reset monster number
IF Parsed.Command1 <> Nul THEN ' check search string length
    FOR Array.Number = 1 TO Number.Monsters ' loop through all room monsters
        Word.Parse1 = False ' reset monster name parse variable
        Word.Parse2 = False ' reset monster name parse variable
        Outpts = MonsterArray(Array.Number).MonsterName ' store monster name
        Outpts = RTRIM$(Outpts) ' trim name
        Outpts = UCASE$(Outpts) ' uppercase name
        ' locate first imbedded space in name
        Word.Parse1 = INSTR(Word.Parse1 + 1, Outpts, " ")
        WHILE Word.Parse1 ' loop until last space in monster name found
            Word.Parse2 = Word.Parse1 ' store position of last space
            Word.Parse1 = INSTR(Word.Parse1 + 1, Outpts, " ") ' locate next space
        WEND ' end loop
        ' monster name is last word of full name
        Inpt = MID$(Outpts, Word.Parse2 + 1)
        ' trim length of monster name to length of search string
        Inpt = LEFT$(Inpt, LEN(Parsed.Command1)) ' trim
        IF Inpt = Parsed.Command1 THEN ' compare monster name to search string
            Parse.Count = Parse.Count + 1 ' increment counter
            ' check increment counter not specified,
            ' or counter equals increment counter.
            IF Parse.Number = False OR Parse.Count = Parse.Number THEN
                ' store monster name
                Outpts = MonsterArray(Array.Number).MonsterName
                Outpts = RTRIM$(Outpts) ' trim name
                Monster.Number = Array.Number ' store monster number
                Last.Monster = Parsed.Command1 ' store monster name
                EXIT FOR ' exit loop through room monsters
            END IF ' end check counters
        END IF ' end compare search strings
    NEXT ' end loop through room monsters
END IF ' end check search string length
END SUB ' end routine to search room monsters for monster name

REM * Filename: dnds7.bas Version: v5.0a r1.0a
REM * This subprogram contains all the main editor routines.

REM * main routine to enter the editor and process edit selections.

SUB Editor
CALL Put.User.Record ' store current user settings
Graphics.Off = False ' reset ansi color
Outpt = "Edit utility v" + Version$ ' editor intro prompt
CALL IO.O ' send output
DO ' loop through edit selections
    Graphics.Off = True ' reset color
    Outpt = "[A]ction" ' edit selection
    CALL IO.O ' send output
    Outpt = "[E]mail" ' edit selection
    CALL IO.O ' send output
    Outpt = "[H]elp" ' edit selection
    CALL IO.O ' send output
    Outpt = "[L]inks" ' edit selection
    CALL IO.O ' send output
    Outpt = "[M]onster" ' edit selection
    CALL IO.O ' send output
    Outpt = "[N]onplayer" ' edit selection
    CALL IO.O ' send output
    Outpt = "[O]bject" ' edit selection
    CALL IO.O ' send output
    Outpt = "[R]oom" ' edit selection
    CALL IO.O ' send output
    Outpt = "[S]pell" ' edit selection
    CALL IO.O ' send output
    Outpt = "[T]reasure" ' edit selection
    CALL IO.O ' send output
    Outpt = "[U]ser" ' edit selection
    CALL IO.O ' send output
    Graphics.Off = False ' reset ansi color
    Outpt = "Edit option(q to quit)? " ' display edit prompt
    No.Input.Out = "Q" ' default entry
    CALL IO.I ' get input
    SELECT CASE UCASE$(Inpt) ' selection of input choice
        CASE "A" ' action selection
            CALL Edit.Action ' action editor
        CASE "E" ' email selection
            CALL Edit.Mail ' email editor
        CASE "H" ' help selection
            CALL Edit.Help ' !edit help
        CASE "L" ' option to edit room links
            CALL Link.Room ' room link edit
        CASE "M" ' monster selection
            CALL Edit.Monster(Rate.Changed) ' monster editor
            IF Rate.Changed THEN ' check to reread monster class
                CALL Read.Monclass ' reread monster class arrays
            END IF ' end check monster class
        CASE "N" ' nonplayer selection
            CALL Edit.Nonplayer ' nonplayer editor
        CASE "O" ' object selection
            CALL Edit.Object ' object editor
        CASE "R" ' rooms selection
            CALL Edit.Room ' room editor
        CASE "S" ' spell selection
            CALL Edit.Spell ' spell editor
        CASE "T" ' treasure selection
            CALL Edit.Treasure ' treasure editor
        CASE "U" ' user selection
            CALL Share.Record(UserFile, User.Index) ' store player record
            CALL Edit.User ' user editor
            CALL Read.Record(UserFile, User.Index) ' read player record
        CASE "Q" ' quit selection
            EXIT DO ' exit editor
    END SELECT ' end choice selection
LOOP ' end edit loop
CALL Get.User.Record ' restore user stats
END SUB ' end main routine to select edit option

REM * routine to edit actions.

SUB Edit.Action
DO ' loop through action edit menu
    Graphics.Off = False ' reset color
    Outpt = "Action edit:" ' make display message
    CALL IO.O ' send message
    Graphics.Off = True ' reset color
    Outpt = "[A]dd" ' make display message
    CALL IO.O ' send message
    Outpt = "[C]hange" ' make display message
    CALL IO.O ' send message
    Outpt = "[L]ist" ' make display message
    CALL IO.O ' send message
    Graphics.Off = False ' reset color
    Outpt = "Action edit option(q to quit)? " ' option input prompt
    No.Input.Out = "Q" ' default input
    CALL IO.I ' get user input
    SELECT CASE UCASE$(Inpt) ' selection of option input
        CASE "A" ' add action option
            New.Record = LOF(ActionFile) / LEN(ActionRecord) + 1
            ActionRecord.SpellTrigger = False
            ActionRecord.MonsterTrigger = False
            ActionRecord.HealthRate = False
            ActionRecord.EncounterRate = False
            ActionRecord.HitPoints = False
            ActionRecord.Inventory = False
            ActionRecord.MonsterTalk = False
            ActionRecord.Fumble = False
            ActionRecord.Teleport = False
            ActionRecord.Level = False
            ActionRecord.Restrictions = False
            ActionRecord.RustRate = False
            ActionRecord.StealRate = False
            ActionRecord.Attribute1 = False
            ActionRecord.Attribute2 = False
            CALL Share.Record(ActionFile, New.Record)
            Outpt = "Action" + STR$(New.Record) + " added." ' make message
            CALL IO.O ' send message
        CASE "L" ' option to list actions
            Max.Actions = LOF(ActionFile) / LEN(ActionRecord) ' store action records
            ' routine to get range values
            CALL Get.Range(Max.Actions, Action.List1, Action.List2)
            ' loop through range values
            Allow.Break = True ' set allow break flag
            Break = False ' reset control-k flag
            Continue = False ' set continuous flag
            FOR Action.Count = Action.List1 TO Action.List2
                CALL Read.Record(ActionFile, Action.Count) ' read action record
                CALL Display.Action(Action.Count) ' routine to display actions
                IF Break THEN ' check control-k flag
                    EXIT FOR ' exit display loop
                END IF ' end check control-k
                IF Continue = False THEN ' check continuous flag
                    CALL More.Prompt ' routine to pause
                    IF No THEN ' check pause input
                        EXIT FOR ' exit action list loop
                    END IF ' end check input
                END IF ' end check continuous flag
            NEXT ' end loop through actions
            Allow.Break = False ' reset allow break flag
            IF Break THEN ' check control-k flag
                Break = False ' reset control-k flag
                Outpt = Nul ' set output to null
                CALL IO.O ' send empty return
            END IF ' end check control-k flag
        CASE "C"
            Outpt = "Enter action number" ' make range prompt
            Max.Actions = LOF(ActionFile) / LEN(ActionRecord) ' store action records
            ' routine to get range of numbers
            CALL Get.Range2(0, Max.Actions, Action.Number)
            IF Action.Number > False THEN ' check number range
                CALL Read.Record(ActionFile, Action.Number) ' get action record
                DO ' loop through action edit menu
                    CALL Display.Action(Action.Number)
                    Graphics.Off = False ' reset color
                    Outpt = "Action edit option(q to quit)? " ' make input prompt
                    No.Input.Out = "Q" ' default input
                    CALL IO.I ' get user input
                    SELECT CASE UCASE$(Inpt) ' selection of action edit option
                        CASE "A" ' option for monster encounter trigger
                            ' routine to get monster number
                            CALL Find.Monster(Monster.Number)
                            ' store monster number trigger
                            ActionRecord.MonsterTrigger = Monster.Number
                        CASE "B" ' option for spell trigger
                            ' routine to get spell number
                            CALL Find.Spell(Spell.Number)
                            ' store spell number trigger
                            ActionRecord.SpellTrigger = Spell.Number
                        CASE "C" ' option monster talk number
                            ' routine to get monster number
                            CALL Find.Monster(Monster.Number)
                            ' store monster talk number trigger
                            ActionRecord.MonsterTalk = Monster.Number
                        CASE "D" ' option for player level entry trigger
                            Graphics.Off = True ' reset color
                            ' make message
                            Outpt = "[H]ighest level (at least) for entry to room"
                            CALL IO.O ' send level trigger display message
                            ' make message
                            Outpt = "[L]owest level (at most) for entry to room"
                            CALL IO.O ' send level trigger display message
                            Graphics.Off = False ' reset color
                            ' make input prompt
                            Outpt = "Action level entry option(q to quit)? "
                            No.Input.Out = "Q" ' default prompt
                            CALL IO.I ' get user input
                            SELECT CASE UCASE$(Inpt) ' selection of level entry option
                                CASE "H" ' high level entry option
                                    Outpt = "Highest level player must be (at least) to enter room? "
                                    No.Input.Out = "1" ' default input
                                    CALL IO.I ' get user input
                                    ActionRecord.Level = INT(VAL(Inpt)) ' store level entry trigger
                                CASE "L" ' low level entry option
                                    Outpt = "Lowest level player must be (at most) to enter room? "
                                    No.Input.Out = "1" ' default input
                                    CALL IO.I ' get user input
                                    ActionRecord.Level = -INT(VAL(Inpt)) ' store level entry trigger
                            END SELECT
                            DO
                                Graphics.Off = False
                                Outpt = "Direction options:"
                                CALL IO.O
                                Graphics.Off = True ' reset color
                                Outpt = "[N]orth" ' make direction display message
                                CALL IO.O ' send direction display message
                                Outpt = "[E]ast" ' make direction display message
                                CALL IO.O ' send direction display message
                                Outpt = "[S]outh" ' make direction display message
                                CALL IO.O ' send direction display message
                                Outpt = "[W]est" ' make direction display message
                                CALL IO.O ' send direction display message
                                Outpt = "[NE]northeast" ' make direction display message
                                CALL IO.O ' send direction display message
                                Outpt = "[SE]southeast" ' make direction display message
                                CALL IO.O ' send direction display message
                                Outpt = "[SW]southwest" ' make direction display message
                                CALL IO.O ' send direction display message
                                Outpt = "[NW]northwest" ' make direction display message
                                CALL IO.O ' send direction display message
                                Outpt = "[U]p" ' make direction display message
                                CALL IO.O ' send direction display message
                                Outpt = "[D]own" ' make direction display message
                                CALL IO.O ' send direction display message
                                Outpt = "[I]n" ' make direction display message
                                CALL IO.O ' send direction display message
                                Outpt = "[O]ut" ' make direction display message
                                CALL IO.O ' send direction display message
                                Outpt = "[G]o to portal" ' make direction display message
                                CALL IO.O ' send direction display message
                                Outpt = "[X]clear directions" ' make direction display message
                                CALL IO.O ' send direction message
                                Graphics.Off = False ' reset color
                                ' input prompt
                                Outpt = "Action entry direction option(q to quit)? "
                                No.Input.Out = "Q" ' default prompt
                                CALL IO.I ' get user input
                                IF UCASE$(Inpt) = "Q" THEN ' compare input to quit option
                                    EXIT DO ' exit direction restriction edit loop
                                END IF ' end compare input option
                                IF UCASE$(Inpt) = "X" THEN ' compare clear directions
                                    ActionRecord.Restrictions = False ' clear directions
                                ELSE ' compare to direction
                                    ' routine to get direction number
                                    CALL Find.Link(Inpt, Direction.Number, False)
                                    IF Direction.Number > False THEN ' compare direction number
                                        ' set the direction bit in the action record trigger flag
                                        ActionRecord.Restrictions = ActionRecord.Restrictions OR 2 ^ Direction.Number
                                    END IF ' end compare clear direction
                                END IF ' end compare direction number
                            LOOP ' loop through level entry trigger edit loop
                        CASE "E" ' option for health hits result
                            Graphics.Off = True ' reset color
                            Outpt = "[F]atigue" ' make health message
                            CALL IO.O ' send health message
                            Outpt = "[V]itality" ' make health message
                            CALL IO.O ' send health message
                            Graphics.Off = False ' reset color
                            ' make input prompt
                            Outpt = "Action health rate option(q to quit)? "
                            No.Input.Out = "Q" ' default input
                            CALL IO.I ' get user input
                            SELECT CASE UCASE$(Inpt) ' selection of health option
                                CASE "F" ' option for action to hit for fatigue
                                    ' make input prompt
                                    Outpt = "Number of hits of fatigue(1-32767)? "
                                    No.Input.Out = "10" ' default input
                                    CALL IO.I ' get user input
                                    ' store health hit result
                                    ActionRecord.HitPoints = INT(VAL(Inpt))
                                CASE "V" ' option for action to hit for vitality
                                    ' make input prompt
                                    Outpt = "Number of hits of vitality(1-32767)? "
                                    No.Input.Out = "10" ' default input
                                    CALL IO.I ' get user input
                                    ' store health hit result
                                    ActionRecord.HitPoints = -INT(VAL(Inpt))
                            END SELECT ' end selection health edit option
                        CASE "F" ' option to edit action rates
                            Graphics.Off = True ' reset color
                            Outpt = "[E]ncounter" ' make rate display message
                            CALL IO.O ' send rate display message
                            Outpt = "[H]ealth" ' make rate display message
                            CALL IO.O ' send rate display message
                            Graphics.Off = False ' reset color
                            Outpt = "Action rate option(q to quit)? " ' make input prompt
                            No.Input.Out = "Q" ' default input
                            CALL IO.I ' get user input
                            SELECT CASE UCASE$(Inpt) ' selection of rate option
                                CASE "E" ' option to change action monster encounter rate
                                    Outpt = "Encounter rate(1-32767)? " ' make input prompt
                                    No.Input.Out = "6" ' default input
                                    CALL IO.I ' get user input
                                    ' store encounter rate
                                    ActionRecord.EncounterRate = INT(VAL(Inpt))
                                CASE "H" ' option to change action health update rate
                                    Outpt = "Health rate(1-32767)? " ' make input prompt
                                    No.Input.Out = "6" ' default input
                                    CALL IO.I ' get user input
                                    ' store health update rate
                                    ActionRecord.HealthRate = INT(VAL(Inpt))
                            END SELECT ' end selection of rate option
                        CASE "G" ' option to edit inventory result options
                            Graphics.Off = True ' reset color
                            Outpt = "[W]eapons" ' make inventory display message
                            CALL IO.O ' send inventory display message
                            Outpt = "[S]hields" ' make inventory display message
                            CALL IO.O ' send inventory display messagec
                            Outpt = "[A]rmor" ' make inventory display message
                            CALL IO.O ' send inventory display message
                            Outpt = "[M]agic items" ' make inventory display message
                            CALL IO.O ' send inventory display message
                            Graphics.Off = False ' reset color
                            Outpt = "Action inventory option(q to quit)? " ' make input prompt
                            No.Input.Out = "Q" ' default input
                            CALL IO.I ' get user input
                            SELECT CASE UCASE$(Inpt) ' selection of inventory option
                                CASE "W" ' option for inventory result
                                    ActionRecord.Inventory = 1 ' store inventory option number
                                CASE "S" ' option for inventory result
                                    ActionRecord.Inventory = 2 ' store inventory option number
                                CASE "A" ' option for inventory result
                                    ActionRecord.Inventory = 3 ' store inventory option number
                                CASE "M" ' option for inventory result
                                    ActionRecord.Inventory = 4 ' store inventory option number
                            END SELECT ' end selection of inventory option
                        CASE "H" ' option for fumble result
                            ActionRecord.Fumble = NOT ActionRecord.Fumble ' set fumble flag
                        CASE "I" ' option for teleport result
                            Outpt = "Room number" ' make range prompt
                            Max.Rooms! = LOF(RoomFile) / LEN(RoomRecord) ' store room file record
                            ' routine to get range of numbers
                            CALL Get.Room.Range2(0!, Max.Rooms!, Room.Number!)
                            ' store room teleport result number
                            ActionRecord.Teleport = Room.Number!
                        CASE "J" ' option to change rust rate
                            ' input prompt
                            Outpt = "Rate for weapons to rust(# of action prompts)? "
                            No.Input.Out = "6" ' default input
                            CALL IO.I ' get user input
                            RustRate = INT(VAL(Inpt)) ' convert input to integer
                            ActionRecord.RustRate = RustRate ' store rust rate
                        CASE "K" ' option to change steal rate
                            Outpt = "Rate for monsters to steal treasure(# of action prompts)? "
                            No.Input.Out = "6" ' default input
                            CALL IO.I ' get user input
                            StealRate = INT(VAL(Inpt)) ' convert input to integer
                            ActionRecord.StealRate = StealRate ' store steal rate
                        CASE "L"
                            DO ' loop through attribute editing
                                Graphics.Off = True ' reset color
                                Outpt = "[A]ir" ' make option message
                                CALL IO.O ' send message
                                Outpt = "[L]and" ' make option message
                                CALL IO.O ' send message
                                Outpt = "[W]ater" ' make option message
                                CALL IO.O ' send message
                                Outpt = "[U]nlit" ' make option message
                                CALL IO.O ' send message
                                Outpt = "[X]Clear" ' make option message
                                CALL IO.O ' send message
                                Graphics.Off = False ' reset color
                                ' make input prompt
                                Outpt = "Action attribute option(q to quit)? "
                                No.Input.Out = "Q" ' default input
                                CALL IO.I ' get user input
                                SELECT CASE UCASE$(Inpt) ' selection of attribute option
                                    CASE "A" ' option for room attribute
                                        ActionRecord.Attribute2 = Air ' set flag
                                    CASE "L" ' option for room attribute
                                        ActionRecord.Attribute2 = Land ' set flag
                                    CASE "W" ' option for room attribute
                                        ActionRecord.Attribute2 = Water ' set flag
                                    CASE "U" ' option for room attribute
                                        ActionRecord.Attribute1 = UnlitRoom ' reset flag
                                    CASE "X" ' option to clear room attributes
                                        ActionRecord.Attribute1 = False ' reset room attribute
                                        ActionRecord.Attribute2 = False ' reset room attribute
                                    CASE "Q" ' option to exit room attribute editing
                                        EXIT DO ' exit room attribute editing
                                END SELECT ' end selection of room attribute editing
                            LOOP ' end loop through room attribute editing menu options
                        CASE "X" ' option to clear triggers and results
                            ActionRecord.SpellTrigger = False
                            ActionRecord.MonsterTrigger = False
                            ActionRecord.HealthRate = False
                            ActionRecord.EncounterRate = False
                            ActionRecord.HitPoints = False
                            ActionRecord.Inventory = False
                            ActionRecord.MonsterTalk = False
                            ActionRecord.Fumble = False
                            ActionRecord.Teleport = False
                            ActionRecord.Level = False
                            ActionRecord.Restrictions = False
                            ActionRecord.RustRate = False
                            ActionRecord.StealRate = False
                            ActionRecord.Attribute1 = False
                            ActionRecord.Attribute2 = False
                            Outpt = "Action cleared." ' make clear message
                            CALL IO.O ' send clear message
                        CASE "Q" ' option to exit room action edit menu
                            CALL Share.Record(ActionFile, Action.Number) ' write action file
                            Outpt = "Action" + STR$(Action.Number) + " changed."
                            CALL IO.O ' send display message
                            EXIT DO ' exit room action edit menu
                    END SELECT ' end selection of action edit options
                LOOP ' end loop through action edit menu
            END IF ' end check range
        CASE "Q" ' selection to quit action editing
            EXIT DO ' exit action editing
    END SELECT ' end action editing selection
LOOP ' end loop through room action menu
END SUB ' end routine to edit room actions

REM * routine to edit monsters file.
REM * output variables:
REM *   Rate.Changed  -  true if any monster rate edited

SUB Edit.Monster (Rate.Changed)
Rate.Changed = False ' reset flag
DO ' loop through monster editing menu
    Graphics.Off = False ' reset color
    Outpt = "Monster edit:" ' make display message
    CALL IO.O ' send message
    Graphics.Off = True ' reset color
    Outpt = "[A]dd" ' make monster edit display message
    CALL IO.O ' send monster edit display message
    Outpt = "[C]hange" ' make monster edit display message
    CALL IO.O ' send monster edit display message
    Outpt = "[L]ist" ' make monster edit display message
    CALL IO.O ' send monster edit display message
    Outpt = "[M]onster class" ' make monster edit display message
    CALL IO.O ' send monster edit display message
    Outpt = "[T]alk responses" ' make monster edit display message
    CALL IO.O ' send monster edit display message
    Graphics.Off = False ' reset color
    Outpt = "Monster edit option(q to quit)? " ' make input prompt
    No.Input.Out = "Q" ' default input
    CALL IO.I ' get user input
    SELECT CASE UCASE$(Inpt) ' selection of monster edit option
        CASE "A" ' option to add an empty monster record
            ' store the last monster record number
            New.Monster = LOF(MonsterFile) / LEN(MonsterRecord) + 1
            MonsterRecord.MonsterName = Nul ' reset monster record variable
            MonsterRecord.PluralName = Nul ' reset monster record variable
            FOR Array.Index = 1 TO 5 ' loop through monster treasure inventory
                ' reset monster record variable
                CALL PutMonsterRecordTreasure(Array.Index, False)
            NEXT ' end loop through monster treasure inventory
            MonsterRecord.Experience = False ' reset monster record variable
            MonsterRecord.NumberAppearing = False ' reset monster record variable
            MonsterRecord.Level = False ' reset monster record variable
            MonsterRecord.Hits = False ' reset monster record variable
            MonsterRecord.Poison = False ' reset monster record variable
            MonsterRecord.LevelDrain = False ' reset monster record variable
            MonsterRecord.Spell = False ' reset monster record variable
            MonsterRecord.Block = False ' reset monster record variable
            MonsterRecord.Prevent = False ' reset monster record variable
            MonsterRecord.Follow = False ' reset monster record variable
            MonsterRecord.Magic = False ' reset monster record variable
            MonsterRecord.Jail = False ' reset monster record variable
            MonsterRecord.Teleport = False ' reset monster record variable
            MonsterRecord.FollowPercent = False ' reset monster record variable
            MonsterRecord.BlockPercent = False ' reset monster record variable
            MonsterRecord.PreventPercent = False ' reset monster record variable
            MonsterRecord.SpellPercent = False ' reset monster record variable
            MonsterRecord.PoisonPercent = False ' reset monster record variable
            MonsterRecord.DrainPercent = False ' reset monster record variable
            MonsterRecord.Rate = False ' reset monster record variable
            MonsterRecord.RatePercent = False ' reset monster record variable
            MonsterRecord.Permanent = False ' reset monster record variable
            MonsterRecord.Gold = False ' reset monster record variable
            FOR Array.Index = 1 TO 5 ' loop through monster talk responses
                ' reset monster record variable
                CALL PutMonsterRecordTalk(Array.Index, False)
            NEXT ' end loop through monster talk responses
            MonsterRecord.Psionic = False ' reset monster record variable
            MonsterRecord.PsionicSpell = False ' reset monster record variable
            CALL Share.Record(MonsterFile, New.Monster) ' routine to write a monster record
        CASE "C" ' option to change monster values
            CALL Find.Monster(Mon.Number) ' get monster number to edit
            IF Mon.Number > False THEN ' compare monster number
                DO ' loop through monster change menu
                    CALL Display.Monster(Mon.Number) ' routine to display monster
                    Graphics.Off = False ' reset color
                    Outpt = "Monster change option(q to quit)? " ' make input prompt
                    No.Input.Out = "Q" ' default input
                    CALL IO.I ' get user input
                    SELECT CASE UCASE$(Inpt) ' selection of monster change option
                        CASE "A" ' option to change monster name
                            Outpt = "Monster name(30 characters)? " ' input prompt
                            CALL IO.I ' get user input
                            MonsterRecord.MonsterName = LCASE$(Inpt) ' store new name
                        CASE "B" ' option to change plural of monster name
                            Outpt = "Plural of name(30 characters)? " ' input prompt
                            CALL IO.I ' get user input
                            ' store new monster plural
                            MonsterRecord.PluralName = LCASE$(Inpt)
                        CASE "C" ' option to change monster level
                            Outpt = "Monster level(1-32767)? " ' input prompt
                            No.Input.Out = "1" ' default input
                            CALL IO.I ' get user input
                            MonsterRecord.Level = INT(VAL(Inpt)) ' store new monster level
                        CASE "D" ' option to change monster magical type
                            Outpt = "Magical monster(y/n)? " ' input prompt
                            No.Input.Out = "Y" ' default input
                            CALL IO.I ' get user input
                            MonsterRecord.Magic = Yes ' store monster type
                        CASE "E" ' option to change monster hit points
                            Outpt = "Hit points? " ' make input prompt
                            No.Input.Out = "1" ' default input
                            CALL IO.I ' get user input
                            MonsterRecord.Hits = CDBL(INT(VAL(Inpt))) ' store monster hits
                        CASE "F" ' option to change monster experience
                            Outpt = "Experience points? " ' input prompt
                            No.Input.Out = "100" ' default input
                            CALL IO.I ' get user input
                            MonsterRecord.Experience = CDBL(INT(VAL(Inpt))) ' store exp
                        CASE "G" ' option to change monster gold hoard
                            Outpt = "Gold points? " ' make input prompt
                            No.Input.Out = "100" ' default prompt
                            CALL IO.I ' get user input
                            MonsterRecord.Gold = CDBL(INT(VAL(Inpt))) ' store monster gold
                        CASE "H" ' option to change number encountered
                            Outpt = "Number appearing(1-32767)? " ' input prompt
                            No.Input.Out = "1" ' default input
                            CALL IO.I ' get user input
                            MonsterRecord.NumberAppearing = INT(VAL(Inpt)) ' store number
                        CASE "I" ' option to change monster poison type
                            Outpt = "Poisonous monster(y/n)? " ' make input prompt
                            No.Input.Out = "Y" ' default prompt
                            CALL IO.I ' get user input
                            MonsterRecord.Poison = False ' clear monster poison
                            MonsterRecord.PoisonPercent = False ' clear monster percent
                            IF Yes THEN ' check yes repsonse
                                Outpt = "Percent chance monster " ' make percent display
                                Outpt = Outpt + "will poison(1-100)? " ' input message
                                No.Input.Out = "50" ' default input
                                CALL IO.I ' get user input
                                Percent = INT(VAL(Inpt)) ' convert input to integer
                                IF Percent >= False AND Percent <= 100 THEN ' check percent
                                    MonsterRecord.Poison = True ' make monster poisonous
                                    MonsterRecord.PoisonPercent = Percent ' store percent
                                END IF ' end check percent
                            END IF ' end check response
                        CASE "J" ' option to change monster level drain type
                            Outpt = "Level draining monster(y/n)? " ' input prompt
                            No.Input.Out = "Y" ' default input
                            CALL IO.I ' get user input
                            MonsterRecord.LevelDrain = False ' clear monster level drain
                            MonsterRecord.DrainPercent = False ' clear monster percent
                            IF Yes THEN ' check yes response
                                Outpt = "Percent chance monster " ' make percent display
                                Outpt = Outpt + "will level drain(1-100)? " ' input message
                                No.Input.Out = "50" ' default input
                                CALL IO.I ' get user input
                                Percent = INT(VAL(Inpt)) ' convert input to integer
                                IF Percent >= False AND Percent <= 100 THEN ' check percent
                                    ' make monster level drainer
                                    MonsterRecord.LevelDrain = True
                                    MonsterRecord.DrainPercent = Percent ' store percent
                                END IF ' end check percent
                            END IF ' end check response
                        CASE "K" ' option to change monster blocks exits type
                            Outpt = "Monster blocks exits(y/n)? " ' make input prompt
                            No.Input.Out = "Y" ' default input
                            CALL IO.I ' get user input
                            MonsterRecord.Block = False ' clear monster block type
                            MonsterRecord.BlockPercent = False ' clear monster percent
                            IF Yes THEN ' check yes response
                                Outpt = "Percent chance monster " ' make percent display
                                Outpt = Outpt + "will block exits(1-100)? " ' input prompt
                                No.Input.Out = "50" ' default input
                                CALL IO.I ' get user input
                                Percent = INT(VAL(Inpt)) ' convert input to integer
                                IF Percent >= False AND Percent <= 100 THEN ' check percent
                                    MonsterRecord.Block = True ' make monster blocker
                                    MonsterRecord.BlockPercent = Percent ' store percent
                                END IF ' end check percent
                            END IF ' end check response
                        CASE "L" ' option to change monster preventing type
                            Outpt = "Monster prevents player from taking treasure(y/n)? "
                            No.Input.Out = "Y" ' default input
                            CALL IO.I ' get user input
                            MonsterRecord.Prevent = False ' clear monster prevent type
                            MonsterRecord.PreventPercent = False ' clear monster percent
                            IF Yes THEN ' check yes response
                                Outpt = "Percent chance monster " ' make input prompt
                                Outpt = Outpt + "prevents player from taking treasure(1-100)? "
                                No.Input.Out = "50" ' default input
                                CALL IO.I ' get user input
                                Percent = INT(VAL(Inpt)) ' convert input to integer
                                IF Percent >= False AND Percent <= 100 THEN ' check percent
                                    MonsterRecord.Prevent = True ' make monster preventer
                                    MonsterRecord.PreventPercent = Percent ' store percent
                                END IF ' end check percent
                            END IF ' end check response
                        CASE "M" ' option to change monster following type
                            Outpt = "Monster follows player(y/n)? " ' make input prompt
                            No.Input.Out = "Y" ' default prompt
                            CALL IO.I ' get user input
                            MonsterRecord.Follow = False ' clear monster follow type
                            MonsterRecord.FollowPercent = False ' clear percent
                            MonsterRecord.Teleport = False ' clear percent
                            IF Yes THEN ' check yes response
                                MonsterRecord.Follow = True ' make monster follows
                                Outpt = "Percent chance monster " ' make percent
                                Outpt = Outpt + "will follow player(1-100)? " ' input prompt
                                No.Input.Out = "50" ' default input
                                CALL IO.I ' get user input
                                Percent = INT(VAL(Inpt)) ' convert input to integer
                                IF Percent >= False AND Percent <= 100 THEN ' check percent
                                    MonsterRecord.FollowPercent = Percent ' store percent
                                END IF ' end check percent
                                Outpt = "Percent chance monster " ' make percent
                                Outpt = Outpt + "will teleport with player(1-100)? " ' prompt
                                No.Input.Out = "50" ' default prompt
                                CALL IO.I ' get user input
                                Percent = INT(VAL(Inpt)) ' convert input to integer
                                IF Percent >= False AND Percent <= 100 THEN ' check percent
                                    MonsterRecord.Teleport = Percent ' store percent
                                END IF ' end check percent
                            END IF ' end check response
                        CASE "N" ' option to change monster spell casting
                            Outpt = "Monster casts spells(y/n)? " ' input prompt
                            No.Input.Out = "Y" ' default input
                            CALL IO.I ' get user input
                            MonsterRecord.Spell = False ' clear monster spell flag
                            MonsterRecord.SpellPercent = False ' clear monster spell number
                            IF Yes THEN ' check yes response
                                CALL Find.Spell(Spell.Number) ' routine to get spell number
                                IF Spell.Number > False THEN ' check spell number
                                    Outpt = "Percent chance monster " ' percent display
                                    Outpt = Outpt + "casts spell(1-100)? " ' input prompt
                                    No.Input.Out = "50" ' default input
                                    CALL IO.I ' get user input
                                    Percent = INT(VAL(Inpt)) ' convert input to integer
                                    IF Percent >= False AND Percent <= 100 THEN ' check percent
                                        ' store monster spell number
                                        MonsterRecord.Spell = Spell.Number
                                        MonsterRecord.SpellPercent = Percent ' store percent
                                    END IF ' end check percent
                                END IF ' end check spell number range
                            END IF ' end check response
                        CASE "O" ' option to change monster jail type
                            Outpt = "Monster jails attacker(y/n)? " ' input prompt
                            No.Input.Out = "Y" ' default input
                            CALL IO.I ' get user input
                            MonsterRecord.Jail = Yes ' store yes response
                            IF Yes THEN ' check response
                                MonsterRecord.Follow = False ' reset flag for teleport
                                Outpt = "Jail room number?" ' prompt for room number
                                CALL IO.I ' get input
                                MonsterRecord.Teleport = INT(VAL(Inpt)) ' store room number
                            END IF ' end check response
                        CASE "P" ' option to change monster rates
                            Rate.Changed = True ' reset flag
                            Outpt = "Monster encounter rate(# of action prompts)? " ' prompt
                            No.Input.Out = "6" ' default input
                            CALL IO.I ' get user input
                            MonsterRecord.Rate = INT(VAL(Inpt)) ' store encounter rate
                            Outpt = "Monster encounter percent(1-100)? " ' input prompt
                            No.Input.Out = "50" ' default input
                            CALL IO.I ' get user input
                            Percent = INT(VAL(Inpt)) ' convert input to integer
                            IF Percent >= False AND Percent <= 100 THEN ' check percent
                                MonsterRecord.RatePercent = Percent ' store percent
                            END IF ' end check percent
                        CASE "R" ' option to change monster permanent type
                            Outpt = "Monster is a permanent monster(y/n)? " ' input prompt
                            No.Input.Out = "N" ' default input
                            CALL IO.I ' get user input
                            MonsterRecord.Permanent = Yes ' store yes response
                        CASE "S" ' option to change monster psionic spell casting
                            Outpt = "Monster uses psionics(y/n)? " ' input prompt
                            No.Input.Out = "Y" ' default input
                            CALL IO.I ' get user input
                            MonsterRecord.Psionic = False ' clear monster psionics
                            MonsterRecord.PsionicSpell = False ' clear monster psionics
                            IF Yes THEN ' check yes response
                                Outpt = "Enter psionic spell:" ' find spell prompt
                                CALL Find.Spell(Spell.Number) ' routine to get spell number
                                IF Spell.Number > False THEN ' check spell number
                                    IF SpellRecord.Psionic = False THEN ' check psionic type
                                        Outpt = "That spell is not a psionic spell!" ' message
                                        CALL IO.O ' send error message
                                    ELSE ' check psionic spell type
                                        MonsterRecord.Psionic = True ' make monster psionic
                                        ' store psi spell
                                        MonsterRecord.PsionicSpell = Spell.Number
                                    END IF ' end check psionic spell
                                END IF ' end check spell number
                            END IF ' end response
                        CASE "T" ' option changes monster treasure inventory
                            Graphics.Off = True ' reset color
                            FOR Array.Number = 1 TO 5 ' loop through monster treasure
                                ' get monster treasure number
                                Treasure.Number = GetMonsterRecordTreasure(Array.Number)
                                ' check file bounds
                                IF Treasure.Number > False AND Treasure.Number <= LOF(TreasureFile) / LEN(TreasureRecord) THEN
                                    ' get treasure record
                                    CALL Read.Record(TreasureFile, Treasure.Number)
                                    Outpt = "[" + MID$(STR$(Array.Number), 2) + "]" + TreasureRecord.TreasureName ' store treasure name
                                    CALL IO.O ' send output message
                                END IF ' end check file bounds
                            NEXT ' end loop through monster treasure
                            Graphics.Off = False ' reset color
                            Outpt = "Enter five treasure names:" ' make display message
                            CALL IO.O ' send display message
                            Outpt = "Press <enter> for none." ' make display message
                            CALL IO.O ' send display message
                            Graphics.Off = True ' reset color
                            FOR Array.Index = 1 TO 5 ' loop through monster inventory
                                ' routine to get treasure number
                                CALL Find.Treasure(Treasure.Number)
                                ' store monster treasure
                                CALL PutMonsterRecordTreasure(Array.Index, Treasure.Number)
                            NEXT ' end loop through monster inventory
                        CASE "Q" ' option to exit monster change menu
                            EXIT DO ' exit monster change menu
                    END SELECT ' end selection of monster change option
                LOOP ' end loop through monster change menu
                CALL Share.Record(MonsterFile, Mon.Number) ' write monster record
                Outpt = "Monster '" + RTRIM$(MonsterRecord.MonsterName) + "' changed."
                CALL IO.O ' send monster changed message
            END IF ' end compare monster number
        CASE "L" ' option to list range of monsters
            Max.Monsters = LOF(MonsterFile) / LEN(MonsterRecord) ' store number records
            ' routine to get range of numbers
            CALL Get.Range(Max.Monsters, Monster.List1, Monster.List2)
            ' loop through range of numbers
            Allow.Break = False ' set allow break flag
            Break = False ' reset control-k flag
            Continue = False ' set continuous flag
            FOR Monster.Count = Monster.List1 TO Monster.List2
                CALL Read.Record(MonsterFile, Monster.Count) ' read monster record
                CALL Display.Monster(Monster.Count) ' routine to display monster
                IF Break THEN ' check control-k flag
                    EXIT FOR ' exit display loop
                END IF ' end check control-k
                IF Continue = False THEN ' check continuous flag
                    CALL More.Prompt ' routine to pause
                    IF No THEN ' check pause response
                        EXIT FOR ' exit list loop
                    END IF ' end check response
                END IF ' end check continuous flag
            NEXT ' end loop through range
            Allow.Break = False ' reset allow break flag
            IF Break THEN ' check control-k flag
                Break = False ' reset control-k flag
                Outpt = Nul ' set output to null
                CALL IO.O ' send empty return
            END IF ' end check control-k flag
        CASE "M" ' option to edit monster class
            DO ' loop through monster class edit menu
                Graphics.Off = True ' reset color
                Outpt = "[A]dd class" ' make monster class display message
                CALL IO.O ' send monster class display message
                Outpt = "[C]hange class" ' make monster class display message
                CALL IO.O ' send monster class display message
                Outpt = "[L]ist class" ' make monster class display message
                CALL IO.O ' send monster class display message
                Graphics.Off = False ' reset color
                Outpt = "Monster class edit option(q to quit)? " ' make input prompt
                No.Input.Out = "Q" ' default input
                CALL IO.I ' get user input
                SELECT CASE UCASE$(Inpt) ' selection of monster class edit option
                    CASE "A" ' option to add empty monster class
                        New.Monster = LOF(MonClassFile) / LEN(MonclassRecord) + 1 ' last record
                        FOR Array.Index = 1 TO 10 ' loop through monster class record
                            ' clear monster class
                            CALL PutMonclassRecordMonsters(Array.Index, False)
                        NEXT ' end loop through record
                        CALL Share.Record(MonClassFile, New.Monster) ' write new monster class record
                        Rate.Changed = True ' reset flag
                    CASE "C" ' option to change a monster class
                        Outpt = "Monster class" ' make range prompt
                        ' get number of records in file
                        Max.Monclasses = LOF(MonClassFile) / LEN(MonclassRecord)
                        ' routine to get range of numbers
                        CALL Get.Range2(0, Max.Monclasses, Monclass.Number)
                        IF Monclass.Number > False THEN ' check number range
                            Outpt = "Enter ten monster names:" ' make display message
                            CALL IO.O ' send display message
                            Outpt = "Press <enter> for none." ' make display message
                            CALL IO.O ' send display message
                            Graphics.Off = True ' reset color
                            FOR Monster.Count = 1 TO 10 ' loop through monster class array
                                ' routine to get monster number
                                CALL Find.Monster(Monster.Number)
                                ' store monster number
                                CALL PutMonclassRecordMonsters(Monster.Count, Monster.Number)
                            NEXT ' end loop through monster class array
                            CALL Share.Record(MonClassFile, Monclass.Number) ' write monster class record
                            ' make message
                            Outpt = "Monclass number" + STR$(Monclass.Number) + " changed."
                            CALL IO.O ' send display message
                            Rate.Changed = True ' reset flag
                        END IF ' end check number range
                    CASE "L" ' option to list range of monster classes
                        Max.Monclasses = LOF(MonClassFile) / LEN(MonclassRecord) ' get records
                        ' routine to get range of numbers
                        CALL Get.Range(Max.Monclasses, Monclass.List1, Monclass.List2)
                        ' loop through range of monster classes
                        Allow.Break = True ' set allow break flag
                        Break = False ' reset control-k flag
                        Continue = False ' set continuous flag
                        FOR Monclass.Number = Monclass.List1 TO Monclass.List2
                            ' get monster class record
                            CALL Read.Record(MonClassFile, Monclass.Number)
                            ' routine to display monclass
                            CALL Display.Monster.Class(Monclass.Number)
                            IF Break THEN ' check control-k flag
                                EXIT FOR ' exit display loop
                            END IF ' end check control-k
                            IF Continue = False THEN ' check continuous flag
                                CALL More.Prompt ' routine to pause
                                IF No THEN ' check pause response
                                    EXIT FOR ' exit monster class list loop
                                END IF ' end check response
                            END IF ' end check continuous flag
                        NEXT ' end loop through range
                        Allow.Break = False ' reset allow break flag
                        IF Break THEN ' check control-k flag
                            Break = False ' reset control-k flag
                            Outpt = Nul ' set output to null
                            CALL IO.O ' send empty return
                        END IF ' end check control-k flag
                    CASE "Q" ' option to exit monster class edit menu
                        EXIT DO ' exit monster class edit menu
                END SELECT ' end selection of monster class option
            LOOP ' end loop through monster class menu
        CASE "T" ' option to edit monster talk responses
            DO ' loop through monster talk response edit menu
                Graphics.Off = True ' reset color
                Outpt = "[A]dd response" ' make monster talk response display message
                CALL IO.O ' send monster talk response display message
                Outpt = "[C]hange response" ' make monster talk response message
                CALL IO.O ' send monster talk response display message
                Outpt = "[L]ist response" ' make monster talk response display message
                CALL IO.O ' send monster talk response display message
                Outpt = "[M]onster response" ' make monster talk response message
                CALL IO.O ' send monster talk response display message
                Graphics.Off = False ' reset color
                Outpt = "Monster talk edit option(q to quit)? " ' make input prompt
                No.Input.Out = "Q" ' default input
                CALL IO.I ' get user input
                SELECT CASE UCASE$(Inpt) ' selection of monster talk response option
                    CASE "A" ' option to add talk response string
                        Outpt = "Enter response(80 characters):" ' make display message
                        CALL IO.O ' send display message
                        Outpt = "Press <enter> for none." ' make display message
                        CALL IO.O ' send display message
                        Outpt = "? " ' make input prompt
                        CALL IO.I ' get user input
                        ' get last response record
                        New.Talk = LOF(MonTalkFile) / LEN(MonsterTalkRecord) + 1
                        MonsterTalkRecord.TalkMessage = Inpt ' store talk response
                        CALL Share.Record(MonTalkFile, New.Talk) ' write talk response record
                    CASE "C" ' option to change a talk response
                        Outpt = "Response number to change" ' make range prompt
                        ' get number of records in file
                        Max.Responses = LOF(MonTalkFile) / LEN(MonsterTalkRecord)
                        ' routine to get range of numbers
                        CALL Get.Range2(0, Max.Responses, Response.Number)
                        IF Response.Number > False THEN ' check number range
                            ' display message
                            Outpt = "Enter response number" + STR$(Response.Number) + ":"
                            CALL IO.O ' send message
                            Outpt = "? " ' make input prompt
                            CALL IO.I ' get user input
                            MonsterTalkRecord.TalkMessage = Inpt ' store talk response string
                            CALL Share.Record(MonTalkFile, Response.Number) ' write talk response record
                            ' display message
                            Outpt = "Response number" + STR$(Response.Number) + " changed."
                            CALL IO.O ' send message
                        END IF ' end check number range
                    CASE "L" ' option to list monster talk responses
                        Allow.Break = True ' set allow break flag
                        Break = False ' reset control-k flag
                        Continue = False ' reset continuous flag
                        Page.Length = False ' clear paginate counter
                        ' loop through file
                        FOR Montalk.Number = 1 TO LOF(MonTalkFile) / LEN(MonsterTalkRecord)
                            ' read monster talk record
                            CALL Read.Record(MonTalkFile, Montalk.Number)
                            Graphics.Off = False ' reset color
                            ' make message
                            Outpt = "Response number" + STR$(Montalk.Number) + ":"
                            CALL IO.O ' send message
                            Graphics.Off = True ' reset color
                            Outpt = RTRIM$(MonsterTalkRecord.TalkMessage) ' store response
                            IF Outpt = Nul THEN ' compare length of talk response
                                Outpt = "<none>" ' reset to none
                            END IF ' end check length
                            CALL IO.O ' send response string
                            IF Break THEN ' check control-k flag
                                EXIT FOR ' exit display loop
                            END IF ' end check control-k
                            Page.Length = Page.Length + 2 ' increment page counter
                            IF Page.Length >= User.PageLength THEN ' check page counter
                                Page.Length = False ' reset counter
                                IF Continue = False THEN ' check continuous flag
                                    CALL More.Prompt ' routine to pause
                                    IF No THEN ' check pause response
                                        EXIT FOR ' exit file loop
                                    END IF ' end check yes
                                END IF ' end check continuous flag
                            END IF ' end check page
                        NEXT ' end loop through file
                        Allow.Break = False ' reset allow break flag
                        IF Break THEN ' check control-k flag
                            Break = False ' reset control-k flag
                            Outpt = Nul ' set output to null
                            CALL IO.O ' send empty return
                        END IF ' end check control-k flag
                        IF Page.Length THEN ' recheck page counter
                            CALL More.Prompt ' routine to pause
                        END IF ' end recheck counter
                    CASE "M" ' option to add talk response numbers to a monster
                        CALL Find.Monster(Monster.Number) ' get monster number to change
                        IF Monster.Number > False THEN ' check monster number
                            CALL Read.Record(MonsterFile, Monster.Number) ' get monster
                            Outpt = "Enter five response numbers:" ' make display message
                            CALL IO.O ' send display message
                            Outpt = "Press <enter> for none." ' make display message
                            CALL IO.O ' send display message
                            FOR Talk.Count = 1 TO 5 ' loop through five responses
                                Outpt = "Enter response number" ' make range prompt
                                Max.Talk = LOF(MonTalkFile) / LEN(MonsterTalkRecord)
                                ' routine to get range of numbers
                                CALL Get.Range2(0, Max.Talk, Talk.Number)
                                ' add talk response number
                                CALL PutMonsterRecordTalk(Talk.Count, Talk.Number)
                                ' message
                                Outpt = "Response number" + STR$(Talk.Count) + " added."
                                CALL IO.O ' send display message
                            NEXT ' end loop through five responses
                            CALL Share.Record(MonsterFile, Monster.Number) ' write monster record
                        END IF ' end check monster number
                    CASE "Q" ' option to exit monster talk response edit menu
                        EXIT DO ' exit monster talk response edit menu
                END SELECT ' end selection of monster talk response option
            LOOP ' end loop through monster talk response edit menu
        CASE "Q" ' option to exit monster edit menu
            EXIT DO ' exit monster edit menu
    END SELECT ' end selection of monster edit options
LOOP ' end loop through monster edit menu
END SUB ' end routine to edit monsters

REM * routine to edit nonplayers file.

SUB Edit.Nonplayer
DO ' loop through nonplayer edit menu
    Graphics.Off = False ' reset color
    Outpt = "Nonplayer edit:" ' make display message
    CALL IO.O ' send message
    Graphics.Off = True ' reset color
    Outpt = "[A]dd" ' make display message
    CALL IO.O ' send message
    Outpt = "[C]hange" ' make display message
    CALL IO.O ' send message
    Outpt = "[L]ist" ' make display message
    CALL IO.O ' send message
    Graphics.Off = False ' reset color
    Outpt = "Nonplayer edit option(q to quit)? " ' option input prompt
    No.Input.Out = "Q" ' default input
    CALL IO.I ' get user input
    SELECT CASE UCASE$(Inpt) ' selection of option input
        CASE "A" ' add nonplayer option
            ' store last nonplayer record number
            New.Nonplayer = LOF(NonPlayerFile) / LEN(MonsterRecord) + 1
            MonsterRecord.MonsterName = Nul ' set nonplayer record items to zero
            MonsterRecord.PluralName = Nul ' reset nonplayer record item
            FOR Array.Number = 1 TO 5 ' loop through nonplayer treasure
                ' set treasure items to zero
                CALL PutMonsterRecordTreasure(Array.Number, False)
            NEXT ' end loop through nonplayer treasure
            MonsterRecord.Experience = False ' set nonplayer record items to zero
            MonsterRecord.NumberAppearing = 1 ' reset nonplayer record items
            MonsterRecord.Level = False ' reset nonplayer record items
            MonsterRecord.Hits = False ' reset nonplayer record items
            MonsterRecord.Gold = False ' reset nonplayer record item
            MonsterRecord.Poison = False ' reset nonplayer record items
            MonsterRecord.LevelDrain = False ' reset nonplayer record items
            MonsterRecord.Spell = False ' reset nonplayer record items
            MonsterRecord.Block = False ' reset nonplayer record items
            MonsterRecord.Prevent = False ' reset nonplayer record items
            MonsterRecord.Follow = False ' reset nonplayer record items
            MonsterRecord.Magic = True ' reset nonplayer record items
            MonsterRecord.Jail = False ' reset nonplayer record items
            MonsterRecord.Teleport = False ' reset nonplayer items
            MonsterRecord.FollowPercent = False ' reset nonplayer items
            MonsterRecord.BlockPercent = False ' reset nonplayer record items
            MonsterRecord.PreventPercent = False ' reset nonplayer record items
            MonsterRecord.SpellPercent = False ' reset nonplayer record items
            MonsterRecord.PoisonPercent = False ' reset nonplayer record items
            MonsterRecord.DrainPercent = False ' reset nonplayer record items
            MonsterRecord.Rate = False ' reset nonplayer record items
            MonsterRecord.RatePercent = False ' reset nonplayer record items
            MonsterRecord.Permanent = -2 ' reset nonplayer record items
            FOR Array.Number = 1 TO 5 ' loop through monster talk responses
                CALL PutMonsterRecordTalk(Array.Number, False) ' reset nonplayer responses
            NEXT ' end loop through monster talk responses
            MonsterRecord.Psionic = False ' reset nonplayer record items
            MonsterRecord.PsionicSpell = False ' reset nonplayer record items
            ' routine to write next nonplayer record
            CALL Share.Record(NonPlayerFile, New.Nonplayer)
        CASE "C" ' option to change nonplayer file record
            ' routine to find nonplayer to edit
            CALL Find.Nonplayer(Nonplayer.Number)
            IF Nonplayer.Number > False THEN ' check nonplayer number
                DO ' loop through nonplayer number edit option menu
                    ' routine to display nonplayer
                    CALL Display.Nonplayer(Nonplayer.Number)
                    Graphics.Off = False ' reset color
                    Outpt = "Nonplayer change option(q to quit)? " ' input prompt
                    No.Input.Out = "Q" ' default prompt
                    CALL IO.I ' get user input
                    SELECT CASE UCASE$(Inpt) ' selection of nonplayer change option
                        CASE "A" ' option to change nonplayer name
                            Outpt = "Nonplayer name(30 characters)? " ' make input prompt
                            CALL IO.I ' get user input
                            MonsterRecord.MonsterName = LCASE$(Inpt) ' store name
                        CASE "B" ' option to change nonplayer rooms
                            Outpt = "Rooms(separated by commas)? " ' make input prompt
                            CALL IO.I ' get user input
                            MonsterRecord.PluralName = Inpt ' store rooms
                        CASE "C" ' option to change nonplayer level
                            Outpt = "Nonplayer level(1-32767)? " ' make input prompt
                            No.Input.Out = "1" ' default input
                            CALL IO.I ' get user input
                            MonsterRecord.Level = INT(VAL(Inpt)) ' store nonplayer level
                        CASE "D" ' option to change nonplayer hit points
                            Outpt = "Hit points? " ' make input prompt
                            No.Input.Out = "1" ' default input
                            CALL IO.I ' get user input
                            MonsterRecord.Hits = CDBL(INT(VAL(Inpt))) ' store nonplayer hits
                        CASE "E" ' option to change nonplayer experience points
                            Outpt = "Experience points? " ' make input prompt
                            No.Input.Out = "1" ' default input
                            CALL IO.I ' get user input
                            MonsterRecord.Experience = CDBL(INT(VAL(Inpt))) ' store exp
                        CASE "F" ' option to change nonplayer gold points
                            Outpt = "Gold points? " ' make input prompt
                            No.Input.Out = "1" ' default input
                            CALL IO.I ' get user input
                            MonsterRecord.Gold = CDBL(INT(VAL(Inpt))) ' store experience
                        CASE "G" ' option to change poisonous nonplayer
                            Outpt = "Poisonous nonplayer(y/n)? " ' make input prompt
                            No.Input.Out = "Y" ' default input
                            CALL IO.I ' get user input
                            MonsterRecord.Poison = False ' reset poisonous nonplayer
                            MonsterRecord.PoisonPercent = False ' reset posionous percent
                            IF Yes THEN ' check user input response
                                MonsterRecord.Poison = True ' set poisonous nonplayer
                                Outpt = "Percent chance nonplayer " ' make input prompt
                                Outpt = Outpt + "will poison(1-100)? " ' input prompt
                                No.Input.Out = "50" ' default input
                                CALL IO.I ' get user input
                                MonsterRecord.PoisonPercent = INT(VAL(Inpt)) ' store percent
                            END IF ' end check user input
                        CASE "H" ' option to change level draining nonplayer
                            Outpt = "Level draining nonplayer(y/n)? " ' make input prompt
                            No.Input.Out = "Y" ' default input
                            CALL IO.I ' get user input
                            MonsterRecord.LevelDrain = False ' reset level drain nonplayer
                            MonsterRecord.DrainPercent = False ' reset level drain nonplayer
                            IF Yes THEN ' check user input response
                                ' set level draining nonplayer
                                MonsterRecord.LevelDrain = True
                                Outpt = "Percent chance nonplayer " ' make input prompt
                                Outpt = Outpt + "will level drain(1-100)? " ' input prompt
                                No.Input.Out = "50" ' default input
                                CALL IO.I ' get user input
                                MonsterRecord.DrainPercent = INT(VAL(Inpt)) ' store percent
                            END IF ' end check user input
                        CASE "I" ' option to change exit blocking nonplayer
                            Outpt = "Nonplayer blocks exits(y/n)? " ' make input prompt
                            No.Input.Out = "Y" ' default input
                            CALL IO.I ' get user input
                            MonsterRecord.Block = False ' reset blocking nonplayer
                            MonsterRecord.BlockPercent = False ' reset blocking nonplayer
                            IF Yes THEN ' check user input response
                                MonsterRecord.Block = True ' set blocking nonplayer
                                Outpt = "Percent chance nonplayer " ' make input prompt
                                Outpt = Outpt + "will block exits(1-100)? " ' input prompt
                                No.Input.Out = "50" ' default input
                                CALL IO.I ' get user input
                                MonsterRecord.BlockPercent = INT(VAL(Inpt)) ' store percent
                            END IF ' end check user input
                        CASE "J" ' option to change preventing nonplayer
                            ' input prompt
                            Outpt = "Nonplayer prevents taking treasure(y/n)? "
                            No.Input.Out = "Y" ' default input
                            CALL IO.I ' get user input
                            MonsterRecord.Prevent = False ' reset preventing nonplayer
                            ' reset preventing nonplayer
                            MonsterRecord.PreventPercent = False
                            IF Yes THEN ' check user input response
                                MonsterRecord.Prevent = True ' set preventing nonplayer
                                Outpt = "Percent chance nonplayer " ' make input prompt
                                Outpt = Outpt + "prevents player from taking treasure(1-100)? "
                                No.Input.Out = "50" ' default input
                                CALL IO.I ' get user input
                                MonsterRecord.PreventPercent = INT(VAL(Inpt)) ' store percent
                            END IF ' end check user input
                        CASE "K" ' option to change following nonplayer
                            Outpt = "Nonplayer follows player(y/n)? " ' make input prompt
                            No.Input.Out = "Y" ' default input
                            CALL IO.I ' get user input
                            MonsterRecord.Follow = False ' reset following nonplayer
                            MonsterRecord.FollowPercent = False ' reset following nonplayer
                            MonsterRecord.Teleport = False ' reset following nonplayer
                            IF Yes THEN ' check user input response
                                MonsterRecord.Follow = True ' set following nonplayer
                                Outpt = "Percent chance nonplayer " ' make input prompt
                                Outpt = Outpt + "will follow player(1-100)? " ' input prompt
                                No.Input.Out = "50" ' default input
                                CALL IO.I ' get user input
                                MonsterRecord.FollowPercent = INT(VAL(Inpt)) ' store percent
                                Outpt = "Percent chance nonplayer " ' make input prompt
                                Outpt = Outpt + "will teleport with player(1-100)? " ' prompt
                                No.Input.Out = "50" ' default input
                                CALL IO.I ' get user input
                                MonsterRecord.Teleport = INT(VAL(Inpt)) ' store percent
                            END IF ' end check user input
                        CASE "L" ' option to change spell casting nonplayer
                            Outpt = "Nonplayer casts spells(y/n)? " ' make input prompt
                            No.Input.Out = "Y" ' default input
                            CALL IO.I ' get user input
                            MonsterRecord.Spell = False ' reset spell casting nonplayer
                            MonsterRecord.SpellPercent = False ' reset casting nonplayer
                            IF Yes THEN ' check user input prompt
                                ' routine to find spell number
                                CALL Find.Spell(Spell.Number)
                                IF Spell.Number > False THEN ' check spell number range
                                    MonsterRecord.Spell = Spell.Number ' store spell number
                                    Outpt = "Percent chance nonplayer " ' make input prompt
                                    Outpt = Outpt + "casts spell(1-100)? " ' input prompt
                                    No.Input.Out = "50" ' default input
                                    CALL IO.I ' get user input
                                    ' store percent
                                    MonsterRecord.SpellPercent = INT(VAL(Inpt))
                                END IF ' end check spell number
                            END IF ' end check user input
                        CASE "M" ' option to change jailing nonplayer
                            Outpt = "Nonplayer jails attacker(y/n)? " ' make input prompt
                            No.Input.Out = "Y" ' default input
                            CALL IO.I ' get user input
                            MonsterRecord.Jail = Yes ' set jailing nonplayer
                            IF Yes THEN ' check response
                                MonsterRecord.Follow = False ' reset flag for teleport
                                Outpt = "Jail room number?" ' prompt for room number
                                CALL IO.I ' get input
                                MonsterRecord.Teleport = INT(VAL(Inpt)) ' store room number
                            END IF ' end check response
                        CASE "N" ' option to change nonplayer encounter rate
                            ' prompt
                            Outpt = "Nonplayer encounter rate(# of action prompts)? "
                            No.Input.Out = "6" ' default input
                            CALL IO.I ' get user input
                            MonsterRecord.Rate = INT(VAL(Inpt)) ' store encounter rate
                            ' make input prompt
                            Outpt = "Nonplayer encounter percent(1-100)? "
                            No.Input.Out = "50" ' default input
                            CALL IO.I ' get user input
                            MonsterRecord.RatePercent = INT(VAL(Inpt)) ' store percent
                        CASE "O" ' option to change nonplayer psionics spell
                            Outpt = "Nonplayer uses psionics(y/n)? " ' make input prompt
                            No.Input.Out = "Y" ' default input
                            CALL IO.I ' get user input
                            MonsterRecord.Psionic = False ' reset nonplayer psionics spell
                            MonsterRecord.PsionicSpell = False ' reset nonplayer psionics
                            IF Yes THEN ' check user input
                                Outpt = "Psionic spell?" ' make input prompt
                                ' routine to find spell number
                                CALL Find.Spell(Spell.Number)
                                IF Spell.Number > False THEN ' check spell number range
                                    IF SpellRecord.Psionic = False THEN ' check psi spell
                                        Outpt = "That spell is not a psionic spell!" ' message
                                        CALL IO.O ' send error message
                                        ' reset nonplayer psionics
                                        MonsterRecord.Psionic = False
                                        MonsterRecord.PsionicSpell = False ' reset psionics
                                    ELSE ' check psionic spell type
                                        MonsterRecord.Psionic = True ' set nonplayer psionics
                                        ' store psi number
                                        MonsterRecord.PsionicSpell = Spell.Number
                                    END IF ' end check psionic spell type
                                END IF ' end check spell number
                            END IF ' end check user input
                        CASE "P" ' option to change nonplayer treasure
                            Outpt = "Enter five treasure names:" ' make display message
                            CALL IO.O ' send message
                            Outpt = "Press <enter> for none." ' make display message
                            CALL IO.O ' send message
                            Graphics.Off = True ' reset color
                            FOR Treasure.Count = 1 TO 5 ' loop through nonplayer treasures
                                ' routine to get treasure number
                                CALL Find.Treasure(Treasure.Number)
                                ' store nonplayer treasure
                                CALL PutMonsterRecordTreasure(Treasure.Count, Treasure.Number)
                            NEXT ' end loop through nonplayer treasures
                        CASE "Q" ' option to exit nonplayer change menu
                            EXIT DO ' exit nonplayer change menu
                    END SELECT ' end selections of nonplayer change options
                LOOP ' end loop through nonplayer change option menu
                CALL Share.Record(NonPlayerFile, Nonplayer.Number) ' write nonplayer file
                Outpt = "Nonplayer '" + RTRIM$(MonsterRecord.MonsterName) + "' changed."
                CALL IO.O ' send display message
            END IF ' end check nonplayer edit number
        CASE "L" ' option to list range of nonplayers
            ' store length of nonplayer file
            Max.Nonplayers = LOF(NonPlayerFile) / LEN(MonsterRecord)
            ' routine to get range of numbers
            CALL Get.Range(Max.Nonplayers, Nonplayer.List1, Nonplayer.List2)
            ' loop through range of nonplayers
            Allow.Break = True ' set allow break flag
            Break = False ' reset control-k flag
            Continue = False ' set continuous flag
            FOR Nonplayers.Count = Nonplayer.List1 TO Nonplayer.List2
                CALL Read.Record(NonPlayerFile, Nonplayers.Count) ' read nonplayer
                ' routine to display nonplayer
                CALL Display.Nonplayer(Nonplayers.Count)
                IF Break THEN ' check control-k flag
                    EXIT FOR ' exit display loop
                END IF ' end check control-k
                Graphics.Off = False ' reset color
                IF Continue = False THEN ' check continuous flag
                    CALL More.Prompt ' routine to pause
                    IF No THEN ' check pause response
                        EXIT FOR ' exit nonplayer list
                    END IF ' end check response
                END IF ' end check continuous flag
            NEXT ' end loop through range of nonplayers
            Allow.Break = False ' reset allow break flag
            IF Break THEN ' check control-k flag
                Break = False ' reset control-k flag
                Outpt = Nul ' set output to null
                CALL IO.O ' send empty return
            END IF ' end check control-k flag
        CASE "Q" ' option to exit nonplayer edit options menu
            EXIT DO ' exit nonplayer edit options menu
    END SELECT ' end selection of nonplayers edit menu
LOOP ' end loop through nonplater edit menu
END SUB ' end routine to edit nonplayers

REM * routine to edit objects file.

SUB Edit.Object
DO ' loop through objects edit menu
    Graphics.Off = False ' reset color
    Outpt = "Object edit:" ' make display message
    CALL IO.O ' send message
    Graphics.Off = True ' reset color
    Outpt = "[A]dd" ' make display message
    CALL IO.O ' send message
    Outpt = "[C]hange" ' make display message
    CALL IO.O ' send message
    Outpt = "[L]ist" ' make display message
    CALL IO.O ' send message
    Graphics.Off = False ' reset color
    Outpt = "Object edit option(q to quit)? " ' make input prompt
    No.Input.Out = "Q" ' default input
    CALL IO.I ' get user input
    SELECT CASE UCASE$(Inpt) ' selection of object edit option
        CASE "A" ' option to add an object record
            New.Object = LOF(ObjectFile) / LEN(ObjectRecord) + 1 'store last object record
            ObjectRecord.ObjectName = Nul ' reset object record item
            ObjectRecord.ShortName = Nul ' reset object record item
            ObjectRecord.LongDesc = Nul ' reset object record item
            ObjectRecord.ShortDesc = Nul ' reset object record item
            ObjectRecord.RoomLink = False ' reset object record item
            ObjectRecord.Invisible = False ' reset object record item
            ObjectRecord.JailTrap = False ' reset object record item
            ObjectRecord.DoorLock = False ' reset object record item
            ObjectRecord.Relocks = False ' reset object record item
            ObjectRecord.Hidden = False ' reset object record item
            ObjectRecord.Closed = False ' reset object record item
            ObjectRecord.Keyed = False ' reset object record item
            ObjectRecord.Trap = False ' reset object record item
            ObjectRecord.Teleport = False ' reset object record item
            ObjectRecord.LightRoom = False ' reset object record item
            ObjectRecord.LightTime = False ' reset object record item
            ObjectRecord.FromHour = False ' reset object record item
            ObjectRecord.FromMin = False ' reset object record item
            ObjectRecord.ToHour = False ' reset object record item
            ObjectRecord.ToMin = False ' reset object record item
            ObjectRecord.Permanent = True ' reset object record item
            CALL Share.Record(ObjectFile, New.Object) ' routine to write object record
        CASE "C" ' option to change an object
            CALL Find.Objects(Object.Number) ' routine to get object number
            IF Object.Number > False THEN ' check object number range
                DO ' loop through object change menu
                    CALL Display.Object(Object.Number) ' routine to display object
                    Graphics.Off = False ' reset color
                    Outpt = "Object change option(q to quit)? " ' make input prompt
                    No.Input.Out = "Q" ' default input
                    CALL IO.I ' get user input
                    SELECT CASE UCASE$(Inpt) ' selection of object change option
                        CASE "A" ' option to change object name
                            Outpt = "Object name(30 characters)? " ' make input prompt
                            CALL IO.I ' get user input
                            ObjectRecord.ObjectName = LCASE$(Inpt) ' store object name
                        CASE "B" ' option to change object mnemonic name
                            Outpt = "Object identifier(30 characters)? " ' make input prompt
                            CALL IO.I ' get user input
                            ObjectRecord.ShortName = UCASE$(Inpt) ' store object mnemonic
                        CASE "C" ' option to change object room number link
                            Outpt = "Room number link? " ' make input prompt
                            CALL IO.I ' get user input
                            ObjectRecord.RoomLink = INT(VAL(Inpt)) ' store object room link
                        CASE "D" ' option to change object trap
                            Outpt = "Trapped portal(y/n)? " ' make input prompt
                            No.Input.Out = "Y" ' default input
                            CALL IO.I ' get user input
                            ObjectRecord.Teleport = False ' reset object trap
                            ObjectRecord.Trap = False ' reset object trap
                            IF Yes THEN ' check user input response
                                Graphics.Off = True ' reset color
                                Outpt = "[P]oison needles" ' make display message
                                CALL IO.O ' send message
                                Outpt = "[T]eleport to room number" ' make display message
                                CALL IO.O ' send message
                                Outpt = "[H]its for fatigue or vitality" ' make message
                                CALL IO.O ' send message
                                Graphics.Off = False ' reset color
                                Outpt = "Object trap option? " ' make input prompt
                                No.Input.Out = "Q" ' default input
                                CALL IO.I ' get user input
                                SELECT CASE UCASE$(Inpt) ' selection of object trap type
                                    CASE "P" ' poison object trap type
                                        ObjectRecord.Trap = 1 ' set object poison trap type
                                    CASE "T" ' teleport object trap type
                                        ObjectRecord.Trap = 2 ' set object teleport trap type
                                        Outpt = "Room number? " ' make input prompt
                                        No.Input.Out = "1" ' default input
                                        CALL IO.I ' get user input
                                        ' store telepot room
                                        ObjectRecord.Teleport = INT(VAL(Inpt))
                                    CASE "H" ' objects hits trap type
                                        Hits.Sign = False ' reset hits sign multiplier
                                        Graphics.Off = True ' reset color
                                        Outpt = "[F]atigue" ' make display message
                                        CALL IO.O ' send message
                                        Outpt = "[V]itality" ' make display message
                                        CALL IO.O ' send message
                                        Outpt = "[N]one" ' make display message
                                        CALL IO.O ' send message
                                        Graphics.Off = False ' reset color
                                        Outpt = "Object trap hits option? " ' make input prompt
                                        No.Input.Out = "Q" ' default input
                                        CALL IO.I ' get user input
                                        SELECT CASE UCASE$(Inpt) ' selection of hits option
                                            CASE "F" ' fatigue option
                                                Hits.Sign = 1 ' set hits sign multiplier
                                            CASE "V" ' vitality option
                                                Hits.Sign = True ' set hits sign multiplier
                                            CASE "N" ' option for none
                                                Hits.Sign = False ' reset sign multiplier
                                        END SELECT ' end hits option selection
                                        Hits.Amount = False ' reset hits number
                                        IF Hits.Sign THEN ' check hits type
                                            ObjectRecord.Trap = 3 ' set object trap type
                                            Outpt = "Number of hits(1-32767)? " ' make input prompt
                                            No.Input.Out = "10" ' default input
                                            CALL IO.I ' get user input
                                            Hits.Amount = INT(VAL(Inpt)) ' store number of hits
                                        END IF ' end check hits type
                                        ' calculate hits
                                        ObjectRecord.Teleport = Hits.Sign * Hits.Amount
                                END SELECT ' end selection of object trap option
                            END IF ' end check user input
                        CASE "E" ' option to change object description
                            ' make display message
                            Outpt = "Long description(80 characters):"
                            CALL IO.O ' send message
                            Outpt = "? " ' make input prompt
                            CALL IO.I ' get user input
                            ObjectRecord.LongDesc = Inpt ' store object description
                        CASE "F" ' option to change object entry description
                            Outpt = "Entry description(80 characters):" ' make message
                            CALL IO.O ' send message
                            Outpt = "? " ' make input prompt
                            CALL IO.I ' get user input
                            ObjectRecord.ShortDesc = Inpt ' store object entry description
                        CASE "G" ' option to change hidden object
                            Outpt = "Hidden object(y/n)? " ' make input prompt
                            No.Input.Out = "Y" ' default input
                            CALL IO.I ' get user input
                            ObjectRecord.Hidden = Yes ' store hidden object type
                        CASE "H" ' option to change invisible object
                            Outpt = "Invisible object(y/n)? " ' make input prompt
                            No.Input.Out = "Y" ' default input
                            CALL IO.I ' get user input
                            ObjectRecord.Invisible = Yes ' store invisible object type
                        CASE "I" ' option to change object jail type
                            Outpt = "Jail attacker trap(y/n)? " ' make input prompt
                            No.Input.Out = "Y" ' default input
                            CALL IO.I ' get user input
                            ObjectRecord.JailTrap = Yes ' store object jail type
                        CASE "J" ' option to change object locked type
                            Outpt = "Locked portal(y/n)? " ' make input prompt
                            No.Input.Out = "Y" ' default input
                            CALL IO.I ' get user input
                            ObjectRecord.DoorLock = False ' reset object lock type
                            ObjectRecord.Closed = False ' reset object lock type
                            IF Yes THEN ' check user input response
                                ObjectRecord.DoorLock = 2 ' set object lock
                                ObjectRecord.Closed = True ' set object lock
                            END IF ' end check user input
                        CASE "K" ' option to change object portal type
                            Outpt = "Relocking portal(y/n)? " ' make input prompt
                            No.Input.Out = "Y" ' default input
                            CALL IO.I ' get user input
                            ObjectRecord.Relocks = Yes ' set object portal type
                        CASE "L" ' option to change object key number
                            Outpt = "Key number(1-32767)? " ' make input prompt
                            No.Input.Out = "1" ' default input
                            CALL IO.I ' get user input
                            ObjectRecord.Keyed = INT(VAL(Inpt)) ' set object key number
                        CASE "M" ' option to change permanent object
                            Outpt = "Permanent Object(y/n)? " ' make input prompt
                            No.Input.Out = "Y" ' default input
                            CALL IO.I ' get user input
                            ObjectRecord.Permanent = Yes ' set permanent object
                        CASE "N" ' option to change object light type
                            Outpt = "Object is a light(y/n)? " ' make input prompt
                            No.Input.Out = "Y" ' default input
                            CALL IO.I ' get user input
                            ObjectRecord.LightRoom = False ' reset object light type
                            ObjectRecord.LightTime = False ' reset object light type
                            IF Yes THEN ' check user input response
                                ObjectRecord.LightRoom = True ' set object light type
                                Outpt = "Object lights at a time(y/n)? " ' make input prompt
                                No.Input.Out = "Y" ' default input
                                CALL IO.I ' get user input
                                IF Yes THEN ' check user input response
                                    ObjectRecord.LightTime = True ' set object light time type
                                    Outpt = "From hour(HH)? " ' make input prompt
                                    CALL IO.I ' get user input
                                    ' store object hour light time
                                    Time.Amount = INT(VAL(Inpt))
                                    ' check hour time range
                                    IF Time.Amount >= 0 AND Time.Amount <= 23 THEN
                                        ' store hour light time
                                        ObjectRecord.FromHour = Time.Amount
                                    END IF ' end check hour time range
                                    Outpt = "From minute(MM)? " ' make input prompt
                                    CALL IO.I ' get user input
                                    ' store object minute light time
                                    Time.Amount = INT(VAL(Inpt))
                                    ' check minute time range
                                    IF Time.Amount >= 0 AND Time.Amount <= 59 THEN
                                        ' store minute light time
                                        ObjectRecord.FromMin = Time.Amount
                                    END IF ' end check minute time range
                                    Outpt = "To hour(HH)? " ' make input prompt
                                    CALL IO.I ' get user input
                                    ' store object hour light time
                                    Time.Amount = INT(VAL(Inpt))
                                    ' check hour time range
                                    IF Time.Amount >= 0 AND Time.Amount <= 23 THEN
                                        ' store hour light time
                                        ObjectRecord.ToHour = Time.Amount
                                    END IF ' end check hour time range
                                    Outpt = "To minute(MM)? " ' make input prompt
                                    CALL IO.I ' get user input
                                    ' store object minute light time
                                    Time.Amount = INT(VAL(Inpt))
                                    ' check minute time range
                                    IF Time.Amount >= 0 AND Time.Amount <= 59 THEN
                                        ' store minute light time
                                        ObjectRecord.ToMin = Time.Amount
                                    END IF ' end check minute time range
                                END IF ' end check user input
                            END IF ' end check user input
                        CASE "Q" ' option to exit object change menu
                            EXIT DO ' exit object change menu
                    END SELECT ' end selection of object change menu options
                LOOP ' end loop through object change options
                CALL Share.Record(ObjectFile, Object.Number) ' write object file
                Outpt = "Object '" + RTRIM$(ObjectRecord.ObjectName) + "' changed."
                CALL IO.O ' send display message
            END IF ' end check object number range
        CASE "L" ' option to list range of objects
            Max.Objects = LOF(ObjectFile) / LEN(ObjectRecord) ' store length of object
            ' routine to get range of objects
            CALL Get.Range(Max.Objects, Objects.List1, Objects.List2)
            ' loop through range of object numbers
            Allow.Break = True ' set allow break flag
            Break = False ' reset control-k flag
            Continue = False ' set continuous flag
            FOR Object.Count = Objects.List1 TO Objects.List2
                CALL Read.Record(ObjectFile, Object.Count) ' read object file
                CALL Display.Object(Object.Count) ' routine to display object
                IF Break THEN ' check control-k flag
                    EXIT FOR ' exit display flag
                END IF ' end check control-k
                Graphics.Off = False ' reset color
                IF Continue = False THEN ' check continuous flag
                    CALL More.Prompt ' routine to pause
                    IF No THEN ' check user input
                        EXIT FOR ' exit list loop
                    END IF ' end check input
                END IF ' end check continuous flag
            NEXT ' end loop through object list
            Allow.Break = False ' reset allow break flag
            IF Break THEN ' check control-k flag
                Break = False ' reset control-k flag
                Outpt = Nul ' set output to null
                CALL IO.O ' send empty return
            END IF ' end check control-k flag
        CASE "Q" ' option to exit object edit menu
            EXIT DO ' exit object edit menu
    END SELECT ' end selection of object edit menu option
LOOP ' end loop through object edit options
END SUB ' end routine to edit objects

REM * routine to edit spells file.

SUB Edit.Spell
DO ' loop through spell edit menu
    Graphics.Off = False ' reset color
    Outpt = "Spell edit:" ' make display message
    CALL IO.O ' send message
    Graphics.Off = True ' reset color
    Outpt = "[A]dd" ' make display message
    CALL IO.O ' send message
    Outpt = "[C]hange" ' make display message
    CALL IO.O ' send message
    Outpt = "[L]ist" ' make display message
    CALL IO.O ' make message
    Graphics.Off = False ' reset color
    Outpt = "Spell edit option(q to quit)? " ' make input prompt
    No.Input.Out = "Q" ' default input
    CALL IO.I ' get user input
    SELECT CASE UCASE$(Inpt) ' selection of spell edit option
        CASE "A" ' option to add a spell
            New.Spell = LOF(SpellFile) / LEN(SpellRecord) + 1 ' store last spell record
            SpellRecord.SpellName = Nul ' reset spell record item
            SpellRecord.Chant = Nul ' reset spell record item
            SpellRecord.Desc = Nul ' reset spell record item
            SpellRecord.Level = False ' reset spell record item
            SpellRecord.SpellType = False ' reset spell record item
            SpellRecord.Teleport = False ' reset spell record item
            SpellRecord.ClassType = False ' reset spell record item
            SpellRecord.Psionic = False ' reset spell record item
            SpellRecord.PsionicMode = False ' reset spell record item
            FOR Array.Index = 1 TO 5 ' loop through spell ingredients
                CALL PutSpellRecordIngred(Array.Index, False) ' reset spell record item
            NEXT ' end loop through spell ingredients
            SpellRecord.SpellFlag = False ' reset spell record item
            CALL Share.Record(SpellFile, New.Spell) ' write next spell record
        CASE "C" ' option to edit a spell
            CALL Find.Spell(Spell.Number) ' routine to get spell number
            IF Spell.Number > False THEN ' check spell number
                DO ' loop through spell edit options menu
                    CALL Display.Spell(Spell.Number) ' routine to display spell
                    Graphics.Off = False ' reset color
                    Outpt = "Spell change option(q to quit)? " ' make input prompt
                    No.Input.Out = "Q" ' default input
                    CALL IO.I ' get user input
                    SELECT CASE UCASE$(Inpt) ' selection of spell edit option
                        CASE "A" ' option to change spell name
                            Outpt = "Spell name(30 characters)? " ' make input prompt
                            CALL IO.I ' get user input
                            SpellRecord.SpellName = LCASE$(Inpt) ' store spell name
                        CASE "B" ' option to change spell chant
                            Outpt = "Spell chant(80 characters):" ' make display message
                            CALL IO.O ' send message
                            Outpt = "? " ' make input prompt
                            CALL IO.I ' get user input
                            SpellRecord.Chant = UCASE$(Inpt) ' store spell chant
                        CASE "C" ' option to change spell description
                            Outpt = "Spell cast description(80 characters):" ' make message
                            CALL IO.O ' send message
                            Outpt = "? " ' make input prompt
                            CALL IO.I ' get user input
                            SpellRecord.Desc = Inpt ' store spell description
                        CASE "D" ' option to change spell level
                            Outpt = "Spell level(1-32767)? " ' make input prompt
                            No.Input.Out = "1" ' default input
                            CALL IO.I ' get user input
                            SpellRecord.Level = INT(VAL(Inpt)) ' store spell level
                        CASE "E" ' option to change spell type
                            CALL Spell.Types ' routine displays spell types
                            Graphics.Off = False ' reset color
                            Outpt = "Spell type option(q to quit)? " ' make input prompt
                            No.Input.Out = "Q" ' default input
                            CALL IO.I ' get user input
                            Inpt = UCASE$(Inpt) ' uppercase input
                            ' compare input
                            Spell.Type = INSTR("ABCDEFGHIJKLMNOPRSTUVWXYZ123", Inpt)
                            ' check input value
                            IF Spell.Type > False AND Spell.Type <= 28 THEN
                                SpellRecord.SpellType = Spell.Type ' store spell type
                                IF Spell.Type = 10 THEN ' check teleport spell
                                    Outpt = "Room it teleports to(1-" + MID$(STR$(LOF(RoomFile) / LEN(RoomRecord)), 2) + ")? " 'prompt
                                    No.Input.Out = "1" ' default input
                                    CALL IO.I ' get user input
                                    SpellRecord.Teleport = INT(VAL(Inpt)) ' store spell room
                                END IF ' end check teleport spell
                            END IF ' end check input value
                        CASE "F" ' option to change psionic spell
                            Outpt = "Spell is psionic spell(y/n)? " ' make input prompt
                            No.Input.Out = "Y" ' default input
                            CALL IO.I ' get user input
                            SpellRecord.Psionic = False ' reset psionic spell
                            SpellRecord.PsionicMode = False ' reset psionic spell
                            IF Yes THEN ' check user input response
                                SpellRecord.Psionic = True ' set psionic spell
                                Graphics.Off = True ' reset color
                                Outpt = "[A]ttack mode" ' make display message
                                CALL IO.O ' send message
                                Outpt = "[D]efense mode" ' make display message
                                CALL IO.O ' send message
                                Graphics.Off = False ' reset color
                                Outpt = "Spell psionic option? " ' make input prompt
                                No.Input.Out = "Q" ' default input
                                CALL IO.I ' get user input
                                SpellRecord.PsionicMode = False ' reset spell psionic type
                                SELECT CASE UCASE$(Inpt) ' selection of psionic type
                                    CASE "A" ' attack psionic type
                                        SpellRecord.PsionicMode = 1 ' store psionic type
                                    CASE "D" ' defense psionic type
                                        SpellRecord.PsionicMode = 2 ' store psionic type
                                END SELECT ' end selection of psionic type
                            END IF ' end check user input
                        CASE "G" ' option to change spell class restrictions
                            DO ' loop through spell cast options menu
                                Graphics.Off = True ' reset color
                                FOR Class.Number = 1 TO 8 ' loop through class names
                                    Outpt = "[" + CHR$(64 + Class.Number) + "]" + RTRIM$(Class.Name(Class.Number))
                                    CALL IO.O ' send class name message
                                NEXT ' end loop through class names
                                Outpt = "[N]one" ' make message
                                CALL IO.O ' send message
                                Outpt = "[X]All" ' make message
                                CALL IO.O ' send message
                                Graphics.Off = False ' reset color
                                Outpt = "Spell class option(q to quit)? " ' make input prompt
                                No.Input.Out = "Q" ' default input
                                CALL IO.I ' get user input
                                SELECT CASE UCASE$(Inpt) ' selection of class name
                                    CASE "A" TO "H" ' range of class names
                                        ' store number of class
                                        Class.Number = INSTR("ABCDEFGH", Inpt)
                                        ' set spell cast restriction flag
                                        SpellRecord.ClassType = SpellRecord.ClassType OR 2 ^ Class.Number
                                    CASE "N" ' option for no class restrictions
                                        ' reset class restriction flag
                                        SpellRecord.ClassType = False
                                    CASE "X" ' option to set all class restrictions
                                        FOR Class.Number = 1 TO 8 ' loop through class types
                                            ' set spell cast restriction flag
                                            SpellRecord.ClassType = SpellRecord.ClassType OR 2 ^ Class.Number
                                        NEXT ' end loop through class types
                                    CASE "Q" ' option to exit spell cast restriction menu
                                        EXIT DO ' exit spell cast restriction menu
                                END SELECT ' end selection of spell cast option
                            LOOP ' end loop through spell cast restriction option menu
                        CASE "H" ' option to change spell ingredients
                            Outpt = "Enter five treasure items for spell ingredients:"
                            CALL IO.O ' send display message
                            Outpt = "Press <enter> for none." ' make display message
                            CALL IO.O ' send message
                            Graphics.Off = True ' reset color
                            FOR Array.Count = 1 TO 5 ' loop through spell ingredients
                                ' routine to get treasure number
                                CALL Find.Treasure(Treasure.Number)
                                ' store spell ingredient
                                CALL PutSpellRecordIngred(Array.Count, Treasure.Number)
                            NEXT ' end loop through spell ingredients
                            Graphics.Off = False ' reset color
                        CASE "I" ' option to change spell ingredient cast type
                            DO ' loop through spell cast type option menu
                                Graphics.Off = True ' reset color
                                Outpt = "[A]Use command" ' make display message
                                CALL IO.O ' send message
                                Outpt = "[B]Read scroll" ' make display message
                                CALL IO.O ' send message
                                Outpt = "[C]Cast spell" ' make display message
                                CALL IO.O ' send message
                                Outpt = "[N]one" ' make display message
                                CALL IO.O ' send message
                                Outpt = "[X]All" ' make display message
                                CALL IO.O ' send message
                                Graphics.Off = False ' reset color
                                Outpt = "Spell ingredient option(q to quit)? " ' make prompt
                                No.Input.Out = "Q" ' default input
                                CALL IO.I ' get user input
                                SELECT CASE UCASE$(Inpt) ' selection of spell cast type
                                    CASE "A" ' option to change spell cast type
                                        ' store spell cast type flag
                                        SpellRecord.SpellFlag = SpellRecord.SpellFlag OR Use.Spell.Type
                                    CASE "B" ' option to change spell cast type
                                        ' store spell cast type flag
                                        SpellRecord.SpellFlag = SpellRecord.SpellFlag OR Scroll.Spell.Type
                                    CASE "C" ' option to change spell cast type
                                        ' store spell cast type flag
                                        SpellRecord.SpellFlag = SpellRecord.SpellFlag OR Cast.Spell.Type
                                    CASE "N" ' option to clear spell cast types
                                        ' clear spell cast type flag
                                        SpellRecord.SpellFlag = False
                                    CASE "X" ' option to add all spell cast type
                                        ' store all spell cast type flag
                                        SpellRecord.SpellFlag = SpellRecord.SpellFlag OR Use.Spell.Type
                                        SpellRecord.SpellFlag = SpellRecord.SpellFlag OR Scroll.Spell.Type
                                        SpellRecord.SpellFlag = SpellRecord.SpellFlag OR Cast.Spell.Type
                                    CASE "Q" ' option to exit spell ingredient cast types menu
                                        EXIT DO ' exit spell ingredient cast types menu
                                END SELECT ' end selection of spell cast type
                            LOOP ' end loop through spell cast types options menu
                        CASE "Q" ' option to exit spell edit menu
                            EXIT DO ' exit spell edit menu
                    END SELECT ' end selection of spell edit menu
                LOOP ' end loop through spell editing
                CALL Share.Record(SpellFile, Spell.Number) ' write spell record
                Outpt = "Spell '" + RTRIM$(SpellRecord.SpellName) + "' changed." ' message
                CALL IO.O ' send display message
            END IF ' end check spell number
        CASE "L" ' option to list range of spells
            Max.Spells = LOF(SpellFile) / LEN(SpellRecord) ' store length of spell file
            ' routine to get range of numbers
            CALL Get.Range(Max.Spells, Spell.List1, Spell.List2)
            ' loop through range of spell numbers
            Allow.Break = True ' set allow break flag
            Break = False ' reset control-k flag
            Continue = False ' set continuous flag
            FOR Spell.Count = Spell.List1 TO Spell.List2
                CALL Read.Record(SpellFile, Spell.Count) ' read spell record
                CALL Display.Spell(Spell.Count) ' routine to display spell
                IF Break THEN ' check control-k flag
                    EXIT FOR ' exit display loop
                END IF ' end check control-k
                Graphics.Off = False ' reset color
                IF Continue = False THEN ' check continuous flag
                    CALL More.Prompt ' routine to pause
                    IF No THEN ' check pause response
                        EXIT FOR ' exit spell list loop
                    END IF ' end check input
                END IF ' end check continuous flag
            NEXT ' end loop through spell list
            Allow.Break = False ' reset allow break flag
            IF Break THEN ' check control-k flag
                Break = False ' reset control-k flag
                Outpt = Nul ' set output to null
                CALL IO.O ' send empty return
            END IF ' end check control-k flag
        CASE "Q" ' option to exit spell menu
            EXIT DO ' exit spell options menu
    END SELECT ' end selection of spell options
LOOP ' end loop through spell options menu
END SUB ' end routine to edit spell

REM * routine to edit treasure file.

SUB Edit.Treasure
DO ' loop through treasure edit menu
    Graphics.Off = False ' reset color
    Outpt = "Treasure edit:" ' make display message
    CALL IO.O ' send message
    Graphics.Off = True ' reset color
    Outpt = "[A]dd" ' make display message
    CALL IO.O ' send message
    Outpt = "[C]hange" ' make display message
    CALL IO.O ' send message
    Outpt = "[L]ist" ' make display message
    CALL IO.O ' send message
    Graphics.Off = False ' reset color
    Outpt = "Treasure edit option(q to quit)? " ' make input prompt
    No.Input.Out = "Q" ' default input
    CALL IO.I ' get user input
    SELECT CASE UCASE$(Inpt) ' selection of treasure edit option
        CASE "A" ' option to add treasure
            New.Treasure = LOF(TreasureFile) / LEN(TreasureRecord) + 1 ' store treasure
            TreasureRecord.Keyed = False ' reset treasure record item
            TreasureRecord.Weight = False ' reset treasure record item
            TreasureRecord.Gold = False ' reset treasure record item
            TreasureRecord.Plus = False ' reset treasure record item
            TreasureRecord.Charges = False ' reset treasure record item
            TreasureRecord.Type = False ' reset treasure record item
            TreasureRecord.Loadable = False ' reset treasure record item
            TreasureRecord.Ammunition = False ' reset treasure record item
            TreasureRecord.AmmoLoads = False ' reset treasure record item
            TreasureRecord.Container = False ' reset treasure record item
            TreasureRecord.Potion = False ' reset treasure record item
            TreasureRecord.Locked = False ' reset treasure record item
            TreasureRecord.Closed = False ' reset treasure record item
            TreasureRecord.Invisible = False ' reset treasure record item
            TreasureRecord.Proficiency = False ' reset treasure record item
            TreasureRecord.TreasureName = Nul ' reset treasure record item
            TreasureRecord.ShortName = Nul ' reset treasure record item
            TreasureRecord.Spell = False ' reset treasure record item
            TreasureRecord.Scroll = False ' reset treasure record item
            TreasureRecord.RingType = False ' reset treasure record item
            TreasureRecord.RingSpell = False ' reset treasure record item
            TreasureRecord.Edible = False ' reset treasure record item
            TreasureRecord.Rustable = False ' reset treasure record item
            TreasureRecord.RustPercent = False ' reset treasure record item
            TreasureRecord.Stealable = False ' reset treasure record item
            TreasureRecord.StealPercent = False ' reset treasure record item
            TreasureRecord.Launchable = False ' reset treasure record item
            TreasureRecord.LaunchLoads = False ' reset treasure record item
            TreasureRecord.LaunchAmmo = False ' reset treasure record item
            TreasureRecord.Movable = False ' reset treasure record item
            TreasureRecord.Vehicle = False ' reset treasure record item
            TreasureRecord.VehicleHits = False ' reset treasure record item
            TreasureRecord.VehicleType = False ' reset treasure record item
            TreasureRecord.LightType = False ' reset treasure record item
            TreasureRecord.LightCharges = False ' reset treasure record item
            TreasureRecord.FuelType = False ' reset treasure record item
            TreasureRecord.FuelCharges = False ' reset treasure record item
            TreasureRecord.Permanent = False ' reset treasure record item
            TreasureRecord.Invisible = False ' reset treasure record item
            CALL Share.Record(TreasureFile, New.Treasure) ' write next treasure record
        CASE "C" ' option to change treasure
            CALL Find.Treasure(Treasure.Number) ' routine to get treasure number
            IF Treasure.Number > False THEN ' check treasure number
                DO ' loop through treasure edit options
                    ' routine to display treasure
                    CALL Display.Treasure(Treasure.Number)
                    Graphics.Off = False ' reset color
                    Outpt = "Treasure change option(q to quit)? " ' make input prompt
                    No.Input.Out = "Q" ' default input
                    CALL IO.I ' get user input
                    SELECT CASE UCASE$(Inpt) ' selection of treasure option
                        CASE "A" ' option to change treasure name
                            Outpt = "Treasure name(30 characters)? " ' make input prompt
                            CALL IO.I ' get user input
                            TreasureRecord.TreasureName = LCASE$(Inpt) ' store treasure name
                        CASE "B" ' option to change treasure mnemonic
                            ' make input prompt
                            Outpt = "Treasure identifier(30 characters)? "
                            CALL IO.I ' get user input
                            ' store treasure mnemonic
                            TreasureRecord.ShortName = UCASE$(Inpt)
                        CASE "C" ' option to change treasure weight
                            Outpt = "Weight(1-32767)? " ' make input prompt
                            No.Input.Out = "1" ' default input
                            CALL IO.I ' get user input
                            TreasureRecord.Weight = INT(VAL(Inpt)) ' store treasure weight
                        CASE "D" ' option to change treasure gold value
                            Outpt = "Gold value? " ' make input prompt
                            No.Input.Out = "10" ' default input
                            CALL IO.I ' get user input
                            TreasureRecord.Gold = INT(VAL(Inpt)) ' store treasure gold value
                        CASE "E" ' option to change treasure type
                            Graphics.Off = True ' reset color
                            Outpt = "[W]eapon" ' make display message
                            CALL IO.O ' send message
                            Outpt = "[S]hield" ' make display message
                            CALL IO.O ' send message
                            Outpt = "[A]rmor" ' make display message
                            CALL IO.O ' send message
                            Outpt = "[T]reasure" ' make display message
                            CALL IO.O ' send message
                            Graphics.Off = False ' reset color
                            Outpt = "Treasure type option? " ' make input prompt
                            No.Input.Out = "Q" ' default input
                            CALL IO.I
                            SELECT CASE UCASE$(Inpt) ' selecction of treasure type
                                CASE "W" ' option for weapon type
                                    TreasureRecord.Type = False ' set treasure type
                                    TreasureRecord.Permanent = False ' set treasure type
                                    Outpt = "Hit plus(1-32767)? " ' make input prompt
                                    No.Input.Out = "10" ' default input
                                    CALL IO.I ' get user input
                                    TreasureRecord.Plus = INT(VAL(Inpt)) ' store treasure plus
                                CASE "S" ' option for shield type
                                    TreasureRecord.Type = True ' set treasure type
                                    TreasureRecord.Permanent = False ' set treasure type
                                CASE "A" ' option for armor type
                                    TreasureRecord.Type = 1 ' set treasure type
                                    TreasureRecord.Permanent = False ' set treasure type
                                CASE "T" ' option for treasure type
                                    TreasureRecord.Plus = False ' set treasure type
                                    TreasureRecord.Charges = False ' set treasure type
                                    TreasureRecord.Type = False ' set treasure type
                                    TreasureRecord.Permanent = False ' set treasure type
                            END SELECT ' end selection of treasure type
                        CASE "F" ' option to change treasure hits plus
                            Outpt = "Hit plus(1-32767)? " ' make input prompt
                            No.Input.Out = "1" ' default input
                            CALL IO.I ' get user input
                            TreasureRecord.Plus = INT(VAL(Inpt)) ' store treasure hits plus
                        CASE "G" ' option to change treasure charges
                            Outpt = "Charges(1-32767)? " ' make input prompt
                            No.Input.Out = "1" ' default input
                            CALL IO.I ' get user input
                            TreasureRecord.Charges = INT(VAL(Inpt)) ' store treasure charges
                        CASE "H" ' option to change treasure weapon type
                            Graphics.Off = True ' reset color
                            Outpt = "[B]lunt" ' make display message
                            CALL IO.O ' send message
                            Outpt = "[P]ole" ' make display message
                            CALL IO.O ' send message
                            Outpt = "[S]harp" ' make display message
                            CALL IO.O ' send message
                            Outpt = "[T]hrusting" ' make display message
                            CALL IO.O ' send message
                            Graphics.Off = False ' reset color
                            Outpt = "Treasure weapon class option? " ' make input prompt
                            No.Input.Out = "Q" ' default input
                            CALL IO.I ' get user input
                            SELECT CASE UCASE$(Inpt) ' selection of treasure weapon type
                                CASE "B" ' blunt weapon type
                                    TreasureRecord.Proficiency = 1 ' set weapon type
                                CASE "P" ' pole weapon type
                                    TreasureRecord.Proficiency = 2 ' set weapon type
                                CASE "S" ' sharp weapon type
                                    TreasureRecord.Proficiency = 3 ' set weapon type
                                CASE "T" ' thrusting weapon type
                                    TreasureRecord.Proficiency = 4 ' set weapon type
                            END SELECT ' end selection of treasure weapon type
                        CASE "I" ' option to change permanent treasure
                            Outpt = "Treasure is permanent(y/n)? " ' make input prompt
                            No.Input.Out = "Y" ' default input
                            CALL IO.I ' get user input
                            TreasureRecord.Permanent = Yes ' store permanent type
                        CASE "J" ' option to change edible treasure
                            Outpt = "Treasure is edible(y/n)? " ' make input prompt
                            No.Input.Out = "Y" ' default input
                            CALL IO.I ' get user input
                            TreasureRecord.Edible = Yes ' store edible type
                        CASE "K" ' option to change treasure spell number
                            Outpt = "Treasure is magical(y/n)? " ' make input prompt
                            No.Input.Out = "Y" ' default input
                            CALL IO.I ' get user input
                            TreasureRecord.Spell = False ' reset treasure spell number
                            IF Yes THEN ' check user input response
                                CALL Find.Spell(Spell.Number) ' routine to get spell number
                                IF Spell.Number > False THEN ' check spell number range
                                    ' store treasure spell number
                                    TreasureRecord.Spell = Spell.Number
                                END IF ' end check spell range
                            END IF ' end check input
                        CASE "L" ' option to change ring treasure
                            Outpt = "Treasure is ring(y/n)? " ' make input prompt
                            No.Input.Out = "Y" ' default input
                            CALL IO.I ' get user input
                            TreasureRecord.RingType = False ' reset treasure ring
                            TreasureRecord.RingSpell = False ' reset treasure ring
                            IF Yes THEN ' check user input response
                                Graphics.Off = True ' reset color
                                Outpt = "[P]oison" ' make display message
                                CALL IO.O ' send message
                                Outpt = "[L]evel Drain" ' make display message
                                CALL IO.O ' send message
                                Outpt = "[S]pell" ' make display message
                                CALL IO.O ' send message
                                Outpt = "[G]eneric spell" ' make display message
                                CALL IO.O ' send message
                                Graphics.Off = False ' reset color
                                Outpt = "Treasure ring option? " ' make input prompt
                                No.Input.Out = "Q" ' default input
                                CALL IO.I ' get user input
                                SELECT CASE UCASE$(Inpt) ' selection of ring type
                                    CASE "P" ' option for anti-poison ring
                                        TreasureRecord.RingSpell = False ' store ring spell type
                                        TreasureRecord.RingType = 1 ' store ring type
                                    CASE "L" ' option for anti-leveldrain ring
                                        TreasureRecord.RingSpell = False ' store ring spell type
                                        TreasureRecord.RingType = 2 ' store ring type
                                    CASE "S" ' option for number of anti-spell ring
                                        ' routine to get spell number
                                        CALL Find.Spell(Spell.Number)
                                        IF Spell.Number > False THEN ' check spell number
                                            ' store ring spell number
                                            TreasureRecord.RingSpell = Spell.Number
                                            TreasureRecord.RingType = 3 ' store ring type
                                        END IF ' end check spell number
                                    CASE "G" ' option for generic anti-spell ring
                                        TreasureRecord.RingSpell = True ' store ring spell type
                                        TreasureRecord.RingType = 3 ' store ring type
                                END SELECT ' end selection of ring type
                            END IF ' end check user input
                        CASE "M" ' option to change treasure light
                            Outpt = "Treasure is a light(y/n)? " ' make input prompt
                            No.Input.Out = "Y" ' default input
                            CALL IO.I ' get user input
                            TreasureRecord.LightType = False ' reset treasure light type
                            TreasureRecord.LightCharges = False ' reset treasure light type
                            IF Yes THEN ' check user input response
                                Outpt = "Light charges(1-32767)? " ' make input prompt
                                CALL IO.I ' get user input
                                Light.Charges = INT(VAL(Inpt)) ' convert input to integer
                                TreasureRecord.LightType = True ' set treasure light type
                                ' store treasure charges
                                TreasureRecord.LightCharges = Light.Charges
                            END IF ' end check user input
                        CASE "N" ' option to change treasure vehicle
                            Outpt = "Treasure is a vehicle(y/n)? " ' make input prompt
                            No.Input.Out = "Y" ' default input
                            CALL IO.I ' get user input
                            TreasureRecord.Vehicle = False ' reset treasure vehicle
                            TreasureRecord.VehicleHits = False ' reset treasure vehicle
                            TreasureRecord.VehicleType = False ' reset treasure vehicle
                            IF Yes THEN ' check user input response
                                Outpt = "Vehicle hits(1-32767)? " ' make input prompt
                                CALL IO.I ' get user input
                                Vehicle.Hits = INT(VAL(Inpt)) ' convert input to integer
                                IF Vehicle.Hits > False THEN ' check input range
                                    TreasureRecord.Vehicle = True ' set treasure vehicle
                                    ' store vehicle hits
                                    TreasureRecord.VehicleHits = Vehicle.Hits
                                    Graphics.Off = True ' reset color
                                    Outpt = "[A]ir" ' make display message
                                    CALL IO.O ' send message
                                    Outpt = "[L]and" ' make display message
                                    CALL IO.O ' send message
                                    Outpt = "[W]ater" ' make display message
                                    CALL IO.O ' send message
                                    Outpt = "[X]All terrain" ' make display message
                                    CALL IO.O ' send message
                                    Graphics.Off = False ' reset color
                                    Outpt = "Vehicle type edit option? " ' make input prompt
                                    No.Input.Out = "Q" ' default input
                                    CALL IO.I ' get user input
                                    SELECT CASE UCASE$(Inpt) ' selection of vehicle type
                                        CASE "A" ' air vehicle type
                                            TreasureRecord.VehicleType = Air ' set vehicle type
                                        CASE "L" ' land vehicle type
                                            TreasureRecord.VehicleType = Land ' set vehicle type
                                        CASE "W" ' underwaater vehicle type
                                            TreasureRecord.VehicleType = Water ' set vehicle type
                                        CASE "X" ' all terrain vehicle type
                                            TreasureRecord.VehicleType = AllTerrain ' vehicle type
                                    END SELECT ' end selection of vehicle type
                                END IF ' end check input range
                            END IF ' end check user input
                        CASE "O" ' option to change treasure ammunition type
                            Outpt = "Treasure loads from devices(y/n)? " ' make input prompt
                            No.Input.Out = "Y" ' default input
                            CALL IO.I ' get user input
                            TreasureRecord.Ammunition = Yes ' set treasure ammunition type
                        CASE "P" ' option to change treasure loadable type
                            Outpt = "Treasure is loadable device(y/n)? " ' make input prompt
                            No.Input.Out = "Y" ' default input
                            CALL IO.I ' get user input
                            ' reset treasure loadable type
                            TreasureRecord.Loadable = False
                            TreasureRecord.AmmoLoads = False ' reset treasure loadable type
                            IF Yes THEN ' check user input response
                                ' write current treasure record
                                CALL Share.Record(TreasureFile, Treasure.Number)
                                Outpt = "Treasure which loads this device:" ' make message
                                CALL IO.O ' send message
                                ' routine to get treasure number
                                CALL Find.Treasure(Treasure.Device)
                                IF Treasure.Device = False THEN ' check number range
                                    ' read current treasure record
                                    CALL Read.Record(TreasureFile, Treasure.Number)
                                    TreasureRecord.Loadable = False ' reset loadable type
                                    TreasureRecord.AmmoLoads = False ' reset loadable type
                                ELSE ' check input range
                                    ' read loadable treasure record
                                    CALL Read.Record(TreasureFile, Treasure.Device)
                                    IF TreasureRecord.Ammunition = False THEN ' check type
                                        ' read current record
                                        CALL Read.Record(TreasureFile, Treasure.Number)
                                        Outpt = "That treasure is not ammunition." ' message
                                        CALL IO.O ' send message
                                        TreasureRecord.Loadable = False ' reset loadable type
                                        TreasureRecord.AmmoLoads = False ' reset loadable type
                                    ELSE ' check ammunition type
                                        ' read current record
                                        CALL Read.Record(TreasureFile, Treasure.Number)
                                        ' set ammunition loadable
                                        TreasureRecord.Loadable = True
                                        ' store ammunition number
                                        TreasureRecord.AmmoLoads = Treasure.Device
                                    END IF ' end check ammunition type
                                END IF ' end check user input range
                                ' write current record number
                                CALL Share.Record(TreasureFile, Treasure.Number)
                            END IF ' end check user input
                        CASE "R" ' option to change treasure launchs
                            ' make input prompt
                            Outpt = "Treasure launchs from devices(y/n)? "
                            No.Input.Out = "Y" ' default input
                            CALL IO.I ' get user input
                            TreasureRecord.LaunchAmmo = Yes ' store launchs type
                        CASE "S" ' option to change treasure launchable type
                            Outpt = "Treasure is launchable device(y/n)? " ' make input
                            No.Input.Out = "Y" ' default input
                            CALL IO.I ' get user input
                            TreasureRecord.Launchable = False ' reset launchable type
                            TreasureRecord.LaunchLoads = False ' reset launchable type
                            IF Yes THEN ' check user input response
                                ' write current treasure record
                                CALL Share.Record(TreasurFile, Treasure.Number)
                                Outpt = "Treasure which launchs from this device:" ' message
                                CALL IO.O ' send message
                                ' routine to get treasure number
                                CALL Find.Treasure(Treasure.Device)
                                ' check treasure number range
                                IF Treasure.Device = False THEN
                                    ' read current treasure record
                                    CALL Read.Record(TreasureFile, Treasure.Number)
                                    TreasureRecord.Launchable = False ' reset launchable type
                                    TreasureRecord.LaunchLoads = False ' reset launchable type
                                ELSE ' check number range
                                    ' read lannchable treasure
                                    CALL Read.Record(TreasureFile, Treasure.Device)
                                    IF TreasureRecord.LaunchAmmo = False THEN ' check launch
                                        ' read current record
                                        CALL Read.Record(TreasureFile, Treasure.Number)
                                        Outpt = "That treasure does not launch from devices."
                                        CALL IO.O ' send error message
                                        ' reset launchable type
                                        TreasureRecord.Launchable = False
                                        TreasureRecord.LaunchLoads = False ' reset launchable
                                    ELSE ' check launchable type
                                        ' read current record
                                        CALL Read.Record(TreasureFile, Treasure.Number)
                                        TreasureRecord.Launchable = True ' set launchable
                                        ' store launch number
                                        TreasureRecord.LaunchLoads = Treasure.Device
                                    END IF ' end check launchable number type
                                END IF ' end check launchable treasure number
                                CALL Share.Record(TreasureFile, Treasure.Number) ' write current record
                            END IF ' end check user input
                        CASE "T" ' option to change movable type
                            Outpt = "Launchable device can be moved(y/n)? " ' make input
                            No.Input.Out = "Y" ' default input
                            CALL IO.I ' get user input
                            TreasureRecord.Movable = Yes ' store movable type
                        CASE "U" ' option  to change coin type
                            Outpt = "Treasure is coins(y/n)? " ' make input prompt
                            No.Input.Out = "Y" ' default input
                            CALL IO.I ' get user input
                            TreasureRecord.Coin = Yes ' store coin type
                        CASE "V" ' option to change potion type
                            Outpt = "Treasure is potion(y/n)? " ' make input prompt
                            No.Input.Out = "Y" ' default input
                            CALL IO.I ' get user input
                            TreasureRecord.Potion = Yes ' store potion type
                        CASE "W" ' option to change scroll type
                            Outpt = "Treasure is scroll(y/n)? " ' make input prompt
                            No.Input.Out = "Y" ' default input
                            CALL IO.I ' get user input
                            TreasureRecord.Scroll = Yes ' store scroll type
                        CASE "X" ' option to change invisible treasure
                            Outpt = "Treasure is invisible(y/n)? " ' make input prompt
                            No.Input.Out = "Y" ' default input
                            CALL IO.I ' get user input
                            TreasureRecord.Invisible = Yes ' store invisible type
                        CASE "Y" ' option to change container type
                            Outpt = "Treasure is a container(y/n)? " ' make input prompt
                            No.Input.Out = "Y" ' default input
                            CALL IO.I ' get user input
                            TreasureRecord.Container = Yes ' store container type
                        CASE "Z" ' option to change container lock
                            Outpt = "Container is locked(y/n)? " ' make input prompt
                            No.Input.Out = "Y" ' default input
                            CALL IO.I ' get user input
                            TreasureRecord.Locked = False ' reset container lock
                            TreasureRecord.Closed = False ' reset container lock
                            IF Yes THEN ' check user input response
                                TreasureRecord.Locked = 1 ' set container lock
                                TreasureRecord.Closed = 1 ' set container lock
                            END IF ' end check user input
                        CASE "1" ' option to change key number
                            Outpt = "Treasure key number(1-32767)? " ' make input prompt
                            No.Input.Out = "1" ' default input
                            CALL IO.I ' get user input
                            TreasureRecord.Keyed = INT(VAL(Inpt)) ' store key number
                        CASE "2" ' option to change treasure fuel
                            Outpt = "Treasure is fuel(y/n)? " ' make input prompt
                            No.Input.Out = "Y" ' default input
                            CALL IO.I ' get user input
                            TreasureRecord.FuelType = False ' reset fuel type
                            TreasureRecord.FuelCharges = False ' reset fuel type
                            IF Yes THEN ' check user input response
                                Outpt = "Fuel charges(1-32767)? " ' make input prompt
                                CALL IO.I ' get user input
                                Fuel.Charges = INT(VAL(Inpt)) ' convert input to integer
                                TreasureRecord.FuelType = True ' set fuel type
                                ' store fuel charges
                                TreasureRecord.FuelCharges = Fuel.Charges
                            END IF ' end check user input
                        CASE "3" ' option to change rusting treasure
                            Outpt = "Treasure can rust(y/n)? " ' make input prompt
                            No.Input.Out = "Y" ' default input
                            CALL IO.I ' get user input
                            TreasureRecord.Rustable = False ' reset rusting treasure
                            TreasureRecord.RustPercent = False ' reset rusting treasure
                            IF Yes THEN ' check user input response
                                Outpt = "Percent chance weapon " ' make input prompt
                                Outpt = Outpt + "will rust during room rust rate(1-100)? "
                                No.Input.Out = "50" ' default input
                                CALL IO.I ' get user input
                                Percent = INT(VAL(Inpt)) ' convert input to integer
                                IF Percent >= 1 AND Percent <= 100 THEN ' check input range
                                    TreasureRecord.Rustable = True ' set rusting treasure
                                    ' store rusting percent
                                    TreasureRecord.RustPercent = Percent
                                ELSE ' check range
                                    TreasureRecord.Rustable = False ' reset rusting treasure
                                    ' reset rusting treasure
                                    TreasureRecord.RustPercent = False
                                END IF ' end check range
                            END IF ' end check user input
                        CASE "4" ' option to change stealable treasure
                            Outpt = "Treasure can be stolen(y/n)? " ' make input prompt
                            No.Input.Out = "Y" ' default input
                            CALL IO.I ' get user input
                            TreasureRecord.Stealable = False ' reset steable treasure
                            TreasureRecord.StealPercent = False ' reset steable treasure
                            IF Yes THEN ' check user input response
                                Outpt = "Percent chance treasure " ' make input prompt
                                Outpt = Outpt + "stolen during room steal rate(1-100)? "
                                No.Input.Out = "50" ' default input
                                CALL IO.I ' get user input
                                Percent = INT(VAL(Inpt)) ' convert input to inetger
                                IF Percent >= 1 AND Percent <= 100 THEN ' check input range
                                    TreasureRecord.Stealable = True ' set steable treasure
                                    ' set steable percent
                                    TreasureRecord.StealPercent = Percent
                                ELSE ' check range
                                    TreasureRecord.Stealable = False ' reset steable treasure
                                    ' reset steable treasure
                                    TreasureRecord.StealPercent = False
                                END IF ' end check range
                            END IF ' end check user input
                        CASE "Q" ' option to exit treasure change menu
                            EXIT DO ' exit treasure change menu
                    END SELECT ' end selection of treasure menu option
                LOOP ' end loop through treasure change menu
                CALL Share.Record(TreasureFile, Treasure.Number) ' write treasure record
                Outpt = "Treasure '" + RTRIM$(TreasureRecord.TreasureName) + "' changed."
                CALL IO.O ' send message
            END IF ' end check number range
        CASE "L" ' option to list range of treasure numbers
            Max.Treasure = LOF(TreasureFile) / LEN(TreasureRecord) ' store length of treasure file
            ' get range of treasure numbers
            CALL Get.Range(Max.Treasure, Treasure.List1, Treasure.List2)
            ' loop through treasure numbers
            Allow.Break = True ' set allow break flag
            Break = False ' reset control-k flag
            Continue = False ' set continuous flag
            FOR Treasure.Count = Treasure.List1 TO Treasure.List2
                CALL Read.Record(TreasureFile, Treasure.Count) ' read treasure record
                CALL Display.Treasure(Treasure.Count) ' routine to display treasure
                IF Break THEN ' check control-k flag
                    EXIT FOR ' exit display loop
                END IF ' end check control-k
                Graphics.Off = False ' reset color
                IF Continue = False THEN ' check continuous flag
                    CALL More.Prompt ' routine to pause
                    IF No THEN ' check pause input response
                        EXIT FOR ' exit loop through treasure numbers
                    END IF ' end check user input
                END IF ' end check continuous flag
            NEXT ' end loop through treasure numbers
            Allow.Break = False ' reset allow break flag
            IF Break THEN ' check control-k flag
                Break = False ' reset control-k flag
                Outpt = Nul ' set output to null
                CALL IO.O ' send empty return
            END IF ' end check control-k flag
        CASE "Q" ' option to exit treasure edit menu
            EXIT DO ' exit treasure edit menu
    END SELECT ' end selection of treasure edit option
LOOP ' end loop through treasure edit menu
END SUB ' end routine to edit treasure

REM * routine to edit user file.

SUB Edit.User
DO ' loop through user edit option menu
    Graphics.Off = False ' reset color
    Outpt = "User edit:" ' make display message
    CALL IO.O ' send message
    Graphics.Off = True ' reset color
    Outpt = "[A]dd" ' make display message
    CALL IO.O ' send message
    Outpt = "[C]hange" ' make display message
    CALL IO.O ' send message
    Outpt = "[D]elete" ' make display message
    CALL IO.O ' send message
    Outpt = "[L]ist" ' make display message
    CALL IO.O ' send message
    Outpt = "[U]pdate" ' make display message
    CALL IO.O ' send message
    Graphics.Off = False ' reset color
    Outpt = "User edit option(q to quit)? " ' make input prompt
    No.Input.Out = "Q" ' default input
    CALL IO.I ' get user input
    SELECT CASE UCASE$(Inpt) ' selection of user edit option
        CASE "A" ' option to add user record
            IF Local.Mode = False THEN ' check local sysop online
                Outpt = "Remote DMs cannot add users." ' make message
                CALL IO.O ' send error message
            ELSE ' check local sysop
                New.User = LOF(UserFile) / LEN(UserRecord) + 1 ' store last user number
                Outpt = Dead$ ' store empty codename
                CALL Valid(Outpt, 30) ' validate codename
                CALL Encrypt(Outpt, True) ' encrypt codename
                UserRecord.CodeName = Outpt ' store empty codename
                Outpt = Deleted$ ' store empty password
                CALL Valid(Outpt, 20) ' validate password
                CALL Encrypt(Outpt, False) ' encrypt password
                UserRecord.PassWord = Outpt ' store empty password
                Outpt = Class.Name(1) ' store defualt class name
                CALL Valid(Outpt, 20) ' validate class name
                CALL Encrypt(Outpt, True) ' encrypt class name
                UserRecord.ClassName = Outpt ' store default class name
                UserRecord.ClassType = Fighter ' store default class type
                FOR Array.Index = 1 TO 7 ' loop through user statistics
                    CALL PutUserRecordStats(Array.Index, 12) ' store default statistic
                NEXT ' end loop through statistics
                FOR Array.Index = 1 TO 4 ' loop through user weapons
                    CALL PutUserRecordWeapons(Array.Index, False) ' store default weapon
                NEXT ' end loop through weapons
                CALL PutUserRecordWeapons(3, 10) ' store default weapon
                UserRecord.Room = 1! ' reset user record item
                UserRecord.Level = 1 ' reset user record item
                UserRecord.Experience = 64 ' reset user record item
                UserRecord.Gold = 2048 ' reset user record item
                UserRecord.Borrow = False ' reset user record item
                UserRecord.NumCalls = False ' reset user record item
                UserRecord.Poison = False ' reset user record item
                UserRecord.Invisible = False ' reset user record item
                UserRecord.Race = 1 ' reset user record item
                UserRecord.Proficiency = 3 ' reset user record item
                UserRecord.Bank = False ' reset user record item
                UserRecord.MonstersKilled = False ' reset user record item
                UserRecord.Brief = False ' reset user record item
                UserRecord.Echo = False ' reset user echo mode
                UserRecord.Linefeeds = False ' reset user linefeed mode
                UserRecord.Linelength = 80 ' reset user linelength
                UserRecord.Pagelength = 24 ' reset pagelength
                UserRecord.Wordwrap = False ' reset user word wrap
                Outpt = DATE$ ' store current date
                CALL Valid(Outpt, 10) ' validate date
                CALL Encrypt(Outpt, True) ' encrypt date
                UserRecord.DateOn = Outpt ' reset user record item
                UserRecord.Align1 = False ' reset user record item
                UserRecord.Align2 = False ' reset user record item
                UserRecord.Flags = False ' reset user record item
                UserRecord.MaxCalls = False ' reset user record item
                UserRecord.FromHour = False ' reset user record item
                UserRecord.FromMin = False ' reset user record item
                UserRecord.ToHour = False ' reset user record item
                UserRecord.ToMin = False ' reset user record item
                ContainerRec.Closed = False ' reset user record item
                ContainerRec.ContainerName = Nul ' reset user record item
                ContainerRec.Keyed = False ' reset user record item
                ContainerRec.Locked = False ' reset user record item
                ContainerRec.ShortName = Nul ' reset user record item
                ContainerRec.Permanent = False ' reset user record item
                CALL Clear.Container(0, True) ' routine to clear the container structure
                FOR Array.Count = 1 TO 3 ' loop through user containers
                    ' store empty container
                    CALL PutUserRecordContainer(Array.Count)
                NEXT ' end loop through containers

                CALL PutUserRecordTrainRoom(1, 36)
                CALL PutUserRecordTrainRoom(2, 4)
                CALL PutUserRecordTrainRoom(3, 123)
                CALL PutUserRecordTrainRoom(4, 163)
                CALL PutUserRecordTrainRoom(5, 162)
                CALL PutUserRecordTrainRoom(6, 426)
                CALL PutUserRecordTrainRoom(7, 421)
                CALL PutUserRecordTrainRoom(8, 417)
                CALL PutUserRecordTrainRoom(9, 1)
                CALL PutUserRecordTrainRoom(10, 1)

                CALL Share.Record(UserFile, New.User) ' write user record
                Outpt = "New user record added." ' make display message
                CALL IO.O ' send message
                CALL Read.Record(UserFile, User.Index)
            END IF ' end check local sysop
        CASE "D" ' option to delete user
            IF Local.Mode = False THEN ' check local sysop online
                Outpt = "Remote DMs cannot delete users." ' make message
                CALL IO.O ' send error message
            ELSE ' check local sysop
                CALL Find.User(User.Found, User.Number) ' get user number
                IF User.Found THEN ' check user found flag
                    IF User.Number = User.Index THEN ' check user number
                        Outpt = "Can't delete you. Try Suicide.." ' make output
                        CALL IO.O ' send output
                    ELSE ' check user number
                        Inpt = UserRecord.CodeName ' store player codename
                        CALL Decrypt(Inpt) ' decrypt codename
                        Inpt = RTRIM$(Inpt) ' trim codename
                        Inpt = LCASE$(Inpt) ' lowercase codename
                        MID$(Inpt, 1, 1) = UCASE$(MID$(Inpt, 1, 1)) ' uppercase first word
                        Outpt = "Delete '" + Inpt + "'. Are you sure(y/n)? " ' make input prompt
                        No.Input.Out = "N" ' default input
                        CALL IO.I ' get user input
                        IF Yes THEN ' check input response
                            Outpt = "User deleted." ' make deleted message
                            CALL IO.O ' send message
                            CALL Delete.User ' routine to clear user record
                            CALL Share.Record(UserFile, User.Number) ' write user record
                        END IF ' end check input response
                    END IF ' end check user number
                END IF ' end check user found flag
            END IF ' end check local sysop online
            CALL Read.Record(UserFile, User.Index)
        CASE "C" ' option to change user
            CALL Find.User(User.Found, User.Number) ' get user number
            IF User.Found THEN ' compare found flag
                DO ' loop through codename edit option menu
                    CALL Display.User(User.Number) ' routine to display player
                    Graphics.Off = False ' reset color
                    Outpt = "User change option(q to quit)? " ' make input prompt
                    No.Input.Out = "Q" ' default input
                    CALL IO.I ' get user input
                    SELECT CASE UCASE$(Inpt) ' selection of player edit option
                        CASE "A" ' option to change player codename
                            IF Local.Mode = False THEN ' check local sysop online
                                Outpt = "Remote DMs cannot edit codename." ' make message
                                CALL IO.O ' send error message
                            ELSE ' check local sysop
                                Outpt = "Codename? " ' make input prompt
                                No.Input.Out = Deleted$ ' default input
                                CALL IO.I ' get user input
                                Inpt = UCASE$(Inpt) ' uppercase input
                                CALL Valid(Inpt, 30) ' validate input
                                IF Inpt = Nul THEN ' check validation
                                    Outpt = "Illegal characters in codename." ' message
                                    CALL IO.O ' send error message
                                ELSE ' check validation
                                    CALL Encrypt(Inpt, True) ' encrypt codename
                                    UserRecord.CodeName = Inpt ' store codename
                                END IF ' end check validate
                            END IF ' end check response
                        CASE "B" ' option to change player password
                            IF Local.Mode = False THEN ' check local sysop online
                                Outpt = "Remote DMs cannot edit password." ' make message
                                CALL IO.O ' send error message
                            ELSE ' check local sysop
                                Inpt = UserRecord.PassWord ' store password
                                CALL Decrypt(Inpt) ' decrypt password
                                IF Inpt = Nul THEN ' check decryption byte checksum
                                    Outpt = "This password has a checksum error." ' message
                                    CALL IO.O ' send error message
                                END IF ' end check byte checksum
                                Outpt = "Password? " ' make input prompt
                                No.Input.Out = Deleted$ ' default input
                                CALL IO.I ' get user input
                                Inpt = UCASE$(Inpt) ' uppercase input
                                CALL Valid(Inpt, 20) ' validate input
                                IF Inpt = Nul THEN ' check validation
                                    Outpt = "Illegal characters in password." ' message
                                    CALL IO.O ' send error message
                                ELSE ' check validation
                                    CALL Encrypt(Inpt, False) ' encrypt password
                                    UserRecord.PassWord = Inpt ' store password
                                END IF ' end check validation
                            END IF ' end check local sysop online
                        CASE "C" ' option to change player level
                            Outpt = "Level(1-32767)? " ' make input prompt
                            No.Input.Out = "1" ' default input
                            CALL IO.I ' get user input
                            Level# = INT(VAL(Inpt)) ' convert input to double
                            IF Level# > False AND Level# <= MaxInt THEN ' compare input range
                                New.Level = CINT(Level#) ' convert input to integer
                                UserRecord.Level = New.Level ' store player level
                            END IF ' end compare input range
                        CASE "D" ' option to change player class number
                            Graphics.Off = True ' reset color
                            IF Local.Mode = False THEN ' check local sysop online
                                Max.Class = 8 ' store maximum class number
                            ELSE ' check local sysop
                                Max.Class = 10 ' store maximum class number
                            END IF ' end check local sysop online
                            ' loop through class number range
                            FOR Class.Count = 1 TO Max.Class
                                ' make class display message
                                Outpt = "[" + MID$(STR$(Class.Count), 2) + "]" + RTRIM$(Class.Name(Class.Count))
                                CALL IO.O ' send message
                            NEXT ' end loop through class numbers
                            Outpt = "Class option" ' make range prompt
                            ' return number from range
                            CALL Get.Range2(0, Max.Class, Class.Number)
                            IF Class.Number > False THEN ' check number bounds
                                ' store player class number
                                UserRecord.ClassType = Class.Number
                            END IF ' end check number bounds
                        CASE "E" ' option to change player weapon type
                            Graphics.Off = True ' reset color
                            FOR Weapon.COunt = 1 TO 4 ' loop through weapon type names
                                ' make weapon type message
                                Outpt = "[" + MID$(STR$(Weapon.COunt), 2) + "]" + RTRIM$(WeaponTypeName(Weapon.COunt))
                                CALL IO.O ' send message
                            NEXT ' end loop through weapon type names
                            Graphics.Off = False ' reset color
                            Outpt = "Weapon proficiency" ' make range prompt
                            CALL Get.Range2(0, 4, Weapon.Value) ' get number from range
                            IF Weapon.Value > False THEN ' check number bounds
                                ' store weapon type number
                                UserRecord.Proficiency = Weapon.Value
                            END IF ' end check number bounds
                        CASE "F" ' option to change weapon proficiency number
                            Outpt = "Blunt weapon proficiency%(1-100)? " ' input prompt
                            No.Input.Out = "10" '  default input
                            CALL IO.I ' get user input
                            CALL PutUserRecordWeapons(1, INT(VAL(Inpt))) ' store weapon value
                        CASE "G" ' option to change weapon proficiency number
                            Outpt = "Pole weapon proficiency%(1-100)? " ' input prompt
                            No.Input.Out = "10" '  default input
                            CALL IO.I ' get user input
                            CALL PutUserRecordWeapons(2, INT(VAL(Inpt))) ' store weapon value
                        CASE "H" ' option to change weapon proficiency number
                            Outpt = "Sharp weapon proficiency%(1-100)? " ' input prompt
                            No.Input.Out = "10" '  default input
                            CALL IO.I ' get user input
                            CALL PutUserRecordWeapons(3, INT(VAL(Inpt))) ' store weapon value
                        CASE "I" ' option to change weapon proficiency number
                            ' input prompt
                            Outpt = "Thrusting weapon proficiency%(1-100)? "
                            No.Input.Out = "10" '  default input
                            CALL IO.I ' get user input
                            CALL PutUserRecordWeapons(4, INT(VAL(Inpt))) ' store weapon value
                        CASE "J" ' option to change player class name
                            Outpt = "Classname? " ' make input prompt
                            No.Input.Out = Deleted$ ' default input
                            CALL IO.I ' get user input
                            CALL Valid(Inpt, 20) ' validate input
                            IF Inpt = Nul THEN ' check validation
                                Outpt = "Illegal characters in classname." ' make message
                                CALL IO.O ' send error message
                            ELSE ' end check validation
                                CALL Encrypt(Inpt, True) ' encrypt clsss name
                                UserRecord.ClassName = Inpt ' store class name
                            END IF ' end check validation
                        CASE "K" ' option to change player statistic
                            Outpt = "Strength? " ' make input prompt
                            No.Input.Out = "12" ' default input
                            CALL IO.I ' get user input
                            CALL PutUserRecordStats(1, INT(VAL(Inpt))) ' store player staistic
                        CASE "L" ' option to change player statistic
                            Outpt = "Intelligence? " ' make input prompt
                            No.Input.Out = "12" ' default input
                            CALL IO.I ' get user input
                            CALL PutUserRecordStats(2, INT(VAL(Inpt))) ' store player staistic
                        CASE "M" ' option to change player statistic
                            Outpt = "Wisdom? " ' make input prompt
                            No.Input.Out = "12" ' default input
                            CALL IO.I ' get user input
                            CALL PutUserRecordStats(3, INT(VAL(Inpt))) ' store player staistic
                        CASE "N" ' option to change player statistic
                            Outpt = "Dexterity? " ' make input prompt
                            No.Input.Out = "12" ' default input
                            CALL IO.I ' get user input
                            CALL PutUserRecordStats(4, INT(VAL(Inpt))) ' store player staistic
                        CASE "O" ' option to change player statistic
                            Outpt = "Constitution? " ' make input prompt
                            No.Input.Out = "12" ' default input
                            CALL IO.I ' get user input
                            CALL PutUserRecordStats(5, INT(VAL(Inpt))) ' store player staistic
                        CASE "P" ' option to change player statistic
                            Outpt = "Piety? " ' make input prompt
                            No.Input.Out = "12" ' default input
                            CALL IO.I ' get user input
                            CALL PutUserRecordStats(6, INT(VAL(Inpt))) ' store player staistic
                        CASE "R" ' option to change player statistic
                            Outpt = "Charisma? " ' make input prompt
                            No.Input.Out = "12" ' default input
                            CALL IO.I ' get user input
                            CALL PutUserRecordStats(7, INT(VAL(Inpt))) ' store player staistic
                        CASE "S" ' option to change player experience
                            Outpt = "Experience? " ' make input prompt
                            No.Input.Out = "128" ' default input
                            CALL IO.I ' get user input
                            UserRecord.Experience = CDBL(INT(VAL(Inpt)))
                        CASE "T" ' option to change player gold hoard
                            Outpt = "Gold? " ' make input prompt
                            No.Input.Out = "256" ' default input
                            CALL IO.I ' get user input
                            UserRecord.Gold = CDBL(INT(VAL(Inpt))) ' store player gold
                        CASE "U" ' option to change player room number
                            Outpt = "Room number? " ' make input prompt
                            No.Input.Out = "1" ' default input
                            CALL IO.I ' get user input
                            UserRecord.Room = INT(VAL(Inpt)) ' store room number
                        CASE "V" ' option to change player calls
                            Graphics.Off = True ' reset color
                            Outpt = "Calls made today? " ' make input prompt
                            No.Input.Out = "0" ' default input
                            CALL IO.I ' get user input
                            UserRecord.NumCalls = INT(VAL(Inpt)) ' store player calls
                            Outpt = "Maximum calls per day? " ' make input prompt
                            No.Input.Out = "5" ' default input
                            CALL IO.I ' get user input
                            UserRecord.MaxCalls = INT(VAL(Inpt)) ' store plater calls
                            Outpt = "Timeon restriction(from hour: form HH)? " ' prompt
                            No.Input.Out = "00" ' default input
                            CALL IO.I ' get user input
                            UserRecord.FromHour = INT(VAL(Inpt)) ' store player time
                            Outpt = "Timeon restriction(from minute: form MM)? " ' prompt
                            No.Input.Out = "00" ' default input
                            CALL IO.I ' get user input
                            UserRecord.FromMin = INT(VAL(Inpt)) ' store player time
                            Outpt = "Timeon restriction(to hour: form HH)? " ' prompt
                            No.Input.Out = "00" ' default input
                            CALL IO.I ' get user input
                            UserRecord.ToHour = INT(VAL(Inpt)) ' store player time
                            Outpt = "Timeon restriction(to minute: form MM)? " ' prompt
                            No.Input.Out = "00" ' default input
                            CALL IO.I ' get user input
                            UserRecord.ToMin = INT(VAL(Inpt)) ' store player time
                            Graphics.Off = False ' reset color
                        CASE "W" ' option to edit player inventory
                            DO ' loop through inventory edit menu
                                Graphics.Off = True ' reset color
                                Outpt = "[C]ontainers" ' make display message
                                CALL IO.O ' send message
                                Outpt = "[O]bjects" ' make display message
                                CALL IO.O ' send message
                                Outpt = "[T]reasure" ' make display message
                                CALL IO.O ' send message
                                Graphics.Off = False ' reset color
                                Outpt = "Inventory edit option(q to quit)? " ' prompt
                                No.Input.Out = "Q" ' default input
                                CALL IO.I ' get user input
                                SELECT CASE UCASE$(Inpt) ' selection of inventory option
                                    CASE "C" ' option to edit player containers
                                        DO ' loop through user container edit menu
                                            Graphics.Off = False ' reset color
                                            Outpt = "User container edit options:" ' make display message
                                            CALL IO.O ' send message
                                            Graphics.Off = True ' reset color
                                            Outpt = "[A]dd" ' make option message
                                            CALL IO.O ' send option message
                                            Outpt = "[C]hange" ' make option message
                                            CALL IO.O ' send option message
                                            Outpt = "[D]elete" ' make option message
                                            CALL IO.O ' send option message
                                            Outpt = "[L]ist" ' make option message
                                            CALL IO.O ' send option message
                                            Graphics.Off = False ' reset color
                                            Outpt = "Enter user container edit option(q to quit)? " ' make option prompt
                                            No.Input.Out = "Q" ' store default input
                                            CALL IO.I ' get option input
                                            SELECT CASE UCASE$(Inpt) ' make selection of input option
                                                CASE "A" ' add container
                                                    Treasure.Added = False ' set container added flag
                                                    FOR Number.Containers = 1 TO 3 ' loop through user containers
                                                        X = GetUserRecord.Container(Number.Containers) ' store user container into record
                                                        IF RTRIM$(ContainerRec.ShortName) = Nul THEN ' compare container name length
                                                            Outpt = "Add container name to slot #" + MID$(STR$(Number.Containers), 2) + "." ' make add message
                                                            CALL IO.O ' send message
                                                            ' routine to get treasure number
                                                            CALL Find.Treasure(Treasure.Number)
                                                            IF Treasure.Number > False THEN ' check treasure number
                                                                IF TreasureRecord.Container THEN ' check treasure type
                                                                    ' store container variables
                                                                    ContainerRec.Closed = TreasureRecord.Closed
                                                                    ContainerRec.ContainerName = TreasureRecord.TreasureName
                                                                    ContainerRec.Locked = TreasureRecord.Locked
                                                                    ContainerRec.Keyed = TreasureRecord.Keyed
                                                                    ContainerRec.ShortName = TreasureRecord.ShortName
                                                                    ContainerRec.Permanent = TreasureRecord.Permanent
                                                                    FOR Container.Count = 1 TO 5 ' loop through container contents
                                                                        ' clear container contents
                                                                        CALL Clear.Container(Container.Count, False)
                                                                    NEXT ' end loop through container
                                                                    CALL PutUserRecordContainer(Number.Containers) ' add container record to room
                                                                    Treasure.Added = True ' set container added flag
                                                                END IF ' end check treasure type
                                                            END IF ' end check treasure number
                                                        END IF ' end check container name length
                                                    NEXT ' end loop through user containers
                                                    IF Treasure.Added THEN ' check added flag
                                                        Outpt = "Container(s) added to user." ' make message
                                                    ELSE ' check added flag
                                                        Outpt = "Container not added to user." ' make message
                                                    END IF ' end check added flag
                                                    CALL IO.O ' display message
                                                CASE "C" ' change container
                                                    Outpt = "Enter container number(1-3)?" ' get container number
                                                    CALL IO.I ' get input
                                                    Container.Number = INT(VAL(Inpt)) ' store input
                                                    IF Container.Number >= 1 AND Container.Number <= 3 THEN ' check container number
                                                        X = GetUserRecordContainer(Container.Number) ' store user container into record
                                                        IF RTRIM$(ContainerRec.ShortName) <> Nul THEN ' compare container name length
                                                            CALL Change.Container(0) ' routine to edit container
                                                            CALL PutUserRecordContainer(Container.Number) ' store container
                                                            Outpt = "Container changed." ' make message
                                                            CALL IO.O ' send message
                                                        END IF ' end check container name
                                                    END IF ' end check container number
                                                CASE "D" ' delete container
                                                    Treasure.Deleted = False ' set container deleted flag
                                                    Outpt = "Enter container number(1-3)?" ' get container number
                                                    CALL IO.I ' get input
                                                    Container.Number = INT(VAL(Inpt)) ' store input
                                                    IF Container.Number >= 1 AND Container.Number <= 3 THEN ' check container number
                                                        X = GetUserRecordContainer(Container.Number) ' store user container into record
                                                        IF RTRIM$(ContainerRec.ShortName) <> Nul THEN ' compare container name length
                                                            CALL Clear.Container(0, True) ' clear entire container record
                                                            CALL PutUserRecordContainer(Container.Number) ' clear user container
                                                            Treasure.Deleted = True ' reset container deleted flag
                                                        END IF ' end compare container in user
                                                    END IF ' end check container number
                                                    IF Treasure.Deleted THEN ' check deleted flag
                                                        Outpt = "Container deleted from user." ' make message
                                                    ELSE ' check added flag
                                                        Outpt = "Container not deleted from user." ' make message
                                                    END IF ' end check added flag
                                                    CALL IO.O ' display message
                                                CASE "L" ' list containers
                                                    Graphics.Off = True ' reset color
                                                    Container.Count = False ' reset container count
                                                    FOR Number.Containers = 1 TO 3 ' loop through all user containers
                                                        X = GetUserRecordContainer(Number.Containers) ' store user container into record
                                                        IF RTRIM$(ContainerRec.ShortName) <> Nul THEN ' compare container name length
                                                            Container.Count = True ' reset count flag
                                                            Outpt = RTRIM$(ContainerRec.ContainerName) ' store container name
                                                            MID$(Outpt, 1, 1) = UCASE$(MID$(Outpt, 1, 1)) ' uppercase first letter
                                                            Outpt = "#" + MID$(STR$(Number.Containers), 2) + ":" + Outpt ' store container number
                                                            IF ContainerRec.Locked > False THEN ' check container is locked
                                                                Outpt = Outpt + "[locked]" ' append message
                                                            ELSE ' container is not locked
                                                                IF ContainerRec.Closed > False THEN ' check container is closed
                                                                    Outpt = Outpt + "[closed]" ' append message
                                                                END IF ' end check closed container
                                                            END IF ' end check locked container
                                                            IF ContainerRec.Keyed THEN ' check container has a key number
                                                                Outpt = Outpt + "(#" + RIGHT$(STR$(ContainerRec.Keyed + 100000!), 5) + ")"
                                                            END IF ' append key number to container name
                                                            Number.Items = False ' reset counter
                                                            FOR Treasure.Number = 1 TO 5 ' loop through items
                                                                Treasure.Item = GetContainerRecInventory(Treasure.Number) ' store item index
                                                                ' compare item range
                                                                IF Treasure.Item > False AND Treasure.Item <= LOF(TreasureFile) / LEN(TreasureRecord) THEN
                                                                    Number.Items = Number.Items + 1 ' increment counter
                                                                END IF ' end check range
                                                            NEXT ' end item loop
                                                            Outpt = Outpt + ":" + STR$(Number.Items)
                                                            IF Number.Items = 1 THEN ' check counter
                                                                Outpt = Outpt + " item." ' append message
                                                            ELSE ' check counter
                                                                Outpt = Outpt + " items." ' append message
                                                            END IF ' end check counter
                                                            CALL IO.O ' display message
                                                        END IF ' end compare container name length
                                                    NEXT ' end container count loop
                                                    IF Container.Count = False THEN
                                                        Outpt = "<no containers>" ' make message
                                                        CALL IO.O ' display message
                                                    END IF
                                                CASE "Q" ' quit container menu
                                                    EXIT DO
                                            END SELECT
                                        LOOP ' end room container loop
                                    CASE "O" ' option to edit player object inventory
                                        DO ' loop through object inventory edit menu
                                            Graphics.Off = False ' reset color
                                            Outpt = "Object inventory options:" ' make message
                                            CALL IO.O ' send message
                                            Graphics.Off = True ' reset color
                                            Outpt = "[A]dd" ' make display message
                                            CALL IO.O ' send message
                                            Outpt = "[D]elete" ' make display message
                                            CALL IO.O ' send message
                                            Outpt = "[L]ist" ' make display message
                                            CALL IO.O ' send message
                                            Graphics.Off = False ' reset color
                                            Outpt = "Object inventory edit option(q to quit)? "
                                            No.Input.Out = "Q" ' default input
                                            CALL IO.I ' get user input
                                            ' selection of object edit
                                            SELECT CASE UCASE$(Inpt)
                                                CASE "A" ' option to add object
                                                    Object.Added = False ' object added flag
                                                    ' routine to get object
                                                    CALL Find.Objects(Object.Number)
                                                    ' check object number
                                                    IF Object.Number > False THEN
                                                        CALL Add.Object(Object.Number, Object.Added)
                                                    END IF ' end check object number range
                                                    IF Object.Added THEN ' check object added flag
                                                        Outpt = "Object added to inventory." ' message
                                                    ELSE ' check added flag
                                                        Outpt = "Object not added to inventory."
                                                    END IF ' end check object added flag
                                                    CALL IO.O ' send message
                                                CASE "D" ' option to delete object from inventory
                                                    Outpt = "Number to delete" ' range prompt
                                                    CALL Get.Range2(0, 20, Object.Number) ' get number
                                                    IF Object.Number > False THEN ' check number bounds
                                                        ' check object
                                                        IF GetUserRecordObject(Object.Number) > False THEN
                                                            CALL Discard.Inventory.Object(Object.Number)
                                                            Outpt = "Object deleted from inventory."
                                                            CALL IO.O ' send message
                                                        ELSE ' check object
                                                            Outpt = "Object not deleted from inventory."
                                                            CALL IO.O ' send message
                                                        END IF ' end check object number range
                                                    END IF ' end check number bounds
                                                CASE "L"
                                                    Objects.Count = False ' reset count flag
                                                    Graphics.Off = True ' reset color
                                                    ' loop through player objects
                                                    FOR Object.Index = 1 TO 20
                                                        ' store object
                                                        Object.Number = GetUserRecordObject(Object.Index)
                                                        ' check object number
                                                        IF Object.Number > False AND Object.Number <= LOF(ObjectFile) / LEN(ObjectRecord) THEN
                                                            ' increment counter
                                                            Objects.Count = True
                                                            ' read object
                                                            CALL Read.Record(ObjectFile, Object.Number)
                                                            ' store
                                                            Object.Name$ = ObjectRecord.ObjectName
                                                            ' lowercase name
                                                            Object.Name$ = LCASE$(Object.Name$)
                                                            Outpt = "[" + MID$(STR$(Object.Index), 2) + "]" + Object.Name$
                                                            CALL IO.O ' send object  name message
                                                        END IF ' end check object number range
                                                    NEXT ' end loop through player object inventory
                                                    IF Objects.Count THEN ' check objects counted
                                                        CALL More.Prompt ' display more prmpt
                                                    ELSE ' check objects counted
                                                        Outpt = "<no objects>" ' make display message
                                                        CALL IO.O ' display message
                                                    END IF ' end objects counted
                                                CASE "Q" ' option to exit inventory object menu
                                                    EXIT DO ' exit inventory object edit menu
                                            END SELECT ' end selection of object edit option
                                        LOOP ' end loop through player object inventory menu
                                    CASE "T" ' option to edit player treasure inventory
                                        DO ' loop through treasure inventory edit menu
                                            Graphics.Off = False ' reset color
                                            Outpt = "Treasure inventory options:" ' message
                                            CALL IO.O ' send message
                                            Graphics.Off = True ' reset color
                                            Outpt = "[A]dd" ' make message
                                            CALL IO.O ' send message
                                            Outpt = "[D]elete" ' make message
                                            CALL IO.O ' send message
                                            Outpt = "[L]ist" ' make message
                                            CALL IO.O ' send message
                                            Graphics.Off = False ' reset color
                                            Outpt = "Treasure inventory edit option(q to quit)? "
                                            No.Input.Out = "Q" ' default input
                                            CALL IO.I ' get user input
                                            SELECT CASE UCASE$(Inpt) ' selection of option
                                                CASE "A" ' option to add inventory treasure
                                                    Treasure.Added = False ' treasure added flag
                                                    ' routine to get number
                                                    CALL Find.Treasure(Treasure.Number)
                                                    IF Treasure.Number > False THEN ' check number
                                                        ' check treasure type
                                                        IF TreasureRecord.Container = False THEN
                                                            CALL TreasureCharges(Treasure.Charges)
                                                            CALL Add.Inventory(Treasure.Number, Treasure.Charges, Treasure.Added)
                                                        END IF ' end check treasure type
                                                    END IF ' end check treasure number
                                                    ' check treasure added flag
                                                    IF Treasure.Added THEN
                                                        Outpt = "Treasure added to inventory."
                                                    ELSE ' check flag
                                                        Outpt = "Treasure not added to inventory."
                                                    END IF ' end check added flag
                                                    CALL IO.O ' send message
                                                CASE "D" ' option to delete treasure number
                                                    Outpt = "Number to delete" ' range prompt
                                                    CALL Get.Range2(0, 20, Treasure.Number) ' get number
                                                    IF Treasure.Number > False THEN ' check number bounds
                                                        ' check treasure index
                                                        IF GetUserRecordInv(Treasure.Number) > False THEN
                                                            IF User.Number = User.Index THEN ' check user
                                                                ' discard online user treasure
                                                                CALL Discard.Inventory(Treasure.Number, True)
                                                            ELSE ' check user
                                                                ' discard offline user treasure
                                                                CALL Discard.Inventory.Treasure(Treasure.Number)
                                                            END IF ' end check user
                                                            Outpt = "Treasure deleted from inventory."
                                                            CALL IO.O ' send message
                                                        ELSE ' end check number range
                                                            Outpt = "Treasure not deleted from inventory."
                                                            CALL IO.O ' send message
                                                        END IF ' end check deleted flag
                                                    END IF ' end check number bounds
                                                CASE "L"
                                                    Treasure.Counted = False ' reset counter
                                                    Graphics.Off = True ' reset color
                                                    ' loop through inventory
                                                    FOR Treasure.Index = 1 TO 20
                                                        ' store number
                                                        Treasure.Number = GetUserRecordInv(Treasure.Index)
                                                        ' check number
                                                        IF Treasure.Number > False AND Treasure.Number <= LOF(TreasureFile) / LEN(TreasureRecord) THEN
                                                            ' increment counter
                                                            Treasure.Counted = True
                                                            ' get name
                                                            CALL Read.Record(TreasureFile, Treasure.Number)
                                                            Treasure.Name$ = TreasureRecord.TreasureName
                                                            ' lowercase name
                                                            Treasure.Name$ = LCASE$(Treasure.Name$)
                                                            Outpt = "[" + MID$(STR$(Treasure.Index), 2) + "]" + Treasure.Name$
                                                            CALL IO.O ' send message
                                                        END IF ' end check treasure number range
                                                    NEXT ' end loop through inventory treasure
                                                    IF Treasure.Counted THEN ' check counter
                                                        CALL More.Prompt ' display more prompt
                                                    ELSE ' check counter
                                                        Outpt = "<no treasure>" ' make message
                                                        CALL IO.O ' display message
                                                    END IF ' end check treasure counted
                                                CASE "Q" ' option to exit inventory treasure edit
                                                    EXIT DO ' exit inventory treasure edit menu
                                            END SELECT ' end selection of treasure edit option
                                        LOOP ' end loop through treasure edit menu
                                    CASE "Q" ' option to exit player inventory edit menu
                                        EXIT DO ' exit player inventory edit menu
                                END SELECT ' end selection of inventory menu option
                            LOOP ' end loop through player inventory edit menu
                        CASE "X" ' option to change player special class
                            DO ' loop through special class edit menu
                                Graphics.Off = True ' reset color
                                Outpt = "[A]Town Mayor" ' make display message
                                CALL IO.O ' send message
                                Outpt = "[B]Governor" ' make display message
                                CALL IO.O ' send message
                                Outpt = "[C]Guild Master" ' make display message
                                CALL IO.O ' send message
                                Outpt = "[D]Sysop" ' make display message
                                CALL IO.O ' send message
                                Outpt = "[E]None" ' make display message
                                CALL IO.O ' send message
                                Graphics.Off = False ' reset color
                                Outpt = "Special class option(q to quit)? " ' input prompt
                                No.Input.Out = "Q" ' default input
                                CALL IO.I ' get user input
                                SELECT CASE UCASE$(Inpt) ' selection of special class
                                    CASE "A" ' option for special class
                                        UserRecord.Flags = UserRecord.Flags OR Special.Char1
                                        Outpt = "Player is now town mayor." ' make message
                                        CALL IO.O ' send message
                                    CASE "B" ' option for special class
                                        UserRecord.Flags = UserRecord.Flags OR Special.Char2
                                        Outpt = "Player is now governor." ' make message
                                        CALL IO.O ' send message
                                    CASE "C" ' option for special class
                                        UserRecord.Flags = UserRecord.Flags OR Special.Char3
                                        Outpt = "Player is now guild master." ' make message
                                        CALL IO.O ' send message
                                    CASE "D" ' option for special class
                                        UserRecord.Flags = UserRecord.Flags OR Special.Char4
                                        Outpt = "Player is now sysop." ' make message
                                        CALL IO.O ' send message
                                    CASE "E" ' option to clear special class
                                        UserRecord.Flags = UserRecord.Flags AND NOT Special.Char1
                                        UserRecord.Flags = UserRecord.Flags AND NOT Special.Char2
                                        UserRecord.Flags = UserRecord.Flags AND NOT Special.Char3
                                        UserRecord.Flags = UserRecord.Flags AND NOT Special.Char4
                                        Outpt = "Player is cleared of special character class."
                                        CALL IO.O ' send message
                                    CASE "Q" ' option to exit special class menu
                                        EXIT DO ' exit special class menu
                                END SELECT ' end selection of special class option
                            LOOP ' end loop through special class option menu
                        CASE "Y" ' option to customize player's training rooms
                            DO ' loop through special class edit menu
                                Graphics.Off = False ' reset color
                                Outpt = "Custom training room menu:" ' output message
                                CALL IO.O ' send output
                                Graphics.Off = True ' reset color
                                FOR Train.Count = 1 TO 10 ' display all classnames
                                    Inpt = CHR$(Train.Count + 64) ' store letter output
                                    ' make class display message
                                    Outpt = "[" + Inpt + "]" + RTRIM$(Class.Name(Train.Count)) + " room number:"
                                    ' store training room value
                                    Train.Temp! = GetUserRecordTrainRoom(Train.Count)
                                    IF Train.Temp! = SFalse THEN ' compare value
                                        Outpt = Outpt + " <none>" ' append message
                                    ELSE ' compare value
                                        Outpt = Outpt + STR$(Train.Temp!) + "." ' append message
                                    END IF ' end compare value
                                    CALL IO.O ' send message
                                NEXT ' end loop through training room array
                                Outpt = "[R]eset rooms" ' make output
                                CALL IO.O ' send output
                                Graphics.Off = False ' reset color
                                Outpt = "Training room option(q to quit)? " ' input prompt
                                No.Input.Out = "Q" ' default input
                                CALL IO.I ' get user input
                                Inpt = UCASE$(Inpt) ' uppercase input
                                SELECT CASE Inpt ' selection of special class
                                    CASE "R" ' option for rooms reset
                                        Outpt = "Reset training rooms(y/n)? " ' message
                                        CALL IO.I ' get response
                                        IF Yes THEN ' check response
                                            ' reset training rooms
                                            CALL PutUserRecordTrainRoom(1, 36)
                                            CALL PutUserRecordTrainRoom(2, 4)
                                            CALL PutUserRecordTrainRoom(3, 123)
                                            CALL PutUserRecordTrainRoom(4, 163)
                                            CALL PutUserRecordTrainRoom(5, 162)
                                            CALL PutUserRecordTrainRoom(6, 426)
                                            CALL PutUserRecordTrainRoom(7, 421)
                                            CALL PutUserRecordTrainRoom(8, 417)
                                            CALL PutUserRecordTrainRoom(9, 1)
                                            CALL PutUserRecordTrainRoom(10, 1)
                                            Outpt = "Training rooms reset." ' message
                                            CALL IO.O ' send message
                                        END IF ' end check response
                                    CASE "Q" ' exit training room loop
                                        EXIT DO ' exit special class menu
                                    CASE ELSE ' check response
                                        IF Inpt >= "A" AND Inpt <= "J" THEN ' check response range
                                            Train.Count = ASC(Inpt) - 64 ' convert to number
                                            Outpt = "Enter 0 for auto-train." ' make message
                                            CALL IO.O ' send output
                                            Graphics.Off = True ' reset color
                                            ' make class range prompt
                                            Outpt = "[" + MID$(STR$(Train.Count), 2) + "]" + RTRIM$(Class.Name(Train.Count)) + " room number"
                                            ' return number from range
                                            Max.Rooms! = LOF(RoomFile) / LEN(RoomRecord)
                                            CALL Get.Room.Range2(0!, Max.Rooms!, Train.Number!)
                                            CALL PutUserRecordTrainRoom(Train.Count, Train.Number!)
                                        END IF ' end check input
                                END SELECT ' end selection of special class option
                            LOOP ' end loop for training rooms
                        CASE "Q" ' option to exit player edit menu
                            EXIT DO ' exit player edit menu
                    END SELECT ' end selection of player edit option
                LOOP ' end loop through player edit menu
                CALL Share.Record(UserFile, User.Number) ' write user record
            END IF ' end check user found flag
            CALL Read.Record(UserFile, User.Index)
        CASE "L" ' option to list user file
            DO ' loop through user file list menu
                Graphics.Off = True ' reset color
                Outpt = "[L]ist" ' make display message
                CALL IO.O ' send message
                Outpt = "[T]op ten" ' make display message
                CALL IO.O ' send message
                Outpt = "[U]sers" ' make display message
                CALL IO.O ' send message
                Graphics.Off = False ' reset color
                Outpt = "User list option(q to quit)? " ' make input prompt
                No.Input.Out = "Q" ' default input
                CALL IO.I ' get user input
                SELECT CASE UCASE$(Inpt) ' selection of user list option
                    CASE "L" ' option to list range of user numbers
                        Outpt = "Enter range:" ' make display message
                        CALL IO.O ' send message
                        Max.Users = LOF(UserFile) / LEN(UserRecord) ' store length of file
                        ' routine to get range of numbers
                        CALL Get.Range(Max.Users, User.List1, User.List2)
                        ' loop through range of user numbers
                        Allow.Break = True ' set allow break flag
                        Break = False ' reset control-k flag
                        Continue = False ' set continuous flag
                        FOR User.Count = User.List1 TO User.List2
                            CALL Read.Record(UserFile, User.Count) ' read user file
                            Code.Name$ = UserRecord.CodeName ' store user codename
                            CALL Decrypt(Code.Name$) ' decrypt codename
                            IF LEFT$(Code.Name$, 9) <> Deleted$ THEN ' check deleted codename
                                CALL Display.User(User.Count) ' routine to display user
                                IF Break THEN ' check control-k flag
                                    EXIT FOR ' exit display loop
                                END IF ' end check control-k
                                IF Continue = False THEN ' check continuous flag
                                    CALL More.Prompt ' routine to pause
                                    IF No THEN ' check input pause response
                                        EXIT FOR ' exit user list loop
                                    END IF ' end check user input
                                END IF ' end check continuous flag
                            END IF '  end check deleted codename
                        NEXT ' end loop through user numbers
                        Allow.Break = False ' reset allow break flag
                        IF Break THEN ' check control-k flag
                            Break = False ' reset control-k flag
                            Outpt = Nul ' set output to null
                            CALL IO.O ' send empty return
                        END IF ' end check control-k flag
                    CASE "T" ' option to list top ten report
                        CALL Top.Ten ' routine for top ten
                    CASE "U" ' option to list user report
                        CALL User.List ' routine for user report
                    CASE "Q" ' option to exit user list menu
                        EXIT DO ' exit user list menu
                END SELECT ' end selection of user list option
            LOOP ' end loop through user list menu
            CALL Read.Record(UserFile, User.Index)
        CASE "U" ' option to update user file
            IF Local.Mode = False THEN ' check local sysop online
                Outpt = "Remote DMs cannot update users." ' make message
                CALL IO.O ' send error message
            ELSE ' check local sysop
                DO
                    Outpt = "Delete " + Dead$ + " users(y/n)? " ' make input prompt
                    No.Input.Out = "Y" ' default input
                    CALL IO.I ' get user input
                    Delete.Dead.Users = Yes ' store input response
                    Outpt = "Delete users before date(y/n)? " ' make input prompt
                    No.Input.Out = "N" ' default input
                    CALL IO.I ' get user input
                    Delete.By.Date = Yes ' store user input response
                    IF Delete.By.Date THEN ' check date update response
                        Outpt = "Enter date(form: MM-DD-YYYY)? " ' make input prompt
                        No.Input.Out = "01-01-1980" ' default input
                        CALL IO.I ' get user input
                        IF ValidDate(Inpt) = 0 THEN
                            Outpt = "Invalid date."
                            CALL IO.O
                        ELSE
                            Delete.Date$ = Inpt
                            EXIT DO
                        END IF
                    ELSE
                        EXIT DO
                    END IF ' end check date update response
                LOOP
                Graphics.Off = False ' reset color
                Outpt = "Updating userfile.." ' make display message
                CALL IO.O ' send message
                Users.Deleted = False ' reset counter
                FOR User.Count = 1 TO LOF(UserFile) / LEN(UserRecord) ' loop through users
                    CALL Read.Record(UserFile, User.Count) ' read next user file record
                    Code.Name$ = UserRecord.CodeName ' store user codename
                    CALL Decrypt(Code.Name$) ' decrypt codename
                    Delete.User.Flag = False ' delete user flag
                    IF LEFT$(Code.Name$, LEN(Deleted$)) <> Deleted$ THEN ' check already deleted
                        IF Delete.Dead.Users THEN ' check update flag
                            IF UserRecord.Level = False THEN ' compare player level
                                Delete.User.Flag = True ' set delete user flag
                            END IF ' end compare player level
                            IF LEFT$(Code.Name$, LEN(Dead$)) = Dead$ THEN ' compare dead player
                                Delete.User.Flag = True ' set delete user flag
                            END IF ' end compare dead player
                        END IF ' end check update flag
                        IF Delete.By.Date THEN ' check date update flag
                            Inpt = UserRecord.DateOn ' store player last date online
                            CALL Decrypt(Inpt) ' decrypt date
                            Var1$ = MID$(Inpt, 7, 4) + MID$(Inpt, 4, 2) + MID$(Inpt, 1, 2)
                            Var2$ = MID$(Delete.Date$, 7, 4) + MID$(Delete.Date$, 4, 2) + MID$(Delete.Date$, 1, 2)
                            IF Var1$ < Var2$ THEN
                                Delete.User.Flag = True ' set delete user flag
                            END IF ' end calculate dates
                        END IF ' end check date update flag
                    END IF ' end check deleted user
                    IF Delete.User.Flag THEN ' check delete user flag
                        IF User.Count <> User.Index THEN ' check to current user
                            Users.Deleted = Users.Deleted + 1 ' increment counter
                            CALL Delete.User ' routine to clear user record
                            CALL Share.Record(UserFile, User.Count) ' write user record
                        END IF ' end check current user
                    END IF ' end check delete user flag
                NEXT ' end loop through user file
                IF Users.Deleted = False THEN ' check counter
                    Outpt = "No users deleted." ' make message
                ELSE ' check counter
                    IF Users.Deleted = 1 THEN ' compare counter
                        Outpt = "1 user deleted." ' make message
                    ELSE ' check counter
                        Outpt = MID$(STR$(Users.Deleted), 2) + " users deleted." ' message
                    END IF ' check counter
                END IF ' end check counter
                CALL IO.O ' display message
                CALL Read.Record(UserFile, User.Index) ' restore current user
            END IF ' end check local sysop
        CASE "Q" ' option to exit user edit menu
            EXIT DO ' exit user edit menu
    END SELECT ' end selection of user edit option
LOOP ' end loop through user edit menu options
END SUB ' end routine to edit user file

REM * routine to find user.
REM * output variables:
REM *   User.Found  -  true if user found
REM *   User.Number  -  user index

SUB Find.User (User.Found, User.Number)
Graphics.Off = False ' reset color
Outpt = "Codename, or number" ' make input prompt
Outpt = Outpt + "(1-" + MID$(STR$(LOF(UserFile) / LEN(UserRecord)), 2) + ")? "
No.Input.Out = "1" ' default input
CALL IO.I ' get user input
CALL Parse.Num(Inpt, Parse.Value) ' routine to parse pound sign
Inpt = UCASE$(Inpt) ' uppercase input
Parse.Number = False ' reset counter
User.Found = False ' reset user found flag
Input.Length = LEN(Inpt) ' store length of input
FOR User.Number = 1 TO LOF(UserFile) / LEN(UserRecord) ' loop through user file
    CALL Read.Record(UserFile, User.Number) ' read user file
    Code.Name$ = UserRecord.CodeName ' store user codename
    CALL Decrypt(Code.Name$) ' decrypt codename
    ' compare codename to input
    IF LEFT$(Code.Name$, Input.Length) = Inpt THEN
        Parse.Number = Parse.Number + 1 ' increment counter
        ' compare counters
        IF Parse.Value = False OR Parse.Number = Parse.Value THEN
            User.Found = True ' set user found flag
            EXIT FOR ' exit user loop
        END IF ' end compare counters
    END IF ' end compare codename
NEXT ' end loop through users
IF User.Found = False THEN ' check user found flag
    User.Number = INT(VAL(Inpt)) ' convert input to integer
    ' check bounds
    IF User.Number > False AND User.Number <= LOF(UserFile) / LEN(UserRecord) THEN
        User.Found = True ' set user found flag
    END IF ' end check file bounds
END IF ' end check user flag
IF User.Found = False THEN ' check user found flag
    Outpt = "Codename not found." ' make not found message
    CALL IO.O ' send message
END IF ' end check user found flag
IF User.Found THEN ' check user found flag
    CALL Read.Record(UserFile, User.Number) ' read user record
    Code.Name$ = UserRecord.CodeName ' store codename
    CALL Decrypt(Code.Name$) ' decrypt codename
    IF LEFT$(Code.Name$, 9) = Deleted$ THEN ' compare deleted codename
        Outpt = "Deleted user record." ' make deleted user message
        CALL IO.O ' send message
        User.Found = False ' reset flag
    END IF ' end check delete codename
END IF ' end check user found
END SUB

' formats a double numeric string
FUNCTION FormatString$ (s#)
x$ = Nul
s$ = STR$(s#)
IF INSTR(s$, "D") THEN ' return string
    FormatString$ = s$
    EXIT FUNCTION
END IF
IF LEFT$(s$, 1) = "-" THEN ' store sign
    e$ = "-"
    s$ = MID$(s$, 2)
END IF
s$ = LTRIM$(s$) ' format string
IF INSTR(s$, ".") THEN
    q$ = MID$(s$, INSTR(s$, "."))
    s$ = LEFT$(s$, INSTR(s$, ".") - 1)
END IF
FOR l = LEN(s$) TO 3 STEP -3
    x$ = MID$(s$, l - 2, 3) + "," + x$
NEXT
IF l > 0 THEN
    x$ = MID$(s$, 1, l) + "," + x$
END IF
IF LEN(s$) < 3 THEN
    x$ = s$
END IF
IF RIGHT$(x$, 1) = "," THEN
    x$ = LEFT$(x$, LEN(x$) - 1)
END IF
x$ = e$ + x$ + q$ ' construct string
FormatString$ = x$
END FUNCTION

FUNCTION ValidDate (Var$)
' mm-dd-yyyy
ValidDate = 0
FOR V = 1 TO 10
    SELECT CASE V
        CASE 3, 6
            IF MID$(Var$, V, 1) = "-" THEN
                Eat$ = ""
            ELSE
                EXIT FUNCTION
            END IF
        CASE ELSE
            IF MID$(Var$, V, 1) >= "0" AND MID$(Var$, V, 1) <= "9" THEN
                Eat$ = ""
            ELSE
                EXIT FUNCTION
            END IF
    END SELECT
NEXT
' mm-dd-yyyy
V = INT(VAL(MID$(Var$, 1, 2)))
IF V >= 1 AND V <= 12 THEN
    Eat$ = ""
ELSE
    EXIT FUNCTION
END IF
V = INT(VAL(MID$(Var$, 4, 2)))
IF V >= 1 AND V <= 31 THEN
    Eat$ = ""
ELSE
    EXIT FUNCTION
END IF
V = INT(VAL(MID$(Var$, 7, 4)))
IF V >= 1980 AND V <= 2079 THEN
    Eat$ = ""
ELSE
    EXIT FUNCTION
END IF
ValidDate = True
END FUNCTION

FUNCTION GetContainerRecCharges (Array.Counter)
GetContainerRecCharges = CVI(MID$(ContainerRec.Charges, (Array.Counter - 1) * 2 + 1, 2))
END FUNCTION

SUB PutContainerRecCharges (Array.Counter, Item.Charges)
MID$(ContainerRec.Charges, (Array.Counter - 1) * 2 + 1, 2) = MKI$(Item.Charges)
END SUB

FUNCTION GetContainerRecInventory (Array.Counter)
GetContainerRecInventory = CVI(MID$(ContainerRec.Inventory, (Array.Counter - 1) * 2 + 1, 2))
END FUNCTION

SUB PutContainerRecInventory (Array.Counter, Item.Charges)
MID$(ContainerRec.Inventory, (Array.Counter - 1) * 2 + 1, 2) = MKI$(Item.Charges)
END SUB

FUNCTION GetContainerRecInvisible (Array.Counter)
GetContainerRecInvisible = CVI(MID$(ContainerRec.Invisible, (Array.Counter - 1) * 2 + 1, 2))
END FUNCTION

SUB PutContainerRecInvisible (Array.Counter, Item.Charges)
MID$(ContainerRec.Invisible, (Array.Counter - 1) * 2 + 1, 2) = MKI$(Item.Charges)
END SUB

SUB PutUserRecordContainer (Var)
SELECT CASE Var
    CASE 1
        UserRecord.Container1 = ContainerRec ' store user container into record
    CASE 2
        UserRecord.Container2 = ContainerRec ' store user container into record
    CASE 3
        UserRecord.Container3 = ContainerRec ' store user container into record
END SELECT
END SUB

FUNCTION GetUserRecordContainer (Var)
SELECT CASE Var
    CASE 1
        ContainerRec = UserRecord.Container1 ' get user container into record
    CASE 2
        ContainerRec = UserRecord.Container2 ' get user container into record
    CASE 3
        ContainerRec = UserRecord.Container3 ' get user container into record
END SELECT
END FUNCTION

FUNCTION GetRoomRecordDirect (Direction.Number) ' get room number direction
GetRoomRecordDirect = CVS(MID$(RoomRecord.Direct, (Direction.Number - 1) * 4 + 1, 4))
END FUNCTION

SUB PutRoomRecordDirect (Array.Index, Var!)
MID$(RoomRecord.Direct, (Array.Index - 1) * 4 + 1, 4) = MKS$(Var!)
END SUB

FUNCTION GetRoomRecordFlags (Array.Number)
GetRoomRecordFlags = CVI(MID$(RoomRecord.Flags, (Array.Number - 1) * 2 + 1, 2))
END FUNCTION

SUB PutRoomRecordFlags (Array.Index, Var)
MID$(RoomRecord.Flags, (Array.Index - 1) * 2 + 1, 2) = MKI$(Var)
END SUB

FUNCTION GetRoomRecordLongDesc$ (Array.Number) ' get room description line
GetRoomRecordLongDesc$ = MID$(RoomRecord.LongDesc, (Array.Number - 1) * 80 + 1, 80)
END FUNCTION

SUB PutRoomRecordLongDesc (Array.Index, Var$) ' set long description
MID$(RoomRecord.LongDesc, (Array.Index - 1) * 80 + 1, 80) = Var$
END SUB

FUNCTION GetRoomRecordObject (Array.Number) ' get object index to file
GetRoomRecordObject = CVI(MID$(RoomRecord.Object, (Array.Number - 1) * 2 + 1, 2))
END FUNCTION

SUB PutRoomRecordObject (Array.Index, Var)
MID$(RoomRecord.Object, (Array.Index - 1) * 2 + 1, 2) = MKI$(Var)
END SUB

FUNCTION GetRoomRecordHiddenObj (Array.Number)
GetRoomRecordHiddenObj = CVI(MID$(RoomRecord.HiddenObj, (Array.Number - 1) * 2 + 1, 2))
END FUNCTION

SUB PutRoomRecordHiddenObj (Array.Index, Var)
MID$(RoomRecord.HiddenObj, (Array.Index - 1) * 2 + 1, 2) = MKI$(Var)
END SUB

FUNCTION GetRoomRecordTreasure (Inventory.Number)
GetRoomRecordTreasure = CVI(MID$(RoomRecord.Treasure, (Inventory.Number - 1) * 2 + 1, 2))
END FUNCTION

SUB PutRoomRecordTreasure (Array.Index, Var)
MID$(RoomRecord.Treasure, (Array.Index - 1) * 2 + 1, 2) = MKI$(Var)
END SUB

FUNCTION GetRoomRecordTreCharges (Inventory.Number)
GetRoomRecordTreCharges = CVI(MID$(RoomRecord.TreCharges, (Inventory.Number - 1) * 2 + 1, 2))
END FUNCTION

SUB PutRoomRecordTreCharges (Array.Index, Var)
MID$(RoomRecord.TreCharges, (Array.Index - 1) * 2 + 1, 2) = MKI$(Var)
END SUB

FUNCTION GetUserRecordInv (Treasure.Index)
GetUserRecordInv = CVI(MID$(UserRecord.Inv, (Treasure.Index - 1) * 2 + 1, 2))
END FUNCTION

SUB PutUserRecordInv (Array.Index, Var)
MID$(UserRecord.Inv, (Array.Index - 1) * 2 + 1, 2) = MKI$(Var)
END SUB

FUNCTION GetUserRecordCharges (Array.Number)
GetUserRecordCharges = CVI(MID$(UserRecord.Charges, (Array.Number - 1) * 2 + 1, 2))
END FUNCTION

SUB PutUserRecordCharges (Array.Number, Var)
MID$(UserRecord.Charges, (Array.Number - 1) * 2 + 1, 2) = MKI$(Var)
END SUB

FUNCTION GetUserRecordObject (Object.Index)
GetUserRecordObject = CVI(MID$(UserRecord.Object, (Object.Index - 1) * 2 + 1, 2))
END FUNCTION

SUB PutUserRecordObject (Array.Index, Var)
MID$(UserRecord.Object, (Array.Index - 1) * 2 + 1, 2) = MKI$(Var)
END SUB

FUNCTION GetUserRecordTrainRoom (Player.Class)
GetUserRecordTrainRoom = CVS(MID$(UserRecord.TrainRoom, (Player.Class - 1) * 4 + 1, 4))
END FUNCTION

SUB PutUserRecordTrainRoom (Train.Count, Var!)
MID$(UserRecord.TrainRoom, (Train.Count - 1) * 4 + 1, 4) = MKS$(Var!)
END SUB

FUNCTION GetUserRecordStats (Var)
GetUserRecordStats = CVI(MID$(UserRecord.Stats, (Var - 1) * 2 + 1, 2))
END FUNCTION

SUB PutUserRecordStats (Var, Var2)
MID$(UserRecord.Stats, (Var - 1) * 2 + 1, 2) = MKI$(Var2)
END SUB

FUNCTION GetUserRecordWeapons (Var)
GetUserRecordWeapons = CVI(MID$(UserRecord.Weapons, (Var - 1) * 2 + 1, 2))
END FUNCTION

SUB PutUserRecordWeapons (Var, Var2)
MID$(UserRecord.Weapons, (Var - 1) * 2 + 1, 2) = MKI$(Var2)
END SUB

FUNCTION GetSpellRecordIngred (Var)
GetSpellRecordIngred = CVI(MID$(SpellRecord.Ingred, (Var - 1) * 2 + 1, 2))
END FUNCTION

SUB PutSpellRecordIngred (Var, Var2)
MID$(SpellRecord.Ingred, (Var - 1) * 2 + 1, 2) = MKI$(Var2)
END SUB

FUNCTION GetMonclassRecordMonsters (Monster.Class.Number)
GetMonclassRecordMonsters = CVI(MID$(MonclassRecord.Monsters, (Monster.Class.Number - 1) * 2 + 1, 2))
END FUNCTION

SUB PutMonclassRecordMonsters (Monster.Classes, Var)
MID$(MonclassRecord.Monsters, (Monster.Classes - 1) * 2 + 1, 2) = MKI$(Var)
END SUB

FUNCTION GetMonsterRecordTreasure (Array.Number)
GetMonsterRecordTreasure = CVI(MID$(MonsterRecord.Treasure, (Array.Number - 1) * 2 + 1, 2))
END FUNCTION

SUB PutMonsterRecordTreasure (Array.Index, Var)
MID$(MonsterRecord.Treasure, (Array.Index - 1) * 2 + 1, 2) = MKI$(Var)
END SUB

FUNCTION GetMonsterRecordTalk (Array.Number)
GetMonsterRecordTalk = CVI(MID$(MonsterRecord.Talk, (Array.Number - 1) * 2 + 1, 2))
END FUNCTION

SUB PutMonsterRecordTalk (Array.Index, Var)
MID$(MonsterRecord.Talk, (Array.Index - 1) * 2 + 1, 2) = MKI$(Var)
END SUB

